

(******************************************************************************)
(*                      Probabilistic Analysis of                             *)
(*                        Cascased PAND Gates                                 *)
(*                                  by                                        *)
(*                          Yassmeen Elderhalli                               *)
(*                      y_elderh@encs.concordia.ca                            *)
(******************************************************************************)

(*loadPath := "/home/yassmeen/Downloads/ProbTheories_HOL-k12" :: !loadPath;

(*set_trace "Unicode" 0;*)

app load ["HolKernel", "Parse", "boolLib", "bossLib", "numLib", "unwindLib", 
"tautLib", "Arith", "prim_recTheory", "combinTheory", "quotientTheory", 
"arithmeticTheory", "realTheory", "hrealTheory", "realaxTheory", "realLib",  
"jrhUtils", "pairTheory", "seqTheory", "limTheory", "transcTheory", "listTheory", 
"mesonLib", "boolTheory", "topologyTheory", "pred_setTheory", "util_probTheory", 
"optionTheory", "numTheory", "sumTheory", "InductiveDefinition", "ind_typeTheory",
"pred_setLib", "iterate_hvgTheory", "card_hvgTheory", "product_hvgTheory", 
"topology_hvgTheory", "derivative_hvgTheory", "integration_hvgTheory",
"real_sigmaTheory", "extreal_hvgTheory", "measure_hvgTheory", "lebesgue_hvgTheory",
"probability_hvgTheory", "lebesgue_measure_hvgTheory", "normal_rv_hvgTheory", "netsTheory", "PIE_EXTREALTheory","dep_rewrite", "integration_before_afterTheory", "DFT_Gates_def_PROBTheory", "CASTheory","WCSPTheory", "WCSP_sharedTheory"];
*)
open HolKernel Parse boolLib bossLib numLib unwindLib tautLib Arith prim_recTheory 
combinTheory quotientTheory arithmeticTheory hrealTheory realaxTheory realTheory 
realLib jrhUtils pairTheory seqTheory limTheory transcTheory listTheory mesonLib 
boolTheory topologyTheory pred_setTheory util_probTheory optionTheory numTheory 
sumTheory InductiveDefinition ind_typeTheory pred_setLib iterate_hvgTheory 
card_hvgTheory product_hvgTheory topology_hvgTheory derivative_hvgTheory
integration_hvgTheory real_sigmaTheory extreal_hvgTheory measure_hvgTheory
lebesgue_hvgTheory probability_hvgTheory lebesgue_measure_hvgTheory normal_rv_hvgTheory netsTheory PIE_EXTREALTheory dep_rewrite integration_before_afterTheory DFT_Gates_def_PROBTheory CASTheory WCSPTheory WCSP_sharedTheory;

val _ = new_theory "AND_FDEP";


infixr 0 ++ << || ORELSEC ## --> THENC;
infix 1 >> |->;
fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
 
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Suff = PARSE_TAC SUFF_TAC;
val POP_ORW = POP_ASSUM (fn thm => ONCE_REWRITE_TAC [thm]);
val !! = REPEAT;
val op++ = op THEN;
val op<< = op THENL;
val op|| = op ORELSE;
val op>> = op THEN1;
val std_ss' = simpLib.++ (std_ss, boolSimps.ETA_ss);

(*---------------------------*)
fun SET_TAC L =
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION,
    IN_INTER, IN_DIFF, IN_INSERT, IN_DELETE, IN_BIGINTER, IN_BIGUNION,
    IN_IMAGE, GSPECIFICATION, IN_DEF] THEN METIS_TAC [];

fun SET_RULE tm = prove(tm,SET_TAC []);


val set_rewrs
= [INTER_EMPTY, INTER_UNIV, UNION_EMPTY, UNION_UNIV, DISJOINT_UNION,
DISJOINT_INSERT, DISJOINT_EMPTY, GSYM DISJOINT_EMPTY_REFL,
DISJOINT_BIGUNION, INTER_SUBSET, INTER_IDEMPOT, UNION_IDEMPOT,
SUBSET_UNION];

val UNIONL_def = Define `(UNIONL [] = {})
/\ (UNIONL (s::ss) = (s:'a ->bool) UNION UNIONL ss)`;


val IN_UNIONL = store_thm
("IN_UNIONL",
``!l (v:'a ). v IN UNIONL l = (?s. MEM s l /\ v IN s)``,
Induct >> RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY]
++ RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY, IN_UNION]
++ PROVE_TAC []);


val elt_rewrs
= [SUBSET_DEF, IN_COMPL, IN_DIFF, IN_UNIV, NOT_IN_EMPTY, IN_UNION,
IN_INTER, IN_IMAGE, IN_FUNSET, IN_DFUNSET, GSPECIFICATION,
DISJOINT_DEF, IN_BIGUNION, IN_BIGINTER, IN_COUNT, IN_o,
IN_UNIONL, IN_DELETE, IN_PREIMAGE, IN_SING, IN_INSERT];


fun rewr_ss ths =
simpLib.++
(std_ss,
simpLib.SSFRAG
{ac = [],
name = NONE,
convs = [],
dprocs = [],
filter = NONE,
rewrs = set_rewrs @ elt_rewrs,
congs = []});
val pset_set_ss = rewr_ss set_rewrs;
val pset_elt_ss = rewr_ss elt_rewrs;
val pset_ss = rewr_ss (set_rewrs @ elt_rewrs);


fun PSET_TAC ths =
REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_set_ss ths
++ REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_elt_ss ths;

fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Simplify = RW_TAC arith_ss;
val Suff = PARSE_TAC SUFF_TAC;
val Know = PARSE_TAC KNOW_TAC;
fun wrap a = [a];
val Rewr = DISCH_THEN (REWRITE_TAC o wrap);
val Rewr' = DISCH_THEN (ONCE_REWRITE_TAC o wrap);
val Cond =
  DISCH_THEN
  (fn mp_th =>
   let
     val cond = fst (dest_imp (concl mp_th))
   in
     KNOW_TAC cond << [ALL_TAC, DISCH_THEN (MP_TAC o MP mp_th)]
   end);
val FUN_EQ_TAC = CONV_TAC (CHANGED_CONV (ONCE_DEPTH_CONV FUN_EQ_CONV));
fun K_TAC _ = ALL_TAC;
fun MESON ths tm = prove(tm,MESON_TAC ths);
fun METIS ths tm = prove(tm,METIS_TAC ths);
val DISC_RW_KILL = DISCH_TAC THEN ONCE_ASM_REWRITE_TAC [] THEN 
                   POP_ASSUM K_TAC;
			   
val IN_REST = store_thm ("IN_REST",
 ``!x:'a. !s. x IN (REST s) <=> x IN s /\ ~(x = CHOICE s)``,
  REWRITE_TAC[REST_DEF, IN_DELETE]);
fun SET_TAC L = 
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN
    SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION, IN_INTER, IN_DIFF, 
      IN_INSERT, IN_DELETE, IN_REST, IN_BIGINTER, IN_BIGUNION, IN_IMAGE, 
      GSPECIFICATION, IN_DEF, EXISTS_PROD] THEN METIS_TAC [];
fun ASSERT_TAC tm = SUBGOAL_THEN tm STRIP_ASSUME_TAC;
fun SET_RULE tm = prove(tm,SET_TAC []);
fun ASM_SET_TAC L = REPEAT (POP_ASSUM MP_TAC) THEN SET_TAC L;
val ASM_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN ARITH_TAC;
val ASM_REAL_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN REAL_ARITH_TAC;

fun DFT_rewrs L =  REPEAT( METIS_TAC L);


val rewrs = [REAL_LE_REFL, REAL_LE_LT, REAL_LE_ANTISYM, REAL_LET_ANTISYM, REAL_LET_TOTAL, REAL_LET_TRANS, real_of_num,REAL_LT_NEG1_1,REAL_LT_NEG1_0,REAL_LE_NEG1_0,REAL_LE_NEG1_1, REAL_LT_TOTAL, max_def, min_def, REAL_LT_TRANS, REAL_LE_TRANS, REAL_MIN_COMM, REAL_MAX_COMM];

val DFT_rewrs_ss  =DFT_rewrs rewrs;



val AND_FDEP_reduced = store_thm("AND_FDEP_reduced",
  ``!X Y Z. D_AND (X ) (FDEP Y Z) = D_OR (D_AND X Y)(D_AND X Z)  ``,
    RW_TAC std_ss[FDEP_OR]
    THEN RW_TAC std_ss[D_AND_OVER_OR]);

val AND_FDEP_event = store_thm("AND_FDEP_event",
  ``!X Y Z t p. DFT_event p (D_AND (X)(FDEP Y Z)) t = (DFT_event p (D_AND X Y) t) UNION (DFT_event p (D_AND X Z) t)    ``,

  RW_TAC std_ss[AND_FDEP_reduced]
  THEN RW_TAC std_ss[OR_UNION]);



          
val IN_MEASURABLE_borel_MAX = store_thm   ( "IN_MEASURABLE_borel_MAX",
``!a f g. (!s. f s<> PosInf /\ f s <> NegInf /\ g s <> PosInf /\ g s <> NegInf)  /\sigma_algebra a /\ (\s. real (f s)) IN measurable a borel /\ (\s. real (g s)) IN measurable a borel ==> (\x. real (max (f x) (g x))) IN measurable a borel``, 

REPEAT STRIP_TAC THEN RW_TAC std_ss [in_measurable_borel, GSYM borel_measurable] THENL
  [RW_TAC std_ss [IN_FUN_SET, IN_UNIV], ALL_TAC]
THEN KNOW_TAC(``!s. max (f s) (g s) <> PosInf /\ max (f s ) (g s) <> NegInf ``)  
     THEN1(RW_TAC std_ss[extreal_max_def]
           THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_trans, le_trans])
THEN DISCH_TAC	   
THEN RW_TAC std_ss[real_def]
THEN RW_TAC std_ss[extreal_max_def]
THEN KNOW_TAC(``!c. {x| (@r. (if f x <= g x then g x else f x) = Normal r) < c} INTER
        space a = ({x| (@r. f x = Normal r) < c} INTER space a) INTER
	          ({x| (@r. g x = Normal r) < c} INTER space a) ``)

		  THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IN_INTER]
		        THEN EQ_TAC
			     THEN1(RW_TAC std_ss[]
 			             THEN1 (SELECT_ELIM_TAC
			                    THEN RW_TAC std_ss[]
				             THEN1(Q.EXISTS_TAC `real (f x)`
				             THEN METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def])
				           THEN  UNDISCH_TAC (`` (@r. g x = Normal r) < c ``)
				           THEN SELECT_ELIM_TAC
				           THEN RW_TAC std_ss[]
				       	         THEN1(Q.EXISTS_TAC(` real (g x)`)
					               THEN METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def])
				           THEN `Normal x' <= Normal x''` by METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans]
	 			           THEN `Normal x'' < Normal c` by METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans, extreal_lt_def, extreal_le_def, extreal_lt_eq]
				           THEN METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans, extreal_lt_def, extreal_le_def, extreal_lt_eq])
				      
				      THEN SELECT_ELIM_TAC THEN RW_TAC std_ss[]
				             THEN1( Q.EXISTS_TAC (`real (g x)`)
				                   THEN METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans, extreal_lt_def, extreal_le_def, extreal_lt_eq])
    				      THEN UNDISCH_TAC (``  (@r. f x = Normal r) < c``)
				      THEN SELECT_ELIM_TAC
				      THEN RW_TAC std_ss[]
				          THEN1(Q.EXISTS_TAC(`real (f x)`)
					            THEN  METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans, extreal_lt_def, extreal_le_def, extreal_lt_eq])
				     THEN FULL_SIMP_TAC std_ss[GSYM extreal_lt_def]	    
				     THEN `Normal x' <= Normal x''` by METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans]
	 			     THEN `Normal x'' < Normal c` by METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans, extreal_lt_def, extreal_le_def, extreal_lt_eq]
				      THEN METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans, extreal_lt_def, extreal_le_def, extreal_lt_eq] )
		THEN RW_TAC std_ss[])

THEN RW_TAC std_ss[]
THEN `!s. f s <> NegInf /\ g s <> NegInf` by METIS_TAC[normal_real, lt_le, le_lt, lt_infty, le_infty, extreal_of_num_def, lt_trans, le_trans, extreal_lt_def, extreal_le_def, extreal_lt_eq]
THEN `!s. g s <> PosInf ` by RW_TAC std_ss[]
THEN `!s. f s <> PosInf` by RW_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[real_def]
THEN FULL_SIMP_TAC std_ss[in_measurable_borel, GSYM borel_measurable]
THEN METIS_TAC[ALGEBRA_INTER, sigma_algebra_def]);

val indep_vars_var = store_thm("indep_vars_var",
  ``! X  p M s t ii. s IN ii /\ t IN ii /\ s <> t ==>  (indep_vars p (\i. M) X ii ==>(indep_var p M (X s) M (X t)))  ``,
  

RW_TAC std_ss[]
THEN KNOW_TAC(``indep_vars p (\i. M) X {s;t:num} ``)
THEN1 ( `{s;t:num} SUBSET ii` by ASM_SET_TAC[]
       THEN METIS_TAC [indep_vars_subset])
THEN DISCH_TAC
THEN WEAKEN_TAC (equal(Term` indep_vars p (\i. M) X ii`))

THEN RW_TAC std_ss[indep_var]
THEN FULL_SIMP_TAC std_ss[indep_vars]
THEN RW_TAC std_ss[]
     THEN1(UNDISCH_TAC(``   !i:num.
             i IN {s; t} ==>
             random_variable (X i) p (m_space M,measurable_sets M)``)
	   THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s:num`)
	   THEN ASM_SET_TAC[])
      THEN1(UNDISCH_TAC(``   !i:num.
             i IN {s; t} ==>
             random_variable (X i) p (m_space M,measurable_sets M)``)
	   THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `t:num`)
	   THEN ASM_SET_TAC[])
	   
THEN `{x:num| (x = 0) \/ (x = 1)}  = {0;1}` by ASM_SET_TAC[]

THEN ONCE_ASM_REWRITE_TAC[]

THEN RW_TAC std_ss [indep_sets]

       THEN1( FULL_SIMP_TAC std_ss[indep_sets])
       THEN1( FULL_SIMP_TAC std_ss [indep_sets]
       	      THEN  KNOW_TAC(``{PREIMAGE f (A:'b->bool) INTER p_space p |
   (f = X i) /\
   A IN measurable_sets M} = {PREIMAGE ((X:num->'a->'b) i) (A:'b->bool) INTER p_space p |
  A IN measurable_sets M}``)
                  THEN1( ASM_SET_TAC[])
	      THEN DISCH_TAC
	      THEN ASM_REWRITE_TAC[]
	      THEN ASM_SET_TAC[])
THEN FULL_SIMP_TAC std_ss[indep_sets]
THEN UNDISCH_TAC(``!J:num->bool.
             J SUBSET {s; t} /\ J <> {} /\ FINITE J ==>
             !A.
                 A IN
                 Pi J
                   (\i.
                        {PREIMAGE f A INTER p_space p |
                        (f = X i) /\ A IN measurable_sets M}) ==>
                 (prob p (BIGINTER {A j | j IN J}) =
                  Normal (product J (\j. real (prob p (A j)))))``)
		THEN DISCH_TAC
		THEN POP_ASSUM (MP_TAC o Q.SPEC `{s;t:num} `)
		THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN FULL_SIMP_TAC std_ss [Pi_iff]
		THEN KNOW_TAC(``!i.{PREIMAGE f A INTER p_space p |
             (f = (X:num->'a->'b) i) /\ A IN measurable_sets M} =
	     {PREIMAGE (X i) A INTER p_space p |
             A IN measurable_sets M} ``) THEN1(ASM_SET_TAC[])
 	        THEN DISCH_TAC
		THEN FULL_SIMP_TAC std_ss[]


		  THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) s) A INTER p_space p| A IN measurable_sets M } SUBSET events p``) THEN1(ASM_SET_TAC[])
		  THEN DISCH_TAC
		  THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) t) A INTER p_space p| A IN measurable_sets M } SUBSET events p``)THEN1(ASM_SET_TAC[])
		  THEN DISCH_TAC
		  THEN  `J SUBSET {0:num;1} = (J = {0:num} ) \/ ( J = {1:num}) \/ (J = {0:num;1})` by ASM_SET_TAC[]
		 THEN FULL_SIMP_TAC std_ss[]
(*3 subgoals*)		 THEN1( KNOW_TAC(``{(A:num->'a->bool) j | j IN {0}} = {A 0} ``)
			 	THEN1(SRW_TAC [] [EXTENSION, GSPECIFICATION])
				THEN DISCH_TAC
				THEN ASM_REWRITE_TAC[]
				THEN RW_TAC std_ss [PRODUCT_SING]
				THEN RW_TAC std_ss [BIGINTER_SING]
				THEN UNDISCH_TAC(`` !i:num.
             i IN {0} ==>
             (A:num->'a->bool) i IN
             {PREIMAGE f A INTER p_space p |
             (f = if i = 0 then (X:num->'a->'b) s else X t) /\ A IN measurable_sets M}``)
				THEN DISCH_TAC

				THEN POP_ASSUM (MP_TAC o Q.SPEC `0:num`)
				THEN `0:num IN {0:num} ` by ASM_SET_TAC[]
				THEN RW_TAC std_ss[]

				THEN `?A'.( A 0 = PREIMAGE (X s) A' INTER p_space p)/\ A' IN measurable_sets M` by 
				(  PSET_TAC[]
				 THEN ASM_REWRITE_TAC[])
				 THEN `random_variable (X s) p (m_space M, measurable_sets M)` by 
				 (UNDISCH_TAC(`` !i:num.
        i IN {s; t} ==>
       random_variable (X i) p
               (m_space M,measurable_sets M)``)

	   	      		      THEN DISCH_TAC
				      THEN POP_ASSUM (MP_TAC o Q.SPEC `s:num`)
				      THEN SRW_TAC[] [EXTENSION, GSPECIFICATION]
				      THEN FULL_SIMP_TAC std_ss[])
				 THEN `PREIMAGE (X s) (A':'b->bool) INTER p_space p IN events p` by 
				      (METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, random_variable_def, sigma_def, p_space_def, IN_SING, SUBSET_DEF, GSPECIFICATION, EXTENSION, IN_DEF])
				 THEN METIS_TAC [PROB_FINITE, normal_real])

(* 2 subgoals A 1*)      	THEN1 (KNOW_TAC(``{(A:num->'a->bool) j | j IN {1}} = {A 1} ``)
			 	THEN1(SRW_TAC [] [EXTENSION, GSPECIFICATION])
				THEN DISCH_TAC
				THEN ASM_REWRITE_TAC[]
				THEN RW_TAC std_ss [PRODUCT_SING]
				THEN RW_TAC std_ss [BIGINTER_SING]
				THEN UNDISCH_TAC(`` !i:num.
				        i IN {1} ==>
			        A i IN
			        {PREIMAGE f A INTER p_space p |
				         (f = if i = 0 then ((X:num->'a->'b) s) else (X t)) /\ A								 IN measurable_sets M}``)
				THEN DISCH_TAC

				THEN POP_ASSUM (MP_TAC o Q.SPEC `1:num`)
				THEN `1:num IN {1:num} ` by ASM_SET_TAC[]
				THEN RW_TAC std_ss[]

				THEN `?A'.( A 1 = PREIMAGE (X t) A' INTER p_space p)/\ A' IN measurable_sets M` by 
				( PSET_TAC[]
				 THEN ASM_REWRITE_TAC[])
				 THEN `random_variable (X t) p (m_space M, measurable_sets M)` by 
				 (UNDISCH_TAC(`` !i:num.
       i IN {s; t} ==>
             random_variable ((X:num->'a->'b) i) p
               (m_space M,measurable_sets M)``)

	   	      		      THEN DISCH_TAC
				      THEN POP_ASSUM (MP_TAC o Q.SPEC `t:num`)
				      THEN SRW_TAC[] [EXTENSION, GSPECIFICATION]
				      THEN FULL_SIMP_TAC std_ss[])
				 THEN `PREIMAGE (X t) (A':'b->bool) INTER p_space p IN events p` by   
				        (METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def, IN_SING, SUBSET_DEF, GSPECIFICATION, EXTENSION, IN_DEF])
				 THEN METIS_TAC [PROB_FINITE, normal_real])


(* 1 subgoal {0;1}*)
THEN KNOW_TAC(``{(A:num->'a->bool) j | j IN {0:num; 1}}={A 0; A 1}``) THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN `FINITE {0:num} /\ FINITE {1:num} /\ DISJOINT {0:num} {1:num}/\ ({0;1:num} = {0} UNION {1})` by SRW_TAC  [] [GSPECIFICATION, EXTENSION, FINITE_DEF, DISJOINT_DEF]
THEN RW_TAC std_ss [BIGINTER_INTER,PRODUCT_UNION, PRODUCT_SING]

THEN KNOW_TAC(``?A'.( (A:num->'a->bool) 0 = PREIMAGE ((X:num->'a->'b) s) A' INTER p_space p)/\ A' IN measurable_sets M``) 
     THEN1 (UNDISCH_TAC(``!i.
             i IN {0; 1:num} ==>
             (A:num->'a->bool) i IN
             {PREIMAGE f A INTER p_space p |
             (f = if i = 0 then (X:num->'a->'b) s else X t) /\ A IN measurable_sets M}``)
     THEN DISCH_TAC
     THEN POP_ASSUM (MP_TAC o Q.SPEC `(0:num) `)
     THEN FULL_SIMP_TAC std_ss[prove(``0 IN {0;(1:num)}``,
     SRW_TAC[][EXTENSION,GSPECIFICATION])]
      THEN `A 0 IN
	 {PREIMAGE f A INTER p_space p |
	  (f = X s) /\ A IN measurable_sets M} = A 0 IN
	  {PREIMAGE (X s) A INTER p_space p|
	  A IN measurable_sets M }` by  ASM_SET_TAC[]
         THEN ASM_REWRITE_TAC[]
        THEN PSET_TAC[])
THEN RW_TAC std_ss[]
THEN ASM_REWRITE_TAC[]

THEN `?A1'.( (A:num->'a->bool) (1:num) = PREIMAGE ((X:num->'a->'b) t) A1' INTER p_space p)/\
A1' IN measurable_sets M` by

  (UNDISCH_TAC(``!i.
             i IN {0; 1:num} ==>
             (A:num->'a->bool) i IN
             {PREIMAGE f A INTER p_space p |
             (f = if i = 0 then (X:num->'a->'b) s else X t) /\ A IN measurable_sets M}``)
     THEN DISCH_TAC
     THEN POP_ASSUM (MP_TAC o Q.SPEC `(1:num) `)
     THEN FULL_SIMP_TAC std_ss[prove(``1 IN {0;(1:num)}``,
     SRW_TAC[][EXTENSION,GSPECIFICATION])]
      THEN `A 1 IN
	 {PREIMAGE f A INTER p_space p |
	  (f = X t) /\ A IN measurable_sets M} = A 1 IN
	  {PREIMAGE (X t) A INTER p_space p|
	  A IN measurable_sets M }` by  ASM_SET_TAC[]
         THEN ASM_REWRITE_TAC[]
        THEN PSET_TAC[])

THEN ASM_REWRITE_TAC[]				
THEN UNDISCH_TAC(`` !A':num->'a->bool.(!i'.
                  i' IN {s:num; t} ==>
                  A' i' IN
                  {PREIMAGE ((X:num->'a->'b) i') A INTER p_space p | A IN measurable_sets M}) ==>
             (prob p (BIGINTER {A' j | (j = s) \/ (j = t)}) =
              Normal (product {s; t} (\j. real (prob p (A' j)))))``)
THEN DISCH_TAC
THEN POP_ASSUM (MP_TAC o Q.SPEC `(\i. if i = s then
				    PREIMAGE ((X:num->'a->'b) s) (A':'b->bool) INTER p_space p else
				    PREIMAGE (X t) (A1':'b->bool) INTER p_space p)`)
THEN DISCH_TAC
 THEN KNOW_TAC(``(!i'.
              i' IN {s:num; t} ==>
              (\i.
                   if i = s then PREIMAGE (X s) A' INTER p_space p
                   else PREIMAGE ((X:num->'a->'b) t) A1' INTER p_space p) i' IN
              {PREIMAGE (X i') A INTER p_space p | A IN measurable_sets M}) ``)
      THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``{if j = (s:num) then PREIMAGE (X s) A' INTER p_space p
              else PREIMAGE ((X:num->'a->'b) t) A1' INTER p_space p |
              j |
              (j = s) \/ (j = t)} =
	      {PREIMAGE (X s) (A':'b->bool) INTER p_space p;
	       PREIMAGE (X t) (A1':'b->bool) INTER p_space p}``)
     THEN1(RW_TAC std_ss [GSPECIFICATION, EXTENSION]
           THEN PSET_TAC [PREIMAGE_def]
           THEN EQ_TAC
	      THEN1(RW_TAC std_ss[]
	            THEN1( ASM_SET_TAC[])
	            THEN ASM_SET_TAC[])
	   THEN DISCH_TAC
          THEN RW_TAC std_ss[]
	  THEN1(Q.EXISTS_TAC`s:num`
		 THEN ASM_SET_TAC[])

          THEN Q.EXISTS_TAC`t:num`
	  THEN RW_TAC std_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN `FINITE {s:num} /\FINITE {t:num} /\ DISJOINT {s:num} {t:num} /\ ( {s:num;t} = {s} UNION {t}) `
  by SRW_TAC[] [GSPECIFICATION, EXTENSION]
THEN  FULL_SIMP_TAC std_ss [PRODUCT_UNION]
THEN FULL_SIMP_TAC std_ss [PRODUCT_SING]
THEN FULL_SIMP_TAC std_ss [BIGINTER_INTER]
THEN FULL_SIMP_TAC std_ss [REAL_MUL_COMM]);

val PRODUCT_UNION_3 = store_thm("PRODUCT_UNION_3",
  ``
     !s1 s2 s3  f.
       ALL_DISTINCT [s1; s2; s3] ==>
       (product {s1; s2; s3} f =
        f s1 * f s2 * f s3 )``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1} /\ FINITE {s2; s3} /\
                 DISJOINT {s1} {s2; s3} /\
		 ({s1; s2; s3} = {s1} UNION {s2; s3})``)
		 
     THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM] )
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s2} /\ FINITE {s3} /\ DISJOINT {s2} {s3} /\
     	         ({s2; s3} = {s2} UNION {s3})``)
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN RW_TAC std_ss [PRODUCT_SING]
THEN RW_TAC std_ss[REAL_MUL_ASSOC]);



val prob_prod_3_of_3 = store_thm("prob_prod_3_of_3",
  `` !p M X ii A  s1 s2 s3 .
       ALL_DISTINCT [s1;s2;s3]  /\
       prob_space p /\  indep_vars p M X ii /\
       {s1;s2;s3} SUBSET ii /\
       (!i.
          i IN {s1;s2;s3} ==>
	  A i IN measurable_sets (M i)) ==>
      (prob p
        ((PREIMAGE (X s1) (A s1) INTER p_space p) INTER
         (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
	 (PREIMAGE (X s3) (A s3) INTER p_space p) ) =
      prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
      prob p (PREIMAGE (X s2) (A s2) INTER p_space p) *
      prob p (PREIMAGE (X s3) (A s3) INTER p_space p))``,


RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
                    (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p |
		     i IN {s1:num;s2; s3}}) =
                  Normal
        	   (product {s1:num;s2;s3}
           	    (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
    THEN1(DEP_REWRITE_TAC [indep_varsD]
          THEN Q.EXISTS_TAC `M`
	  THEN Q.EXISTS_TAC `ii`
	  THEN RW_TAC std_ss[]
	  THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM, SUBSET_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p |
                  i IN {s1:num;s2;s3}} =
		 {PREIMAGE (X s1) (A s1) INTER p_space p;
		  PREIMAGE (X s2) (A s2) INTER p_space p;
		  PREIMAGE (X s3) (A s3) INTER p_space p}``)
      THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[BIGINTER_3_sets]
THEN RW_TAC std_ss[PRODUCT_UNION_3]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\
                 random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\
		 random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) ``)
      THEN1(FULL_SIMP_TAC std_ss [indep_vars]
            THEN UNDISCH_TAC(`` !i:num.
                                      i IN ii ==>
                                    random_variable ((X:num->'a->'b) i) p (m_space (M i),
				      measurable_sets (M i))``)
            THEN DISCH_TAC
	    THEN RW_TAC std_ss[]
	         THEN1( POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	                THEN ASM_SET_TAC[])
	         THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
		       THEN ASM_SET_TAC[])
		 THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
		       THEN ASM_SET_TAC[])
		 
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p ``)
	THEN1(REWRITE_TAC [events_def]
	      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	           THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
	      THEN DISCH_TAC
	      THEN ONCE_ASM_REWRITE_TAC[]
	      THEN REWRITE_TAC [p_space_def, m_space_def]
	      THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
	                        (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
				(A:num->'b->bool) s3 IN measurable_sets (M s3) ``)
                   THEN1(RW_TAC std_ss[]
		         THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3} ==>
			                       (A:num->'b->bool) i IN measurable_sets (M i)``)
                               THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
			       THEN ASM_SET_TAC[])
			 THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3} ==>
			                       (A:num->'b->bool) i IN measurable_sets (M i)``)
			       THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
			       THEN ASM_SET_TAC[])
			 THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3} ==>
			                        (A:num->'b->bool) i IN measurable_sets (M i)``)
				THEN DISCH_TAC
				THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
		         THEN ASM_SET_TAC[])
          THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_3, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);



val lem1 = store_thm("lem1",
  `` ! k x y z d. k <> PosInf /\ k <> NegInf/\
           x <> PosInf /\ x <> NegInf /\
           y <> PosInf /\ y <> NegInf /\
           z <> PosInf /\ z <> NegInf /\
	   d <> PosInf /\ d <> NegInf 
            ==> (k+x + -(y*z *d) = k+x- y* z*d)``,
Cases
THEN1( RW_TAC std_ss[])
THEN1(RW_TAC std_ss[])
THEN RW_TAC std_ss[]
THEN DEP_REWRITE_TAC[extreal_sub_add]
THEN RW_TAC std_ss[]
THEN KNOW_TAC(``y*z <> PosInf /\ y*z <> NegInf``)  THEN1(RW_TAC std_ss[mul_extreal_not_infty])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[mul_extreal_not_infty,add_not_infty]
THEN DISJ1_TAC
THEN RW_TAC std_ss[]
      THEN1(METIS_TAC [add_not_infty, extreal_not_infty])
THEN METIS_TAC [mul_extreal_not_infty] );     

val indep_vars3_def = Define `indep_vars3 X Y Z p = indep_vars p  (\i. lborel)
	 (\i. if i = 0 then (\s. real (X s))
	      else if i = 1 then  (\s. real (Y s))
	      else (\s. real ( Z s))) {0;1;2:num}`;

val prob_AND_FDEP_event = store_thm ("prob_AND_FDEP_event",
  `` !X Y Z p t.
       rv_gt0_ninfinity [X; Y;Z] /\ All_distinct_events  p [D_AND X Y; D_AND X Z]  t /\
        indep_vars3 X Y Z p==>
       (prob p (DFT_event p (D_AND (X)(FDEP Y Z)) t) =
        CDF p (\s. real (X s)) t *  CDF p (\s. real (Y s)) t +
	CDF p (\s. real (X s)) t *  CDF p (\s. real (Z s)) t -
        CDF p (\s. real (X s)) t *  CDF p (\s. real (Y s)) t *  CDF p (\s. real (Z s)) t)``,
RW_TAC std_ss[ AND_FDEP_event,All_distinct_events_def, rv_gt0_ninfinity_def, indep_vars3_def]
THEN FULL_SIMP_TAC std_ss [MAP]
THEN `indep_vars p (\i. if i = 0 then lborel else if i = 1 then lborel else lborel) (\i.
                if i = 0 then (\s. real (X s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s))) {0; 1; 2}` by RW_TAC std_ss[]
THEN KNOW_TAC(``indep_var p lborel (\x. real (X x)) lborel (\x. real (Y x))``)
     THEN1(MP_TAC (Q.ISPECL [`(\i.
                if i = 0:num then (\s. real ((X:'a->extreal) s))
                else if i = 1:num then (\s. real ((Y:'a->extreal) s))
                else (\s. real ((Z:'a->extreal) s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `(lborel)`, `0:num`, `1:num`, `{0;1;2:num}`] indep_vars_var)
	   THEN PSET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``indep_var p lborel (\x. real (X x)) lborel (\x. real (Z x)) ``)
     THEN1(MP_TAC (Q.ISPECL [`(\i.
                if i = 0:num then (\s. real ((X:'a->extreal) s))
                else if i = 1:num then (\s. real ((Y:'a->extreal) s))
                else (\s. real ((Z:'a->extreal) s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `(lborel)`, `0:num`, `2:num`, `{0;1;2:num}`] indep_vars_var)
	   THEN PSET_TAC[])
THEN DISCH_TAC
THEN  KNOW_TAC(``indep_var p lborel (\x. real (Y x)) lborel (\x. real (Z x)) ``)
     THEN1(MP_TAC (Q.ISPECL [`(\i.
                if i = 0:num then (\s. real ((X:'a->extreal) s))
                else if i = 1:num then (\s. real ((Y:'a->extreal) s))
                else (\s. real ((Z:'a->extreal) s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `(lborel)`, `1:num`, `2:num`, `{0;1;2:num}`] indep_vars_var)
	   THEN PSET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``random_variable (\s. real ((X:'a->extreal) s)) p borel /\
                 random_variable  (\s. real (Y s)) p borel /\
		  random_variable  (\s. real (Z s)) p borel``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars] 
     	   THEN UNDISCH_TAC(`` !i:num.
       i IN {0; 1; 2} ==>
             random_variable
               (if i = 0 then (\s. real (X s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s))) p
               (m_space lborel,measurable_sets lborel)``)
	  THEN DISCH_TAC
	  THEN RW_TAC std_ss []
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `0:num `)
	  	THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN METIS_TAC [subsets_def, measurable_sets_def, borel, lborel, sigma_def, m_space_def, space_def])
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `1:num `)
	  	THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN METIS_TAC [subsets_def, measurable_sets_def, borel, lborel, sigma_def, m_space_def, space_def])
	  THEN POP_ASSUM (MP_TAC o Q.SPEC `2:num `)
	  	THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN METIS_TAC [subsets_def, measurable_sets_def, borel, lborel, sigma_def, m_space_def, space_def])
THEN DISCH_TAC
THEN KNOW_TAC(``prob_space p``)
    THEN1(FULL_SIMP_TAC std_ss[indep_var, indep_vars, indep_sets])

THEN DISCH_TAC
THEN KNOW_TAC(``DFT_event p X t IN events p``)
     THEN1(RW_TAC std_ss [DFT_event_def]
      THEN KNOW_TAC(``{s:'a | X s <= Normal t}=PREIMAGE X {s |  s <= Normal t}``) THEN1(
      	             SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
      THEN DISCH_TAC
      THEN ASM_REWRITE_TAC[]
      THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
      THEN REWRITE_TAC [events_def]
      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``) THEN1(
	       	    		RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
      THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
      THEN REWRITE_TAC [p_space_def, m_space_def]
      THEN KNOW_TAC(``prob_space p``) THEN1(FULL_SIMP_TAC std_ss [indep_var, indep_vars, indep_sets])
      THEN DISCH_TAC
      THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets])
THEN DISCH_TAC

THEN KNOW_TAC(``DFT_event p Y t IN events p``)
     THEN1(RW_TAC std_ss [DFT_event_def]
      THEN KNOW_TAC(``{s:'a | Y s <= Normal t}=PREIMAGE Y {s |  s <= Normal t}``) THEN1(
      	             SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
      THEN DISCH_TAC
      THEN ASM_REWRITE_TAC[]
      THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
      THEN REWRITE_TAC [events_def]
      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``) THEN1(
	       	    		RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
      THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
      THEN REWRITE_TAC [p_space_def, m_space_def]
      THEN KNOW_TAC(``prob_space p``) THEN1(FULL_SIMP_TAC std_ss [indep_var, indep_vars, indep_sets])
      THEN DISCH_TAC
      THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets])
THEN DISCH_TAC
THEN KNOW_TAC(``DFT_event p Z t IN events p``)
     THEN1(RW_TAC std_ss [DFT_event_def]
      THEN KNOW_TAC(``{s:'a | Z s <= Normal t}=PREIMAGE Z {s |  s <= Normal t}``) THEN1(
      	             SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
      THEN DISCH_TAC
      THEN ASM_REWRITE_TAC[]
      THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
      THEN REWRITE_TAC [events_def]
      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``) THEN1(
	       	    		RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
      THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
      THEN REWRITE_TAC [p_space_def, m_space_def]
      THEN KNOW_TAC(``prob_space p``) THEN1(FULL_SIMP_TAC std_ss [indep_var, indep_vars, indep_sets])
      THEN DISCH_TAC
      THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets])
THEN DISCH_TAC

THEN KNOW_TAC(``DFT_event p (D_AND X Y) t UNION DFT_event p (D_AND X Z) t =
                 union_list[DFT_event p (D_AND X Y) t ;DFT_event p (D_AND X Z) t]``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, union_list_def])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN DEP_REWRITE_TAC[PROB_INCLUSION_EXCLUSION_PRINCIPLE]
THEN RW_TAC list_ss[]
     THEN1( RW_TAC std_ss[EVENTS_INTER, AND_INTER])
     THEN1(RW_TAC std_ss[EVENTS_INTER, AND_INTER])
THEN RW_TAC std_ss [sum_set_def]


THEN KNOW_TAC(``{t' | t' SUBSET {DFT_event p (D_AND X Y) t; DFT_event p (D_AND X Z) t} /\ t' <> {}}  =
                 {{DFT_event p (D_AND X Y) t}; {DFT_event p (D_AND X Z) t}; {DFT_event p (D_AND X Y) t; DFT_event p (D_AND X Z) t}}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
     THEN EQ_TAC
     	  THEN1(ASM_SET_TAC[])
     	  THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN DEP_REWRITE_TAC[PIE_lem2_or]
THEN RW_TAC std_ss[]
THEN1  (RW_TAC std_ss[extreal_pow_def]
       THEN DEP_REWRITE_TAC[PIE_lemma6]
       THEN DEP_REWRITE_TAC[PIE_lemma3]

THEN DEP_REWRITE_TAC[PROB_FINITE,EVENTS_COUNTABLE_INTER,COUNTABLE_IMAGE,finite_countable,IMAGE_FINITE]
       THEN (PSET_TAC[FINITE_SING,EVENTS_INTER,NOT_SING_EMPTY] THEN SRW_TAC[][FINITE_SING] THEN METIS_TAC[EVENTS_INTER, AND_INTER]))
THEN RW_TAC std_ss[]
THEN RW_TAC real_ss[BIGINTER_SING,CARD_SING,CARD_INSERT,GSYM ADD1]
THEN FULL_SIMP_TAC list_ss[ALL_DISTINCT]
THEN PSET_TAC[]
THEN RW_TAC real_ss[extreal_pow_def]
THEN SRW_TAC[][FINITE_SING]
THEN RW_TAC real_ss[pow,prove (``!A B:extreal->bool. A <> B  ==> ((CARD {A;B} = SUC (SUC 0)) )``, RW_TAC list_ss[ALL_DISTINCT] THEN DEP_REWRITE_TAC[CARD_DEF] THEN SRW_TAC[][FINITE_SING])]
THEN DEP_REWRITE_TAC[PIE_lemma5]
THEN DEP_REWRITE_TAC[PROB_FINITE,EVENTS_COUNTABLE_INTER,COUNTABLE_IMAGE,finite_countable,IMAGE_FINITE]
THEN RW_TAC std_ss[] THEN  PSET_TAC[AND_INTER,FINITE_SING,EVENTS_INTER,NOT_SING_EMPTY] THEN SRW_TAC[][FINITE_SING]
THEN RW_TAC std_ss[prove(``Normal 1 = 1``, RW_TAC std_ss[extreal_of_num_def] THEN METIS_TAC[]),mul_lone]
THEN RW_TAC std_ss[GSYM CDF_prob_DFT_event]
THEN RW_TAC std_ss [GSYM AND_INTER]

THEN KNOW_TAC(`` prob p (DFT_event p (D_AND X Y) t)  =
                 CDF p (\s. real (X s)) t * CDF p (\s. real (Y s)) t``)
     THEN1( DEP_REWRITE_TAC[PROB_AND]
            THEN RW_TAC std_ss[rv_gt0_ninfinity_def])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``prob p (DFT_event p (D_AND X Z) t) =
                CDF p (\s. real (X s)) t * CDF p (\s. real (Z s)) t``)
     THEN1(DEP_REWRITE_TAC[PROB_AND]
            THEN RW_TAC std_ss[rv_gt0_ninfinity_def])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]		
THEN KNOW_TAC(`` prob p (DFT_event p (D_AND (D_AND X Y) (D_AND X Z)) t) =
                CDF p (\s. real (X s)) t * CDF p (\s. real (Y s)) t *CDF p (\s. real (Z s)) t``)
     THEN1(RW_TAC std_ss[AND_INTER]
          THEN KNOW_TAC(``(DFT_event p X t INTER DFT_event p Y t INTER
      (DFT_event p X t INTER DFT_event p Z t)) =
        DFT_event p X t INTER DFT_event p Y t  INTER DFT_event p Z t ``)
               THEN1(METIS_TAC [INTER_ASSOC, INTER_COMM, INTER_IDEMPOT])
           THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
 	   THEN RW_TAC std_ss [DFT_event_def]
	   THEN `!X. {s| X s <= Normal t} = PREIMAGE X {s| s <= Normal t} ` by SRW_TAC [] [GSPECIFICATION, EXTENSION,PREIMAGE_def]
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
	   THEN `FINITE {0;1;2:num}` by SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
          if i = 0 then (\s. real ((X:'a->extreal) s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s))):num->'a->real`,`{0;1;2}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`,`2:num` ]  prob_prod_3_of_3)
	   THEN RW_TAC std_ss[]
	   THEN `ALL_DISTINCT [0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	   THEN `{0;1;2:num} SUBSET {0;1;2}` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
	   THEN `{u| u<=t} IN measurable_sets lborel`  by RW_TAC std_ss[lborel, measurable_sets_def, subsets_def, borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
	   THEN ` (!i. i IN {0; 1; 2} ==> {u | u <= t} IN measurable_sets lborel)` by RW_TAC std_ss[]
 	   THEN FULL_SIMP_TAC std_ss[]
	   THEN METIS_TAC [CDF_def, distribution_def, INTER_ASSOC])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``CDF p (\s. real (X s)) t <> PosInf /\ CDF p (\s. real (X s)) t<> NegInf ``)
     THEN1(REWRITE_TAC[CDF_def]
           THEN `{u| u<=t} IN measurable_sets lborel`  by RW_TAC std_ss[lborel, measurable_sets_def, subsets_def, borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
	   THEN `(PREIMAGE (\s. real (X s)){y| y<=t}  INTER p_space p) IN events p` by METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets]
            THEN RW_TAC std_ss[distribution_def,PROB_FINITE])
THEN DISCH_TAC

THEN KNOW_TAC(``CDF p (\s. real (Y s)) t <> PosInf /\ CDF p (\s. real (Y s)) t<> NegInf ``)
     THEN1(REWRITE_TAC[CDF_def]
           THEN `{u| u<=t} IN measurable_sets lborel`  by RW_TAC std_ss[lborel, measurable_sets_def, subsets_def, borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
	   THEN `(PREIMAGE (\s. real (Y s)){y| y<=t}  INTER p_space p) IN events p` by METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets]
            THEN RW_TAC std_ss[distribution_def,PROB_FINITE])
THEN DISCH_TAC

THEN KNOW_TAC(``CDF p (\s. real (Z s)) t <> PosInf /\ CDF p (\s. real (Z s)) t<> NegInf ``)
     THEN1(REWRITE_TAC[CDF_def]
           THEN `{u| u<=t} IN measurable_sets lborel`  by RW_TAC std_ss[lborel, measurable_sets_def, subsets_def, borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
	   THEN `(PREIMAGE (\s. real (Z s)){y| y<=t}  INTER p_space p) IN events p` by METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets]
            THEN RW_TAC std_ss[distribution_def,PROB_FINITE])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[lem1]
THEN RW_TAC std_ss[mul_extreal_not_infty]);

val _ = export_theory();
