

(******************************************************************************)
(*                      Probabilistic Analysis of                             *)
(*                        Cascased PAND Gates                                 *)
(*                                  by                                        *)
(*                          Yassmeen Elderhalli                               *)
(*                      y_elderh@encs.concordia.ca                            *)
(******************************************************************************)

(*loadPath := "/home/yassmeen/Downloads/ProbTheories_HOL-k12" :: !loadPath;
(*set_trace "Unicode" 0;*)

app load ["HolKernel", "Parse", "boolLib", "bossLib", "numLib", "unwindLib", 
"tautLib", "Arith", "prim_recTheory", "combinTheory", "quotientTheory", 
"arithmeticTheory", "realTheory", "hrealTheory", "realaxTheory", "realLib",  
"jrhUtils", "pairTheory", "seqTheory", "limTheory", "transcTheory", "listTheory", 
"mesonLib", "boolTheory", "topologyTheory", "pred_setTheory", "util_probTheory", 
"optionTheory", "numTheory", "sumTheory", "InductiveDefinition", "ind_typeTheory",
"pred_setLib", "iterate_hvgTheory", "card_hvgTheory", "product_hvgTheory", 
"topology_hvgTheory", "derivative_hvgTheory", "integration_hvgTheory",
"real_sigmaTheory", "extreal_hvgTheory", "measure_hvgTheory", "lebesgue_hvgTheory",
"probability_hvgTheory", "lebesgue_measure_hvgTheory", "normal_rv_hvgTheory", "netsTheory", "PIE_EXTREALTheory","dep_rewrite", "integration_before_afterTheory", "DFT_Gates_def_PROBTheory", "CASTheory","WCSPTheory", "WCSP_sharedTheory"];
*)
open HolKernel Parse boolLib bossLib numLib unwindLib tautLib Arith prim_recTheory 
combinTheory quotientTheory arithmeticTheory hrealTheory realaxTheory realTheory 
realLib jrhUtils pairTheory seqTheory limTheory transcTheory listTheory mesonLib 
boolTheory topologyTheory pred_setTheory util_probTheory optionTheory numTheory 
sumTheory InductiveDefinition ind_typeTheory pred_setLib iterate_hvgTheory 
card_hvgTheory product_hvgTheory topology_hvgTheory derivative_hvgTheory
integration_hvgTheory real_sigmaTheory extreal_hvgTheory measure_hvgTheory
lebesgue_hvgTheory probability_hvgTheory lebesgue_measure_hvgTheory normal_rv_hvgTheory netsTheory PIE_EXTREALTheory dep_rewrite integration_before_afterTheory DFT_Gates_def_PROBTheory CASTheory WCSPTheory WCSP_sharedTheory;


val _ = new_theory "CPDFT";


infixr 0 ++ << || ORELSEC ## --> THENC;
infix 1 >> |->;
fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
 
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Suff = PARSE_TAC SUFF_TAC;
val POP_ORW = POP_ASSUM (fn thm => ONCE_REWRITE_TAC [thm]);
val !! = REPEAT;
val op++ = op THEN;
val op<< = op THENL;
val op|| = op ORELSE;
val op>> = op THEN1;
val std_ss' = simpLib.++ (std_ss, boolSimps.ETA_ss);

(*---------------------------*)
fun SET_TAC L =
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION,
    IN_INTER, IN_DIFF, IN_INSERT, IN_DELETE, IN_BIGINTER, IN_BIGUNION,
    IN_IMAGE, GSPECIFICATION, IN_DEF] THEN METIS_TAC [];

fun SET_RULE tm = prove(tm,SET_TAC []);


val set_rewrs
= [INTER_EMPTY, INTER_UNIV, UNION_EMPTY, UNION_UNIV, DISJOINT_UNION,
DISJOINT_INSERT, DISJOINT_EMPTY, GSYM DISJOINT_EMPTY_REFL,
DISJOINT_BIGUNION, INTER_SUBSET, INTER_IDEMPOT, UNION_IDEMPOT,
SUBSET_UNION];

val UNIONL_def = Define `(UNIONL [] = {})
/\ (UNIONL (s::ss) = (s:'a ->bool) UNION UNIONL ss)`;


val IN_UNIONL = store_thm
("IN_UNIONL",
``!l (v:'a ). v IN UNIONL l = (?s. MEM s l /\ v IN s)``,
Induct >> RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY]
++ RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY, IN_UNION]
++ PROVE_TAC []);


val elt_rewrs
= [SUBSET_DEF, IN_COMPL, IN_DIFF, IN_UNIV, NOT_IN_EMPTY, IN_UNION,
IN_INTER, IN_IMAGE, IN_FUNSET, IN_DFUNSET, GSPECIFICATION,
DISJOINT_DEF, IN_BIGUNION, IN_BIGINTER, IN_COUNT, IN_o,
IN_UNIONL, IN_DELETE, IN_PREIMAGE, IN_SING, IN_INSERT];


fun rewr_ss ths =
simpLib.++
(std_ss,
simpLib.SSFRAG
{ac = [],
name = NONE,
convs = [],
dprocs = [],
filter = NONE,
rewrs = set_rewrs @ elt_rewrs,
congs = []});
val pset_set_ss = rewr_ss set_rewrs;
val pset_elt_ss = rewr_ss elt_rewrs;
val pset_ss = rewr_ss (set_rewrs @ elt_rewrs);


fun PSET_TAC ths =
REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_set_ss ths
++ REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_elt_ss ths;

fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Simplify = RW_TAC arith_ss;
val Suff = PARSE_TAC SUFF_TAC;
val Know = PARSE_TAC KNOW_TAC;
fun wrap a = [a];
val Rewr = DISCH_THEN (REWRITE_TAC o wrap);
val Rewr' = DISCH_THEN (ONCE_REWRITE_TAC o wrap);
val Cond =
  DISCH_THEN
  (fn mp_th =>
   let
     val cond = fst (dest_imp (concl mp_th))
   in
     KNOW_TAC cond << [ALL_TAC, DISCH_THEN (MP_TAC o MP mp_th)]
   end);
val FUN_EQ_TAC = CONV_TAC (CHANGED_CONV (ONCE_DEPTH_CONV FUN_EQ_CONV));
fun K_TAC _ = ALL_TAC;
fun MESON ths tm = prove(tm,MESON_TAC ths);
fun METIS ths tm = prove(tm,METIS_TAC ths);
val DISC_RW_KILL = DISCH_TAC THEN ONCE_ASM_REWRITE_TAC [] THEN 
                   POP_ASSUM K_TAC;
			   
val IN_REST = store_thm ("IN_REST",
 ``!x:'a. !s. x IN (REST s) <=> x IN s /\ ~(x = CHOICE s)``,
  REWRITE_TAC[REST_DEF, IN_DELETE]);
fun SET_TAC L = 
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN
    SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION, IN_INTER, IN_DIFF, 
      IN_INSERT, IN_DELETE, IN_REST, IN_BIGINTER, IN_BIGUNION, IN_IMAGE, 
      GSPECIFICATION, IN_DEF, EXISTS_PROD] THEN METIS_TAC [];
fun ASSERT_TAC tm = SUBGOAL_THEN tm STRIP_ASSUME_TAC;
fun SET_RULE tm = prove(tm,SET_TAC []);
fun ASM_SET_TAC L = REPEAT (POP_ASSUM MP_TAC) THEN SET_TAC L;
val ASM_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN ARITH_TAC;
val ASM_REAL_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN REAL_ARITH_TAC;

fun DFT_rewrs L =  REPEAT( METIS_TAC L);


val rewrs = [REAL_LE_REFL, REAL_LE_LT, REAL_LE_ANTISYM, REAL_LET_ANTISYM, REAL_LET_TOTAL, REAL_LET_TRANS, real_of_num,REAL_LT_NEG1_1,REAL_LT_NEG1_0,REAL_LE_NEG1_0,REAL_LE_NEG1_1, REAL_LT_TOTAL, max_def, min_def, REAL_LT_TRANS, REAL_LE_TRANS, REAL_MIN_COMM, REAL_MAX_COMM];

val DFT_rewrs_ss  =DFT_rewrs rewrs;



val CDF_GT0 = store_thm("CDF_GT0",
  `` !p X t. (!s. 0<= X s) ==> (CDF p X t = distribution p X {y | 0<= y /\ y<=t})
  ``,
  RW_TAC std_ss[CDF_def, distribution_def]
  THEN KNOW_TAC (`` PREIMAGE X {y:real | y <= t} = PREIMAGE X {y | 0<= y /\y <= t}``)
THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
THEN DISCH_TAC
THEN RW_TAC std_ss[]);


val CPDFT_reduce = store_thm("CPDFT_reduce",
  ``!X Y Z. (!s. ALL_DISTINCT [X s; Y s; Z s]) ==>
    (P_AND(P_AND Z Y) X = D_AND (D_AND (X)(D_BEFORE Z Y))(D_BEFORE Y X)) ``,
    RW_TAC std_ss[P_AND_def, D_AND_def, D_BEFORE_def]
    THEN KNOW_TAC(``!s.( if (if Z s <= Y s then Y s else PosInf) <= X s then X s else PosInf) =
   (
        max (max (X s) (if Z s < Y s then Z s else PosInf))
          (if Y s < X s then Y s else PosInf))``)

	  THEN1(RW_TAC std_ss[extreal_max_def] THEN REPEAT (POP_ASSUM MP_TAC) THEN RW_TAC std_ss[] THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [lt_infty, le_infty, lt_le, le_lt, extreal_lt_def, extreal_le_def, ALL_DISTINCT, MEM, lt_total, le_refl, le_total, lte_trans, let_trans ])
THEN RW_TAC std_ss[]);	  



val CPDFT_event = store_thm("CPDFT_event",
  ``!X Y Z t:real p. rv_gt0_ninfinity [X;Y;Z] /\ (!s. ALL_DISTINCT [X s; Y s ; Z s]) ==> (DFT_event p (D_AND (D_AND (X)(D_BEFORE Z Y))(D_BEFORE Y X)) t  = {s| 0 <= Z s /\ 0 <= Y s /\ 0 <= X s /\ Z s < Y s /\ Y s < X s /\ X s <= Normal t} INTER p_space p)``,
RW_TAC std_ss [AND_INTER, rv_gt0_ninfinity_def]
THEN RW_TAC std_ss [DFT_event_def, D_BEFORE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1 (RW_TAC std_ss[]
           THEN METIS_TAC [extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_trans, le_trans, lt_infty, le_infty, lt_refl, le_refl, ALL_DISTINCT, MEM])
THEN METIS_TAC [extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_trans, le_trans, lt_infty, le_infty, lt_refl, le_refl, ALL_DISTINCT, MEM]);	   

val PREIMAGE_CPDFT_event = store_thm("PREIMAGE_CPDFT_event",
  ``  !X Y Z t:real p.  rv_gt0_ninfinity [X;Y;Z] /\ (!s. ALL_DISTINCT [X s; Y s ; Z s]) ==> 
(DFT_event p (D_AND (D_AND (X)(D_BEFORE Z Y))(D_BEFORE Y X)) t = PREIMAGE (\s. (X s , Y s, Z s)) {(x,y,z)| 0 <= z  /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x  <= Normal t} INTER p_space p)``,

RW_TAC std_ss[CPDFT_event]
THEN RW_TAC std_ss[PREIMAGE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[] 
           THEN Q.EXISTS_TAC `(X x, Y x, Z x)`
           THEN FULL_SIMP_TAC std_ss[LAMBDA_PROD, PAIR])
THEN STRIP_TAC
THEN `x':extreal#extreal#extreal = (FST x', FST (SND x'), SND (SND x'))` by RW_TAC std_ss[PAIR, LAMBDA_PROD]
THEN KNOW_TAC (``(\ (x,y,z).
              ((x,y,z),
               0 <= z /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x <= Normal t))
           x' = ((FST x', FST (SND x'), SND(SND x')), 0<= SND (SND x') /\ 0<= FST (SND x') /\ 0<= FST x' /\ SND (SND x') < FST (SND x') /\ FST (SND x') < FST x' /\ FST x' <= Normal t) ``)
     THEN1(KNOW_TAC (``(\ (x,y,z).
              ((x,y,z),
               0 <= z /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x <= Normal t))
           x' = (\ (x,y,z).
              ((x,y,z),
               0 <= z /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x <= Normal t))
            (FST x', FST (SND x'), SND (SND x'))``)
	    THEN1(ONCE_ASM_REWRITE_TAC [] THEN  METIS_TAC [PAIR, LAMBDA_PROD])	    
	    THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
	    THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[PAIR, LAMBDA_PROD]
THEN KNOW_TAC(``(SND (SND (x':extreal#extreal#extreal)) = (Z:'a->extreal) x )/\
                 (FST x' = (X:'a->extreal) x )/\ ( FST(SND x') = (Y:'a->extreal) x)``)
     THEN1(`x':extreal#extreal#extreal = (FST x', FST (SND x'), SND (SND x'))` by METIS_TAC[PAIR, LAMBDA_PROD]
          THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
THEN DISCH_TAC THEN METIS_TAC [GSYM PAIR, LAMBDA_PROD]);	    


val PREIMAGE_3rv_EXTREAL_REAL_CPDFT = store_thm("PREIMAGE_3rv_EXTREAL_REAL_CPDFT",
  `` ! X Y Z p t. rv_gt0_ninfinity [X; Y; Z] /\ 0 <= t ==>
  (PREIMAGE (\s. (X s ,( Y s, Z s))) {(x,(y,z)) | 0 <= z  /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x  <= Normal t} INTER p_space p = PREIMAGE (\s. real(X s) ,( real(Y s), real(Z s))) {(x',(y',z'))| 0 <= z'  /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <=  t} INTER p_space p) ``,

SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def, rv_gt0_ninfinity_def]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
          THEN Q.EXISTS_TAC `(real (X x), real (Y x), real (Z x))`
	  THEN KNOW_TAC(``( \ (x,y,z).
              ((x,(y,z)),
               0 <= z /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x <= Normal t))
           x'  = 
              ((FST x',(FST (SND x'),SND (SND x'))),
               0 <= SND (SND x') /\ 0 <= FST (SND x') /\ 0 <= FST x' /\ SND (SND x') < FST (SND x') /\ FST (SND x') < FST x' /\ FST x' <= Normal t) ``)
	       THEN1(KNOW_TAC(``(\ (x,y,z).
        ((x,(y,z)),
               0 <= z /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x <= Normal t))
           x'  =
	 (\ (x,y,z).
        ((x,(y,z)),
               0 <= z /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x <= Normal t))
        (FST x', FST (SND x'), SND (SND x'))``)
	            THEN1(`x' = (FST x', FST(SND x'), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
		          THEN ONCE_ASM_REWRITE_TAC[]
			  THEN METIS_TAC[PAIR, LAMBDA_PROD])
		THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]	  
		THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
	 THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN ` 0 <> PosInf /\ 0 <> NegInf` by METIS_TAC [lt_infty, le_infty, extreal_of_num_def]
	 THEN `!s. X s <> NegInf /\ Y s <> NegInf /\ Z s <> NegInf` by   METIS_TAC [lt_infty, le_infty, extreal_of_num_def]
	 THEN RW_TAC std_ss[GSYM real_of_0]
	      THEN1(METIS_TAC[extreal_real_le])
	      THEN1(METIS_TAC [extreal_real_le])
	      THEN1(METIS_TAC [extreal_real_le])
 	      THEN1(FULL_SIMP_TAC std_ss[PAIR]
	            THEN METIS_TAC [extreal_real_lt, PAIR, LAMBDA_PROD])
 	      THEN1(FULL_SIMP_TAC std_ss[PAIR]THEN METIS_TAC [extreal_real_lt])
	 THEN RW_TAC std_ss[real_def]
	 THEN SELECT_ELIM_TAC
 	 THEN RW_TAC std_ss[]
	      THEN1(Q.EXISTS_TAC `real (X x)`
	            THEN METIS_TAC [normal_real])
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN METIS_TAC [extreal_le_def])
THEN STRIP_TAC
THEN RW_TAC std_ss[]
THEN Q.EXISTS_TAC `(X x, Y x, Z x)`
THEN KNOW_TAC (``(\ (x,y,z).
        ((x,(y,z)),
         0 <= z /\ 0 <= y /\ 0 <= x /\ z < y /\ y < x /\ x <= Normal t))
     (X x,Y x,Z x) =
     
        ((X x,(Y x,Z x)),
         0 <= Z x /\ 0 <= Y x /\ 0 <= X x /\ Z x < Y x /\ Y x< X x /\ X x <= Normal t)``)
     THEN1(RW_TAC std_ss[PAIR, LAMBDA_PROD])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]

THEN KNOW_TAC(``(\ (x',y',z').
              ((x',(y',z')),
               0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t:real))
           x' = 
              ((FST x',(FST (SND x'),SND (SND x'))),
               0 <= SND (SND x')  /\ 0 <= FST (SND x')  /\ 0 <= FST x'  /\ SND (SND x')  <FST (SND x')  /\ FST (SND x') < FST x' /\ FST x' <= t:real)``)
     THEN1(` (\ (x',y',z').
              ((x',y',z'),
               0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t:real))
           x'=
	   (\ (x',y',z').
              ((x',y',z'),
               0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t:real))
           (FST x', FST (SND x'), SND (SND x'))` by METIS_TAC[PAIR, LAMBDA_PROD]
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
THEN DISCH_TAC	   
THEN FULL_SIMP_TAC std_ss[]
THEN  ` 0 <> PosInf /\ 0 <> NegInf` by METIS_TAC [lt_infty, le_infty, extreal_of_num_def]
THEN `!s. X s <> NegInf /\ Y s <> NegInf /\ Z s <> NegInf` by   METIS_TAC [lt_infty, le_infty, extreal_of_num_def]
THEN RW_TAC std_ss[]
    THEN1(FULL_SIMP_TAC std_ss[PAIR] THEN METIS_TAC [extreal_real_lt])
    THEN1(FULL_SIMP_TAC std_ss[PAIR] THEN METIS_TAC [extreal_real_lt])
THEN `real (X x) <= t` by FULL_SIMP_TAC std_ss[]
THEN POP_ASSUM MP_TAC
THEN REWRITE_TAC[real_def]
THEN  SELECT_ELIM_TAC
THEN RW_TAC std_ss[]
     THEN1(Q.EXISTS_TAC ` real (X x)`
           THEN RW_TAC std_ss[normal_real])
THEN RW_TAC std_ss[extreal_le_def]);

val indicator_of_indicator_CPDFT = store_thm("indicator_of_indicator_CPDFT",
  `` ! x y z t:real. indicator_fn {(x',y',z') |
             0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} (x,y,z) =
	      indicator_fn {x'| 0<= x' /\ x' <=t } x *
	      indicator_fn {y' | 0<= y' /\ y' < x} y *
	      indicator_fn {z'| 0<= z' /\ z'< y} z``,

 
REPEAT GEN_TAC
THEN RW_TAC std_ss [indicator_fn_def]
  THEN1(METIS_TAC [mul_lone])

THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN METIS_TAC [mul_rzero, mul_comm]);


val BIGUNION_CPDFT_subevent0 = store_thm(
  "BIGUNION_CPDFT_subevent0",
  `` !t q0  q1 .BIGUNION {{x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
                         ({y'| 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS
		      {z' | 0<= z' /\ z' < real q0 }) | q0 IN Q_set} =
		      {x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
                         {(y',z')| 0 <= y' /\ y' < real q1 /\ z' < y'/\ 0<= z'}``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(STRIP_TAC
           THEN UNDISCH_TAC (`` !(x:real#real#real).
              
                 x IN s <=>
             (0 <= FST x /\ FST x <= t /\ real q1 < FST x) /\
             (0 <= FST (SND x) /\ FST (SND x) < real q1 /\
              real q0 < FST (SND x)) /\ 0 <= SND (SND x) /\
             SND (SND x) < real q0``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:real#real#real `)
          THEN `x:real#real#real = (FST x, FST (SND x), SND(SND x))` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `SND (x:real#real#real) = (FST (SND x), SND (SND x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EXISTS_TAC(``SND (x:real#real#real) ``)
           THEN RW_TAC std_ss [LAMBDA_PAIR]
	   THEN METIS_TAC [REAL_LT_TRANS])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]

THEN  KNOW_TAC(``?q0. q0 IN Q_set/\ Normal (SND (x':real#real) ) < q0 /\  q0 < Normal (FST x')``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r0. q0 = Normal r0``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (SND (SND (x:real#real#real))) < Normal( FST (SND x))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC

THEN EXISTS_TAC (``{x|   0 <= FST x /\ FST x <= t /\ real q1 < FST x /\
                0 <= FST (SND x) /\ FST (SND x) < real q1 /\
                 real (Normal r0) < FST (SND x) /\ 0 <= SND (SND x) /\
                SND (SND x) < real (Normal r0) }  ``)
     

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r0`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);


val CPDFT_subevent0_measurable_set = store_thm(
 "CPDFT_subevent0_measurable_set",
  ``!t q0 q1. {x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
            ({y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS
             {z' | 0 <= z' /\ z' < real q0}) IN measurable_sets (pair_measure lborel  (pair_measure lborel lborel) )``,

RW_TAC std_ss [measurable_sets_def, pair_measure, measure_of, lborel, m_space_def]
    THEN1 (KNOW_TAC(``  {x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
    	  		({y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS
			{z' | 0 <= z' /\ z' < real q0}) IN
                        {a CROSS b | a IN subsets borel /\
 			     b IN sigma_sets (univ(:real) CROSS univ(:real))
       {a CROSS b | a IN subsets borel /\ b IN subsets borel}}``)
           THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	         THEN EXISTS_TAC (``({x' | 0 <= x' /\ x' <= t /\ real q1 < x'},
        {y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS {z' | 0 <= z' /\ z' < real q0}) ``)
                THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                      THEN1(  KNOW_TAC(`` {x' | 0 <= x' /\ x' <= t /\ real q1 < x'} =
					    {x' | 0 <= x'} INTER
					    {x' | x' <= t} INTER
					    {x' | real q1 < x'} ``)
					    THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
					          THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC])
	                       THEN DISCH_TAC
			       THEN ASM_REWRITE_TAC[]
                               THEN `{x' | 0 <= x'} IN subsets borel /\
			             {x' | x' <= t} IN subsets borel /\
				     {x' | real q1 < x'} IN subsets borel` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_LEFT,CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT] 
 			       THEN  METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])

                     THEN KNOW_TAC(``  {y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS

		                        {z' | 0 <= z' /\ z' < real q0}  IN
				       {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
                          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			        THEN EXISTS_TAC (``( {y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'},
				     		   {z' | 0 <= z' /\ z' < real q0} ) ``)
			        THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                                     THEN1( KNOW_TAC(``   {y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} =
			    	 	       {y' | 0 <= y'} INTER
					       {y'| y' < real q1} INTER
					       {y' | real q0 < y'}``)
			        THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
				      THEN METIS_TAC[CONJ_ASSOC, CONJ_COMM])
 			   THEN DISCH_TAC				      
			   THEN ASM_REWRITE_TAC[]
			   THEN `{y' | 0 <= y'} IN subsets borel /\
				 {y' | y' < real q1} IN subsets borel /\
 				 {y' | real q0 < y'} IN subsets borel` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_LEFT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, CLOSED_INTERVAL_RIGHT]
    		          THEN 	METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
			  
                               THEN `{z' | 0 <= z' /\ z' < real q0} = {z' | 0 <= z'} INTER {z'| z' < real q0} ` 		   by SRW_TAC [] [GSPECIFICATION, EXTENSION]
              		       THEN ASM_REWRITE_TAC[]
	      		       THEN ` {z' | 0 <=  z'} IN subsets borel /\ {z' | z' < real q0} IN subsets borel`
	         by METIS_TAC [borel_open, borel_closed, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_LEFT]
             	    	      THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
              THEN DISCH_TAC
              THEN METIS_TAC [sigma_sets_basic])
	  THEN DISCH_TAC 
	  THEN METIS_TAC [sigma_sets_basic])    
   THEN1( KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
         THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
	         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(``{(a:real->bool) CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET
          POW univ(:real#real) ``)
             THEN1( METIS_TAC [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
	 THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[])
THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC (``univ(:real) CROSS univ(:real # real)  = univ(:real#real#real) ``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]	
THEN FULL_SIMP_TAC std_ss[GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV]);



val BIGUNION_CPDFT_subevent0_measurable_set = store_thm(
  "BIGUNION_CPDFT_subevent0_measurable_set",
  ``!t q1.   {x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
          {(y',z') | 0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z'} IN
	  measurable_sets (pair_measure lborel (pair_measure lborel lborel) )  ``,

RW_TAC std_ss [GSYM BIGUNION_CPDFT_subevent0]

THEN KNOW_TAC (``!q0. q0 IN Q_set ==> ({x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
     ({y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS
     {z' | 0 <= z' /\ z' < real q0}))   IN measurable_sets (pair_measure lborel (pair_measure lborel lborel) )``)
     THEN1(METIS_TAC[CPDFT_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
   BIGUNION
     {{x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
     ({y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS
     {z' | 0 <= z' /\ z' < real q0}) |
     q0 IN Q_set} = BIGUNION
  (IMAGE (\q0. {x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
     ({y' | 0 <= y' /\ y' < real q1 /\ real q0 < y'} CROSS
     {z' | 0 <= z' /\ z' < real q0}) )
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel (pair_measure lborel lborel) )``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel,sigma_finite_measure_pair_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );


val BIGUNION_CPDFT_subevent = store_thm(
  "BIGUNION_CPDFT_subevent",
  `` !t. BIGUNION
           {{x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
          {(y',z') | 0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z'} |
           q1 IN Q_set}  =
	   {(x',(y',z')) |
      0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t}``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN REWRITE_TAC[LAMBDA_PAIR]
	   
           THEN UNDISCH_TAC (`` !x.
               x IN s <=>
             (0 <= FST x /\ FST x <= t /\ real q1 < FST x) /\
             ?x'.
                 (SND x,T) =
                 (\ (y',z').
                      ((y',z'),0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z')) x'``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:real#real#real `)
          THEN `x:real#real#real = (FST x, FST (SND x), SND(SND  x))` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `SND (x:real#real#real) = (FST (SND x), SND (SND x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN KNOW_TAC(`` (\ (y':real,z':real). ((y',z'),0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z'))
           x''
              = (x'', 0<= FST x'' /\ FST  x'' < real q1 /\ SND x''< FST x'' /\ 0<= SND x'') ``)
                THEN1 ( KNOW_TAC (``  ( \(y',z'). ((y',z'),0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z')) x'' =   (\ (y',z'). ((y',z'),0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z'))  (FST x'' , SND x'') ``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC
	 THEN Q.EXISTS_TAC(`(FST x, FST (SND x), SND (SND x)) `)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]
THEN  KNOW_TAC(``?q1. q1 IN Q_set/\ Normal (FST (SND(x':real#real#real) )) < q1 /\  q1 < Normal (FST x')``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r1. q1 = Normal r1``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (FST (SND(x:real#real#real))) < Normal  (FST x)``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{x| 0 <= FST x /\ FST x <= t /\ real (Normal r1) < FST x /\
                0 <= FST (SND x) /\ FST (SND x) < real (Normal r1) /\
                SND (SND x) < FST (SND x) /\ 0 <= SND (SND x)}``)
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r1`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);



val CPDFT_event_measurable_set = store_thm(
  "CPDFT_event_measurable_set",
  `` !t.
	  {(x',(y',z')) |
             0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} IN
       measurable_sets (pair_measure lborel (pair_measure lborel lborel) )``,


RW_TAC std_ss [GSYM BIGUNION_CPDFT_subevent]
THEN KNOW_TAC (``!q1. q1 IN Q_set ==> ({x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
     {(y',z') | 0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z'})   IN measurable_sets (pair_measure lborel (pair_measure lborel lborel) )``)
     THEN1(METIS_TAC[BIGUNION_CPDFT_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
     {{x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
     {(y',z') | 0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z'} |
     q1 IN Q_set} = BIGUNION
  (IMAGE (\q1. {x' | 0 <= x' /\ x' <= t /\ real q1 < x'} CROSS
     {(y',z') | 0 <= y' /\ y' < real q1 /\ z' < y' /\ 0 <= z'})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel (pair_measure lborel lborel) )``)
     THEN1(METIS_TAC[ sigma_finite_measure_pair_lborel,measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );






val prob_event_3rv_2indep_thd = store_thm("prob_event_3rv_2indep_thd",
  `` ! p X Y Z M1 M2  A.

prob_space p /\ measure_space M1 /\ measure_space M2 /\
     sigma_finite_measure M1 /\ sigma_finite_measure M2 /\
     indep_varp p M1 X M2  (\x. (Y x,Z x))  /\
     A IN measurable_sets (pair_measure M1 M2) ==>
     (prob p (PREIMAGE (\x. (X x,Y x,Z x)) A INTER p_space p) =
      pos_fn_integral (distr p M1 X)
        (\x.
           pos_fn_integral (distr p M2 (\x. (Y x,Z x)))
             (\(y,z). indicator_fn A (x,y,z))))``,

RW_TAC std_ss []
THEN KNOW_TAC(``prob p (PREIMAGE (\x. ((X:'a->'b) x,(Y:'a->'c) x,(Z:'a->'d) x)) A INTER p_space p) =
                 distribution p (\x. (X x, Y x, Z x)) A  ``)
     THEN1(METIS_TAC [distribution_def])		  
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(\x. ((Y:'a->'c) x, (Z:'a->'d) x)) IN measurable (m_space p, measurable_sets p) (m_space M2, measurable_sets M2) /\ (X:'a->'b) IN measurable (m_space p, measurable_sets p) (m_space M1, measurable_sets M1)``)
     THEN1(METIS_TAC [random_variable_def, indep_varp_def, p_space_def, events_def])
THEN DISCH_TAC

THEN KNOW_TAC(``(\x. (X:'a->'b) x, (Y:'a-> 'c) x , (Z:'a->'d) x )  IN measurable (m_space p, measurable_sets p) (m_space (pair_measure M1 M2), measurable_sets (pair_measure M1 M2))``)
     THEN1(METIS_TAC[p_space_def, m_space_def, measurable_Pair, prob_space_def])
THEN DISCH_TAC
THEN KNOW_TAC (``measure_space (pair_measure (M1: ('b->bool)#(('b->bool)->bool)#(('b->bool)->extreal))(M2: ('c#'d->bool)#(('c#'d->bool)->bool)#(('c#'d->bool)->extreal)) )``)
     THEN1(METIS_TAC [measure_space_pair_measure'])
THEN DISCH_TAC
THEN KNOW_TAC(``random_variable (\x. (X:'a->'b) x, (Y:'a->'c) x , (Z:'a->'d) x) p   (m_space (pair_measure M1 M2) , measurable_sets (pair_measure M1 M2)) ``)
    THEN1(METIS_TAC [random_variable_def, measurable_Pair, p_space_def, prob_space_def, events_def, subsets_def, measurable_sets_def, m_space_def, space_def] ) 
THEN DISCH_TAC
THEN KNOW_TAC(``distribution p (\x. ((X:'a->'b) x,(Y:'a->'c) x,(Z:'a->'d) x)) A =
                 pos_fn_integral (distr p (pair_measure M1 M2) (\x. ((X:'a->'b) x,(Y:'a->'c) x,(Z:'a->'d) x)) ) (indicator_fn A)``)

     THEN1( RW_TAC std_ss [distribution_lebesgue_thm2_pos_fn])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]


THEN RW_TAC std_ss [indep_varp_pair_distr]
THEN DEP_REWRITE_TAC [GSYM pos_fn_integral_fst]
THEN RW_TAC std_ss []
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
     THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A'`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'b->bool) IN measurable_sets M1``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE  (X: 'a->'b)  a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
    THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A''`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'c#'d->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (\x. ((Y:'a->'c)x , (Z:'a->'d) x)) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
    THEN1(METIS_TAC [indicator_fn_pos_le])
    THEN1( MP_TAC(Q.ISPECL[`(m_space (pair_measure (distr p M1 (X:'a->'b)) (distr p M2 (\x. ((Y:'a->'c) x, (Z:'a->'d) x)) )),
   measurable_sets
     (pair_measure (distr p M1 X ) (distr p M2 (\x. (Y x, Z x)))))` ,`1:real` ,`A:('b#'c#'d)->bool` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)
          THEN RW_TAC std_ss[]
          THEN KNOW_TAC(``sigma_finite_measure (distr p M2 (\x. ((Y:'a->'c)x, (Z:'a->'d) x)))``)
	      THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A''`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'c#'d->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (\x. ((Y:'a->'c) x, (Z:'a->'d)x)) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
	  THEN DISCH_TAC	   
          THEN KNOW_TAC(``measure_space (pair_measure (distr p M1 X ) (distr p M2 (\x. ((Y:'a->'c) x, (Z:'a->'d) x ))))``)
               THEN1 (METIS_TAC [measure_space_pair_measure', measure_space_distr, random_variable_def, prob_space_def, p_space_def, m_space_def, events_def, subsets_def, measurable_sets_def] )
	  THEN DISCH_TAC
	  THEN KNOW_TAC(``sigma_algebra (m_space (pair_measure (distr p M1 X ) (distr p M2 (\x. ((Y:'a->'c) x, (Z:'a->'d) x )))),
   measurable_sets
     (pair_measure (distr p M1 X ) (distr p M2 (\x. ((Y:'a->'c) x, (Z:'a->'d) x )))))``)
               THEN1(FULL_SIMP_TAC std_ss [measure_space_def])
          THEN DISCH_TAC
          THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(``A IN
       subsets
         (m_space
            (pair_measure (distr p M1 X ) (distr p M2 (\x. ((Y:'a->'c) x, (Z:'a->'d) x )))),
          measurable_sets
            (pair_measure (distr p M1 X ) (distr p M2 (\x. ((Y:'a->'c) x, (Z:'a->'d) x )))))``)
	       THEN1(RW_TAC std_ss [subsets_def]
	             THEN RW_TAC std_ss [distr]
	             THEN RW_TAC std_ss [distr, measurable_sets_def, subsets_def, pair_measure, measure_of]
                          THEN1(FULL_SIMP_TAC std_ss[m_space_def]
			  THEN FULL_SIMP_TAC std_ss [pair_measure]
			  THEN FULL_SIMP_TAC std_ss [measurable_sets_def, subsets_def, pair_measure, measure_of])
	             THEN FULL_SIMP_TAC std_ss[m_space_def]
			  THEN FULL_SIMP_TAC std_ss [pair_measure]
			  THEN FULL_SIMP_TAC std_ss [measurable_sets_def, subsets_def, pair_measure, measure_of])
        THEN DISCH_TAC
	THEN FULL_SIMP_TAC std_ss[]
	THEN FULL_SIMP_TAC std_ss [REAL_LE_01]
	THEN KNOW_TAC(``indicator_fn (A:'b#'c#'d->bool) = (\x. Normal 1 * indicator_fn A x)``)
             THEN1(`indicator_fn (A:'b#'c#'d->bool) = (\x.  indicator_fn (A:'b#'c#'d->bool) x)` by  METIS_TAC [ETA_THM]
	            THEN ONCE_ASM_REWRITE_TAC[]
		    THEN `(\x. indicator_fn (A:'b#'c#'d->bool) x) = (\x. 1:extreal * indicator_fn (A:'b#'c#'d->bool) x)` by METIS_TAC [mul_lone]
		   THEN METIS_TAC [extreal_of_num_def])
	THEN DISCH_TAC
	THEN ONCE_ASM_REWRITE_TAC[]
	THEN FULL_SIMP_TAC std_ss[])
THEN RW_TAC std_ss [LAMBDA_PROD, PAIR]);


val prob_CPDFT_initial = store_thm("prob_CPDFT_initial",
  `` ! X Y Z p t. prob_space p/\
  0<= t /\ indep_varp p lborel (\x. real (X x)) (pair_measure lborel lborel)
  (\x. (real (Y x), real (Z x))) 
  /\  rv_gt0_ninfinity [X;Y;Z] /\ (!s. ALL_DISTINCT [X s; Y s ; Z s]) /\
  {(x',y',z') |
         0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} IN measurable_sets (pair_measure lborel (pair_measure lborel lborel))==>

(prob p (DFT_event p (D_AND (D_AND (X)(D_BEFORE Z Y))(D_BEFORE Y X)) t) =

pos_fn_integral (distr p lborel (\x. real (X x)))
             (\ x.
	     
                  pos_fn_integral (distr p (pair_measure lborel lborel) (\x. (real (Y x),real(Z x))))
                    (\ (y,z). indicator_fn {(x',y',z') |
         0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} (x,y,z))) )  ``,


RW_TAC std_ss[PREIMAGE_CPDFT_event]
THEN RW_TAC std_ss[PREIMAGE_3rv_EXTREAL_REAL_CPDFT]
THEN MP_TAC (Q.ISPECL [`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)` ,
`(\x. real ((X:'a->extreal) x))`,`(\x. real ((Y:'a->extreal) x))`,
`(\x. real ((Z:'a->extreal) x))`, `lborel`,`pair_measure lborel lborel`, ` {(x',y',z') |
         0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t:real}`] prob_event_3rv_2indep_thd)
THEN RW_TAC std_ss[measure_space_lborel, measure_space_pair_measure', sigma_finite_measure_lborel, sigma_finite_measure_pair_lborel]
THEN FULL_SIMP_TAC std_ss[CPDFT_event_measurable_set]);



val prob_CPDFT_initial2 = store_thm("prob_CPDFT_initial2",
  ``! X Y Z A p f_x . prob_space p  /\ random_variable (\x. real (X x)) p (m_space lborel, measurable_sets lborel)
  /\
  random_variable (\x. real (Y x)) p (m_space lborel, measurable_sets lborel) /\ random_variable (\x. real (Z x)) p (m_space lborel, measurable_sets lborel) /\
  A IN measurable_sets (pair_measure lborel (pair_measure lborel lborel ) ) /\

  distributed p lborel (\x. real (X x)) f_x /\ (!x. 0<= f_x x) ==> ( 
  pos_fn_integral (distr p lborel (\x. real (X x)))
             (\x.
                  pos_fn_integral
		  
                    (distr p (pair_measure lborel lborel)

(\x. (real (Y x),real (Z x))))
                    (\ (y,z).
                         indicator_fn
                           A  (x,y,z))) =
  pos_fn_integral lborel 
             (\x. f_x x *
                  pos_fn_integral
                    (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\ (y,z).
                         indicator_fn
                           A (x,y,z))))``,

RW_TAC std_ss[distributed_def]
THEN MP_TAC(Q.ISPECL [`f_x :real->extreal`, `(\x:real.
          pos_fn_integral
            (distr p (pair_measure lborel lborel)
               (\x. (real ((Y:'a->extreal) x),real (Z x))))
            (\ (y:real,z:real). indicator_fn A (x,y,z)))`, `lborel`]pos_fn_integral_density_1)

THEN RW_TAC std_ss [measure_space_lborel, sigma_finite_measure_lborel]
THEN KNOW_TAC(``!(x:real). 0 <= pos_fn_integral
                (distr p (pair_measure lborel lborel)
                   (\x. (real (Y x),real (Z x))))
                (\ (y,z). indicator_fn A (x,y,z))``)

     THEN1(RW_TAC std_ss[]
           THEN MP_TAC(Q.ISPECL[`(distr p (pair_measure lborel lborel) (\x. (real (Y x),real (Z x))))`, ` (\ (y:real,z:real). indicator_fn A (x:real,y,z))  `]  pos_fn_integral_pos)
           THEN RW_TAC std_ss[indicator_fn_pos_le]

	   THEN KNOW_TAC(``(\x. ( real ((Y:'a->extreal) x), real ((Z:'a->extreal) x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel)) /\ (\x. real ((X:'a->extreal)x)) IN measurable (m_space p, measurable_sets p) (m_space lborel, measurable_sets lborel)``)
     THEN1(RW_TAC std_ss[]
           THEN FULL_SIMP_TAC std_ss[random_variable_def, p_space_def, events_def]THEN METIS_TAC [random_variable_def, indep_varp_def, p_space_def, events_def, measure_space_lborel, sigma_finite_measure_lborel, measure_space_pair_measure', measurable_Pair, prob_space_def])
           THEN DISCH_TAC

            THEN `measure_space
           (distr p (pair_measure lborel lborel)
              (\x. (real ((Y:'a->extreal) x),real (Z x))))` by METIS_TAC [measure_space_distr,random_variable_def, measure_space_lborel, prob_space_def, p_space_def, events_def, measurable_sets_def, subsets_def, space_def, measure_space_pair_measure', sigma_finite_measure_lborel]
           THEN FULL_SIMP_TAC std_ss[indicator_fn_pos_le, ETA_THM, PAIR, LAMBDA_PROD]
	   THEN `!(p1:real,p2:real). T` by RW_TAC std_ss[GSYM LAMBDA_PROD, PAIR]
	   THEN FULL_SIMP_TAC std_ss[])
	   
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]

THEN KNOW_TAC(``(\x:real.
              pos_fn_integral
                (distr p (pair_measure lborel lborel)
                   (\x. (real ((Y:'a->extreal) x),real (Z x))))
                (\(y:real,z:real). indicator_fn A (x,y,z))) IN
         measurable (m_space lborel,measurable_sets lborel) Borel``)

     THEN1(MP_TAC(Q.ISPECL[` indicator_fn (A:(real#real#real->bool)) `,`lborel `, `(distr p (pair_measure lborel lborel) (\x. (real ((Y:'a->extreal) x),real ((Z:'a->extreal) x))))`] borel_measurable_pos_integral_fst)
     
             THEN RW_TAC std_ss [measure_space_pair_measure', sigma_finite_measure_lborel, measure_space_lborel]
	      THEN  KNOW_TAC(``(\x. ( real ((Y:'a->extreal) x), real ((Z:'a->extreal) x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel)) /\ (\x. real ((X:'a->extreal)x)) IN measurable (m_space p, measurable_sets p) (m_space lborel, measurable_sets lborel)``)
     THEN1(RW_TAC std_ss[]
           THEN FULL_SIMP_TAC std_ss[random_variable_def, p_space_def, events_def]THEN METIS_TAC [random_variable_def, indep_varp_def, p_space_def, events_def, measure_space_lborel, sigma_finite_measure_lborel, measure_space_pair_measure', measurable_Pair, prob_space_def])
           THEN DISCH_TAC

            THEN `measure_space
           (distr p (pair_measure lborel lborel)
              (\x. (real ((Y:'a->extreal) x),real (Z x))))` by METIS_TAC [measure_space_distr,random_variable_def, measure_space_lborel, prob_space_def, p_space_def, events_def, measurable_sets_def, subsets_def, space_def, measure_space_pair_measure', sigma_finite_measure_lborel]
           THEN FULL_SIMP_TAC std_ss[indicator_fn_pos_le, ETA_THM, PAIR, LAMBDA_PROD]
	   THEN `!(p1:real,p1':real, p2':real). T` by RW_TAC std_ss[GSYM LAMBDA_PROD, PAIR]
	   THEN FULL_SIMP_TAC std_ss[]
	   THEN `sigma_finite_measure
           (distr p (pair_measure lborel lborel)
              (\x. (real ((Y:'a->extreal) x),real (Z x))))` by METIS_TAC[sigma_finite_measure_distr_pair_lborel, random_variable_def, m_space_def, p_space_def, events_def]
	   THEN KNOW_TAC(`` indicator_fn A IN
         measurable
           (m_space
              (pair_measure lborel
                 (distr p (pair_measure lborel lborel)
                    (\x. (real ((Y:'a->extreal) x),real ((Z:'a->extreal) x))))),
            measurable_sets
              (pair_measure lborel
                 (distr p (pair_measure lborel lborel)
                    (\x. (real (Y x),real (Z x)))))) Borel``)
	      THEN1(`(m_space
              (pair_measure lborel
                 (distr p (pair_measure lborel lborel)
                    (\x. (real ((Y:'a->extreal) x),real ((Z:'a->extreal) x))))),
            measurable_sets
              (pair_measure lborel
                 (distr p (pair_measure lborel lborel)
                    (\x. (real (Y x),real (Z x)))))) =
		    (m_space (pair_measure lborel (pair_measure lborel lborel)), measurable_sets (pair_measure lborel (pair_measure lborel lborel)))` by RW_TAC std_ss[pair_measure, measure_of, lborel, distr, m_space_def, measurable_sets_def]
		    THEN ONCE_ASM_REWRITE_TAC[]

		    THEN MP_TAC(Q.ISPECL[`(m_space (pair_measure lborel (pair_measure lborel lborel) ),
    measurable_sets
     (pair_measure lborel (pair_measure lborel lborel)))` ,`1:real` ,`A:real#real#real->bool` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)

             
          THEN RW_TAC std_ss [ REAL_LE_01]
	  THEN FULL_SIMP_TAC std_ss[]
	  THEN `measure_space (pair_measure lborel (pair_measure lborel lborel))` by METIS_TAC[measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel, sigma_finite_measure_pair_lborel]
	  THEN `sigma_algebra
         (m_space (pair_measure lborel (pair_measure lborel lborel)),
          measurable_sets
            (pair_measure lborel (pair_measure lborel lborel) ))` by FULL_SIMP_TAC std_ss[measure_space_def]
	    THEN FULL_SIMP_TAC std_ss[]
	  
          THEN ` A IN
         subsets
        (m_space (pair_measure lborel (pair_measure lborel lborel)),
         measurable_sets
           (pair_measure lborel (pair_measure lborel lborel) ))` by METIS_TAC[pair_measure, lborel, measure_of, subsets_def, m_space_def, measurable_sets_def]
	   THEN FULL_SIMP_TAC std_ss[]
          THEN KNOW_TAC(``(\ (x:real#real#real). Normal 1 * indicator_fn A x) = (\x. indicator_fn A x)``)
               THEN1(METIS_TAC [extreal_of_num_def, mul_lone, PAIR, LAMBDA_PROD])
          THEN DISCH_TAC
          THEN MP_TAC(Q.ISPECL [`indicator_fn (A:real#real#real->bool) `]LAMBDA_PROD)                       THEN DISCH_TAC
		     
          THEN FULL_SIMP_TAC std_ss[]
          THEN METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])

THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN `!x:real . x IN m_space lborel` by RW_TAC std_ss[m_space_def, IN_UNIV, lborel]
THEN FULL_SIMP_TAC std_ss[]
THEN `!(p1:real) (p2:real) (x:real). max 0 (indicator_fn A (x:real, p1, p2)) =indicator_fn A (x,p1,p2)` by RW_TAC std_ss[extreal_max_def, indicator_fn_pos_le] 
THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]);




(************************)

val measure_space_distr_pair = store_thm("measure_space_distr_pair",
  ``  
! X Y p M1 M2. measure_space M1 /\ measure_space M2 /\ sigma_finite_measure M2 /\ random_variable X p(m_space M1, measurable_sets M1) /\ random_variable Y p (m_space M2, measurable_sets M2) ==> measure_space (distr p (pair_measure M1 M2) (\x. X x, Y x))``,
RW_TAC std_ss[random_variable_def, p_space_def, events_def]
THEN KNOW_TAC(``(\x. (X x, Y x)) IN measurable (m_space p, measurable_sets p)(m_space (pair_measure M1 M2), measurable_sets (pair_measure M1 M2)) ``)
    THEN1(METIS_TAC [prob_space_def, measurable_Pair])
THEN DISCH_TAC
THEN METIS_TAC [measure_space_distr, measure_space_pair_measure', prob_space_def]);


val CPDFT_A_Ayz_indicator_of_indicator = store_thm(
  "CPDFT_A_Ayz_indicator_of_indicator",
  `` !x y z t:real.  indicator_fn {(x',y',z') |
          0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} (x,y,z) =
	     indicator_fn {x'| 0<= x' /\ x' <= t} x*
	     indicator_fn {(y',z') |  0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x } (y,z)``,

RW_TAC std_ss[indicator_fn_def]

  THEN1(METIS_TAC [mul_lone])

THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
       THEN `(y,z) IN {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} =
             (y,z) IN {(y',z') |(\y'' z''.  0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
     
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
       THEN `(y,z) IN {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} =
             (y,z) IN {(y',z') |(\y'' z''.  0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
     
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      
THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
       THEN `(y,z) IN {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} =
             (y,z) IN {(y',z') |(\y'' z''.  0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
     
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`(x:real,y,z) IN
      {(x',y',z') |
       0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} <=>
      (x,y,z) IN
      {(x',y',z') |
       (\x'' y'' z''.
           0 <= z'' /\ 0 <= y'' /\ 0 <= x'' /\ z'' < y'' /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
       THEN `(y,z) IN {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} =
             (y,z) IN {(y',z') |(\y'' z''.  0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
     
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN METIS_TAC [mul_rzero, mul_comm]);

val real_indicator_fn_pos_le = store_thm("real_indicator_fn_pos_le",
  `` ! A s. 0<= real (indicator_fn A s) ``,
RW_TAC std_ss [real_def, indicator_fn_def]
    THEN1(RW_TAC std_ss [REAL_LE_REFL])
    THEN1(RW_TAC std_ss [REAL_LE_REFL])
    THEN1(SELECT_ELIM_TAC
           THEN RW_TAC std_ss[]
                THEN1( Q.EXISTS_TAC `1`
                      THEN RW_TAC std_ss[normal_real, extreal_of_num_def] )
                      THEN  `x:real = 1` by METIS_TAC[extreal_11, extreal_of_num_def]
                      THEN METIS_TAC [REAL_LE_01])
THEN SELECT_ELIM_TAC		 
THEN RW_TAC std_ss[]		    
     THEN1(Q.EXISTS_TAC`0`
          THEN RW_TAC std_ss [normal_real, extreal_of_num_def])
THEN  `x:real = 0` by METIS_TAC [extreal_11, extreal_of_num_def]
THEN METIS_TAC [REAL_LE_REFL]);



val prob_CPDFT_initial3 = store_thm("prob_CPDFT_initial3",
  `` ! Y Z f_x p t:real.prob_space p /\ random_variable (\x. real (Y x)) p
            (m_space lborel,measurable_sets lborel) /\
          random_variable (\x. real (Z x)) p
            (m_space lborel,measurable_sets lborel) ==>
	   ( pos_fn_integral lborel
             (\x.
                  f_x x *
                  pos_fn_integral
                    (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\ (y,z). indicator_fn {(x',y',z') |
          0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} (x,y,z))) =
	     pos_fn_integral lborel
             (\x.
                  f_x x * indicator_fn {x'| 0<= x' /\ x' <= t} x*

                pos_fn_integral
                    (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\ (y,z). indicator_fn {(y',z') |
          0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x } (y,z))))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``!x. pos_fn_integral
                    (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\ (y,z). indicator_fn {(x',y',z') |
          0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} (x,y,z))=
	       indicator_fn {x'| 0<= x' /\ x' <= t} x*

                pos_fn_integral
                    (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\ (y,z). indicator_fn {(y',z') |
          0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x } (y,z))  ``)

     THEN1(RW_TAC std_ss[]
           THEN MP_TAC (Q.ISPECL [`distr p (pair_measure lborel lborel) (\x. real ((Y:'a->extreal) x), real ((Z:'a->extreal)x))`, `(\x. real (indicator_fn {x':real| 0<= x' /\ x' <=t} x))`, ` (\ (y,z). indicator_fn {(y',z') |
          0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x } (y:real,z))` ] pos_fn_integral_fun_mul1)
           THEN RW_TAC std_ss[measure_space_distr_pair, sigma_finite_measure_lborel, measure_space_lborel]
	  THEN KNOW_TAC(``!(x':real#real).
              x' IN
              m_space
                (distr p (pair_measure lborel lborel)
                   (\x. (real (Y x),real (Z x))))``)
               THEN1(RW_TAC std_ss[lborel,m_space_def, distr, pair_measure, measure_of, measurable_sets_def]  
                    THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
		    THEN ONCE_ASM_REWRITE_TAC[]
		    THEN RW_TAC std_ss[IN_UNIV])
	  THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(``!(x':real#real).
              0 <=
              (\ (y:real,z:real).
                   indicator_fn
                     {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                     (y,z)) x' ``)
                THEN1(RW_TAC std_ss[]
		      THEN `x':real#real = (FST x', SND x')` by RW_TAC std_ss[PAIR]
                      THEN ONCE_ASM_REWRITE_TAC[]
		      THEN `(\(y:real,z:real).
        indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
          (y,z)) (FST x',SND x') = 
        indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (FST x',SND x')` by RW_TAC std_ss[PAIR, LAMBDA_PROD]
	 	      THEN ONCE_ASM_REWRITE_TAC[]
		      THEN RW_TAC std_ss[indicator_fn_pos_le])
	    THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[GSYM normal_real_indicator_fn]
	    THEN RW_TAC std_ss[CPDFT_A_Ayz_indicator_of_indicator]
	    THEN FULL_SIMP_TAC std_ss[PAIR, LAMBDA_PROD, ETA_THM]
	    THEN FULL_SIMP_TAC std_ss[real_indicator_fn_pos_le])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[mul_comm,mul_assoc]);





val CPDFT_Ayz_measurable_set = store_thm(
  "CPDFT_Ayz_measurable_set",
  `` !x. {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} IN
         measurable_sets (pair_measure lborel lborel)``,

RW_TAC std_ss[]
THEN KNOW_TAC(``  {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} = 
				 
                  BIGUNION {{y'|  0<= y' /\  real q < y' /\ y' < x}  CROSS
  	                    {z'|  0 <= z' /\  z' < real q } | q IN Q_set} ``)

     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EQ_TAC
                THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]	    
           	       THEN KNOW_TAC (`` 0 <= SND (x':real#real) /\ 0 <= FST x' /\ SND x'  < FST x'   /\ FST x' < x ``)
   		            THEN1(FULL_SIMP_TAC std_ss[ LAMBDA_PAIR])
           	       THEN DISCH_TAC
           	       THEN KNOW_TAC (``Normal (SND (x':real#real)) < Normal (FST x')``)
	                    THEN1(METIS_TAC [extreal_lt_eq])
                       THEN DISCH_TAC
	               THEN KNOW_TAC(``?q. q IN Q_set/\ Normal (SND x') < q /\  q < Normal (FST x')``)
	                    THEN1(METIS_TAC [Q_DENSE_IN_R])
                       THEN RW_TAC std_ss[]
	               THEN KNOW_TAC(``FST (x':real#real) = real (Normal(FST x'))``)
                            THEN1(RW_TAC std_ss [GSYM real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``SND (x':real#real) = real(Normal (SND x'))``)
	                    THEN1(METIS_TAC[real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``?r. q = Normal r``) 
                            THEN1(METIS_TAC[Q_not_infty])
  	               THEN RW_TAC std_ss[]
		       THEN Q.EXISTS_TAC (`{y| 0 <= FST y /\ real (Normal r) < FST y /\ FST y < x /\
                0 <= SND y /\ SND y < real (Normal r)}`)
		       THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                       THEN Q.EXISTS_TAC(`Normal r`)     
                       THEN SRW_TAC[] [GSPECIFICATION, EXTENSION]
                       THEN EQ_TAC
                       THEN RW_TAC std_ss[])
	               THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                       THEN RW_TAC std_ss [LAMBDA_PAIR]
                       THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR] )
THEN DISCH_TAC	   
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!q. q IN Q_set ==>{y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q} IN measurable_sets (pair_measure lborel lborel) ``)
     THEN1 (RW_TAC std_ss [pair_measure, measure_of, lborel,m_space_def, measurable_sets_def ]
           THEN1 (KNOW_TAC(``{y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
   {z' | 0 <= z' /\ z' < real q} IN {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)

                  THEN1 (KNOW_TAC(`` {y' | 0 <= y' /\ real q < y' /\ y' < x} IN subsets borel``) 
                         THEN1(KNOW_TAC(`` {y' | 0 <= y' /\ real q < y' /\ y' < x} =
		                        {y' | 0 <= y'} INTER
					{y' | real q < y'} INTER
				       {y' | y' < x } ``) 
                               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			             THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
			 THEN DISCH_TAC
			 THEN ONCE_ASM_REWRITE_TAC[]
                         THEN `{y' | 0 <= y'} IN subsets borel /\
				       {y' | real q < y' } IN subsets borel /\
				       {y' | y' < x} IN subsets borel`
			    by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT] 	        
                        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
	   THEN DISCH_TAC
           THEN KNOW_TAC (`` {z' | 0 <= z' /\ z' < real q} IN subsets borel ``)
               THEN1(`{z' | 0 <= z' /\ z' < real q}  =
	                  {z' | 0<= z'} INTER
	                  {z' |  z' < real q} ` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CONJ_COMM]
	                THEN `{z' |  0<= z' } IN subsets borel /\
		              {z' |  z' < real q} IN subsets borel`   by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
		        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
          THEN DISCH_TAC
          THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	  THEN Q.EXISTS_TAC (`({y' | 0 <= y' /\ real q < y' /\ y' < x},
        {z' | 0 <= z' /\ z' < real q})   `)
	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
		 
         THEN DISCH_TAC
         THEN METIS_TAC [sigma_sets_basic])
         THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
     THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
     THEN KNOW_TAC(`` {a CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET POW univ(:real#real)``)
          THEN1( RW_TAC std_ss [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
     THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN KNOW_TAC(``BIGUNION
    {{y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q} | q IN Q_set} =
     BIGUNION
  (IMAGE (\q. {y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q} )
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN MP_TAC(Q.ISPECL[`pair_measure lborel lborel ` ,` (\q.
             {y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q})` ] MEASURE_SPACE_BIGUNION_Q)
THEN RW_TAC std_ss[]);







val CPDFT_Ayz_measurable_set = store_thm(
  "CPDFT_Ayz_measurable_set",
  `` !x. {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} IN
         measurable_sets (pair_measure lborel lborel)``,

RW_TAC std_ss[]
THEN KNOW_TAC(``  {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} = 
				 
                  BIGUNION {{y'|  0<= y' /\  real q < y' /\ y' < x}  CROSS
  	                    {z'|  0 <= z' /\  z' < real q } | q IN Q_set} ``)

     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EQ_TAC
                THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]	    
           	       THEN KNOW_TAC (`` 0 <= SND (x':real#real) /\ 0 <= FST x' /\ SND x'  < FST x'   /\ FST x' < x ``)
   		            THEN1(FULL_SIMP_TAC std_ss[ LAMBDA_PAIR])
           	       THEN DISCH_TAC
           	       THEN KNOW_TAC (``Normal (SND (x':real#real)) < Normal (FST x')``)
	                    THEN1(METIS_TAC [extreal_lt_eq])
                       THEN DISCH_TAC
	               THEN KNOW_TAC(``?q. q IN Q_set/\ Normal (SND x') < q /\  q < Normal (FST x')``)
	                    THEN1(METIS_TAC [Q_DENSE_IN_R])
                       THEN RW_TAC std_ss[]
	               THEN KNOW_TAC(``FST (x':real#real) = real (Normal(FST x'))``)
                            THEN1(RW_TAC std_ss [GSYM real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``SND (x':real#real) = real(Normal (SND x'))``)
	                    THEN1(METIS_TAC[real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``?r. q = Normal r``) 
                            THEN1(METIS_TAC[Q_not_infty])
  	               THEN RW_TAC std_ss[]
		       THEN Q.EXISTS_TAC (`{y| 0 <= FST y /\ real (Normal r) < FST y /\ FST y < x /\
                0 <= SND y /\ SND y < real (Normal r)}`)
		       THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                       THEN Q.EXISTS_TAC(`Normal r`)     
                       THEN SRW_TAC[] [GSPECIFICATION, EXTENSION]
                       THEN EQ_TAC
                       THEN RW_TAC std_ss[])
	               THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                       THEN RW_TAC std_ss [LAMBDA_PAIR]
                       THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR] )
THEN DISCH_TAC	   
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!q. q IN Q_set ==>{y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q} IN measurable_sets (pair_measure lborel lborel) ``)
     THEN1 (RW_TAC std_ss [pair_measure, measure_of, lborel,m_space_def, measurable_sets_def ]
           THEN1 (KNOW_TAC(``{y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
   {z' | 0 <= z' /\ z' < real q} IN {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)

                  THEN1 (KNOW_TAC(`` {y' | 0 <= y' /\ real q < y' /\ y' < x} IN subsets borel``) 
                         THEN1(KNOW_TAC(`` {y' | 0 <= y' /\ real q < y' /\ y' < x} =
		                        {y' | 0 <= y'} INTER
					{y' | real q < y'} INTER
				       {y' | y' < x } ``) 
                               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			             THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
			 THEN DISCH_TAC
			 THEN ONCE_ASM_REWRITE_TAC[]
                         THEN `{y' | 0 <= y'} IN subsets borel /\
				       {y' | real q < y' } IN subsets borel /\
				       {y' | y' < x} IN subsets borel`
			    by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT] 	        
                        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
	   THEN DISCH_TAC
           THEN KNOW_TAC (`` {z' | 0 <= z' /\ z' < real q} IN subsets borel ``)
               THEN1(`{z' | 0 <= z' /\ z' < real q}  =
	                  {z' | 0<= z'} INTER
	                  {z' |  z' < real q} ` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CONJ_COMM]
	                THEN `{z' |  0<= z' } IN subsets borel /\
		              {z' |  z' < real q} IN subsets borel`   by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
		        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
          THEN DISCH_TAC
          THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	  THEN Q.EXISTS_TAC (`({y' | 0 <= y' /\ real q < y' /\ y' < x},
        {z' | 0 <= z' /\ z' < real q})   `)
	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
		 
         THEN DISCH_TAC
         THEN METIS_TAC [sigma_sets_basic])
         THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
     THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
     THEN KNOW_TAC(`` {a CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET POW univ(:real#real)``)
          THEN1( RW_TAC std_ss [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
     THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN KNOW_TAC(``BIGUNION
    {{y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q} | q IN Q_set} =
     BIGUNION
  (IMAGE (\q. {y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q} )
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN MP_TAC(Q.ISPECL[`pair_measure lborel lborel ` ,` (\q.
             {y' | 0 <= y' /\ real q < y' /\ y' < x} CROSS
     {z' | 0 <= z' /\ z' < real q})` ] MEASURE_SPACE_BIGUNION_Q)
THEN RW_TAC std_ss[]);

val indicator_of_indicator_Ayz_Ay_Az = store_thm(
  "indicator_of_indicator_Ayz_Ay_Az",
  `` !y z x:real. indicator_fn
                {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)=
           indicator_fn {y'| 0 <= y' /\ y' <x} y *
	   indicator_fn{z'| 0<= z' /\ z' < y} z``,

RW_TAC std_ss[indicator_fn_def]

  THEN1(METIS_TAC [mul_lone])

THEN1(`(y:real,z) IN
      {(y',z') |
       0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} <=>
      (y,z) IN
      {(y',z') |
       (\y'' z''.
           0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
       
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`(y:real,z) IN
      {(y',z') |
       0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} <=>
      (y,z) IN
      {(y',z') |
       (\y'' z''.
           0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
       
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`(y:real,z) IN
      {(y',z') |
       0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} <=>
      (y,z) IN
      {(y',z') |
       (\y'' z''.
           0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
       
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(y:real,z) IN
      {(y',z') |
       0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} <=>
      (y,z) IN
      {(y',z') |
       (\y'' z''.
           0 <= z'' /\ 0 <= y'' /\ z'' < y'' /\ y'' < x) y' z'}` by RW_TAC std_ss[]
       
      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN METIS_TAC [mul_rzero, mul_comm]);


val prob_event_le_gt0 = store_thm("prob_event_le_gt0",
  ``!X p t:real M.
          random_variable X p (m_space M,measurable_sets M) /\
          measure_space M /\ {y | 0<=y /\ y <= t} IN measurable_sets M ==>
          (prob p (PREIMAGE X {y | 0<= y /\ y <= t} INTER p_space p) =
           integral (distr p M X) (indicator_fn {y | 0<= y /\ y <= t}))``,
RW_TAC std_ss[prob_event_gen]);


val CDF_pos_fn_integral_indicator_gt0 = store_thm("CDF_pos_fn_integral_indicator_gt0",
  `` !X p t M.
     random_variable X p (m_space M,measurable_sets M) /\ (!s. 0<= X s) /\
     measure_space M /\ {y | 0<=y /\ y <= t} IN measurable_sets M ==>
     (CDF p X t = pos_fn_integral (distr p M X) (\x. indicator_fn {y |0<=y /\ y <= t} x))``,
METIS_TAC [CDF_GT0, prob_event_le_gt0, distribution_def, integral_pos_fn_distr]);


val CDF_pos_fn_integral_indicator2 = store_thm("CDF_pos_fn_integral_indicator2",
  `` !X p t M.
     random_variable X p (m_space M,measurable_sets M) /\ (!s. 0<= X s) /\
     measure_space M /\ (!t. {y | 0 <=y /\ y <= t} IN measurable_sets M) ==>
    ( CDF p X t = pos_fn_integral (distr p M X) (\x. indicator_fn {y |0<=y /\  y <= t} x))``,
RW_TAC std_ss[]
THEN KNOW_TAC(``!t. CDF p X t = pos_fn_integral (distr p M X) (\x. indicator_fn {y | 0<=y/\ y <= t} x)``)
     THEN1(METIS_TAC [CDF_GT0, prob_event_le_gt0, distribution_def, integral_pos_fn_distr, CDF_pos_fn_integral_indicator_gt0])
THEN RW_TAC std_ss[]);


val CDF_def3 = store_thm("CDF_def3",
  ``  !p X x.
     prob_space p /\ (!s. 0<= X s) /\
     (!n.
        PREIMAGE X {y | x - 1 / &SUC n < y /\ y <= x} INTER p_space p IN
        events p /\
        PREIMAGE X {y | y <= x} INTER p_space p IN events p /\
        PREIMAGE X {y | y < x} INTER p_space p IN events p /\
        PREIMAGE X {y | y <= x - 1 / &SUC n} INTER p_space p IN
        events p) /\
     PREIMAGE X {y | y = x} INTER p_space p IN events p /\
     (!z.  (\x. real (CDF p X x)) contl z) ==>
     (CDF p X x =
      prob p (PREIMAGE X {y |0<=y /\  y < x} INTER p_space p))``,
RW_TAC std_ss[CDF_def2]
THEN KNOW_TAC(``PREIMAGE (X:'a->real) {y | y < x}=PREIMAGE X {y | 0<= y /\y < x}  ``)
     THEN1(SRW_TAC [][GSPECIFICATION, EXTENSION, PREIMAGE_def])
THEN RW_TAC std_ss[]);
           

 val prob_CPDFT_initial4 = store_thm("prob_CPDFT_initial4",
  `` !p Y Z  x f_y.  indep_var p lborel (\x. real (Y x)) lborel (\x. real (Z x)) /\
     	      	  distributed p lborel (\x. real (Y x)) f_y /\
		  rv_gt0_ninfinity[Z] /\
		  (!x.  0<= f_y x) /\
		  (!z. (\x. real (CDF p (\x. real (Z x)) x)) contl z)==> 
     	      	  ( pos_fn_integral (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\ (y,z).
                         indicator_fn
                           {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                           (y,z))=
			   pos_fn_integral lborel
			        (\y.
				   f_y y * indicator_fn {y' | 0 <= y' /\ y' < x} y *
				     CDF p (\x. real (Z x)) y))  ``,

RW_TAC std_ss[]
THEN `random_variable (\x. real ((Y:'a->extreal) x )) p (m_space lborel, measurable_sets lborel)` by METIS_TAC[indep_var_rv]
THEN `random_variable (\x. real ((Z:'a->extreal) x )) p (m_space lborel, measurable_sets lborel)` by METIS_TAC[indep_var_rv]

THEN KNOW_TAC(`` pos_fn_integral
     (distr p (pair_measure lborel lborel) (\x. (real (Y x),real (Z x))))
     (\(y,z).
          indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
            (y,z)) = pos_fn_integral
     ( pair_measure (distr p lborel (\x. real (Y x))) (distr p lborel (\x. real (Z x)))) 
     (\(y,z).
          indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
            (y,z))``)
     THEN1(MP_TAC (Q.ISPECL [`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `lborel` , `(\x. real ((Y:'a->extreal)x))`, `lborel`, `(\x. real ((Z:'a->extreal)x))`, `{(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x:real}`]  indep_var_pair_distr)
            THEN `prob_space p` by FULL_SIMP_TAC std_ss[indep_varp_def, random_variable_def]
	    THEN RW_TAC std_ss[ measure_space_lborel, sigma_finite_measure_lborel, CPDFT_Ayz_measurable_set]
	    THEN `(\ (y:real,z).
          indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
            (y,z)) = indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}` by METIS_TAC[ETA_THM, LAMBDA_PROD, PAIR]
            THEN RW_TAC std_ss[])
THEN DISCH_TAC	    
THEN ONCE_ASM_REWRITE_TAC[]
THEN MP_TAC (Q.ISPECL [`(\ (y,z).
          indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z:real))`,`distr p lborel (\x. real ((Y:'a->extreal)x))`,`distr p lborel (\x. real ((Z:'a->extreal)x))` ] pos_fn_integral_fst)
THEN RW_TAC std_ss[]	  
THEN `measure_space (distr p lborel (\x. real (Y x)))` by METIS_TAC [indep_varp_def, measure_space_distr, p_space_def, events_def, measure_space_lborel, sigma_finite_measure_lborel, random_variable_def, prob_space_def]
THEN `measure_space (distr p lborel (\x. real (Z x)))` by METIS_TAC [indep_varp_def, measure_space_distr, p_space_def, events_def, measure_space_lborel, sigma_finite_measure_lborel, random_variable_def, prob_space_def]
THEN ` sigma_finite_measure (distr p lborel (\x. real (Y x))) /\
         sigma_finite_measure (distr p lborel (\x. real (Z x)))` by METIS_TAC [sigma_finite_measure_distr, indep_varp_def, random_variable_def]
THEN FULL_SIMP_TAC std_ss[]
THEN  KNOW_TAC(``!x':real#real.
              0 <=
              (\(y,z).
                   indicator_fn
                     {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                     (y,z)) x'``)
      THEN1(RW_TAC std_ss[]
            THEN `x':real#real = (FST x', SND x')` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	    THEN ONCE_ASM_REWRITE_TAC[]
	    THEN RW_TAC std_ss[PAIR, LAMBDA_PROD, PAIR, indicator_fn_pos_le])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]	    
THEN KNOW_TAC(``(m_space
              (pair_measure (distr p lborel (\x. real (Y x)))
                 (distr p lborel (\x. real ((Z:'a->extreal) x)))),
            measurable_sets
              (pair_measure (distr p lborel (\x. real (Y x)))
                 (distr p lborel (\x. real (Z x))))) = (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel)) ``)
    THEN1(RW_TAC std_ss[]
          THEN RW_TAC std_ss[distr, m_space_def, measurable_sets_def, dist, pair_measure, measure_of, lborel])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (\(y,z).
              indicator_fn
                {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)) IN
         measurable
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel ``)
    THEN1( MP_TAC(Q.ISPECL[`(m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel))`, `{(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x:real}`,`(\ (y,z).
              indicator_fn
                {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z:real))` ] IN_MEASURABLE_BOREL_INDICATOR)
             THEN RW_TAC std_ss []
    	      THEN `{(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} IN subsets (m_space (pair_measure lborel lborel) ,measurable_sets (pair_measure lborel lborel)) ` by RW_TAC std_ss [ CPDFT_Ayz_measurable_set,subsets_def ]
	              THEN FULL_SIMP_TAC std_ss[]
		      THEN `measure_space(pair_measure lborel lborel)` by RW_TAC std_ss[measure_space_pair_measure', sigma_finite_measure_lborel, measure_space_lborel]
		      THEN `sigma_algebra
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel))` by FULL_SIMP_TAC std_ss[measure_space_def]
	     		    
                      THEN FULL_SIMP_TAC std_ss[]
		      THEN KNOW_TAC(``!x':real#real.
              x' IN
              space
                (m_space (pair_measure lborel lborel),
                 measurable_sets (pair_measure lborel lborel)) ``)
          	      THEN1(RW_TAC std_ss[]
		            THEN RW_TAC std_ss[space_def, pair_measure, lborel, m_space_def, measurable_sets_def, measure_of]
			    THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
			    THEN ONCE_ASM_REWRITE_TAC[]
			    THEN RW_TAC std_ss[IN_UNIV])
                    THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
		    THEN KNOW_TAC(`` !x':real#real.
              (\ (y,z).
                   indicator_fn
                     {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                     (y,z)) x' =
              indicator_fn
                {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} x'``)
		         THEN1(RW_TAC std_ss[]
			       THEN `x':real#real = (FST x', SND x')` by RW_TAC std_ss[PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
			       THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
                   THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]		   
THEN `pos_fn_integral
           (pair_measure (distr p lborel (\x. real (Y x)))
              (distr p lborel (\x. real (Z x))))
           (\(y,z).
                indicator_fn
                  {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)) =
      pos_fn_integral (distr p lborel (\x. real (Y x)))
           (\y.
                pos_fn_integral (distr p lborel (\x. real (Z x)))
                  (\z.
                       indicator_fn
                         {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                         (y,z)))` by RW_TAC std_ss[]		  
THEN ONCE_ASM_REWRITE_TAC[]
THEN WEAKEN_TAC (equal(Term`pos_fn_integral (distr p lborel (\x. real (Y x)))
           (\x'.
                pos_fn_integral (distr p lborel (\x. real (Z x)))
                  (\y.
                       indicator_fn
                         {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                         (x',y))) =
         pos_fn_integral
           (pair_measure (distr p lborel (\x. real (Y x)))
              (distr p lborel (\x. real (Z x))))
           (\(y,z).
                indicator_fn
                  {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)) `))

THEN MP_TAC (Q.ISPECL [`f_y:real->extreal`, `(\y:real.
          pos_fn_integral (distr p lborel (\x. real ((Z:'a->extreal) x)))
            (\z.
                 indicator_fn
                   {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x:real} (y,z)))`, `lborel` ] pos_fn_integral_density_1)
THEN ` f_y IN measurable (m_space lborel,measurable_sets lborel) Borel /\
    (!x. 0 <= f_y x) /\ AE lborel {x | 0 <= f_y x} ` by FULL_SIMP_TAC std_ss[distributed_def]

THEN WEAKEN_TAC (equal(Term`pos_fn_integral
           (pair_measure (distr p lborel (\x. real (Y x)))
              (distr p lborel (\x. real (Z x))))
           (\(y,z).
                indicator_fn
                  {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)) =
         pos_fn_integral (distr p lborel (\x. real (Y x)))
           (\y.
                pos_fn_integral (distr p lborel (\x. real (Z x)))
                  (\z.
                       indicator_fn
                         {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                         (y,z)))
   `))

THEN RW_TAC std_ss[measure_space_lborel]
THEN KNOW_TAC(`` (\y.
              pos_fn_integral (distr p lborel (\x. real (Z x)))
                (\z.
                     indicator_fn
                       {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                       (y,z))) IN
         measurable (m_space lborel,measurable_sets lborel) Borel``)

     THEN1( MP_TAC(Q.ISPECL[`(\ (y,z:real).
               indicator_fn
                 {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z))`, `lborel`, `distr p lborel (\x. real ((Z:'a->extreal)x))`] borel_measurable_pos_integral_fst)
	   THEN RW_TAC std_ss [measure_space_lborel, measure_space_distr, sigma_finite_measure_distr]
           THEN KNOW_TAC(`` (\ (y,z).
	   
              indicator_fn
                {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)) IN
         measurable
           (m_space (pair_measure lborel (distr p lborel (\x. real (Z x)))),
            measurable_sets
              (pair_measure lborel (distr p lborel (\x. real (Z x))))) Borel ``)
	   THEN1(`(m_space (pair_measure lborel (distr p lborel (\x. real (Z x)))),
      measurable_sets (pair_measure lborel (distr p lborel (\x. real (Z x))))) =
      (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel))` by RW_TAC std_ss[pair_measure, measure_of, lborel, m_space_def, measurable_sets_def, distr]
                  THEN ONCE_ASM_REWRITE_TAC[]
		  THEN MP_TAC (Q.ISPECL [`(m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel))`, `{(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x:real}`, `( \ (y,z).
        indicator_fn {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
          (y,z:real))`  ]IN_MEASURABLE_BOREL_INDICATOR)
	  	  THEN RW_TAC std_ss[sigma_algebra_pair_lborel])
	 THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	 THEN `!x. x IN m_space lborel` by RW_TAC std_ss[m_space_def, lborel, IN_UNIV]
	 THEN FULL_SIMP_TAC std_ss[extreal_max_def, indicator_fn_pos_le])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN `!x'.
              0 <=
              pos_fn_integral (distr p lborel (\x. real (Z x)))
                (\z.
                     indicator_fn
                       {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x}
                       (x',z))` by FULL_SIMP_TAC std_ss[pos_fn_integral_pos, indicator_fn_pos_le]
THEN FULL_SIMP_TAC std_ss[]
THEN `pos_fn_integral (distr p lborel (\x. real (Y x)))
     (\y.
          pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z.
                 indicator_fn
                   {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z))) =
		   pos_fn_integral (density lborel f_y)
     (\y.
          pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z.
                 indicator_fn
                   {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)))` by FULL_SIMP_TAC std_ss[distributed_def]
THEN ONCE_ASM_REWRITE_TAC[]		   
THEN ONCE_ASM_REWRITE_TAC[]		   
THEN ONCE_REWRITE_TAC[indicator_of_indicator_Ayz_Ay_Az]
THEN KNOW_TAC(``!y:real. pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z.
                 indicator_fn {y' | 0 <= y' /\ y' < x} y *
                 indicator_fn {z' | 0 <= z' /\ z' < y} z) =
           indicator_fn {y' | 0 <= y' /\ y' < x} y*
             pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z.
                 indicator_fn {z' | 0 <= z' /\ z' < y} z) ``)
      THEN1(RW_TAC std_ss[]
            THEN MP_TAC (Q.ISPECL [`distr p lborel (\x. real ((Z:'a->extreal) x))`, `(\y. real (indicator_fn {y' | 0 <= y' /\ y' < x:real} y))`, `(\z. indicator_fn {z' | 0 <= z' /\ z' < y:real} z)` ]pos_fn_integral_fun_mul1)
            THEN RW_TAC std_ss[]
 	    THEN KNOW_TAC(``!x':real.
              x' IN m_space (distr p lborel (\x. real (Z x))) ``)
	         THEN1(RW_TAC std_ss[distr, m_space_def, lborel]
		       THEN RW_TAC std_ss[IN_UNIV])
	    THEN DISCH_TAC	       
	    THEN FULL_SIMP_TAC std_ss[indicator_fn_pos_le, real_indicator_fn_pos_le]
	    THEN FULL_SIMP_TAC std_ss[GSYM normal_real_indicator_fn])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``pos_fn_integral (distr p lborel (\x. real (Y x)))
     (\y.
          indicator_fn {y' | 0 <= y' /\ y' < x} y *
          pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z)) =
	    pos_fn_integral (density lborel f_y)
     (\y.
          indicator_fn {y' | 0 <= y' /\ y' < x} y *
          pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z))``)

     THEN1(FULL_SIMP_TAC std_ss[distributed_def])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]     
THEN `random_variable (\x. real (Z x)) p (m_space lborel , measurable_sets lborel)` by FULL_SIMP_TAC std_ss[indep_varp_def]

THEN KNOW_TAC(``!s. 0 <= real (Z s)``)
     THEN1(RW_TAC std_ss[]
          THEN ` 0 <= Z s` by FULL_SIMP_TAC std_ss[rv_gt0_ninfinity_def]

          THEN  ` 0 <> PosInf /\ 0 <> NegInf` by METIS_TAC [lt_infty, le_infty, extreal_of_num_def]
	 THEN `!s.  Z s <> PosInf /\Z s <> NegInf` by   METIS_TAC [lt_infty, le_infty, extreal_of_num_def, rv_gt0_ninfinity_def]
	 THEN RW_TAC std_ss[GSYM real_of_0]
         THEN METIS_TAC [extreal_real_le])
THEN DISCH_TAC  	 
THEN `measure_space lborel` by RW_TAC std_ss[measure_space_lborel]
THEN KNOW_TAC(``!y. {z' | 0 <= z' /\ z' <= y} IN
        	      measurable_sets lborel ``)
	 	 THEN1(`measurable_sets lborel = subsets borel` by RW_TAC std_ss[subsets_def, m_space_def, measurable_sets_def, lborel, borel]
                       THEN ONCE_ASM_REWRITE_TAC[]
		       THEN RW_TAC std_ss[]
		       THEN `{z' | 0 <= z' /\ z' <= y} = {z'|0<=z'} INTER {z'|z'<=y}` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
		       THEN ONCE_ASM_REWRITE_TAC[]
		       THEN `{z'| 0 <= z'} IN subsets borel /\
		             {z' | z' <= y} IN subsets borel` by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT] 	        
                        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC
THEN KNOW_TAC(``!y. pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z)  = CDF p (\x. real (Z x)) y``)
     THEN1(RW_TAC std_ss[]
          THEN MP_TAC (Q.ISPECL [`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `(\x. real ((Z:'a->extreal) x))`, `y:real `] CDF_def3)
	  THEN RW_TAC std_ss[]
          THEN KNOW_TAC(`` (!n.
              PREIMAGE (\x. real (Z x)) {y' | y - 1 / &SUC n < y' /\ y' <= y} INTER
              p_space p IN events p /\
              PREIMAGE (\x. real (Z x)) {y' | y' <= y} INTER p_space p IN
              events p /\
              PREIMAGE (\x. real (Z x)) {y' | y' < y} INTER p_space p IN
              events p /\
              PREIMAGE (\x. real (Z x)) {y' | y' <= y - 1 / &SUC n} INTER
              p_space p IN events p)``)
	     THEN1(STRIP_TAC
	           THEN KNOW_TAC(``{y' | y - 1 / &SUC n < y' /\ y' <= y} IN subsets borel /\
		         {y' | y' <= y} IN subsets borel /\
			 {y' | y' < y} IN subsets borel /\
			 {y' | y' <= y - 1 / &SUC n} IN subsets borel``)
			 THEN1( `{y' | y - 1 / &SUC n < y' /\ y' <= y} =
			             {y' | y - 1 / &SUC n < y'} INTER {y'| y' <= y}` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
			       THEN ONCE_ASM_REWRITE_TAC[]
 			       THEN METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT, sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
		  THEN DISCH_TAC
		  THEN METIS_TAC [IN_MEASURABLE, subsets_def, measurable_sets_def, events_def, space_def, m_space_def, p_space_def, prob_space_def, lborel, borel, random_variable_def])
        THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	THEN `prob_space p` by FULL_SIMP_TAC std_ss[random_variable_def]
	THEN KNOW_TAC (``PREIMAGE (\x. real (Z x)) {y' | y' = y} INTER p_space p IN events p  ``)
	     THEN1(`{y' | y' = y} IN subsets borel` by RW_TAC std_ss[CLOSED_STANDARD_HYPERPLANE, borel_closed]
	           THEN METIS_TAC [IN_MEASURABLE, subsets_def, measurable_sets_def, events_def, space_def, m_space_def, p_space_def, prob_space_def, lborel, borel, random_variable_def])
		   
	THEN DISCH_TAC
	THEN KNOW_TAC(`` pos_fn_integral (distr p lborel (\x. real (Z x)))
     (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z) = distribution p (\x. real ((Z:'a->extreal) x)) {z' | 0<= z' /\ z' <y}  ``)
            THEN1(`measure_space (distr p lborel (\x. real (Z x)))` by RW_TAC std_ss[measure_space_distr, measure_space_lborel, prob_space_def, random_variable_def]
                  THEN KNOW_TAC(``{z' | 0<= z' /\ z' < y} IN measurable_sets lborel``)
		      THEN1( RW_TAC std_ss[measurable_sets_def,  lborel]
		             THEN `{z' | 0<= z' /\ z' <y} = {z'|0<= z'} INTER {z'| z'<y}` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
			     THEN ONCE_ASM_REWRITE_TAC[]
			     THEN METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT, sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
		  THEN DISCH_TAC
		  THEN METIS_TAC[GSYM distribution_lebesgue_thm2_pos_fn])
       THEN DISCH_TAC
       THEN ONCE_ASM_REWRITE_TAC[]
       THEN METIS_TAC [CDF_def3, distribution_def])
THEN DISCH_TAC       
THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[mul_comm, mul_assoc]);



val indep_CPAND_def = Define `indep_CPAND X Y Z p=
       indep_varp p lborel (λx. real (X x)) (pair_measure lborel lborel)
           (λx. (real (Y x),real (Z x))) ∧
         indep_var p lborel (λx. real (Y x)) lborel (λx. real (Z x))`;


val prob_CPDFT = store_thm("prob_CPDFT",
  ``!p X Y Z t f_y f_x.  prob_space p /\ 0 <= t /\
         indep_CPAND X Y Z p /\ rv_gt0_ninfinity [X; Y; Z] /\
          (!s. ALL_DISTINCT [X s; Y s; Z s])/\
	  distributed p lborel (\x. real (X x)) f_x /\ (!x. 0 <= f_x x)/\
          distributed p lborel (\x. real (Y x)) f_y /\
          (!y. 0 <= f_y y) /\ cont_CDF p (\x. real (Z x)) ==>
          (prob p
             (DFT_event p (P_AND (P_AND Z Y) X) t) =
       pos_fn_integral lborel
             (\x.
                  f_x x *  indicator_fn {x' | 0 <= x' /\ x' <= t} x*
                  pos_fn_integral lborel
                   (\y.
                      f_y y * indicator_fn {y' | 0 <= y' /\ y' < x} y *
                        CDF p (\x. real (Z x)) y)))``,
`!t.
          {(x',y',z') |
          0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\ x' <= t} IN
          measurable_sets (pair_measure lborel (pair_measure lborel lborel))` by RW_TAC std_ss[CPDFT_event_measurable_set]
THEN RW_TAC std_ss[indep_CPAND_def,cont_CDF_def] 	  
THEN RW_TAC std_ss[CPDFT_reduce]	  
THEN RW_TAC std_ss[prob_CPDFT_initial, CPDFT_event_measurable_set]
THEN `random_variable (\x. real (X x)) p (m_space lborel,measurable_sets lborel) /\
      random_variable (\x. real (Y x)) p (m_space lborel,measurable_sets lborel) /\
      random_variable (\x. real (Z x)) p (m_space lborel,measurable_sets lborel) ` by METIS_TAC[indep_varp_def, indep_var_rv]
THEN `rv_gt0_ninfinity[Z]` by FULL_SIMP_TAC std_ss[rv_gt0_ninfinity_def]

THEN KNOW_TAC(``(pos_fn_integral (distr p lborel (\x. real (X x)))
             (\x.
                  pos_fn_integral
                    (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\(y,z). indicator_fn {(x',y',z') |
                   0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\
                   x' <= t} (x,y,z))) =
           pos_fn_integral lborel
             (\x.
                  f_x x *
                  pos_fn_integral
                    (distr p (pair_measure lborel lborel)
                       (\x. (real (Y x),real (Z x))))
                    (\(y,z). indicator_fn {(x',y',z') |
                   0 <= z' /\ 0 <= y' /\ 0 <= x' /\ z' < y' /\ y' < x' /\
                   x' <= t} (x,y,z))))``)
     THEN1(DEP_REWRITE_TAC[prob_CPDFT_initial2]
           THEN RW_TAC std_ss[])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[prob_CPDFT_initial3]
THEN KNOW_TAC(``!x. (pos_fn_integral
             (distr p (pair_measure lborel lborel)
                (\x. (real (Y x),real (Z x))))
             (\(y,z).
                  indicator_fn
                    {(y',z') | 0 <= z' /\ 0 <= y' /\ z' < y' /\ y' < x} (y,z)) =
           pos_fn_integral lborel
             (\y.
                  f_y y * indicator_fn {y' | 0 <= y' /\ y' < x} y *
                  CDF p (\x. real (Z x)) y)) ``)
     THEN1( RW_TAC std_ss[prob_CPDFT_initial4])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]     
THEN RW_TAC std_ss[]);


val _ = export_theory();
