

(******************************************************************************)
(*                      Probabilistic Analysis of                             *)
(*                            WSP_OR                                          *)
(*                                  by                                        *)
(*                          Yassmeen Elderhalli                               *)
(*                      y_elderh@encs.concordia.ca                            *)
(******************************************************************************)

(*loadPath := "/home/yassmeen/Downloads/ProbTheories_HOL-k12" :: !loadPath;
(*set_trace "Unicode" 0;*)

app load ["HolKernel", "Parse", "boolLib", "bossLib", "numLib", "unwindLib", 
"tautLib", "Arith", "prim_recTheory", "combinTheory", "quotientTheory", 
"arithmeticTheory", "realTheory", "hrealTheory", "realaxTheory", "realLib",  
"jrhUtils", "pairTheory", "seqTheory", "limTheory", "transcTheory", "listTheory", 
"mesonLib", "boolTheory", "topologyTheory", "pred_setTheory", "util_probTheory", 
"optionTheory", "numTheory", "sumTheory", "InductiveDefinition", "ind_typeTheory",
"pred_setLib", "iterate_hvgTheory", "card_hvgTheory", "product_hvgTheory", 
"topology_hvgTheory", "derivative_hvgTheory", "integration_hvgTheory",
"real_sigmaTheory", "extreal_hvgTheory", "measure_hvgTheory", "lebesgue_hvgTheory",
"probability_hvgTheory", "lebesgue_measure_hvgTheory", "normal_rv_hvgTheory", "netsTheory", "PIE_EXTREALTheory","dep_rewrite", "integration_before_afterTheory", "DFT_Gates_def_PROBTheory", "CASTheory","WCSPTheory", "WCSP_sharedTheory" , "AND_FDEPTheory"];
*)
open HolKernel Parse boolLib bossLib numLib unwindLib tautLib Arith prim_recTheory 
combinTheory quotientTheory arithmeticTheory hrealTheory realaxTheory realTheory 
realLib jrhUtils pairTheory seqTheory limTheory transcTheory listTheory mesonLib 
boolTheory topologyTheory pred_setTheory util_probTheory optionTheory numTheory 
sumTheory InductiveDefinition ind_typeTheory pred_setLib iterate_hvgTheory 
card_hvgTheory product_hvgTheory topology_hvgTheory derivative_hvgTheory
integration_hvgTheory real_sigmaTheory extreal_hvgTheory measure_hvgTheory
lebesgue_hvgTheory probability_hvgTheory lebesgue_measure_hvgTheory normal_rv_hvgTheory netsTheory PIE_EXTREALTheory dep_rewrite integration_before_afterTheory DFT_Gates_def_PROBTheory CASTheory WCSPTheory WCSP_sharedTheory AND_FDEPTheory;
val _ = new_theory "WSP_OR";


infixr 0 ++ << || ORELSEC ## --> THENC;
infix 1 >> |->;
fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
 
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Suff = PARSE_TAC SUFF_TAC;
val POP_ORW = POP_ASSUM (fn thm => ONCE_REWRITE_TAC [thm]);
val !! = REPEAT;
val op++ = op THEN;
val op<< = op THENL;
val op|| = op ORELSE;
val op>> = op THEN1;
val std_ss' = simpLib.++ (std_ss, boolSimps.ETA_ss);

(*---------------------------*)
fun SET_TAC L =
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION,
    IN_INTER, IN_DIFF, IN_INSERT, IN_DELETE, IN_BIGINTER, IN_BIGUNION,
    IN_IMAGE, GSPECIFICATION, IN_DEF] THEN METIS_TAC [];

fun SET_RULE tm = prove(tm,SET_TAC []);


val set_rewrs
= [INTER_EMPTY, INTER_UNIV, UNION_EMPTY, UNION_UNIV, DISJOINT_UNION,
DISJOINT_INSERT, DISJOINT_EMPTY, GSYM DISJOINT_EMPTY_REFL,
DISJOINT_BIGUNION, INTER_SUBSET, INTER_IDEMPOT, UNION_IDEMPOT,
SUBSET_UNION];

val UNIONL_def = Define `(UNIONL [] = {})
/\ (UNIONL (s::ss) = (s:'a ->bool) UNION UNIONL ss)`;


val IN_UNIONL = store_thm
("IN_UNIONL",
``!l (v:'a ). v IN UNIONL l = (?s. MEM s l /\ v IN s)``,
Induct >> RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY]
++ RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY, IN_UNION]
++ PROVE_TAC []);


val elt_rewrs
= [SUBSET_DEF, IN_COMPL, IN_DIFF, IN_UNIV, NOT_IN_EMPTY, IN_UNION,
IN_INTER, IN_IMAGE, IN_FUNSET, IN_DFUNSET, GSPECIFICATION,
DISJOINT_DEF, IN_BIGUNION, IN_BIGINTER, IN_COUNT, IN_o,
IN_UNIONL, IN_DELETE, IN_PREIMAGE, IN_SING, IN_INSERT];


fun rewr_ss ths =
simpLib.++
(std_ss,
simpLib.SSFRAG
{ac = [],
name = NONE,
convs = [],
dprocs = [],
filter = NONE,
rewrs = set_rewrs @ elt_rewrs,
congs = []});
val pset_set_ss = rewr_ss set_rewrs;
val pset_elt_ss = rewr_ss elt_rewrs;
val pset_ss = rewr_ss (set_rewrs @ elt_rewrs);


fun PSET_TAC ths =
REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_set_ss ths
++ REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_elt_ss ths;

fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Simplify = RW_TAC arith_ss;
val Suff = PARSE_TAC SUFF_TAC;
val Know = PARSE_TAC KNOW_TAC;
fun wrap a = [a];
val Rewr = DISCH_THEN (REWRITE_TAC o wrap);
val Rewr' = DISCH_THEN (ONCE_REWRITE_TAC o wrap);
val Cond =
  DISCH_THEN
  (fn mp_th =>
   let
     val cond = fst (dest_imp (concl mp_th))
   in
     KNOW_TAC cond << [ALL_TAC, DISCH_THEN (MP_TAC o MP mp_th)]
   end);
val FUN_EQ_TAC = CONV_TAC (CHANGED_CONV (ONCE_DEPTH_CONV FUN_EQ_CONV));
fun K_TAC _ = ALL_TAC;
fun MESON ths tm = prove(tm,MESON_TAC ths);
fun METIS ths tm = prove(tm,METIS_TAC ths);
val DISC_RW_KILL = DISCH_TAC THEN ONCE_ASM_REWRITE_TAC [] THEN 
                   POP_ASSUM K_TAC;
			   
val IN_REST = store_thm ("IN_REST",
 ``!x:'a. !s. x IN (REST s) <=> x IN s /\ ~(x = CHOICE s)``,
  REWRITE_TAC[REST_DEF, IN_DELETE]);
fun SET_TAC L = 
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN
    SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION, IN_INTER, IN_DIFF, 
      IN_INSERT, IN_DELETE, IN_REST, IN_BIGINTER, IN_BIGUNION, IN_IMAGE, 
      GSPECIFICATION, IN_DEF, EXISTS_PROD] THEN METIS_TAC [];
fun ASSERT_TAC tm = SUBGOAL_THEN tm STRIP_ASSUME_TAC;
fun SET_RULE tm = prove(tm,SET_TAC []);
fun ASM_SET_TAC L = REPEAT (POP_ASSUM MP_TAC) THEN SET_TAC L;
val ASM_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN ARITH_TAC;
val ASM_REAL_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN REAL_ARITH_TAC;

fun DFT_rewrs L =  REPEAT( METIS_TAC L);


val rewrs = [REAL_LE_REFL, REAL_LE_LT, REAL_LE_ANTISYM, REAL_LET_ANTISYM, REAL_LET_TOTAL, REAL_LET_TRANS, real_of_num,REAL_LT_NEG1_1,REAL_LT_NEG1_0,REAL_LE_NEG1_0,REAL_LE_NEG1_1, REAL_LT_TOTAL, max_def, min_def, REAL_LT_TRANS, REAL_LE_TRANS, REAL_MIN_COMM, REAL_MAX_COMM];

val DFT_rewrs_ss  = DFT_rewrs rewrs;



val indep_var_set_WOR_def = Define `indep_var_set_WOR Y X_a X_d Z p t =
                           indep_vars p (\i. lborel)
              (\i.
                   if i = 0 then (\s. real (X_d s))
                   else if i = 1 then (\s. real (Y s))
                   else (\s. real (Z s))) {0; 1; 2} /\
		   indep_sets p
              (\i.
                   {if i = 0 then DFT_event p (WSP Y X_a X_d) t
                    else DFT_event p Z t}) {0; 1} /\
            random_variable (\s. real (X_a s)) p
              (m_space lborel,measurable_sets lborel) `;


val prob_WSP_OR = store_thm("prob_WSP_OR",
  `` !Y X_a X_d Z p t f_xy f_y f_cond.
            (DISJOINT_WSP Y X_a X_d  t) /\
            (!s. ALL_DISTINCT [Y s; X_a s; X_d s; Z s]) /\
            All_distinct_events p [WSP Y X_a X_d; Z] t /\
            rv_gt0_ninfinity [Y; X_d; X_a; Z] /\ 0 <= t /\ 0 <= t /\
            (!x y.
                 cond_density lborel lborel p (\s. real (X_a s))
                   (\s. real (Y s)) y f_xy f_y f_cond) /\
            den_gt0_ninfinity f_xy f_y f_cond /\
            cont_CDF p (\s. real (X_d s)) /\
            measurable_CDF p (\s. real (X_d s)) /\
           indep_var_set_WOR Y X_a X_d Z p t ==>
            (prob p (DFT_event p (D_OR (WSP Y X_a X_d) Z) t) =
	     pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_y y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond y x)) +
   pos_fn_integral lborel
     (\y.
          f_y y *
          (indicator_fn {u | 0 <= u /\ u <= t} y *
           CDF p (\s. real (X_d s)) y)) + CDF p (\s. real (Z s)) t 
   -(pos_fn_integral lborel
       (\y.
            indicator_fn {u | 0 <= u /\ u <= t} y * f_y y *
            pos_fn_integral lborel
              (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond y x)) +
     pos_fn_integral lborel
       (\y.
            f_y y *
            (indicator_fn {u | 0 <= u /\ u <= t} y *
             CDF p (\s. real (X_d s)) y))) * CDF p (\s. real (Z s)) t
	     )``,			       

RW_TAC std_ss[OR_UNION, rv_gt0_ninfinity_def, All_distinct_events_def, indep_var_set_WOR_def]
THEN `indep_vars p (\i. if i = 0 then lborel else if i = 1 then lborel else lborel) (\i.
                if i = 0 then (\s. real (X_d s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s))) {0; 1; 2}` by RW_TAC std_ss[]
THEN KNOW_TAC(``indep_var p lborel (\x. real (X_d x)) lborel (\x. real (Y x))``)
     THEN1(MP_TAC (Q.ISPECL [`(\i:num.
                if i = 0 then (\s. real ((X_d:'a->extreal) s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `lborel`, `0:num`, `1:num`, `{0;1;2:num}`] indep_vars_var)
           THEN `0 IN {0; 1; 2:num} /\ 1 IN {0; 1; 2:num} /\ 0 <> 1:num` by ASM_SET_TAC[]
	   THEN RW_TAC std_ss[])
	   
THEN DISCH_TAC
THEN KNOW_TAC(``indep_var p lborel (\x. real (X_d x)) lborel (\x. real (Z x)) ``)
     THEN1(MP_TAC (Q.ISPECL [`(\i:num.
                if i = 0 then (\s. real ((X_d:'a->extreal) s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `lborel`, `0:num`, `2:num`, `{0;1;2:num}`] indep_vars_var)
           THEN `0 IN {0; 1; 2:num} /\ 2 IN {0; 1; 2:num} /\ 0 <> 2:num` by ASM_SET_TAC[]
	   THEN RW_TAC std_ss[])
THEN DISCH_TAC
THEN KNOW_TAC(``indep_var p lborel (\x. real (Y x)) lborel (\x. real (Z x)) ``)
     THEN1(MP_TAC (Q.ISPECL [`(\i:num.
                if i = 0 then (\s. real ((X_d:'a->extreal) s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `lborel`, `1:num`, `2:num`, `{0;1;2:num}`] indep_vars_var)
           THEN `1 IN {0; 1; 2:num} /\ 2 IN {0; 1; 2:num} /\ 1 <> 2:num` by ASM_SET_TAC[]
	   THEN RW_TAC std_ss[])
THEN DISCH_TAC
THEN KNOW_TAC(``random_variable (\s. real ((X_d:'a->extreal) s)) p borel /\
                 random_variable  (\s. real (Y s)) p borel /\
		  random_variable  (\s. real (Z s)) p borel``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars] 
     	   THEN UNDISCH_TAC(`` !i:num.
       i IN {0; 1; 2} ==>
             random_variable
               (if i = 0 then (\s. real (X_d s))
                else if i = 1 then (\s. real (Y s))
                else (\s. real (Z s))) p
               (m_space lborel,measurable_sets lborel)``)
	  THEN DISCH_TAC
	  THEN RW_TAC std_ss []
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `0:num `)
	  	THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN METIS_TAC [subsets_def, measurable_sets_def, borel, lborel, sigma_def, m_space_def, space_def])
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `1:num `)
	  	THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN METIS_TAC [subsets_def, measurable_sets_def, borel, lborel, sigma_def, m_space_def, space_def])
	  THEN POP_ASSUM (MP_TAC o Q.SPEC `2:num `)
	  	THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN METIS_TAC [subsets_def, measurable_sets_def, borel, lborel, sigma_def, m_space_def, space_def])
THEN DISCH_TAC
THEN KNOW_TAC(``prob_space p``)
    THEN1(FULL_SIMP_TAC std_ss[indep_var, indep_vars, indep_sets])

THEN DISCH_TAC
THEN KNOW_TAC(``DFT_event p X_d t IN events p``)
     THEN1(RW_TAC std_ss [DFT_event_def]
      THEN KNOW_TAC(``{s:'a | X_d s <= Normal t}=PREIMAGE X_d {s |  s <= Normal t}``) THEN1(
      	             SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
      THEN DISCH_TAC
      THEN ASM_REWRITE_TAC[]
      THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
      THEN REWRITE_TAC [events_def]
      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``) THEN1(
	       	    		RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
      THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
      THEN REWRITE_TAC [p_space_def, m_space_def]
      THEN KNOW_TAC(``prob_space p``) THEN1(FULL_SIMP_TAC std_ss [indep_var, indep_vars, indep_sets])
      THEN DISCH_TAC
      THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets])
THEN DISCH_TAC

THEN KNOW_TAC(``DFT_event p X_a t IN events p``)
     THEN1(RW_TAC std_ss [DFT_event_def]
      THEN KNOW_TAC(``{s:'a | X_a s <= Normal t}=PREIMAGE X_a {s |  s <= Normal t}``) THEN1(
      	             SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
      THEN DISCH_TAC
      THEN ASM_REWRITE_TAC[]
      THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
      THEN REWRITE_TAC [events_def]
      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``) THEN1(
	       	    		RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
      THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
      THEN REWRITE_TAC [p_space_def, m_space_def]
      THEN KNOW_TAC(``prob_space p``) THEN1(FULL_SIMP_TAC std_ss [indep_var, indep_vars, indep_sets])
      THEN DISCH_TAC
      THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets])
THEN DISCH_TAC

THEN KNOW_TAC(``DFT_event p Y t IN events p``)
     THEN1(RW_TAC std_ss [DFT_event_def]
      THEN KNOW_TAC(``{s:'a | Y s <= Normal t}=PREIMAGE Y {s |  s <= Normal t}``) THEN1(
      	             SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
      THEN DISCH_TAC
      THEN ASM_REWRITE_TAC[]
      THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
      THEN REWRITE_TAC [events_def]
      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``) THEN1(
	       	    		RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
      THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
      THEN REWRITE_TAC [p_space_def, m_space_def]
      THEN KNOW_TAC(``prob_space p``) THEN1(FULL_SIMP_TAC std_ss [indep_var, indep_vars, indep_sets])
      THEN DISCH_TAC
      THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets])
THEN DISCH_TAC
THEN KNOW_TAC(``DFT_event p Z t IN events p``)
     THEN1(RW_TAC std_ss [DFT_event_def]
      THEN KNOW_TAC(``{s:'a | Z s <= Normal t}=PREIMAGE Z {s |  s <= Normal t}``) THEN1(
      	             SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
      THEN DISCH_TAC
      THEN ASM_REWRITE_TAC[]
      THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
      THEN REWRITE_TAC [events_def]
      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``) THEN1(
	       	    		RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
      THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
      THEN REWRITE_TAC [p_space_def, m_space_def]
      THEN KNOW_TAC(``prob_space p``) THEN1(FULL_SIMP_TAC std_ss [indep_var, indep_vars, indep_sets])
      THEN DISCH_TAC
      THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def, lborel_le,lborel, borel, sigma_def, indep_var, indep_vars, indep_sets])
THEN DISCH_TAC

THEN KNOW_TAC(``DFT_event p (WSP Y X_a X_d) t IN events p ``)
     THEN1(KNOW_TAC(``WSP Y X_a X_d = D_OR (D_AND (X_a)(D_BEFORE Y X_a))(D_AND (Y)(D_BEFORE X_d Y)) ``)   
           THEN1(FULL_SIMP_TAC std_ss[D_AND_def, NEVER_def]
	         THEN RW_TAC std_ss[D_OR_def, D_AND_def,D_BEFORE_def, WSP_def, D_SIMULT_def]
		 THEN KNOW_TAC(`` !s. (
        min
          (min
             (min (max (X_a s) (if Y s < X_a s then Y s else PosInf))
                (max (Y s) (if X_d s < Y s then X_d s else PosInf)))
             (if Y s = X_a s then X_a s else PosInf))
          (if Y s = X_d s then X_d s else PosInf)) =
   (
        min (max (X_a s) (if Y s < X_a s then Y s else PosInf))
          (max (Y s) (if X_d s < Y s then X_d s else PosInf)))``)
	         THEN1( RW_TAC std_ss[extreal_max_def, extreal_min_def] THEN REPEAT (POP_ASSUM MP_TAC) THEN RW_TAC std_ss[] THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC[lt_infty, le_infty, lt_le, le_lt, lt_trans, le_trans, extreal_lt_def, extreal_le_def, ALL_DISTINCT, MEM])
		 THEN RW_TAC std_ss[])
	 THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]	 
         THEN RW_TAC std_ss[OR_UNION]
	 THEN KNOW_TAC(`` DFT_event p (D_AND X_a (D_BEFORE Y X_a)) t IN events p ``)
	     THEN1(RW_TAC std_ss[DFT_event_AFTER_PREIMAGE_GT0, rv_gt0_ninfinity_def]
	          THEN `{(u,w) | u < w /\ 0 <= w /\ w <= t} IN measurable_sets (pair_measure lborel lborel)` by RW_TAC std_ss[after_set_IN_MEASURABLE_SETS_PAIR_lborel]
                  THEN `(\x. (real (Y x),real (X_a x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel))`
		       by METIS_TAC[random_variable_def, p_space_def, events_def, m_space_def, measurable_sets_def, measurable_Pair, prob_space_def,  measure_space_lborel, borel, lborel, subsets_def, space_def, sigma_def]
		  THEN METIS_TAC[IN_MEASURABLE, random_variable_def, space_def, m_space_def, subsets_def, measurable_sets_def, events_def, p_space_def])
        THEN DISCH_TAC
	THEN KNOW_TAC(`` DFT_event p (D_AND Y (D_BEFORE X_d Y)) t IN events p``)
	 THEN1(RW_TAC std_ss[DFT_event_AFTER_PREIMAGE_GT0, rv_gt0_ninfinity_def]
	          THEN `{(u,w) | u < w /\ 0 <= w /\ w <= t} IN measurable_sets (pair_measure lborel lborel)` by RW_TAC std_ss[after_set_IN_MEASURABLE_SETS_PAIR_lborel]
                  THEN `(\x. (real (X_d x),real (Y x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel))`
		       by METIS_TAC[random_variable_def, p_space_def, events_def, m_space_def, measurable_sets_def, measurable_Pair, prob_space_def,  measure_space_lborel, borel, lborel, subsets_def, space_def, sigma_def]
		  THEN METIS_TAC[IN_MEASURABLE, random_variable_def, space_def, m_space_def, subsets_def, measurable_sets_def, events_def, p_space_def])
        THEN DISCH_TAC
	THEN METIS_TAC [EVENTS_UNION])
THEN DISCH_TAC
THEN KNOW_TAC(``DFT_event p (WSP Y X_a X_d) t UNION DFT_event p Z t=
                 union_list[DFT_event p (WSP Y X_a X_d) t; DFT_event p Z t]``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, union_list_def])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN DEP_REWRITE_TAC[PROB_INCLUSION_EXCLUSION_PRINCIPLE]
THEN RW_TAC list_ss[]

     THEN1(RW_TAC std_ss[EVENTS_INTER, AND_INTER])
     THEN1(RW_TAC std_ss[EVENTS_INTER, AND_INTER])
THEN RW_TAC std_ss [sum_set_def]


THEN KNOW_TAC(``{t' | t' SUBSET {DFT_event p (WSP Y X_a X_d) t; DFT_event p Z t} /\ t' <> {}}  =
                 {{DFT_event p (WSP Y X_a X_d) t}; {DFT_event p Z t}; {DFT_event p (WSP Y X_a X_d) t; DFT_event p Z t}}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
     THEN EQ_TAC
     	  THEN1(ASM_SET_TAC[])
     	  THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN DEP_REWRITE_TAC[PIE_lem2_or]
THEN RW_TAC std_ss[]
     THEN1(FULL_SIMP_TAC std_ss[MAP])
     THEN1(RW_TAC std_ss[extreal_pow_def]
       THEN DEP_REWRITE_TAC[PIE_lemma6]
       THEN DEP_REWRITE_TAC[PIE_lemma3]
       THEN DEP_REWRITE_TAC[PROB_FINITE,EVENTS_COUNTABLE_INTER,COUNTABLE_IMAGE,finite_countable,IMAGE_FINITE]
       THEN (PSET_TAC[FINITE_SING,EVENTS_INTER,NOT_SING_EMPTY] THEN SRW_TAC[][FINITE_SING] THEN METIS_TAC[EVENTS_INTER, AND_INTER]))
THEN RW_TAC std_ss[]
THEN RW_TAC real_ss[BIGINTER_SING,CARD_SING,CARD_INSERT,GSYM ADD1]
THEN FULL_SIMP_TAC list_ss[ALL_DISTINCT]
THEN PSET_TAC[]
THEN RW_TAC real_ss[extreal_pow_def]
THEN SRW_TAC[][FINITE_SING]
THEN RW_TAC real_ss[pow,prove (``!A B:extreal->bool. A <> B  ==> ((CARD {A;B} = SUC (SUC 0)) )``, RW_TAC list_ss[ALL_DISTINCT] THEN DEP_REWRITE_TAC[CARD_DEF] THEN SRW_TAC[][FINITE_SING])]
THEN DEP_REWRITE_TAC[PIE_lemma5]
THEN DEP_REWRITE_TAC[PROB_FINITE,EVENTS_COUNTABLE_INTER,COUNTABLE_IMAGE,finite_countable,IMAGE_FINITE]
THEN RW_TAC std_ss[] THEN  PSET_TAC[AND_INTER,FINITE_SING,EVENTS_INTER,NOT_SING_EMPTY] THEN SRW_TAC[][FINITE_SING]
THEN RW_TAC std_ss[prove(``Normal 1 = 1``, RW_TAC std_ss[extreal_of_num_def] THEN METIS_TAC[]),mul_lone]
THEN KNOW_TAC(``DFT_event p (WSP Y X_a X_d) t INTER DFT_event p Z t IN events p``)
    THEN1(METIS_TAC [EVENTS_INTER])
THEN DISCH_TAC
THEN RW_TAC real_ss[PROB_FINITE,prove (``!A B C:extreal. A <> PosInf /\ A <> NegInf /\ B <> PosInf /\ B <> NegInf /\ C <> PosInf /\ C <> NegInf  ==>( A + B + - C = A + B - C)``, REPEAT Cases THEN RW_TAC std_ss[] THEN DEP_REWRITE_TAC [extreal_sub_add] THEN RW_TAC std_ss[extreal_not_infty, add_not_infty] )]

THEN RW_TAC std_ss[GSYM CDF_prob_DFT_event]
THEN KNOW_TAC(`` prob p (DFT_event p (WSP Y X_a X_d) t INTER DFT_event p Z t) =
                 prob p (DFT_event p (WSP Y X_a X_d) t) * prob p ( DFT_event p Z t)``)

     THEN1( MP_TAC (Q.ISPECL [`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,`(\i:num.
                {if i = 0 then DFT_event p (WSP (Y:'a->extreal) X_a X_d) t
                 else DFT_event p (Z:'a->extreal) t})`,`{0;1:num}`, `(\i:num.
                if i = 0 then DFT_event p (WSP (Y:'a->extreal) X_a X_d) t
                 else DFT_event p (Z:'a->extreal) t)`, `0:num`, `1:num`]prob_prod_any_2_of_10_set)

            THEN RW_TAC std_ss[]
	    THEN `{0;1} SUBSET {0;1:num}` by ASM_SET_TAC[]
	    THEN `(!i:num.
              i IN {0; 1} ==>
              (if i = 0 then DFT_event p (WSP Y X_a X_d) t
               else DFT_event p Z t) IN
              {if i = 0 then DFT_event p (WSP Y X_a X_d) t
               else DFT_event p Z t})` by ASM_SET_TAC[]
	    THEN FULL_SIMP_TAC std_ss[]
	    THEN KNOW_TAC(``(!i:num.
              i IN {0; 1} ==>
              (if i = 0 then DFT_event p (WSP Y X_a X_d) t
               else DFT_event p Z t) IN events p) ``)
	        THEN1(RW_TAC std_ss[])
	    THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[GSYM CDF_prob_DFT_event]
THEN KNOW_TAC(`` prob p (DFT_event p (WSP Y X_a X_d) t) =
                  pos_fn_integral lborel
             (\y.
                  indicator_fn {u | 0 <= u /\ u <= t} y * f_y y *
                  pos_fn_integral lborel
                    (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond y x)) +
           pos_fn_integral lborel
             (\y.
                  f_y y *
                  (indicator_fn {u | 0 <= u /\ u <= t} y *
                   CDF p (\s. real (X_d s)) y))``)
     THEN1(DEP_REWRITE_TAC[prob_WSP]
           THEN RW_TAC std_ss[]
           THEN Q.EXISTS_TAC `f_xy`
     	   THEN RW_TAC std_ss[]
	          THEN1(METIS_TAC [ALL_DISTINCT, MEM])
	   THEN RW_TAC std_ss[rv_gt0_ninfinity_def])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]   	       
THEN RW_TAC std_ss[]);


val _ = export_theory();
