(******************************************************************************)
(*                       Probabilistic Analysis of                            *)
(*                        PAND CSP                                            *)
(*                                  by                                        *)
(*                          Yassmeen Elderhalli                               *)
(*                      y_elderh@encs.concordia.ca                            *)
(******************************************************************************)

(*loadPath := "/home/yassmeen/Downloads/ProbTheories_HOL-k12" :: !loadPath;
loadPath := "/home/yassmeen/Downloads/My-Theories" :: !loadPath;


(*set_trace "Unicode" 0;*)


app load ["HolKernel", "Parse", "boolLib", "bossLib", "numLib", "unwindLib", 
"tautLib", "Arith", "prim_recTheory", "combinTheory", "quotientTheory", 
"arithmeticTheory", "realTheory", "hrealTheory", "realaxTheory", "realLib",  
"jrhUtils", "pairTheory", "seqTheory", "limTheory", "transcTheory", "listTheory", 
"mesonLib", "boolTheory", "topologyTheory", "pred_setTheory", "util_probTheory", 
"optionTheory", "numTheory", "sumTheory", "InductiveDefinition", "ind_typeTheory",
"pred_setLib", "iterate_hvgTheory", "card_hvgTheory", "product_hvgTheory", 
"topology_hvgTheory", "derivative_hvgTheory", "integration_hvgTheory",
"real_sigmaTheory", "extreal_hvgTheory", "measure_hvgTheory", "lebesgue_hvgTheory",
"probability_hvgTheory", "lebesgue_measure_hvgTheory", "normal_rv_hvgTheory", "netsTheory", "PIE_EXTREALTheory","dep_rewrite", "integration_before_afterTheory", "DFT_Gates_def_PROBTheory" ,"CASTheory","WCSPTheory", "AND_FDEPTheory", "WCSP_sharedTheory"];
*)
open HolKernel Parse boolLib bossLib numLib unwindLib tautLib Arith prim_recTheory 
combinTheory quotientTheory arithmeticTheory hrealTheory realaxTheory realTheory 
realLib jrhUtils pairTheory seqTheory limTheory transcTheory listTheory mesonLib 
boolTheory topologyTheory pred_setTheory util_probTheory optionTheory numTheory 
sumTheory InductiveDefinition ind_typeTheory pred_setLib iterate_hvgTheory 
card_hvgTheory product_hvgTheory topology_hvgTheory derivative_hvgTheory
integration_hvgTheory real_sigmaTheory extreal_hvgTheory measure_hvgTheory
lebesgue_hvgTheory probability_hvgTheory lebesgue_measure_hvgTheory normal_rv_hvgTheory netsTheory PIE_EXTREALTheory dep_rewrite integration_before_afterTheory DFT_Gates_def_PROBTheory  CASTheory WCSPTheory AND_FDEPTheory WCSP_sharedTheory ;


val _ = new_theory "prob_PAND_CSP";
fun SET_TAC L =
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION,
    IN_INTER, IN_DIFF, IN_INSERT, IN_DELETE, IN_BIGINTER, IN_BIGUNION,
    IN_IMAGE, GSPECIFICATION, IN_DEF] THEN METIS_TAC [];

fun SET_RULE tm = prove(tm,SET_TAC []);


val set_rewrs
= [INTER_EMPTY, INTER_UNIV, UNION_EMPTY, UNION_UNIV, DISJOINT_UNION,
DISJOINT_INSERT, DISJOINT_EMPTY, GSYM DISJOINT_EMPTY_REFL,
DISJOINT_BIGUNION, INTER_SUBSET, INTER_IDEMPOT, UNION_IDEMPOT,
SUBSET_UNION];

val UNIONL_def = Define `(UNIONL [] = {})
/\ (UNIONL (s::ss) = (s:'a ->bool) UNION UNIONL ss)`;


val IN_UNIONL = store_thm
("IN_UNIONL",
``!l (v:'a ). v IN UNIONL l = (?s. MEM s l /\ v IN s)``,
Induct THEN1 RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY]
THEN RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY, IN_UNION]
THEN PROVE_TAC []);


val elt_rewrs
= [SUBSET_DEF, IN_COMPL, IN_DIFF, IN_UNIV, NOT_IN_EMPTY, IN_UNION,
IN_INTER, IN_IMAGE, IN_FUNSET, IN_DFUNSET, GSPECIFICATION,
DISJOINT_DEF, IN_BIGUNION, IN_BIGINTER, IN_COUNT, IN_o,
IN_UNIONL, IN_DELETE, IN_PREIMAGE, IN_SING, IN_INSERT];


fun rewr_ss ths =
simpLib.++
(std_ss,
simpLib.SSFRAG
{ac = [],
name = NONE,
convs = [],
dprocs = [],
filter = NONE,
rewrs = set_rewrs @ elt_rewrs,
congs = []});
val pset_set_ss = rewr_ss set_rewrs;
val pset_elt_ss = rewr_ss elt_rewrs;
val pset_ss = rewr_ss (set_rewrs @ elt_rewrs);


fun PSET_TAC ths =
REPEAT (POP_ASSUM MP_TAC)
THEN RW_TAC pset_set_ss ths
THEN  REPEAT (POP_ASSUM MP_TAC)
THEN RW_TAC pset_elt_ss ths;

val Simplify = RW_TAC arith_ss;
fun wrap a = [a];
val Rewr = DISCH_THEN (REWRITE_TAC o wrap);
val Rewr' = DISCH_THEN (ONCE_REWRITE_TAC o wrap);
val FUN_EQ_TAC = CONV_TAC (CHANGED_CONV (ONCE_DEPTH_CONV FUN_EQ_CONV));
fun K_TAC _ = ALL_TAC;
fun MESON ths tm = prove(tm,MESON_TAC ths);
fun METIS ths tm = prove(tm,METIS_TAC ths);
val DISC_RW_KILL = DISCH_TAC THEN ONCE_ASM_REWRITE_TAC [] THEN 
                   POP_ASSUM K_TAC;
			   
val IN_REST = store_thm ("IN_REST",
 ``!x:'a. !s. x IN (REST s) <=> x IN s /\ ~(x = CHOICE s)``,
  REWRITE_TAC[REST_DEF, IN_DELETE]);
fun SET_TAC L = 
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN
    SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION, IN_INTER, IN_DIFF, 
      IN_INSERT, IN_DELETE, IN_REST, IN_BIGINTER, IN_BIGUNION, IN_IMAGE, 
      GSPECIFICATION, IN_DEF, EXISTS_PROD] THEN METIS_TAC [];
fun ASSERT_TAC tm = SUBGOAL_THEN tm STRIP_ASSUME_TAC;
fun SET_RULE tm = prove(tm,SET_TAC []);
fun ASM_SET_TAC L = REPEAT (POP_ASSUM MP_TAC) THEN SET_TAC L;
val ASM_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN ARITH_TAC;
val ASM_REAL_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN REAL_ARITH_TAC;


fun DFT_rewrs L =  REPEAT( METIS_TAC L);


val rewrs = [REAL_LE_REFL, REAL_LE_LT, REAL_LE_ANTISYM, REAL_LET_ANTISYM, REAL_LET_TOTAL, REAL_LET_TRANS, real_of_num,REAL_LT_NEG1_1,REAL_LT_NEG1_0,REAL_LE_NEG1_0,REAL_LE_NEG1_1, REAL_LT_TOTAL, max_def, min_def, REAL_LT_TRANS, REAL_LE_TRANS, REAL_MIN_COMM, REAL_MAX_COMM];

val DFT_rewrs_ss  =DFT_rewrs rewrs;
val PAND_CSP_event = store_thm("PAND_CSP_event",
  `` ! X Y Z p t. rv_gt0_ninfinity [X;Y;Z] /\ (!s. ALL_DISTINCT [X s; Y s ; Z s]) ==>

   (DFT_event p (D_AND (D_AND (X)(D_BEFORE Z Y))(D_BEFORE Y X)) t =
           {s|  0 <= Z s /\  Z s < Y s /\ 0 <= Y s /\ Y s <= Normal t /\Y s < X s /\ X s <=Normal t } INTER p_space p)``,

RW_TAC std_ss[AND_INTER, rv_gt0_ninfinity_def] 
THEN RW_TAC std_ss[D_BEFORE_def,DFT_event_def] 
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
            THEN METIS_TAC [extreal_not_infty, extreal_le_def, extreal_lt_def, lt_le, le_lt, lt_infty, le_infty])
THEN RW_TAC std_ss []
THEN METIS_TAC [extreal_not_infty, extreal_le_def, extreal_lt_def, lt_le, le_lt, lt_infty, le_infty, let_trans, lte_trans, ALL_DISTINCT, MEM]);

val event_PAND_CSP_indicator_of_indicator= store_thm("event_PAND_CSP_indicator_of_indicator",
  ``  !x y z t:real.
       indicator_fn
         {((x',y'),z') |
          0 <= z' /\  z' < y' /\ 0 <= y' /\ y' <= t /\  y'  < x'  /\ x' <=t} ((x,y),z) =
       indicator_fn {z' | 0 <= z' /\ z' < y} z *
       indicator_fn {x' |  y < x' /\ x' <= t} x *
       indicator_fn {y' | 0 <= y' /\ y' <= t } y``,
       
REPEAT GEN_TAC
THEN RW_TAC std_ss [indicator_fn_def]
  THEN1(METIS_TAC [mul_lone])

THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <=z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)



THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)



THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)




THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN METIS_TAC [mul_rzero, mul_comm]);


val BIGUNION_event_PAND_CSP_subevent0 = store_thm(
  "BIGUNION_event_PAND_CSP_subevent0",
  `` !t q1 q2.BIGUNION {({x' | real q0 < x' /\ x' <= t} CROSS
                         {y'| real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0 }) CROSS
		      {z' |0 <= z' /\  z' < real q1 }| q0 IN Q_set}=
		      {(x',y') | real q1 < y'/\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} CROSS
		      {z' | 0<= z'/\ z' < real q1}  ``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(STRIP_TAC
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
              
              x IN s <=>
            ((real q0 < FST (FST x) /\ FST (FST x) <= t) /\
              real q1 < SND (FST x) /\ 0 <= SND (FST x) /\
              SND (FST x) <= t /\ SND (FST x) < real q0) /\ 0 <= SND x /\
             SND x < real q1``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)
          THEN `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EXISTS_TAC(``FST (x:(real#real)#real) ``)
           THEN RW_TAC std_ss [LAMBDA_PAIR]
	   THEN METIS_TAC [REAL_LT_TRANS])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]

THEN  KNOW_TAC(``?q0. q0 IN Q_set/\ Normal (SND (x':real#real) ) < q0 /\  q0 < Normal (FST x')``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r0. q0 = Normal r0``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (SND (FST (x:(real#real)#real))) < Normal( FST (FST x))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC

THEN EXISTS_TAC (``{x| real (Normal r0) < FST (FST x) /\ FST (FST x) <= t /\
                 real q1 < SND (FST x) /\ 0 <= SND (FST x) /\
                 SND (FST x) <= t /\ SND (FST x) < real (Normal r0) /\ 0 <= SND x /\
                SND x < real q1}  ``)
     

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r0`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);


val event_PAND_CSP_subevent0_measurable_set = store_thm(
  "event_PAND_CSP_subevent0_measurable_set",
  ``!t q0 q1. ({x' | real q0 < x' /\ x' <= t} CROSS
            {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
            {z' | 0 <= z' /\ z' < real q1} IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,

RW_TAC std_ss [measurable_sets_def, pair_measure, measure_of, lborel, m_space_def]
    THEN1 (KNOW_TAC(``  {x' | real q0 < x' /\ x' <= t} CROSS
                        {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0} CROSS
   			{z' | 0 <= z' /\ z' < real q1} IN
                       {(a:real reln) CROSS (b:real->bool) |
                         a IN
                         sigma_sets (univ(:real) CROSS univ(:real))
                         {a CROSS b | a IN subsets borel /\ b IN subsets borel} /\
                         b IN subsets borel}``)
           THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	         THEN EXISTS_TAC (``({x' | real q0 < x' /\ x' <= t} CROSS
        {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0},
        {z' | 0 <= z' /\ z' < real q1}) ``)
                THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                     THEN1(KNOW_TAC(``{x' | real q0 < x' /\ x' <= t} CROSS
   		                      {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0} IN
				       {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
                          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			        THEN EXISTS_TAC (``({x' | real q0 < x' /\ x' <= t},
                              {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0}) ``)
			        THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                                     THEN1( KNOW_TAC(``{x' | real q0 < x' /\ x' <= t} =
					    {x' | real q0 < x'} INTER
					    {x' | x' <= t} ``)
					    THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
					          THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC])
				    THEN DISCH_TAC
				    THEN ASM_REWRITE_TAC[]
                                    THEN `{x' | real q0 < x'} IN subsets borel /\
					    {x' | x' <= t} IN subsets borel` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_LEFT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT] 
 			            THEN  METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
			    THEN KNOW_TAC(``   {y' |  real q1 <y' /\ 0 <= y' /\ y' <= t /\ y' < real q0} =
			    	 	       {y' |  real q1 < y'} INTER
					       {y'| 0 <= y'} INTER
					       {y' | y' <= t} INTER
					       {y' | y' < real q0}``)
			        THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
				      THEN METIS_TAC[CONJ_ASSOC, CONJ_COMM])
 			   THEN DISCH_TAC				      
			   THEN ASM_REWRITE_TAC[]
			   THEN `{y' | real q1 < y'} IN subsets borel /\
			         {y' | 0 <= y'} IN subsets borel /\
				 {y' | y' <= t} IN subsets borel /\
 				 {y' | y' < real q0} IN subsets borel` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_LEFT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, CLOSED_INTERVAL_RIGHT]
    		          THEN 	METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
	           THEN DISCH_TAC 
		   THEN METIS_TAC [sigma_sets_basic])
              THEN `{z' | 0<= z' /\ z' < real q1} = {z' | 0 <= z'} INTER {z'| z' < real q1} ` 		   by SRW_TAC [] [GSPECIFICATION, EXTENSION]
              THEN ASM_REWRITE_TAC[]
	      THEN `{z' | 0 <= z'} IN subsets borel /\ {z' | z' < real q1} IN subsets borel`
	         by METIS_TAC [borel_open,borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
             THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
        THEN DISCH_TAC
        THEN METIS_TAC [sigma_sets_basic])
   THEN1( KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
         THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
	         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(``{(a:real->bool) CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET
          POW univ(:real#real) ``)
             THEN1( METIS_TAC [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
	 THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[])
THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]	
THEN FULL_SIMP_TAC std_ss[GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV]);



val BIGUNION_event_PAND_CSP_subevent0_measurable_set = store_thm(
  "BIGUNION_event_PAND_CSP_subevent0_measurable_set",
  ``!t q1 q2.   {(x',y') |
           real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} 
           CROSS {z' | 0 <= z' /\ z' < real q1} IN
	  measurable_sets (pair_measure (pair_measure lborel lborel) lborel)  ``,

RW_TAC std_ss [GSYM BIGUNION_event_PAND_CSP_subevent0]

THEN KNOW_TAC (``!q0. q0 IN Q_set ==> (({x' | real q0 < x' /\ x' <= t} CROSS
     {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
     {z' | 0 <= z' /\ z' < real q1})   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[event_PAND_CSP_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
   BIGUNION
     {({x' | real q0 < x' /\ x' <= t} CROSS
     {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
     {z' | 0 <= z' /\ z' < real q1} |
     q0 IN Q_set} = BIGUNION
  (IMAGE (\q0. ({x' | real q0 < x' /\ x' <= t} CROSS
     {y' | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
     {z' | 0 <= z' /\ z' < real q1} )
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );



val BIGUNION_event_PAND_CSP_subevent = store_thm(
  "BIGUNION_event_PAND_CSP_subevent",
  `` !t. BIGUNION
           {{(x',y') | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} CROSS
          {z' | 0 <= z' /\ z' < real q1} |
           q1 IN Q_set}  =
	   {((x',y'),z') |
        0<= z' /\ z' < y' /\ 0<=y'  /\ y' <= t /\ y' < x' /\ x' <= t}``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN RW_TAC std_ss [LAMBDA_PAIR]
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
               x IN s <=>
            (?x'.
                  (FST x,T) =
                  (\(x',y').
                       ((x',y'),
                        real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\
                        x' <= t)) x') /\ 0 <= SND x /\ SND x < real q1
``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)
          THEN `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN KNOW_TAC(``(\ (x':real,y').
             ((x',y'),
               real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t)) x''
              = (x'', real q1 < SND x''/\ 0 <= SND x'' /\ SND x'' <=t /\ SND x''< FST x'' /\ FST x''<=t ) ``)
                THEN1 ( KNOW_TAC (`` (\ (x':real,y').
        ((x',y'),real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t))
     x'' =  (\(x',y').
        ((x',y'),real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t))
      (FST x'' , SND x'')``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC
	 THEN Q.EXISTS_TAC(`(FST (FST (x:(real#real)#real)), SND (FST x), SND x) `)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]
THEN  KNOW_TAC(``?q1. q1 IN Q_set/\ Normal (SND (SND(x':real#real#real) )) < q1 /\  q1 < Normal (FST (SND x'))``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r1. q1 = Normal r1``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (SND (SND(x':real#real#real))) < Normal(FST  (SND x'))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{x| (real (Normal r1) < SND (FST x) /\ 0 <= SND (FST x) /\
                 SND (FST x) <= t /\ SND (FST x) < FST (FST x) /\
                 FST (FST x) <= t) /\ 0 <= SND x /\ SND x < real (Normal r1)}``)
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r1`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);





val PAND_CSP_event_measurable_set = store_thm(
  "PAND_CSP_event_measurable_set",
  `` !t.
	  {((x',y'),z') |
          0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
       measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,


RW_TAC std_ss [GSYM BIGUNION_event_PAND_CSP_subevent]
THEN KNOW_TAC (``!q1. q1 IN Q_set ==> (({(x',y') | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}) CROSS
     {z' | 0 <= z' /\ z' < real q1})   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[BIGUNION_event_PAND_CSP_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
     {{(x',y') | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} CROSS
        {z' | 0 <= z' /\ z' < real q1} |
     q1 IN Q_set} = BIGUNION
  (IMAGE (\q1. {(x',y') | real q1 < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} CROSS
        {z' | 0 <= z' /\ z' < real q1})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );


val event_PAND_CSP_indicator_of_indicator_Axy = store_thm(
  "event_PAND_CSP_indicator_of_indicator_Axy",
  `` ! x y z (t:real).
        indicator_fn  {((x',y'),z') |
          0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} ((x,y),z) =
	indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)*
	indicator_fn {z' | 0 <= z' /\ z' < y} z  ``,

RW_TAC std_ss [indicator_fn_def]
      THEN1(RW_TAC std_ss[mul_lone])
      THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

    THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
 THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           0 <= z'' /\ z'' < y'' /\ 0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
 THEN RW_TAC std_ss [mul_rzero, mul_lzero]);
   



val joint_density_3rv_1_PAND_CSP = store_thm("joint_density_3rv_1_PAND_CSP",

``!p X Y Z  f_xy t.
     prob_space p /\ random_variable Z p borel /\
     (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
     distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy  ==>
     (pos_fn_integral
        (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
        (\ (x,y).
           pos_fn_integral (distr p lborel Z)
             (\z. indicator_fn {((x',y'),z') |
            0 <= z' /\ z' < y' /\ 0<=y' /\ y' <=t /\  y' < x' /\ x' <= t} ((x,y),z))) =
      pos_fn_integral lborel
        (\y.
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {(x',y')|  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                pos_fn_integral (distr p lborel Z)
                  (\z. indicator_fn {z' |0 <= z' /\ z' <y} z))))``,

RW_TAC std_ss[distributed_def]
THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
			       THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                     THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
                               THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]

THEN MP_TAC(Q.ISPECL[` f_xy:(real#real)->extreal `, `(\ (x:real,y:real).
     pos_fn_integral (distr p lborel Z)
       (\z. indicator_fn {((x',y'),z') |
                    0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((x,y),z)))` , `(pair_measure lborel lborel)` ]  pos_fn_integral_density_1)
THEN RW_TAC std_ss[measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]  THEN UNDISCH_TAC (``f_xy IN
      measurable
        (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel)) Borel``)
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``{((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} IN measurable_sets (pair_measure (pair_measure lborel lborel)lborel) ``)
    THEN1(RW_TAC std_ss [PAND_CSP_event_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC (`` {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
          measurable_sets (pair_measure lborel lborel)``)
	  THEN1(RW_TAC std_ss[event1_shared_spare_Axy_measurable_set])
THEN DISCH_TAC
THEN `!x y z. indicator_fn
            {((x',y'),z') |
           0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}
            ((x,y),z) =
          indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) * indicator_fn {z' | 0 <= z' /\ z' < y} z` by RW_TAC std_ss[event_PAND_CSP_indicator_of_indicator_Axy]
THEN KNOW_TAC(`` !x y. {z' | 0 <= z' /\ z' < y:real} IN measurable_sets lborel``)
     THEN1( `measurable_sets lborel = subsets borel` by METIS_TAC [borel , lborel, subsets_def, measurable_sets_def]
          THEN ONCE_ASM_REWRITE_TAC[]
	  THEN `!x y . {z' | 0 <= z' /\ z' < y} = {z' | 0 <= z'} INTER {z'| z' < y:real}` by SRW_TAC  [] [GSPECIFICATION, EXTENSION]
	  THEN `!x y. {z' | 0 <= z'} IN subsets borel  /\ {z' | z' < y} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT]
          THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC	  

THEN KNOW_TAC(``(\(x:real,y:real).  pos_fn_integral (distr p lborel Z) (\z. indicator_fn {((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((x,y),z:real))) IN
                                   measurable (m_space (pair_measure lborel lborel),
                                   measurable_sets (pair_measure lborel lborel)) Borel``)

     THEN1( MP_TAC(Q.ISPECL[` indicator_fn ({((x',y'),z') |
                     0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}) `, `pair_measure lborel lborel`, `distr p lborel (Z:'a->real)` ] borel_measurable_pos_integral_fst)
             THEN RW_TAC std_ss [measure_space_pair_measure', sigma_finite_measure_lborel]
	     THEN `measure_space (pair_measure lborel lborel)` by METIS_TAC [measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]
	     THEN FULL_SIMP_TAC std_ss[]
	     THEN `measure_space (distr p lborel (Z:'a->real))` by 
	               (FULL_SIMP_TAC std_ss [random_variable_def]
                          
		         THEN KNOW_TAC(``(Z:'a->real) IN measurable (m_space p, measurable_sets p)
			                (m_space lborel, measurable_sets lborel)``)
			      THEN1( UNDISCH_TAC(``(Z:'a->real) IN measurable (p_space p,events p) (m_space lborel, measurable_sets lborel)``)
		         	      THEN DISCH_TAC
			              THEN METIS_TAC [measure_space_distr, measure_space_lborel, prob_space_def, random_variable_def, p_space_def, events_def, measurable_sets_def, subsets_def, borel, lborel, m_space_def, space_def, space_borel])
			THEN  METIS_TAC [measure_space_distr, measure_space_lborel, prob_space_def, random_variable_def, p_space_def, events_def, measurable_sets_def, subsets_def, borel, lborel, m_space_def, space_def, space_borel])
	      THEN FULL_SIMP_TAC std_ss[]
              	      
	      THEN KNOW_TAC(`` sigma_finite_measure (distr p lborel (Z:'a->real)) ``)
		   THEN1(METIS_TAC[sigma_finite_measure_distr, random_variable_def])
              THEN DISCH_TAC
              THEN FULL_SIMP_TAC std_ss[]
              THEN KNOW_TAC(``(!(x:(real#real)#real). 0 <= indicator_fn {((x',y'),z') |
                    0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} x)``)  
                   THEN1(METIS_TAC [indicator_fn_pos_le])
              THEN DISCH_TAC
	      THEN FULL_SIMP_TAC std_ss[]
              THEN KNOW_TAC(``indicator_fn {((x',y'),z') |
                    0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} IN
                            measurable (m_space (pair_measure (pair_measure lborel lborel)(distr p lborel (Z:'a->real))),
                            measurable_sets (pair_measure (pair_measure lborel lborel)(distr p lborel Z))) Borel``)
                  THEN1(MP_TAC(Q.ISPECL[`(m_space (pair_measure (pair_measure lborel lborel) (distr p lborel (Z:'a->real))),
    measurable_sets(pair_measure (pair_measure lborel lborel) (distr p lborel Z))) ` ,`1:real` ,`{((x',y'),z') |
                    0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)
 
                        THEN RW_TAC std_ss [ REAL_LE_01]
	                THEN KNOW_TAC(``m_space (pair_measure (pair_measure lborel lborel)(distr p lborel Z)) =
			                  m_space (pair_measure (pair_measure lborel lborel) lborel)``)
	                       THEN1(RW_TAC std_ss [distr, m_space_def, pair_measure, measure_of])
	                THEN DISCH_TAC
	                THEN FULL_SIMP_TAC std_ss[]
	                THEN KNOW_TAC(``measurable_sets (pair_measure (pair_measure lborel lborel) (distr p lborel Z)) =
			                 measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
                             THEN1(REWRITE_TAC[m_space_def, distr, measurable_sets_def, pair_measure, measure_of, subsets_def, lborel])
	               THEN DISCH_TAC
	               THEN FULL_SIMP_TAC std_ss[]
	               THEN `sigma_algebra (m_space (pair_measure (pair_measure lborel lborel) lborel),
                             measurable_sets (pair_measure (pair_measure lborel lborel) lborel))`
			      by RW_TAC std_ss[sigma_algebra_pair_pair_lborel]
	               THEN FULL_SIMP_TAC std_ss[]
	  
                       THEN ` {((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} IN subsets (m_space (pair_measure (pair_measure lborel lborel) lborel),
                            measurable_sets (pair_measure (pair_measure lborel lborel) lborel))`
			    by METIS_TAC[pair_measure, lborel, measure_of, subsets_def, m_space_def, measurable_sets_def]
	               THEN FULL_SIMP_TAC std_ss[]
                       THEN KNOW_TAC(``!(x: (real#real)#real). x = ((FST (FST x),SND (FST x)), SND x) ``)
                           THEN1(METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])
                       THEN DISCH_TAC     
                       THEN KNOW_TAC(``(\ (x:(real#real)#real). Normal 1 * indicator_fn {((x',y'),z') |
                       0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} x) = (\x. indicator_fn {((x',y'),z') |
                      0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} x)``)
                            THEN1(METIS_TAC [extreal_of_num_def, mul_lone, PAIR, LAMBDA_PROD])
                       THEN DISCH_TAC
                       THEN MP_TAC(Q.ISPECL [`indicator_fn ({((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}:(real#real)#real->bool) `]LAMBDA_PROD)
		       THEN FULL_SIMP_TAC std_ss[]
                       THEN METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])
             THEN DISCH_TAC
             THEN FULL_SIMP_TAC std_ss[]
             THEN KNOW_TAC(``!(x:real#real). x IN m_space (pair_measure lborel lborel)``)
                  THEN1(REWRITE_TAC [m_space_def, pair_measure, measure_of, lborel]
                        THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CROSS_DEF]
                        THEN ONCE_ASM_REWRITE_TAC[]
	                THEN METIS_TAC [IN_UNIV])
             THEN DISCH_TAC
             THEN FULL_SIMP_TAC std_ss[]
             THEN KNOW_TAC(``!(x:real#real) ( y:real). max 0 (indicator_fn {((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} (x,y)) = indicator_fn {((x',y'),z') |
                     0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} (x,y) ``)
                  THEN1(RW_TAC std_ss [extreal_max_def])
             THEN DISCH_TAC     
             THEN FULL_SIMP_TAC std_ss[]
             THEN KNOW_TAC(``(!(x:real#real) (y:real). pos_fn_integral (distr p lborel Z) (\y:real. indicator_fn {((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} (x,y)) = 
                               pos_fn_integral (distr p lborel Z) (\y. indicator_fn {((x',y'),z') |
                    0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((FST x, SND x),y)))``)
                  THEN1(METIS_TAC [PAIR, ETA_THM, LAMBDA_PROD])
             THEN DISCH_TAC
             THEN UNDISCH_TAC (`` (\x. pos_fn_integral (distr p lborel (Z:'a->real))(\y. indicator_fn {((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} (x,y))) IN
                                   measurable (m_space (pair_measure lborel lborel),
                                   measurable_sets (pair_measure lborel lborel)) Borel``)
             THEN ONCE_ASM_REWRITE_TAC[]
             THEN FULL_SIMP_TAC std_ss[PAIR, LAMBDA_PROD, ETA_THM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!(x:real#real). 0 <= (\(x,y). pos_fn_integral (distr p lborel (Z:'a->real))
                                             (\z. indicator_fn {((x',y'),z') |
                    0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((x,y),z))) x``)

     THEN1(RW_TAC std_ss[]

           THEN KNOW_TAC(``(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                                         (\z. indicator_fn  {z' | 0 <= z' /\ z' < y} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) x =
                               ( pos_fn_integral (distr p lborel Z) (\z. indicator_fn  {z' | 0 <= z' /\ z' < SND x} z * indicator_fn   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x))``)
 
                THEN1(`x:real#real = (FST x, SND x)` by RW_TAC std_ss [PAIR]
                       THEN ONCE_ASM_REWRITE_TAC[]
                       THEN RW_TAC std_ss[PAIR, ETA_THM, LAMBDA_PROD])
           THEN DISCH_TAC
           THEN FULL_SIMP_TAC std_ss[]
           THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`, ` (\z:real. indicator_fn {z' | 0 <= z' /\ z' < SND x} z * indicator_fn  {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real))  `]  pos_fn_integral_pos)
           THEN RW_TAC std_ss[indicator_fn_pos_le]
           THEN `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC [measure_space_distr,random_variable_def, measure_space_lborel, prob_space_def, p_space_def, events_def, measurable_sets_def, subsets_def, space_def]
           THEN FULL_SIMP_TAC std_ss[]
           THEN KNOW_TAC(`` pos_fn_integral (distr p lborel Z) (\z:real. indicator_fn {z' |0 <= z' /\ z' < SND x} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) =
	                     pos_fn_integral (distr p lborel Z) (\z:real. indicator_fn {z' | 0 <= z' /\ z' < SND x} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST (x:real#real),SND x) )``)
               THEN1(METIS_TAC[PAIR])
	   THEN DISCH_TAC
           THEN

               KNOW_TAC(``pos_fn_integral (distr p lborel Z)
	                      (\z:real. indicator_fn {z' | 0 <= z' /\ z' < SND x} z* indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) =
                                    pos_fn_integral (distr p lborel Z)
                     (\z. indicator_fn {z' | 0 <= z' /\ z' < SND x} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x ,SND x) )``)
                THEN1(METIS_TAC [PAIR, LAMBDA_PROD] )
     
           THEN DISCH_TAC
	   THEN KNOW_TAC(``x:real#real = (FST x, SND x)``)
                THEN1(METIS_TAC [PAIR])
	   THEN DISCH_TAC
           THEN ONCE_ASM_REWRITE_TAC[]
	   THEN KNOW_TAC(``(!x':real. 0 <= indicator_fn {z' | 0 <= z' /\ z' < SND x} x' * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) ``)
	        THEN1 (METIS_TAC [le_mul, indicator_fn_pos_le])
           THEN DISCH_TAC
	   THEN FULL_SIMP_TAC std_ss[PAIR, mul_comm] )

THEN DISCH_TAC
THEN KNOW_TAC(``(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
       (\z.indicator_fn {z' |  0 <= z' /\ z' < y} z * indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) ))  =
                  (\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
       (\z. indicator_fn {((x',y'),z') |
               0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}((x,y),z))) ``)
     THEN1(RW_TAC std_ss[PAIR, LAMBDA_PROD, ETA_THM, pair_exists, mul_comm])
THEN DISCH_TAC
THEN KNOW_TAC (``(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
            (\z.indicator_fn {z' | 0 <= z' /\ z' < y} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) IN
                   measurable (m_space (pair_measure lborel lborel),
                   measurable_sets (pair_measure lborel lborel)) Borel``)

      THEN1(METIS_TAC[PAIR, LAMBDA_PROD, ETA_THM, pair_exists])
THEN DISCH_TAC
THEN KNOW_TAC(``(!x. 0 <=  (\ (x:real,y:real).pos_fn_integral (distr p lborel (Z:'a->real))
               (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) x) ``)
     THEN1(METIS_TAC[])
THEN DISCH_TAC     
THEN WEAKEN_TAC (equal(Term`(\ (x:real,y:real).  pos_fn_integral (distr p lborel (Z:'a->real))
            (\z. indicator_fn  {z' | 0 <= z' /\ z' < y} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) =
       (\(x,y). pos_fn_integral (distr p lborel Z) (\z. indicator_fn {((x',y'),z') |
               0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((x,y),z)))`))

THEN FULL_SIMP_TAC std_ss[]       
THEN KNOW_TAC(``pos_fn_integral (density (pair_measure lborel lborel) f_xy)
         (\x.(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z. indicator_fn {z' | 0 <= z' /\ z' < y}  z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) x) =
		    pos_fn_integral (density (pair_measure lborel lborel) f_xy)
                     (\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z.indicator_fn {z' | 0 <=z' /\ z' < y} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) ``)
     THEN1(METIS_TAC[ETA_ETA_THM_PAIR])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``pos_fn_integral (pair_measure lborel lborel)
         (\x. f_xy x * (\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z * indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x,y))) x) =
		    pos_fn_integral (pair_measure lborel lborel)
           (\ (x:real,y:real). f_xy (x,y) * pos_fn_integral (distr p lborel Z)
                 (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) ``)
      THEN1(RW_TAC std_ss[ETA_ETA_THM_PAIR, PAIR, ETA_THM, LAMBDA_PROD, mul_comm])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(pos_fn_integral (density (pair_measure lborel lborel) f_xy)
        (\ (x,y). pos_fn_integral (distr p lborel Z) (\z. indicator_fn {((x',y'),z') |
               0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((x,y),z))) =
	        pos_fn_integral (pair_measure lborel lborel) (\ (x,y).  f_xy (x,y) *
		    pos_fn_integral (distr p lborel Z) (\z.  indicator_fn {((x',y'),z') |
               0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((x,y),z) ) ))``)

      THEN1(FULL_SIMP_TAC std_ss [PAIR, ETA_THM, LAMBDA_PROD, mul_comm])

THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``  pos_fn_integral (density (pair_measure lborel lborel) f_xy)
     (\ (x,y).
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn
                   {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                 indicator_fn {z' | 0<= z' /\ z' < y} z))  =  pos_fn_integral (pair_measure lborel lborel)
           (\ (x,y).
                f_xy (x,y) *
                pos_fn_integral (distr p lborel Z)
                  (\z.
                        indicator_fn {z' | 0 <= z' /\ z' < y} z *
                            indicator_fn
                              {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)))  ``)

      THEN1(FULL_SIMP_TAC std_ss[mul_comm])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` (\ (x:real,y:real).
          f_xy (x,y) *
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | 0 <= z' /\ z' < y} z *
                 indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) =
		   (\ (x:real,y:real).
          f_xy (x,y) * indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | 0 <= z' /\ z' < y} z  ))``)
      THEN1(KNOW_TAC(``   !(x:real) (y:real).
          
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | 0 <= z' /\ z' < y} z *
                 indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)) =
		  
        indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | 0 <= z' /\ z' < y} z  ) ``)
            THEN1( RW_TAC std_ss[]
                  THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`, `(\ (x:real,y:real). real (indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)) )` ,`indicator_fn {z' | 0:real <= z' /\ z' < y} `] pos_fn_integral_fun_mul1)
                 THEN RW_TAC std_ss []
                 THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                 THEN `!x. x IN m_space (distr p lborel (Z:'a->real))` by METIS_TAC [distr,lborel, m_space_def, IN_UNIV]
                 THEN `(!x'. 0 <= indicator_fn {z' | 0:real <= z' /\ z' < y:real} x')` by RW_TAC std_ss [indicator_fn_pos_le]
		 THEN FULL_SIMP_TAC std_ss [PAIR, LAMBDA_PROD]
                 THEN FULL_SIMP_TAC std_ss[GSYM PAIR, GSYM LAMBDA_PROD]
                 THEN  KNOW_TAC(``(!p:real#real. 0 <= real (indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  p))``)  
			
		  THEN1(RW_TAC std_ss [real_def, indicator_fn_def]
           	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(SELECT_ELIM_TAC
                             THEN RW_TAC std_ss[]
                                   THEN1( Q.EXISTS_TAC `1`
		                        THEN RW_TAC std_ss[normal_real, extreal_of_num_def] )
		                        THEN  `x:real = 1` by METIS_TAC[extreal_11, extreal_of_num_def]
                         	         THEN METIS_TAC [REAL_LE_01])
                            THEN SELECT_ELIM_TAC		 
	                    THEN RW_TAC std_ss[]		    
                                THEN1(Q.EXISTS_TAC`0`
                                     THEN RW_TAC std_ss [normal_real, extreal_of_num_def])
	                    THEN  `x:real = 0` by METIS_TAC [extreal_11, extreal_of_num_def]
                            THEN METIS_TAC [REAL_LE_REFL])
		 THEN DISCH_TAC	    
                 THEN FULL_SIMP_TAC std_ss[]
		 THEN KNOW_TAC(``!(x':real#real). Normal (real (indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x':real#real))) = indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x'``)  
                    THEN1(RW_TAC std_ss[normal_real_indicator_fn])
    		 THEN  DISCH_TAC
    		 THEN FULL_SIMP_TAC std_ss[]
    		 THEN FULL_SIMP_TAC std_ss[mul_comm])
	 THEN DISCH_TAC
	 THEN ONCE_ASM_REWRITE_TAC[]
	 THEN RW_TAC std_ss [mul_comm, mul_assoc])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN  MP_TAC(Q.ISPECL[`(\ (x:real,y:real). f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) * pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {z' | 0 <= z' /\ z' < y}  z  ))`, ` lborel` ,`lborel`] pos_fn_integral_snd  )
THEN RW_TAC std_ss [measure_space_lborel, sigma_finite_measure_lborel]
THEN KNOW_TAC(``(!(x:real#real).
          0 <= (\ (x:real,y:real). f_xy (x,y) * indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) * 
             pos_fn_integral (distr p lborel (Z:'a->real))
               (\z. indicator_fn  {z' | 0 <= z' /\ z' < y} z )) x)``)
     THEN1(RW_TAC std_ss[]
           THEN `(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
           THEN ONCE_ASM_REWRITE_TAC[]
           THEN KNOW_TAC(``(\ (x:real,y:real). f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
	                pos_fn_integral (distr p lborel Z)
			(\z:real. indicator_fn  {z' | 0 <= z' /\ z' < y} z ))(FST x,SND x) =
            ( f_xy (x:real#real) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x, SND x) *
	    pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn  {z' | 0<= z' /\ z' <SND x} z))``)
                THEN1(RW_TAC std_ss [PAIR, ETA_THM, LAMBDA_PROD])
           THEN DISCH_TAC
           THEN ONCE_ASM_REWRITE_TAC[]
	   THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                
	   THEN  MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`,`(\z:real. indicator_fn  {z' | 0 <= z' /\ z' < SND( x:real#real)} z )  `]    pos_fn_integral_pos  )
           
	   THEN  RW_TAC std_ss [measure_space_distr, measure_space_lborel, prob_space_def, le_mul, indicator_fn_pos_le] )
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN WEAKEN_TAC (equal(Term`!x:real#real.
         0 <=
         (\(x,y).
            f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
            pos_fn_integral (distr p lborel Z) (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z))
           x`))

THEN KNOW_TAC(``(\ (x:real,y:real).
          f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *  pos_fn_integral (distr p lborel (Z:'a->real))
            (\z:real. indicator_fn  {z' | 0 <= z' /\ z' < y} z) ) IN
            measurable (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel``)

    THEN1( KNOW_TAC (``(\ (x,y).
        f_xy (x,y) *
        indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
          (x,y) *
        pos_fn_integral (distr p lborel Z)
          (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z)) IN
   measurable
     (m_space (pair_measure lborel lborel),
      measurable_sets (pair_measure lborel lborel)) Borel
    =  (\ (x,y).
        f_xy (x,y) *
         pos_fn_integral (distr p lborel Z)
          (\z.  indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
          (x,y)* indicator_fn {z' | 0<= z' /\ z' < y} z)) IN
   measurable
     (m_space (pair_measure lborel lborel),
      measurable_sets (pair_measure lborel lborel)) Borel``)
          THEN1(RW_TAC std_ss [mul_comm, mul_assoc])
	  THEN DISCH_TAC
	  THEN ASM_REWRITE_TAC[]
	  THEN MP_TAC(Q.ISPECL[`(m_space (pair_measure lborel lborel),
           measurable_sets (pair_measure lborel lborel))`
	   , `(f_xy:real#real->extreal) `,
	   ` (\ (x:real,y:real).  pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x,y)*  indicator_fn {z' | 0 <= z' /\ z' < y} z))`, `(\ (x:real,y:real). f_xy (x:real,y:real)  * pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x,y) * indicator_fn {z' | 0 <= z' /\ z' < y}  z))` ]  IN_MEASURABLE_BOREL_MUL)
	   
           THEN RW_TAC std_ss[measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel, sigma_algebra_pair_lborel]
            THEN KNOW_TAC(``!(x:real#real).
                x IN  space  (m_space (pair_measure lborel lborel),
                measurable_sets (pair_measure lborel lborel))``)
                 THEN1(REWRITE_TAC [space_def, pair_measure, measure_of, m_space_def, measurable_sets_def, lborel]
		       THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CROSS_DEF]
		       THEN ONCE_ASM_REWRITE_TAC[]
		       THEN METIS_TAC [IN_UNIV])
            THEN DISCH_TAC		       
            THEN FULL_SIMP_TAC std_ss[]
	    THEN KNOW_TAC(``!(x:real#real). f_xy x <> NegInf``)
                 THEN1(RW_TAC std_ss[]
		      THEN METIS_TAC [extreal_of_num_def,extreal_lt_def, extreal_le_def, le_lt, lt_le,lt_infty,le_infty])
            THEN DISCH_TAC
	    THEN FULL_SIMP_TAC std_ss[]
            THEN KNOW_TAC(``!x.
              (\ (x,y).
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                            0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | 0 <= z' /\ z' < y} z)) x <>
              NegInf``)
	      THEN1( RW_TAC std_ss[PAIR, LAMBDA_PROD]
	             THEN KNOW_TAC(`` (\ (x,y).
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
               indicator_fn {z' | 0 <= z' /\ z' < y} z)) x = 
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x, SND x) *
               indicator_fn {z' | 0 <= z' /\ z' < SND x} z) ``)
	       THEN1(`(x:real#real) = (FST x , SND x)` by RW_TAC std_ss[PAIR, LAMBDA_PROD]
	             THEN ONCE_ASM_REWRITE_TAC[]
		     THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	       THEN DISCH_TAC
	       THEN ONCE_ASM_REWRITE_TAC[]
	       THEN RW_TAC std_ss [PAIR]
	       THEN `
	       pos_fn_integral (distr p lborel Z)
     (\z.
          indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x *
          indicator_fn {z' | 0 <= z' /\ z' < SND x} z) =
	      pos_fn_integral (distr p lborel Z)
     (\z. indicator_fn
               {((x',y'),z') |
              0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}
               ((FST x, SND x),z))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	      THEN ONCE_ASM_REWRITE_TAC[]
	      THEN `!(x:real#real) z. 0<= indicator_fn
            {((x',y'),z') |0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}
            ((FST x,SND x),z)` by  METIS_TAC[indicator_fn_pos_le]
              THEN `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
               THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`,`(\z:real. indicator_fn
            {((x',y'),z') |
           0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}
            ((FST x,SND x),z))  `]  pos_fn_integral_pos)
           THEN DISCH_TAC
	   THEN FULL_SIMP_TAC std_ss[indicator_fn_pos_le]
           THEN  METIS_TAC [extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def, mul_extreal_not_infty])
     THEN DISCH_TAC
     THEN FULL_SIMP_TAC std_ss[]
     THEN KNOW_TAC (`` !x.
              (\ (x,y).
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                             0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | 0 <= z' /\ z' < y} z)) x <> PosInf``)

          THEN1(RW_TAC std_ss[]
	        THEN  KNOW_TAC(`` (\ (x,y).
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
               indicator_fn {z' | 0 <= z' /\ z' < y} z)) x = 
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x, SND x) *
               indicator_fn {z' | 0 <= z' /\ z' < SND x} z) ``)
	       THEN1(`(x:real#real) = (FST x , SND x)` by RW_TAC std_ss[PAIR, LAMBDA_PROD]
	             THEN ONCE_ASM_REWRITE_TAC[]
		     THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	       THEN DISCH_TAC
	       THEN ONCE_ASM_REWRITE_TAC[]
	       THEN RW_TAC std_ss [PAIR]
               THEN KNOW_TAC(``pos_fn_integral (distr p lborel Z)
     (\z.
          indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x *
          indicator_fn {z' | 0 <= z' /\ z' < SND x} z) =
	  indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x*
	  pos_fn_integral (distr p lborel Z)
         (\z. indicator_fn {z' | 0 <= z' /\ z' < SND x} z) ``)
                 THEN1(RW_TAC std_ss[]
                  THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`, `(\ (x':real,y:real). real (indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) )` ,`indicator_fn {z' | 0 <= z' /\ z' < SND (x:real#real)} `] pos_fn_integral_fun_mul1)
                 THEN RW_TAC std_ss []
                 THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                 THEN `!x. x IN m_space (distr p lborel (Z:'a->real))` by METIS_TAC [distr,lborel, m_space_def, IN_UNIV]
		 THEN FULL_SIMP_TAC std_ss[]
                 THEN `(!x'. 0 <= indicator_fn {z' | 0 <= z' /\ z' < SND(x:real#real)} x')` by RW_TAC std_ss [indicator_fn_pos_le]
		 THEN FULL_SIMP_TAC std_ss [PAIR, LAMBDA_PROD]
                 THEN FULL_SIMP_TAC std_ss[GSYM PAIR, GSYM LAMBDA_PROD]
                 THEN  KNOW_TAC(``(0 <= real (indicator_fn  {(x',y') |
                               0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x:real#real)))``)  
                       THEN1(RW_TAC std_ss [real_def, indicator_fn_def]
           	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(SELECT_ELIM_TAC
                             THEN RW_TAC std_ss[]
                                   THEN1( Q.EXISTS_TAC `1`
		                        THEN RW_TAC std_ss[normal_real, extreal_of_num_def] )
		                        THEN  `x':real = 1` by METIS_TAC[extreal_11, extreal_of_num_def]
                         	         THEN METIS_TAC [REAL_LE_01])
                            THEN SELECT_ELIM_TAC		 
	                    THEN RW_TAC std_ss[]		    
                                THEN1(Q.EXISTS_TAC`0`
                                     THEN RW_TAC std_ss [normal_real, extreal_of_num_def])
	                    THEN  `x':real = 0` by METIS_TAC [extreal_11, extreal_of_num_def]
                            THEN METIS_TAC [REAL_LE_REFL])
		 THEN DISCH_TAC	    
                 THEN FULL_SIMP_TAC std_ss[]
		 THEN KNOW_TAC(``!(x:real#real). Normal (real (indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real))) = indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x``)  
          		      THEN1(RW_TAC std_ss[normal_real_indicator_fn])
           THEN DISCH_TAC
    	   THEN FULL_SIMP_TAC std_ss[])
     THEN DISCH_TAC
     THEN ONCE_ASM_REWRITE_TAC[]
      THEN `!x. indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real) <> PosInf /\
		            indicator_fn  {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real) <> NegInf` by 
			        (RW_TAC std_ss [indicator_fn_def]
			        THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def])

          THEN KNOW_TAC(``  0<=  pos_fn_integral (distr p lborel (Z:'a->real))
          (\z. indicator_fn {z' | 0 <= z' /\ z' < SND( x:real#real)} z) ``)
              THEN1(RW_TAC std_ss[]
         

           THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`,`(\z:real. indicator_fn {z' | 0 <= z' /\ z' < SND (x:real#real)}  z )  `]  pos_fn_integral_pos)
           THEN DISCH_TAC
	   THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                
           THEN FULL_SIMP_TAC std_ss [indicator_fn_pos_le])
	   THEN DISCH_TAC
	   THEN  ` pos_fn_integral (distr p lborel (Z:'a->real)) (\z:real. indicator_fn {z' | 0 <= z' /\ z' < SND( x:real#real) } z) <> NegInf`
                      by  METIS_TAC [extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def, mul_extreal_not_infty]
	   THEN KNOW_TAC(`` pos_fn_integral (distr p lborel (Z:'a->real)) (\z:real. indicator_fn {z' | 0<= z' /\ z' < SND( x:real#real)} z) <> PosInf``)THEN1 
		                  (RW_TAC std_ss[]
				   THEN KNOW_TAC(``
		                  pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {z' | 0<= z' /\ z' < SND(x:real#real)} z) = measure (distr p lborel Z) {z' | 0 <= z' /\ z' < SND x}``)
                                THEN1(RW_TAC std_ss [measure_def, distr, GSYM distribution_def]
               	                      THEN RW_TAC std_ss [GSYM prob_def, GSYM p_space_def]
		      		      THEN `prob p (PREIMAGE (Z:'a->real) {z' | 0 <= z' /\ z' < SND( x:real#real)} INTER p_space p) =
			           distribution p Z {z' | 0 <= z' /\ z' < SND x}`by METIS_TAC[GSYM distribution_def]
                      		   THEN ONCE_ASM_REWRITE_TAC[]
                      		   THEN `distribution p (Z:'a->real) {z' | 0 <= z' /\ z' < SND (x:real#real)} =
			           pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {z' | 0 <= z' /\ z' < SND (x:real#real)} z)` by METIS_TAC [distribution_lebesgue_thm2_pos_fn, measure_space_lborel]
		                   THEN ONCE_ASM_REWRITE_TAC[]
                      		   THEN RW_TAC std_ss [GSYM distribution_def]  
                      		   THEN `(p_space lborel,measurable_sets lborel,distribution p (Z:'a->real)) = distr p lborel Z` by 
                                         (RW_TAC std_ss[distr, p_space_def, distribution_def, prob_def])
		                   THEN ONCE_ASM_REWRITE_TAC[]
		      		   THEN RW_TAC std_ss[]  )
	                  THEN DISCH_TAC  
        		  THEN KNOW_TAC(``measure (distr p lborel (Z:'a->real)) {z' |0<= z' /\ z' < SND( x:real#real)} <> PosInf``)
	     		       THEN1(`sigma_finite_measure (distr p lborel (Z:'a->real))` by RW_TAC std_ss [sigma_finite_measure_distr]
	                             THEN KNOW_TAC(``{z' | 0  <= z' /\ z' < SND( x:real#real)} IN measurable_sets (distr p lborel (Z:'a->real))``)
	   			          THEN1(RW_TAC std_ss[distr, measurable_sets_def])
		                    THEN DISCH_TAC
		    		    THEN WEAKEN_TAC (equal(Term` pos_fn_integral (distr p lborel Z) (\z. indicator_fn {z' |0 <= z' /\ z' < SND( x:real#real)} z) =  measure (distr p lborel (Z:'a->real)) {z' | 0 <= z' /\ z' < SND x} `))
                                   THEN RW_TAC std_ss [distr, measure_def]
		                   THEN RW_TAC std_ss [GSYM prob_def, GSYM p_space_def]
		                   THEN KNOW_TAC(``PREIMAGE (Z:'a->real) {z' | 0 <= z' /\ z' <SND( x:real#real)} INTER p_space p IN events p``)
                                        THEN1(FULL_SIMP_TAC std_ss [random_variable_def]
                             	              THEN METIS_TAC [IN_MEASURABLE, events_def, measurable_sets_def, random_variable_def, p_space_def, m_space_def, space_def, subsets_def, prob_space_def, measure_space_def])
                                  THEN DISCH_TAC
		                  THEN METIS_TAC [PROB_FINITE])
                         THEN DISCH_TAC
                         THEN FULL_SIMP_TAC std_ss[])
	  THEN DISCH_TAC		 
	  THEN FULL_SIMP_TAC std_ss[]
	  THEN METIS_TAC [mul_extreal_not_infty])
          THEN DISCH_TAC
          THEN FULL_SIMP_TAC std_ss[]
          THEN KNOW_TAC(``    (\(x,y).
              pos_fn_integral (distr p lborel Z)
                (\z.
                     indicator_fn
                       {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
                       (x,y) * indicator_fn {z' | 0 <= z' /\ z' < y} z)) IN
         measurable
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel``)

               THEN1( KNOW_TAC (`` ! x y.
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
               indicator_fn {z' | 0<= z' /\ z' < y} z) =  
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
             {((x',y'),z') |
                         0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ((x,y),z))``)
                     THEN1( RW_TAC std_ss[])
		THEN DISCH_TAC
		THEN ONCE_ASM_REWRITE_TAC[]
 		THEN MP_TAC(Q.ISPECL[`(\ ((x,y),z). indicator_fn
                 {((x',y'),z') |
                 0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} ) ((x,y),z) `,`pair_measure lborel lborel`, `lborel` ] borel_measurable_pos_integral_fst)
		 THEN RW_TAC std_ss[sigma_finite_measure_lborel, measure_space_lborel, measure_space_pair_measure' ])
		 THEN DISCH_TAC
                 THEN FULL_SIMP_TAC std_ss[]
		 THEN KNOW_TAC(``!x.
              (\ (x,y).
                   f_xy (x,y) *
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                            0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | 0 <=z' /\ z' < y} z)) x =
              f_xy x *
              (\ (x,y).
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                       0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | 0 <= z' /\ z' < y} z)) x ``)
                       THEN1(RW_TAC std_ss[]
		            THEN `(x:real#real) = (FST x , SND x)` by RW_TAC std_ss[PAIR]
                            THEN ONCE_ASM_REWRITE_TAC[]
			    THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
		THEN DISCH_TAC	    
                THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]);





val joint_density_3rv_2_PAND_CSP = store_thm("joint_density_3rv_2_PAND_CSP",

``!p X Y Z f_xy t.
     prob_space p /\ random_variable Z p borel /\
     (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
     distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy 
     ==>
     (pos_fn_integral
        (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
        (\ (x,y).
           pos_fn_integral (distr p lborel Z)
             (\z. indicator_fn   {((x',y'),z') |
            0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}((x,y),z))) =
      pos_fn_integral lborel
        (\y. indicator_fn {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t} x *
                pos_fn_integral (distr p lborel Z)
                  (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z))))``,

REPEAT GEN_TAC

THEN  KNOW_TAC(``{((x',y'),z') |
                   0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} IN measurable_sets (pair_measure (pair_measure lborel lborel)lborel) ``)
    THEN1(RW_TAC std_ss [PAND_CSP_event_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC (`` {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
          measurable_sets (pair_measure lborel lborel)``)
	  THEN1(RW_TAC std_ss[event1_shared_spare_Axy_measurable_set])
THEN DISCH_TAC
THEN `!x y z. indicator_fn
            {((x',y'),z') |
            0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}
            ((x,y),z) =
          indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) * indicator_fn {z' | 0 <= z' /\ z' < y} z` by RW_TAC std_ss[event_PAND_CSP_indicator_of_indicator_Axy]
THEN KNOW_TAC(`` !x y. {z' | 0 <= z' /\ z' < y:real} IN measurable_sets lborel``)
     THEN1( `measurable_sets lborel = subsets borel` by METIS_TAC [borel , lborel, subsets_def, measurable_sets_def]
          THEN ONCE_ASM_REWRITE_TAC[]
	  THEN `!x y . {z' |0 <= z' /\ z' < y} = {z' | 0 <= z'} INTER {z'| z' < y:real}` by SRW_TAC  [] [GSPECIFICATION, EXTENSION]
	  THEN `!x y. {z' | 0 <= z'} IN subsets borel  /\ {z' | z' < y} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
          THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC	  
THEN `!x y.   indicator_fn {(x',y') |   0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) =
          indicator_fn {x' | x' | y < x' /\ x' <= t} x *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y ` by RW_TAC std_ss[event1_shared_spare_indicator_of_indicator_Axy_Ax_Ay]
THEN KNOW_TAC(``!y.  {x' | y < x' /\ x' <= t} IN measurable_sets lborel ``)
     THEN1(RW_TAC std_ss[]
           THEN `measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN `{x' | y < x' /\ x' <= t} =
	         {x' | y <  x'} INTER
		 {x' | x' <= t } ` by (SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
          THEN ONCE_ASM_REWRITE_TAC[]
          THEN `{x' | y < x'} IN subsets borel /\
		 {x' | x' <= t } IN subsets borel  ` by METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC
THEN KNOW_TAC (``{y' | 0 <= y' /\ y' <= t} IN measurable_sets lborel``)
     THEN1(`measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
             THEN `{y' | 0 <= y' /\ y' <= t} = {y' | 0<= y'} INTER {y' | y' <= t}` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
	     THEN `{y' | 0<= y'} IN subsets borel /\ {y' | y' <= t} IN subsets borel` by
	     METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])

THEN DISCH_TAC	   

THEN MP_TAC(Q.ISPECL[`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,`X:'a->real`, `Y:'a->real`, `Z:'a->real`, `f_xy:(real#real)->extreal`, `t:real` ] joint_density_3rv_1_PAND_CSP)
THEN RW_TAC std_ss[] THEN FULL_SIMP_TAC std_ss[]

THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
			       THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                     THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
                               THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]


THEN  MP_TAC(Q.ISPECL[`lborel`,`(\y. real(indicator_fn {y' | 0 <= y' /\ y' <= t:real} y))`, `(\ (x,y).
          f_xy (x:real,y:real) * indicator_fn {x' | y < x' /\ x' <= t:real} x  *
          pos_fn_integral (distr p lborel (Z:'a->real)) (\z:real. indicator_fn {z' | 0  <= z' /\ z' < y:real} z))` ] pos_fn_integral_fun_mul2)

THEN RW_TAC std_ss[ measure_space_lborel]
THEN `!(x:real) (y:real).
          x IN m_space lborel` by METIS_TAC [lborel, m_space_def, IN_UNIV]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!(x:real) (y:real).
          0 <=
          f_xy (x,y) *
              indicator_fn {x' |y < x' /\ x' <= t} x *
              pos_fn_integral (distr p lborel Z)
                (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z)) ``)
     THEN1(RW_TAC std_ss[]
           THEN `0<= indicator_fn {x' |  y < x' /\ x' <= t:real} x `by RW_TAC std_ss [indicator_fn_pos_le]
           THEN `measure_space (distr p lborel (Z:'a->real))` by  METIS_TAC [measure_space_distr, prob_space_def, p_space_def, events_def, random_variable_def, measure_space_lborel]
	   
          THEN `0<= pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn  {z' | 0 <= z' /\ z' < y:real}z)`  by  METIS_TAC [pos_fn_integral_pos, measure_space_lborel, indicator_fn_pos_le]

        THEN RW_TAC std_ss[le_mul])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!y:real. 0 <= real (indicator_fn {y' | 0 <= y' /\ y' <= t:real} y))``)
     THEN1( `!y. indicator_fn {y' | real 0 <= y' /\ y' <= t:real} (y:real) <> PosInf /\
		            indicator_fn {y' | real 0 <= y' /\ y' <= t:real} y <> NegInf` by 
			        (RW_TAC std_ss [indicator_fn_def]
			        THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def])

           THEN `0 = real 0` by 
                       (RW_TAC std_ss[real_def]
		       THEN SELECT_ELIM_TAC
                       THEN RW_TAC std_ss[]  		      
                            THEN1(Q.EXISTS_TAC `0`
			          THEN METIS_TAC [extreal_of_num_def])
		       THEN METIS_TAC [extreal_of_num_def, extreal_11])
	   THEN ASM_REWRITE_TAC[]
	   THEN `0<> PosInf /\ 0 <> NegInf` by METIS_TAC [extreal_of_num_def, lt_infty]
	   THEN RW_TAC std_ss[extreal_real_le]
	   THEN RW_TAC std_ss[indicator_fn_pos_le])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN `!y:real. indicator_fn {y' |0 <= y' /\ y' <= t:real} y <> PosInf /\ indicator_fn {y' |0 <= y' /\ y' <= t:real} y<> NegInf` by METIS_TAC [indicator_fn_not_eq_infty]
THEN `!y:real. Normal (real (indicator_fn {y' | 0 <= y' /\ y' <= t} y)) = indicator_fn {y' | 0 <= y' /\ y' <= t} y` by RW_TAC std_ss [normal_real_indicator_fn]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!y:real.
         
           (\x:real.
              indicator_fn {y' | 0 <= y' /\ y' <= t:real} y *
              (f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t} x *
               pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z:real. indicator_fn {z' | 0 <= z' /\ z' < y} z))) =
		 
           (\x:real.
               
              f_xy (x,y) * (indicator_fn {x' | y < x' /\ x' <= t} x * indicator_fn {y' |0 <= y' /\ y' <= t:real} y)*
               pos_fn_integral (distr p lborel Z)
                 (\z:real. indicator_fn {z' | 0 <= z' /\ z' < y} z))``)
      THEN1(RW_TAC std_ss[]
            THEN  ABS_TAC
	    THEN METIS_TAC[mul_comm, mul_assoc])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]);




val joint_density_3rv_3_PAND_CSP = store_thm("joint_density_3rv_3_PAND_CSP",

``!p X Y Z  f_xy  t.
     prob_space p /\ random_variable Z p borel /\
     (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
     (!z. 0<= Z z ) /\
     distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy /\
      (!z. (\x. real (CDF p Z x)) contl z)==>
     (pos_fn_integral
        (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
        (\ (x,y).
           pos_fn_integral (distr p lborel Z)
             (\z. indicator_fn {((x',y'),z') |
                  0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}
((x,y),z))) =
      pos_fn_integral lborel
        (\y. indicator_fn  {y' | 0 <= y' /\ y' <= t} y*
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {x' |  y < x' /\ x' <= t} x *
                CDF p  Z  y )))``,

REPEAT GEN_TAC
THEN KNOW_TAC(``{((x',y'),z') |
                0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real} IN measurable_sets (pair_measure (pair_measure lborel lborel)lborel) ``)
    THEN1(RW_TAC std_ss [PAND_CSP_event_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC (`` {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
          measurable_sets (pair_measure lborel lborel)``)
	  THEN1(RW_TAC std_ss[event1_shared_spare_Axy_measurable_set])
THEN DISCH_TAC
THEN `!x y z. indicator_fn
            {((x',y'),z') |
           0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}
            ((x,y),z) =
          indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) * indicator_fn {z' | 0 <= z' /\ z' < y} z` by RW_TAC std_ss[event_PAND_CSP_indicator_of_indicator_Axy]
THEN KNOW_TAC(`` !x y. {z' | 0 <= z' /\ z' < y:real} IN measurable_sets lborel``)
     THEN1( `measurable_sets lborel = subsets borel` by METIS_TAC [borel , lborel, subsets_def, measurable_sets_def]
          THEN ONCE_ASM_REWRITE_TAC[]
	  THEN `!x y . {z' | 0 <= z' /\ z' < y} = {z' | 0 <= z'} INTER {z'| z' < y:real}` by SRW_TAC  [] [GSPECIFICATION, EXTENSION]
	  THEN `!x y. {z' | 0 <= z'} IN subsets borel  /\ {z' | z' < y} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
          THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC

THEN `!x y.   indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) =
          indicator_fn {x' | y < x' /\ x' <= t} x *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y ` by RW_TAC std_ss[event1_shared_spare_indicator_of_indicator_Axy_Ax_Ay]
THEN KNOW_TAC(``!y.  {x' |  y < x' /\ x' <= t} IN measurable_sets lborel ``)
     THEN1(RW_TAC std_ss[]
           THEN `measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN `{x' | y < x' /\ x' <= t} =
	         {x' | y < x'} INTER
		 {x' | x' <= t }` by (SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
          THEN ONCE_ASM_REWRITE_TAC[]
          THEN `{x' | y < x'} IN subsets borel /\
		 {x' | x' <= t } IN subsets borel  ` by METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC
THEN KNOW_TAC (``{y' |0 <= y' /\ y' <= t} IN measurable_sets lborel``)
     THEN1(`measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
 	   THEN ` {y' | 0 <= y' /\ y' <= t} = {y' | 0<= y'} INTER {y' | y' <= t}` by SRW_TAC [][GSPECIFICATION, EXTENSION]
	   THEN `{y' | 0<= y'} IN subsets borel /\ {y' | y' <= t} IN subsets borel`  by METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])

THEN DISCH_TAC	   


THEN DISCH_TAC
THEN `pos_fn_integral (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
   (\( x:real,y:real).
      pos_fn_integral (distr p lborel (Z:'a->real))
        (\z:real. indicator_fn  {((x',y'),z') |
        0 <= z' /\ z' < y' /\  0<=y' /\ y' <=t /\ y' < x' /\ x' <= t:real}  ((x,y),z))) =
	pos_fn_integral lborel
        (\y:real.
           indicator_fn  {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x:real.
                f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t} x *
                pos_fn_integral (distr p lborel (Z:'a->real))
                  (\z:real. indicator_fn {z' | 0 <= z' /\ z' < y} z)))`by METIS_TAC[joint_density_3rv_2_PAND_CSP]
THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[CDF_def]
THEN KNOW_TAC(``!y.  pos_fn_integral (distr p lborel Z)
                   (\z. indicator_fn {z' | 0 <= z' /\ z' < y} z) = distribution p Z {z'| 0<=z' /\ z' <y} ``)
     THEN1 (`borel = (m_space lborel, measurable_sets lborel)` by METIS_TAC [PAIR, lborel, borel, m_space_def, measurable_sets_def, subsets_def, space_def, space_borel]
             THEN `measure_space p` by FULL_SIMP_TAC std_ss[prob_space_def]
	     THEN `random_variable Z p (m_space lborel, measurable_sets lborel)` by FULL_SIMP_TAC std_ss[]
	    THEN `measure_space lborel` by RW_TAC std_ss [measure_space_lborel]  
    	    THEN METIS_TAC[GSYM distribution_lebesgue_thm2_pos_fn, prob_space_def, p_space_def, m_space_def, distributed_def])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` !y. CDF p Z y = distribution p Z {z' | z'< y}``)
     THEN1(RW_TAC std_ss[]
           THEN DEP_REWRITE_TAC[CDF_def2]
	   THEN `!n. {y' | y - 1 / &SUC n < y'} IN subsets borel` by METIS_TAC[borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
 	   THEN ` {y' | y' <= y} IN subsets borel` by METIS_TAC[borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT] 
	   THEN RW_TAC std_ss[]
	        THEN1(`{y' | y - 1 / &SUC n < y' /\ y' <= y:real} = {y' | y - 1 / &SUC n < y'} INTER
		                                                   {y' | y' <= y}` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
                       THEN `{y' | y - 1 / &SUC n < y' /\ y' <= y:real} IN subsets borel` by METIS_TAC [sigma_algebra_borel, sigma_algebra_def, ALGEBRA_INTER]
                      THEN METIS_TAC [IN_MEASURABLE, random_variable_def, subsets_def, space_def, m_space_def, measurable_sets_def, events_def, p_space_def])
		THEN1(METIS_TAC [IN_MEASURABLE, random_variable_def, subsets_def, space_def, m_space_def, measurable_sets_def, events_def, p_space_def])
		THEN1(`{y'|y'<y} IN subsets borel` by METIS_TAC[borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
		      THEN METIS_TAC [IN_MEASURABLE, random_variable_def, subsets_def, space_def, m_space_def, measurable_sets_def, events_def, p_space_def])
		THEN1(METIS_TAC [IN_MEASURABLE, random_variable_def, subsets_def, space_def, m_space_def, measurable_sets_def, events_def, p_space_def, borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT])
		THEN1( METIS_TAC [IN_MEASURABLE, random_variable_def, subsets_def, space_def, m_space_def, measurable_sets_def, events_def, p_space_def, borel_closed,CLOSED_STANDARD_HYPERPLANE])

                THEN1(RW_TAC std_ss[CDF_def])
		THEN RW_TAC std_ss[distribution_def])
THEN DISCH_TAC 		
THEN `!y. distribution p Z {y' | y' <= y} = CDF p Z y` by RW_TAC std_ss [GSYM CDF_def]
THEN ONCE_ASM_REWRITE_TAC[]
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!y. distribution p Z {z' | 0 <= z' /\ z' < y:real} =
                    distribution p Z {z' | z' < y}``)
     THEN1(RW_TAC std_ss[distribution_def]
           THEN KNOW_TAC(``PREIMAGE Z {z' | 0 <= z' /\ z' < y:real}=
	                   PREIMAGE Z {z' | z' < y}``)
                THEN1(RW_TAC std_ss[PREIMAGE_def]
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN RW_TAC std_ss[])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[]);


val cond_density_3rv_PAND_CSP = store_thm("cond_density_3rv_PAND_CSP",
  `` !p X Y Z f_xy  f_y f_cond t.
     prob_space p /\ den_gt0_ninfinity f_xy f_y f_cond  /\
     (!y. cond_density lborel lborel p X Y  y f_xy  f_y f_cond ) /\
     random_variable Z p borel ==>
     (pos_fn_integral lborel
        (\y.
           indicator_fn {y' |0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t}  x *
               CDF p Z y)) =
      pos_fn_integral lborel
        (\y.
           f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
                CDF p Z y)))``,

RW_TAC std_ss[den_gt0_ninfinity_def]
THEN `!y:real. f_y y <> 0 /\  f_y y <> NegInf /\ f_y y <> PosInf ` by METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty, lt_le, le_lt]
THEN `!x y. f_xy (x:real,y:real) = f_cond y x * f_y y` by METIS_TAC [cond_density_joint_marginal]
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` !y.
        pos_fn_integral lborel
          (\x.
               f_y y *
               (\x.
                    indicator_fn {x' |y < x' /\ x' <= t} x *
                   CDF p Z y) x *
               f_cond y x) =
        f_y y *
        pos_fn_integral lborel
          (\x.
               (\x.
                    indicator_fn {x' |y < x' /\ x' <= t} x *
                    CDF p Z y) x *
               f_cond y x)``)

     THEN1(RW_TAC std_ss[]

          THEN MP_TAC(Q.ISPECL[`lborel`, `f_cond:real->real->extreal`, `f_y : real->extreal`, ` (\x:real.
            indicator_fn {x' |y < x' /\ x' <= t} x *
          CDF p Z y)` ] lemma5_fun_mul_fun)
	  THEN RW_TAC std_ss [measure_space_lborel]
	  THEN `!x:real. x IN m_space lborel ` by RW_TAC std_ss [lborel, m_space_def, IN_UNIV]
	  THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(``(!x:real.
               0 <=
               indicator_fn  {x' | y < x' /\ x' <= t} x *
               CDF p Z y)``) THEN1 
               (RW_TAC std_ss[]
	       THEN `0<= indicator_fn   {x' |y < x' /\ x' <= t}(x:real)` by RW_TAC std_ss [indicator_fn_pos_le]
	       
	       THEN KNOW_TAC(``0<= CDF p Z y``)
	            THEN1(`borel = (m_space lborel, measurable_sets lborel)` by METIS_TAC[borel, lborel, m_space_def, space_def, subsets_def, measurable_sets_def, PAIR, space_borel]
		    THEN `!t. {y|y<=t} IN subsets borel` by METIS_TAC[borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT]
		    THEN `subsets borel = measurable_sets lborel` by METIS_TAC [subsets_def, measurable_sets_def]
		    THEN METIS_TAC[CDF_pos])
               THEN METIS_TAC[le_mul])
	 THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	 THEN `!y:real. 0 < f_y y`  by RW_TAC std_ss[]
	 THEN `!y:real. 0<= f_y y /\ 0<> f_y y` by METIS_TAC [lt_le]
	 THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC	 
THEN ASM_REWRITE_TAC[]
THEN `!(y:real) (x:real).  f_y y *
             (indicator_fn {x' | y < x' /\ x' <= t} x *
              CDF p Z y) *
             f_cond y x =  f_cond y x * f_y y *
             indicator_fn {x' |y < x' /\ x' <= t} x *
              CDF p Z y` by METIS_TAC[mul_assoc, mul_comm]
THEN FULL_SIMP_TAC std_ss[]
THEN `!y:real. indicator_fn {y' | 0<= y' /\ y' <= t}  y *
     (f_y y *
      pos_fn_integral lborel
        (\x:real.
           indicator_fn {x' | y < x' /\ x' <= t} x *
           CDF p Z y *
           f_cond y x)) = indicator_fn {y' | 0<= y' /\  y' <= t}  y *
     f_y y *
      pos_fn_integral lborel
        (\x.
           indicator_fn {x' | y < x' /\ x' <= t} x *
          CDF p Z y *
           f_cond y x)  ` by METIS_TAC [mul_assoc, mul_comm]
THEN ASM_REWRITE_TAC[]
THEN `!(y:real) (x:real). indicator_fn {x' | y < x' /\ x' <= t} x *
          CDF p Z y *
          f_cond y x =
	  f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
          CDF p Z y` by METIS_TAC [mul_assoc, mul_comm]
THEN ASM_REWRITE_TAC[]
THEN `!y:real.  indicator_fn  {y' | 0<= y' /\ y' <= t} y * f_y y *
     pos_fn_integral lborel
       (\x:real.
          f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
          CDF p Z y) =
	  f_y y * indicator_fn  {y' | 0<= y' /\ y' <= t} y * 
     pos_fn_integral lborel
       (\x.
          f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
          CDF p Z y)` by METIS_TAC [mul_comm]
THEN ASM_REWRITE_TAC[]);



val cond_density_3rv_PAND_CSP_1 = store_thm("cond_density_3rv_PAND_CSP_1",
  `` !p X Y Z f_xy  f_y f_cond t.
     prob_space p /\  random_variable Z p borel /\ (! x:real y:real . 0 <= f_cond y x)   ==>
     (pos_fn_integral lborel
        (\y.
           f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
                CDF p Z y)) =
	pos_fn_integral lborel
        (\y.
           f_y y * CDF p Z y *  indicator_fn {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x))	)``,

RW_TAC std_ss[]
THEN MP_TAC(Q.ISPECL[`lborel`,`(\y:real. real (CDF p Z y))`,
         `(\ (x:real,y). f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x) ` ] pos_fn_integral_fun_mul2)

THEN RW_TAC std_ss[measure_space_lborel]
THEN KNOW_TAC(``!y. 0 <= real (CDF p Z y) ``)

     THEN1(KNOW_TAC(``!y. 0<= CDF p Z y``)
	            THEN1(`borel = (m_space lborel, measurable_sets lborel)` by METIS_TAC[borel, lborel, m_space_def, space_def, subsets_def, measurable_sets_def, PAIR, space_borel]
		    THEN `!t. {y|y<=t} IN subsets borel` by METIS_TAC[borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT]
		    THEN `subsets borel = measurable_sets lborel` by METIS_TAC [subsets_def, measurable_sets_def]
		    THEN METIS_TAC[CDF_pos])
          THEN DISCH_TAC
	  THEN KNOW_TAC(``!y. CDF p Z y <> NegInf ``)
	       THEN1(METIS_TAC[extreal_of_num_def, lt_infty, lt_le, le_lt])
	  THEN DISCH_TAC
	  THEN KNOW_TAC(``!y. CDF p Z y <> PosInf  ``)
	       THEN1(RW_TAC std_ss[CDF_def, distribution_def]
	             THEN KNOW_TAC(``  (PREIMAGE Z {y' | y' <= y:real} INTER p_space p) IN events p``)
		          THEN1(`borel = (m_space lborel, measurable_sets lborel)` by METIS_TAC[borel, lborel, m_space_def, space_def, subsets_def, measurable_sets_def, PAIR, space_borel]
		                THEN `!t. {y|y<=t} IN subsets borel` by METIS_TAC[borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT]
		    		THEN `subsets borel = measurable_sets lborel` by METIS_TAC [subsets_def, measurable_sets_def]
		    		THEN METIS_TAC[IN_MEASURABLE, m_space_def, p_space_def, space_def, events_def, subsets_def, measurable_sets_def, random_variable_def])
		    THEN DISCH_TAC
		    THEN RW_TAC std_ss[ PROB_FINITE])
	  THEN RW_TAC std_ss[real_def]
	  THEN SELECT_ELIM_TAC
	  THEN RW_TAC std_ss[]
	       THEN1(Q.EXISTS_TAC `real (CDF p Z y)`
	             THEN METIS_TAC [normal_real])
          THEN `0<= CDF p Z y` by RW_TAC std_ss[] 
          THEN METIS_TAC[extreal_le_eq, extreal_of_num_def])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]	  
THEN `!x:real y:real. x IN m_space lborel` by RW_TAC std_ss[m_space_def, lborel, IN_UNIV]
THEN FULL_SIMP_TAC std_ss[]
THEN `!x y. 0 <= f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x` by RW_TAC std_ss[indicator_fn_pos_le, le_mul]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!y. Normal (real (CDF p Z y)) = CDF p Z y ``)
     THEN1( KNOW_TAC(``!y. CDF p Z y <> PosInf /\ CDF p Z y <> NegInf  ``)
	       THEN1( KNOW_TAC(`` !y. (PREIMAGE Z {y' | y' <= y:real} INTER p_space p) IN events p``)
		          THEN1(`borel = (m_space lborel, measurable_sets lborel)` by METIS_TAC[borel, lborel, m_space_def, space_def, subsets_def, measurable_sets_def, PAIR, space_borel]
		                THEN `!t. {y|y<=t} IN subsets borel` by METIS_TAC[borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT]
		    		THEN `subsets borel = measurable_sets lborel` by METIS_TAC [subsets_def, measurable_sets_def]
		    		THEN METIS_TAC[IN_MEASURABLE, m_space_def, p_space_def, space_def, events_def, subsets_def, measurable_sets_def, random_variable_def])
		    THEN DISCH_TAC
		    THEN METIS_TAC[CDF_def, distribution_def, PROB_FINITE])
	  THEN DISCH_TAC THEN METIS_TAC [normal_real])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN `!x y.  CDF p Z y *
                    (f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x) =
		    f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
                 CDF p Z y` by METIS_TAC [mul_comm, mul_assoc]
THEN FULL_SIMP_TAC std_ss[]	 
THEN ONCE_ASM_REWRITE_TAC[]
THEN `!y.  f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          (CDF p Z y *
           pos_fn_integral lborel
             (\x. f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x)) =
	   f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          CDF p Z y *
           pos_fn_integral lborel
             (\x. f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x)` by METIS_TAC[mul_assoc]
THEN RW_TAC std_ss[]
THEN `!y.  f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y * CDF p Z y =
           f_y y * CDF p Z y * indicator_fn {y' | 0 <= y' /\ y' <= t} y` by METIS_TAC [mul_comm, mul_assoc]
THEN RW_TAC std_ss[]);




(************ prob of event Aa.(A1 before A2)  .(A2 before Aa) (PAND_CSP)***************)


val DFT_event_PAND_CSP_INTER = store_thm("DFT_event_PAND_CSP_INTER",
  ``!X Y Z p t.
          rv_gt0_ninfinity [X; Y; Z] /\ (!s. ALL_DISTINCT [X s; Y s; Z s]) ==>
           (DFT_event p (D_AND (D_AND X (D_BEFORE Z Y)) (D_BEFORE Y X)) t =
	   PREIMAGE (\x. ((X x, Y x), Z x)) {((x,y),z) |
            0 <= z /\ z < y /\  0<=y /\ y <= Normal t /\ y < x /\ x <= Normal t} INTER p_space p )
	    ``,


RW_TAC std_ss [AND_INTER, rv_gt0_ninfinity_def]
THEN RW_TAC std_ss [DFT_event_def, D_BEFORE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
	         THEN Q.EXISTS_TAC (`(X x,Y x,Z x) `)
                 THEN FULL_SIMP_TAC std_ss [LAMBDA_PROD, PAIR]
	         THEN METIS_TAC [lt_trans])
           THEN METIS_TAC [extreal_not_infty, le_infty, lt_infty])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
     THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	    
   THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[]
	    THEN METIS_TAC [lt_le, le_lt, lt_trans, le_trans])
	    
     THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[]
	    THEN METIS_TAC [lt_le, le_lt, lt_trans, le_trans])
	    
         THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
  THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
THEN KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
               x <= Normal t)) (x':extreal#extreal#extreal) =
	       ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  0<= SND (SND x') /\ SND (SND x') < FST (SND x') /\ 0<= FST (SND x') /\ FST(SND x') <=Normal t /\ FST (SND x') < FST x' /\ FST x'  <= Normal t) ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
 THEN DISCH_TAC	
 THEN FULL_SIMP_TAC std_ss[]);







val PREIMAGE_EXTREAL_REAL_3RV_event_PAND_CSP = store_thm(
  "PREIMAGE_EXTREAL_REAL_3RV_event_PAND_CSP",
  `` !t X Y Z.  rv_gt0_ninfinity [X;Y;Z] ==>
  (PREIMAGE (\s. ((X s,Y s),Z s))
     {((x,y),z) |
      0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
             x <= Normal t} =
   PREIMAGE (\s. ((real (X s),real (Y s)),real (Z s)))
     {((x',y'),z') |
 0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\
             x' <= t}) ``,

RW_TAC std_ss [rv_gt0_ninfinity_def, PREIMAGE_def]
THEN KNOW_TAC(``!s. ((X s,Y s),Z s) IN
   {((x,y),z) |
   0 <= z /\ z < y /\ 0 <= y /\ y <= Normal t /\ y < x /\
             x <= Normal t} =
    0 <= Z s /\ Z s < Y s /\ 0 <= Y s /\ Y s <= Normal t /\
           Y s < X s /\ X s <= Normal t``)
     THEN1(RW_TAC std_ss [IN_ELIM_PAIR_THM_1])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!s. ((real (X s),real (Y s)),real (Z s)) IN
   {((x',y'),z') |
  0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}=
   0<= real (Z s) /\  real (Z s) < real (Y s) /\  0 <= real (Y s) /\ real (Y s) <=t/\ real (Y s) < real (X s) /\  real (X s) <= t ``)
  THEN1(RW_TAC std_ss [IN_ELIM_PAIR_THM_1])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN `X x <> PosInf` by RW_TAC std_ss[] 
THEN `Y x <> PosInf` by RW_TAC std_ss[]
THEN `Z x <> PosInf` by RW_TAC std_ss[]
THEN `X x <> NegInf /\ Y x <> NegInf /\ Z x <> NegInf` by METIS_TAC [le_lt, lt_le, extreal_le_def, extreal_le_def, lt_infty, le_infty, le_trans, lt_trans, extreal_of_num_def]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1(`0 = real 0` by RW_TAC std_ss[real_of_0]
	          THEN ONCE_ASM_REWRITE_TAC[]
		  THEN `0<= Z x` by RW_TAC std_ss[]
		  THEN  `0 <> PosInf /\ 0<> NegInf` by METIS_TAC [extreal_lt_def, extreal_le_def, lt_infty, le_infty, lt_le, le_lt, extreal_of_num_def]
		  THEN METIS_TAC [extreal_real_le])
	   THEN1(METIS_TAC [extreal_real_lt])
	   THEN1(`0 = real 0` by RW_TAC std_ss[real_of_0]
	          THEN ONCE_ASM_REWRITE_TAC[]
		  THEN `0<= Z x` by RW_TAC std_ss[]
		  THEN  `0 <> PosInf /\ 0<> NegInf` by METIS_TAC [extreal_lt_def, extreal_le_def, lt_infty, le_infty, lt_le, le_lt, extreal_of_num_def]
		  THEN METIS_TAC [extreal_real_le])
	   THEN1(RW_TAC std_ss [real_def]
	         THEN SELECT_ELIM_TAC
                 THEN RW_TAC std_ss[]
		      THEN1( Q.EXISTS_TAC `real (Y x)`
		            THEN METIS_TAC [normal_real])
                 THEN METIS_TAC [extreal_le_eq])
	   THEN1(METIS_TAC[extreal_real_lt])
	   THEN RW_TAC std_ss [real_def]
	         THEN SELECT_ELIM_TAC
                 THEN RW_TAC std_ss[]
		      THEN1( Q.EXISTS_TAC `real (X x)`
		            THEN METIS_TAC [normal_real])
                 THEN METIS_TAC [extreal_le_eq])
	
THEN RW_TAC std_ss[]		 
     THEN1(METIS_TAC [extreal_real_lt])
     THEN1(UNDISCH_TAC(``real (Y x) <= t ``)
           THEN REWRITE_TAC[real_def]
	   THEN SELECT_ELIM_TAC
	   THEN RW_TAC std_ss[]
	        THEN1(Q.EXISTS_TAC (`real (Y x)`)
		     THEN METIS_TAC [normal_real])
	   THEN RW_TAC std_ss [extreal_le_eq])
     THEN1(METIS_TAC[extreal_real_lt])	   
THEN UNDISCH_TAC(`` real (X x) <= t``)	   
THEN REWRITE_TAC [real_def]
THEN SELECT_ELIM_TAC
THEN RW_TAC std_ss[]
     THEN1(Q.EXISTS_TAC (`real (X x)`)
           THEN METIS_TAC [normal_real])
THEN RW_TAC std_ss [extreal_le_eq]);




val prob_event_PAND_CSP_INTER = store_thm("prob_event_PAND_CSP_INTER",

  ``!p t X Y Z f_xy f_cond  f_y  .
     prob_space p /\ den_gt0_ninfinity f_xy f_y f_cond /\ rv_gt0_ninfinity [X; Y; Z] /\ (!s. ALL_DISTINCT [X s; Y s; Z s])/\
       (!y. cond_density lborel lborel p (\x. real (X x))  (\x. real (Y  x))  y f_xy f_y f_cond ) /\ indep_varp p (pair_measure lborel lborel)
           (\x. (real (X x),real (Y x))) lborel (\x. real (Z x)) /\
        (!x. f_xy x <> PosInf)  /\  random_variable (\x. real (Z x)) p borel /\
	cont_CDF p (\s. real (Z s)) ==>
  
   ( prob p (DFT_event p (D_AND (D_AND X (D_BEFORE Z Y)) (D_BEFORE Y X)) t) =
       pos_fn_integral lborel
             (\y.
                  f_y y * CDF p (\s. real (Z s)) y * indicator_fn {y' |  0 <= y' /\ y' <= t} y *
                  pos_fn_integral lborel
                    (\x.
                         f_cond y x  * indicator_fn {x' | y < x' /\ x' <= t} x )))``,


RW_TAC std_ss[cont_CDF_def]
THEN  MP_TAC(Q.ISPECL [`p: ('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `(\x. real ((X:'a->extreal) x))` , `(\x. real ((Y:'a->extreal) x))`, `(\x. real ((Z:'a->extreal) x))`, `(pair_measure lborel lborel)` , `lborel`, ` {((x',y'),z') | 0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t:real}`] prob_event_3rv_2indep)

THEN RW_TAC std_ss[]
THEN ` measure_space (pair_measure lborel lborel)` by METIS_TAC [measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]
THEN `measure_space lborel` by RW_TAC std_ss [measure_space_lborel]
THEN `sigma_finite_measure (pair_measure lborel lborel)` by RW_TAC std_ss[sigma_finite_measure_pair_lborel]
THEN `sigma_finite_measure lborel` by RW_TAC std_ss [sigma_finite_measure_lborel]
THEN `{((x',y'),z') |
        0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t:real} IN
         measurable_sets (pair_measure (pair_measure lborel lborel) lborel)` by RW_TAC std_ss[PAND_CSP_event_measurable_set]
THEN FULL_SIMP_TAC std_ss[]
THEN RW_TAC std_ss[DFT_event_PAND_CSP_INTER]
THEN RW_TAC std_ss[PREIMAGE_EXTREAL_REAL_3RV_event_PAND_CSP, rv_gt0_ninfinity_def] 
THEN KNOW_TAC(``( pos_fn_integral
     (distr p (pair_measure lborel lborel) (\x. (real (X x),real (Y x))))
     (\ (x,y).
          pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z.
                 indicator_fn
                   {((x',y'),z') |
                    0 <= z' /\ z' < y' /\ 0 <= y' /\ y' <= t /\ y' < x' /\
                   x' <= t} ((x,y),z))) =
     pos_fn_integral lborel
       (\y.
            indicator_fn {y' | 0<= y' /\ y' <= t} y *
            pos_fn_integral lborel
              (\x.
                   f_xy (x,y) *
                   indicator_fn {x' |  y < x' /\ x' <= t} x *
                   CDF p (\s. real (Z s)) y))) ``)
      THEN1( MP_TAC(Q.ISPECL [`p: ('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,
                       `(\x. real ((X:'a->extreal) x))` , `(\x. real ((Y:'a->extreal) x))`, `(\x. real ((Z:'a->extreal) x))`, `f_xy : (real#real) ->extreal`,`t:real` ] joint_density_3rv_3_PAND_CSP)
           THEN RW_TAC std_ss[]
           THEN FULL_SIMP_TAC std_ss[den_gt0_ninfinity_def, cond_density_def]
	   THEN KNOW_TAC(``!(x:real#real). 0<= f_xy x``) 
                THEN1(RW_TAC std_ss []
		      THEN `(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
		      THEN ONCE_ASM_REWRITE_TAC[]
		      THEN FULL_SIMP_TAC std_ss[])
	  THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(``(!z. 0 <= real (Z z)) ``)
	       THEN1(FULL_SIMP_TAC std_ss[rv_gt0_ninfinity_def]
	             THEN RW_TAC std_ss[real_def]
		          THEN1(RW_TAC std_ss[REAL_LE_REFL])
		     THEN SELECT_ELIM_TAC
                     THEN RW_TAC std_ss[]
		          THEN1(Q.EXISTS_TAC `real (Z z)`
			        THEN `Z z <> PosInf` by RW_TAC std_ss[]
				THEN RW_TAC std_ss[normal_real])
		     THEN `0 <= Z z` by RW_TAC std_ss[]
		     THEN METIS_TAC [extreal_le_eq, extreal_of_num_def])
          THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[])	  

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]	  
THEN DEP_REWRITE_TAC[cond_density_3rv_PAND_CSP]
THEN RW_TAC std_ss[]
     THEN1( EXISTS_TAC(``(\x. real ((X:'a->extreal) x))``)
            THEN EXISTS_TAC(``(\x. real ((Y:'a->extreal) x))``)
	    THEN RW_TAC std_ss[])
THEN DEP_REWRITE_TAC[cond_density_3rv_PAND_CSP_1]
THEN RW_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[den_gt0_ninfinity_def]);


val _ = export_theory();
