(******************************************************************************)
(*                       Probabilistic Analysis of                            *)
(*                        Cardiac Assist System                               *)
(*                                  by                                        *)
(*                          Yassmeen Elderhalli                               *)
(*                      y_elderh@encs.concordia.ca                            *)
(******************************************************************************)

(*loadPath := "/home/yassmeen/Downloads/ProbTheories_HOL-k12" :: !loadPath;


(*set_trace "Unicode" 0;*)



app load ["HolKernel", "Parse", "boolLib", "bossLib", "numLib", "unwindLib", 
"tautLib", "Arith", "prim_recTheory", "combinTheory", "quotientTheory", 
"arithmeticTheory", "realTheory", "hrealTheory", "realaxTheory", "realLib",  
"jrhUtils", "pairTheory", "seqTheory", "limTheory", "transcTheory", "listTheory", 
"mesonLib", "boolTheory", "topologyTheory", "pred_setTheory", "util_probTheory", 
"optionTheory", "numTheory", "sumTheory", "InductiveDefinition", "ind_typeTheory",
"pred_setLib", "iterate_hvgTheory", "card_hvgTheory", "product_hvgTheory", 
"topology_hvgTheory", "derivative_hvgTheory", "integration_hvgTheory",
"real_sigmaTheory", "extreal_hvgTheory", "measure_hvgTheory", "lebesgue_hvgTheory",
"probability_hvgTheory", "lebesgue_measure_hvgTheory", "normal_rv_hvgTheory", "netsTheory", "PIE_EXTREALTheory","dep_rewrite", "integration_before_afterTheory", "DFT_Gates_def_PROBTheory"];
*)
open HolKernel Parse boolLib bossLib numLib unwindLib tautLib Arith prim_recTheory 
combinTheory quotientTheory arithmeticTheory hrealTheory realaxTheory realTheory 
realLib jrhUtils pairTheory seqTheory limTheory transcTheory listTheory mesonLib 
boolTheory topologyTheory pred_setTheory util_probTheory optionTheory numTheory 
sumTheory InductiveDefinition ind_typeTheory pred_setLib iterate_hvgTheory 
card_hvgTheory product_hvgTheory topology_hvgTheory derivative_hvgTheory
integration_hvgTheory real_sigmaTheory extreal_hvgTheory measure_hvgTheory
lebesgue_hvgTheory probability_hvgTheory lebesgue_measure_hvgTheory normal_rv_hvgTheory netsTheory PIE_EXTREALTheory dep_rewrite integration_before_afterTheory DFT_Gates_def_PROBTheory ;



val _ = new_theory "CAS";
fun SET_TAC L =
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION,
    IN_INTER, IN_DIFF, IN_INSERT, IN_DELETE, IN_BIGINTER, IN_BIGUNION,
    IN_IMAGE, GSPECIFICATION, IN_DEF] THEN METIS_TAC [];

fun SET_RULE tm = prove(tm,SET_TAC []);


val set_rewrs
= [INTER_EMPTY, INTER_UNIV, UNION_EMPTY, UNION_UNIV, DISJOINT_UNION,
DISJOINT_INSERT, DISJOINT_EMPTY, GSYM DISJOINT_EMPTY_REFL,
DISJOINT_BIGUNION, INTER_SUBSET, INTER_IDEMPOT, UNION_IDEMPOT,
SUBSET_UNION];

val UNIONL_def = Define `(UNIONL [] = {})
/\ (UNIONL (s::ss) = (s:'a ->bool) UNION UNIONL ss)`;


val IN_UNIONL = store_thm
("IN_UNIONL",
``!l (v:'a ). v IN UNIONL l = (?s. MEM s l /\ v IN s)``,
Induct THEN1 RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY]
THEN RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY, IN_UNION]
THEN PROVE_TAC []);


val elt_rewrs
= [SUBSET_DEF, IN_COMPL, IN_DIFF, IN_UNIV, NOT_IN_EMPTY, IN_UNION,
IN_INTER, IN_IMAGE, IN_FUNSET, IN_DFUNSET, GSPECIFICATION,
DISJOINT_DEF, IN_BIGUNION, IN_BIGINTER, IN_COUNT, IN_o,
IN_UNIONL, IN_DELETE, IN_PREIMAGE, IN_SING, IN_INSERT];


fun rewr_ss ths =
simpLib.++
(std_ss,
simpLib.SSFRAG
{ac = [],
name = NONE,
convs = [],
dprocs = [],
filter = NONE,
rewrs = set_rewrs @ elt_rewrs,
congs = []});
val pset_set_ss = rewr_ss set_rewrs;
val pset_elt_ss = rewr_ss elt_rewrs;
val pset_ss = rewr_ss (set_rewrs @ elt_rewrs);


fun PSET_TAC ths =
REPEAT (POP_ASSUM MP_TAC)
THEN RW_TAC pset_set_ss ths
THEN  REPEAT (POP_ASSUM MP_TAC)
THEN RW_TAC pset_elt_ss ths;

val Simplify = RW_TAC arith_ss;
fun wrap a = [a];
val Rewr = DISCH_THEN (REWRITE_TAC o wrap);
val Rewr' = DISCH_THEN (ONCE_REWRITE_TAC o wrap);
val FUN_EQ_TAC = CONV_TAC (CHANGED_CONV (ONCE_DEPTH_CONV FUN_EQ_CONV));
fun K_TAC _ = ALL_TAC;
fun MESON ths tm = prove(tm,MESON_TAC ths);
fun METIS ths tm = prove(tm,METIS_TAC ths);
val DISC_RW_KILL = DISCH_TAC THEN ONCE_ASM_REWRITE_TAC [] THEN 
                   POP_ASSUM K_TAC;
			   
val IN_REST = store_thm ("IN_REST",
 ``!x:'a. !s. x IN (REST s) <=> x IN s /\ ~(x = CHOICE s)``,
  REWRITE_TAC[REST_DEF, IN_DELETE]);
fun SET_TAC L = 
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN
    SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION, IN_INTER, IN_DIFF, 
      IN_INSERT, IN_DELETE, IN_REST, IN_BIGINTER, IN_BIGUNION, IN_IMAGE, 
      GSPECIFICATION, IN_DEF, EXISTS_PROD] THEN METIS_TAC [];
fun ASSERT_TAC tm = SUBGOAL_THEN tm STRIP_ASSUME_TAC;
fun SET_RULE tm = prove(tm,SET_TAC []);
fun ASM_SET_TAC L = REPEAT (POP_ASSUM MP_TAC) THEN SET_TAC L;
val ASM_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN ARITH_TAC;
val ASM_REAL_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN REAL_ARITH_TAC;



val hot_shared_spare = store_thm("hot_shared_spare",
  `` !A B C_a C_d.
       (!s. ALL_DISTINCT [A s; B s; C_a s]) /\ (C_a = C_d) ==>
       (D_AND (shared_spare A B C_a C_d) (shared_spare B A C_a C_d) =
        D_AND (D_AND A B) C_a)``,

RW_TAC std_ss [shared_spare_def, D_AND_def, D_OR_def, D_BEFORE_def, D_SIMULT_def]
THEN KNOW_TAC(`` !s.    
(
   max
     (min
        (min (max (A s) (if C_a s < A s then C_a s else PosInf))
           (max (A s) (if B s < A s then B s else PosInf)))
        (max (C_a s) (if A s < C_a s then A s else PosInf)))
     (min
        (min (max (B s) (if C_a s < B s then C_a s else PosInf))
           (max (B s) (if A s < B s then A s else PosInf)))
        (max (C_a s) (if B s < C_a s then B s else PosInf)))) =
( max (max (A s) (B s)) (C_a s))``)
      THEN1(RW_TAC std_ss []
            THEN RW_TAC std_ss[]
	    THEN METIS_TAC [min_infty, max_infty, lt_infty, lt_le, lt_total, lte_trans, let_trans, extreal_lt_def, extreal_le_def, extreal_min_def, extreal_max_def, ALL_DISTINCT, MEM, max_refl, min_refl, ALL_DISTINCT, MEM, le_antisym ])
THEN RW_TAC std_ss[]);



val AND_OR_ID_LEFT = store_thm("AND_OR_ID_LEFT",
  ``! A B. D_AND A (D_OR A B) = A``,
RW_TAC std_ss[D_AND_def, D_OR_def]
THEN  METIS_TAC [min_infty, max_infty, lt_infty, lt_le, lt_total, lte_trans, let_trans, extreal_lt_def, extreal_le_def, extreal_min_def, extreal_max_def, ALL_DISTINCT, MEM, max_refl, min_refl, ALL_DISTINCT, MEM, le_antisym ]);

val OR_AND_ID_LEFT = store_thm("OR_AND_ID_LEFT",
 ``  ! A B. D_OR A (D_AND A B) = A``,
RW_TAC std_ss[D_AND_def, D_OR_def]
THEN  METIS_TAC [min_infty, max_infty, lt_infty, lt_le, lt_total, lte_trans, let_trans, extreal_lt_def, extreal_le_def, extreal_min_def, extreal_max_def, ALL_DISTINCT, MEM, max_refl, min_refl, ALL_DISTINCT, MEM, le_antisym ]);

val  HCAS = store_thm("HCAS",
  `` !PA PB PS MS MA MB CS SS P B.
       (!s. ALL_DISTINCT [MA s; MS s; PA s; PB s; PS s]) ==>
       (D_OR
          (D_OR
             (D_AND (shared_spare PA PB PS PS)
                (shared_spare PB PA PS PS))
             (D_OR (P_AND MS MA) (HSP MA MB)))
          (HSP (FDEP (D_OR CS SS) P) (FDEP (D_OR CS SS) B)) =
        D_OR
          (D_OR (D_OR (D_OR CS SS) (D_OR (P_AND MS MA) (D_AND MA MB)))
             (D_AND P B)) (D_AND (D_AND PA PB) PS))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``!s. ALL_DISTINCT [(PA :'a->extreal) s; PB s ;PS s]``)
     THEN1(METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [hot_shared_spare]
THEN RW_TAC std_ss [HSP_AND]
THEN RW_TAC std_ss [FDEP_OR]
THEN KNOW_TAC(``(D_AND (D_OR (D_OR CS SS) P) (D_OR (D_OR CS SS) B)) = D_OR (D_OR (CS)(SS))(D_AND P B)``)
     THEN1(RW_TAC std_ss [D_AND_OVER_OR]
           THEN RW_TAC std_ss [D_OR_ASSOC]
	   THEN KNOW_TAC(``(D_AND (D_OR CS (D_OR SS P)) CS) = CS``)
	        THEN1(RW_TAC std_ss[GSYM D_OR_ASSOC]
		      THEN RW_TAC std_ss[D_AND_COMM]
		      THEN RW_TAC std_ss [D_AND_OVER_OR]
                      THEN RW_TAC std_ss [D_AND_ID]
		      THEN RW_TAC std_ss[D_OR_OVER_AND]
		      THEN RW_TAC std_ss [D_OR_ID]
		      THEN RW_TAC std_ss [AND_OR_ID_LEFT]
		      THEN RW_TAC std_ss [D_OR_ID]
		      THEN RW_TAC std_ss [D_AND_OVER_OR]
		      THEN RW_TAC std_ss [D_AND_ID]
		      THEN RW_TAC std_ss [OR_AND_ID_LEFT])
	   THEN DISCH_TAC
 	   THEN ASM_REWRITE_TAC[]
           THEN RW_TAC std_ss [D_AND_COMM]
	   THEN RW_TAC std_ss [D_AND_OVER_OR]
	   THEN RW_TAC std_ss [D_AND_ID]
	   THEN RW_TAC std_ss [OR_AND_ID_LEFT]
	   THEN RW_TAC std_ss[D_OR_COMM]
	   THEN RW_TAC std_ss [OR_AND_ID_LEFT]
	   THEN RW_TAC std_ss [GSYM D_OR_ASSOC]
	   THEN KNOW_TAC(``(D_OR (D_OR CS SS) (D_AND B CS)) = D_OR CS SS``)
	        THEN1(`D_OR CS SS = D_OR SS CS ` by RW_TAC std_ss [D_OR_COMM]
		      THEN ASM_REWRITE_TAC []
		      THEN RW_TAC std_ss [D_OR_ASSOC]
		      THEN `D_AND B CS = D_AND CS B` by RW_TAC std_ss [D_AND_COMM]
		      THEN ASM_REWRITE_TAC []
		      THEN RW_TAC std_ss [OR_AND_ID_LEFT])
           THEN DISCH_TAC
	   THEN ASM_REWRITE_TAC[]
	   THEN KNOW_TAC(``D_OR (D_OR (D_OR CS SS) (D_AND B P)) (D_AND B SS)
	                   = D_OR (D_OR (D_OR CS SS) (D_AND B SS)) (D_AND B P)  ``)
                THEN1(METIS_TAC [D_AND_COMM, D_OR_COMM, D_AND_ASSOC, D_OR_ASSOC])
	   THEN DISCH_TAC
	   THEN ASM_REWRITE_TAC[]
	   THEN KNOW_TAC(``(D_OR (D_OR CS SS) (D_AND B SS)) = D_OR CS SS``)
	        THEN1(RW_TAC std_ss [D_OR_ASSOC]
		      THEN `D_AND B SS = D_AND SS B` by RW_TAC std_ss [D_AND_COMM]
		      THEN ASM_REWRITE_TAC []
		      THEN RW_TAC std_ss [OR_AND_ID_LEFT])
           THEN DISCH_TAC
	   THEN ASM_REWRITE_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]

THEN METIS_TAC [D_AND_COMM, D_OR_COMM, D_AND_ASSOC, D_OR_ASSOC]);


 val HCAS_final = store_thm("HCAS_final",
  `` !PA PB PS  MS MA MB CS SS P B.
     (!s. ALL_DISTINCT [MA s; MS s; PA s; PB s; PS s]) ==>
     (D_OR
        (D_OR
           (D_AND (shared_spare PA PB PS PS)
              (shared_spare PB PA PS PS))
           (D_OR (P_AND MS MA) (HSP MA MB)))
        (HSP (FDEP (D_OR CS SS) P) (FDEP (D_OR CS SS) B)) =
      D_OR
        (D_OR
           (D_OR (D_OR CS SS)
              (D_OR (D_AND MA (D_BEFORE MS MA)) (D_AND MA MB)))
           (D_AND P B)) (D_AND (D_AND PA PB) PS))``,

RW_TAC std_ss [HCAS]
THEN KNOW_TAC(``P_AND MS MA = D_AND MA (D_BEFORE MS MA)``)
     THEN1(RW_TAC  std_ss[PAND_DEF2, D_INCLUSIVE_BEFORE_def2]
           THEN KNOW_TAC (``D_SIMULT MS MA = NEVER``)
	        THEN1(RW_TAC std_ss [D_SIMULT_def, NEVER_def]
		     THEN METIS_TAC [ALL_DISTINCT, MEM])
           THEN DISCH_TAC
	   THEN ASM_REWRITE_TAC[]
	   THEN RW_TAC std_ss [D_OR_NEVER])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]);

val HCAS_final_2 = store_thm("HCAS_final_2",
  ``!PA PB  PS MS MA MB CS SS P B.
     (!s. ALL_DISTINCT [MA s; MS s; PA s; PB s; PS s]) ==>
     (D_OR
        (D_OR
           (D_AND (shared_spare PA PB PS PS)
              (shared_spare PB PA PS PS))
           (D_OR (P_AND MS MA) (HSP MA MB)))
        (HSP (FDEP (D_OR CS SS) P) (FDEP (D_OR CS SS) B)) =
      D_OR
        (D_OR
           (D_OR (D_OR (D_OR CS SS) (D_AND MA (D_BEFORE MS MA)))
              (D_AND MA MB)) (D_AND P B)) (D_AND (D_AND PA PB) PS))``,
RW_TAC std_ss [HCAS_final]
THEN METIS_TAC[D_OR_ASSOC, D_OR_COMM]);

val event_before_union_event = store_thm("event_before_union_event",
  ``   !A B C p t.
       DFT_event p (D_BEFORE A B) t INTER DFT_event p C t =
       {s | A s < B s /\ A s <= Normal t} INTER DFT_event p C t UNION
       {s | B s <= A s /\ PosInf <= Normal t} INTER DFT_event p C t``,
       
RW_TAC std_ss [DFT_event_def, D_BEFORE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN METIS_TAC [extreal_not_infty, le_infty]);

val  event_and_before_union = store_thm("event_and_before_union",
  ``   !A B C p t.
       DFT_event p (D_AND B (D_BEFORE A B)) t INTER DFT_event p C t =
       {s | A s < B s /\ B s <= Normal t} INTER DFT_event p C t UNION
       {s | B s <= A s /\ PosInf <= Normal t} INTER DFT_event p C t``,
       
RW_TAC std_ss [DFT_event_def, D_BEFORE_def, D_AND_def, extreal_max_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1( FULL_SIMP_TAC std_ss[]
	          THEN METIS_TAC [extreal_le_def, extreal_lt_def, le_lt, lt_le])
           THEN1(METIS_TAC [extreal_le_def, extreal_lt_def, le_lt, lt_le])
	   THEN  FULL_SIMP_TAC std_ss[]
	   THEN METIS_TAC [extreal_le_def, extreal_lt_def, le_lt, lt_le, le_infty, lt_infty])
THEN METIS_TAC [extreal_le_def, extreal_lt_def, le_lt, lt_le, le_infty, lt_infty]);


val disjoint_and_before_inter_event = store_thm(
  "disjoint_and_before_inter_event",
  ``  !A B C p t.
       DISJOINT
         ({s | A s < B s /\ B s <= Normal t} INTER DFT_event p C t)
         ({s | B s <= A s /\ PosInf <= Normal t} INTER DFT_event p C t)``,

RW_TAC std_ss [DFT_event_def, DISJOINT_DEF]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN RW_TAC std_ss[]
THEN KNOW_TAC(``((~(A x < B x) \/ ~(B x <= Normal t)) \/ ~(C x <= Normal t) \/
 x NOTIN p_space p) \/ (~(B x <= A x) \/ ~(PosInf <= Normal t)) \/
~(C x <= Normal t) \/ x NOTIN p_space p
= ((~(A x < B x) \/(~(B x <= A x) \/ ~(B x <= Normal t)) \/ ~(C x <= Normal t) \/
 x NOTIN p_space p) \/  ~(PosInf <= Normal t)) \/
~(C x <= Normal t) \/ x NOTIN p_space p
``)
     THEN1(METIS_TAC [DISJ_COMM, DISJ_ASSOC])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN RW_TAC std_ss [extreal_lt_def]
THEN METIS_TAC [EXCLUDED_MIDDLE]);


val add_assoc_5_var = store_thm("add_assoc_5_var",
  `` !A1 A2 A3 A4 A5.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf ==>
       (A1 + (A2 + (A3 + (A4 + A5))) = A1 + A2 + A3 + A4 + A5)
  ``,
METIS_TAC[add_assoc,extreal_hvgTheory.add_not_infty]);


val add_assoc_5_var1 = store_thm("add_assoc_5_var1",
  ``   !A1 A2 A3 A4 A5.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf ==>
       (A1 + (A2 + A3 + A4 + A5) = A1 + A2 + A3 + A4 + A5)
 ``,

METIS_TAC[add_assoc,extreal_hvgTheory.add_not_infty]);


val add_assoc_10_var = store_thm("add_assoc_10_var",
  `` !A1 A2 A3 A4 A5 A6 A7 A8 A9 A10.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf /\ A6 <> PosInf /\ A7 <> PosInf /\ A8 <> PosInf /\
       A9 <> PosInf /\ A10 <> PosInf ==>
       (A1 +
        (A2 + (A3 + (A4 + (A5 + (A6 + (A7 + (A8 + (A9 + A10)))))))) =
        A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9 + A10)

 ``,
RW_TAC std_ss [add_assoc_5_var]
THEN KNOW_TAC(``(A6 + A7 + A8 + A9 + A10) <> PosInf``)
     THEN1(RW_TAC std_ss [ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(`` (A5 + (A6 + A7 + A8 + A9 + A10)) = A5+ A6 +A7 +A8 +A9 +A10``)
     THEN1(METIS_TAC[add_assoc,extreal_hvgTheory.add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A5+A6 + A7 + A8 + A9 + A10) <> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(`` (A4 + (A5+ A6 + A7 + A8 + A9 + A10)) = A4+ A5+ A6 +A7 +A8 +A9 +A10``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A4+A5+A6 + A7 + A8 + A9 + A10) <> PosInf``)
     THEN1 (METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(`` (A3 + (A4+A5+ A6 + A7 + A8 + A9 + A10)) = A3+A4+ A5+ A6 +A7 +A8 +A9 +A10``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A3+A4+A5+A6 + A7 + A8 + A9 + A10) <> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(`` (A2 + (A3+A4+A5+ A6 + A7 + A8 + A9 + A10)) = A2+A3+A4+ A5+ A6 +A7 +A8 +A9 +A10``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A2+A3+A4+A5+A6 + A7 + A8 + A9 + A10) <> PosInf``)
     THEN1 (METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty]);



val add_assoc_20_var = store_thm("add_assoc_20_var",
  `` 
     !A1 A2 A3 A4 A5 A6 A7 A8 A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19
        A20.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf /\ A6 <> PosInf /\ A7 <> PosInf /\ A8 <> PosInf /\
       A9 <> PosInf /\ A10 <> PosInf /\ A11 <> PosInf /\
       A12 <> PosInf /\ A13 <> PosInf /\ A14 <> PosInf /\
       A15 <> PosInf /\ A16 <> PosInf /\ A17 <> PosInf /\
       A18 <> PosInf /\ A19 <> PosInf /\ A20 <> PosInf ==>
       (A1 +
        (A2 +
         (A3 +
          (A4 +
           (A5 +
            (A6 +
             (A7 +
              (A8 +
               (A9 +
                (A10 +
                 (A11 +
                  (A12 +
                   (A13 +
                    (A14 +
                     (A15 +
                      (A16 +
                       (A17 + (A18 + (A19 + A20)))))))))))))))))) =
        A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9 + A10 + A11 + A12 +
        A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20)
 ``,

RW_TAC std_ss [add_assoc_10_var]
THEN KNOW_TAC(``(A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A10 +
         (A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) = A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A9 +
         (A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A8 +
         (A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A8+A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``) 

     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A7 +
         (A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A7+A8+A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A6 +
         (A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A6+A7+A8+A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A5 +
         (A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A5+A6+A7+A8+A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1 (DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A4 +
         (A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A4+A5+A6+A7+A8+A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A3 +
         (A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A3+A4+A5+A6+A7+A8+A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1 (METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A2 +
         (A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)) =A2+A3+A4+A5+A6+A7+A8+A9+ A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20``)
     THEN1 (DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A2+A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +
          A20)<> PosInf``)
     THEN1(METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty]);
 
val add_assoc_30_var = store_thm("add_assoc_30_var",
  ``  !A1 A2 A3 A4 A5 A6 A7 A8 A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19
        A20 A21 A22 A23 A24 A25 A26 A27 A28 A29 A30.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf /\ A6 <> PosInf /\ A7 <> PosInf /\ A8 <> PosInf /\
       A9 <> PosInf /\ A10 <> PosInf /\ A11 <> PosInf /\
       A12 <> PosInf /\ A13 <> PosInf /\ A14 <> PosInf /\
       A15 <> PosInf /\ A16 <> PosInf /\ A17 <> PosInf /\
       A18 <> PosInf /\ A19 <> PosInf /\ A20 <> PosInf /\
       A21 <> PosInf /\ A22 <> PosInf /\ A23 <> PosInf /\
       A24 <> PosInf /\ A25 <> PosInf /\ A26 <> PosInf /\
       A27 <> PosInf /\ A28 <> PosInf /\ A29 <> PosInf /\
       A30 <> PosInf ==>
       (A1 +
        (A2 +
         (A3 +
          (A4 +
           (A5 +
            (A6 +
             (A7 +
              (A8 +
               (A9 +
                (A10 +
                 (A11 +
                  (A12 +
                   (A13 +
                    (A14 +
                     (A15 +
                      (A16 +
                       (A17 +
                        (A18 +
                         (A19 +
                          (A20 +
                           (A21 +
                            (A22 +
                             (A23 +
                              (A24 +
                               (A25 +
                                (A26 +
                                 (A27 +
                                  (A28 +
                                   (A29 +
                                    A30)))))))))))))))))))))))))))) =
        A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9 + A10 + A11 + A12 +
        A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 +
        A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30)
 ``,

RW_TAC std_ss [add_assoc_20_var]
THEN KNOW_TAC(``(A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A10 +
         (A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) = A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1 (METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A9 +
         (A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) = A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)

     THEN1( DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A8 +
         (A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) =A8+ A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A7 +
         (A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) =A7+A8+ A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A6 +
         (A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) =A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A5 +
         (A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) =A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A4 +
         (A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) =A4+A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A3 +
         (A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) =A3+A4+A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1( METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A2 +
         (A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30)) =A2+A3+A4+A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
A26 + A27 + A28 + A29 + A30``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A2+A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 +
          A30) <> PosInf``)
     THEN1 (METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty] THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty]);
 

val add_assoc_40_var = store_thm("add_assoc_40_var",
  ``      !A1 A2 A3 A4 A5 A6 A7 A8 A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19
        A20 A21 A22 A23 A24 A25 A26 A27 A28 A29 A30 A31 A32 A33 A34 A35
        A36 A37 A38 A39 A40.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf /\ A6 <> PosInf /\ A7 <> PosInf /\ A8 <> PosInf /\
       A9 <> PosInf /\ A10 <> PosInf /\ A11 <> PosInf /\
       A12 <> PosInf /\ A13 <> PosInf /\ A14 <> PosInf /\
       A15 <> PosInf /\ A16 <> PosInf /\ A17 <> PosInf /\
       A18 <> PosInf /\ A19 <> PosInf /\ A20 <> PosInf /\
       A21 <> PosInf /\ A22 <> PosInf /\ A23 <> PosInf /\
       A24 <> PosInf /\ A25 <> PosInf /\ A26 <> PosInf /\
       A27 <> PosInf /\ A28 <> PosInf /\ A29 <> PosInf /\
       A30 <> PosInf /\ A31 <> PosInf /\ A32 <> PosInf /\
       A33 <> PosInf /\ A34 <> PosInf /\ A35 <> PosInf /\
       A36 <> PosInf /\ A37 <> PosInf /\ A38 <> PosInf /\
       A39 <> PosInf /\ A40 <> PosInf ==>
       (A1 +
        (A2 +
         (A3 +
          (A4 +
           (A5 +
            (A6 +
             (A7 +
              (A8 +
               (A9 +
                (A10 +
                 (A11 +
                  (A12 +
                   (A13 +
                    (A14 +
                     (A15 +
                      (A16 +
                       (A17 +
                        (A18 +
                         (A19 +
                          (A20 +
                           (A21 +
                            (A22 +
                             (A23 +
                              (A24 +
                               (A25 +
                                (A26 +
                                 (A27 +
                                  (A28 +
                                   (A29 +
                                    (A30 +
                                     (A31 +
                                      (A32 +
                                       (A33 +
                                        (A34 +
                                         (A35 +
                                          (A36 +
                                           (A37 +
                                            (A38 +
                                             (A39 +
                                              A40)))))))))))))))))))))))))))))))))))))) =
        A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9 + A10 + A11 + A12 +
        A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 +
        A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 +
        A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40)
 ``,

RW_TAC std_ss [add_assoc_30_var]
THEN KNOW_TAC(``(A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A10 +
         (A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A9 +
         (A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) = A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A8 +
         (A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A8+ A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A7 +
         (A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A7+A8+ A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
     THEN1 (DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A6 +
         (A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
    THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN  DISCH_TAC
THEN KNOW_TAC(``(A5 +
         (A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A4 +
         (A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A4+A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40) <> PosInf``)
     THEN1 (METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A3 +
         (A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A3+A4+A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20	             + A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          	 A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
                 A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A2 +
         (A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
          A40)) =A2+A3+A4+A5+A6+A7+A8+ A9+A10 + A11 + A12 + A13 +
	  A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	  A38 + A39 + A40 ``)
    THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A2+A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +		      A20 + A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
                  A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 +
                  A40) <> PosInf``)
     THEN1(METIS_TAC[add_not_infty])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
THEN RW_TAC std_ss []
THEN METIS_TAC[ add_not_infty]);


val add_assoc_50_var = store_thm("add_assoc_50_var",
  `` 
     !A1 A2 A3 A4 A5 A6 A7 A8 A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19
        A20 A21 A22 A23 A24 A25 A26 A27 A28 A29 A30 A31 A32 A33 A34 A35
        A36 A37 A38 A39 A40 A41 A42 A43 A44 A45 A46 A47 A48 A49 A50.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf /\ A6 <> PosInf /\ A7 <> PosInf /\ A8 <> PosInf /\
       A9 <> PosInf /\ A10 <> PosInf /\ A11 <> PosInf /\
       A12 <> PosInf /\ A13 <> PosInf /\ A14 <> PosInf /\
       A15 <> PosInf /\ A16 <> PosInf /\ A17 <> PosInf /\
       A18 <> PosInf /\ A19 <> PosInf /\ A20 <> PosInf /\
       A21 <> PosInf /\ A22 <> PosInf /\ A23 <> PosInf /\
       A24 <> PosInf /\ A25 <> PosInf /\ A26 <> PosInf /\
       A27 <> PosInf /\ A28 <> PosInf /\ A29 <> PosInf /\
       A30 <> PosInf /\ A31 <> PosInf /\ A32 <> PosInf /\
       A33 <> PosInf /\ A34 <> PosInf /\ A35 <> PosInf /\
       A36 <> PosInf /\ A37 <> PosInf /\ A38 <> PosInf /\
       A39 <> PosInf /\ A40 <> PosInf /\ A41 <> PosInf /\
       A42 <> PosInf /\ A43 <> PosInf /\ A44 <> PosInf /\
       A45 <> PosInf /\ A46 <> PosInf /\ A47 <> PosInf /\
       A48 <> PosInf /\ A49 <> PosInf /\ A50 <> PosInf ==>
       (A1 +
        (A2 +
         (A3 +
          (A4 +
           (A5 +
            (A6 +
             (A7 +
              (A8 +
               (A9 +
                (A10 +
                 (A11 +
                  (A12 +
                   (A13 +
                    (A14 +
                     (A15 +
                      (A16 +
                       (A17 +
                        (A18 +
                         (A19 +
                          (A20 +
                           (A21 +
                            (A22 +
                             (A23 +
                              (A24 +
                               (A25 +
                                (A26 +
                                 (A27 +
                                  (A28 +
                                   (A29 +
                                    (A30 +
                                     (A31 +
                                      (A32 +
                                       (A33 +
                                        (A34 +
                                         (A35 +
                                          (A36 +
                                           (A37 +
                                            (A38 +
                                             (A39 +
                                              (A40 +
                                               (A41 +
                                                (A42 +
                                                 (A43 +
                                                  (A44 +
                                                   (A45 +
                                                    (A46 +
                                                     (A47 +
                                                      (A48 +
                                                       (A49 +
                                                        A50)))))))))))))))))))))))))))))))))))))))))))))))) =
        A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9 + A10 + A11 + A12 +
        A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 +
        A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 +
        A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 +
        A43 + A44 + A45 + A46 + A47 + A48 + A49 + A50)
 ``,

RW_TAC std_ss [add_assoc_40_var]
THEN KNOW_TAC(``(A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A10 +
          (A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
    THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A9 +
          (A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A9+  A10 + A11 + A12 + A13 +
           A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
 	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A8 +
          (A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A8+A9+  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
    THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A7 +
          (A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A7+A8+A9+  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A6 +
          (A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A6+A7+A8+A9+  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A5 +
          (A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A5+A6+A7+A8+A9+  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A4 +
          (A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A4+A5+A6+A7+A8+A9+  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
          A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A3 +
          (A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A3+A4+A5+A6+A7+A8+A9+  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20	      + A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1(METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A2 +
          (A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 +
           A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
           A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
           A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
           A50)) =A2+A3+A4+A5+A6+A7+A8+A9+  A10 + A11 + A12 + A13 +
	   A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	   A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	   A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	   A50``)
     THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A2+A3+A4+A5+A6+A7+A8+A9+A10+A11 + A12 + A13 + A14 + A15 + A16 + A17 + A18 + A19 +	      A20 + A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
          A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
          A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
          A50) <> PosInf``)
     THEN1( METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
THEN RW_TAC std_ss []
THEN METIS_TAC[ add_not_infty]);




val  add_assoc_63_var = store_thm("add_assoc_63_var",
  `` 


     !A1 A2 A3 A4 A5 A6 A7 A8 A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19
        A20 A21 A22 A23 A24 A25 A26 A27 A28 A29 A30 A31 A32 A33 A34 A35
        A36 A37 A38 A39 A40 A41 A42 A43 A44 A45 A46 A47 A48 A49 A50 A51
        A52 A53 A54 A55 A56 A57 A58 A59 A60 A61 A62 A63.
       A1 <> PosInf /\ A2 <> PosInf /\ A3 <> PosInf /\ A4 <> PosInf /\
       A5 <> PosInf /\ A6 <> PosInf /\ A7 <> PosInf /\ A8 <> PosInf /\
       A9 <> PosInf /\ A10 <> PosInf /\ A11 <> PosInf /\
       A12 <> PosInf /\ A13 <> PosInf /\ A14 <> PosInf /\
       A15 <> PosInf /\ A16 <> PosInf /\ A17 <> PosInf /\
       A18 <> PosInf /\ A19 <> PosInf /\ A20 <> PosInf /\
       A21 <> PosInf /\ A22 <> PosInf /\ A23 <> PosInf /\
       A24 <> PosInf /\ A25 <> PosInf /\ A26 <> PosInf /\
       A27 <> PosInf /\ A28 <> PosInf /\ A29 <> PosInf /\
       A30 <> PosInf /\ A31 <> PosInf /\ A32 <> PosInf /\
       A33 <> PosInf /\ A34 <> PosInf /\ A35 <> PosInf /\
       A36 <> PosInf /\ A37 <> PosInf /\ A38 <> PosInf /\
       A39 <> PosInf /\ A40 <> PosInf /\ A41 <> PosInf /\
       A42 <> PosInf /\ A43 <> PosInf /\ A44 <> PosInf /\
       A45 <> PosInf /\ A46 <> PosInf /\ A47 <> PosInf /\
       A48 <> PosInf /\ A49 <> PosInf /\ A50 <> PosInf /\
       A51 <> PosInf /\ A52 <> PosInf /\ A53 <> PosInf /\
       A54 <> PosInf /\ A54 <> PosInf /\ A55 <> PosInf /\
       A56 <> PosInf /\ A57 <> PosInf /\ A58 <> PosInf /\
       A59 <> PosInf /\ A60 <> PosInf /\ A61 <> PosInf /\
       A62 <> PosInf /\ A63 <> PosInf ==>
       (A1 +
        (A2 +
         (A3 +
          (A4 +
           (A5 +
            (A6 +
             (A7 +
              (A8 +
               (A9 +
                (A10 +
                 (A11 +
                  (A12 +
                   (A13 +
                    (A14 +
                     (A15 +
                      (A16 +
                       (A17 +
                        (A18 +
                         (A19 +
                          (A20 +
                           (A21 +
                            (A22 +
                             (A23 +
                              (A24 +
                               (A25 +
                                (A26 +
                                 (A27 +
                                  (A28 +
                                   (A29 +
                                    (A30 +
                                     (A31 +
                                      (A32 +
                                       (A33 +
                                        (A34 +
                                         (A35 +
                                          (A36 +
                                           (A37 +
                                            (A38 +
                                             (A39 +
                                              (A40 +
                                               (A41 +
                                                (A42 +
                                                 (A43 +
                                                  (A44 +
                                                   (A45 +
                                                    (A46 +
                                                     (A47 +
                                                      (A48 +
                                                       (A49 +
                                                        (A50 +
                                                         (A51 +
                                                          (A52 +
                                                           (A53 +
                                                            (A54 +
                                                             (A55 +
                                                              (A56 +
                                                               (A57 +
                                                                (A58 +
                                                                 (A59 +
                                                                  (A60 +
                                                                   (A61 +
                                                                    (A62 +
                                                                     A63))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) =
        A1 + A2 + A3 + A4 + A5 + A6 + A7 + A8 + A9 + A10 + A11 + A12 +
        A13 + A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 +
        A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 +
        A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 +
        A43 + A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 +
        A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 + A63)

 ``,

RW_TAC std_ss [add_assoc_50_var]
THEN KNOW_TAC(``(A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
      THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A13 +
            (A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) = A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
      THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A12 +
            (A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) = A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1 (DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss [] THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
      THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A11 +
            (A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) = A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
       THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
       THEN RW_TAC std_ss []
       THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
     THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A10 +
            (A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) = A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1( DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
      THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A9 +
            (A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
      THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A8 +
            (A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A8+A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
     THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A7 +
            (A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A7+A8+A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23
             + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
     THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A6 +
            (A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A6+A7+A8+A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 +              A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
     THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A5 +
            (A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A5+A6+A7+A8+A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
       THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
       THEN RW_TAC std_ss []
       THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A5+A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22              + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
     THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A4 +
            (A5+A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 +
             A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A4+A5+A6+A7+A8+A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A4+A5+A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 +		 A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
     THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A3 +
            (A4+A5+A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 +             A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A3+A4+A5+A6+A7+A8+A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A3+A4+A5+A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21             + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
      THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``(A2 +
            (A3+A4+A5+A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 +
	     A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63)) =A2+A3+A4+A5+A6+A7+A8+A9+ A10+A11+A12+A13 +
	     A14 + A15 + A16 + A17 + A18 + A19 + A20 + A21 + A22 + A23 + A24 + A25 +
	     A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
	     A38 + A39 + A40 + A41 + A42 + A43 + A44 + A45 + A46 + A47 + A48 + A49 +
	     A50 + A51 + A52 + A53 + A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 +
	     A62 + A63``)
      THEN1(DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC[ add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(A2+A3+A4+A5+A6+A7+A8+A9+A10+A11+A12+A13+A14 + A15 + A16 + A17 + A18 + A19 + A20 +		 A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 +
             A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 + A42 + A43 +
             A44 + A45 + A46 + A47 + A48 + A49 + A50 + A51 + A52 + A53 +
             A54 + A55 + A56 + A57 + A58 + A59 + A60 + A61 + A62 +
             A63) <> PosInf``)
      THEN1 (METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[add_assoc,extreal_hvgTheory.add_not_infty]
THEN RW_TAC std_ss []
THEN METIS_TAC[ add_not_infty]
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[] );


val PIE_lem4_63_prod = store_thm ( "PIE_lem4_63_prod",
  `` !f A B C D E k. ALL_DISTINCT [A;B;C; D; E; k] /\
       	     (!x.
         x IN {{A}; {B}; {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } ==>
         f x <> PosInf) ==> 
     (EXTREAL_SUM_IMAGE f {{A}; {B}; {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } =		

  f {A} + f{B}+ f {C} + f {D} +f{E} +f {k}+ f {A; B} +f {A; C} +f {A;D}  +f {A;E} +f{A;k}+f {B; C}+f {B;D} +f {B;E}+f {B;k}+f {C;D}+f {C;E}+f {C;k}+f{D;E}+f{D;k}+f{E;k}+f {A; B; C}+f {A;B;D} +f{A;B;E}+f{A;B;k} +f {A;C;D}+ f{A;C;E}+f{A;C;k}+f{A;D;E}+f{A;D;k}+f{A;E;k}+f{B;C;D}+f{B;C;E}+f{B;C;k}+f{B;D;E}+f{B;D;k}+f{B;E;k}+f{C;D;E}+f{C;D;k}+f{C;E;k}+f{D;E;k}+f{A;B;C;D}+f{A;B;C;E}+f{A;B;C;k}+f{A;B;D;E}+f{A;B;D;k}+f  {A;B;E;k}+f {A;C;D;E}+f{ A;C;D;k}+f {A;C;E;k}+f {A;D;E;k}+f  {B;C;D;E}+f {B;C;D;k} +f {B;C;E;k} +f {B;D;E;k} +f{C;D;E;k} +f{A;B;C;D;E}+f {A;B;C;D;k}+f {A;B;C;E;k} +f{A;B;D;E;k} +f{A;C;D;E;k}+f {B;C;D;E;k}+f {A;B;C;D;E;k} )``,

RW_TAC std_ss[ALL_DISTINCT,MEM]
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{B}:'a->bool; {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{B}; {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss[prove(``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (( {{B}; {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A}) = ( {{B}; {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]

(* solving for B*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{C}:'a->bool; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss[prove(``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (( { {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B}) = ( { {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{C}:'a->bool; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss[prove(``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (( { {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B}) = ( { {C}; {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]


THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  {{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN WEAKEN_TAC (equal (Term `!x.
        x IN {{B:'a}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(*  solving for C *)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{D:'a};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss[prove(``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (( {  {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C}) = ( { {D};{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {{C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)


THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {C:'a}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* solving for D*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{E:'a}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {D}) = ( { {E}; {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN { {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {D:'a}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* solving for E*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{k:'a}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {E}) = ( { {k}; {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {E:'a}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* solving for {k} *)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A; B} :'a->bool; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {k}) = ( { {A; B}; {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN { {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {k:'a}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* solving for A B*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A; C}:'a->bool; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;B}) = ( { {A; C}; {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {A:'a; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))




(* solving for {A C}*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {A;D}:'a->bool ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;C}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({   {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;C}) = ( { {A;D} ; {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {A:'a; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))



(* solving for {A;D}*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;E}:'a->bool ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {  {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({    {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;D}) = ( { {A;E} ;{A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {A:'a; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))






(* solvong for A E*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;k}:'a->bool; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {  {A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;E}) = ( { {A;k}; {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  {  {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { {A:'a; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))




(*solving for A K*)


THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {B; C} :'a->bool; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {   {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;k}) = ( {  {B; C}; {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           {A:'a; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(*solving for B C*)


THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {B;D}:'a->bool ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {    {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({   {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B;C}) = ( {  {B;D} ; {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           {B:'a; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* solving for B D*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {B;E}:'a->bool; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {   {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B;D}) = ( {   {B;E}; {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
           {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           {B:'a; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(*solving for B E *)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {B;k}:'a->bool; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {   {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B;E}) = ( {   {B;k}; {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            {B:'a; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* solving for B K*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{    {C;D} :'a->bool; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {    {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B;k}) = ( {    {C;D}; {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
             {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
             {B:'a; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

 


(* soving for C D*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{    {C;E}:'a->bool; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {    {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C;D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C;D}) = ( {     {C;E}; {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
              {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
             {C:'a; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* soving for C E*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{    {C;k} :'a->bool;{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {     {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C;E}) = ( {      {C;k};{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
            {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
             {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
             {C:'a; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* solving for C k*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {D;E}:'a->bool;{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {    {D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C;k}) = ( {  {D;E};{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
           {C:'a; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))





(* solving for D E*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {D;k}:'a->bool;{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {    {D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{D;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {D;E}) = ( {  {D;k};{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            {D:'a; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))



(* solving for D k*)


THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {E;k}:'a->bool; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {    {E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{D;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {D;k}) = ( {  {E;k}; {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
           {D:'a; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))



(* solving for E  k*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {A; B; C}:'a->bool; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {   {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {E;k}) = ( {   {A; B; C}; {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
           {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
           {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
           {E:'a; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))



(* solving for A B C*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;B;D}:'a->bool;{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {   {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; B; C}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; C}) = ( {    {A;B;D};{A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
           {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
           {A:'a; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))




(* solve for A B D*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A;B;E}:'a->bool;{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; B; D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; D}) = ( {   {A;B;E};{A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
          {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
         {A:'a; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* solve for A B E*)
THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A;B;k}:'a->bool;{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; B; E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; E}) = ( { {A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
           {A:'a; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* solve for A B E*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A;B;k}:'a->bool;{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; B; E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; E}) = ( { {A;B;k};{A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
           {A:'a; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* solving for A B k*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A;C;D}:'a->bool;{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; B; k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; k}) = ( { {A;C;D};{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A;B;k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
           {A;B;k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            {A:'a;B;k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* solving for A C D*)


THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A;C;E}:'a->bool;{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; C; D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; C; D}) = ( { {A;C;E};{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
           {A:'a; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            {A:'a; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A C E*)


THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A;C;k}:'a->bool;{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; C; E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; C; E}) = ( { {A;C;k};{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            {A:'a; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            {A:'a; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))



(* A C K*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;D;E}:'a->bool;{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; C; k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; C; k}) = ( { {A;D;E};{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            {A:'a; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* A D E*)
THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;D;k}:'a->bool;{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; D; E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; D; E}) = ( { {A;D;k};{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
                     {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
           {A:'a; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* A D k*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;E;k}:'a->bool;{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A; D; k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; D; k}) = ( {{A;E;k};{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
          {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
            {A:'a; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A E k*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{B;C;D}:'a->bool;{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;E; k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; E; k}) = ( {{B;C;D};{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
           {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
         {A:'a; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* B C D*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{B;C;E}:'a->bool;{B; C; k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C;D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; C; D}) = ( {{B;C;E};{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
            {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
            {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
        {B:'a; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* B C E*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{B;C;k}:'a->bool;{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C; E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; C; E}) = ( {{B;C;k};{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
             {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
             {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
         {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))






(* B C k*)



THEN 
 MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{B;D;E}:'a->bool;{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C; k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; C; k}) = ( {{B;D;E};{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
           
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
       
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(*B D E*)

 THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{B;D;k}:'a->bool;{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;D; E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; D; E}) = ( {{B;D;k};{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           
            {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
           
            {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
       
            {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))
(* B D k*)


 THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{B;E;k}:'a->bool;{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;D; k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; D; k}) = ( {{B;E;k};{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           
           {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
           
           {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
       
          {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* B E k*)

 THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{C;D;E}:'a->bool;{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; E; k}) = ( {{C;D;E};{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           
           {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
           
          {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
       
           {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* C D E*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{C;D;k}:'a->bool;{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C;D;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C; D; E}) = ( {{C;D;k};{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          
           
            {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
            
            
           
          {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
            
       
           {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* C D k*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{C;E;k}:'a->bool;{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C;D;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C; D; k}) = ( {{C;E;k};{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* C E k*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{D;E;k}:'a->bool;{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C; E; k}) = ( {{D;E;k};{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
        {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
     {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
             {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* D E K*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;B;C;D}:'a->bool;{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{D;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {D; E; k}) = ( {{A;B;C;D};{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
         {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
      {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
              {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A B C D*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;B;C;E}:'a->bool;{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;C;D}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; C;D}) = ( {{A;B;C;E};{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
         {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
           {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
            {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A B C E*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;B;C;k}:'a->bool;{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;C;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; C;E}) = ( {{A;B;C;k};{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
          {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
           {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A B C k*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;B;D;E}:'a->bool;{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;C;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; C;k}) = ( {{A;B;D;E};{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A B D E*)


THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{{A;B;D;k}:'a->bool;  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;D;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; D;E}) = ( {{A;B;D;k};  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
           {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
           {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
           {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A B D K*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;B;E;k}:'a->bool; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;D;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; D;k}) = ( {  {A;B;E;k}; {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
            {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
            {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
            {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A B E K*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {A;C;D;E}:'a->bool;{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {   {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({   {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; B; E;k}) = ( {   {A;C;D;E};{ A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
          {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
          {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
          {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A C D E*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   { A;C;D;k}:'a->bool; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {  { A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;C;D;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({   { A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; C; D;E}) = ( {  { A;C;D;k}; {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
                 {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
                 {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
      
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A C D K*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {A;C;E;k}:'a->bool; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {   {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;C;D;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({   {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; C; D;k}) = ( {   {A;C;E;k}; {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
            {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
            {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
      
           {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A C E K*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {A;D;E;k}:'a->bool;  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {    {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;C;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({    {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; C; E;k}) = ( {  {A;D;E;k};  {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
             {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
            {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
       {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A D E K*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{    {B;C;D;E}:'a->bool; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {     {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;D;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({     {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A; D; E;k}) = ( {   {B;C;D;E}; {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
           {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
        {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
        {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))
(*B C D E*)

THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{    {B;C;D;k}:'a->bool ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C;D;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({   {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; C;D;E}) = ( {  {B;C;D;k} ; {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
      
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
                 {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
      
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* B C D K*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{    {B;C;E;k}:'a->bool ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C;D;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({    {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B; C;D;k}) = ( {  {B;C;E;k} ; {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
      
           {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
               {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
      
          {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* B C E K*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{    {B;D;E;k}:'a->bool ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {  {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B;C;E;k}) = ( {  {B;D;E;k} ;{C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
      
           {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
          {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
      
           {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* B D E K*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {C;D;E;k}:'a->bool ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;D;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {B;D;E;k}) = ( {  {C;D;E;k} ;{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
      
           {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         
           {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         
      
          {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* C D E K*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;B;C;D;E}:'a->bool; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {{A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{C;D;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {C;D;E;k}) = ( { {A;B;C;D;E}; {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
      
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* A B C D E *)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;B;C;D;k}:'a->bool; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;C;D;E}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;B;C;D;E}) = ( {  {A;B;C;D;k}; {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
      
            {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
         {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
 {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* A B C D k*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{ {A;B;C;E;k}:'a->bool ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {  {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;C;D;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;B;C;D;k}) = ( {  {A;B;C;E;k} ;{A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
      
      {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
      {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
 {A:'a; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(* A B C E K*)
THEN  MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;B;D;E;k}:'a->bool ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;C;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;B;C;E;k}) = ( {  {A;B;D;E;k} ;{A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
               {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
      
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))



(* A B D E K*)

THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;C;D;E;k}:'a->bool; {B;C;D;E;k}; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;B;D;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({ {A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;B;D;E;k}) = ( { {A;C;D;E;k}; {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
          {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
      
          {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
         {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))

(* repeat from here*)
(* A C D E K*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{   {B;C;D;E;k}:'a->bool; {A;B;C;D;E;k}   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE { {B;C;D;E;k}; {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{A;C;D;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({  {B;C;D;E;k}; {A;B;C;D;E;k}   } DELETE {A;C;D;E;k}) = ( {  {B;C;D;E;k}; {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
           {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by     
  (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
      
           {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))




(* B C D E K*)
THEN MP_TAC(Q.ISPECL [`f:('a->bool)->extreal`,`{  {A;B;C;D;E;k}:'a->bool   }`] EXTREAL_SUM_IMAGE_PROPERTY_POS)


THEN FULL_SIMP_TAC std_ss[prove(``!A B C D E k . FINITE {  {A;B;C;D;E;k}   }``, SRW_TAC[][FINITE_SING])]
THEN DISCH_THEN (MP_TAC o Q.SPECL [`{B;C;D;E;k}:'a->bool`])
THEN FULL_SIMP_TAC std_ss[]


THEN RW_TAC std_ss [ prove (``!A B C D E k.( A <> B /\ A <> C /\ A <> D /\ A <> E /\ A <> k /\ B <> C /\ B <> D /\ B <> E /\ B <> k /\ C <> D /\ C <> E /\ C <> k /\ D <> E /\ D <> k /\ E <> k)  ==> (({   {A;B;C;D;E;k}   } DELETE {B;C;D;E;k}) = ( {  {A;B;C;D;E;k}   }))``, SRW_TAC[][EXTENSION,GSPECIFICATION,ALL_DISTINCT,MEM] THEN EQ_TAC THEN RW_TAC std_ss[]  THEN METIS_TAC[])]




THEN POP_ASSUM MP_TAC
THEN `(!x.
         x IN {  
       
          
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)` by 
         (MATCH_MP_TAC (prove(``!A B C D E k f. (!x.
             x IN {{A};{B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
           {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
           {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D};
           {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
             f x <> PosInf) ==> (!x.
         x IN  { 
      
        
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
         f x <> PosInf)``, SRW_TAC[][EXTENSION,GSPECIFICATION] THEN  RW_TAC std_ss [] THEN METIS_TAC[]))
      THEN RW_TAC std_ss[])
      
THEN FULL_SIMP_TAC std_ss[]
THEN REPEAT (WEAKEN_TAC is_eq)

THEN WEAKEN_TAC (equal (Term `!x.
        x IN { 
           
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==> f x <> PosInf`))


(*solving for A B C D E k*)

THEN RW_TAC std_ss[EXTREAL_SUM_IMAGE_SING]
THEN DEP_REWRITE_TAC[ add_assoc_63_var]



THEN WEAKEN_TAC is_eq THEN

WEAKEN_TAC (equal (Term `(!x.
          x IN
          {{A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} ==>
          f x <> PosInf) ==>
       (SIGMA f
          {{A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k};
           {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E};
           {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
           {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
           {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k};
           {A; C; D; E}; {A; C; D; k}; {A; C; E; k}; {A; D; E; k};
           {B; C; D; E}; {B; C; D; k}; {B; C; E; k}; {B; D; E; k};
           {C; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
           {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
           {B; C; D; E; k}; {A; B; C; D; E; k}} =
        f {A; B; E} +
        (f {A; B; k} +
         (f {A; C; D} +
          (f {A; C; E} +
           (f {A; C; k} +
            (f {A; D; E} +
             (f {A; D; k} +
              (f {A; E; k} +
               (f {B; C; D} +
                (f {B; C; E} +
                 (f {B; C; k} +
                  (f {B; D; E} +
                   (f {B; D; k} +
                    (f {B; E; k} +
                     (f {C; D; E} +
                      (f {C; D; k} +
                       (f {C; E; k} +
                        (f {D; E; k} +
                         (f {A; B; C; D} +
                          (f {A; B; C; E} +
                           (f {A; B; C; k} +
                            (f {A; B; D; E} +
                             (f {A; B; D; k} +
                              (f {A; B; E; k} +
                               (f {A; C; D; E} +
                                (f {A; C; D; k} +
                                 (f {A; C; E; k} +
                                  (f {A; D; E; k} +
                                   (f {B; C; D; E} +
                                    (f {B; C; D; k} +
                                     (f {B; C; E; k} +
                                      (f {B; D; E; k} +
                                       (f {C; D; E; k} +
                                        (f {A; B; C; D; E} +
                                         (f {A; B; C; D; k} +
                                          (f {A; B; C; E; k} +
                                           (f {A; B; D; E; k} +
                                            (f {A; C; D; E; k} +
                                             (f {B; C; D; E; k} +
                                              SIGMA f
                                                {{A; B; C; D; E;
                                                  k}})))))))))))))))))))))))))))))))))))))))`))


THEN POP_ASSUM MP_TAC THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]);




val POW_set_2 = store_thm("POW_set_2",
  ``      !A1 A2. POW {A1; A2} = {{}; {A1}; {A2}; {A1; A2}} ``,
  RW_TAC std_ss [POW_DEF]
  THEN RW_TAC std_ss [SUBSET_DEF]
  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
  THEN METIS_TAC[NOT_IN_EMPTY]);

val POW_set_3 = store_thm("POW_set_3",
  ``  !A1 A2 A3.
       POW {A1; A2; A3} =
       {{}; {A1}; {A2}; {A3}; {A1; A2}; {A1; A3}; {A2; A3};
        {A1; A2; A3}} ``,


ONCE_REWRITE_TAC [POW_INSERT]
THEN RW_TAC std_ss [POW_set_2]
THEN KNOW_TAC (``IMAGE ($INSERT A1)  {{}; {A2}; {A3}; {A2; A3}} = {{A1};{A1;A2};{A1;A3};{A1;A2;A3}}                   ``)
    THEN1(SRW_TAC [][GSPECIFICATION, EXTENSION]
          THEN RW_TAC std_ss[]
	  THEN EQ_TAC
	       THEN1(METIS_TAC[])
	  THEN RW_TAC std_ss[]
	       THEN1(Q.EXISTS_TAC`{}`
	             THEN METIS_TAC[NOT_IN_EMPTY])
  	       THEN1(Q.EXISTS_TAC`{A2}` THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	             THEN METIS_TAC[NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC`{A3}`
	       THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	       THEN METIS_TAC[NOT_IN_EMPTY])
	  THEN Q.EXISTS_TAC`{A2;A3}`
	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	  THEN METIS_TAC[NOT_IN_EMPTY])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN METIS_TAC[]);

val  POW_set_4 = store_thm("POW_set_4",
  `` 
     !A B C D.
       POW {A; B; C; D} =
       {{}; {A}; {B}; {C}; {D}; {A; B}; {A; C}; {A; D}; {B; C}; {B; D};
        {C; D}; {A; B; C}; {A; B; D}; {A; C; D}; {B; C; D};
        {A; B; C; D}} ``,

RW_TAC std_ss[]
THEN ONCE_REWRITE_TAC [POW_INSERT]
THEN RW_TAC std_ss [POW_set_3]
THEN KNOW_TAC(``IMAGE ($INSERT A)
  {{}; {B}; {C}; {D}; {B; C}; {B; D}; {C; D}; {B; C; D}} = 
       {{A}; {A;B}; {A;C}; {A;D}; {A;B; C}; {A;B; D}; {A;C; D}; {A;B; C; D}}``)
     THEN1(SRW_TAC [][GSPECIFICATION, EXTENSION]
           THEN1 (RW_TAC std_ss[]
	          THEN EQ_TAC
		       THEN1(METIS_TAC[])
		  THEN RW_TAC std_ss[]
		       THEN1(Q.EXISTS_TAC`{}`
		             THEN METIS_TAC[NOT_IN_EMPTY])
		       THEN1(Q.EXISTS_TAC`{B}`
		             THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
			     THEN METIS_TAC[NOT_IN_EMPTY])
		       THEN1(Q.EXISTS_TAC`{C}`
		             THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		             THEN METIS_TAC[NOT_IN_EMPTY])
		       THEN1(Q.EXISTS_TAC`{D}`
		             THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
			     THEN METIS_TAC[NOT_IN_EMPTY])
		       THEN1(Q.EXISTS_TAC`{B;C}`
		             THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
			     THEN METIS_TAC[NOT_IN_EMPTY])
                       THEN1(Q.EXISTS_TAC`{B;D}`
		             THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		             THEN METIS_TAC[NOT_IN_EMPTY])
		       THEN1(Q.EXISTS_TAC`{C;D}`
		             THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
			     THEN METIS_TAC[NOT_IN_EMPTY])
		       THEN  Q.EXISTS_TAC`{C;D;B}`
		             THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
			     THEN METIS_TAC[NOT_IN_EMPTY]))
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(METIS_TAC[])
THEN RW_TAC std_ss[]
THEN METIS_TAC[]);


val POW_set_5 = store_thm("POW_set_5",
  ``  !A B C D E.
       POW {A; B; C; D; E} =
       {{}; {A}; {B}; {C}; {D}; {E}; {A; B}; {A; C}; {A; D}; {A; E};
        {B; C}; {B; D}; {B; E}; {C; D}; {C; E}; {D; E}; {A; B; C};
        {A; B; D}; {A; B; E}; {A; C; D}; {A; C; E}; {A; D; E};
        {B; C; D}; {B; C; E}; {B; D; E}; {C; D; E}; {A; B; C; D};
        {A; B; C; E}; {A; B; D; E}; {A; C; D; E}; {B; C; D; E};
        {A; B; C; D; E}}
 ``,


ONCE_REWRITE_TAC [POW_INSERT]
THEN RW_TAC std_ss [POW_set_4]
THEN KNOW_TAC(``IMAGE ($INSERT A)
  {{}; {B}; {C}; {D}; {E}; {B; C}; {B; D}; {B; E}; {C; D}; {C; E};
   {D; E}; {B; C; D}; {B; C; E}; {B; D; E}; {C; D; E}; {B; C; D; E}} =
{{A}; {A; B}; {A; C}; {A; D}; {A; E}; {A; B; C}; {A; B; D}; {A; B; E};
 {A; C; D}; {A; C; E}; {A; D; E}; {A; B; C; D}; {A; B; C; E};
 {A; B; D; E}; {A; C; D; E}; {A; B; C; D; E}}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
          THEN EQ_TAC
	       THEN1(METIS_TAC[])
          THEN RW_TAC std_ss[]
	       THEN1( Q.EXISTS_TAC `{}`
	              THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{B}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1 (Q.EXISTS_TAC `{C}` THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
	              THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{D}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{E}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{B;C}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{B;D}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{B;E}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{C;D}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
               THEN1(Q.EXISTS_TAC `{C;E}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{D;E}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{B;C;D}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
               THEN1(Q.EXISTS_TAC `{B;C;E}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	       THEN1(Q.EXISTS_TAC `{B;D;E}`
         	      THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
               THEN1(Q.EXISTS_TAC `{C;D;E}`
	             THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
		     THEN METIS_TAC [NOT_IN_EMPTY])
	THEN Q.EXISTS_TAC `{B;C;D;E}`
	THEN SRW_TAC [] [GSPECIFICATION , EXTENSION]
	THEN METIS_TAC [NOT_IN_EMPTY])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN RW_TAC std_ss[]
THEN METIS_TAC[]);


val POW_set_6 = store_thm("POW_set_6",
  ``! A B C D E k.  POW {A ; B; C; D; E; k}  =  
{{};{A}; {B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
         {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E}; {C; k};
         {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D}; {A; B; E};
         {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k}; {A; D; E};
         {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E}; {B; C; k};
         {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E}; {C; D; k};
         {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E}; {A; B; C; k};
         {A; B; D; E}; {A; B; D; k}; {A; B; E; k}; {A; C; D; E};
         {A; C; D; k}; {A; C; E; k}; {A; D; E; k}; {B; C; D; E};
         {B; C; D; k}; {B; C; E; k}; {B; D; E; k}; {C; D; E; k};
         {A; B; C; D; E}; {A; B; C; D; k}; {A; B; C; E; k};
         {A; B; D; E; k}; {A; C; D; E; k}; {B; C; D; E; k};
         {A; B; C; D; E; k}}``,


ONCE_REWRITE_TAC [POW_INSERT]
THEN RW_TAC std_ss [POW_set_5]
THEN KNOW_TAC(`` IMAGE ($INSERT A)
  {{}; {B}; {C}; {D}; {E}; {k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D};
   {C; E}; {C; k}; {D; E}; {D; k}; {E; k}; {B; C; D}; {B; C; E};
   {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E}; {C; D; k};
   {C; E; k}; {D; E; k}; {B; C; D; E}; {B; C; D; k}; {B; C; E; k};
   {B; D; E; k}; {C; D; E; k}; {B; C; D; E; k}} =
{{A}; {A; B}; {A; C}; {A; D}; {A; E}; {A; k}; {A; B; C}; {A; B; D};
 {A; B; E}; {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k}; {A; D; E};
 {A; D; k}; {A; E; k}; {A; B; C; D}; {A; B; C; E}; {A; B; C; k};
 {A; B; D; E}; {A; B; D; k}; {A; B; E; k}; {A; C; D; E}; {A; C; D; k};
 {A; C; E; k}; {A; D; E; k}; {A; B; C; D; E}; {A; B; C; D; k};
 {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k}; {A; B; C; D; E; k}}
``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EQ_TAC
	        THEN1(METIS_TAC[])
	   THEN RW_TAC std_ss[]
	        THEN1(Q.EXISTS_TAC `{}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{B}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{C}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1 (Q.EXISTS_TAC `{D}`
		       THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		       THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{E}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{B;C}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{B;D}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{B;E}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{C;D}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{C;E}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{C;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{D;E}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{D;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{E;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;C;D}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{B;C;E}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;C;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;D;E}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;D;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
	        THEN1(Q.EXISTS_TAC `{B;E;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1( Q.EXISTS_TAC `{C;D;E}`
		       THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		       THEN METIS_TAC [NOT_IN_EMPTY])
                THEN1(Q.EXISTS_TAC `{C;D;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{C;E;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{D;E;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;C;D;E}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;C;D;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;C;E;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{B;D;E;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
		THEN1(Q.EXISTS_TAC `{C;D;E;k}`
		      THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		      THEN METIS_TAC [NOT_IN_EMPTY])
        THEN  Q.EXISTS_TAC `{B;C;D;E;k}`
	THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	THEN METIS_TAC [NOT_IN_EMPTY])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss []
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	   THEN  METIS_TAC[])
THEN RW_TAC std_ss []
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN  METIS_TAC[]);

val POW_DELETE_EMPTY = store_thm("POW_DELETE_EMPTY",
  `` !s. POW s DELETE {} = {t|t SUBSET s /\ t <> {}}``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, POW_DEF]
THEN METIS_TAC[]);


val POW_set_6_DELETE_EMPTY = store_thm("POW_set_6_DELETE_EMPTY",
  `` ! A B C D E k. POW {A; B; C; D; E; k} DELETE {} = 
{ {A}; {B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
      {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E}; {C; k};
      {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D}; {A; B; E};
      {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k}; {A; D; E}; {A; D; k};
      {A; E; k}; {B; C; D}; {B; C; E}; {B; C; k}; {B; D; E}; {B; D; k};
      {B; E; k}; {C; D; E}; {C; D; k}; {C; E; k}; {D; E; k};
      {A; B; C; D}; {A; B; C; E}; {A; B; C; k}; {A; B; D; E};
      {A; B; D; k}; {A; B; E; k}; {A; C; D; E}; {A; C; D; k};
      {A; C; E; k}; {A; D; E; k}; {B; C; D; E}; {B; C; D; k};
      {B; C; E; k}; {B; D; E; k}; {C; D; E; k}; {A; B; C; D; E};
      {A; B; C; D; k}; {A; B; C; E; k}; {A; B; D; E; k};
      {A; C; D; E; k}; {B; C; D; E; k}; {A; B; C; D; E; k}}``,

RW_TAC std_ss [POW_set_6]
THEN PSET_TAC[DELETE_DEF]
THEN KNOW_TAC(``{{}; {A}; {B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
 {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E}; {C; k}; {D; E};
 {D; k}; {E; k}; {A; B; C}; {A; B; D}; {A; B; E}; {A; B; k}; {A; C; D};
 {A; C; E}; {A; C; k}; {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D};
 {B; C; E}; {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
 {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
 {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k}; {A; C; D; E};
 {A; C; D; k}; {A; C; E; k}; {A; D; E; k}; {B; C; D; E}; {B; C; D; k};
 {B; C; E; k}; {B; D; E; k}; {C; D; E; k}; {A; B; C; D; E};
 {A; B; C; D; k}; {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
 {B; C; D; E; k}; {A; B; C; D; E; k}} = {{A}; {B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D}; {A; E};
 {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E}; {C; k}; {D; E};
 {D; k}; {E; k}; {A; B; C}; {A; B; D}; {A; B; E}; {A; B; k}; {A; C; D};
 {A; C; E}; {A; C; k}; {A; D; E}; {A; D; k}; {A; E; k}; {B; C; D};
 {B; C; E}; {B; C; k}; {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E};
 {C; D; k}; {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E};
 {A; B; C; k}; {A; B; D; E}; {A; B; D; k}; {A; B; E; k}; {A; C; D; E};
 {A; C; D; k}; {A; C; E; k}; {A; D; E; k}; {B; C; D; E}; {B; C; D; k};
 {B; C; E; k}; {B; D; E; k}; {C; D; E; k}; {A; B; C; D; E};
 {A; B; C; D; k}; {A; B; C; E; k}; {A; B; D; E; k}; {A; C; D; E; k};
 {B; C; D; E; k}; {A; B; C; D; E; k};{}}``)

     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EQ_TAC
	        THEN1(DISCH_TAC
                     THEN RW_TAC std_ss []
		     THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		     THEN METIS_TAC[])
            THEN DISCH_TAC
	    THEN RW_TAC std_ss []
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	    THEN METIS_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(DISCH_TAC
           THEN RW_TAC std_ss []
	   THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	   THEN METIS_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss []
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN METIS_TAC[]);



val PIE_set_alt_form_lem_6 = store_thm("PIE_set_alt_form_lem_6",
  ``! A B C D E k. {t|t SUBSET {A;B;C;D;E;k} /\ t <> {}} = 
 { {A}; {B}; {C}; {D}; {E}; {k}; {A; B}; {A; C}; {A; D};
        {A; E}; {A; k}; {B; C}; {B; D}; {B; E}; {B; k}; {C; D}; {C; E};
        {C; k}; {D; E}; {D; k}; {E; k}; {A; B; C}; {A; B; D}; {A; B; E};
        {A; B; k}; {A; C; D}; {A; C; E}; {A; C; k}; {A; D; E};
        {A; D; k}; {A; E; k}; {B; C; D}; {B; C; E}; {B; C; k};
        {B; D; E}; {B; D; k}; {B; E; k}; {C; D; E}; {C; D; k};
        {C; E; k}; {D; E; k}; {A; B; C; D}; {A; B; C; E}; {A; B; C; k};
        {A; B; D; E}; {A; B; D; k}; {A; B; E; k}; {A; C; D; E};
        {A; C; D; k}; {A; C; E; k}; {A; D; E; k}; {B; C; D; E};
        {B; C; D; k}; {B; C; E; k}; {B; D; E; k}; {C; D; E; k};
        {A; B; C; D; E}; {A; B; C; D; k}; {A; B; C; E; k};
        {A; B; D; E; k}; {A; C; D; E; k}; {B; C; D; E; k};
        {A; B; C; D; E; k}}``,
RW_TAC std_ss [GSYM POW_DELETE_EMPTY]
THEN RW_TAC std_ss [POW_set_6_DELETE_EMPTY]);



val  extreal_sub_sub = store_thm("extreal_sub_sub",
 `` !x y.
       x <> PosInf /\ x <> NegInf /\ y <> PosInf /\ y <> NegInf ==>
       (-x + -y = -x - y) ``,

RW_TAC std_ss[]
THEN DEP_REWRITE_TAC [extreal_sub_add]
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [GSYM sub_lzero]
THEN METIS_TAC[sub_not_infty, lt_infty, le_infty, lt_le, le_lt, extreal_of_num_def]);



val extreal_add_sub_2_var = store_thm("extreal_add_sub_2_var",
  `` 
     !x y.
       x <> PosInf /\ y <> PosInf /\ x <> NegInf /\ y <> NegInf ==>
       x - y <> PosInf /\ x - y <> NegInf /\ -x - y <> PosInf /\
       -x - y <> NegInf /\ -x + -y <> PosInf /\ -x + -y <> NegInf ``,

RW_TAC std_ss[]
THEN METIS_TAC[extreal_sub_add, extreal_sub_sub,sub_not_infty, extreal_of_num_def, lt_infty, le_infty, lt_le, le_lt, minus_x_not_infty]);


val lemma_add_1 = store_thm("lemma_add_1",
  ``  !A57 A58 A59 A60 A61 A62 A63.
       A57 <> PosInf /\ A57 <> NegInf /\ A58 <> PosInf /\
       A58 <> NegInf /\ A59 <> PosInf /\ A59 <> NegInf /\
       A60 <> PosInf /\ A60 <> NegInf /\ A61 <> PosInf /\
       A61 <> NegInf /\ A62 <> PosInf /\ A62 <> NegInf /\
       A63 <> PosInf /\ A63 <> NegInf ==>
       (A57 + A58 + A59 + A60 + A61 + A62 + -A63 =
        A57 + A58 + A59 + A60 + A61 + A62 - A63)``,

RW_TAC std_ss []
THEN DEP_REWRITE_TAC [extreal_sub_add, add_not_infty, sub_not_infty]
THEN METIS_TAC [add_not_infty]);



val sub_lneg = store_thm
  ("sub_lneg",``!x y. (x <> NegInf /\ y <> NegInf) \/ (x <> PosInf /\ y <> PosInf) ==> (-x - y = -(x + y))``,
 REPEAT Cases THEN RW_TAC std_ss [extreal_sub_def,extreal_add_def,extreal_ainv_def,REAL_SUB_LNEG]);

val neg_sub = store_thm
  ("neg_sub",``!x y. (x <> NegInf /\ x <> PosInf) \/ (y <> NegInf /\ y <> PosInf) ==> (-(x - y) = y - x)``,
REPEAT Cases THEN RW_TAC std_ss [extreal_sub_def,extreal_ainv_def,REAL_NEG_SUB]);
;

val extreal_add_sub_63  = store_thm ("extreal_add_sub_63",``

!A1 A2 A3 A4 A5 A6 A7 A8 A9 A10 A11 A12 A13 A14 A15 A16 A17 A18 A19
        A20 A21 A22 A23 A24 A25 A26 A27 A28 A29 A30 A31 A32 A33 A34 A35
        A36 A37 A38 A39 A40 A41 A42 A43 A44 A45 A46 A47 A48 A49 A50 A51
        A52 A53 A54 A55 A56 A57 A58 A59 A60 A61 A62 A63.
       A1 <> PosInf /\ A1 <> NegInf /\ A2 <> PosInf /\ A2 <> NegInf /\
       A3 <> PosInf /\ A3 <> NegInf /\ A4 <> PosInf /\ A4 <> NegInf /\
       A5 <> PosInf /\ A5 <> NegInf /\ A6 <> PosInf /\ A6 <> NegInf /\
       A7 <> PosInf /\ A7 <> NegInf /\ A8 <> PosInf /\ A8 <> NegInf /\
       A9 <> PosInf /\ A9 <> NegInf /\ A10 <> PosInf /\ A10 <> NegInf /\
       A11 <> PosInf /\ A11 <> NegInf /\ A12 <> PosInf/\ A12 <> NegInf /\
       A13 <> PosInf /\
       A13 <> NegInf /\ A14 <> PosInf /\ A14 <> NegInf /\
       A15 <> PosInf /\ A15 <> NegInf /\ A16 <> PosInf /\
       A16 <> NegInf /\ A17 <> PosInf /\ A17 <> NegInf /\
       A18 <> PosInf /\ A18 <> NegInf /\ A19 <> PosInf /\
       A19 <> NegInf /\ A20 <> PosInf /\ A20 <> NegInf /\
       A21 <> PosInf /\ A21 <> NegInf /\ A22 <> PosInf /\
       A22 <> NegInf /\ A23 <> PosInf /\ A23 <> NegInf /\
       A24 <> PosInf /\ A24 <> NegInf /\ A25 <> PosInf /\
       A25 <> NegInf /\ A26 <> PosInf /\ A25 <> NegInf /\
       A26 <> PosInf /\ A26 <> NegInf /\ A27 <> PosInf /\
       A27 <> NegInf /\ A28 <> PosInf /\ A28 <> NegInf /\
       A29 <> PosInf /\ A29 <> NegInf /\ A30 <> PosInf /\
       A30 <> NegInf /\ A31 <> PosInf /\ A31 <> NegInf /\
       A32 <> PosInf /\ A32 <> NegInf /\ A33 <> PosInf /\
       A33 <> NegInf /\ A34 <> PosInf /\ A34 <> NegInf /\
       A35 <> PosInf /\ A35 <> NegInf /\ A36 <> PosInf /\
       A36 <> NegInf /\ A37 <> PosInf /\ A37 <> NegInf /\
       A38 <> PosInf /\ A38 <> NegInf /\ A39 <> PosInf /\
       A39 <> NegInf /\ A40 <> PosInf /\ A40 <> NegInf /\
       A41 <> PosInf /\ A41 <> NegInf /\ A42 <> PosInf /\
       A42 <> NegInf /\ A43 <> PosInf /\ A43 <> NegInf /\
       A44 <> PosInf /\ A44 <> NegInf /\ A45 <> PosInf /\
       A45 <> NegInf /\ A46 <> PosInf /\ A46 <> NegInf /\
       A47 <> PosInf /\ A47 <> NegInf /\ A48 <> PosInf /\
       A48 <> NegInf /\ A49 <> PosInf /\ A49 <> NegInf /\
       A50 <> PosInf /\ A50 <> NegInf /\ A51 <> PosInf /\
       A51 <> NegInf /\ A52 <> PosInf /\ A52 <> NegInf /\
       A53 <> PosInf /\ A53 <> NegInf /\ A54 <> PosInf /\
       A54 <> NegInf /\ A55 <> PosInf /\ A55 <> NegInf /\
       A56 <> PosInf /\ A56 <> NegInf /\ A57 <> PosInf /\
       A57 <> NegInf /\ A58 <> PosInf /\ A58 <> NegInf /\
       A59 <> PosInf /\ A59 <> NegInf /\ A60 <> PosInf /\
       A60 <> NegInf /\ A61 <> PosInf /\ A61 <> NegInf /\
       A62 <> PosInf /\ A62 <> NegInf /\ A63 <> PosInf /\
       A63 <> NegInf ==>
       (A1 + A2 + A3 + A4 + A5 + A6 + -A7 + -A8 + -A9 + -A10 + -A11 +
        -A12 + -A13 + -A14 + -A15 + -A16 + -A17 + -A18 + -A19 + -A20 +
        -A21 + A22 + A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 +
        A31 + A32 + A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 +
        A41 + -A42 + -A43 + -A44 + -A45 + -A46 + -A47 + -A48 + -A49 +
        -A50 + -A51 + -A52 + -A53 + -A54 + -A55 + -A56 + A57 + A58 +
        A59 + A60 + A61 + A62 + -A63 =
        A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 -
        A13 - A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 +
        A23 + A24 + A25 + A26 + A27 + A28 + A29 + A30 + A31 + A32 +
        A33 + A34 + A35 + A36 + A37 + A38 + A39 + A40 + A41 - A42 -
        A43 - A44 - A45 - A46 - A47 - A48 - A49 - A50 - A51 - A52 -
        A53 - A54 - A55 - A56 + A57 + A58 + A59 + A60 + A61 + A62 - A63) ``,


RW_TAC std_ss[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 <> PosInf /\
     		 A1 + A2 + A3 + A4 + A5 + A6 <> NegInf ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
          THEN METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 + -A7 = A1 + A2 + A3 + A4 + A5 + A6  -A7``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
          THEN METIS_TAC [add_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` A1 + A2 + A3 + A4 + A5 + A6 - A7 <> PosInf /\
                  A1 + A2 + A3 + A4 + A5 + A6 - A7 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
          THEN RW_TAC std_ss []
	  THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 + -A8 =
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add]
           THEN METIS_TAC [add_not_infty, sub_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 <> PosInf /\
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add]
           THEN METIS_TAC [add_not_infty, sub_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 + -A9 =
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add]
           THEN METIS_TAC [add_not_infty, sub_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 <> PosInf /\
     		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 + -A10 =
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add]
           THEN METIS_TAC [add_not_infty, sub_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 <> PosInf /\
     		  A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 + -A11 =
     		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add]
           THEN METIS_TAC [add_not_infty, sub_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 <> PosInf /\
     		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 <> NegInf``)
     THEN1( DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 + -A12 =
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add]
           THEN METIS_TAC [add_not_infty, sub_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 <>
                 PosInf /\
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 + -A13 =
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add]
           THEN RW_TAC arith_ss [add_not_infty, sub_not_infty, minus_x_not_infty]
	   THEN METIS_TAC [add_not_infty, sub_not_infty])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 <>
                 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 <>
		 NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 +
     		 -A14 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 - A14``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 <> PosInf /\
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 + -A15 =
 		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
       		A14 - A15 + -A16 =
		A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		A14 - A15 - A16``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 <> NegInf``)
     THEN1( DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 + -A17 =
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 + -A18 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub] )
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 <> PosInf /\
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
          THEN RW_TAC std_ss []
	  THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 + -A19 =
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19``)
     THEN1( DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 <> PosInf /\
        	 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 + -A20 =
 		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub] )
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 <> PosInf /\
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 <> NegInf``)
     THEN1( DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 + -A21 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 <> NegInf``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [extreal_sub_add, extreal_sub_sub, minus_x_not_infty, add_not_infty, sub_not_infty,add_not_infty])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 <>
		 NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 <>
		 NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 <> PosInf /\
  		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 <> NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 <> NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 <> NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
                 A26 + A27 + A28 + A29 + A30 <> PosInf /\
                 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
 		 A26 + A27 + A28 + A29 + A30 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 <> NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 <> NegInf``)
    THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 <> NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 <>
		 NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 <>
		 NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
            THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 + -A42 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 ``)
    THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
          THEN RW_TAC std_ss []
          THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 <> NegInf``)
    THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 + -A43 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
           THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 <> NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
             THEN RW_TAC std_ss []
	     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 + -A44 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
           THEN RW_TAC std_ss []
	   THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
            THEN RW_TAC std_ss []
 	    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 + -A45 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 <> NegInf``)

     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 + -A46 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
 		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 + -A47 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 ``)
      THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty] THEN RW_TAC std_ss []
      THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 + -A48 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 <>
		 NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 + -A49 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 <>
		 NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 +
		 -A50 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 + -A51 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 + -A52 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 ``)
    THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 + -A53 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 + -A54 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
     		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 <> NegInf``)
    THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 + -A55 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 ``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 + -A56 =
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56``)
     THEN1(DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN  KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                  A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		  A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		  A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 <> PosInf /\
		  A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		  A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		  A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		  A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		  A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 <> NegInf``)
    THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 <> NegInf``)
      THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
      THEN RW_TAC std_ss []
      THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 <> NegInf``)
    THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
    THEN RW_TAC std_ss []
    THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 + A60 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 + A60 <>
		 NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 + A60 + A61 <>
		 PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 + A60 + A61 <>
		 NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN KNOW_TAC(``A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
                 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 + A60 + A61 +
		 A62 <> PosInf /\
		 A1 + A2 + A3 + A4 + A5 + A6 - A7 - A8 - A9 - A10 - A11 - A12 - A13 -
		 A14 - A15 - A16 - A17 - A18 - A19 - A20 - A21 + A22 + A23 + A24 + A25 +
		 A26 + A27 + A28 + A29 + A30 + A31 + A32 + A33 + A34 + A35 + A36 + A37 +
		 A38 + A39 + A40 + A41 - A42 - A43 - A44 - A45 - A46 - A47 - A48 - A49 -
		 A50 - A51 - A52 - A53 - A54 - A55 - A56 + A57 + A58 + A59 + A60 + A61 +
		 A62 <> NegInf``)
     THEN1 (DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
     THEN RW_TAC std_ss []
     THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC [extreal_sub_add,add_not_infty, sub_not_infty, minus_x_not_infty]
THEN RW_TAC std_ss []
THEN METIS_TAC [add_not_infty, sub_not_infty, minus_x_not_infty, extreal_sub_add, extreal_sub_sub] );



val EVENTS_INTER_3 = store_thm("EVENTS_INTER_3",
  ``  !A1 A2 A3 p.
       prob_space p /\ A1 IN events p /\ A2 IN events p /\
       A3 IN events p ==>
       A1 INTER A2 INTER A3 IN events p  ``,
RW_TAC std_ss []
THEN METIS_TAC [PROB_FINITE, EVENTS_INTER]);

val EVENTS_INTER_4 = store_thm("EVENTS_INTER_4",``

!A1 A2 A3 A4 p.
       prob_space p /\ A1 IN events p /\ A2 IN events p /\
       A3 IN events p /\ A4 IN events p ==>
       A1 INTER A2 INTER A3 INTER A4 IN events p ``,


RW_TAC std_ss []
THEN METIS_TAC [PROB_FINITE, EVENTS_INTER, EVENTS_INTER_3]);

val EVENTS_INTER_5 = store_thm("EVENTS_INTER_5",``

  !A1 A2 A3 A4 A5 p.
       prob_space p /\ A1 IN events p /\ A2 IN events p /\
       A3 IN events p /\ A4 IN events p /\ A5 IN events p ==>
       A1 INTER A2 INTER A3 INTER A4 INTER A5 IN events p
``,

RW_TAC std_ss []
THEN METIS_TAC [PROB_FINITE, EVENTS_INTER, EVENTS_INTER_3, EVENTS_INTER_4]);

val EVENTS_INTER_6 = store_thm("EVENTS_INTER_6",``
  !A1 A2 A3 A4 A5 A6 p.
       prob_space p /\ A1 IN events p /\ A2 IN events p /\
       A3 IN events p /\ A4 IN events p /\ A5 IN events p /\
       A6 IN events p ==>
       A1 INTER A2 INTER A3 INTER A4 INTER A5 INTER A6 IN events p
``,

RW_TAC std_ss []
THEN METIS_TAC [PROB_FINITE, EVENTS_INTER, EVENTS_INTER_3, EVENTS_INTER_4, EVENTS_INTER_5]);


val PROB_PIE_63 = store_thm("PROB_PIE_63",
  ``!A1 A2 A3 A4 A5 A6 p.
       ALL_DISTINCT [A1; A2; A3; A4; A5; A6] /\ prob_space p /\
       A1 IN events p /\ A2 IN events p /\ A3 IN events p /\
       A4 IN events p /\ A5 IN events p /\ A6 IN events p ==>
       (prob p (union_list [A1; A2; A3; A4; A5; A6]) =
        prob p A1 + prob p A2 + prob p A3 + prob p A4 + prob p A5 + prob p A6 -
prob p (A1 INTER A2) - prob p (A1 INTER A3) - prob p (A1 INTER A4) -
prob p (A1 INTER A5) - prob p (A1 INTER A6) - prob p (A2 INTER A3) -
prob p (A2 INTER A4) - prob p (A2 INTER A5) - prob p (A2 INTER A6) -
prob p (A3 INTER A4) - prob p (A3 INTER A5) - prob p (A3 INTER A6) -
prob p (A4 INTER A5) - prob p (A4 INTER A6) - prob p (A5 INTER A6) +
prob p (A1 INTER A2 INTER A3) + prob p (A1 INTER A2 INTER A4) +
prob p (A1 INTER A2 INTER A5) + prob p (A1 INTER A2 INTER A6) +
prob p (A1 INTER A3 INTER A4) + prob p (A1 INTER A3 INTER A5) +
prob p (A1 INTER A3 INTER A6) + prob p (A1 INTER A4 INTER A5) +
prob p (A1 INTER A4 INTER A6) + prob p (A1 INTER A5 INTER A6) +
prob p (A2 INTER A3 INTER A4) + prob p (A2 INTER A3 INTER A5) +
prob p (A2 INTER A3 INTER A6) + prob p (A2 INTER A4 INTER A5) +
prob p (A2 INTER A4 INTER A6) + prob p (A2 INTER A5 INTER A6) +
prob p (A3 INTER A4 INTER A5) + prob p (A3 INTER A4 INTER A6) +
prob p (A3 INTER A5 INTER A6) + prob p (A4 INTER A5 INTER A6) -
prob p (A1 INTER A2 INTER A3 INTER A4) -
prob p (A1 INTER A2 INTER A3 INTER A5) -
prob p (A1 INTER A2 INTER A3 INTER A6) -
prob p (A1 INTER A2 INTER A4 INTER A5) -
prob p (A1 INTER A2 INTER A4 INTER A6) -
prob p (A1 INTER A2 INTER A5 INTER A6) -
prob p (A1 INTER A3 INTER A4 INTER A5) -
prob p (A1 INTER A3 INTER A4 INTER A6) -
prob p (A1 INTER A3 INTER A5 INTER A6) -
prob p (A1 INTER A4 INTER A5 INTER A6) -
prob p (A2 INTER A3 INTER A4 INTER A5) -
prob p (A2 INTER A3 INTER A4 INTER A6) -
prob p (A2 INTER A3 INTER A5 INTER A6) -
prob p (A2 INTER A4 INTER A5 INTER A6) -
prob p (A3 INTER A4 INTER A5 INTER A6) +
prob p (A1 INTER A2 INTER A3 INTER A4 INTER A5) +
prob p (A1 INTER A2 INTER A3 INTER A4 INTER A6) +
prob p (A1 INTER A2 INTER A3 INTER A5 INTER A6) +
prob p (A1 INTER A2 INTER A4 INTER A5 INTER A6) +
prob p (A1 INTER A3 INTER A4 INTER A5 INTER A6) +
prob p (A2 INTER A3 INTER A4 INTER A5 INTER A6) -
prob p (A1 INTER A2 INTER A3 INTER A4 INTER A5 INTER A6))``

,
RW_TAC std_ss[]
THEN DEP_REWRITE_TAC[PROB_INCLUSION_EXCLUSION_PRINCIPLE]
THEN RW_TAC list_ss[] THEN RW_TAC std_ss[]
THEN DEP_REWRITE_TAC [PIE_set_alt_form_lem_6]
THEN RW_TAC std_ss [sum_set_def]
THEN DEP_REWRITE_TAC[PIE_lem4_63_prod]
THEN RW_TAC std_ss[]
    THEN1 (RW_TAC std_ss[extreal_pow_def]
           THEN DEP_REWRITE_TAC[PIE_lemma6]
           THEN DEP_REWRITE_TAC[PIE_lemma3]
           THEN DEP_REWRITE_TAC[PROB_FINITE,EVENTS_COUNTABLE_INTER,COUNTABLE_IMAGE,finite_countable,IMAGE_FINITE]
           THEN (PSET_TAC[FINITE_SING,EVENTS_INTER,NOT_SING_EMPTY] THEN SRW_TAC[][FINITE_SING] THEN METIS_TAC[EVENTS_INTER]))
       
THEN RW_TAC std_ss[]
THEN RW_TAC real_ss[BIGINTER_SING,CARD_SING,CARD_INSERT,GSYM ADD1]
THEN FULL_SIMP_TAC list_ss[ALL_DISTINCT]
THEN PSET_TAC[]
THEN RW_TAC real_ss[extreal_pow_def]
THEN SRW_TAC[][FINITE_SING]
THEN RW_TAC real_ss[pow,prove (``!A B C:extreal->bool. A <> B /\ A <> C /\ B <> C ==> ((CARD {A;B} = SUC (SUC 0)) /\ (CARD {A;B;C} = SUC (SUC (SUC 0))))``, RW_TAC list_ss[ALL_DISTINCT]THEN  DEP_REWRITE_TAC[CARD_DEF] THEN SRW_TAC[][FINITE_SING])]
THEN DEP_REWRITE_TAC[PIE_lemma5]
THEN DEP_REWRITE_TAC[PROB_FINITE,EVENTS_COUNTABLE_INTER,COUNTABLE_IMAGE,finite_countable,IMAGE_FINITE]
THEN RW_TAC std_ss[] THEN  PSET_TAC[FINITE_SING,EVENTS_INTER,NOT_SING_EMPTY] THEN SRW_TAC[][FINITE_SING]
THEN RW_TAC std_ss[prove(``Normal 1 = 1``, RW_TAC std_ss[extreal_of_num_def] THEN METIS_TAC[]),mul_lone]
     THEN1 (RW_TAC std_ss [GSYM extreal_sub_add, PROB_FINITE, EVENTS_INTER] THEN METIS_TAC [EVENTS_INTER])
THEN RW_TAC std_ss[INTER_ASSOC]
THEN DEP_REWRITE_TAC [extreal_add_sub_63, PROB_FINITE, EVENTS_INTER, EVENTS_INTER_3, EVENTS_INTER_4, EVENTS_INTER_5, EVENTS_INTER_6]
THEN RW_TAC std_ss[PROB_FINITE, EVENTS_INTER, EVENTS_INTER_3, EVENTS_INTER_4, EVENTS_INTER_5, EVENTS_INTER_6]);



val PROB_PIE_CAS_63_lem1 = store_thm("PROB_PIE_CAS_63_lem1",
  ``      !CS SS MA MS MB P B PA PB PS_a p t.
       ALL_DISTINCT
         [DFT_event p CS t; DFT_event p SS t;
          DFT_event p (D_AND MA (D_BEFORE MS MA)) t;
          DFT_event p (D_AND MA MB) t; DFT_event p (D_AND P B) t;
          DFT_event p (D_AND (D_AND PA PB) PS_a) t] /\ prob_space p /\
       DFT_event p CS t IN events p /\ DFT_event p SS t IN events p /\
       DFT_event p (D_AND MA (D_BEFORE MS MA)) t IN events p /\
       DFT_event p (D_AND MA MB) t IN events p /\
       DFT_event p (D_AND P B) t IN events p /\
       DFT_event p (D_AND (D_AND PA PB) PS_a) t IN events p ==>
       (prob p
          (union_list
             [DFT_event p CS t; DFT_event p SS t;
              DFT_event p (D_AND MA (D_BEFORE MS MA)) t;
              DFT_event p (D_AND MA MB) t; DFT_event p (D_AND P B) t;
              DFT_event p (D_AND (D_AND PA PB) PS_a) t]) =
        prob p (DFT_event p CS t) + prob p (DFT_event p SS t) +
prob p (DFT_event p (D_AND MA (D_BEFORE MS MA)) t) +
prob p (DFT_event p (D_AND MA MB) t) +
prob p (DFT_event p (D_AND P B) t) +
prob p (DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p (DFT_event p CS t INTER DFT_event p SS t) -
prob p
  (DFT_event p CS t INTER DFT_event p (D_AND MA (D_BEFORE MS MA)) t) -
prob p (DFT_event p CS t INTER DFT_event p (D_AND MA MB) t) -
prob p (DFT_event p CS t INTER DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p CS t INTER DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p SS t INTER DFT_event p (D_AND MA (D_BEFORE MS MA)) t) -
prob p (DFT_event p SS t INTER DFT_event p (D_AND MA MB) t) -
prob p (DFT_event p SS t INTER DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p SS t INTER DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t) -
prob p
  (DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p (DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA MB) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND P B) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t) +
prob p
  (DFT_event p CS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t) +
prob p
  (DFT_event p CS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND P B) t) +
prob p
  (DFT_event p CS t INTER DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t) +
prob p
  (DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t) +
prob p
  (DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p SS t INTER DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND P B) t) +
prob p
  (DFT_event p SS t INTER DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p SS t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t) +
prob p
  (DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t) -
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p CS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p CS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p CS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p CS t INTER DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t) -
prob p
  (DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p SS t INTER DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p CS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) +
prob p
  (DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t) -
prob p
  (DFT_event p CS t INTER DFT_event p SS t INTER
   DFT_event p (D_AND MA (D_BEFORE MS MA)) t INTER
   DFT_event p (D_AND MA MB) t INTER DFT_event p (D_AND P B) t INTER
   DFT_event p (D_AND (D_AND PA PB) PS_a) t))  ``,

RW_TAC std_ss [PROB_PIE_63]);


val indep_vars_10_def = Define `  indep_vars_10 p M A0 A1 A2 A3 A4 A5 A6 A7 A8 A9 =
     indep_vars p (\i. M)
       (\i.
          if i = 0 then A0
          else if i = 1 then A1
          else if i = 2 then A2
          else if i = 3 then A3
          else if i = 4 then A4
          else if i = 5 then A5
          else if i = 6 then A6
          else if i = 7 then A7
          else if i = 8 then A8
          else A9) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}`;



val normal_real_mul_3 = store_thm("normal_real_mul_3",
  ``  !x y z.
       x <> PosInf /\ x <> NegInf /\ y <> PosInf /\ y <> NegInf /\
       z <> PosInf /\ z <> NegInf ==>
       (Normal (real x * real y * real z) =
        Normal (real x) * Normal (real y) * Normal (real z))  ``,

REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);

val normal_real_mul_4 = store_thm("normal_real_mul_4",
  ``  !x1 x2 x3 x4.
       x1 <> PosInf /\ x1 <> NegInf /\ x2 <> PosInf /\ x2 <> NegInf /\
       x3 <> PosInf /\ x3 <> NegInf /\ x4 <> PosInf /\ x4 <> NegInf ==>
       (Normal (real x1 * real x2 * real x3 * real x4) =
        Normal (real x1) * Normal (real x2) * Normal (real x3) *
        Normal (real x4))``,
	
REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);

 
val normal_real_mul_5 = store_thm("normal_real_mul_5",
  ``  !x1 x2 x3 x4 x5.
       x1 <> PosInf /\ x1 <> NegInf /\ x2 <> PosInf /\ x2 <> NegInf /\
       x3 <> PosInf /\ x3 <> NegInf /\ x4 <> PosInf /\ x4 <> NegInf /\
       x5 <> PosInf /\ x5 <> NegInf==>
       (Normal (real x1 * real x2 * real x3 * real x4 * real x5) =
        Normal (real x1) * Normal (real x2) * Normal (real x3) *
        Normal (real x4) * Normal (real x5))``,
	
REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);


val normal_real_mul_6 = store_thm("normal_real_mul_6",
  ``  !x1 x2 x3 x4 x5 x6.
       x1 <> PosInf /\ x1 <> NegInf /\ x2 <> PosInf /\ x2 <> NegInf /\
       x3 <> PosInf /\ x3 <> NegInf /\ x4 <> PosInf /\ x4 <> NegInf /\
       x5 <> PosInf /\ x5 <> NegInf /\ x6 <> PosInf /\ x6 <> NegInf==>
       (Normal (real x1 * real x2 * real x3 * real x4 * real x5 * real x6) =
        Normal (real x1) * Normal (real x2) * Normal (real x3) *
        Normal (real x4) * Normal (real x5) * Normal (real x6))``,
	
REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);

val normal_real_mul_7 = store_thm("normal_real_mul_7",
  ``  !x1 x2 x3 x4 x5 x6 x7.
       x1 <> PosInf /\ x1 <> NegInf /\ x2 <> PosInf /\ x2 <> NegInf /\
       x3 <> PosInf /\ x3 <> NegInf /\ x4 <> PosInf /\ x4 <> NegInf /\
       x5 <> PosInf /\ x5 <> NegInf /\ x6 <> PosInf /\ x6 <> NegInf /\
       x7 <> PosInf /\ x7 <> NegInf==>
       (Normal (real x1 * real x2 * real x3 * real x4 * real x5 * real x6 *
       real x7) =
        Normal (real x1) * Normal (real x2) * Normal (real x3) *
        Normal (real x4) * Normal (real x5) * Normal (real x6) *
	Normal (real x7))``,
	
REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);

val normal_real_mul_8 = store_thm("normal_real_mul_8",
  ``  !x1 x2 x3 x4 x5 x6 x7 x8.
       x1 <> PosInf /\ x1 <> NegInf /\ x2 <> PosInf /\ x2 <> NegInf /\
       x3 <> PosInf /\ x3 <> NegInf /\ x4 <> PosInf /\ x4 <> NegInf /\
       x5 <> PosInf /\ x5 <> NegInf /\ x6 <> PosInf /\ x6 <> NegInf /\
       x7 <> PosInf /\ x7 <> NegInf /\ x8 <> PosInf /\ x8 <> NegInf==>
       (Normal (real x1 * real x2 * real x3 * real x4 * real x5 * real x6 *
       real x7 * real x8) =
        Normal (real x1) * Normal (real x2) * Normal (real x3) *
        Normal (real x4) * Normal (real x5) * Normal (real x6) *
	Normal (real x7) * Normal (real x8))``,
	
REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);


val normal_real_mul_9 = store_thm("normal_real_mul_9",
  ``  !x1 x2 x3 x4 x5 x6 x7 x8 x9.
       x1 <> PosInf /\ x1 <> NegInf /\ x2 <> PosInf /\ x2 <> NegInf /\
       x3 <> PosInf /\ x3 <> NegInf /\ x4 <> PosInf /\ x4 <> NegInf /\
       x5 <> PosInf /\ x5 <> NegInf /\ x6 <> PosInf /\ x6 <> NegInf /\
       x7 <> PosInf /\ x7 <> NegInf /\ x8 <> PosInf /\ x8 <> NegInf /\
       x9 <> PosInf /\ x9 <> NegInf==>
       (Normal (real x1 * real x2 * real x3 * real x4 * real x5 * real x6 *
       real x7 * real x8* real x9 ) =
        Normal (real x1) * Normal (real x2) * Normal (real x3) *
        Normal (real x4) * Normal (real x5) * Normal (real x6) *
	Normal (real x7) * Normal (real x8) * Normal (real x9))``,
	
REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);


val normal_real_mul_10 = store_thm("normal_real_mul_10",
  ``  !x1 x2 x3 x4 x5 x6 x7 x8 x9 x10.
       x1 <> PosInf /\ x1 <> NegInf /\ x2 <> PosInf /\ x2 <> NegInf /\
       x3 <> PosInf /\ x3 <> NegInf /\ x4 <> PosInf /\ x4 <> NegInf /\
       x5 <> PosInf /\ x5 <> NegInf /\ x6 <> PosInf /\ x6 <> NegInf /\
       x7 <> PosInf /\ x7 <> NegInf /\ x8 <> PosInf /\ x8 <> NegInf /\
       x9 <> PosInf /\ x9 <> NegInf /\ x10 <> PosInf /\ x10 <> NegInf==>
       (Normal (real x1 * real x2 * real x3 * real x4 * real x5 * real x6 *
       real x7 * real x8* real x9 * real x10) =
        Normal (real x1) * Normal (real x2) * Normal (real x3) *
        Normal (real x4) * Normal (real x5) * Normal (real x6) *
	Normal (real x7) * Normal (real x8) * Normal (real x9) *
	Normal (real x10))``,
	
REPEAT Cases
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss [real_def]
THEN METIS_TAC [extreal_mul_def]);


val PRODUCT_UNION_5 = store_thm("PRODUCT_UNION_5",
  ``
     !s1 s2 s3 s4 s5 f.
       ALL_DISTINCT [s1; s2; s3; s4; s5] ==>
       (product {s1; s2; s3; s4; s5} f =
        f s1 * f s2 * f s3 * f s4 * f s5)``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1} /\ FINITE {s2; s3; s4; s5} /\
                 DISJOINT {s1} {s2; s3; s4; s5} /\
		 ({s1; s2; s3; s4; s5} = {s1} UNION {s2; s3; s4; s5})``)
		 
     THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM] )
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s2} /\ FINITE {s3; s4; s5} /\ DISJOINT {s2} {s3; s4; s5} /\
     	         ({s2; s3; s4; s5} = {s2} UNION {s3; s4; s5})``)
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s3} /\ FINITE {s4; s5} /\ DISJOINT {s3} {s4; s5} /\
                 ({s3; s4; s5} = {s3} UNION {s4; s5})``)
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF]
           THEN METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s4} /\ FINITE {s5} /\ DISJOINT {s4} {s5} /\
                ({s4;s5} = {s4} UNION {s5})``)
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN RW_TAC std_ss [PRODUCT_SING]
THEN RW_TAC std_ss[REAL_MUL_ASSOC]);



val PRODUCT_UNION_6 = store_thm("PRODUCT_UNION_6",
  ``  !s1 s2 s3 s4 s5 s6 f.
       ALL_DISTINCT [s1; s2; s3; s4; s5; s6] ==>
       (product {s1; s2; s3; s4; s5; s6} f =
        f s1 * f s2 * f s3 * f s4 * f s5 * f s6)``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1; s2; s3; s4; s5} /\ FINITE {s6} /\
                 DISJOINT {s1; s2; s3; s4; s5} {s6} /\
		 ({s1; s2; s3; s4; s5; s6} = {s1; s2; s3; s4; s5} UNION {s6})``)
     THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM] )
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``ALL_DISTINCT [s1;s2;s3;s4;s5]``)
      THEN1 (METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION_5, PRODUCT_SING]);

val PRODUCT_UNION_7= store_thm("PRODUCT_UNION_7",
  ``!s1 s2 s3 s4 s5 s6 s7 f.
       ALL_DISTINCT [s1; s2; s3; s4; s5; s6; s7] ==>
       (product {s1; s2; s3; s4; s5; s6; s7} f =
        f s1 * f s2 * f s3 * f s4 * f s5 * f s6 * f s7) ``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1; s2; s3; s4; s5; s6} /\ FINITE {s7} /\
            DISJOINT {s1; s2; s3; s4; s5; s6} {s7} /\
            ({s1; s2; s3; s4; s5; s6; s7} = {s1; s2; s3; s4; s5; s6} UNION {s7})``)
     THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM] )
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``ALL_DISTINCT [s1;s2;s3;s4;s5;s6]``)
     THEN1 (METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION_6, PRODUCT_SING]);


val PRODUCT_UNION_8= store_thm("PRODUCT_UNION_8",
  `` 
     !s1 s2 s3 s4 s5 s6 s7 s8 f.
       ALL_DISTINCT [s1; s2; s3; s4; s5; s6; s7; s8] ==>
       (product {s1; s2; s3; s4; s5; s6; s7; s8} f =
        f s1 * f s2 * f s3 * f s4 * f s5 * f s6 * f s7 * f s8)``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1; s2; s3; s4; s5; s6; s7} /\ FINITE {s8} /\
                 DISJOINT {s1; s2; s3; s4; s5; s6; s7} {s8} /\
		 ({s1; s2; s3; s4; s5; s6; s7; s8} =
		  {s1; s2; s3; s4; s5; s6; s7} UNION {s8})``)
    THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, FINITE_DEF]
          THEN METIS_TAC [ALL_DISTINCT, MEM] )
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``ALL_DISTINCT [s1;s2;s3;s4;s5;s6;s7]``)
     THEN1 (METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION_7, PRODUCT_SING]);


val PRODUCT_UNION_9= store_thm("PRODUCT_UNION_9",
  ``    !s1 s2 s3 s4 s5 s6 s7 s8 s9 f.
       ALL_DISTINCT [s1; s2; s3; s4; s5; s6; s7; s8; s9] ==>
       (product {s1; s2; s3; s4; s5; s6; s7; s8; s9} f =
        f s1 * f s2 * f s3 * f s4 * f s5 * f s6 * f s7 * f s8 * f s9)``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1; s2; s3; s4; s5; s6; s7; s8} /\ FINITE {s9} /\
                 DISJOINT {s1; s2; s3; s4; s5; s6; s7; s8} {s9} /\
		 ({s1; s2; s3; s4; s5; s6; s7; s8; s9} =
		  {s1; s2; s3; s4; s5; s6; s7; s8} UNION {s9})``)
     THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM] )
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``ALL_DISTINCT [s1;s2;s3;s4;s5;s6;s7;s8]``)
     THEN1 (METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION_8, PRODUCT_SING]);



val PRODUCT_UNION_10= store_thm("PRODUCT_UNION_10",
  ``!s1 s2 s3 s4 s5 s6 s7 s8 s9 s10 f.
       ALL_DISTINCT [s1; s2; s3; s4; s5; s6; s7; s8; s9; s10] ==>
       (product {s1; s2; s3; s4; s5; s6; s7; s8; s9; s10} f =
        f s1 * f s2 * f s3 * f s4 * f s5 * f s6 * f s7 * f s8 * f s9 *
        f s10)``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1; s2; s3; s4; s5; s6; s7; s8; s9} /\ FINITE {s10} /\
                 DISJOINT {s1; s2; s3; s4; s5; s6; s7; s8; s9} {s10} /\
		 ({s1; s2; s3; s4; s5; s6; s7; s8; s9; s10} =
		  {s1; s2; s3; s4; s5; s6; s7; s8; s9} UNION {s10})``)
     THEN1( SRW_TAC[] [GSPECIFICATION, EXTENSION, FINITE_DEF]
            THEN METIS_TAC [ALL_DISTINCT, MEM] )
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``ALL_DISTINCT [s1;s2;s3;s4;s5;s6;s7;s8;s9]``)
     THEN1 (KNOW_TAC(``[s1;s2;s3;s4;s5;s6;s7;s8;s9;s10]=[s1;s2;s3;s4;s5;s6;s7;s8;s9]++[s10]``)
            THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION])
	    THEN METIS_TAC [ALL_DISTINCT_APPEND,ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION_9, PRODUCT_SING]);


val prob_prod_any_2_of_10 = store_thm("prob_prod_any_2_of_10",
  ``   !p M X ii A s t.
       s <> t /\ prob_space p /\ indep_vars p M X ii /\
       {s; t} SUBSET ii /\
       (!i. i IN {s; t} ==> A i IN measurable_sets (M i)) ==>
       (prob p
          (PREIMAGE (X s) (A s) INTER p_space p INTER
           (PREIMAGE (X t) (A t) INTER p_space p)) =
        prob p (PREIMAGE (X s) (A s) INTER p_space p) *
        prob p (PREIMAGE (X t) (A t) INTER p_space p))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s:num;t}``)
     THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``prob p (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p | i IN {s; t}}) =
     		 Normal
		   (product {s:num; t}
		        (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))) ``)

      THEN1(DEP_REWRITE_TAC [indep_varsD]
            THEN Q.EXISTS_TAC `M`
	    THEN Q.EXISTS_TAC `ii`
	    THEN RW_TAC std_ss[]
	    THEN SRW_TAC[] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p | i IN {s:num; t}} =
                 {PREIMAGE (X s) (A s) INTER p_space p ;
		  PREIMAGE (X t) (A t) INTER p_space p}``)
     THEN1 (ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``BIGINTER
                   {PREIMAGE ((X:num->'a->'b) s) (A s) INTER p_space p;
                    PREIMAGE (X t) (A t) INTER p_space p} =
		 (PREIMAGE (X s) (A s) INTER p_space p) INTER
                 (PREIMAGE (X t) (A t) INTER p_space p )``)
      THEN1(RW_TAC std_ss [BIGINTER_INTER])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s:num} /\ FINITE {t:num} /\
                 DISJOINT {s:num} {t:num} /\ ({s:num;t} = {s} UNION {t})``)

     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF, DISJOINT_DEF])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN RW_TAC std_ss [PRODUCT_SING]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s) p (m_space (M s), measurable_sets (M s)) /\
                 random_variable  ((X:num->'a->'b) t)p (m_space (M t), measurable_sets (M t))``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars]
           THEN UNDISCH_TAC(`` !i:num.
           			   i IN ii ==>
                            random_variable ((X:num->'a->'b) i) p (m_space (M i),
			            measurable_sets (M i))``)

	   THEN DISCH_TAC
	   THEN RW_TAC std_ss[]
	        THEN1 (POP_ASSUM (MP_TAC o Q.SPEC `s:num `)
	               THEN ASM_SET_TAC[])

           THEN POP_ASSUM (MP_TAC o Q.SPEC `t:num `)
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s) (A s) INTER p_space p IN events p/\
                 PREIMAGE ((X:num->'a->'b) t) (A t) INTER p_space p IN events p``)
     THEN1( REWRITE_TAC [events_def]
            THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
                 THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
            THEN DISCH_TAC
            THEN ONCE_ASM_REWRITE_TAC[]
            THEN REWRITE_TAC [p_space_def, m_space_def]
            THEN KNOW_TAC(`` (A:num->'b->bool) s IN measurable_sets (M s) /\
                              (A:num->'b->bool) t IN measurable_sets (M t)``)
                 THEN1(RW_TAC std_ss[]
 		       THEN1(UNDISCH_TAC(`` !i:num. i IN {s; t} ==>
		               (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s:num `)
			     THEN ASM_SET_TAC[])
		THEN UNDISCH_TAC(`` !i:num. i IN {s; t} ==>
		                     (A:num->'b->bool) i IN measurable_sets (M i)``)
                THEN DISCH_TAC
		THEN POP_ASSUM (MP_TAC o Q.SPEC `t:num `)
		THEN ASM_SET_TAC[])
           THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [normal_real_mul, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);



val prob_prod_any_2_of_10_set = store_thm("prob_prod_any_2_of_10_set",
  `` 
     !p ff ii A s t.
       s <> t /\ prob_space p /\ indep_sets p ff ii /\
       {s; t} SUBSET ii /\ (!i. i IN {s; t} ==> A i IN ff i) /\
       (!i. i IN {s; t} ==> A i IN events p) ==>
       (prob p (A s INTER A t) = prob p (A s) * prob p (A t))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s:num;t}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s:num;t} }) =
      Normal (product {s:num;t} (\j. real (prob p (A j)))))``) 
     THEN1(DEP_REWRITE_TAC [indep_setsD] 
          THEN Q.EXISTS_TAC `ff` 
          THEN Q.EXISTS_TAC `ii` 
          THEN RW_TAC std_ss[] 
          THEN SRW_TAC[] [GSPECIFICATION, EXTENSION]) 
THEN DISCH_TAC 
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s:num; t}}) = A s INTER A t``) 
     THEN1( ASM_SET_TAC[]) 
THEN DISCH_TAC 
THEN ASM_REWRITE_TAC[] 
THEN FULL_SIMP_TAC std_ss[] 
THEN KNOW_TAC(``FINITE {s:num} /\ FINITE {t:num} /\ 
                 DISJOINT {s:num} {t:num} /\ 
                 ({s:num;t} = {s} UNION {t})``) 
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF, DISJOINT_DEF])  
THEN DISCH_TAC 
THEN RW_TAC std_ss [PRODUCT_UNION] 
THEN RW_TAC std_ss [PRODUCT_SING] 
THEN KNOW_TAC(``(A:num->'a->bool) s IN events p``) 
     THEN1( UNDISCH_TAC(``!i. i IN {s:num; t} ==> A i IN events p``)  
            THEN DISCH_TAC 
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s:num `) 
	    THEN DISCH_TAC 
	    THEN ASM_SET_TAC[]) 
THEN DISCH_TAC 
THEN KNOW_TAC(``(A:num->'a->bool) t IN events p``) 
     THEN1( UNDISCH_TAC(``!i. i IN {s:num; t} ==> A i IN events p``) 
            THEN DISCH_TAC 
            THEN POP_ASSUM (MP_TAC o Q.SPEC `t:num `) 
  	    THEN DISCH_TAC 
	    THEN ASM_SET_TAC[]) 
THEN DISCH_TAC 
THEN RW_TAC std_ss [normal_real_mul, PROB_FINITE] 
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);


val BIGINTER_3_sets = store_thm("BIGINTER_3_sets",
  `` ! x y z . BIGINTER {x;y;z} = x INTER y INTER z``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);

val BIGINTER_4_sets = store_thm("BIGINTER_4_sets",
  `` ! x1 x2 x3 x4 .
       BIGINTER {x1;x2;x3;x4} = x1 INTER x2 INTER x3 INTER x4``,

SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);


val BIGINTER_5_sets = store_thm("BIGINTER_5_sets",
  ``  ! x1 x2 x3 x4 x5 .
         BIGINTER {x1;x2;x3;x4; x5} = x1 INTER x2 INTER x3 INTER x4 INTER x5``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);

val BIGINTER_6_sets = store_thm("BIGINTER_6_sets",
  ``  ! x1 x2 x3 x4 x5 x6.
          BIGINTER {x1;x2;x3;x4; x5;x6} = x1 INTER x2 INTER x3 INTER x4 INTER x5 INTER x6``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);

val BIGINTER_7_sets = store_thm("BIGINTER_7_sets",
  ``  ! x1 x2 x3 x4 x5 x6 x7.
         BIGINTER {x1;x2;x3;x4; x5;x6;x7} =
	 x1 INTER x2 INTER x3 INTER x4 INTER x5 INTER x6 INTER x7``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);


val BIGINTER_8_sets = store_thm("BIGINTER_8_sets",
  ``  ! x1 x2 x3 x4 x5 x6 x7 x8.
        BIGINTER {x1;x2;x3;x4; x5;x6;x7;x8} =
	x1 INTER x2 INTER x3 INTER x4 INTER x5 INTER x6 INTER x7 INTER x8``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);


val BIGINTER_9_sets = store_thm("BIGINTER_9_sets",
  ``  ! x1 x2 x3 x4 x5 x6 x7 x8 x9.
       BIGINTER {x1;x2;x3;x4; x5;x6;x7;x8;x9} =
       x1 INTER x2 INTER x3 INTER x4 INTER x5 INTER x6 INTER x7 INTER x8 INTER x9``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);



val BIGINTER_10_sets = store_thm("BIGINTER_10_sets",
  ``  ! x1 x2 x3 x4 x5 x6 x7 x8 x9 x10.
        BIGINTER {x1;x2;x3;x4; x5;x6;x7;x8;x9;x10} =
	x1 INTER x2 INTER x3 INTER x4 INTER x5 INTER x6 INTER x7 INTER x8 INTER x9 INTER x10``,
SRW_TAC [] [GSPECIFICATION, EXTENSION, BIGINTER]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[])
THEN ASM_SET_TAC[]);



val prob_prod_any_3_of_10 = store_thm("prob_prod_any_3_of_10",
  ``   !p M X ii A s t k.
       ALL_DISTINCT [s; t; k] /\ prob_space p /\ indep_vars p M X ii /\
       {s; t; k} SUBSET ii /\
       (!i. i IN {s; t; k} ==> A i IN measurable_sets (M i)) ==>
       (prob p
          (PREIMAGE (X s) (A s) INTER p_space p INTER
           (PREIMAGE (X t) (A t) INTER p_space p) INTER
           (PREIMAGE (X k) (A k) INTER p_space p)) =
        prob p (PREIMAGE (X s) (A s) INTER p_space p) *
        prob p (PREIMAGE (X t) (A t) INTER p_space p) *
        prob p (PREIMAGE (X k) (A k) INTER p_space p))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s:num;t;k}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
                    (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p | i IN {s:num;t;k}}) =
        	      Normal
        	         (product {s:num;t;k}
           		   (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
     THEN1 (DEP_REWRITE_TAC [indep_varsD]
            THEN Q.EXISTS_TAC `M`
	    THEN Q.EXISTS_TAC `ii`
	    THEN RW_TAC std_ss[]
	    THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p | i IN {s:num; t;k}} =
                 {PREIMAGE (X s) (A s) INTER p_space p;
		  PREIMAGE (X t) (A t) INTER p_space p;
		  PREIMAGE (X k) (A k) INTER p_space p}``)
		 
    THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``BIGINTER
                  {PREIMAGE ((X:num->'a->'b) s) (A s) INTER p_space p;
                   PREIMAGE (X t) (A t) INTER p_space p;
		   PREIMAGE (X k) (A k) INTER p_space p} =
		 (PREIMAGE (X s) (A s) INTER p_space p) INTER
                 (PREIMAGE (X t) (A t) INTER p_space p) INTER
		 (PREIMAGE (X k) (A k) INTER p_space p)``)
     THEN1(RW_TAC std_ss [BIGINTER_3_sets])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s:num} /\ FINITE {t:num;k} /\DISJOINT {s:num} {t:num;k} /\
                 ({s:num;t;k } = {s} UNION {t;k})``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
           THEN1(METIS_TAC[ALL_DISTINCT, MEM])
           THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {t:num} /\ FINITE {k:num} /\
                 DISJOINT {t:num} {k:num} /\
		 ({t:num;k } = {t} UNION {k})``)
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
           THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN RW_TAC std_ss [PRODUCT_SING]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s) p (m_space (M s), measurable_sets (M s)) /\
                 random_variable  ((X:num->'a->'b) t)p (m_space (M t), measurable_sets (M t)) /\
		 random_variable ((X:num->'a->'b) k) p (m_space (M k), measurable_sets (M k))``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars]
           THEN UNDISCH_TAC(`` !i:num.
       	   			  i IN ii ==>
				   random_variable ((X:num->'a->'b) i) p (m_space (M i),
				     measurable_sets (M i))``)
	   THEN DISCH_TAC
	   THEN RW_TAC std_ss[]
	        THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s:num `)
		      THEN ASM_SET_TAC[])
                THEN1(POP_ASSUM (MP_TAC o Q.SPEC `t:num `)
		      THEN ASM_SET_TAC[])
          THEN POP_ASSUM (MP_TAC o Q.SPEC `k:num `)
	  THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s) (A s) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) t) (A t) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) k) (A k) INTER p_space p IN events p``)
     THEN1(REWRITE_TAC [events_def]
           THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	        THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
	   THEN DISCH_TAC
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN REWRITE_TAC [p_space_def, m_space_def]
	   THEN KNOW_TAC(`` (A:num->'b->bool) s IN measurable_sets (M s) /\
	                     (A:num->'b->bool) t IN measurable_sets (M t) /\
			     (A:num->'b->bool) k IN measurable_sets (M k)``)
                THEN1 (RW_TAC std_ss[]
		       THEN1(UNDISCH_TAC(`` !i:num. i IN {s; t;k} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s:num `)
			     THEN ASM_SET_TAC[])
			THEN1(UNDISCH_TAC(`` !i:num. i IN {s; t;k} ==>
			                      (A:num->'b->bool) i IN measurable_sets (M i)``)
			      THEN DISCH_TAC
			      THEN POP_ASSUM (MP_TAC o Q.SPEC `t:num `)
			      THEN ASM_SET_TAC[])
		THEN UNDISCH_TAC(`` !i. i IN {s:num; t; k} ==>
		                     (A:num->'b->bool) i IN measurable_sets (M i)``)
		THEN DISCH_TAC
		THEN POP_ASSUM (MP_TAC o Q.SPEC `k:num `)
		THEN ASM_SET_TAC[])
         THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_3, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);



val prob_prod_any_3_of_10_set = store_thm("prob_prod_any_3_of_10_set",
  ``! p ff  ii A  (s:num) t k.
        ALL_DISTINCT [s;t;k] /\ prob_space p/\ indep_sets p ff ii /\
	{s:num;t;k} SUBSET ii /\ (!i. i IN {s:num;t;k} ==> A i  IN ff i) /\
	(!i. i IN {s:num;t;k} ==> A i IN events p) ==>
	(prob p (A s INTER A t INTER A k) =
         prob p (A s) * prob p (A t) * prob p (A k))  ``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s:num;t;k}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s:num;t;k} }) =
                  Normal (product {s:num;t;k} (\j. real (prob p (A j)))))``)
     THEN1( DEP_REWRITE_TAC [indep_setsD]
            THEN Q.EXISTS_TAC `ff`
	    THEN Q.EXISTS_TAC `ii`
	    THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s:num; t;k}}) = A s INTER A t INTER A k``)
     THEN1( ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s:num} /\ FINITE {t:num;k} /\
                 DISJOINT {s:num} {t:num;k} /\ ({s:num;t;k} =
		 {s} UNION {t;k})``)
      THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF, DISJOINT_DEF, ALL_DISTINCT, MEM]
             THEN METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {t:num} /\ FINITE {k:num} /\
                 DISJOINT {t:num} {k:num} /\
		 ({t:num;k} = {t} UNION {k})``)
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF, DISJOINT_DEF, ALL_DISTINCT, MEM]
            THEN METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN RW_TAC std_ss [PRODUCT_SING]
THEN KNOW_TAC(``(A:num->'a->bool) s IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s:num; t;k} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) t IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s:num; t;k} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `t:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) k IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s:num; t;k} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `k:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_3, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);




val prob_prod_any_4_of_10 = store_thm("prob_prod_any_4_of_10",
  ``
   !p M X ii A s1 s2 s3 s4.
       ALL_DISTINCT [s1; s2; s3; s4] /\ prob_space p /\
       indep_vars p M X ii /\ {s1; s2; s3; s4} SUBSET ii /\
       (!i. i IN {s1; s2; s3; s4} ==> A i IN measurable_sets (M i)) ==>
       (prob p
          (PREIMAGE (X s1) (A s1) INTER p_space p INTER
           (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
           (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
           (PREIMAGE (X s4) (A s4) INTER p_space p)) =
        prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
        prob p (PREIMAGE (X s2) (A s2) INTER p_space p) *
        prob p (PREIMAGE (X s3) (A s3) INTER p_space p) *
        prob p (PREIMAGE (X s4) (A s4) INTER p_space p))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3;s4}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
                   (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p | i IN {s1:num;s2;s3;s4}}) =
                    Normal
        	     (product {s1:num;s2;s3;s4}
           	       (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
     THEN1(DEP_REWRITE_TAC [indep_varsD]
          THEN Q.EXISTS_TAC `M`
	  THEN Q.EXISTS_TAC `ii`
	  THEN RW_TAC std_ss[]
	  THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p | i IN {s1:num;s2;s3;s4}} =
                 {PREIMAGE (X s1) (A s1) INTER p_space p;
		  PREIMAGE (X s2) (A s2) INTER p_space p;
		  PREIMAGE (X s3) (A s3) INTER p_space p;
		  PREIMAGE (X s4) (A s4) INTER p_space p}``)
     THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``BIGINTER
                  {PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p;
                   PREIMAGE (X s2) (A s2) INTER p_space p;
		   PREIMAGE (X s3) (A s3) INTER p_space p;PREIMAGE (X s4) (A s4) INTER p_space p} =
		 (PREIMAGE (X s1) (A s1) INTER p_space p) INTER
		 (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
		 (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
		 (PREIMAGE (X s4) (A s4) INTER p_space p)``)
      THEN1(RW_TAC std_ss [BIGINTER_4_sets])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num} /\ FINITE {s2:num;s3;s4} /\
                 DISJOINT {s1:num} {s2:num;s3;s4} /\
		 ({s1:num;s2;s3;s4 } = {s1} UNION {s2;s3;s4})``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
           THEN1(METIS_TAC[ALL_DISTINCT, MEM])
	   THEN1(METIS_TAC[ALL_DISTINCT, MEM])
	   THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s2:num} /\ FINITE {s3:num;s4} /\
                 DISJOINT {s2:num} {s3:num;s4} /\
		 ({s2:num;s3;s4 } = {s2} UNION {s3;s4})``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
          THEN1(METIS_TAC[ALL_DISTINCT, MEM])
          THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s3:num} /\ FINITE {s4:num} /\
                 DISJOINT {s3:num} {s4:num} /\
		 ({s3:num;s4 } = {s3} UNION {s4})``)
     THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
            THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN RW_TAC std_ss [PRODUCT_SING]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\                  random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\                  random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) /\
                 random_variable ((X:num->'a->'b) s4) p (m_space (M s4), measurable_sets (M s4)) ``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars]
           THEN UNDISCH_TAC(`` !i:num.
                                  i IN ii ==>
				  random_variable ((X:num->'a->'b) i) p (m_space (M i),
				    measurable_sets (M i))``)
				  
           THEN DISCH_TAC
	   THEN RW_TAC std_ss[]
	        THEN1 (POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	               THEN ASM_SET_TAC[])
	        THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
		      THEN ASM_SET_TAC[])
		THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
		      THEN ASM_SET_TAC[])
                THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
		THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s4) (A s4) INTER p_space p IN events p``)
     THEN1 (REWRITE_TAC [events_def]
            THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
                 THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
            THEN DISCH_TAC
            THEN ONCE_ASM_REWRITE_TAC[]
	    THEN REWRITE_TAC [p_space_def, m_space_def]
	    THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
                              (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
		  	      (A:num->'b->bool) s3 IN measurable_sets (M s3) /\
		  	      (A:num->'b->bool) s4 IN measurable_sets (M s4)``)
     	         THEN1(RW_TAC std_ss[]
           	       THEN1( UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4} ==>
	                                      (A:num->'b->bool) i IN measurable_sets (M i)``)
		  	      THEN DISCH_TAC
		  	      THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
		  THEN ASM_SET_TAC[])
	        THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4} ==>
	                              (A:num->'b->bool) i IN measurable_sets (M i)``)
                      THEN DISCH_TAC
		      THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
		      THEN ASM_SET_TAC[])
               THEN1( UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4} ==>
	                              (A:num->'b->bool) i IN measurable_sets (M i)``)
		      THEN DISCH_TAC
		      THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
		      THEN ASM_SET_TAC[])
              THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4} ==>
	                          (A:num->'b->bool) i IN measurable_sets (M i)``)
	      THEN DISCH_TAC
	      THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	      THEN ASM_SET_TAC[])
         THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_4, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);



val prob_prod_any_4_of_10_set = store_thm("prob_prod_any_4_of_10_set",
  ``! p ff  ii A  s1 s2 s3 s4.
       ALL_DISTINCT [s1;s2;s3;s4:num] /\ prob_space p /\
       indep_sets p ff ii /\ {s1; s2; s3; s4:num} SUBSET ii /\
       (!i. i IN {s1;s2;s3;s4:num} ==> A i  IN ff i ) /\
       (!i. i IN {s1;s2;s3;s4:num} ==> A i IN events p) ==>
       (prob p (A s1 INTER A s2 INTER A s3 INTER A s4) =
        prob p (A s1) * prob p (A s2) * prob p (A s3) * prob p (A s4))  ``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s4;s4}``)
THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s1:num;s2;s3;s4} }) =
                  Normal (product {s1:num;s2;s3;s4} (\j. real (prob p (A j)))))``)
     THEN1(DEP_REWRITE_TAC [indep_setsD]
           THEN Q.EXISTS_TAC `ff`
	   THEN Q.EXISTS_TAC `ii`
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s1:num; s2;s3;s4}}) =
                  A s1 INTER A s2 INTER A s3 INTER A s4``)
     THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num} /\ FINITE {s2:num;s3;s4} /\
                 DISJOINT {s1:num} {s2:num;s3;s4} /\
		 ({s1:num;s2;s3;s4 } = {s1} UNION {s2;s3;s4})``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
           THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s2:num} /\ FINITE {s3:num;s4} /\
                 DISJOINT {s2:num} {s3:num;s4} /\
		 ({s2:num;s3;s4 } = {s2} UNION {s3;s4})``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
           THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN KNOW_TAC(``FINITE {s3:num} /\ FINITE {s4:num} /\
                 DISJOINT {s3:num} {s4:num} /\
		 ({s3:num;s4 } = {s3} UNION {s4})``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]
           THEN METIS_TAC[ALL_DISTINCT, MEM])
THEN RW_TAC std_ss [PRODUCT_UNION]
THEN RW_TAC std_ss [PRODUCT_SING]
THEN KNOW_TAC(``(A:num->'a->bool) s1 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4} ==>
                          A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s2 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4} ==>
                          A i IN events p``)
 	  THEN DISCH_TAC
	  THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	  THEN DISCH_TAC
	  THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s3 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s4 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_4, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);


val prob_prod_any_5_of_10 = store_thm("prob_prod_any_5_of_10",
  ``
  !p M X ii A  s1 s2 s3 s4 s5.
     ALL_DISTINCT [s1;s2;s3;s4;s5]  /\ prob_space p /\  indep_vars p M X ii /\
     {s1;s2;s3;s4;s5} SUBSET ii /\ (!i. i IN {s1;s2;s3;s4;s5} ==> A i IN measurable_sets (M i)) ==>
     (prob p
        (PREIMAGE (X s1) (A s1) INTER p_space p INTER
         (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
	 (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
	 (PREIMAGE (X s4) (A s4) INTER p_space p) INTER
         (PREIMAGE (X s5) (A s5) INTER p_space p)) =
      prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
      prob p (PREIMAGE (X s2) (A s2) INTER p_space p)*
      prob p (PREIMAGE (X s3) (A s3) INTER p_space p)*
      prob p (PREIMAGE (X s4) (A s4) INTER p_space p)*
      prob p (PREIMAGE (X s5) (A s5) INTER p_space p))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3;s4;s5}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
        (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p | i IN {s1:num;s2;s3;s4;s5}}) =
          Normal
           (product {s1:num;s2;s3;s4;s5}
            (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
     THEN1(DEP_REWRITE_TAC [indep_varsD]
           THEN Q.EXISTS_TAC `M`
	   THEN Q.EXISTS_TAC `ii`
	   THEN RW_TAC std_ss[]
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p | i IN {s1:num;s2;s3;s4;s5}} =
                 {PREIMAGE (X s1) (A s1) INTER p_space p;
		  PREIMAGE (X s2) (A s2) INTER p_space p;
		  PREIMAGE (X s3) (A s3) INTER p_space p;
		  PREIMAGE (X s4) (A s4) INTER p_space p;
		  PREIMAGE (X s5) (A s5) INTER p_space p}``)
    THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[BIGINTER_5_sets]
THEN RW_TAC std_ss[PRODUCT_UNION_5]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\
                 random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\
		 random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) /\
		 random_variable ((X:num->'a->'b) s4) p (m_space (M s4), measurable_sets (M s4)) /\
		 random_variable ((X:num->'a->'b) s5) p (m_space (M s5), measurable_sets (M s5)) ``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars]
           THEN UNDISCH_TAC(`` !i:num.
           		         i IN ii ==>
        			 random_variable ((X:num->'a->'b) i) p (m_space (M i),
				    measurable_sets (M i))``)
	   THEN DISCH_TAC
	   THEN RW_TAC std_ss[]
	   THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	         THEN ASM_SET_TAC[])
	   THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	         THEN ASM_SET_TAC[])
		 
           THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	         THEN ASM_SET_TAC[])
           THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	         THEN ASM_SET_TAC[])
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s4) (A s4) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s5) (A s5) INTER p_space p IN events p``)
     THEN1(REWRITE_TAC [events_def]
           THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	        THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
	   THEN DISCH_TAC
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN REWRITE_TAC [p_space_def, m_space_def]
	   THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
	                     (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
			     (A:num->'b->bool) s3 IN measurable_sets (M s3) /\
			     (A:num->'b->bool) s4 IN measurable_sets (M s4) /\
			     (A:num->'b->bool) s5 IN measurable_sets (M s5)``)
		THEN1(RW_TAC std_ss[]
		      THEN1 (UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5} ==>
		                            (A:num->'b->bool) i IN measurable_sets (M i)``)
		             THEN DISCH_TAC
		             THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
 			     THEN ASM_SET_TAC[])
                     THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5} ==>
		                           (A:num->'b->bool) i IN measurable_sets (M i)``)
			   THEN DISCH_TAC
			   THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
			   THEN ASM_SET_TAC[])
		     THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			   THEN DISCH_TAC
			   THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
			   THEN ASM_SET_TAC[])
		     THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5} ==>
		                           (A:num->'b->bool) i IN measurable_sets (M i)``)
			   THEN DISCH_TAC
			   THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
			   THEN ASM_SET_TAC[])
		     THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5} ==>
		                           (A:num->'b->bool) i IN measurable_sets (M i)``)
		     THEN DISCH_TAC
		     THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
		     THEN ASM_SET_TAC[])
               THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_5, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);


val prob_prod_any_5_of_10_set = store_thm("prob_prod_any_5_of_10_set",
  ``! p ff  ii A  s1 s2 s3 s4 s5.
      ALL_DISTINCT [s1:num;s2;s3;s4;s5] /\ prob_space p /\
      indep_sets p ff ii /\ {s1:num;s2;s3;s4;s5} SUBSET ii /\
      (!i. i IN {s1:num;s2;s3;s4;s5} ==> A i  IN ff i ) /\
      (!i. i IN {s1:num;s2;s3;s4;s5} ==> A i IN events p) ==>
      (prob p (A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5) =
       prob p (A s1) * prob p (A s2) * prob p (A s3) * prob p (A s4) * prob p (A s5))  ``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s4;s4;s5}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s1:num;s2;s3;s4;s5} }) =
                  Normal (product {s1:num;s2;s3;s4;s5} (\j. real (prob p (A j)))))``)
     THEN1(DEP_REWRITE_TAC [indep_setsD]
           THEN Q.EXISTS_TAC `ff`
	   THEN Q.EXISTS_TAC `ii`
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s1:num; s2;s3;s4;s5}}) =
                 A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5``)
     THEN1( ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN RW_TAC std_ss[PRODUCT_UNION_5]
THEN KNOW_TAC(``(A:num->'a->bool) s1 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s2 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s3 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s4 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s5 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5} ==> A i IN events p``)
           THEN DISCH_TAC
           THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_5, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);



val prob_prod_any_6_of_10 = store_thm("prob_prod_any_6_of_10",
  `` !p M X ii A  s1 s2 s3 s4 s5 s6.
      ALL_DISTINCT [s1;s2;s3;s4;s5;s6]  /\ prob_space p /\
      indep_vars p M X ii /\ {s1;s2;s3;s4;s5;s6} SUBSET ii /\
      (!i. i IN {s1;s2;s3;s4;s5;s6} ==> A i IN measurable_sets (M i)) ==>
      (prob p
        (PREIMAGE (X s1) (A s1) INTER p_space p INTER
         (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
	 (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
	 (PREIMAGE (X s4) (A s4) INTER p_space p) INTER
         (PREIMAGE (X s5) (A s5) INTER p_space p) INTER
         (PREIMAGE (X s6) (A s6) INTER p_space p)) =
      prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
      prob p (PREIMAGE (X s2) (A s2) INTER p_space p)*
      prob p (PREIMAGE (X s3) (A s3) INTER p_space p)*
      prob p (PREIMAGE (X s4) (A s4) INTER p_space p)*
      prob p (PREIMAGE (X s5) (A s5) INTER p_space p) *
      prob p (PREIMAGE (X s6) (A s6) INTER p_space p))``,


RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3;s4;s5;s6}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p
                  (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p | i IN {s1:num;s2;s3;s4;s5;s6}}) =
                    Normal
        	    (product {s1:num;s2;s3;s4;s5;s6}
           	    (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
     THEN1(DEP_REWRITE_TAC [indep_varsD]
          THEN Q.EXISTS_TAC `M`
	  THEN Q.EXISTS_TAC `ii`
	  THEN RW_TAC std_ss[]
	  THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p | i IN {s1;s2;s3;s4;s5;s6}} =
                 {PREIMAGE (X (s1:num)) (A s1) INTER p_space p;
		  PREIMAGE (X s2) (A s2) INTER p_space p;
		  PREIMAGE (X s3) (A s3) INTER p_space p;
		  PREIMAGE (X s4) (A s4) INTER p_space p;
		  PREIMAGE (X s5) (A s5) INTER p_space p;
		  PREIMAGE (X s6) (A s6) INTER p_space p}``)
     THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[BIGINTER_6_sets]
THEN RW_TAC std_ss[PRODUCT_UNION_6]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\
                 random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\
		 random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) /\
		 random_variable ((X:num->'a->'b) s4) p (m_space (M s4), measurable_sets (M s4)) /\
                 random_variable ((X:num->'a->'b) s5) p (m_space (M s5), measurable_sets (M s5)) /\
		 random_variable ((X:num->'a->'b) s6) p (m_space (M s6), measurable_sets (M s6)) ``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars]
           THEN UNDISCH_TAC(`` !i:num.
           			 i IN ii ==>
				 random_variable ((X:num->'a->'b) i) p (m_space (M i),
				     measurable_sets (M i))``)
	   THEN DISCH_TAC
	   THEN RW_TAC std_ss[]
	        THEN1( POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	        THEN ASM_SET_TAC[])
	   THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	         THEN ASM_SET_TAC[])
	   THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	         THEN ASM_SET_TAC[])
	   THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	         THEN ASM_SET_TAC[])
	   THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	         THEN ASM_SET_TAC[])
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
	        THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s4) (A s4) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s5) (A s5) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s6) (A s6) INTER p_space p IN events p``)
      THEN1(REWRITE_TAC [events_def]
            THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	         THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
	    THEN DISCH_TAC
	    THEN ONCE_ASM_REWRITE_TAC[]
	    THEN REWRITE_TAC [p_space_def, m_space_def]
            THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
	                      (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
			      (A:num->'b->bool) s3 IN measurable_sets (M s3) /\
			      (A:num->'b->bool) s4 IN measurable_sets (M s4) /\
			      (A:num->'b->bool) s5 IN measurable_sets (M s5) /\
			      (A:num->'b->bool) s6 IN measurable_sets (M s6)``)
                 THEN1(RW_TAC std_ss[]
		       THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
			     THEN ASM_SET_TAC[])
        	       THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6} ==>
		                              (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
			     THEN ASM_SET_TAC[])
		       THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
			     THEN ASM_SET_TAC[])
		       THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6} ==>
		             (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
			     THEN ASM_SET_TAC[])
		       THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
			     THEN ASM_SET_TAC[])
		THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6} ==>
		                     (A:num->'b->bool) i IN measurable_sets (M i)``)
		THEN DISCH_TAC
		THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
		THEN ASM_SET_TAC[])
       THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_6, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);


val prob_prod_any_6_of_10_set = store_thm("prob_prod_any_6_of_10_set",
  ``! p ff ii A  s1 s2 s3 s4 s5 s6.
        ALL_DISTINCT [s1:num;s2;s3;s4;s5;s6] /\ prob_space p /\
	indep_sets p ff ii /\ {s1:num;s2;s3;s4;s5;s6} SUBSET ii /\
	(!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i  IN ff i ) /\
	(!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i IN events p) ==>
	(prob p
	   (A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER A s6) =
         prob p (A s1) * prob p (A s2) * prob p (A s3) * prob p (A s4) *
	 prob p (A s5) * prob p (A s6))  ``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s4;s4;s5;s6}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s1:num;s2;s3;s4;s5;s6} }) =
                   Normal (product {s1:num;s2;s3;s4;s5;s6} (\j. real (prob p (A j)))))``)
     THEN1(DEP_REWRITE_TAC [indep_setsD]
           THEN Q.EXISTS_TAC `ff`
	   THEN Q.EXISTS_TAC `ii`
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s1:num; s2;s3;s4;s5;s6}}) =
                 A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER A s6``)
     THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN RW_TAC std_ss[PRODUCT_UNION_6]
THEN KNOW_TAC(``(A:num->'a->bool) s1 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s2 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s3 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s4 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s5 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s6 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_6, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);


val prob_prod_any_7_of_10 = store_thm("prob_prod_any_7_of_10",
  ``!p M X ii A  s1 s2 s3 s4 s5 s6 s7.
        ALL_DISTINCT [s1;s2;s3;s4;s5;s6;s7]  /\ prob_space p /\
	indep_vars p M X ii /\ {s1;s2;s3;s4;s5;s6;s7} SUBSET ii /\
	(!i. i IN {s1;s2;s3;s4;s5;s6;s7} ==> A i IN measurable_sets (M i)) ==>
       (prob p
        (PREIMAGE (X s1) (A s1) INTER p_space p INTER
         (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
	 (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
	 (PREIMAGE (X s4) (A s4) INTER p_space p) INTER
         (PREIMAGE (X s5) (A s5) INTER p_space p) INTER
         (PREIMAGE (X s6) (A s6) INTER p_space p) INTER
         (PREIMAGE (X s7) (A s7) INTER p_space p)) =
      prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
      prob p (PREIMAGE (X s2) (A s2) INTER p_space p)*
      prob p (PREIMAGE (X s3) (A s3) INTER p_space p)*
      prob p (PREIMAGE (X s4) (A s4) INTER p_space p)*
      prob p (PREIMAGE (X s5) (A s5) INTER p_space p) *
      prob p (PREIMAGE (X s6) (A s6) INTER p_space p) *
      prob p (PREIMAGE (X s7) (A s7) INTER p_space p))``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3;s4;s5;s6;s7}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
                  (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p |
		                          i IN {s1:num;s2;s3;s4;s5;s6;s7}}) =
                   Normal
                   (product {s1:num;s2;s3;s4;s5;s6;s7}
                      (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
     THEN1(DEP_REWRITE_TAC [indep_varsD]
           THEN Q.EXISTS_TAC `M`
	   THEN Q.EXISTS_TAC `ii`
	   THEN RW_TAC std_ss[]
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p |
                   i IN {s1:num;s2;s3;s4;s5;s6;s7}} =
		 {PREIMAGE (X s1) (A s1) INTER p_space p;
		  PREIMAGE (X s2) (A s2) INTER p_space p;
		  PREIMAGE (X s3) (A s3) INTER p_space p;
		  PREIMAGE (X s4) (A s4) INTER p_space p;
		  PREIMAGE (X s5) (A s5) INTER p_space p;
		  PREIMAGE (X s6) (A s6) INTER p_space p;
		  PREIMAGE (X s7) (A s7) INTER p_space p}``)
      THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[BIGINTER_7_sets]
THEN RW_TAC std_ss[PRODUCT_UNION_7]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\
                 random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\
		 random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) /\
		 random_variable ((X:num->'a->'b) s4) p (m_space (M s4), measurable_sets (M s4)) /\
		 random_variable ((X:num->'a->'b) s5) p (m_space (M s5), measurable_sets (M s5)) /\
		 random_variable ((X:num->'a->'b) s6) p (m_space (M s6), measurable_sets (M s6)) /\
		 random_variable ((X:num->'a->'b) s7) p (m_space (M s7), measurable_sets (M s7)) ``)
      THEN1(FULL_SIMP_TAC std_ss [indep_vars]
            THEN UNDISCH_TAC(`` !i:num.
            	 		   i IN ii ==>
                                 random_variable ((X:num->'a->'b) i) p (m_space (M i),
				         measurable_sets (M i))``)
	    THEN DISCH_TAC
	    THEN RW_TAC std_ss[]
	         THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
		       THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
		       THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
		       THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
		       THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
		       THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
		       THEN ASM_SET_TAC[])
		 THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
		 THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s4) (A s4) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s5) (A s5) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s6) (A s6) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s7) (A s7) INTER p_space p IN events p``)
     THEN1(REWRITE_TAC [events_def]
           THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	        THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
	   THEN DISCH_TAC
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN REWRITE_TAC [p_space_def, m_space_def]
	   THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
	                     (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
			     (A:num->'b->bool) s3 IN measurable_sets (M s3) /\
			     (A:num->'b->bool) s4 IN measurable_sets (M s4) /\
			     (A:num->'b->bool) s5 IN measurable_sets (M s5) /\
			     (A:num->'b->bool) s6 IN measurable_sets (M s6) /\
			     (A:num->'b->bool) s7 IN measurable_sets (M s7)``)
		THEN1(RW_TAC std_ss[]
		      THEN1( UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7} ==>
		                              (A:num->'b->bool) i IN measurable_sets (M i)``)
			      THEN DISCH_TAC
			      THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
			      THEN ASM_SET_TAC[])
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
                 	     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
			     THEN ASM_SET_TAC[])
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			    THEN DISCH_TAC
			    THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
			    THEN ASM_SET_TAC[])
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7} ==>
		                            (A:num->'b->bool) i IN measurable_sets (M i)``)
			    THEN DISCH_TAC
			    THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
			    THEN ASM_SET_TAC[])
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7} ==>
		                            (A:num->'b->bool) i IN measurable_sets (M i)``)
			    THEN DISCH_TAC
			    THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
			    THEN ASM_SET_TAC[])
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			     THEN DISCH_TAC
			     THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
			     THEN ASM_SET_TAC[])
		      THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7} ==>
		                           (A:num->'b->bool) i IN measurable_sets (M i)``)
		      THEN DISCH_TAC
		      THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
		      THEN ASM_SET_TAC[])
	        THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_7, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);

val prob_prod_any_7_of_10_set = store_thm("prob_prod_any_7_of_10_set",
  ``! p ff ii A s1 s2 s3 s4 s5 s6 s7.
       ALL_DISTINCT [s1:num;s2;s3;s4;s5;s6;s7] /\ prob_space p /\
       indep_sets p ff ii /\ {s1:num;s2;s3;s4;s5;s6;s7} SUBSET ii /\
       (!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i  IN ff i ) /\
       (!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p) ==>
       (prob p
           (A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER
	    A s6 INTER A s7) =
        prob p (A s1) * prob p (A s2) * prob p (A s3) * prob p (A s4) *
	prob p (A s5) * prob p (A s6)* prob p (A s7))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s4;s4;s5;s6;s7}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s1:num;s2;s3;s4;s5;s6;s7} }) =
                  Normal (product {s1:num;s2;s3;s4;s5;s6;s7} (\j. real (prob p (A j)))))``)
     THEN1(DEP_REWRITE_TAC [indep_setsD]
           THEN Q.EXISTS_TAC `ff`
	   THEN Q.EXISTS_TAC `ii`
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s1:num; s2;s3;s4;s5;s6;s7}}) =
                 A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER A s6 INTER A s7``)
    THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN RW_TAC std_ss[PRODUCT_UNION_7]
THEN KNOW_TAC(``(A:num->'a->bool) s1 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s2 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s3 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s4 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s5 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s6 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s7 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_7, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);



val prob_prod_any_8_of_10 = store_thm("prob_prod_any_8_of_10",
  `` !p M X ii A  s1 s2 s3 s4 s5 s6 s7 s8.
       ALL_DISTINCT [s1;s2;s3;s4;s5;s6;s7;s8]  /\ prob_space p /\
       indep_vars p M X ii /\ {s1;s2;s3;s4;s5;s6;s7;s8} SUBSET ii /\
       (!i.
          i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
	  A i IN measurable_sets (M i)) ==>
       (prob p
        (PREIMAGE (X s1) (A s1) INTER p_space p INTER
         (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
	 (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
	 (PREIMAGE (X s4) (A s4) INTER p_space p) INTER
         (PREIMAGE (X s5) (A s5) INTER p_space p) INTER
         (PREIMAGE (X s6) (A s6) INTER p_space p) INTER
         (PREIMAGE (X s7) (A s7) INTER p_space p) INTER
	 (PREIMAGE (X s8) (A s8) INTER p_space p)) =
      prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
      prob p (PREIMAGE (X s2) (A s2) INTER p_space p)*
      prob p (PREIMAGE (X s3) (A s3) INTER p_space p)*
      prob p (PREIMAGE (X s4) (A s4) INTER p_space p)*
      prob p (PREIMAGE (X s5) (A s5) INTER p_space p) *
      prob p (PREIMAGE (X s6) (A s6) INTER p_space p) *
      prob p (PREIMAGE (X s7) (A s7) INTER p_space p) *
      prob p (PREIMAGE (X s8) (A s8) INTER p_space p))``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3;s4;s5;s6;s7;s8}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
                    (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p |
		     i IN {s1:num;s2;s3;s4;s5;s6;s7;s8}}) =
                   Normal
                    (product {s1:num;s2;s3;s4;s5;s6;s7;s8}
           	    (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
    THEN1(DEP_REWRITE_TAC [indep_varsD]
          THEN Q.EXISTS_TAC `M`
	  THEN Q.EXISTS_TAC `ii`
	  THEN RW_TAC std_ss[]
	  THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p |
                   i IN {s1:num;s2;s3;s4;s5;s6;s7;s8}} =
		  {PREIMAGE (X s1) (A s1) INTER p_space p;
		   PREIMAGE (X s2) (A s2) INTER p_space p;
		   PREIMAGE (X s3) (A s3) INTER p_space p;
		   PREIMAGE (X s4) (A s4) INTER p_space p;
		   PREIMAGE (X s5) (A s5) INTER p_space p;
		   PREIMAGE (X s6) (A s6) INTER p_space p;
		   PREIMAGE (X s7) (A s7) INTER p_space p;
		   PREIMAGE (X s8) (A s8) INTER p_space p}``)
     THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[BIGINTER_8_sets]
THEN RW_TAC std_ss[PRODUCT_UNION_8]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\
                 random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\
		 random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) /\
		 random_variable ((X:num->'a->'b) s4) p (m_space (M s4), measurable_sets (M s4)) /\
		 random_variable ((X:num->'a->'b) s5) p (m_space (M s5), measurable_sets (M s5)) /\
		 random_variable ((X:num->'a->'b) s6) p (m_space (M s6), measurable_sets (M s6)) /\
		 random_variable ((X:num->'a->'b) s7) p (m_space (M s7), measurable_sets (M s7)) /\
		 random_variable ((X:num->'a->'b) s8) p (m_space (M s8), measurable_sets (M s8))  ``)
    THEN1(FULL_SIMP_TAC std_ss [indep_vars]
          THEN UNDISCH_TAC(`` !i:num.
                                    i IN ii ==>
                                    random_variable ((X:num->'a->'b) i) p (m_space (M i),
				    measurable_sets (M i))``)
	  THEN DISCH_TAC
	  THEN RW_TAC std_ss[]
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	         THEN ASM_SET_TAC[])
          THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	        THEN ASM_SET_TAC[])
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	       THEN ASM_SET_TAC[])
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	        THEN ASM_SET_TAC[])
          THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	        THEN ASM_SET_TAC[])
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
	        THEN ASM_SET_TAC[])
	  THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
	        THEN ASM_SET_TAC[])
	  THEN POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
	  THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s4) (A s4) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s5) (A s5) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s6) (A s6) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s7) (A s7) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s8) (A s8) INTER p_space p IN events p``)
     THEN1(REWRITE_TAC [events_def]
           THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	        THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
           THEN DISCH_TAC
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN REWRITE_TAC [p_space_def, m_space_def]
	   THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
	                     (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
			     (A:num->'b->bool) s3 IN measurable_sets (M s3) /\
			     (A:num->'b->bool) s4 IN measurable_sets (M s4) /\
			     (A:num->'b->bool) s5 IN measurable_sets (M s5) /\
			     (A:num->'b->bool) s6 IN measurable_sets (M s6) /\
			     (A:num->'b->bool) s7 IN measurable_sets (M s7) /\
			     (A:num->'b->bool) s8 IN measurable_sets (M s8)``)
		THEN1(RW_TAC std_ss[]
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			    THEN DISCH_TAC
			    THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
			    THEN ASM_SET_TAC[])
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		                            (A:num->'b->bool) i IN measurable_sets (M i)``)
			    THEN DISCH_TAC
			    THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
			    THEN ASM_SET_TAC[])
		     THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		     			   (A:num->'b->bool) i IN measurable_sets (M i)``)
			   THEN DISCH_TAC
			   THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
			   THEN ASM_SET_TAC[])
		    THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		                           (A:num->'b->bool) i IN measurable_sets (M i)``)
			  THEN DISCH_TAC
			  THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
			  THEN ASM_SET_TAC[])
		   THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		                         (A:num->'b->bool) i IN measurable_sets (M i)``)
			THEN DISCH_TAC
			THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
			THEN ASM_SET_TAC[])
		   THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		                         (A:num->'b->bool) i IN measurable_sets (M i)``)
			 THEN DISCH_TAC
			 THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
			 THEN ASM_SET_TAC[])
		   THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		                         (A:num->'b->bool) i IN measurable_sets (M i)``)
                         THEN DISCH_TAC
			 THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
			 THEN ASM_SET_TAC[])
                   THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8} ==>
		                         (A:num->'b->bool) i IN measurable_sets (M i)``)
		   THEN DISCH_TAC
		   THEN POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
		   THEN ASM_SET_TAC[])
           THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_8, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);


val prob_prod_any_8_of_10_set = store_thm("prob_prod_any_8_of_10_set",
  ``! p ff ii A s1 s2 s3 s4 s5 s6 s7 s8.
       ALL_DISTINCT [s1:num;s2;s3;s4;s5;s6;s7;s8] /\ prob_space p /\
       indep_sets p ff ii /\ {s1:num;s2;s3;s4;s5;s6;s7;s8} SUBSET ii /\
       (!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i  IN ff i ) /\
       (!i.
          i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p) ==>
      (prob p
         (A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER
	  A s6 INTER A s7 INTER A s8) =
       prob p (A s1) * prob p (A s2) * prob p (A s3) * prob p (A s4) *
       prob p (A s5) * prob p (A s6)* prob p (A s7) * prob p (A s8)  ) ``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s4;s4;s5;s6;s7;s8}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s1:num;s2;s3;s4;s5;s6;s7;s8} }) =
                  Normal (product {s1:num;s2;s3;s4;s5;s6;s7;s8} (\j. real (prob p (A j)))))``)
     THEN1(DEP_REWRITE_TAC [indep_setsD]
           THEN Q.EXISTS_TAC `ff`
	   THEN Q.EXISTS_TAC `ii`
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s1:num; s2;s3;s4;s5;s6;s7;s8}}) =
                   A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER
		   A s6 INTER A s7 INTER A s8``)
    THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN RW_TAC std_ss[PRODUCT_UNION_8]
THEN KNOW_TAC(``(A:num->'a->bool) s1 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s2 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s3 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s4 IN events p``)
     THEN1( UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
            THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	    THEN DISCH_TAC
	    THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s5 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s6 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s7 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s8 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_8, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);



val prob_prod_any_9_of_10 = store_thm("prob_prod_any_9_of_10",
  `` !p M X ii A  s1 s2 s3 s4 s5 s6 s7 s8 s9.
       ALL_DISTINCT [s1;s2;s3;s4;s5;s6;s7;s8;s9]  /\ prob_space p /\
       indep_vars p M X ii /\ {s1;s2;s3;s4;s5;s6;s7;s8;s9} SUBSET ii /\
       (!i.
          i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
	  A i IN measurable_sets (M i)) ==>
     (prob p
        (PREIMAGE (X s1) (A s1) INTER p_space p INTER
         (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
	 (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
	 (PREIMAGE (X s4) (A s4) INTER p_space p) INTER
         (PREIMAGE (X s5) (A s5) INTER p_space p) INTER
         (PREIMAGE (X s6) (A s6) INTER p_space p) INTER
         (PREIMAGE (X s7) (A s7) INTER p_space p) INTER
	 (PREIMAGE (X s8) (A s8) INTER p_space p) INTER
	 (PREIMAGE (X s9) (A s9) INTER p_space p)) =
      prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
      prob p (PREIMAGE (X s2) (A s2) INTER p_space p)*
      prob p (PREIMAGE (X s3) (A s3) INTER p_space p)*
      prob p (PREIMAGE (X s4) (A s4) INTER p_space p)*
      prob p (PREIMAGE (X s5) (A s5) INTER p_space p) *
      prob p (PREIMAGE (X s6) (A s6) INTER p_space p) *
      prob p (PREIMAGE (X s7) (A s7) INTER p_space p) *
      prob p (PREIMAGE (X s8) (A s8) INTER p_space p) *
      prob p (PREIMAGE (X s9) (A s9) INTER p_space p))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3;s4;s5;s6;s7;s8;s9}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
                   (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p |
		    i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9}}) =
      	   	   Normal
        	   (product {s1:num;s2;s3;s4;s5;s6;s7;s8;s9}
           	   (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
    THEN1(DEP_REWRITE_TAC [indep_varsD]
         THEN Q.EXISTS_TAC `M`
	 THEN Q.EXISTS_TAC `ii`
	 THEN RW_TAC std_ss[]
	 THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p |
                  i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9}} =
		 {PREIMAGE (X s1) (A s1) INTER p_space p;
		  PREIMAGE (X s2) (A s2) INTER p_space p;
		  PREIMAGE (X s3) (A s3) INTER p_space p;
		  PREIMAGE (X s4) (A s4) INTER p_space p;
		  PREIMAGE (X s5) (A s5) INTER p_space p;
		  PREIMAGE (X s6) (A s6) INTER p_space p;
		  PREIMAGE (X s7) (A s7) INTER p_space p;
		  PREIMAGE (X s8) (A s8) INTER p_space p;
		  PREIMAGE (X s9) (A s9) INTER p_space p}``)
     THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[BIGINTER_9_sets]
THEN RW_TAC std_ss[PRODUCT_UNION_9]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\
                 random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\
		 random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) /\
		 random_variable ((X:num->'a->'b) s4) p (m_space (M s4), measurable_sets (M s4)) /\
		 random_variable ((X:num->'a->'b) s5) p (m_space (M s5), measurable_sets (M s5)) /\
		 random_variable ((X:num->'a->'b) s6) p (m_space (M s6), measurable_sets (M s6)) /\
		 random_variable ((X:num->'a->'b) s7) p (m_space (M s7), measurable_sets (M s7)) /\
		 random_variable ((X:num->'a->'b) s8) p (m_space (M s8), measurable_sets (M s8)) /\
		 random_variable ((X:num->'a->'b) s9) p (m_space (M s9), measurable_sets (M s9)) ``)
     THEN1(FULL_SIMP_TAC std_ss [indep_vars]
           THEN UNDISCH_TAC(`` !i:num.
           			     i IN ii ==>
 				     random_variable ((X:num->'a->'b) i) p (m_space (M i),
				           measurable_sets (M i))``)
           THEN DISCH_TAC
           THEN RW_TAC std_ss[]
	        THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
		      THEN ASM_SET_TAC[])
	        THEN1 (POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
		       THEN ASM_SET_TAC[])
		THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
		      THEN ASM_SET_TAC[])
		THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
		      THEN ASM_SET_TAC[])
		THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
		      THEN ASM_SET_TAC[])
		THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
		      THEN ASM_SET_TAC[])
		THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
		      THEN ASM_SET_TAC[])
		THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
		      THEN ASM_SET_TAC[])
           THEN POP_ASSUM (MP_TAC o Q.SPEC `s9:num `)
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s4) (A s4) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s5) (A s5) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s6) (A s6) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s7) (A s7) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s8) (A s8) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s9) (A s9) INTER p_space p IN events p``)
     THEN1(REWRITE_TAC [events_def]
           THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	        THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
	   THEN DISCH_TAC
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN REWRITE_TAC [p_space_def, m_space_def]
	   THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
                 	     (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
			     (A:num->'b->bool) s3 IN measurable_sets (M s3) /\
			     (A:num->'b->bool) s4 IN measurable_sets (M s4) /\
			     (A:num->'b->bool) s5 IN measurable_sets (M s5) /\
			     (A:num->'b->bool) s6 IN measurable_sets (M s6) /\
			     (A:num->'b->bool) s7 IN measurable_sets (M s7) /\
			     (A:num->'b->bool) s8 IN measurable_sets (M s8) /\
			     (A:num->'b->bool) s9 IN measurable_sets (M s9)``)
		THEN1(RW_TAC std_ss[]
		      THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		                             (A:num->'b->bool) i IN measurable_sets (M i)``)
			    THEN DISCH_TAC
			    THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
			    THEN ASM_SET_TAC[])
		     THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		                           (A:num->'b->bool) i IN measurable_sets (M i)``)
			   THEN DISCH_TAC
			   THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
			   THEN ASM_SET_TAC[])
		    THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		                          (A:num->'b->bool) i IN measurable_sets (M i)``)
			  THEN DISCH_TAC
			  THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
			  THEN ASM_SET_TAC[])
		   THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		   			 (A:num->'b->bool) i IN measurable_sets (M i)``)
			 THEN DISCH_TAC
			 THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
			 THEN  ASM_SET_TAC[])
                   THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		   			 (A:num->'b->bool) i IN measurable_sets (M i)``)
		         THEN DISCH_TAC
			 THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
			 THEN ASM_SET_TAC[])
                  THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		                        (A:num->'b->bool) i IN measurable_sets (M i)``)
			THEN DISCH_TAC
			THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
			THEN ASM_SET_TAC[])
		  THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		  			(A:num->'b->bool) i IN measurable_sets (M i)``)
                        THEN DISCH_TAC
			THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
			THEN ASM_SET_TAC[])
		  THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		  			(A:num->'b->bool) i IN measurable_sets (M i)``)
			THEN DISCH_TAC
			THEN POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
			THEN ASM_SET_TAC[])
		 THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9} ==>
		 		       (A:num->'b->bool) i IN measurable_sets (M i)``)
		 THEN DISCH_TAC
		 THEN POP_ASSUM (MP_TAC o Q.SPEC `s9:num `)
		 THEN ASM_SET_TAC[])
          THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_9, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);


val prob_prod_any_9_of_10_set = store_thm("prob_prod_any_9_of_10_set",
  ``! p ff  ii A  s1 s2 s3 s4 s5 s6 s7 s8 s9.
       ALL_DISTINCT [s1:num;s2;s3;s4;s5;s6;s7;s8;s9] /\ prob_space p /\
       indep_sets p ff ii /\ {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} SUBSET ii /\
       (!i.
          i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i  IN ff i ) /\
       (!i.
          i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==>
	  A i IN events p) ==>
       (prob p 
          (A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER
	   A s6 INTER A s7 INTER A s8 INTER A s9) =
        prob p (A s1) * prob p (A s2) * prob p (A s3) * prob p (A s4) *
	prob p (A s5) * prob p (A s6) * prob p (A s7) * prob p (A s8) *
	prob p ( A s9) ) ``,
RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s4;s4;s5;s6;s7;s8;s9}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(``(prob p (BIGINTER {A j | j IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} }) =
                 Normal (product {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} (\j. real (prob p (A j)))))``)
     THEN1(DEP_REWRITE_TAC [indep_setsD]
           THEN Q.EXISTS_TAC `ff`
	   THEN Q.EXISTS_TAC `ii`
	   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(BIGINTER {A j | j IN {s1:num; s2;s3;s4;s5;s6;s7;s8;s9}}) =
                  A s1 INTER A s2 INTER A s3 INTER A s4 INTER A s5 INTER
		  A s6 INTER A s7 INTER A s8 INTER A s9``)
     THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN RW_TAC std_ss[PRODUCT_UNION_9]
THEN KNOW_TAC(``(A:num->'a->bool) s1 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s2 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s3 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s4 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s5 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s6 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s7 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s8 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``(A:num->'a->bool) s9 IN events p``)
     THEN1(UNDISCH_TAC(``!i. i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9} ==> A i IN events p``)
           THEN DISCH_TAC
	   THEN POP_ASSUM (MP_TAC o Q.SPEC `s9:num `)
	   THEN DISCH_TAC
	   THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_9, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);

val prob_prod_10_of_10 = store_thm("prob_prod_10_of_10",
  `` !p M X ii A  s1 s2 s3 s4 s5 s6 s7 s8 s9 s10.
       ALL_DISTINCT [s1;s2;s3;s4;s5;s6;s7;s8;s9;s10]  /\
       prob_space p /\  indep_vars p M X ii /\
       {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} SUBSET ii /\
       (!i.
          i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
	  A i IN measurable_sets (M i)) ==>
      (prob p
        (PREIMAGE (X s1) (A s1) INTER p_space p INTER
         (PREIMAGE (X s2) (A s2) INTER p_space p) INTER
	 (PREIMAGE (X s3) (A s3) INTER p_space p) INTER
	 (PREIMAGE (X s4) (A s4) INTER p_space p) INTER
         (PREIMAGE (X s5) (A s5) INTER p_space p) INTER
         (PREIMAGE (X s6) (A s6) INTER p_space p) INTER
         (PREIMAGE (X s7) (A s7) INTER p_space p) INTER
	 (PREIMAGE (X s8) (A s8) INTER p_space p) INTER
	 (PREIMAGE (X s9) (A s9) INTER p_space p) INTER
	 (PREIMAGE (X s10) (A s10) INTER p_space p)) =
      prob p (PREIMAGE (X s1) (A s1) INTER p_space p) *
      prob p (PREIMAGE (X s2) (A s2) INTER p_space p)*
      prob p (PREIMAGE (X s3) (A s3) INTER p_space p)*
      prob p (PREIMAGE (X s4) (A s4) INTER p_space p)*
      prob p (PREIMAGE (X s5) (A s5) INTER p_space p) *
      prob p (PREIMAGE (X s6) (A s6) INTER p_space p) *
      prob p (PREIMAGE (X s7) (A s7) INTER p_space p) *
      prob p (PREIMAGE (X s8) (A s8) INTER p_space p) *
      prob p (PREIMAGE (X s9) (A s9) INTER p_space p) *
      prob p (PREIMAGE (X s10) (A s10) INTER p_space p))``,


RW_TAC std_ss[]
THEN KNOW_TAC(``FINITE {s1:num;s2;s3;s4;s5;s6;s7;s8;s9;s10}``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, FINITE_DEF])
THEN DISCH_TAC
THEN KNOW_TAC(`` (prob p
                    (BIGINTER {PREIMAGE (X i) (A i) INTER p_space p |
		     i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9;s10}}) =
                  Normal
        	   (product {s1:num;s2;s3;s4;s5;s6;s7;s8;s9;s10}
           	    (\i. real (prob p (PREIMAGE (X i) (A i) INTER p_space p)))))``)
    THEN1(DEP_REWRITE_TAC [indep_varsD]
          THEN Q.EXISTS_TAC `M`
	  THEN Q.EXISTS_TAC `ii`
	  THEN RW_TAC std_ss[]
	  THEN SRW_TAC[] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``{PREIMAGE ((X:num->'a->'b) i) (A i) INTER p_space p |
                  i IN {s1:num;s2;s3;s4;s5;s6;s7;s8;s9;s10}} =
		 {PREIMAGE (X s1) (A s1) INTER p_space p;
		  PREIMAGE (X s2) (A s2) INTER p_space p;
		  PREIMAGE (X s3) (A s3) INTER p_space p;
		  PREIMAGE (X s4) (A s4) INTER p_space p;
		  PREIMAGE (X s5) (A s5) INTER p_space p;
		  PREIMAGE (X s6) (A s6) INTER p_space p;
		  PREIMAGE (X s7) (A s7) INTER p_space p;
		  PREIMAGE (X s8) (A s8) INTER p_space p;
		  PREIMAGE (X s9) (A s9) INTER p_space p;
		  PREIMAGE (X s10) (A s10) INTER p_space p}``)
      THEN1(ASM_SET_TAC[])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[BIGINTER_10_sets]
THEN RW_TAC std_ss[PRODUCT_UNION_10]
THEN KNOW_TAC(``random_variable ((X:num->'a->'b) s1) p (m_space (M s1), measurable_sets (M s1)) /\
                 random_variable  ((X:num->'a->'b) s2)p (m_space (M s2), measurable_sets (M s2)) /\
		 random_variable ((X:num->'a->'b) s3) p (m_space (M s3), measurable_sets (M s3)) /\
		 random_variable ((X:num->'a->'b) s4) p (m_space (M s4), measurable_sets (M s4)) /\
		 random_variable ((X:num->'a->'b) s5) p (m_space (M s5), measurable_sets (M s5)) /\
		 random_variable ((X:num->'a->'b) s6) p (m_space (M s6), measurable_sets (M s6)) /\
		 random_variable ((X:num->'a->'b) s7) p (m_space (M s7), measurable_sets (M s7)) /\
		 random_variable ((X:num->'a->'b) s8) p (m_space (M s8), measurable_sets (M s8)) /\
		 random_variable ((X:num->'a->'b) s9) p (m_space (M s9), measurable_sets (M s9)) /\
		 random_variable ((X:num->'a->'b) s10) p (m_space (M s10), measurable_sets (M s10))``)
      THEN1(FULL_SIMP_TAC std_ss [indep_vars]
            THEN UNDISCH_TAC(`` !i:num.
                                      i IN ii ==>
                                    random_variable ((X:num->'a->'b) i) p (m_space (M i),
				      measurable_sets (M i))``)
            THEN DISCH_TAC
	    THEN RW_TAC std_ss[]
	         THEN1( POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
	                THEN ASM_SET_TAC[])
	         THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
		       THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
		       THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
		      THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
		      THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
		      THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
		      THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
		      THEN ASM_SET_TAC[])
		 THEN1(POP_ASSUM (MP_TAC o Q.SPEC `s9:num `)
		       THEN ASM_SET_TAC[])
		 THEN POP_ASSUM (MP_TAC o Q.SPEC `s10:num `)
		 THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE ((X:num->'a->'b) s1) (A s1) INTER p_space p IN events p /\
                 PREIMAGE ((X:num->'a->'b) s2) (A s2) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s3) (A s3) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s4) (A s4) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s5) (A s5) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s6) (A s6) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s7) (A s7) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s8) (A s8) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s9) (A s9) INTER p_space p IN events p /\
		 PREIMAGE ((X:num->'a->'b) s10) (A s10) INTER p_space p IN events p``)
	THEN1(REWRITE_TAC [events_def]
	      THEN KNOW_TAC(``measurable_sets p = subsets (p_space p, measurable_sets p)``)
	           THEN1(RW_TAC std_ss [p_space_def, measurable_sets_def, subsets_def])
	      THEN DISCH_TAC
	      THEN ONCE_ASM_REWRITE_TAC[]
	      THEN REWRITE_TAC [p_space_def, m_space_def]
	      THEN KNOW_TAC(`` (A:num->'b->bool) s1 IN measurable_sets (M s1) /\
	                        (A:num->'b->bool) s2 IN measurable_sets (M s2) /\
				(A:num->'b->bool) s3 IN measurable_sets (M s3) /\
				(A:num->'b->bool) s4 IN measurable_sets (M s4) /\
				(A:num->'b->bool) s5 IN measurable_sets (M s5) /\
				(A:num->'b->bool) s6 IN measurable_sets (M s6) /\
				(A:num->'b->bool) s7 IN measurable_sets (M s7) /\
				(A:num->'b->bool) s8 IN measurable_sets (M s8) /\
				(A:num->'b->bool) s9 IN measurable_sets (M s9) /\
				(A:num->'b->bool) s10 IN measurable_sets (M s10)``)
                   THEN1(RW_TAC std_ss[]
		         THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                       (A:num->'b->bool) i IN measurable_sets (M i)``)
                               THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s1:num `)
			       THEN ASM_SET_TAC[])
			 THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                       (A:num->'b->bool) i IN measurable_sets (M i)``)
			       THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s2:num `)
			       THEN ASM_SET_TAC[])
			 THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                        (A:num->'b->bool) i IN measurable_sets (M i)``)
				THEN DISCH_TAC
				THEN POP_ASSUM (MP_TAC o Q.SPEC `s3:num `)
				THEN ASM_SET_TAC[])
			 THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                       (A:num->'b->bool) i IN measurable_sets (M i)``)
			       THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s4:num `)
			       THEN ASM_SET_TAC[])
			 THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                       (A:num->'b->bool) i IN measurable_sets (M i)``)
			       THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s5:num `)
			       THEN ASM_SET_TAC[])
                         THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                        (A:num->'b->bool) i IN measurable_sets (M i)``)
			       THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s6:num `)
			       THEN ASM_SET_TAC[])
                         THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                       (A:num->'b->bool) i IN measurable_sets (M i)``)
			       THEN DISCH_TAC
			       THEN POP_ASSUM (MP_TAC o Q.SPEC `s7:num `)
			       THEN ASM_SET_TAC[])
			 THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                        (A:num->'b->bool) i IN measurable_sets (M i)``)
				THEN DISCH_TAC
				THEN POP_ASSUM (MP_TAC o Q.SPEC `s8:num `)
				THEN ASM_SET_TAC[])
                        THEN1(UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
			                      (A:num->'b->bool) i IN measurable_sets (M i)``)
			      THEN DISCH_TAC
			      THEN POP_ASSUM (MP_TAC o Q.SPEC `s9:num `)
			      THEN ASM_SET_TAC[])
                       THEN UNDISCH_TAC(`` !i:num. i IN {s1;s2;s3;s4;s5;s6;s7;s8;s9;s10} ==>
		                            (A:num->'b->bool) i IN measurable_sets (M i)``)
		       THEN DISCH_TAC
		       THEN POP_ASSUM (MP_TAC o Q.SPEC `s10:num `)
		       THEN ASM_SET_TAC[])
	THEN METIS_TAC [random_variable_def, IN_MEASURABLE, events_def, subsets_def, p_space_def, measurable_sets_def, EVENTS_INTER, EVENTS_SPACE, m_space_def, space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [REAL_MUL_ASSOC]
THEN RW_TAC std_ss [normal_real_mul_10, PROB_FINITE]
THEN RW_TAC std_ss [normal_real, PROB_FINITE]);




val indep_vars_indep_var_MA_MS = store_thm("indep_vars_indep_var_MA_MS",
  `` 
! CS SS MA MS MB P B PA PB PS_a p.
   indep_vars_10 p lborel CS SS MA MS MB P B PA PB PS_a ==>
   indep_var p lborel MS lborel MA``,

RW_TAC std_ss[indep_vars_10_def]

THEN KNOW_TAC(``indep_vars p (\i. lborel) (\i.
       if i = 0 then CS
       else if i = 1 then SS
       else if i = 2 then MA
       else if i = 3 then MS
       else if i = 4 then MB
       else if i = 5 then P
       else if i = 6 then B
       else if i = 7 then PA
       else if i = 8 then PB
       else PS_a){2;3:num} ``)
THEN1 ( `{2;3:num} SUBSET {0;1;2;3;4;5;6;7;8;9:num}` by ASM_SET_TAC[]
       THEN METIS_TAC [indep_vars_subset])
THEN DISCH_TAC
THEN WEAKEN_TAC (equal(Term`indep_vars p (\i. lborel)
        (\i.
           if i = 0 then CS
           else if i = 1 then SS
           else if i = 2 then MA
           else if i = 3 then MS
           else if i = 4 then MB
           else if i = 5 then P
           else if i = 6 then B
           else if i = 7 then PA
           else if i = 8 then PB
           else PS_a) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}`))

THEN RW_TAC std_ss[indep_var]
THEN FULL_SIMP_TAC std_ss[indep_vars]


THEN KNOW_TAC(``indep_sets p
        (\i.
           {PREIMAGE f A INTER p_space p |
            (f =
             if i = 2 then MA
             else  MS
             ) /\ A IN measurable_sets lborel}) {2; 3}
``)

THEN1( MATCH_MP_TAC indep_sets_mono_sets
       THEN Q.EXISTS_TAC`(\i.
           {PREIMAGE f A INTER p_space p |
            (f =
             if i = 0 then CS
             else if i = 1 then SS
             else if i = 2 then MA
             else if i = 3 then MS
             else if i = 4 then MB
             else if i = 5 then P
             else if i = 6 then B
             else if i = 7 then PA
             else if i = 8 then PB
             else PS_a) /\ A IN measurable_sets lborel})`
	     THEN RW_TAC std_ss[] THEN ASM_SET_TAC[])
THEN DISCH_TAC

THEN KNOW_TAC (``  indep_sets p
        (\i.
           {PREIMAGE f A INTER p_space p |
            (f = if i = 2 then MA else MS) /\
            A IN measurable_sets lborel}) {2; 3}=  indep_sets p
        (\i.
           {PREIMAGE f A INTER p_space p |
            (f = if i = 0 then MS else MA) /\
            A IN measurable_sets lborel}) {0; 1}``)

 THEN1( RW_TAC std_ss [indep_sets]

       THEN1(   FULL_SIMP_TAC std_ss[indep_sets])
       THEN1( FULL_SIMP_TAC std_ss [indep_sets]
       	      THEN KNOW_TAC(``{PREIMAGE f (A:real->bool) INTER p_space p |
(f = if i = (0:num) then MA else MS) /\ A IN measurable_sets lborel} = {PREIMAGE (if i = 0:num then MA
 else MS )(A:real->bool) INTER p_space p |
  A IN measurable_sets lborel}``)  THEN1(ASM_SET_TAC[])
              THEN DISCH_TAC
	      THEN ASM_REWRITE_TAC[]
	      THEN ASM_SET_TAC[])
	      
       THEN FULL_SIMP_TAC std_ss[indep_sets]
       THEN    UNDISCH_TAC(``!J.
        J SUBSET {2:num; 3} /\ J <> {} /\ FINITE J ==>
        !A.
          A IN
          Pi J
            (\i.
               {PREIMAGE f A INTER p_space p |
                (f = if i = 2 then MA else MS) /\
                A IN measurable_sets lborel}) ==>
          (prob p (BIGINTER {A j | j IN J}) =
           Normal (product J (\j. real (prob p (A j)))))
 ``)
		THEN DISCH_TAC
		THEN  POP_ASSUM (MP_TAC o Q.SPEC `{2;3:num} `)
		THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		THEN FULL_SIMP_TAC std_ss [Pi_iff]
                THEN KNOW_TAC(``!i.
        i IN {2:num; 3} ==>
        {PREIMAGE (if i = 2 then MA else MS) A INTER p_space p |
                  A IN measurable_sets lborel} SUBSET events p``)
		  THEN1 (ASM_SET_TAC[])
		  THEN DISCH_TAC
		  THEN FULL_SIMP_TAC std_ss[]

		  THEN KNOW_TAC(``{PREIMAGE MS A INTER p_space p| A IN measurable_sets lborel } SUBSET events p``) THEN1(ASM_SET_TAC[])
		  THEN DISCH_TAC
		  THEN KNOW_TAC(``{PREIMAGE MA A INTER p_space p| A IN measurable_sets lborel } SUBSET events p``)THEN1(ASM_SET_TAC[])
		  THEN DISCH_TAC
		  THEN `J SUBSET {0:num;1} = (J = {0:num} ) \/ ( J = {1:num}) \/ (J = {0:num;1})` by ASM_SET_TAC[]
		 THEN FULL_SIMP_TAC std_ss[]
(*3 subgoals*)		 THEN1( KNOW_TAC(``{(A:num->'a->bool) j | j IN {0}} = {A 0} ``)
			 	THEN1(SRW_TAC [] [EXTENSION, GSPECIFICATION])
				THEN DISCH_TAC
				THEN ASM_REWRITE_TAC[]
				THEN RW_TAC std_ss [PRODUCT_SING]
				THEN RW_TAC std_ss [BIGINTER_SING]
				THEN UNDISCH_TAC(`` !i:num.
				        i IN {0} ==>
			        A i IN
			        {PREIMAGE f A INTER p_space p |
				         (f = if i = 0 then MS else MA) /\ A								 IN measurable_sets lborel}``)
				THEN DISCH_TAC

				THEN POP_ASSUM (MP_TAC o Q.SPEC `0:num`)
				THEN `0:num IN {0:num} ` by ASM_SET_TAC[]
				THEN RW_TAC std_ss[]

				THEN `?A'.( A 0 = PREIMAGE MS A' INTER p_space p)/\ A' IN measurable_sets lborel` by 
				( `A 0 IN
				{PREIMAGE f A INTER p_space p |
				 (f = MS) /\ A IN measurable_sets lborel} = A 0 IN {PREIMAGE MS A INTER p_space p|A IN measurable_sets lborel }` by  ASM_SET_TAC[]
				 THEN ASM_REWRITE_TAC[]
				 THEN PSET_TAC[]
				 THEN ASM_REWRITE_TAC[])
				 THEN `random_variable MS p borel` by 
				 (UNDISCH_TAC(`` !i:num.
        i IN {2; 3} ==>
        random_variable
          (if i = 0 then CS
           else if i = 1 then SS
           else if i = 2 then MA
           else if i = 3 then MS
           else if i = 4 then MB
           else if i = 5 then P
           else if i = 6 then B
           else if i = 7 then PA
           else if i = 8 then PB
           else PS_a) p (m_space lborel,measurable_sets lborel)``)

	   	      		      THEN DISCH_TAC
				      THEN POP_ASSUM (MP_TAC o Q.SPEC `3:num`)
				      THEN SRW_TAC[] [EXTENSION, GSPECIFICATION]
				      THEN `borel = (m_space lborel,measurable_sets lborel)` by METIS_TAC[measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def]
				      THEN FULL_SIMP_TAC std_ss[])
				 THEN `PREIMAGE MS (A':real->bool) INTER p_space p IN events p` by 
				      (METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def, IN_SING, SUBSET_DEF, GSPECIFICATION, EXTENSION, IN_DEF])
				 THEN METIS_TAC [PROB_FINITE, normal_real])

(* 2 subgoals A 1*)      	THEN1 (KNOW_TAC(``{(A:num->'a->bool) j | j IN {1}} = {A 1} ``)
			 	THEN1(SRW_TAC [] [EXTENSION, GSPECIFICATION])
				THEN DISCH_TAC
				THEN ASM_REWRITE_TAC[]
				THEN RW_TAC std_ss [PRODUCT_SING]
				THEN RW_TAC std_ss [BIGINTER_SING]
				THEN UNDISCH_TAC(`` !i:num.
				        i IN {1} ==>
			        A i IN
			        {PREIMAGE f A INTER p_space p |
				         (f = if i = 0 then MS else MA) /\ A								 IN measurable_sets lborel}``)
				THEN DISCH_TAC

				THEN POP_ASSUM (MP_TAC o Q.SPEC `1:num`)
				THEN `1:num IN {1:num} ` by ASM_SET_TAC[]
				THEN RW_TAC std_ss[]

				THEN `?A'.( A 1 = PREIMAGE MA A' INTER p_space p)/\ A' IN measurable_sets lborel` by 
				( `A 1 IN
				{PREIMAGE f A INTER p_space p |
				 (f = MA) /\ A IN measurable_sets lborel} = A 1 IN {PREIMAGE MA A INTER p_space p|A IN measurable_sets lborel }` by  ASM_SET_TAC[]
				 THEN ASM_REWRITE_TAC[]
				 THEN PSET_TAC[]
				 THEN ASM_REWRITE_TAC[])
				 THEN `random_variable MA p borel` by 
				 (UNDISCH_TAC(`` !i:num.
        i IN {2; 3} ==>
        random_variable
          (if i = 0 then CS
           else if i = 1 then SS
           else if i = 2 then MA
           else if i = 3 then MS
           else if i = 4 then MB
           else if i = 5 then P
           else if i = 6 then B
           else if i = 7 then PA
           else if i = 8 then PB
           else PS_a) p (m_space lborel,measurable_sets lborel)``)

	   	      		      THEN DISCH_TAC
				      THEN POP_ASSUM (MP_TAC o Q.SPEC `2:num`)
				      THEN SRW_TAC[] [EXTENSION, GSPECIFICATION]
				      THEN `borel = (m_space lborel,measurable_sets lborel)` by METIS_TAC[measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def]
				      THEN FULL_SIMP_TAC std_ss[])
				 THEN `PREIMAGE MA (A':real->bool) INTER p_space p IN events p` by   
				        (METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def, IN_SING, SUBSET_DEF, GSPECIFICATION, EXTENSION, IN_DEF])
				 THEN METIS_TAC [PROB_FINITE, normal_real])


	(* 1 subgoal {0;1}*)	 THEN KNOW_TAC(``{(A:num->'a->bool) j | j IN {0:num; 1}}={A 0; A 1}					``) THEN1(ASM_SET_TAC[])
	     	     		 THEN DISCH_TAC
				 THEN ASM_REWRITE_TAC[]
				 THEN `FINITE {0:num} /\ FINITE {1:num} /\ DISJOINT {0:num} {1:num}/\ ({0;1:num} = {0} UNION {1})` by SRW_TAC  [] [GSPECIFICATION, EXTENSION, FINITE_DEF, DISJOINT_DEF]
				 THEN RW_TAC std_ss [BIGINTER_INTER,PRODUCT_UNION, PRODUCT_SING]

				 THEN `?A'.( A 0 = PREIMAGE MS A' INTER p_space p)/\ A' IN measurable_sets lborel` by
				 (  UNDISCH_TAC(``!i.
				         i IN {0:num; 1} ==>
					 (A:num->'a->bool) i IN
			                 {PREIMAGE f A INTER p_space p |
			                 (f = if i = 0 then MS else MA) /\ A IN measurable_sets lborel}``)
					 THEN DISCH_TAC
					 THEN POP_ASSUM (MP_TAC o Q.SPEC `(0:num) `)
					 THEN FULL_SIMP_TAC std_ss[prove(``0 IN {0;(1:num)}``,
					      SRW_TAC[][EXTENSION,GSPECIFICATION])]
					 THEN `A 0 IN
					 {PREIMAGE f A INTER p_space p |
					  (f = MS) /\ A IN measurable_sets lborel} = A 0 IN
					  {PREIMAGE MS A INTER p_space p|
					  A IN measurable_sets lborel }` by  ASM_SET_TAC[]
					  THEN ASM_REWRITE_TAC[]
					  THEN PSET_TAC[])
			        THEN ASM_REWRITE_TAC[]

				THEN `?A1'.( A 1 = PREIMAGE MA A1' INTER p_space p)/\
				      A1' IN measurable_sets lborel` by 
				      (UNDISCH_TAC(``!i.
				            i IN {0:num; 1} ==>
					    (A:num->'a->bool) i IN
					    {PREIMAGE f A INTER p_space p |
					     (f = if i = 0 then MS else MA) /\
					     A IN measurable_sets lborel}``)
					     THEN DISCH_TAC

					     THEN POP_ASSUM (MP_TAC o Q.SPEC `(1:num) `)


					     THEN FULL_SIMP_TAC std_ss[prove(``1 IN {0;(1:num)}``,
					     SRW_TAC[][EXTENSION,GSPECIFICATION])]
					     THEN `A 1 IN {PREIMAGE f A INTER p_space p |
					      (f = MA) /\ A IN measurable_sets lborel} =
					      A 1 IN {PREIMAGE MA A INTER p_space p|
					      A IN measurable_sets lborel }` by  ASM_SET_TAC[]
					      THEN ASM_REWRITE_TAC[]
					      THEN PSET_TAC[])
				    THEN ASM_REWRITE_TAC[]
				

				    THEN UNDISCH_TAC(`` !A':num->'a->bool.
				         (!i.
                                         i IN {2:num; 3} ==>
                                         A' i IN
                                         {PREIMAGE f A INTER p_space p |
                                         (f = if i = 2 then MA else MS) /\
                                        A IN measurable_sets lborel}) ==>
                                        (prob p (BIGINTER {A' j | (j = 2) \/ (j = 3)}) =
                                        Normal (product {2; 3} (\j. real (prob p (A' j)))))``)

				    THEN DISCH_TAC
				    THEN POP_ASSUM (MP_TAC o Q.SPEC `(\i. if i = 2 then
				    PREIMAGE MA (A1':real->bool) INTER p_space p else
				    PREIMAGE MS (A':real->bool) INTER p_space p)`)
				    THEN DISCH_TAC

				    THEN KNOW_TAC(``(!i.
				        i IN {2; 3:num} ==>
					 (\i.
					  if i = 2 then PREIMAGE MA A1' INTER p_space p
                                        else PREIMAGE MS A' INTER p_space p) i IN
                                       {PREIMAGE f A INTER p_space p |
                                       (f = if i = 2 then MA else MS) /\
                                       A IN measurable_sets lborel}) ``)
				       THEN1(ASM_SET_TAC[])
				       THEN DISCH_TAC
				       THEN FULL_SIMP_TAC std_ss[]
				       THEN KNOW_TAC(``{if j:num = 2 then PREIMAGE MA
				       (A1':real->bool) INTER p_space p
				       else PREIMAGE MS (A':real->bool) INTER p_space p |
             			       (j = 2) \/ (j = 3)} = {PREIMAGE MA (A1':real->bool)
				       INTER p_space p;PREIMAGE MS (A':real->bool) INTER p_space p }					   ``)
				           THEN1(RW_TAC std_ss [GSPECIFICATION, EXTENSION]
					         THEN PSET_TAC [PREIMAGE_def]
						 THEN EQ_TAC
						      THEN1(RW_TAC std_ss[]
						            THEN1( ASM_SET_TAC[])
							    THEN ASM_SET_TAC[])
						      THEN DISCH_TAC
						      THEN RW_TAC std_ss[]
						      	   THEN1(Q.EXISTS_TAC`2:num`
								 THEN ASM_SET_TAC[])

							    THEN Q.EXISTS_TAC`3:num`
							    THEN RW_TAC std_ss[])
				           THEN DISCH_TAC
					   THEN FULL_SIMP_TAC std_ss[]
					   THEN `FINITE {2:num} /\FINITE {3:num} /\ DISJOINT {2:num}
					   {3:num} /\ ( {2:num;3} = {2} UNION {3}) `
					   by SRW_TAC[] [GSPECIFICATION, EXTENSION]
					   THEN FULL_SIMP_TAC std_ss [PRODUCT_UNION]
					   THEN FULL_SIMP_TAC std_ss [PRODUCT_SING]
					   THEN FULL_SIMP_TAC std_ss [BIGINTER_INTER]
					   THEN FULL_SIMP_TAC std_ss [REAL_MUL_COMM]
					   THEN METIS_TAC [INTER_COMM])
		
	THEN DISCH_TAC 	
	THEN RW_TAC std_ss[]
	     THEN1(UNDISCH_TAC(``!i:num.
        i IN {2; 3} ==>
        random_variable
          (if i = 0 then CS
           else if i = 1 then SS
           else if i = 2 then MA
           else if i = 3 then MS
           else if i = 4 then MB
           else if i = 5 then P
           else if i = 6 then B
           else if i = 7 then PA
           else if i = 8 then PB
           else PS_a) p (m_space lborel,measurable_sets lborel)
``)
		   THEN DISCH_TAC	
		   THEN POP_ASSUM (MP_TAC o Q.SPEC `3:num`)
		   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION])
	     THEN1(UNDISCH_TAC(``!i:num.
        i IN {2; 3} ==>
        random_variable
          (if i = 0 then CS
           else if i = 1 then SS
           else if i = 2 then MA
           else if i = 3 then MS
           else if i = 4 then MB
           else if i = 5 then P
           else if i = 6 then B
           else if i = 7 then PA
           else if i = 8 then PB
           else PS_a) p (m_space lborel,measurable_sets lborel)
``)
		   THEN DISCH_TAC	
		   THEN POP_ASSUM (MP_TAC o Q.SPEC `2:num`)
		   THEN SRW_TAC[] [GSPECIFICATION, EXTENSION])
	    THEN `{x | (x = 0) \/ (x = 1:num)} = {0;1:num}`  by ASM_SET_TAC[]  
	    THEN ASM_REWRITE_TAC[]
	    THEN FULL_SIMP_TAC std_ss[]);


val ALL_DISTINCT_RV_DEF = Define `
ALL_DISTINCT_RV [A0; A1; A2; A3; A4; A5; A6; A7; A8; A9] p t =
ALL_DISTINCT [DFT_event p A0 t; DFT_event p A1 t;
            DFT_event p (D_AND A2 (D_BEFORE A3 A2)) t;
            DFT_event p (D_AND A2 A4) t; DFT_event p (D_AND A5 A6) t;
            DFT_event p (D_AND (D_AND A7 A8) A9) t]/\
rv_gt0_ninfinity  [A0; A1; A2; A3; A4; A5; A6; A7; A8; A9] /\ (!s. ALL_DISTINCT (MAP (\a. a s )  [A0; A1; A2; A3; A4; A5; A6; A7; A8; A9] ))`;


val CAS_UNION_LIST = store_thm("CAS_UNION_LIST",
  ``   !PA PB PS MS MA MB CS SS P B p t.
DFT_event p ( D_OR
        (D_OR
           (D_OR (D_OR (D_OR CS SS) (D_AND MA (D_BEFORE MS MA)))
              (D_AND MA MB)) (D_AND P B)) (D_AND (D_AND PA PB) PS)) t  = (union_list
             [DFT_event p CS t; DFT_event p SS t;
              DFT_event p (D_AND MA (D_BEFORE MS MA)) t;
              DFT_event p (D_AND MA MB) t; DFT_event p (D_AND P B) t;
              DFT_event p (D_AND (D_AND PA PB) PS) t])``,
RW_TAC std_ss [OR_UNION]
THEN RW_TAC std_ss [union_list_def]
THEN PSET_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION, UNION_COMM, UNION_ASSOC]
THEN EQ_TAC THEN RW_TAC std_ss[] THEN RW_TAC std_ss[]);


val indep_vars_sets_def = Define `
 indep_vars_sets [A0; A1; A2; A3; A4; A5; A6; A7; A8; A9] p t =
     indep_vars_10 p lborel (\s. real (A0 s)) (\s. real (A1 s))
       (\s. real (A2 s)) (\s. real (A3 s)) (\s. real (A4 s))
       (\s. real (A5 s)) (\s. real (A6 s)) (\s. real (A7 s))
       (\s. real (A8 s)) (\s. real (A9 s)) /\
     indep_sets p
       (\i.
          {if i = 0 then
             PREIMAGE (\s. real (A0 s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (A1 s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (A3 s),real (A2 s)))
               {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (A4 s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (A5 s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (A6 s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (A7 s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (A8 s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (A9 s)) {u | u <= t} INTER p_space p})
       {0; 1; 2; 3; 4; 5; 6; 7; 8}`;


val CAS_PROB = store_thm ("CAS_PROB",
  ``  
     !CS SS MA MS MB P B PA PB PS p t f_MA.
       0 <= t /\ prob_space p /\
       ALL_DISTINCT_RV [CS; SS; MA; MS; MB; P; B; PA; PB; PS] p t /\
       indep_vars_sets [CS; SS; MA; MS; MB; P; B; PA; PB; PS] p t /\
       distributed p lborel (\s. real (MA s)) f_MA /\
       (!y. 0 <= f_MA y) /\ cont_CDF p (\s. real (MS s)) /\
       measurable_CDF p (\s. real (MS s)) ==>
        (prob p
          (DFT_event p
             (D_OR
                (D_OR
                   (D_AND (shared_spare PA PB PS PS)
                      (shared_spare PB PA PS PS))
                   (D_OR (P_AND MS MA) (HSP MA MB)))
                (HSP (FDEP (D_OR CS SS) P) (FDEP (D_OR CS SS) B))) t) =
        CDF p (\s. real (CS s)) t + CDF p (\s. real (SS s)) t +
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) +
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t +
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t +
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t -
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) -
        CDF p (\s. real (SS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t -
        CDF p (\s. real (SS s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t -
        CDF p (\s. real (SS s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t -
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t -
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t -
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t +
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t +
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t +
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t +
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        CDF p (\s. real (SS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t +
        CDF p (\s. real (SS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        CDF p (\s. real (SS s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t +
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t +
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t -
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t -
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        CDF p (\s. real (SS s)) t * CDF p (\s. real (MA s)) t *
        CDF p (\s. real (MB s)) t * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (P s)) t *
        CDF p (\s. real (B s)) t * CDF p (\s. real (PA s)) t *
        CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        CDF p (\s. real (MA s)) t * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t +
        CDF p (\s. real (CS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t +
        CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t -
        CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
        pos_fn_integral lborel
          (\y.
             f_MA y *
             (indicator_fn {u | 0 <= u /\ u <= t} y *
              CDF p (\s. real (MS s)) y)) * CDF p (\s. real (MB s)) t *
        CDF p (\s. real (P s)) t * CDF p (\s. real (B s)) t *
        CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
        CDF p (\s. real (PS s)) t)``,

RW_TAC std_ss[indep_vars_sets_def, ALL_DISTINCT_RV_DEF, rv_gt0_ninfinity_def]
THEN FULL_SIMP_TAC std_ss[MAP]
THEN `!s. 0<= CS s /\ CS s <> PosInf /\ 0<= SS s /\ SS s <> PosInf /\
          0<= MA s /\ MA s <> PosInf /\ 0<= MS s /\ MS s <> PosInf /\
	  0<= MB s /\ MB s <> PosInf /\ 0<= P s /\ P s <> PosInf /\
	  0<= B s /\ B s <> PosInf /\ 0<= PA s /\ PA s <> PosInf /\
	  0<= PB s /\ PB s <> PosInf /\ 0<= PS s /\ PS s <> PosInf ` by RW_TAC std_ss[]
THEN KNOW_TAC(``!t:real. {(u,w) | u < w /\ 0 <= w /\ w <= t} IN measurable_sets (pair_measure lborel lborel)``)
     THEN1(METIS_TAC [after_set_IN_MEASURABLE_SETS_PAIR_lborel])
THEN DISCH_TAC     
THEN KNOW_TAC (``!s. (ALL_DISTINCT [(MA:'a->extreal) s ; MS s ; PA s; PB s; PS s])``)
              THEN1(FULL_SIMP_TAC std_ss[MAP]
	            THEN METIS_TAC [ALL_DISTINCT, MEM])
THEN RW_TAC std_ss[ HCAS_final_2,CAS_UNION_LIST]
THEN KNOW_TAC (``random_variable (\s. real (CS s)) p borel /\
                  random_variable (\s. real (SS s)) p borel /\
		  random_variable (\s. real (MA s)) p borel /\
		  random_variable (\s. real (MS s)) p borel /\
		  random_variable (\s. real (MB s)) p borel /\
		  random_variable (\s. real (P s)) p borel /\
		  random_variable (\s. real (B s)) p borel /\
		  random_variable (\s. real (PA s)) p borel /\
		  random_variable (\s. real (PB s)) p borel /\
		  random_variable (\s. real (PS s)) p borel ``)
      THEN1(FULL_SIMP_TAC std_ss [indep_vars_10_def, indep_vars]
           THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
                THEN1( METIS_TAC [m_space_def, measurable_sets_def, space_def, subsets_def, lborel, borel, sigma_def])
      THEN DISCH_TAC
      THEN RW_TAC std_ss []
           THEN1(UNDISCH_TAC (``!i:num.
				i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        			random_variable
          			(if i = 0 then (\s. real (CS s))
          			 else if i = 1 then (\s. real (SS s))
          			 else if i = 2 then (\s. real (MA s))
          	                 else if i = 3 then (\s. real (MS s))
            			 else if i = 4 then (\s. real (MB s))
           			 else if i = 5 then (\s. real (P s))
           			 else if i = 6 then (\s. real (B s))
           			 else if i = 7 then (\s. real (PA s))
           			 else if i = 8 then (\s. real (PB s))
           			 else (\s. real (PS s))) p
          			   (m_space lborel,measurable_sets lborel)  ``) 
	         THEN DISCH_TAC
	    	 THEN POP_ASSUM (MP_TAC o Q.SPEC `0:num `)
	    	 THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	    THEN1( UNDISCH_TAC (``!i:num.
                                        i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        				random_variable
          				(if i = 0 then (\s. real (CS s))
           				else if i = 1 then (\s. real (SS s))
           				else if i = 2 then (\s. real (MA s))
           				else if i = 3 then (\s. real (MS s))
           				else if i = 4 then (\s. real (MB s))
           				else if i = 5 then (\s. real (P s))
           				else if i = 6 then (\s. real (B s))
           				else if i = 7 then (\s. real (PA s))
          				else if i = 8 then (\s. real (PB s))
           				else (\s. real (PS s))) p
          				(m_space lborel,measurable_sets lborel)  ``) 
	          THEN DISCH_TAC
	    	  THEN POP_ASSUM (MP_TAC o Q.SPEC `1:num `)
	    	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1(UNDISCH_TAC (``!i:num.
				      i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        			      random_variable
          			      (if i = 0 then (\s. real (CS s))
           			      else if i = 1 then (\s. real (SS s))
           			      else if i = 2 then (\s. real (MA s))
          			      else if i = 3 then (\s. real (MS s))
           			      else if i = 4 then (\s. real (MB s))
           			      else if i = 5 then (\s. real (P s))
           			      else if i = 6 then (\s. real (B s))
           			      else if i = 7 then (\s. real (PA s))
           			      else if i = 8 then (\s. real (PB s))
           			      else (\s. real (PS s))) p
          			      (m_space lborel,measurable_sets lborel)  ``) 
	         THEN DISCH_TAC
	         THEN POP_ASSUM (MP_TAC o Q.SPEC `2:num `)
	         THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	    THEN1( UNDISCH_TAC (``!i:num.
                                        i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        				random_variable
          				(if i = 0 then (\s. real (CS s))
          				 else if i = 1 then (\s. real (SS s))
           				 else if i = 2 then (\s. real (MA s))
           				 else if i = 3 then (\s. real (MS s))
           				 else if i = 4 then (\s. real (MB s))
           				 else if i = 5 then (\s. real (P s))
           				 else if i = 6 then (\s. real (B s))
           				 else if i = 7 then (\s. real (PA s))
           				 else if i = 8 then (\s. real (PB s))
          				 else (\s. real (PS s))) p
          				 (m_space lborel,measurable_sets lborel)  ``) 
	         THEN DISCH_TAC
	         THEN POP_ASSUM (MP_TAC o Q.SPEC `3:num `)
	         THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
				       i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        			       random_variable
         			        (if i = 0 then (\s. real (CS s))
          				 else if i = 1 then (\s. real (SS s))
           				 else if i = 2 then (\s. real (MA s))
           				 else if i = 3 then (\s. real (MS s))
           				 else if i = 4 then (\s. real (MB s))
           				 else if i = 5 then (\s. real (P s))
           				 else if i = 6 then (\s. real (B s))
           				 else if i = 7 then (\s. real (PA s))
           				 else if i = 8 then (\s. real (PB s))
          				 else (\s. real (PS s))) p
          				 (m_space lborel,measurable_sets lborel)  ``) 
	          THEN DISCH_TAC
	          THEN POP_ASSUM (MP_TAC o Q.SPEC `4:num `)
	          THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
				       i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        			       random_variable
          			       (if i = 0 then (\s. real (CS s))
           			       else if i = 1 then (\s. real (SS s))
           			       else if i = 2 then (\s. real (MA s))
           			       else if i = 3 then (\s. real (MS s))
           			       else if i = 4 then (\s. real (MB s))
           			       else if i = 5 then (\s. real (P s))
           			       else if i = 6 then (\s. real (B s))
           			       else if i = 7 then (\s. real (PA s))
           			       else if i = 8 then (\s. real (PB s))
           			       else (\s. real (PS s))) p
           			       (m_space lborel,measurable_sets lborel)  ``) 
	          THEN DISCH_TAC
	    	  THEN POP_ASSUM (MP_TAC o Q.SPEC `5:num `)
	    	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
        i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        random_variable
          (if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s))
           else (\s. real (PS s))) p
          (m_space lborel,measurable_sets lborel)  ``) 
	    THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `6:num `)
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
        i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        random_variable
          (if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s))
           else (\s. real (PS s))) p
          (m_space lborel,measurable_sets lborel)  ``) 
	    THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `7:num `)
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
        i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        random_variable
          (if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s))
           else (\s. real (PS s))) p
          (m_space lborel,measurable_sets lborel)  ``) 
	    THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `8:num `)
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
        i IN {0; 1; 2; 3; 4; 5; 6; 7; 8; 9} ==>
        random_variable
          (if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s))
           else (\s. real (PS s))) p
          (m_space lborel,measurable_sets lborel)  ``) 
	    THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `9:num `)
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	  ) 
THEN DISCH_TAC
THEN KNOW_TAC(``{u|u<=t:real} IN measurable_sets lborel``)
THEN1( RW_TAC std_ss [lborel_le])
THEN DISCH_TAC  
THEN KNOW_TAC(``{u|u<=t:real} IN subsets borel``)
THEN1( METIS_TAC [measurable_sets_def, lborel, borel , subsets_def, sigma_def])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (MA s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,  random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (MS s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE(\s. real ( MB s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel, random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE(\s. real ( P s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel, random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real ( B s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel, random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,  random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,  random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       

THEN KNOW_TAC(``PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. (real (MS s), real (MA s))) {(u,w) | u < w /\ 0<=w/\w<=  t} INTER p_space p IN                       events p ``)
     THEN1 ( KNOW_TAC(``(\s. (real(MS s) , real(MA s))) IN measurable (p_space p , events p) (m_space (pair_measure lborel lborel),measurable_sets (pair_measure lborel lborel))``) 
              THEN1(FULL_SIMP_TAC std_ss [random_variable_def]
	             THEN METIS_TAC [ measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, measurable_Pair, measure_space_lborel, prob_space_def])
      THEN DISCH_TAC
      
      
      THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, PREIMAGE_EXTREAL_REAL_2RV])		    
THEN DISCH_TAC
THEN KNOW_TAC(``DFT_event p CS t IN events p /\ DFT_event p SS t IN events p /\
       DFT_event p (D_AND MA (D_BEFORE MS MA)) t IN events p /\
       DFT_event p (D_AND MA MB) t IN events p /\
       DFT_event p (D_AND P B) t IN events p /\
       DFT_event p (D_AND (D_AND PA PB) PS) t IN events p  ``) THEN1(
       KNOW_TAC(``(!s. 0<= (MA:'a->extreal) s)``) THEN1(
       FULL_SIMP_TAC std_ss [rv_gt0_ninfinity_def]) THEN
       DISCH_TAC THEN
       KNOW_TAC(``(DFT_event p (D_AND MA (D_BEFORE MS MA)) t =
      PREIMAGE (\x. (MS x,MA x))
        {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER p_space p)``) THEN1(
	METIS_TAC[DFT_event_AND_BEFORE_PREIMAGE_GT0])
	THEN DISCH_TAC
	THEN ASM_REWRITE_TAC[] THEN
	REWRITE_TAC [AND_INTER] THEN
       REWRITE_TAC [DFT_PREIMAGE] THEN
       FULL_SIMP_TAC std_ss [rv_gt0_ninfinity_def] THEN 
       
       RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV, EVENTS_INTER])
THEN DISCH_TAC
THEN RW_TAC std_ss[PROB_PIE_CAS_63_lem1]
THEN KNOW_TAC(``prob p (DFT_event p CS t) = CDF p (\s. real (CS s)) t``)
THEN1( DEP_REWRITE_TAC[ CDF_prob_DFT_event]
THEN
RW_TAC std_ss[]) 

THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN 
KNOW_TAC(``prob p (DFT_event p SS t) = CDF p (\s. real (SS s)) t``)
THEN1 (
DEP_REWRITE_TAC[ CDF_prob_DFT_event]
THEN RW_TAC std_ss[])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN RW_TAC std_ss [GSYM AND_INTER] THEN

KNOW_TAC(``(D_AND (D_AND (D_AND CS SS) (D_AND MA (D_BEFORE MS MA)))
              (D_AND MA MB)) = (D_AND (D_AND (D_AND CS SS) (D_AND MA (D_BEFORE MS MA))) MB
        )``) THEN1(

RW_TAC std_ss [D_AND_ASSOC] THEN

KNOW_TAC(``(D_AND MA (D_AND (D_BEFORE MS MA) (D_AND MA MB))) = (D_AND MA (D_AND (D_BEFORE MS MA) MB))``) THEN1 (

METIS_TAC [D_AND_ASSOC, D_AND_ID, D_AND_COMM]
) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[]
) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN


KNOW_TAC(``(D_AND (D_AND SS (D_AND MA (D_BEFORE MS MA))) (D_AND MA MB)) = (D_AND (D_AND SS (D_AND MA (D_BEFORE MS MA))) MB)``) THEN1(
METIS_TAC [D_AND_ASSOC, D_AND_ID, D_AND_COMM]

) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
KNOW_TAC(``(D_AND (D_AND CS (D_AND MA (D_BEFORE MS MA))) (D_AND MA MB)) = (D_AND (D_AND CS (D_AND MA (D_BEFORE MS MA)))MB)``) THEN1 (

METIS_TAC [D_AND_ASSOC, D_AND_ID, D_AND_COMM]
 ) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

KNOW_TAC(``(D_AND (D_AND MA (D_BEFORE MS MA)) (D_AND MA MB)) = (D_AND (D_AND MA (D_BEFORE MS MA)) MB)``) THEN1(

METIS_TAC [D_AND_ASSOC, D_AND_ID, D_AND_COMM]

) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
RW_TAC std_ss [AND_INTER] THEN
KNOW_TAC(``DFT_event p MA t INTER DFT_event p (D_BEFORE MS MA) t =  PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p``) THEN1(

RW_TAC std_ss [GSYM AND_INTER] THEN
RW_TAC std_ss [DFT_event_AND_BEFORE_PREIMAGE_GT0]
) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

RW_TAC std_ss[DFT_PREIMAGE]

THEN

KNOW_TAC(``
prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) =  prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p)
``)
THEN1 (RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
THEN 

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `2:num`,`4:num`  ]  prob_prod_any_2_of_10) THEN
RW_TAC std_ss[] THEN 
KNOW_TAC(``{2; 4} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num}``) THEN1
(SRW_TAC [] [GSPECIFICATION, EXTENSION] )THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN

KNOW_TAC(``(prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) = (CDF p (\s.real (MA s)) t ) * (CDF p (\s. real (MB s)) t) ``) THEN1(
RW_TAC std_ss [ GSYM DFT_PREIMAGE] THEN
RW_TAC std_ss[ CDF_prob_DFT_event] )THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE B {u | u <= Normal t} INTER p_space p)) = prob p
  (PREIMAGE P {u | u <= Normal t} INTER p_space p )* prob p
   (PREIMAGE B {u | u <= Normal t} INTER p_space p) ``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
THEN 

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `5:num`,`6:num`  ]  prob_prod_any_2_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{5; 6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num}``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION] ) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN

KNOW_TAC(``(prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p)) = (CDF p (\s.real (P s)) t ) * (CDF p (\s. real (B s)) t) ``) THEN1(
METIS_TAC [ CDF_prob_DFT_event,GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p))  = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) *prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) ``) THEN1 (
   
RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN


MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`  ]  prob_prod_any_2_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0; 1} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num}``) THEN1 (
SRW_TAC [] [GSPECIFICATION, EXTENSION]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] )THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN


KNOW_TAC(``(prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p)) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real (SS s)) t) ``) THEN1(
RW_TAC std_ss [ GSYM DFT_PREIMAGE] THEN
RW_TAC std_ss[ CDF_prob_DFT_event]) THEN
DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE PS {u | u <= Normal t} INTER p_space p)) = prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *prob p
   (PREIMAGE PS {u | u <= Normal t} INTER p_space p) ``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN
MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `7:num`,`8:num`, `9:num` ]  prob_prod_any_3_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{7; 8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [7;8;9:num] ``) THEN1 (
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] )THEN

DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (PA s)) t ) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real(PS s)) t) ``) THEN1(
RW_TAC std_ss [ GSYM DFT_PREIMAGE] THEN
RW_TAC std_ss[ CDF_prob_DFT_event]) THEN
DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) ``) THEN1(
    RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`2:num`, `4:num` ]  prob_prod_any_3_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0; 2;4} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;2;4:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN 
KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(


RW_TAC std_ss[] ) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[] THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN

KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real (MA s)) t) * (CDF p (\s. real(MB s)) t) ``) THEN1 (
METIS_TAC [ CDF_prob_DFT_event, GSYM DFT_PREIMAGE] ) THEN

DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN

KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) =prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p)  * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) ``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`5:num`, `6:num` ]  prob_prod_any_3_of_10) THEN
	   
RW_TAC std_ss[] THEN
KNOW_TAC(``{0; 5;6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;5;6:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN

DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN
FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[] THEN
METIS_TAC[INTER_ASSOC]) THEN
 
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN

KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real(B s)) t) ``) THEN1(
METIS_TAC [ CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN

DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq  THEN


KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN


MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `1:num`,`2:num`, `4:num` ]  prob_prod_any_3_of_10) THEN
	   
RW_TAC std_ss[] THEN
KNOW_TAC(``{1; 2;4} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [1;2;4:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION]) THEN
DISCH_TAC THEN

FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN
KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN 
FULL_SIMP_TAC std_ss[] THEN
METIS_TAC [INTER_ASSOC] )THEN 
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (SS s)) t ) * (CDF p (\s. real( MA s)) t) * (CDF p (\s. real(MB s)) t) ``) THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC (``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p)  * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `1:num`,`5:num`, `6:num` ]  prob_prod_any_3_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{1; 5;6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [1;5;6:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (SS s)) t ) * (CDF p (\s. real( P s)) t) * (CDF p (\s. real(B s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN

KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =   prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1 (




RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`7:num`, `8:num`, `9:num` ]  prob_prod_any_4_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0; 7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real( PA s)) t) * (CDF p (\s. real(PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC(``
prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = 
prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(



RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `1:num`,`7:num`, `8:num`, `9:num` ]  prob_prod_any_4_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{1; 7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [1;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (SS s)) t ) * (CDF p (\s. real( PA s)) t) * (CDF p (\s. real(PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC (``prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``) THEN1 (



RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `2:num`,`4:num`, `5:num`, `6:num` ]  prob_prod_any_4_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{2;4;5;6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [2;4;5;6:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (MA s)) t ) * (CDF p (\s. real( MB s)) t) * (CDF p (\s. real(P s)) t) * (CDF p (\s. real (B s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN




KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)``) THEN1(



RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`, `2:num`, `4:num` ]  prob_prod_any_4_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;2;4} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;2;4:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real( SS s)) t) * (CDF p (\s. real(MA s)) t) * (CDF p (\s. real (MB s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``) THEN1(


RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`, `5:num`, `6:num` ]  prob_prod_any_4_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;5;6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;5;6:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real( SS s)) t) * (CDF p (\s. real(P s)) t) * (CDF p (\s. real (B s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN

KNOW_TAC(``prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(





RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `2:num`,`4:num`, `7:num`, `8:num`,`9:num` ]  prob_prod_any_5_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{2;4;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [2;4;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (MA s)) t ) * (CDF p (\s. real( MB s)) t) * (CDF p (\s. real(PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN

KNOW_TAC(``prob p
  (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE B {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(





RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `5:num`,`6:num`, `7:num`, `8:num`,`9:num` ]  prob_prod_any_5_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (P s)) t ) * (CDF p (\s. real( B s)) t) * (CDF p (\s. real(PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(



RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`, `7:num`, `8:num`,`9:num` ]  prob_prod_any_5_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real( SS s)) t) * (CDF p (\s. real(PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``) THEN1(




RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`2:num`, `4:num`, `5:num`,`6:num` ]  prob_prod_any_5_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;2;4;5;6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;2;4;5;6:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real( MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``) THEN1(





RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `1:num`,`2:num`, `4:num`, `5:num`,`6:num` ]  prob_prod_any_5_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{1;2;4;5;6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [1;2;4;5;6:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE P {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE B {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (SS s)) t ) * (CDF p (\s. real( MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(





RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`2:num`, `4:num`, `7:num`,`8:num`,`9:num` ]  prob_prod_any_6_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;2;4;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;2;4;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real( MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) ``) THEN1(





RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`5:num`, `6:num`, `7:num`,`8:num`,`9:num` ]  prob_prod_any_6_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real( P s)) t) * (CDF p (\s. real(B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p)*prob p 
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) ``) THEN1(





RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `1:num`,`2:num`, `4:num`, `7:num`,`8:num`,`9:num` ]  prob_prod_any_6_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{1;2;4;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [1;2;4;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p)*prob p 
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (SS s)) t ) * (CDF p (\s. real( MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) ``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `1:num`,`5:num`, `6:num`, `7:num`,`8:num`,`9:num` ]  prob_prod_any_6_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{1;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [1;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (SS s)) t ) * (CDF p (\s. real( P s)) t) * (CDF p (\s. real(B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN







KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)))=prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`, `2:num`, `4:num`,`5:num`,`6:num` ]  prob_prod_any_6_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;2;4;5;6} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;2;4;5;6:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real(SS s)) t) * (CDF p (\s. real(MA s)) t) * (CDF p (\s. real (MB s)) t) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN








KNOW_TAC(``prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `2:num`,`4:num`, `5:num`, `6:num`,`7:num`,`8:num`,`9:num` ]  prob_prod_any_7_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{2;4;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [2;4;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(`` prob p
  (PREIMAGE MA {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (MA s)) t ) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real(P s)) t) * (CDF p (\s. real (B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s))t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`, `2:num`, `4:num`,`7:num`,`8:num`,`9:num` ]  prob_prod_any_7_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;2;4;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;2;4;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``  prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real(SS s)) t) * (CDF p (\s. real(MA s)) t) * (CDF p (\s. real (MB s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s))t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN







KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))  =prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`, `5:num`, `6:num`,`7:num`,`8:num`,`9:num` ]  prob_prod_any_7_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real(SS s)) t) * (CDF p (\s. real(P s)) t) * (CDF p (\s. real (B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s)) t) * (CDF p (\s. real (PS s))t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`2:num`, `4:num`, `5:num`,`6:num`,`7:num`,`8:num`,`9:num` ]  prob_prod_any_8_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;2;4;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;2;4;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``  prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (CS s)) t ) * (CDF p (\s. real(MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s))t) * (CDF p (\s. real (PS s))t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `1:num`,`2:num`, `4:num`, `5:num`,`6:num`,`7:num`,`8:num`,`9:num` ]  prob_prod_any_8_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{1;2;4;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [1;2;4;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(``  prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s.real (SS s)) t ) * (CDF p (\s. real(MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s))t) * (CDF p (\s. real (PS s))t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`,`2:num`, `4:num`, `5:num`,`6:num`,`7:num`,`8:num`,`9:num` ]  prob_prod_any_9_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;2;4;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;2;4;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s. real (CS s)) t) *(CDF p (\s.real (SS s)) t ) * (CDF p (\s. real(MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s))t) * (CDF p (\s. real (PS s))t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN







KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``) THEN1(

RW_TAC std_ss [PREIMAGE_EXTREAL_REAL] THEN

MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
           if i = 0 then (\s. real (CS s))
           else if i = 1 then (\s. real (SS s))
           else if i = 2 then (\s. real (MA s))
           else if i = 3 then (\s. real (MS s))
           else if i = 4 then (\s. real (MB s))
           else if i = 5 then (\s. real (P s))
           else if i = 6 then (\s. real (B s))
           else if i = 7 then (\s. real (PA s))
           else if i = 8 then (\s. real (PB s)) else (\s. real (PS s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8;9}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`,`2:num`, `4:num`, `5:num`,`6:num`,`7:num`,`8:num`,`9:num` ]  prob_prod_any_9_of_10) THEN
RW_TAC std_ss[] THEN
KNOW_TAC(``{0;1;2;4;5;6;7;8;9} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8; 9:num} /\ ALL_DISTINCT [0;1;2;4;5;6;7;8;9:num]``) THEN1(
SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM]) THEN
DISCH_TAC THEN
FULL_SIMP_TAC std_ss[indep_vars_10_def] THEN

KNOW_TAC(``indep_vars p (\i. lborel)
         (\i.
            if i = 0 then (\s. real ((CS:'a->extreal) s))
            else if i = 1 then (\s. real ((SS:'a->extreal) s))
            else if i = 2 then (\s. real (MA s))
            else if i = 3 then (\s. real (MS s))
            else if i = 4 then (\s. real (MB s))
            else if i = 5 then (\s. real (P s))
            else if i = 6 then (\s. real (B s))
            else if i = 7 then (\s. real (PA s))
            else if i = 8 then (\s. real (PB s))
            else (\s. real (PS s))) {0; 1; 2; 3; 4; 5; 6; 7; 8; 9}``) THEN1(
RW_TAC std_ss[]) THEN
DISCH_TAC THEN
METIS_TAC [INTER_ASSOC]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN
WEAKEN_TAC is_eq THEN



KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE MA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s. real (CS s)) t) *(CDF p (\s.real (SS s)) t ) * (CDF p (\s. real(MA s)) t) * (CDF p (\s. real(MB s)) t) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t) * (CDF p (\s. real (PA s)) t) * (CDF p (\s. real (PB s))t) * (CDF p (\s. real (PS s))t) ``) THEN1 (
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE]) THEN
DISCH_TAC THEN ASM_REWRITE_TAC[] THEN

WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)``)

THEN1(
RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
THEN




MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.  if i = 0 then  PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
            else
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p):num->'a->bool`, `0:num`,`2:num`]  prob_prod_any_2_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1:num;2;3;4;5;6;7;8]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(`` (!i.
          i IN {0; 2:num} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 2} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) = CDF p(\s. real (CS s)) t *
prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p)``)

THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN








KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)``)

THEN1(
RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
THEN




MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.  if i = 1 then  PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
            else
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p):num->'a->bool`, `1:num`,`2:num`]  prob_prod_any_2_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1:num;2;3;4;5;6;7;8]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(`` (!i:num.
          i IN {1; 2} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {1; 2} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) = CDF p(\s. real (SS s)) t *
prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p)``)

THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN








KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p INTER (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) = prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER 
   p_space p )
   * prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p)``)

THEN1(
RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
THEN




MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.  if i = 2 then  PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p  else
               PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
            ):num->'a->bool`, `2:num`,`3:num`]  prob_prod_any_2_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{2;3} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1:num;2;3;4;5;6;7;8]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN KNOW_TAC(``(!i:num.
          i IN {2; 3} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 3} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p)* prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p) 
 = prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * (CDF p (\s. real (MB s)) t)``)

THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN










KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) =prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p ) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p ) * prob p 
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``)

THEN1(
RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
THEN




MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.  if i = 2 then  PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p  else if i = 4 then 
               PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p else PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
           
            ):num->'a->bool`, `2:num`,`4:num`,`5:num`]  prob_prod_any_3_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{2;4;5} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [2:num;4;5]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 4; 5} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 4; 5} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p ) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p ) * prob p 
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)
 = prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p ) * (CDF p (\s. real ( P s)) t) * ( CDF p (\s. real (B s)) t)``)

THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p))= prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p 
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)``)

THEN1(
RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
THEN




MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.  if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p else PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p  
            ):num->'a->bool`, `0:num`,`1:num`,`2:num`]  prob_prod_any_3_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 1; 2} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 1; 2} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p 
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) = (CDF p (\s. real (CS s)) t) * (CDF p (\s. real (SS s)) t) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)``)

THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)``)

THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.  if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p else if i = 2 then  PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p  else PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p  
            ):num->'a->bool`, `0:num`,`2:num`,`3:num`]  prob_prod_any_3_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;3} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;3:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 2; 3} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 2; 3} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
 prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
 prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p)=
 (CDF p (\s. real (CS s)) t) *
 prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * (CDF p (\s. real (MB s)) t)``)

THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN




KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p    (PREIMAGE MB {u | u <= Normal t} INTER p_space p)``)

THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.  if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p else if i = 2 then  PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p  else PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p  
            ):num->'a->bool`, `1:num`,`2:num`,`3:num`]  prob_prod_any_3_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2;3} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1;2;3:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {1; 2; 3} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {1; 2; 3} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
 prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
 prob p (PREIMAGE MB {u | u <= Normal t} INTER p_space p)=
 (CDF p (\s. real (SS s)) t) *
 prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * (CDF p (\s. real (MB s)) t)``)

THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN






KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p 
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p  ) * prob p 
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)

THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	      else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	      else PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
            ):num->'a->bool`, `2:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_4_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{2;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [2;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p 
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p  ) * prob p 
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) =
    prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * (CDF p (\s. real (PA s)) t) *
     (CDF p (\s. real (PB s)) t) *
     (CDF p (\s. real (PS s)) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``)

THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	      else  PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	          ):num->'a->bool`, `0:num`,`2:num`,`4:num`,`5:num`]  prob_prod_any_4_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;4;5:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;4;5:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 2; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 2; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) =
    (CDF p (\s. real (CS s)) t) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN




KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``)

THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	      else  PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	          ):num->'a->bool`, `1:num`,`2:num`,`4:num`,`5:num`]  prob_prod_any_4_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2;4;5:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1;2;4;5:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {1; 2; 4; 5} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {1; 2; 4; 5} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) =
    (CDF p (\s. real (SS s)) t) * prob p 
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * (CDF p (\s. real (P s)) t) * (CDF p (\s. real (B s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p 
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``)

THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i =  2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	      else  if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	      else PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	          ):num->'a->bool`, `2:num`,`3:num`,`4:num`,`5:num`]  prob_prod_any_4_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{2;3;4;5:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [2;3;4;5:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 3; 4; 5} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p}) ``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {2; 3; 4; 5} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p 
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p 
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) = prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * (CDF p (\s. real (MB s)) t) *
   (CDF p (\s. real (P s)) t) *
   (CDF p (\s. real (B s)) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)``)
THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i =  0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p

else PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	          ):num->'a->bool`, `0:num`,`1:num`,`2:num`,`3:num`]  prob_prod_any_4_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2;3:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2;3:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 1; 2; 3} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 1; 2; 3} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p) IN
          events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p)=  (CDF p (\s. real (CS s)) t)
   * (CDF p (\s. real (SS s)) t)  * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * (CDF p (\s. real (MB s)) t)  ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN







KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)
THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i =  0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	      else if i = 7 then  PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     
	      else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
):num->'a->bool`, `0:num`,`2:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_5_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 2; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 2; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s. real (CS s)) t) *
    prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)*
    ( CDF p (\s. real (PA s)) t)  *
    ( CDF p (\s. real (PB s)) t)
    * (CDF p (\s. real (PS s)) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN




KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)
    
THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i =  1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	      else if i = 7 then  PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     
	      else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
):num->'a->bool`, `1:num`,`2:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_5_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1;2;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {1; 2; 6; 7; 8} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {1; 2; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) = (CDF p (\s. real (SS s)) t) *
    prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p)*
    ( CDF p (\s. real (PA s)) t)  *
    ( CDF p (\s. real (PB s)) t)
    * (CDF p (\s. real (PS s)) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN



KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) ``)

THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i =  0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	      else  PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	   
):num->'a->bool`, `0:num`,`1:num`,`2:num`,`4:num`,`5:num`]  prob_prod_any_5_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2;4;5:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2;4;5:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 1; 2; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``  (!i:num.
          i IN {0; 1; 2; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p) ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) =
    (CDF p (\s. real (CS s)) t)*
     (CDF p (\s. real (SS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
     (CDF p (\s. real (P s)) t)*
      (CDF p (\s. real (B s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(`` prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p )  * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	      else if i =  6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	      else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	      else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	   
):num->'a->bool`, `2:num`,`3:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_5_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{2;3;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [2;3;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 3; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``   (!i:num.
          i IN {2; 3; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p )  * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
    prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * (CDF p (\s. real (MB s)) t)
   * (CDF p (\s. real (PA s)) t)
   * (CDF p (\s. real (PB s)) t)
   * (CDF p (\s. real (PS s)) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN






KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	      else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	      else  PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	   
):num->'a->bool`, `0:num`,`2:num`,`3:num`,`4:num`,`5:num`]  prob_prod_any_5_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;3;4;5:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;3;4;5:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 2; 3; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``    (!i:num.
          i IN {0; 2; 3; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``  prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) =
    (CDF p (\s. real (CS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
    (CDF p (\s. real (MB s)) t)*
    (CDF p (\s. real (P s)) t) *
    (CDF p (\s. real (B s)) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN









KNOW_TAC(`` prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	      else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	      else  PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	   
):num->'a->bool`, `1:num`,`2:num`,`3:num`,`4:num`,`5:num`]  prob_prod_any_5_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2;3;4;5:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1;2;3;4;5:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {1; 2; 3; 4; 5} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``    (!i:num.
          i IN {1; 2; 3; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``  prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) =
    (CDF p (\s. real (SS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
    (CDF p (\s. real (MB s)) t)*
    (CDF p (\s. real (P s)) t) *
    (CDF p (\s. real (B s)) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN











KNOW_TAC(`` prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)  *prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	       if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	      else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	      else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `2:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_6_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{2;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [2;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 4; 5; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``    (!i:num.
          i IN {2; 4; 5; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)   ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``  prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)  *prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * (CDF p (\s. real (P s) ) t) *
   (CDF p (\s. real (B s) ) t) *
   (CDF p (\s. real (PA s) ) t) *
    (CDF p (\s. real (PB s) ) t)*
    (CDF p (\s. real (PS s) ) t) ``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN







KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	      else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`2:num`,`3:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_6_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;3;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;3;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 2; 3; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``   (!i:num.
          i IN {0; 2; 3; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)   ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``  prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p) =
(CDF p (\s. real (CS s))t) *
prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
(CDF p (\s. real (MB s))t)*
(CDF p (\s. real (PA s))t)*
(CDF p (\s. real (PB s))t)*
(CDF p (\s. real (PS s))t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
   (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) INTER
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
   (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) * prob p
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	     
	     else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	      else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`1:num`,`2:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_6_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 1; 2; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``    (!i:num.
          i IN {0; 1; 2; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p (PREIMAGE CS {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE SS {u | u <= Normal t} INTER p_space p) *
prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) *
prob p (PREIMAGE PA {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PB {u | u <= Normal t} INTER p_space p) *
prob p (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
(CDF p (\s. real (CS s)) t) *
(CDF p (\s. real (SS s)) t) *
prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) *
(CDF p (\s. real (PA s)) t) *
(CDF p (\s. real (PB s)) t)*
(CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
   (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
     prob p
   (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	    
	     else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`2:num`,`3:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_6_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;3;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;3;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 2; 3; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``  (!i:num.
          i IN {0; 2; 3; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
   (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
     (CDF p (\s. real (CS s)) t) *
     prob p
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) *
    (CDF p (\s. real (MB s)) t) *
    (CDF p (\s. real (PA s)) t) *
    (CDF p (\s. real (PB s)) t) *
    (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) INTER
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
     prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.
	      if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	    
	     else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `1:num`,`2:num`,`3:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_6_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2;3;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1;2;3;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {1; 2; 3; 6; 7; 8} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``  (!i:num.
          i IN {1; 2; 3; 6; 7; 8} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) * prob p
    (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p 
    (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
     (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
     (CDF p (\s. real (SS s)) t) *
     prob p
    (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p) *
    (CDF p (\s. real (MB s)) t) *
    (CDF p (\s. real (PA s)) t) *
    (CDF p (\s. real (PB s)) t) *
    (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN






KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)))=prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	    
	     else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`1:num`,`2:num`,`3:num`,`4:num`,`5:num`]  prob_prod_any_6_of_10_set)
		  THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2;3;4;5:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2;3;4;5:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 1; 2; 3; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``  (!i:num.
          i IN {0; 1; 2; 3; 4; 5} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p) IN
          events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)=
    (CDF p (\s. real (CS s)) t) *
    (CDF p (\s. real (SS s)) t) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
    (CDF p (\s. real (MB s)) t) *
    (CDF p (\s. real (P s)) t)*
    (CDF p (\s. real (B s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
    prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
     	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`2:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_7_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {0; 2; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 2; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
   (CDF p (\s. real (CS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
   (CDF p (\s. real (P s)) t) * 
   (CDF p (\s. real (B s)) t)* 
   (CDF p (\s. real (PA s)) t) * 
   (CDF p (\s. real (PB s)) t)*
   (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
    prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
     	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `1:num`,`2:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_7_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1;2;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {1; 2; 4; 5; 6; 7; 8} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {1; 2; 4; 5; 6; 7; 8} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
   (CDF p (\s. real (SS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
   (CDF p (\s. real (P s)) t) * 
   (CDF p (\s. real (B s)) t)* 
   (CDF p (\s. real (PA s)) t) * 
   (CDF p (\s. real (PB s)) t)*
   (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN




KNOW_TAC(``prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
    prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
     	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `2:num`,`3:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_7_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{2;3;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [2;3;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
          i IN {2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
         i IN {2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(`` prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
    prob p
  (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
   p_space p) *
   (CDF p (\s. real (MB s)) t) *
   (CDF p (\s. real (P s)) t)*
   (CDF p (\s. real (B s)) t)*
   (CDF p (\s. real (PA s)) t)*
   (CDF p (\s. real (PB s)) t) *
   (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN







KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
    prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	     else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	     
	     else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
             else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`1:num`,`2:num`,`3:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_7_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2;3;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2;3;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
         i IN {0; 1; 2; 3; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 1; 2; 3; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``  prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
  (CDF p (\s. real (CS s)) t) * 
  (CDF p (\s. real (SS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
    (CDF p (\s. real (MB s)) t) *
    (CDF p (\s. real (PA s)) t)*
    (CDF p (\s. real (PB s)) t)*
    (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN






KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
    prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	     else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	     
	     else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`1:num`,`2:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_8_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
	i IN {0; 1; 2; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 1; 2; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``      prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
(CDF p (\s. real (CS s))t) * 
(CDF p (\s. real (SS s))t) * 
prob p   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
    (CDF p (\s. real (P s))t) *
    (CDF p (\s. real (B s))t) *
    (CDF p (\s. real (PA s))t)*
    (CDF p (\s. real (PB s))t) *
    (CDF p (\s. real (PS s))t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`2:num`,`3:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_8_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;2;3;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;2;3;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
i IN {0; 2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
           i IN {0; 2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p) ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``      prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=

(CDF p (\s. real (CS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *  
(CDF p (\s. real (MB s)) t) *
 (CDF p (\s. real (P s)) t) *
 (CDF p (\s. real (B s)) t) *
 (CDF p (\s. real (PA s)) t) *
 (CDF p (\s. real (PB s)) t) *
 (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN



KNOW_TAC(``prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) = prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `1:num`,`2:num`,`3:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_8_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{1;2;3;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [1;2;3;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
i IN {1; 2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
           i IN {1; 2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p) ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``      prob p
  (PREIMAGE SS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p 
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p ) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p)* prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=

(CDF p (\s. real (SS s)) t)* prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *  
(CDF p (\s. real (MB s)) t) *
 (CDF p (\s. real (P s)) t) *
 (CDF p (\s. real (B s)) t) *
 (CDF p (\s. real (PA s)) t) *
 (CDF p (\s. real (PB s)) t) *
 (CDF p (\s. real (PS s)) t)``)
  
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN





KNOW_TAC(``prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) INTER
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) INTER
   (PREIMAGE P {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE B {u | u <= Normal t} INTER p_space p)) INTER
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)``)


THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
      THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num. {if i = 0 then
              PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
            else if i = 1 then
              PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
            else if i = 2 then
              PREIMAGE (\s. (real (MS s),real (MA s)))
                {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
            else if i = 3 then
              PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
            else if i = 4 then
              PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
            else if i = 5 then
              PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
            else if i = 6 then
              PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
            else if i = 7 then
              PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
            else
              PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
              p_space p})`, `{0:num; 1; 2; 3; 4; 5; 6; 7; 8}`,`(\i:num.

	      if i = 0 then PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
	      else if i = 1 then PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
	      else if i = 2 then PREIMAGE (\s. (real (MS s),real (MA s))){(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
	     else if i = 3 then PREIMAGE (\s. real (MB s)) {u | u <=  t} INTER p_space p
	     else if i = 4 then PREIMAGE (\s. real (P s)) {u | u <=  t} INTER p_space p
	     else if i = 5 then PREIMAGE (\s. real (B s)) {u | u <=  t} INTER p_space p
	     else if i = 6 then PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p
	     else if i = 7 then PREIMAGE (\s. real (PB s)) {u | u <=  t} INTER p_space p
	     else  PREIMAGE (\s. real (PS s)) {u | u <=  t} INTER p_space p
	     
	    
):num->'a->bool`, `0:num`,`1:num`,`2:num`,`3:num`,`4:num`,`5:num`,`6:num`,`7:num`,`8:num`]  prob_prod_any_9_of_10_set) THEN


RW_TAC std_ss[]
THEN KNOW_TAC(``{0;1;2;3;4;5;6;7;8:num} SUBSET {0; 1; 2; 3; 4; 5; 6; 7; 8:num} ``)
THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN
KNOW_TAC (``ALL_DISTINCT [0;1;2;3;4;5;6;7;8:num]``)
THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!i:num.
 i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN
          {if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p})``)
THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
THEN
PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
THEN RW_TAC real_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!i:num.
          i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
          (if i = 0 then
             PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
           else if i = 2 then
             PREIMAGE (\s. (real (MS s),real (MA s)))
               {(u,w) | u < w /\0<=w/\ w <= t} INTER p_space p
           else if i = 3 then
             PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p
           else if i = 4 then
             PREIMAGE (\s. real (P s)) {u | u <= t} INTER p_space p
           else if i = 5 then
             PREIMAGE (\s. real (B s)) {u | u <= t} INTER p_space p
           else if i = 6 then
             PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
           else if i = 7 then
             PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
           else
             PREIMAGE (\s. real (PS s)) {u | u <= t} INTER
             p_space p) IN events p)  ``)
THEN1(RW_TAC std_ss [])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[] THEN METIS_TAC [INTER_ASSOC] ) THEN

DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq THEN


KNOW_TAC(``  prob p
  (PREIMAGE CS {u | u <= Normal t} INTER p_space p ) * prob p
   (PREIMAGE SS {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) * prob p
   (PREIMAGE MB {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE P {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE B {u | u <= Normal t} INTER p_space p) * prob p
   (PREIMAGE PA {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PB {u | u <= Normal t} INTER p_space p) * prob p
    (PREIMAGE PS {u | u <= Normal t} INTER p_space p)=
    (CDF p (\s. real (CS s)) t) *
    (CDF p (\s. real (SS s)) t) *
     prob p(PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) *
    (CDF p (\s. real (MB s)) t) *
    (CDF p (\s. real (P s)) t) *
    (CDF p (\s. real (B s)) t) *
    (CDF p (\s. real (PA s)) t) *
    (CDF p (\s. real (PB s)) t) *
    (CDF p (\s. real (PS s)) t) 
    ``)
 
THEN1(
METIS_TAC [CDF_prob_DFT_event, GSYM DFT_PREIMAGE])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN WEAKEN_TAC is_eq
THEN KNOW_TAC (``indep_var p lborel (\s. real (MS s)) lborel (\s. real ( MA s))``)
     THEN1( METIS_TAC [indep_vars_indep_var_MA_MS])
THEN DISCH_TAC
THEN KNOW_TAC(``prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) =  pos_fn_integral lborel
        (\y. f_MA y * (indicator_fn {u | 0<=u/\u <= t} y * CDF p (\s. real (MS s)) y))``)
THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL_2RV]
       THEN METIS_TAC [prob_after])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]);

val _ = export_theory();








