(******************************************************************************)
(*                       Probabilistic Analysis of                            *)
(*                        Cardiac Assist System with different modes of spare *)
(*                                  by                                        *)
(*                          Yassmeen Elderhalli                               *)
(*                      y_elderh@encs.concordia.ca                            *)
(******************************************************************************)

(*loadPath := "/home/yassmeen/Downloads/ProbTheories_HOL-k12" :: !loadPath;
loadPath := "/home/yassmeen/Downloads/My-Theories" :: !loadPath;


(*set_trace "Unicode" 0;*)


app load ["HolKernel", "Parse", "boolLib", "bossLib", "numLib", "unwindLib", 
"tautLib", "Arith", "prim_recTheory", "combinTheory", "quotientTheory", 
"arithmeticTheory", "realTheory", "hrealTheory", "realaxTheory", "realLib",  
"jrhUtils", "pairTheory", "seqTheory", "limTheory", "transcTheory", "listTheory", 
"mesonLib", "boolTheory", "topologyTheory", "pred_setTheory", "util_probTheory", 
"optionTheory", "numTheory", "sumTheory", "InductiveDefinition", "ind_typeTheory",
"pred_setLib", "iterate_hvgTheory", "card_hvgTheory", "product_hvgTheory", 
"topology_hvgTheory", "derivative_hvgTheory", "integration_hvgTheory",
"real_sigmaTheory", "extreal_hvgTheory", "measure_hvgTheory", "lebesgue_hvgTheory",
"probability_hvgTheory", "lebesgue_measure_hvgTheory", "normal_rv_hvgTheory", "netsTheory", "PIE_EXTREALTheory","dep_rewrite", "integration_before_afterTheory", "DFT_Gates_def_PROBTheory" ,"CASTheory","WCSPTheory", "AND_FDEPTheory", "WCSP_sharedTheory", "prob_PAND_CSPTheory"];
*)
open HolKernel Parse boolLib bossLib numLib unwindLib tautLib Arith prim_recTheory 
combinTheory quotientTheory arithmeticTheory hrealTheory realaxTheory realTheory 
realLib jrhUtils pairTheory seqTheory limTheory transcTheory listTheory mesonLib 
boolTheory topologyTheory pred_setTheory util_probTheory optionTheory numTheory 
sumTheory InductiveDefinition ind_typeTheory pred_setLib iterate_hvgTheory 
card_hvgTheory product_hvgTheory topology_hvgTheory derivative_hvgTheory
integration_hvgTheory real_sigmaTheory extreal_hvgTheory measure_hvgTheory
lebesgue_hvgTheory probability_hvgTheory lebesgue_measure_hvgTheory normal_rv_hvgTheory netsTheory PIE_EXTREALTheory dep_rewrite integration_before_afterTheory DFT_Gates_def_PROBTheory  CASTheory WCSPTheory AND_FDEPTheory WCSP_sharedTheory prob_PAND_CSPTheory;



val _ = new_theory "CAS_general";
fun SET_TAC L =
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION,
    IN_INTER, IN_DIFF, IN_INSERT, IN_DELETE, IN_BIGINTER, IN_BIGUNION,
    IN_IMAGE, GSPECIFICATION, IN_DEF] THEN METIS_TAC [];

fun SET_RULE tm = prove(tm,SET_TAC []);


val set_rewrs
= [INTER_EMPTY, INTER_UNIV, UNION_EMPTY, UNION_UNIV, DISJOINT_UNION,
DISJOINT_INSERT, DISJOINT_EMPTY, GSYM DISJOINT_EMPTY_REFL,
DISJOINT_BIGUNION, INTER_SUBSET, INTER_IDEMPOT, UNION_IDEMPOT,
SUBSET_UNION];

val UNIONL_def = Define `(UNIONL [] = {})
/\ (UNIONL (s::ss) = (s:'a ->bool) UNION UNIONL ss)`;


val IN_UNIONL = store_thm
("IN_UNIONL",
``!l (v:'a ). v IN UNIONL l = (?s. MEM s l /\ v IN s)``,
Induct THEN1 RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY]
THEN RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY, IN_UNION]
THEN PROVE_TAC []);


val elt_rewrs
= [SUBSET_DEF, IN_COMPL, IN_DIFF, IN_UNIV, NOT_IN_EMPTY, IN_UNION,
IN_INTER, IN_IMAGE, IN_FUNSET, IN_DFUNSET, GSPECIFICATION,
DISJOINT_DEF, IN_BIGUNION, IN_BIGINTER, IN_COUNT, IN_o,
IN_UNIONL, IN_DELETE, IN_PREIMAGE, IN_SING, IN_INSERT];


fun rewr_ss ths =
simpLib.++
(std_ss,
simpLib.SSFRAG
{ac = [],
name = NONE,
convs = [],
dprocs = [],
filter = NONE,
rewrs = set_rewrs @ elt_rewrs,
congs = []});
val pset_set_ss = rewr_ss set_rewrs;
val pset_elt_ss = rewr_ss elt_rewrs;
val pset_ss = rewr_ss (set_rewrs @ elt_rewrs);


fun PSET_TAC ths =
REPEAT (POP_ASSUM MP_TAC)
THEN RW_TAC pset_set_ss ths
THEN  REPEAT (POP_ASSUM MP_TAC)
THEN RW_TAC pset_elt_ss ths;

val Simplify = RW_TAC arith_ss;
fun wrap a = [a];
val Rewr = DISCH_THEN (REWRITE_TAC o wrap);
val Rewr' = DISCH_THEN (ONCE_REWRITE_TAC o wrap);
val FUN_EQ_TAC = CONV_TAC (CHANGED_CONV (ONCE_DEPTH_CONV FUN_EQ_CONV));
fun K_TAC _ = ALL_TAC;
fun MESON ths tm = prove(tm,MESON_TAC ths);
fun METIS ths tm = prove(tm,METIS_TAC ths);
val DISC_RW_KILL = DISCH_TAC THEN ONCE_ASM_REWRITE_TAC [] THEN 
                   POP_ASSUM K_TAC;
			   
val IN_REST = store_thm ("IN_REST",
 ``!x:'a. !s. x IN (REST s) <=> x IN s /\ ~(x = CHOICE s)``,
  REWRITE_TAC[REST_DEF, IN_DELETE]);
fun SET_TAC L = 
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN
    SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION, IN_INTER, IN_DIFF, 
      IN_INSERT, IN_DELETE, IN_REST, IN_BIGINTER, IN_BIGUNION, IN_IMAGE, 
      GSPECIFICATION, IN_DEF, EXISTS_PROD] THEN METIS_TAC [];
fun ASSERT_TAC tm = SUBGOAL_THEN tm STRIP_ASSUME_TAC;
fun SET_RULE tm = prove(tm,SET_TAC []);
fun ASM_SET_TAC L = REPEAT (POP_ASSUM MP_TAC) THEN SET_TAC L;
val ASM_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN ARITH_TAC;
val ASM_REAL_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN REAL_ARITH_TAC;





val CAS_TE1 = store_thm("CAS_TE1",
  ``!T P B_a B_d.  (D_BEFORE B_a P = NEVER) ==> ((WSP (D_OR T P) (D_OR T B_a)
        (D_OR T B_d)) = D_OR T (WSP P B_a B_d))``,


RW_TAC std_ss[WSP_def, D_AND_def, D_OR_def, D_BEFORE_def, D_SIMULT_def, NEVER_def]
THEN KNOW_TAC(`` !s.(
        min
          (min
             (min
                (max (min (T' s) (B_a s))
                   (if min (T' s) ((P:'a->extreal) s) < min (T' s) (B_a s) then
                      min (T' s) (P s)
                    else PosInf))
                (max (min (T' s) (P s))
                   (if min (T' s) (B_d s) < min (T' s) (P s) then
                      min (T' s) (B_d s)
                    else PosInf)))
             (if min (T' s) (P s) = min (T' s) (B_a s) then
                min (T' s) (B_a s)
              else PosInf))
          (if min (T' s) (P s) = min (T' s) (B_d s) then min (T' s) (B_d s)
           else PosInf)) =
   (
        min (T' s)
          (min
             (min
                (min (max (B_a s) (if P s < B_a s then P s else PosInf))
                   (max (P s) (if B_d s < P s then B_d s else PosInf)))
                (if P s = B_a s then B_a s else PosInf))
             (if P s = B_d s then B_d s else PosInf)))``)
	   THEN1(RW_TAC std_ss [] THEN FULL_SIMP_TAC std_ss[D_BEFORE_def, NEVER_def, D_AND_def]THEN METIS_TAC[extreal_min_def, extreal_max_def, lt_infty, le_infty, lt_le, le_lt, extreal_lt_def, extreal_le_def, lt_total, le_total, lte_trans, let_trans, min_infty, max_infty,ALL_DISTINCT, MEM, le_antisym] )
THEN RW_TAC std_ss[]);


val  CAS_general_reduced = store_thm("CAS_general_reduced",

  `` !PA PB PS MS MA MB B_a B_d CS SS P B.
       (!s. ALL_DISTINCT [MA s; MS s; PA s; PB s; PS s; MB s;P s; B_d s;B_a s; CS s; SS s ]) /\  ( D_BEFORE B_a P = NEVER )  ==>
       (D_OR
          (D_OR
             (D_AND (shared_spare PA PB PS PS)
                (shared_spare PB PA PS PS))
             (D_OR (P_AND MS MA) (CSP MA MB)))
          (WSP (FDEP (D_OR CS SS) P) (FDEP (D_OR CS SS) B_a) (FDEP (D_OR CS SS) B_d)) =
        D_OR
          (D_OR (D_OR (D_OR CS SS) (D_OR (P_AND MS MA) (CSP MA MB)))
             (WSP P B_a B_d)) (D_AND (D_AND PA PB) PS))``,

RW_TAC std_ss[]
THEN KNOW_TAC(``!s. ALL_DISTINCT [(PA :'a->extreal) s; PB s ;PS s]``)
     THEN1(METIS_TAC [ALL_DISTINCT, MEM])
THEN DISCH_TAC
THEN RW_TAC std_ss [hot_shared_spare]
THEN RW_TAC std_ss [HSP_AND]
THEN RW_TAC std_ss [FDEP_OR]
THEN KNOW_TAC(``(WSP (D_OR (D_OR CS SS) P) (D_OR (D_OR CS SS) B_a)
        (D_OR (D_OR CS SS) B_d)) = D_OR (D_OR (CS)(SS))(WSP P B_a B_d)``)
     THEN1(RW_TAC std_ss [CAS_TE1])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]     
THEN RW_TAC std_ss[]
THEN METIS_TAC [D_OR_ASSOC, D_OR_COMM]);

val CSP_def2 = store_thm("CSP_def2",
  ``!X Y. CSP Y X = D_AND X (D_BEFORE Y X)  ``,
RW_TAC std_ss[D_AND_def, D_BEFORE_def, CSP_def]
THEN KNOW_TAC(`` !s. ( if Y s < X s then X s else PosInf) =
   ( max (X s) (if Y s < X s then Y s else PosInf))``)

     THEN1( RW_TAC std_ss [] THEN FULL_SIMP_TAC std_ss[D_BEFORE_def, NEVER_def, D_AND_def]THEN METIS_TAC[extreal_min_def, extreal_max_def, lt_infty, le_infty, lt_le, le_lt, extreal_lt_def, extreal_le_def, lt_total, le_total, lte_trans, let_trans, min_infty, max_infty,ALL_DISTINCT, MEM, le_antisym])
THEN RW_TAC std_ss[]);     




val CAS_general_final = store_thm("CAS_general_final",
  `` !PA PB PS MS MA MB B_a B_d CS SS P B.
       (!s. ALL_DISTINCT [MA s; MS s; PA s; PB s; PS s; MB s; P s; B_d s;B_a s; CS s; SS s ]) /\  ( D_BEFORE B_a P = NEVER )  ==>
       (D_OR
          (D_OR
             (D_AND (shared_spare PA PB PS PS)
                (shared_spare PB PA PS PS))
             (D_OR (P_AND MS MA) (CSP MA MB)))
          (WSP (FDEP (D_OR CS SS) P) (FDEP (D_OR CS SS) B_a) (FDEP (D_OR CS SS) B_d)) =
	   D_OR CS
     (D_OR SS
        (D_OR (D_AND MA (D_BEFORE MS MA))
           (D_OR (D_AND MB (D_BEFORE MA MB))
              (D_OR (D_AND B_a (D_BEFORE P B_a))
                 (D_OR (D_AND P (D_BEFORE B_d P)) (D_AND (D_AND PA PB) PS)))))) ) ``,

RW_TAC std_ss[CAS_general_reduced]
THEN RW_TAC std_ss[PAND_DEF2, CSP_def2, WSP_def]
THEN KNOW_TAC(``D_INCLUSIVE_BEFORE MS MA = D_BEFORE MS MA ``)
    THEN1(RW_TAC std_ss[D_INCLUSIVE_BEFORE_def2]
          THEN KNOW_TAC(``D_SIMULT MS MA = NEVER``)
	        THEN1(RW_TAC std_ss[D_SIMULT_def, NEVER_def]
		      THEN FULL_SIMP_TAC std_ss[ALL_DISTINCT, MEM])
          THEN RW_TAC std_ss[D_OR_NEVER])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN `(D_SIMULT P B_a) = NEVER` by (RW_TAC std_ss[D_SIMULT_def, NEVER_def]
		      THEN FULL_SIMP_TAC std_ss[ALL_DISTINCT, MEM])
THEN `(D_SIMULT P B_d) = NEVER` by (RW_TAC std_ss[D_SIMULT_def, NEVER_def]
		      THEN FULL_SIMP_TAC std_ss[ALL_DISTINCT, MEM])

THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[D_OR_NEVER]
THEN RW_TAC std_ss[D_OR_ASSOC]);


val CAS_genral_UNION_LIST = store_thm("CAS_general_UNION_LIST",
  ``   !PA PB PS MS MA MB CS SS P B_a B_d p t.
DFT_event p (D_OR CS
             (D_OR SS
                (D_OR (D_AND MA (D_BEFORE MS MA))
                   (D_OR (D_AND MB (D_BEFORE MA MB))
                      (D_OR (D_AND B_a (D_BEFORE P B_a))
                         (D_OR (D_AND P (D_BEFORE B_d P))
                            (D_AND (D_AND PA PB) PS))))))) t  = (union_list
             [DFT_event p CS t; DFT_event p SS t;
              DFT_event p (D_AND MA (D_BEFORE MS MA)) t;
              DFT_event p (D_AND MB (D_BEFORE MA MB)) t;
	      DFT_event p (D_OR (D_AND B_a (D_BEFORE P B_a))(D_AND P (D_BEFORE B_d P))) t;
              DFT_event p (D_AND (D_AND PA PB) PS) t])``,
RW_TAC std_ss [OR_UNION]
THEN RW_TAC std_ss [union_list_def]
THEN PSET_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION, UNION_COMM, UNION_ASSOC]
THEN EQ_TAC THEN RW_TAC std_ss[] THEN RW_TAC std_ss[]);


val ALL_DISTINCT_RVg_def = Define `ALL_DISTINCT_RVg [A0; A1; A2; A3; A4; A5; A6; A7; A8; A9; A10] p t=
   (All_distinct_events p [ A6; A7; (D_AND A4 (D_BEFORE A3 A4));
                              (D_AND A5 (D_BEFORE A4 A5));
                              (D_OR (D_AND A9 (D_BEFORE A8 A9))(D_AND A8 (D_BEFORE A10 A8)));
			      D_AND (D_AND A0 A1) A2] t) /\

   rv_gt0_ninfinity [A0; A1 ; A2 ; A3 ; A4 ; A5; A6 ;A7 ;A8 ;A9; A10] /\
   (!s. ALL_DISTINCT (MAP (\a. a s) [A0; A1; A2; A3; A4; A5; A6; A7; A8; A9; A10]))`;



val PAND_CSP_event = store_thm("PAND_CSP_event",
  `` ! X Y Z p t. rv_gt0_ninfinity [X;Y;Z] /\ (!s. ALL_DISTINCT [X s; Y s ; Z s]) ==>

   (DFT_event p (D_AND (D_AND (X)(D_BEFORE Z Y))(D_BEFORE Y X)) t =
           {s|  0 <= Z s /\  Z s < Y s /\ 0 <= Y s /\ Y s <= X s /\ X s <=Normal t } INTER p_space p)``,

RW_TAC std_ss[AND_INTER, rv_gt0_ninfinity_def] 
THEN RW_TAC std_ss[D_BEFORE_def,DFT_event_def] 
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1 (METIS_TAC [lt_le, le_lt])
	   THEN1(METIS_TAC [extreal_not_infty, extreal_le_def, extreal_lt_def, lt_le, le_lt, lt_infty, le_infty])
	   THEN1(METIS_TAC [extreal_not_infty, extreal_le_def, extreal_lt_def, lt_le, le_lt, lt_infty, le_infty])
	   THEN METIS_TAC [extreal_not_infty, extreal_le_def, extreal_lt_def, lt_le, le_lt, lt_infty, le_infty])
THEN RW_TAC std_ss []
THEN METIS_TAC [extreal_not_infty, extreal_le_def, extreal_lt_def, lt_le, le_lt, lt_infty, le_infty, let_trans, lte_trans, ALL_DISTINCT, MEM]);



    
val indep_vars_setsg_def = Define `
    indep_vars_setsg [A0; A1; A2; A3; A4; A5; A6; A7; A8; A9; A10] p t =
    random_variable (\s. real (A9 s)) p borel /\
   random_variable (\s. real (A5 s)) p borel/\

     indep_vars p (\i. lborel)
         (\i. if i = 0 then (\s. real (A0 s))
	      else if i = 1 then (\s. real (A1 s))
	      else if i = 2 then (\s. real (A2 s))
	      else if i = 3 then (\s. real (A3 s))
	      else if i = 4 then (\s. real (A4 s))
	      else if i = 5 then (\s. real (A6 s))
	      else if i = 6 then (\s. real (A7 s))
	      else if i = 7 then (\s. real (A8 s))
	      else (\s. real (A10 s))) {0;1;2;3;4;5;6;7;8}
	       /\
     indep_sets p
       (\i.
          {if i = 0 then
             PREIMAGE (\s. real (A0 s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (A1 s)) {u | u <= t} INTER p_space p
           else if i = 2 then
	    PREIMAGE (\s. real (A2 s)) {u | u <= t} INTER p_space p
           else if i = 3 then
	    DFT_event p (CSP A4 A5) t 
	   else if i = 4 then
	    DFT_event p (WSP A8 A9 A10) t
	   else if i = 5 then
	    PREIMAGE (\s. real (A6 s)) {u | u <= t} INTER p_space p
          else  PREIMAGE (\s. real (A7 s)) {u | u <= t} INTER p_space p})
          
	     {0; 1; 2; 3; 4; 5;6} /\
	     
      indep_sets p
       (\i.
          {if i = 0 then
             PREIMAGE (\s. real (A0 s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (A1 s)) {u | u <= t} INTER p_space p
           else if i = 2 then
	    PREIMAGE (\s. real (A2 s)) {u | u <= t} INTER p_space p
           else if i = 3 then
	    PREIMAGE (\s. (real (A3 s),real (A4 s)))
              {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
           else if i = 4 then
	    DFT_event p (WSP A8 A9 A10) t
	   else if i = 5 then
	    PREIMAGE (\s. real (A6 s)) {u | u <= t} INTER p_space p
          else  PREIMAGE (\s. real (A7 s)) {u | u <= t} INTER p_space p})
          
	     {0; 1; 2; 3; 4; 5;6} /\
	 indep_sets p
       (\i.
          {if i = 0 then
             PREIMAGE (\s. real (A0 s)) {u | u <= t} INTER p_space p
           else if i = 1 then
             PREIMAGE (\s. real (A1 s)) {u | u <= t} INTER p_space p
           else if i = 2 then
	    PREIMAGE (\s. real (A2 s)) {u | u <= t} INTER p_space p
           else if i = 3 then
            PREIMAGE (\s. (real (A3 s),real (A4 s)))
              {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
	      DFT_event p (CSP A4 A5) t 
           else if i = 4 then
	    DFT_event p (WSP A8 A9 A10) t
	   else if i = 5 then
	    PREIMAGE (\s. real (A6 s)) {u | u <= t} INTER p_space p
          else  PREIMAGE (\s. real (A7 s)) {u | u <= t} INTER p_space p})
          
	     {0; 1; 2; 3; 4; 5;6} `;

val CAS_general_PROB = store_thm ("CAS_general_PROB",
  ``  
     !CS SS MA MS MB P B_a B_d PA PB PS p t f_MA f_BP f_P f_cond_P_B f_MB_MA f_cond_MA_MB.
       0 <= t /\ prob_space p /\ (D_BEFORE B_a P = NEVER) /\
       ALL_DISTINCT_RVg [PA;  PB;  PS;  MS;  MA;  MB;  CS;  SS;  P ; B_a;  B_d] p t /\
       indep_vars_setsg [PA;  PB;  PS;  MS;  MA;  MB;  CS;  SS;  P ; B_a;  B_d] p t /\
      DISJOINT_WSP  P B_a B_d t /\
       (!x y.
            cond_density lborel lborel p (\s. real (B_a s)) (\s. real (P s))
              y f_BP f_P f_cond_P_B) /\ den_gt0_ninfinity f_BP f_P f_cond_P_B /\
       cont_CDF p (\s. real (B_d s)) /\ measurable_CDF p (\s. real (B_d s)) /\
       (!x y.
            cond_density lborel lborel p (\s. real (MB s)) (\s. real (MA s)) y
              f_MB_MA f_MA f_cond_MA_MB) /\  den_gt0_ninfinity f_MB_MA f_MA f_cond_MA_MB /\ 

           indep_varp p (pair_measure lborel lborel)
            (\x. (real (MB x),real (MA x))) lborel (\x. real (MS x)) /\
         (!x. f_MB_MA x <> PosInf) /\
       cont_CDF p (\s. real (MS s)) /\
       measurable_CDF p (\s. real (MS s)) ==>
        (prob p
          (DFT_event p
             (D_OR
             (D_OR
                (D_AND (shared_spare PA PB PS PS) (shared_spare PB PA PS PS))
                (D_OR (P_AND MS MA) (CSP MA MB)))
             (WSP (FDEP (D_OR CS SS) P) (FDEP (D_OR CS SS) B_a)
                (FDEP (D_OR CS SS) B_d))) t) =  CDF p (\s. real (CS s)) t + CDF p (\s. real (SS s)) t +
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) +
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) +
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t -
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) -
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) -
   CDF p (\s. real (CS s)) t *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) -
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) -
   CDF p (\s. real (SS s)) t *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   CDF p (\s. real (SS s)) t * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) -
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t -
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t -
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) +
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) +
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t +
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t +
   CDF p (\s. real (SS s)) t *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t +
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) -
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t -
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t *
   CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y *
          (indicator_fn {u | 0 <= u /\ u <= t} y * CDF p (\s. real (MS s)) y)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
          pos_fn_integral lborel
            (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   CDF p (\s. real (CS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t +
   CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t -
   CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
   pos_fn_integral lborel
     (\y.
          f_MA y * CDF p (\s. real (MS s)) y *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x. f_cond_MA_MB y x * indicator_fn {x' | y < x' /\ x' <= t} x)) *
   (pos_fn_integral lborel
      (\y.
           indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
           pos_fn_integral lborel
             (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
    pos_fn_integral lborel
      (\y.
           f_P y *
           (indicator_fn {u | 0 <= u /\ u <= t} y *
            CDF p (\s. real (B_d s)) y))) * CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t)``,

RW_TAC std_ss[indep_vars_setsg_def, ALL_DISTINCT_RVg_def, rv_gt0_ninfinity_def]
THEN FULL_SIMP_TAC std_ss[MAP]
THEN `!s. 0<= CS s /\ CS s <> PosInf /\ 0<= SS s /\ SS s <> PosInf /\
          0<= MA s /\ MA s <> PosInf /\ 0<= MS s /\ MS s <> PosInf /\
	  0<= MB s /\ MB s <> PosInf /\ 0<= P s /\ P s <> PosInf /\
	  0<= B_a s /\ B_a s <> PosInf /\0<= B_a s /\ B_a s <> PosInf/\
	  0<= PA s /\ PA s <> PosInf /\
	  0<= PB s /\ PB s <> PosInf /\ 0<= PS s /\ PS s <> PosInf ` by RW_TAC std_ss[]
THEN KNOW_TAC(``!t:real. {(u,w) | u < w /\ 0 <= w /\ w <= t} IN measurable_sets (pair_measure lborel lborel)``)
     THEN1(METIS_TAC [after_set_IN_MEASURABLE_SETS_PAIR_lborel])
THEN DISCH_TAC     
THEN KNOW_TAC (``!s. (ALL_DISTINCT [(MA:'a->extreal) s; MS s; PA s; PB s; PS s; MB s; P s; B_d s; B_a s; CS s; SS s])``)
              THEN1(RW_TAC std_ss[]
	            THEN UNDISCH_TAC(``!s.
             ALL_DISTINCT
               [(PA:'a->extreal) s; PB s; PS s; MS s; MA s; MB s; CS s; SS s; P s; B_a s;
                B_d s]``)
		     THEN DISCH_TAC
		     THEN POP_ASSUM (MP_TAC o Q.SPEC `s:'a `)
		     THEN RW_TAC std_ss [ALL_DISTINCT, MEM])
THEN RW_TAC std_ss[ CAS_general_final,CAS_genral_UNION_LIST]
THEN KNOW_TAC (``random_variable (\s. real (CS s)) p borel /\
                  random_variable (\s. real (SS s)) p borel /\
		  random_variable (\s. real (MA s)) p borel /\
		  random_variable (\s. real (MS s)) p borel /\
		  random_variable (\s. real (MB s)) p borel /\
		  random_variable (\s. real (P s)) p borel /\
		  random_variable (\s. real (B_d s)) p borel /\
		  random_variable (\s. real (B_a s)) p borel /\
		  random_variable (\s. real (PA s)) p borel /\
		  random_variable (\s. real (PB s)) p borel /\
		  random_variable (\s. real (PS s)) p borel ``)
      THEN1(FULL_SIMP_TAC std_ss [indep_vars_10_def, indep_vars]
           THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
                THEN1( METIS_TAC [m_space_def, measurable_sets_def, space_def, subsets_def, lborel, borel, sigma_def])
      THEN DISCH_TAC
      THEN RW_TAC std_ss []
           THEN1(UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	         THEN DISCH_TAC
	    	 THEN POP_ASSUM (MP_TAC o Q.SPEC `5:num `)
	    	 THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	    THEN1( UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	        THEN DISCH_TAC
	    	  THEN POP_ASSUM (MP_TAC o Q.SPEC `6:num `)
	    	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1(UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	               THEN DISCH_TAC
	         THEN POP_ASSUM (MP_TAC o Q.SPEC `4:num `)
	         THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	    THEN1( UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	        THEN DISCH_TAC
	         THEN POP_ASSUM (MP_TAC o Q.SPEC `3:num `)
	         THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	         THEN DISCH_TAC
	          THEN POP_ASSUM (MP_TAC o Q.SPEC `7:num `)
	          THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	               THEN DISCH_TAC
	    	  THEN POP_ASSUM (MP_TAC o Q.SPEC `8:num `)
	    	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	        THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `0:num `)
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN1( UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	          THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `1:num `)
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   THEN UNDISCH_TAC (``!i:num.
             i IN {0; 1; 2; 3; 4; 5; 6; 7; 8} ==>
             random_variable
               (if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))) p
               (m_space lborel,measurable_sets lborel)``) 
	     THEN DISCH_TAC
	    THEN POP_ASSUM (MP_TAC o Q.SPEC `2:num `)
	    THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
	   
THEN DISCH_TAC
THEN KNOW_TAC(``{u|u<=t:real} IN measurable_sets lborel``)
THEN1( RW_TAC std_ss [lborel_le])
THEN DISCH_TAC  
THEN KNOW_TAC(``{u|u<=t:real} IN subsets borel``)
THEN1( METIS_TAC [measurable_sets_def, lborel, borel , subsets_def, sigma_def])
THEN DISCH_TAC
THEN KNOW_TAC(``PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (MA s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,  random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (MS s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE(\s. real ( MB s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel, random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE(\s. real ( P s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel, random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real ( B_a s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel, random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real ( B_d s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel, random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       

THEN KNOW_TAC(``PREIMAGE (\s. real (PA s)) {u | u <=  t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,  random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       
THEN KNOW_TAC(``PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1(  RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,  random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       

THEN KNOW_TAC(``PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p IN events p ``)
     THEN1( RW_TAC std_ss [events_def]
            THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,             random_variable_def, sigma_def, p_space_def] )
THEN DISCH_TAC 	       



THEN KNOW_TAC(``PREIMAGE (\s. (real (MS s), real (MA s))) {(u,w) | u < w /\ 0<=w/\w<=  t} INTER p_space p IN                       events p ``)
     THEN1 ( KNOW_TAC(``(\s. (real(MS s) , real(MA s))) IN measurable (p_space p , events p) (m_space (pair_measure lborel lborel),measurable_sets (pair_measure lborel lborel))``) 
              THEN1(FULL_SIMP_TAC std_ss [random_variable_def]
	             THEN METIS_TAC [ measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, measurable_Pair, measure_space_lborel, prob_space_def])
      THEN DISCH_TAC
      
      
      THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, PREIMAGE_EXTREAL_REAL_2RV])		    
THEN DISCH_TAC
THEN KNOW_TAC(``{(x,y) | y < x /\ x <= t /\ 0 <= y /\ y <= t} IN measurable_sets (pair_measure lborel lborel) ``)
     THEN1(RW_TAC std_ss[CSP_set_IN_MEASURABLE_SETS_PAIR_lborel])
THEN DISCH_TAC     
THEN KNOW_TAC(``  PREIMAGE (\s. (real (MB s),real (MA s)))
                     {(x,y) | y < x /\ x <= t /\ 0 <= y /\ y <= t} INTER
                   p_space p IN events p``)
     THEN1(KNOW_TAC(``(\s. (real(MB s) , real(MA s))) IN measurable (p_space p , events p) (m_space (pair_measure lborel lborel),measurable_sets (pair_measure lborel lborel))``) 
              THEN1(FULL_SIMP_TAC std_ss [random_variable_def]
	             THEN METIS_TAC [ measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, measurable_Pair, measure_space_lborel, prob_space_def])
      THEN DISCH_TAC
      
      THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, PREIMAGE_EXTREAL_REAL_2RV_BEFORE_CSP])
THEN DISCH_TAC

THEN KNOW_TAC(``  PREIMAGE (\s. (real (B_a s),real (P s)))
                     {(x,y) | y < x /\ x <= t /\ 0 <= y /\ y <= t} INTER
                   p_space p IN events p``)
     THEN1(KNOW_TAC(``(\s. (real(B_a s) , real(P s))) IN measurable (p_space p , events p) (m_space (pair_measure lborel lborel),measurable_sets (pair_measure lborel lborel))``) 
              THEN1(FULL_SIMP_TAC std_ss [random_variable_def]
	             THEN METIS_TAC [ measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, measurable_Pair, measure_space_lborel, prob_space_def])
      THEN DISCH_TAC
      
      THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, PREIMAGE_EXTREAL_REAL_2RV_BEFORE_CSP])
THEN DISCH_TAC


THEN KNOW_TAC(``  PREIMAGE (\s. (real (P s),real (B_d s)))
                     {(x,y) | y < x /\ x <= t /\ 0 <= y /\ y <= t} INTER
                   p_space p IN events p``)
     THEN1(KNOW_TAC(``(\s. (real(P s) , real(B_d s))) IN measurable (p_space p , events p) (m_space (pair_measure lborel lborel),measurable_sets (pair_measure lborel lborel))``) 
              THEN1(FULL_SIMP_TAC std_ss [random_variable_def]
	             THEN METIS_TAC [ measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, measurable_Pair, measure_space_lborel, prob_space_def])
      THEN DISCH_TAC
      
      THEN METIS_TAC [IN_MEASURABLE, measurable_sets_def, subsets_def, m_space_def, events_def, space_def, lborel, borel,random_variable_def, sigma_def, p_space_def, PREIMAGE_EXTREAL_REAL_2RV_BEFORE_CSP])
THEN DISCH_TAC





THEN KNOW_TAC(``DFT_event p CS t IN events p /\ DFT_event p SS t IN events p /\
       DFT_event p (D_AND MA (D_BEFORE MS MA)) t IN events p /\
       DFT_event p (D_AND MB (D_BEFORE MA MB)) t IN events p /\
       DFT_event p
           (D_OR (D_AND B_a (D_BEFORE P B_a)) (D_AND P (D_BEFORE B_d P))) t IN events p /\
       DFT_event p (D_AND (D_AND PA PB) PS) t IN events p  ``)
    THEN1(
       KNOW_TAC(``(!s. 0<= (MA:'a->extreal) s)``) THEN1(
       FULL_SIMP_TAC std_ss [rv_gt0_ninfinity_def]) THEN
       DISCH_TAC THEN
       KNOW_TAC(``(DFT_event p (D_AND MA (D_BEFORE MS MA)) t =
      PREIMAGE (\x. (MS x,MA x))
        {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER p_space p)``) THEN1(
	METIS_TAC[DFT_event_AND_BEFORE_PREIMAGE_GT0])
	THEN DISCH_TAC
	THEN ASM_REWRITE_TAC[] THEN
	REWRITE_TAC [AND_INTER] THEN
       REWRITE_TAC [DFT_PREIMAGE] THEN
       FULL_SIMP_TAC std_ss [rv_gt0_ninfinity_def] THEN 
       
       RW_TAC std_ss [PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV, EVENTS_INTER]
       THEN1( KNOW_TAC(``PREIMAGE (\s. real (MB s)) {u | u <= t} INTER p_space p INTER
                       (PREIMAGE (D_BEFORE MA MB) {u | u <= Normal t} INTER p_space p)=
                        PREIMAGE (\s. (real (MB s),real (MA s)))
           {(x,y) | y < x /\ x <= t /\ 0 <= y /\ y <= t} INTER p_space p ``)
	   THEN1(RW_TAC std_ss[GSYM AND_INTER, GSYM DFT_PREIMAGE, GSYM PREIMAGE_EXTREAL_REAL]
	         THEN RW_TAC std_ss[GSYM CSP_def2]
		 THEN RW_TAC std_ss[CSP_event]
		 THEN KNOW_TAC(``{s | MA s < MB s /\ MB s <= Normal t /\ MA s <= Normal t} INTER p_space p =
		        PREIMAGE (\s. (MB s, MA s))
             {(x,y) | y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t} INTER p_space p``)
	              THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
		            THEN EQ_TAC
			         THEN1(RW_TAC std_ss[]
				       THEN Q.EXISTS_TAC(`(MB x, MA x) `)
				       THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
			    THEN STRIP_TAC
			    THEN KNOW_TAC(``(\ (x,y). ((x,y),y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t))
           (x':extreal#extreal) = (x', SND x' < FST x' /\ FST x' <= Normal t /\ 0 <= SND x' /\ SND x' <= Normal t) ``)
	                        THEN1(`(x':extreal#extreal) = (FST x', SND x')` by RW_TAC std_ss[PAIR]
				      THEN ONCE_ASM_REWRITE_TAC[]
				      THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
			    THEN DISCH_TAC
			    THEN FULL_SIMP_TAC std_ss[]
			    THEN `FST (x':extreal#extreal) = MB x` by RW_TAC std_ss[PAIR]
			    THEN `SND (x':extreal#extreal) = MA x` by RW_TAC std_ss[PAIR]
			    THEN FULL_SIMP_TAC std_ss[])
		THEN DISCH_TAC
		THEN ONCE_ASM_REWRITE_TAC[]
  		THEN METIS_TAC[PREIMAGE_EXTREAL_REAL_2RV_BEFORE_CSP])
           THEN DISCH_TAC		
           THEN ONCE_ASM_REWRITE_TAC[]			    
           THEN RW_TAC std_ss[])
     THEN  RW_TAC std_ss[GSYM DFT_PREIMAGE]     
     THEN RW_TAC std_ss[OR_UNION]
     THEN RW_TAC std_ss[GSYM CSP_def2]
     THEN RW_TAC std_ss[CSP_event]
     THEN KNOW_TAC(``{s | P s < B_a s /\ B_a s <= Normal t /\ P s <= Normal t} INTER p_space p IN events p ``) 

     	  THEN1(KNOW_TAC(``{s | P s < B_a s /\ B_a s <= Normal t /\ P s <= Normal t} INTER p_space p =
		        PREIMAGE (\s. (B_a s, P s))
             {(x,y) | y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t} INTER p_space p``)
	              THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
		            THEN EQ_TAC
			         THEN1(RW_TAC std_ss[]
				       THEN Q.EXISTS_TAC(`(B_a x, P x) `)
				       THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
			    THEN STRIP_TAC
			    THEN KNOW_TAC(``(\ (x,y). ((x,y),y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t))
           (x':extreal#extreal) = (x', SND x' < FST x' /\ FST x' <= Normal t /\ 0 <= SND x' /\ SND x' <= Normal t) ``)
	                        THEN1(`(x':extreal#extreal) = (FST x', SND x')` by RW_TAC std_ss[PAIR]
				      THEN ONCE_ASM_REWRITE_TAC[]
				      THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
			    THEN DISCH_TAC
			    THEN FULL_SIMP_TAC std_ss[]
			    THEN `FST (x':extreal#extreal) = B_a x` by RW_TAC std_ss[PAIR]
			    THEN `SND (x':extreal#extreal) = P x` by RW_TAC std_ss[PAIR]
			    THEN FULL_SIMP_TAC std_ss[])
		THEN DISCH_TAC
		THEN ONCE_ASM_REWRITE_TAC[]
  		THEN METIS_TAC[PREIMAGE_EXTREAL_REAL_2RV_BEFORE_CSP])
      THEN DISCH_TAC
      THEN KNOW_TAC(`` {s | B_d s < P s /\ P s <= Normal t /\ B_d s <= Normal t} INTER p_space p IN
   events p``)
           THEN1(KNOW_TAC(``{s | B_d s < P s /\ P s <= Normal t /\ B_d s <= Normal t} INTER p_space p =
		        PREIMAGE (\s. (P s, B_d s))
             {(x,y) | y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t} INTER p_space p``)
	              THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
		            THEN EQ_TAC
			         THEN1(RW_TAC std_ss[]
				       THEN Q.EXISTS_TAC(`(P x, B_d x) `)
				       THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
			    THEN STRIP_TAC
			    THEN KNOW_TAC(``(\ (x,y). ((x,y),y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t))
           (x':extreal#extreal) = (x', SND x' < FST x' /\ FST x' <= Normal t /\ 0 <= SND x' /\ SND x' <= Normal t) ``)
	                        THEN1(`(x':extreal#extreal) = (FST x', SND x')` by RW_TAC std_ss[PAIR]
				      THEN ONCE_ASM_REWRITE_TAC[]
				      THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
			    THEN DISCH_TAC
			    THEN FULL_SIMP_TAC std_ss[]
			    THEN `FST (x':extreal#extreal) = P x` by RW_TAC std_ss[PAIR]
			    THEN `SND (x':extreal#extreal) = B_d x` by RW_TAC std_ss[PAIR]
			    THEN FULL_SIMP_TAC std_ss[])
		THEN DISCH_TAC
		THEN ONCE_ASM_REWRITE_TAC[]
  		THEN METIS_TAC[PREIMAGE_EXTREAL_REAL_2RV_BEFORE_CSP])
      THEN DISCH_TAC
      THEN RW_TAC std_ss[]
    THEN METIS_TAC[EVENTS_UNION])
THEN DISCH_TAC
THEN `ALL_DISTINCT
           [DFT_event p CS t; DFT_event p SS t;
            DFT_event p (D_AND MA (D_BEFORE MS MA)) t;
            DFT_event p (D_AND MB (D_BEFORE MA MB)) t;
            DFT_event p
              (D_OR (D_AND B_a (D_BEFORE P B_a)) (D_AND P (D_BEFORE B_d P)))
              t; DFT_event p (D_AND (D_AND (PA)(PB))(PS)) t]` by  FULL_SIMP_TAC std_ss[All_distinct_events_def, MAP]
THEN DEP_REWRITE_TAC[PROB_PIE_63]
THEN RW_TAC std_ss[]
THEN KNOW_TAC(``prob p (DFT_event p CS t) = CDF p (\s. real (CS s)) t``)
THEN1( DEP_REWRITE_TAC[ CDF_prob_DFT_event]
THEN
RW_TAC std_ss[]) 

THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``prob p (DFT_event p SS t) = CDF p (\s. real (SS s)) t``)
     THEN1 (DEP_REWRITE_TAC[ CDF_prob_DFT_event]
             THEN RW_TAC std_ss[])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``D_AND MB (D_BEFORE MA MB) = CSP MA MB ``)
    THEN1(RW_TAC std_ss[CSP_def2])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]    
THEN KNOW_TAC(``D_OR (D_AND B_a (D_BEFORE P B_a)) (D_AND P (D_BEFORE B_d P)) = WSP P B_a B_d ``)
     THEN1(RW_TAC std_ss[WSP_def]
           THEN KNOW_TAC(``D_SIMULT P B_a = NEVER``)
	        THEN1(RW_TAC std_ss[D_SIMULT_def, NEVER_def]
		      THEN `!s. P s <> B_a s` by METIS_TAC [ALL_DISTINCT, MEM]
  		      THEN RW_TAC std_ss[])
	    THEN RW_TAC std_ss[]	      
       	    THEN KNOW_TAC(``D_SIMULT P B_d = NEVER``)
	        THEN1(RW_TAC std_ss[D_SIMULT_def, NEVER_def]
		      THEN `!s. P s <> B_d s` by METIS_TAC [ALL_DISTINCT, MEM]
  		      THEN RW_TAC std_ss[])
	    THEN RW_TAC std_ss[D_OR_NEVER])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]	    
THEN RW_TAC std_ss[AND_INTER]
THEN KNOW_TAC(``DFT_event p MA t INTER DFT_event p (D_BEFORE MS MA) t =  PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
     p_space p``)
     THEN1(RW_TAC std_ss [GSYM AND_INTER]
           THEN RW_TAC std_ss [DFT_event_AND_BEFORE_PREIMAGE_GT0])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[DFT_PREIMAGE]
THEN KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p)) =
      CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t``)
     THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
            THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
                if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `5:num`,`6:num`  ]  prob_prod_any_2_of_10)
		THEN RW_TAC std_ss[]
		THEN `{5;6:num} SUBSET {0;1;2;3;4;5;6;7;8:num}` by ASM_SET_TAC[]
		THEN FULL_SIMP_TAC std_ss[]
		THEN METIS_TAC[ CDF_prob_DFT_event, DFT_PREIMAGE, PREIMAGE_EXTREAL_REAL])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq
THEN KNOW_TAC(`` prob p
     (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE PS {u | u <= Normal t} INTER p_space p)) =
      CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t``)
     THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
            THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
                if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `0:num`,`1:num`, `2:num`  ]  prob_prod_any_3_of_10)
		THEN RW_TAC std_ss[]
		THEN `{0;1;2:num} SUBSET {0;1;2;3;4;5;6;7;8:num}` by ASM_SET_TAC[]
		THEN `ALL_DISTINCT [0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
		THEN FULL_SIMP_TAC std_ss[]
		THEN METIS_TAC[ CDF_prob_DFT_event, DFT_PREIMAGE, PREIMAGE_EXTREAL_REAL])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq
THEN  KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
     CDF p (\s. real (CS s)) t *  CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t``)
     THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
            THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
                if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `5:num`,`0:num`,`1:num`, `2:num`  ]  prob_prod_any_4_of_10)
		THEN RW_TAC std_ss[]
		THEN `{5;0;1;2:num} SUBSET {0;1;2;3;4;5;6;7;8:num}` by ASM_SET_TAC[]
		THEN `ALL_DISTINCT [5;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
		THEN FULL_SIMP_TAC std_ss[]
		THEN METIS_TAC[INTER_ASSOC,CDF_prob_DFT_event,DFT_PREIMAGE, PREIMAGE_EXTREAL_REAL])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq
THEN KNOW_TAC(`` prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
     CDF p (\s. real (SS s)) t *  CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t``)
     THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
            THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
                if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `6:num`,`0:num`,`1:num`, `2:num`  ]  prob_prod_any_4_of_10)
		THEN RW_TAC std_ss[]
		THEN `{6;0;1;2:num} SUBSET {0;1;2;3;4;5;6;7;8:num}` by ASM_SET_TAC[]
		THEN `ALL_DISTINCT [6;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
		THEN FULL_SIMP_TAC std_ss[]
		THEN METIS_TAC[INTER_ASSOC,CDF_prob_DFT_event,DFT_PREIMAGE, PREIMAGE_EXTREAL_REAL])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq
THEN KNOW_TAC(``prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
      CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *  CDF p (\s. real (PA s)) t * CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t``)
     THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL]
            THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`, `(\i. lborel):num->((real->bool)#((real->bool)->bool)#((real->bool)->extreal))`, ` (\i.
                if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real (PB s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s))):num->'a->real`,`{0;1;2;3;4;5;6;7;8}:num->bool`, `(\i. {u|u<=t:real}):num->real->bool `, `5:num`,`6:num`,`0:num`,`1:num`, `2:num`  ]  prob_prod_any_5_of_10)
		THEN RW_TAC std_ss[]
		THEN `{5;6;0;1;2:num} SUBSET {0;1;2;3;4;5;6;7;8:num}` by ASM_SET_TAC[]
		THEN `ALL_DISTINCT [5;6;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
		THEN FULL_SIMP_TAC std_ss[]
		THEN METIS_TAC[INTER_ASSOC,CDF_prob_DFT_event,DFT_PREIMAGE, PREIMAGE_EXTREAL_REAL])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq
THEN KNOW_TAC(`` PREIMAGE (CSP MA MB) {u | u <= Normal t} INTER p_space p =
                 DFT_event p (CSP MA MB) t``)
      THEN1(RW_TAC std_ss[GSYM DFT_PREIMAGE])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      
THEN `PREIMAGE (WSP P B_a B_d) {u | u <= Normal t} INTER p_space p =
      DFT_event p (WSP P B_a B_d) t` by RW_TAC std_ss[GSYM DFT_PREIMAGE]
THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq  
THEN KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t) =
       CDF p (\s. real (CS s)) t * prob p (DFT_event p (CSP MA MB) t)``)
     THEN1( RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
	    THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then  PREIMAGE (\s. real (CS s)) {u | u <=  t} INTER p_space p
            else
              DFT_event p (CSP MA MB)t ):num->'a->bool`, `5:num`,`3:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5; 3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN KNOW_TAC(`` (!i:num.
              i IN {5; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else DFT_event p (CSP MA MB) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
               i IN {5; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else DFT_event p (CSP MA MB) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      




THEN KNOW_TAC(`` prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t) =
       CDF p (\s. real (SS s)) t * prob p (DFT_event p (CSP MA MB) t)``)
     THEN1( RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
	    THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 6 then  PREIMAGE (\s. real (SS s)) {u | u <=  t} INTER p_space p
            else
              DFT_event p (CSP MA MB)t ):num->'a->bool`, `6:num`,`3:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6; 3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN KNOW_TAC(`` (!i:num.
              i IN {6; 3} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (CSP MA MB) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
               i IN {6; 3} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (CSP MA MB) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      




THEN KNOW_TAC(`` prob p
    (DFT_event p (CSP MA MB) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       prob p (DFT_event p (CSP MA MB) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t``)
     THEN1( RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
	    THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 3 then  DFT_event p (CSP MA MB)t
		    else if i = 0 then PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else  PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
 ):num->'a->bool`, `3:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_4_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN KNOW_TAC(`` (!i:num.
              i IN {3; 0; 1; 2} ==>
              (if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
              i IN {3; 0; 1; 2} ==>
              (if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      
THEN KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      DFT_event p (CSP MA MB) t) =
       CDF p (\s. real (CS s)) t *
       CDF p (\s. real (SS s)) t *
       prob p (DFT_event p (CSP MA MB) t) ``)
     THEN1( RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
	    THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then  PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else  DFT_event p (CSP MA MB)t ):num->'a->bool`, `5:num`,`6:num`, `3:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN KNOW_TAC(`` (!i:num.
              i IN {5; 6; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (CSP MA MB) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 6; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (CSP MA MB) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``  prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       CDF p (\s. real (CS s)) t *
       prob p (DFT_event p (CSP MA MB) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t ``)
     THEN1( RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
	    THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then  DFT_event p (CSP MA MB)t
		 else if i = 0 then PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p ):num->'a->bool`, `5:num`,`3:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN KNOW_TAC(`` (!i:num.
                i IN {5; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``  prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       CDF p (\s. real (SS s)) t *
       prob p (DFT_event p (CSP MA MB) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t ``)
     THEN1( RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
	    THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 6 then   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then  DFT_event p (CSP MA MB)t
		 else if i = 0 then PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p ):num->'a->bool`, `6:num`,`3:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN KNOW_TAC(`` (!i:num.
                i IN {6; 3; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {6; 3; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``  prob p (DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t) =

                  prob p (DFT_event p (CSP MA MB) t) * prob p (DFT_event p (WSP P B_a B_d) t)``)
     THEN1( MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 3 then  DFT_event p (CSP MA MB) t
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `3:num`,`4:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN  KNOW_TAC(`` (!i:num.
                 i IN {3; 4} ==>
              (if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {3; 4} ==>
              (if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``  prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t) =
       CDF p (\s. real (CS s)) t *
      prob p (DFT_event p (CSP MA MB) t) * prob p (DFT_event p (WSP P B_a B_d) t)``)
     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
		   else if i = 3 then  DFT_event p (CSP MA MB) t
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `5:num`,`3:num`,`4:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                 i IN {5; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``  prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t) =
       CDF p (\s. real (SS s)) t *
      prob p (DFT_event p (CSP MA MB) t) * prob p (DFT_event p (WSP P B_a B_d) t)``)
     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		   else if i = 3 then  DFT_event p (CSP MA MB) t
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `6:num`,`3:num`,`4:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                 i IN {6; 3; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {6; 3; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      





THEN KNOW_TAC(``  prob p
     (DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
        prob p (DFT_event p (CSP MA MB) t) * prob p (DFT_event p (WSP P B_a B_d) t) *
	CDF p (\s. real (PA s)) t *
	CDF p (\s. real (PB s)) t *
	CDF p (\s. real (PS s)) t ``)
     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 3 then  DFT_event p (CSP MA MB) t
		   else if i = 4 then DFT_event p (WSP P B_a B_d) t
		   else  if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `3:num`,`4:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;4; 0; 1; 2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                     i IN {3; 4; 0; 1; 2} ==>
              (if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {3; 4; 0; 1; 2} ==>
              (if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])


THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``  prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t)=
        CDF p (\s. real (CS s)) t *
	CDF p (\s. real (SS s)) t *
	prob p (DFT_event p (CSP MA MB) t) * prob p (DFT_event p (WSP P B_a B_d) t) ``)
     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then  PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                   else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		   else if i = 3 then  DFT_event p (CSP MA MB) t
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `5:num`,`6:num`,`3:num`, `4:num`]  prob_prod_any_4_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6; 3; 4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                    i IN {5; 6; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {5; 6; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p) ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``   prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      DFT_event p (CSP MA MB) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
        CDF p (\s. real (CS s)) t *
	CDF p (\s. real (SS s)) t *
	prob p (DFT_event p (CSP MA MB) t) *
	CDF p (\s. real (PA s)) t *
	CDF p (\s. real (PB s)) t *
	CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then  PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                   else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		   else if i = 3 then  DFT_event p (CSP MA MB) t
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                   else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                  else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`6:num`,`3:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6; 3; 0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {5; 6; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
              i IN {5; 6; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``   prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
        CDF p (\s. real (CS s)) t *
	CDF p (\s. real (SS s)) t *
	prob p (DFT_event p (WSP  P B_a B_d) t) *
	CDF p (\s. real (PA s)) t *
	CDF p (\s. real (PB s)) t *
	CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then  PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                   else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		   else if i = 4 then  DFT_event p (WSP P B_a B_d) t
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                   else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                  else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`6:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6; 4; 0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                 i IN {5; 6; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 6; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)   ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``    prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
        CDF p (\s. real (CS s)) t *
        prob p (DFT_event p (CSP MA MB)t) *
	prob p (DFT_event p (WSP  P B_a B_d) t) *
	CDF p (\s. real (PA s)) t *
	CDF p (\s. real (PB s)) t *
	CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then  PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                   else if i = 3 then DFT_event p (CSP MA MB) t 
		   else if i = 4 then  DFT_event p (WSP P B_a B_d) t
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                   else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                  else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`3:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3; 4; 0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               i IN {5; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``    prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
        CDF p (\s. real (SS s)) t *
        prob p (DFT_event p (CSP MA MB)t) *
	prob p (DFT_event p (WSP  P B_a B_d) t) *
	CDF p (\s. real (PA s)) t *
	CDF p (\s. real (PB s)) t *
	CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 6 then  PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                   else if i = 3 then DFT_event p (CSP MA MB) t 
		   else if i = 4 then  DFT_event p (WSP P B_a B_d) t
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                   else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                  else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `6:num`,`3:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3; 4; 0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               i IN {6; 3; 4; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {6; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])



THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``   prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      DFT_event p (CSP MA MB) t INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
       CDF p (\s. real (CS s)) t *
       CDF p (\s. real (SS s)) t *
        prob p (DFT_event p (CSP MA MB)t) *
	prob p (DFT_event p (WSP  P B_a B_d) t) *
	CDF p (\s. real (PA s)) t *
	CDF p (\s. real (PB s)) t *
	CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then  PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                   
		   else if i = 6 then  PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                   else if i = 3 then DFT_event p (CSP MA MB) t 
		   else if i = 4 then  DFT_event p (WSP P B_a B_d) t
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                   else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                  else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`6:num`,`3:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_7_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3; 4; 0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               i IN {5; 6; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 6; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC(``   prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (WSP P B_a B_d) t) =
       CDF p (\s. real (CS s)) t *
      
	prob p (DFT_event p (WSP  P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then  PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                   
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `5:num`,`4:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               i IN {5; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {5; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC(``   prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (WSP P B_a B_d) t) =
       CDF p (\s. real (SS s)) t *
      
	prob p (DFT_event p (WSP  P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 6 then  PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                   
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `6:num`,`4:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               i IN {6; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {6; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``  prob p
     (DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       prob p (DFT_event p (WSP  P B_a B_d) t) *
       CDF p (\s. real (PA s)) t  *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 4 then  DFT_event p (WSP P B_a B_d) t
		      else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p ):num->'a->bool`, `4:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_4_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               
              i IN {4; 0; 1; 2} ==>
              (if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {4; 0; 1; 2} ==>
              (if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``   prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      DFT_event p (WSP P B_a B_d) t) =
      CDF p (\s. real (CS s)) t  *
       CDF p (\s. real (SS s)) t * 
       prob p (DFT_event p (WSP  P B_a B_d) t) ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else  DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `5:num`,`6:num`, `4:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               
               i IN {5; 6; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})  ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 6; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN events p)  ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC(``    prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
      CDF p (\s. real (CS s)) t  *
       
       prob p (DFT_event p (WSP  P B_a B_d) t) *
       CDF p (\s. real (PA s)) t  *
       CDF p (\s. real (PB s)) t  *
       CDF p (\s. real (PS s)) t ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 4 then  DFT_event p (WSP P B_a B_d) t
		 else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               
               
              i IN {5; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})  ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p) ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``    prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
      CDF p (\s. real (SS s)) t  *
       
       prob p (DFT_event p (WSP  P B_a B_d) t) *
       CDF p (\s. real (PA s)) t  *
       CDF p (\s. real (PB s)) t  *
       CDF p (\s. real (PS s)) t ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else if i = 4 then  DFT_event p (WSP P B_a B_d) t
		 else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `6:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               
               
              i IN {6; 4; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})  ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {6; 4; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p) ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``    prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p)) =
      CDF p (\s. real (CS s)) t  *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else  PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p ):num->'a->bool`, `5:num`,`3:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
              i IN {5; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p) IN
              events p) ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``    prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p)) =
      CDF p (\s. real (SS s)) t  *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else  PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p ):num->'a->bool`, `6:num`,`3:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
              i IN {6; 3} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {6; 3} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p) IN
              events p) ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``    prob p
     (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
      p_space p INTER DFT_event p (WSP P B_a B_d) t)=
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 3 then  PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
		     else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `3:num`,`4:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                i IN {3; 4} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
          i IN {3; 4} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN events p) ``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``   prob p
     (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
      p_space p INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))=
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
        CDF p (\s. real (PA s)) t *
   CDF p (\s. real (PB s)) t * CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if i = 3 then  PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
		     else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `3:num`,`0:num`,`1:num`,`2:num`]  prob_prod_any_4_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                i IN {3; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
         i IN {3; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``    prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p)) =
       CDF p (\s. real (CS s)) t *
      CDF p (\s. real (SS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if  i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p):num->'a->bool`, `5:num`,`6:num`,`3:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
               i IN {5; 6; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
          i IN {5; 6; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``     prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (WSP P B_a B_d) t) =
       CDF p (\s. real (CS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if  i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                  else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `5:num`,`3:num`,`4:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {5; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
          i IN {5; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``     prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (WSP P B_a B_d) t) =
       CDF p (\s. real (SS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if  i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                  else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
		   else DFT_event p (WSP P B_a B_d) t):num->'a->bool`, `6:num`,`3:num`,`4:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {6; 3; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
          i IN {6; 3; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       CDF p (\s. real (CS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if  i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                  else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`3:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {5; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])


THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       CDF p (\s. real (SS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num. if  i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                  else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `6:num`,`3:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {6; 3; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {6; 3; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN  KNOW_TAC(``prob p
     (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
      p_space p INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
 		   else if i = 4 then DFT_event p (WSP P B_a B_d) t  
 		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `3:num`,`4:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {3; 4; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {3; 4; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN  KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (WSP P B_a B_d) t) =
       CDF p (\s. real (CS s)) t *
       CDF p (\s. real (SS s)) t *
       
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t) ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
 		   else  DFT_event p (WSP P B_a B_d) t ):num->'a->bool`, `5:num`,`6:num`,`3:num`, `4:num`]  prob_prod_any_4_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {5; 6; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 6; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN  KNOW_TAC(``prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))  =
       CDF p (\s. real (CS s)) t *
       CDF p (\s. real (SS s)) t *
       
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
 		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`6:num`,`3:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                   i IN {5; 6; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 6; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN  KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       CDF p (\s. real (CS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t  
                 else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
		
		 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`3:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {5; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
          i IN {5; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      




THEN  KNOW_TAC(`` prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       CDF p (\s. real (SS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t  
                 else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
		
		 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `6:num`,`3:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                  i IN {6; 3; 4; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
          i IN {6; 3; 4; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
       CDF p (\s. real (CS s)) t * CDF p (\s. real (SS s)) t *
       prob p (PREIMAGE (\s. (MS s,MA s)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else  if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t  
                 else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
		
		 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p):num->'a->bool`, `5:num`,`6:num`,`3:num`,`4:num`, `0:num`, `1:num`, `2:num`]  prob_prod_any_7_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                 i IN {5; 6; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 6; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(RW_TAC std_ss [])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      





THEN KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) =
     CDF p (\s. real (CS s)) t *
     prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t):num->'a->bool`, `5:num`,`3:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                   i IN {5; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t) IN events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC(`` prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) =
     CDF p (\s. real (SS s)) t *
     prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t):num->'a->bool`, `6:num`,`3:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                   i IN {6; 3} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {6; 3} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t) IN events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      




THEN KNOW_TAC(``prob p
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p INTER DFT_event p (CSP MA MB) t INTER
       DFT_event p (WSP P B_a B_d) t)  =
     prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
		    else DFT_event p (WSP P B_a B_d ) t):num->'a->bool`, `3:num`,`4:num`]  prob_prod_any_2_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                    i IN {3; 4} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {3; 4} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC(``prob p
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p INTER DFT_event p (CSP MA MB) t INTER
       (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
        (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
        (PREIMAGE PS {u | u <= Normal t} INTER p_space p)))  =
     prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
      CDF p (\s. real (PA s)) t *
      CDF p (\s. real (PB s)) t *
      CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
		    else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                ):num->'a->bool`, `3:num`,`0:num`,`1:num`,`2:num`]  prob_prod_any_4_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                    i IN {3; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
               i IN {3; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(`` prob p
      (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
        p_space p) INTER DFT_event p (CSP MA MB) t)  =
	CDF p (\s. real (CS s)) t *
      CDF p (\s. real (SS s)) t *
     prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.   if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
		     ):num->'a->bool`, `5:num`,`6:num`,`3:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                    i IN {5; 6; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {5; 6; 3} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t) IN events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(`` prob p
      (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
        p_space p) INTER DFT_event p (CSP MA MB) t INTER
       DFT_event p (WSP P B_a B_d) t)  =
	CDF p (\s. real (CS s)) t *
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.   if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
		   else DFT_event p (WSP P B_a B_d) t
		     ):num->'a->bool`, `5:num`,`3:num`,`4:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                   i IN {5; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(`` prob p
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
        p_space p) INTER DFT_event p (CSP MA MB) t INTER
       DFT_event p (WSP P B_a B_d) t)  =
	CDF p (\s. real (SS s)) t *
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t)``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.   if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
		   else DFT_event p (WSP P B_a B_d) t
		     ):num->'a->bool`, `6:num`,`3:num`,`4:num`]  prob_prod_any_3_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                   i IN {6; 3; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p}) ``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {6; 3; 4} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(`` prob p
      (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
        p_space p) INTER DFT_event p (CSP MA MB) t INTER
       (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
        (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
        (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
	CDF p (\s. real (CS s)) t *
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.   if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     
                   DFT_event p (CSP MA MB) t

		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                
               	     ):num->'a->bool`, `5:num`,`3:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                      i IN {5; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
              i IN {5; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      




THEN KNOW_TAC(`` prob p
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
        p_space p) INTER DFT_event p (CSP MA MB) t INTER
       (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
        (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
        (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
	CDF p (\s. real (SS s)) t *
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.   if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     
                   DFT_event p (CSP MA MB) t

		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                
               	     ):num->'a->bool`, `6:num`,`3:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                      i IN {6; 3; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
              i IN {6; 3; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      

THEN KNOW_TAC(``  prob p
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p INTER DFT_event p (CSP MA MB) t INTER
       DFT_event p (WSP P B_a B_d) t INTER
       (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
        (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
        (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
	prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
       CDF p (\s. real (PA s)) t *
       CDF p (\s. real (PB s)) t *
       CDF p (\s. real (PS s)) t``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.   if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     DFT_event p (CSP MA MB) t
		   else if i = 4 then DFT_event p (WSP P B_a B_d) t 
		   else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else 
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                
               	     ):num->'a->bool`, `3:num`,`4:num`,`0:num`, `1:num`, `2:num`]  prob_prod_any_5_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                     i IN {3; 4; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {3; 4; 0; 1; 2} ==>
              (if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p) ``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``  prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t INTER
      DFT_event p (WSP P B_a B_d) t) =
      CDF p (\s. real (CS s)) t *
       CDF p (\s. real (SS s)) t *
      
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t) ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     DFT_event p (CSP MA MB) t
		   else DFT_event p (WSP P B_a B_d) t 
		   ):num->'a->bool`, `5:num`,`6:num`,`3:num`, `4:num`]  prob_prod_any_4_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3;4} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;4:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                    i IN {5; 6; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 6; 3; 4} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else DFT_event p (WSP P B_a B_d) t) IN events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(``  prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
      CDF p (\s. real (CS s)) t *
       CDF p (\s. real (SS s)) t *
      
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
         CDF p (\s. real (PA s)) t *
      CDF p (\s. real (PB s)) t *
      CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else if i = 6 then PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
		 else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     DFT_event p (CSP MA MB) t
		  else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else  PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 ):num->'a->bool`, `5:num`,`6:num`,`3:num`, `0:num`,`1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                            i IN {5; 6; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
           i IN {5; 6; 3; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      


THEN KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t INTER
      DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
      CDF p (\s. real (CS s)) t *
    
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
         CDF p (\s. real (PA s)) t *
      CDF p (\s. real (PB s)) t *
      CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                  else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     DFT_event p (CSP MA MB) t
		     else if i = 4 then DFT_event p (WSP P B_a B_d) t
		     else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else  PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 ):num->'a->bool`, `5:num`,`3:num`,`4:num`, `0:num`,`1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                              i IN {5; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {5; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC(`` prob p
     (PREIMAGE SS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t INTER
      DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
      CDF p (\s. real (SS s)) t *
    
      prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
         CDF p (\s. real (PA s)) t *
      CDF p (\s. real (PB s)) t *
      CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                  else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     DFT_event p (CSP MA MB) t
		     else if i = 4 then DFT_event p (WSP P B_a B_d) t
		     else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else  PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 ):num->'a->bool`, `6:num`,`3:num`,`4:num`, `0:num`,`1:num`, `2:num`]  prob_prod_any_6_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{6;3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [6;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
                              i IN {6; 3; 4; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
            i IN {6; 3; 4; 0; 1; 2} ==>
              (if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC(`` prob p
     (PREIMAGE CS {u | u <= Normal t} INTER p_space p INTER
      (PREIMAGE SS {u | u <= Normal t} INTER p_space p) INTER
      (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t INTER
      DFT_event p (WSP P B_a B_d) t INTER
      (PREIMAGE PA {u | u <= Normal t} INTER p_space p INTER
       (PREIMAGE PB {u | u <= Normal t} INTER p_space p) INTER
       (PREIMAGE PS {u | u <= Normal t} INTER p_space p))) =
        CDF p (\s. real (CS s)) t *
        CDF p (\s. real (SS s)) t *
       prob p ((PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
       p_space p) INTER DFT_event p (CSP MA MB) t) *
       prob p (DFT_event p (WSP P B_a B_d) t) *
         CDF p (\s. real (PA s)) t *
      CDF p (\s. real (PB s)) t *
      CDF p (\s. real (PS s)) t  ``)

     THEN1(RW_TAC std_ss[PREIMAGE_EXTREAL_REAL, PREIMAGE_EXTREAL_REAL_2RV]
           THEN MP_TAC(Q.ISPECL [`p:('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)`,  ` (\i:num.
                {if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else if i = 2 then
                   PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 else if i = 3 then
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                   DFT_event p (CSP MA MB) t
                 else if i = 4 then DFT_event p (WSP P B_a B_d) t
                 else if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                 else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})`, `{0:num; 1; 2; 3; 4; 5;6}`,`(\i:num.  if i = 5 then
                   PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
                    else if i = 6 then
                   PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
                  else if i = 3 then 
                   PREIMAGE (\s. (real (MS s),real (MA s)))
                     {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
		     DFT_event p (CSP MA MB) t
		     else if i = 4 then DFT_event p (WSP P B_a B_d) t
		     else if i = 0 then
                   PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
                 else if i = 1 then
                   PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
                 else  PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
                 ):num->'a->bool`, `5:num`,`6:num`,`3:num`,`4:num`, `0:num`,`1:num`, `2:num`]  prob_prod_any_7_of_10_set)
	
          THEN RW_TAC std_ss[]
          THEN `{5;6;3;4;0;1;2} SUBSET {0; 1; 2; 3; 4; 5;6:num}` by ASM_SET_TAC[]
	  THEN `ALL_DISTINCT [5;6;3;4;0;1;2:num]` by RW_TAC std_ss[ALL_DISTINCT, MEM]
	  THEN  KNOW_TAC(`` (!i:num.
            i IN {5; 6; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              {if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else if i = 2 then
                 PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p})``)
               THEN1( RW_TAC real_ss[ALL_DISTINCT, MEM ] 
 	            THEN PSET_TAC[GSPECIFICATION, EXTENSION,MEM, ALL_DISTINCT]
		    THEN RW_TAC real_ss[])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(`` (!i:num.
             i IN {5; 6; 3; 4; 0; 1; 2} ==>
              (if i = 5 then
                 PREIMAGE (\s. real (CS s)) {u | u <= t} INTER p_space p
               else if i = 6 then
                 PREIMAGE (\s. real (SS s)) {u | u <= t} INTER p_space p
               else if i = 3 then
                 PREIMAGE (\s. (real (MS s),real (MA s)))
                   {(u,w) | u < w /\ 0 <= w /\ w <= t} INTER p_space p INTER
                 DFT_event p (CSP MA MB) t
               else if i = 4 then DFT_event p (WSP P B_a B_d) t
               else if i = 0 then
                 PREIMAGE (\s. real (PA s)) {u | u <= t} INTER p_space p
               else if i = 1 then
                 PREIMAGE (\s. real (PB s)) {u | u <= t} INTER p_space p
               else PREIMAGE (\s. real (PS s)) {u | u <= t} INTER p_space p) IN
              events p)``)
	       THEN1(METIS_TAC [EVENTS_INTER])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
	 THEN  RW_TAC std_ss[]
	 THEN  METIS_TAC[GSYM DFT_PREIMAGE, CDF_prob_DFT_event, PREIMAGE_EXTREAL_REAL, INTER_ASSOC])

THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      



THEN KNOW_TAC (``indep_var p lborel (\s. real (MS s)) lborel (\s. real ( MA s))``)
     THEN1(MP_TAC(Q.ISPECL [`(\i:num.
                if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real ((PB:'a->extreal) s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `lborel`, `3:num`, `4:num`, `{0;1;2;3;4;5;6;7;8:num}` ]indep_vars_var )
            THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``prob p
   (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0<=w/\w <= Normal t} INTER
    p_space p) =  pos_fn_integral lborel
        (\y. f_MA y * (indicator_fn {u | 0<=u/\u <= t} y * CDF p (\s. real (MS s)) y))``)
THEN1( RW_TAC std_ss [PREIMAGE_EXTREAL_REAL_2RV]
       THEN FULL_SIMP_TAC std_ss[cond_density_def]
       THEN `!y. 0<= f_MA y` by METIS_TAC [den_gt0_ninfinity_def, extreal_of_num_def, lt_le, le_lt]
       THEN METIS_TAC[prob_after])

THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` prob p (DFT_event p (WSP P B_a B_d ) t) =
       pos_fn_integral lborel
          (\y.
               indicator_fn {u | 0 <= u /\ u <= t} y * f_P y *
               pos_fn_integral lborel
                 (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_P_B y x)) +
        pos_fn_integral lborel
          (\y.
               f_P y *
               (indicator_fn {u | 0 <= u /\ u <= t} y *
                CDF p (\s. real (B_d s)) y))``)
     THEN1(`!s. ALL_DISTINCT [B_a s; B_d s; P s]` by (FULL_SIMP_TAC std_ss[ALL_DISTINCT, MEM] THEN RW_TAC std_ss[ALL_DISTINCT, MEM])
           THEN KNOW_TAC (``indep_var p lborel (\s. real (B_d s)) lborel (\s. real ( P s))``)
    	   	 THEN1(MP_TAC(Q.ISPECL [`(\i:num.
                if i = 0 then (\s. real (PA s))
                else if i = 1 then (\s. real ((PB:'a->extreal) s))
                else if i = 2 then (\s. real (PS s))
                else if i = 3 then (\s. real (MS s))
                else if i = 4 then (\s. real (MA s))
                else if i = 5 then (\s. real (CS s))
                else if i = 6 then (\s. real (SS s))
                else if i = 7 then (\s. real (P s))
                else (\s. real (B_d s)))`, `p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `lborel`, `8:num`, `7:num`, `{0;1;2;3;4;5;6;7;8:num}` ]indep_vars_var )
                THEN ASM_SET_TAC[])
	  THEN DISCH_TAC
	  THEN `rv_gt0_ninfinity [B_a; B_d; P]` by (FULL_SIMP_TAC std_ss[rv_gt0_ninfinity_def] )
	  THEN METIS_TAC[prob_WSP])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      
THEN KNOW_TAC(``prob p (DFT_event p (CSP MA MB) t ) =
                 pos_fn_integral lborel
          (\y.
               indicator_fn {u | 0 <= u /\ u <= t} y * f_MA y *
               pos_fn_integral lborel
                 (\x. indicator_fn {w | y < w /\ w <= t} x * f_cond_MA_MB y x)) ``)
     THEN1(`rv_gt0_ninfinity[MB; MA]` by FULL_SIMP_TAC std_ss[rv_gt0_ninfinity_def]
     	   THEN `!y. 0 < f_MA y /\ f_MA y<> PosInf` by FULL_SIMP_TAC std_ss[den_gt0_ninfinity_def]
	   THEN METIS_TAC [prob_CSP])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[] THEN WEAKEN_TAC is_eq      
THEN KNOW_TAC(`` (PREIMAGE (\x. (MS x,MA x)) {(u,w) | u < w /\ 0 <= w /\ w <= Normal t} INTER
      p_space p INTER DFT_event p (CSP MA MB) t) =
      (DFT_event p (D_AND (D_AND MB (D_BEFORE MS MA)) (D_BEFORE MA MB)) t)      ``)

    THEN1(`rv_gt0_ninfinity[MS; MA]` by FULL_SIMP_TAC std_ss[rv_gt0_ninfinity_def]
          THEN RW_TAC std_ss[GSYM DFT_event_AFTER_PREIMAGE_GT0,  PREIMAGE_EXTREAL_REAL_2RV]
	  THEN RW_TAC std_ss[GSYM AND_INTER]
	  THEN REWRITE_TAC[CSP_def2]
	  THEN KNOW_TAC(`` (D_AND (D_AND MA (D_BEFORE MS MA)) (D_AND MB (D_BEFORE MA MB)))=
	                    (D_AND (D_AND MB (D_BEFORE MS MA)) (D_BEFORE MA MB))``)
	       THEN1(METIS_TAC[LEFT_AND_OVER_BEFORE_ABSORB, D_AND_COMM, D_AND_ASSOC])
	  THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
	  THEN RW_TAC std_ss[])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``  prob p
     (DFT_event p (D_AND (D_AND MB (D_BEFORE MS MA)) (D_BEFORE MA MB)) t) =
       pos_fn_integral lborel
             (\y.
                  f_MA y * CDF p (\s. real (MS s)) y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
                  pos_fn_integral lborel
                    (\x.
                         f_cond_MA_MB y x *
                         indicator_fn {x' | y < x' /\ x' <= t} x )) ``)
     THEN1(`rv_gt0_ninfinity[MB; MA; MS]` by FULL_SIMP_TAC std_ss[rv_gt0_ninfinity_def]			       THEN `!s. ALL_DISTINCT[MB s; MA s ; MS s]` by FULL_SIMP_TAC std_ss[ALL_DISTINCT, MEM]
           THEN METIS_TAC[prob_event_PAND_CSP_INTER])
THEN DISCH_TAC THEN RW_TAC std_ss[]);

val _ = export_theory();
