
(******************************************************************************)
(*                       Probabilistic Analysis of                            *)
(*                        Spare gates with shared events                      *)
(*                                  by                                        *)
(*                          Yassmeen Elderhalli                               *)
(*                      y_elderh@encs.concordia.ca                            *)
(******************************************************************************)

(*loadPath := "/home/yassmeen/Downloads/ProbTheories_HOL-k12" :: !loadPath;
(*set_trace "Unicode" 0;*)

app load ["HolKernel", "Parse", "boolLib", "bossLib", "numLib", "unwindLib", 
"tautLib", "Arith", "prim_recTheory", "combinTheory", "quotientTheory", 
"arithmeticTheory", "realTheory", "hrealTheory", "realaxTheory", "realLib",  
"jrhUtils", "pairTheory", "seqTheory", "limTheory", "transcTheory", "listTheory", 
"mesonLib", "boolTheory", "topologyTheory", "pred_setTheory", "util_probTheory", 
"optionTheory", "numTheory", "sumTheory", "InductiveDefinition", "ind_typeTheory",
"pred_setLib", "iterate_hvgTheory", "card_hvgTheory", "product_hvgTheory", 
"topology_hvgTheory", "derivative_hvgTheory", "integration_hvgTheory",
"real_sigmaTheory", "extreal_hvgTheory", "measure_hvgTheory", "lebesgue_hvgTheory",
"probability_hvgTheory", "lebesgue_measure_hvgTheory", "normal_rv_hvgTheory", "netsTheory", "PIE_EXTREALTheory","dep_rewrite", "integration_before_afterTheory", "DFT_Gates_def_PROBTheory", "CASTheory","WCSPTheory"];
*)
open HolKernel Parse boolLib bossLib numLib unwindLib tautLib Arith prim_recTheory 
combinTheory quotientTheory arithmeticTheory hrealTheory realaxTheory realTheory 
realLib jrhUtils pairTheory seqTheory limTheory transcTheory listTheory mesonLib 
boolTheory topologyTheory pred_setTheory util_probTheory optionTheory numTheory 
sumTheory InductiveDefinition ind_typeTheory pred_setLib iterate_hvgTheory 
card_hvgTheory product_hvgTheory topology_hvgTheory derivative_hvgTheory
integration_hvgTheory real_sigmaTheory extreal_hvgTheory measure_hvgTheory
lebesgue_hvgTheory probability_hvgTheory lebesgue_measure_hvgTheory normal_rv_hvgTheory netsTheory PIE_EXTREALTheory dep_rewrite integration_before_afterTheory DFT_Gates_def_PROBTheory CASTheory WCSPTheory;


val _ = new_theory "WCSP_shared";


infixr 0 ++ << || ORELSEC ## --> THENC;
infix 1 >> |->;
fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
 
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Suff = PARSE_TAC SUFF_TAC;
val POP_ORW = POP_ASSUM (fn thm => ONCE_REWRITE_TAC [thm]);
val !! = REPEAT;
val op++ = op THEN;
val op<< = op THENL;
val op|| = op ORELSE;
val op>> = op THEN1;
val std_ss' = simpLib.++ (std_ss, boolSimps.ETA_ss);

(*---------------------------*)
fun SET_TAC L =
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION,
    IN_INTER, IN_DIFF, IN_INSERT, IN_DELETE, IN_BIGINTER, IN_BIGUNION,
    IN_IMAGE, GSPECIFICATION, IN_DEF] THEN METIS_TAC [];

fun SET_RULE tm = prove(tm,SET_TAC []);


val set_rewrs
= [INTER_EMPTY, INTER_UNIV, UNION_EMPTY, UNION_UNIV, DISJOINT_UNION,
DISJOINT_INSERT, DISJOINT_EMPTY, GSYM DISJOINT_EMPTY_REFL,
DISJOINT_BIGUNION, INTER_SUBSET, INTER_IDEMPOT, UNION_IDEMPOT,
SUBSET_UNION];

val UNIONL_def = Define `(UNIONL [] = {})
/\ (UNIONL (s::ss) = (s:'a ->bool) UNION UNIONL ss)`;


val IN_UNIONL = store_thm
("IN_UNIONL",
``!l (v:'a ). v IN UNIONL l = (?s. MEM s l /\ v IN s)``,
Induct >> RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY]
++ RW_TAC std_ss [UNIONL_def, MEM, NOT_IN_EMPTY, IN_UNION]
++ PROVE_TAC []);


val elt_rewrs
= [SUBSET_DEF, IN_COMPL, IN_DIFF, IN_UNIV, NOT_IN_EMPTY, IN_UNION,
IN_INTER, IN_IMAGE, IN_FUNSET, IN_DFUNSET, GSPECIFICATION,
DISJOINT_DEF, IN_BIGUNION, IN_BIGINTER, IN_COUNT, IN_o,
IN_UNIONL, IN_DELETE, IN_PREIMAGE, IN_SING, IN_INSERT];


fun rewr_ss ths =
simpLib.++
(std_ss,
simpLib.SSFRAG
{ac = [],
name = NONE,
convs = [],
dprocs = [],
filter = NONE,
rewrs = set_rewrs @ elt_rewrs,
congs = []});
val pset_set_ss = rewr_ss set_rewrs;
val pset_elt_ss = rewr_ss elt_rewrs;
val pset_ss = rewr_ss (set_rewrs @ elt_rewrs);


fun PSET_TAC ths =
REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_set_ss ths
++ REPEAT (POP_ASSUM MP_TAC)
++ RW_TAC pset_elt_ss ths;

fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;
val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Simplify = RW_TAC arith_ss;
val Suff = PARSE_TAC SUFF_TAC;
val Know = PARSE_TAC KNOW_TAC;
fun wrap a = [a];
val Rewr = DISCH_THEN (REWRITE_TAC o wrap);
val Rewr' = DISCH_THEN (ONCE_REWRITE_TAC o wrap);
val Cond =
  DISCH_THEN
  (fn mp_th =>
   let
     val cond = fst (dest_imp (concl mp_th))
   in
     KNOW_TAC cond << [ALL_TAC, DISCH_THEN (MP_TAC o MP mp_th)]
   end);
val FUN_EQ_TAC = CONV_TAC (CHANGED_CONV (ONCE_DEPTH_CONV FUN_EQ_CONV));
fun K_TAC _ = ALL_TAC;
fun MESON ths tm = prove(tm,MESON_TAC ths);
fun METIS ths tm = prove(tm,METIS_TAC ths);
val DISC_RW_KILL = DISCH_TAC THEN ONCE_ASM_REWRITE_TAC [] THEN 
                   POP_ASSUM K_TAC;
			   
val IN_REST = store_thm ("IN_REST",
 ``!x:'a. !s. x IN (REST s) <=> x IN s /\ ~(x = CHOICE s)``,
  REWRITE_TAC[REST_DEF, IN_DELETE]);
fun SET_TAC L = 
    POP_ASSUM_LIST(K ALL_TAC) THEN REPEAT COND_CASES_TAC THEN
    REWRITE_TAC (append [EXTENSION, SUBSET_DEF, PSUBSET_DEF, DISJOINT_DEF,
    SING_DEF] L) THEN
    SIMP_TAC std_ss [NOT_IN_EMPTY, IN_UNIV, IN_UNION, IN_INTER, IN_DIFF, 
      IN_INSERT, IN_DELETE, IN_REST, IN_BIGINTER, IN_BIGUNION, IN_IMAGE, 
      GSPECIFICATION, IN_DEF, EXISTS_PROD] THEN METIS_TAC [];
fun ASSERT_TAC tm = SUBGOAL_THEN tm STRIP_ASSUME_TAC;
fun SET_RULE tm = prove(tm,SET_TAC []);
fun ASM_SET_TAC L = REPEAT (POP_ASSUM MP_TAC) THEN SET_TAC L;
val ASM_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN ARITH_TAC;
val ASM_REAL_ARITH_TAC = REPEAT (POP_ASSUM MP_TAC) THEN REAL_ARITH_TAC;

fun DFT_rewrs L =  REPEAT( METIS_TAC L);


val rewrs = [REAL_LE_REFL, REAL_LE_LT, REAL_LE_ANTISYM, REAL_LET_ANTISYM, REAL_LET_TOTAL, REAL_LET_TRANS, real_of_num,REAL_LT_NEG1_1,REAL_LT_NEG1_0,REAL_LE_NEG1_0,REAL_LE_NEG1_1, REAL_LT_TOTAL, max_def, min_def, REAL_LT_TRANS, REAL_LE_TRANS, REAL_MIN_COMM, REAL_MAX_COMM];

val DFT_rewrs_ss  =DFT_rewrs rewrs;



val indep_varp_def= Define `indep_varp p M1 XY M2 Z  =
               random_variable XY p (m_space M1,measurable_sets M1) /\
          random_variable Z p (m_space M2,measurable_sets M2) /\
          indep_sets p
           (\i.
                 {if i:num = 0 then PREIMAGE XY A INTER p_space p
                 else PREIMAGE Z B INTER p_space p |
                 if i = 0 then A IN measurable_sets M1
                 else B IN measurable_sets M2}) {0; 1}`;





val indep_varp_distribution_imp = store_thm ("indep_varp_distribution_imp",
  ``!p M1 XY M2 Z. prob_space p /\ measure_space M1 /\ measure_space M2 /\
                  sigma_finite_measure M2 /\ indep_varp p M1 XY M2 Z ==> 
      random_variable XY p (m_space M1, measurable_sets M1) /\ 
      random_variable Z p (m_space M2, measurable_sets M2) /\
      (measure_of (pair_measure (distr p M1 XY) (distr p M2 Z)) =
       measure_of (distr p (pair_measure M1 M2) (\x. (XY  x, Z x))))``,
REPEAT GEN_TAC THEN STRIP_TAC
THEN  RW_TAC std_ss []
      THEN1(FULL_SIMP_TAC std_ss [indep_varp_def])
      THEN1(FULL_SIMP_TAC std_ss [indep_varp_def])
THEN  
  `indep_varp p M1 XY M2 Z ==>
      random_variable (\x. (XY x, Z x)) p 
      (m_space (pair_measure M1 M2), measurable_sets (pair_measure M1 M2))` by
   (FULL_SIMP_TAC std_ss [indep_varp_def] THEN
    FULL_SIMP_TAC std_ss [random_variable_def]  THEN
    SIMP_TAC std_ss [p_space_def, events_def] THEN MATCH_MP_TAC measurable_Pair THEN
    FULL_SIMP_TAC std_ss [prob_space_def, p_space_def, events_def]) THEN
  `indep_varp p M1 XY M2 Z ==> prob_space (distr p M1 XY)` by
   (FULL_SIMP_TAC std_ss [indep_varp_def, random_variable_def] THEN 
   METIS_TAC [prob_space_distr, random_variable_def, p_space_def, events_def]) THEN
  `indep_varp p M1 XY M2 Z ==> prob_space (distr p M2 Z)` by
  (FULL_SIMP_TAC std_ss [indep_varp_def, random_variable_def] THEN 
   METIS_TAC [prob_space_distr, random_variable_def, p_space_def, events_def]) THEN
  Q_TAC SUFF_TAC `indep_varp p M1 XY M2 Z ==>
                  (measure_of (pair_measure (distr p M1 XY) (distr p M2 Z)) =
                   measure_of (distr p (pair_measure M1 M2) (\x. XY x,Z x)))` THEN1(
  DISCH_TAC THEN METIS_TAC[])
  THEN 
   DISCH_TAC THEN MATCH_MP_TAC pair_measure_eqI THEN
   Q_TAC SUFF_TAC `sigma_finite_measure (distr p M1 XY)` THENL
   [DISCH_TAC THEN ASM_SIMP_TAC std_ss [],
    RW_TAC std_ss [sigma_finite_measure] THEN Q.EXISTS_TAC `{m_space M1}` THEN
    SIMP_TAC std_ss [countable_sing] THEN
    CONJ_TAC THENL 
    [RW_TAC std_ss [SUBSET_DEF, distr, measurable_sets_def, IN_SING] THEN
     ASM_SIMP_TAC std_ss [MEASURE_SPACE_MSPACE_MEASURABLE], ALL_TAC] THEN
    CONJ_TAC THENL [SIMP_TAC std_ss [BIGUNION_SING, distr, m_space_def], ALL_TAC] THEN
    RW_TAC std_ss [IN_SING] THEN FULL_SIMP_TAC std_ss [prob_space_def] THEN 
    `m_space M1 = p_space (distr p M1 XY)` by METIS_TAC [p_space_def, m_space_def, distr] THEN
    ASM_SIMP_TAC std_ss [num_not_infty]] THEN
   Q_TAC SUFF_TAC `sigma_finite_measure (distr p M2 Z)` THENL
   [DISCH_TAC THEN ASM_SIMP_TAC std_ss [],
    RW_TAC std_ss [sigma_finite_measure] THEN Q.EXISTS_TAC `{m_space M2}` THEN
    SIMP_TAC std_ss [countable_sing] THEN
    CONJ_TAC THENL 
    [RW_TAC std_ss [SUBSET_DEF, distr, measurable_sets_def, IN_SING] THEN
     ASM_SIMP_TAC std_ss [MEASURE_SPACE_MSPACE_MEASURABLE], ALL_TAC] THEN
    CONJ_TAC THENL [SIMP_TAC std_ss [BIGUNION_SING, distr, m_space_def], ALL_TAC] THEN
    RW_TAC std_ss [IN_SING] THEN FULL_SIMP_TAC std_ss [prob_space_def] THEN 
    `m_space M2 = p_space (distr p M2 Z)` by METIS_TAC [p_space_def, m_space_def, distr] THEN
    ASM_SIMP_TAC std_ss [num_not_infty]] THEN
   CONJ_TAC THENL
   [MATCH_MP_TAC measure_space_distr THEN
    FULL_SIMP_TAC std_ss [prob_space_def, measure_space_pair_measure'] THEN
    FULL_SIMP_TAC std_ss [random_variable_def, p_space_def, events_def],
    ALL_TAC] THEN
   CONJ_TAC THENL
   [MATCH_MP_TAC measure_space_distr THEN FULL_SIMP_TAC std_ss [prob_space_def] THEN
    FULL_SIMP_TAC std_ss [random_variable_def, p_space_def, events_def, indep_varp_def] ,
    ALL_TAC] THEN
   CONJ_TAC THENL
   [MATCH_MP_TAC measure_space_distr THEN FULL_SIMP_TAC std_ss [prob_space_def] THEN
    FULL_SIMP_TAC std_ss [random_variable_def, p_space_def, events_def, indep_varp_def],
    ALL_TAC] THEN
   CONJ_TAC THENL
   [Q_TAC SUFF_TAC `{a CROSS b | a IN measurable_sets M1 /\ b IN measurable_sets M2} 
                     SUBSET POW (m_space M1 CROSS m_space M2)` THENL
    [DISCH_TAC THEN ASM_SIMP_TAC std_ss [],
SIMP_TAC std_ss [CROSS_DEF, SUBSET_DEF, POW_DEF, GSPECIFICATION, EXISTS_PROD] THEN
     RW_TAC std_ss [] THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXISTS_PROD] THEN
     `p_1 SUBSET m_space M1` by METIS_TAC [MEASURABLE_SETS_SUBSET_SPACE] THEN
     `p_2 SUBSET m_space M2` by METIS_TAC [MEASURABLE_SETS_SUBSET_SPACE] THEN
     ASM_SET_TAC []] THEN
    FULL_SIMP_TAC std_ss [pair_measure, measure_of, measurable_sets_def, distr, m_space_def],
    ALL_TAC] THEN
   RW_TAC std_ss [distr, measurable_sets_def] THEN SIMP_TAC std_ss [GSYM distr] THEN
   ONCE_REWRITE_TAC [EQ_SYM_EQ] THEN SIMP_TAC std_ss [distr, measure_def] THEN
   SIMP_TAC std_ss [GSYM prob_def, GSYM p_space_def] THEN
   `PREIMAGE (\x. (XY x,Z x)) (A CROSS B) INTER p_space p = PREIMAGE (XY:'a->'b) A INTER p_space p INTER PREIMAGE (Z:'a->'c) B INTER p_space p` by (SRW_TAC[] [GSPECIFICATION, EXTENSION, PREIMAGE_def] THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM] )
 THEN ASM_REWRITE_TAC[]
 THEN FULL_SIMP_TAC std_ss [indep_varp_def]
 THEN KNOW_TAC(``PREIMAGE (XY:'a->'b) A INTER p_space p INTER PREIMAGE (Z:'a->'c) B INTER p_space p = ((\i:num.
      if i: num = 0 then PREIMAGE XY A INTER p_space p
      else PREIMAGE Z B INTER p_space p) 0 INTER (\i:num.
      if i: num = 0 then PREIMAGE XY A INTER p_space p
      else PREIMAGE Z B INTER p_space p) 1 )``)
    THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC[CONJ_ASSOC, CONJ_COMM])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``prob p (PREIMAGE (XY:'a->'b) A INTER p_space p) *
prob p (PREIMAGE (Z:'a->'c) B INTER p_space p)
 =  prob p ((\i:num.
      if i:num = 0 then PREIMAGE XY A INTER p_space p
      else PREIMAGE Z B INTER p_space p) 0) * prob p( (\i:num.
      if i:num = 0 then PREIMAGE XY A INTER p_space p
      else PREIMAGE Z B INTER p_space p) 1)``)
    THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC[CONJ_ASSOC, CONJ_COMM])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN MATCH_MP_TAC prob_prod_any_2_of_10_set 
THEN Q.EXISTS_TAC` (\i.
          {if i:num = 0 then PREIMAGE XY A INTER p_space p
           else PREIMAGE Z B INTER p_space p |
           if i:num = 0 then A IN measurable_sets M1
           else B IN measurable_sets M2})`
THEN Q.EXISTS_TAC`{0;1}`
THEN RW_TAC std_ss[]
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def])
     THEN1(ASM_SET_TAC[])
     THEN1(ASM_SET_TAC[])
     THEN1(METIS_TAC [IN_MEASURABLE, p_space_def, events_def, random_variable_def, subsets_def, space_def])
THEN METIS_TAC [IN_MEASURABLE, p_space_def, events_def, random_variable_def, subsets_def, space_def]);


val indep_varp_pair_distr = store_thm("indep_varp_pair_distr",
  `` !p M1 XY M2 Z A.
     prob_space p /\ measure_space M1 /\ measure_space M2 /\
     sigma_finite_measure M2 /\ indep_varp p M1 XY M2 Z  /\
     A IN measurable_sets (pair_measure M1 M2)==>
      (pos_fn_integral (distr p (pair_measure M1 M2 ) (\x. (XY x, Z x))) (indicator_fn A) =
      pos_fn_integral (pair_measure (distr p M1 XY) (distr p M2 Z)) (indicator_fn A))``,
RW_TAC std_ss[]
THEN KNOW_TAC(``XY IN measurable (m_space p, measurable_sets p) (m_space M1, measurable_sets M1) /\ Z IN measurable (m_space p, measurable_sets p) (m_space M2, measurable_sets M2)``)
     THEN1(METIS_TAC [random_variable_def, indep_varp_def, p_space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [GSYM pos_fn_integral_measure_of, measure_space_pair_measure', measure_space_distr]
THEN KNOW_TAC(``(\x. (XY:'a->'b) x , (Z:'a->'c) x )  IN measurable (m_space p, measurable_sets p) (m_space (pair_measure M1 M2), measurable_sets (pair_measure M1 M2))``)
     THEN1(METIS_TAC[p_space_def, m_space_def, measurable_Pair, prob_space_def])
THEN DISCH_TAC
THEN KNOW_TAC (``measure_space (pair_measure (M1: ('b->bool)#(('b->bool)->bool)#(('b->bool)->extreal)) (M2: ('c->bool)#(('c->bool)->bool)#(('c->bool)->extreal)))``)
     THEN1(METIS_TAC [measure_space_pair_measure'])
THEN DISCH_TAC
THEN  KNOW_TAC (``measure_space (distr p (pair_measure M1 M2) (\x. ((XY:'a->'b) x,(Z:'a->'c) x)))``)
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
THEN KNOW_TAC(``pos_fn_integral (distr p (pair_measure M1 M2) (\x. ((XY:'a->'b) x,(Z:'a->'c) x)))
  (indicator_fn A) = pos_fn_integral (measure_of (distr p (pair_measure M1 M2) (\x. (XY x,Z x))))
  (indicator_fn A)``)

    THEN1(METIS_TAC[GSYM pos_fn_integral_measure_of, measure_space_pair_measure', measure_space_distr, measure_space_pair_measure, prob_space_def])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN RW_TAC std_ss [GSYM indep_varp_distribution_imp]
THEN KNOW_TAC(``measure_space (pair_measure (distr p M1 (XY:'a->'b))(distr p M2 (Z:'a->'c)))``)

    THEN1( KNOW_TAC(``measure_space (distr p M1 XY) /\ measure_space (distr p M2 (Z:'a->'c))``)

       THEN1(METIS_TAC [measure_space_distr, prob_space_def])
       THEN DISCH_TAC
       THEN KNOW_TAC (``sigma_finite_measure (distr p M2 (Z:'a->'c))``)
            THEN1( RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A'`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'c->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (Z:'a->'c) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
      THEN DISCH_TAC
      THEN METIS_TAC [measure_space_pair_measure'])
THEN DISCH_TAC
THEN RW_TAC std_ss [pos_fn_integral_measure_of]);



val indep_var_pair_distr = store_thm("indep_var_pair_distr",
  `` !p M1 X M2 Z A.
     prob_space p /\ measure_space M1 /\ measure_space M2 /\
     sigma_finite_measure M2 /\ indep_var p M1 X M2 Z  /\
     A IN measurable_sets (pair_measure M1 M2)==>
      (pos_fn_integral (distr p (pair_measure M1 M2 ) (\x. (X x, Z x))) (indicator_fn A) =
      pos_fn_integral (pair_measure (distr p M1 X) (distr p M2 Z)) (indicator_fn A))``,
RW_TAC std_ss[]
THEN KNOW_TAC(``random_variable X p (m_space M1, measurable_sets M1) ``)
     THEN1(FULL_SIMP_TAC std_ss[indep_var, indep_vars]
          THEN UNDISCH_TAC(`` !i:num.
             i IN {x | (x = 0) \/ (x = 1)} ==>
             random_variable (if i = 0 then X else Z) p
               (m_space (if i = 0 then M1 else M2),
                measurable_sets (if i = 0 then M1 else M2))``)
	  THEN DISCH_TAC THEN POP_ASSUM (MP_TAC o Q.SPEC `0:num`)
	  THEN ASM_SET_TAC[])
THEN DISCH_TAC
THEN KNOW_TAC(``random_variable Z p (m_space M2, measurable_sets M2) ``)
     THEN1(FULL_SIMP_TAC std_ss[indep_var, indep_vars]
          THEN UNDISCH_TAC(`` !i:num.
             i IN {x | (x = 0) \/ (x = 1)} ==>
             random_variable (if i = 0 then X else Z) p
               (m_space (if i = 0 then M1 else M2),
                measurable_sets (if i = 0 then M1 else M2))``)
	  THEN DISCH_TAC THEN POP_ASSUM (MP_TAC o Q.SPEC `1:num`)
	  THEN PSET_TAC[])
THEN DISCH_TAC	  

THEN KNOW_TAC(``(X:'a->'b) IN measurable (m_space p, measurable_sets p) (m_space M1, measurable_sets M1) /\ (Z:'a->'b) IN measurable (m_space p, measurable_sets p) (m_space M2, measurable_sets M2)``)
     THEN1(METIS_TAC [random_variable_def,   p_space_def, events_def])
THEN DISCH_TAC
THEN RW_TAC std_ss [GSYM pos_fn_integral_measure_of, measure_space_pair_measure', measure_space_distr]
THEN KNOW_TAC(``(\x. (X:'a->'b) x , (Z:'a->'b) x )  IN measurable (m_space p, measurable_sets p) (m_space (pair_measure M1 M2), measurable_sets (pair_measure M1 M2))``)
     THEN1(METIS_TAC[p_space_def, m_space_def, measurable_Pair, prob_space_def])
THEN DISCH_TAC
THEN KNOW_TAC (``measure_space (pair_measure (M1: ('b->bool)#(('b->bool)->bool)#(('b->bool)->extreal)) (M2: ('b->bool)#(('b->bool)->bool)#(('b->bool)->extreal)))``)
     THEN1(METIS_TAC [measure_space_pair_measure'])
THEN DISCH_TAC
THEN  KNOW_TAC (``measure_space (distr p (pair_measure M1 M2) (\x. ((X:'a->'b) x,(Z:'a->'b) x)))``)
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
THEN KNOW_TAC(``pos_fn_integral (distr p (pair_measure M1 M2) (\x. ((X:'a->'b) x,(Z:'a->'b) x)))
  (indicator_fn A) = pos_fn_integral (measure_of (distr p (pair_measure M1 M2) (\x. (X x,Z x))))
  (indicator_fn A)``)

    THEN1(METIS_TAC[GSYM pos_fn_integral_measure_of, measure_space_pair_measure', measure_space_distr, measure_space_pair_measure, prob_space_def])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN RW_TAC std_ss [GSYM indep_var_distribution_imp]
THEN KNOW_TAC(``measure_space (pair_measure (distr p M1 (X:'a->'b))(distr p M2 (Z:'a->'b)))``)

    THEN1( KNOW_TAC(``measure_space (distr p M1 (X:'a->'b)) /\ measure_space (distr p M2 (Z:'a->'b))``)

       THEN1(METIS_TAC [measure_space_distr, prob_space_def])
       THEN DISCH_TAC
       THEN KNOW_TAC (``sigma_finite_measure (distr p M2 (Z:'a->'b))``)
            THEN1( RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A'`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'b->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (Z:'a->'b) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
      THEN DISCH_TAC
      THEN METIS_TAC [measure_space_pair_measure'])
THEN DISCH_TAC
THEN RW_TAC std_ss [pos_fn_integral_measure_of]);





val prob_event_3rv_2indep = store_thm("prob_event_3rv_2indep",
  `` ! p X Y Z M1 M2  A.

prob_space p /\ measure_space M1 /\ measure_space M2 /\
     sigma_finite_measure M1 /\ sigma_finite_measure M2 /\
     indep_varp p M1 (\x. (X x,Y x)) M2 Z /\
     A IN measurable_sets (pair_measure M1 M2) ==>
     (prob p (PREIMAGE (\x. ((X x,Y x),Z x)) A INTER p_space p) =
      pos_fn_integral (distr p M1 (\x. (X x,Y x)))
        (\(x,y).
           pos_fn_integral (distr p M2 Z)
             (\z. indicator_fn A ((x,y),z))))``,

RW_TAC std_ss []
THEN KNOW_TAC(``prob p (PREIMAGE (\x. (((X:'a->'b) x,(Y:'a->'c) x),(Z:'a->'d) x)) A INTER p_space p) =
                 distribution p (\x. (X x, Y x), Z x) A  ``)
     THEN1(METIS_TAC [distribution_def])		  
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(\x. ((X:'a->'b) x, (Y:'a->'c) x)) IN measurable (m_space p, measurable_sets p) (m_space M1, measurable_sets M1) /\ (Z:'a->'d) IN measurable (m_space p, measurable_sets p) (m_space M2, measurable_sets M2)``)
     THEN1(METIS_TAC [random_variable_def, indep_varp_def, p_space_def, events_def])
THEN DISCH_TAC

THEN KNOW_TAC(``(\x. ((X:'a->'b) x, (Y:'a-> 'c) x) , (Z:'a->'d) x )  IN measurable (m_space p, measurable_sets p) (m_space (pair_measure M1 M2), measurable_sets (pair_measure M1 M2))``)
     THEN1(METIS_TAC[p_space_def, m_space_def, measurable_Pair, prob_space_def])
THEN DISCH_TAC
THEN KNOW_TAC (``measure_space (pair_measure (M1: ('b#'c->bool)#(('b#'c->bool)->bool)#(('b#'c->bool)->extreal)) (M2: ('d->bool)#(('d->bool)->bool)#(('d->bool)->extreal)))``)
     THEN1(METIS_TAC [measure_space_pair_measure'])
THEN DISCH_TAC
THEN KNOW_TAC(``random_variable (\x. ((X:'a->'b) x, (Y:'a->'c) x) , (Z:'a->'d) x) p   (m_space (pair_measure M1 M2) , measurable_sets (pair_measure M1 M2)) ``)
    THEN1(METIS_TAC [random_variable_def, measurable_Pair, p_space_def, prob_space_def, events_def, subsets_def, measurable_sets_def, m_space_def, space_def] ) 
THEN DISCH_TAC
THEN KNOW_TAC(``distribution p (\x. (((X:'a->'b) x,(Y:'a->'c) x),(Z:'a->'d) x)) A =
                 pos_fn_integral (distr p (pair_measure M1 M2) (\x. (((X:'a->'b) x,(Y:'a->'c) x),(Z:'a->'d) x)) ) (indicator_fn A)``)

     THEN1( RW_TAC std_ss [distribution_lebesgue_thm2_pos_fn])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN RW_TAC std_ss [indep_varp_pair_distr]
THEN DEP_REWRITE_TAC [GSYM pos_fn_integral_fst]
THEN RW_TAC std_ss []
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
     THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A'`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'b#'c->bool) IN measurable_sets M1``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (\x. (X: 'a->'b) x, (Y:'a->'c) x) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
    THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A''`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'d->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (Z:'a->'d) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
    THEN1(METIS_TAC [indicator_fn_pos_le])
    THEN1( MP_TAC(Q.ISPECL[`(m_space (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d))),
   measurable_sets
     (pair_measure (distr p M1 (\x. (X x,Y x))) (distr p M2 Z)))` ,`1:real` ,`A:('b#'c)#'d->bool` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)
          THEN RW_TAC std_ss[]
          THEN KNOW_TAC(``sigma_finite_measure (distr p M2 (Z:'a->'d))``)
	      THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A''`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'d->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (Z:'a->'d) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
	  THEN DISCH_TAC	   
          THEN KNOW_TAC(``measure_space (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d)))``)
               THEN1 (METIS_TAC [measure_space_pair_measure', measure_space_distr, random_variable_def, prob_space_def, p_space_def, m_space_def, events_def, subsets_def, measurable_sets_def] )
	  THEN DISCH_TAC
	  THEN KNOW_TAC(``sigma_algebra (m_space (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d))),
   measurable_sets
     (pair_measure (distr p M1 (\x. (X x,Y x))) (distr p M2 Z)))``)
               THEN1(FULL_SIMP_TAC std_ss [measure_space_def])
          THEN DISCH_TAC
          THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(``A IN
       subsets
         (m_space
            (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d))),
          measurable_sets
            (pair_measure (distr p M1 (\x. (X x,Y x)))
               (distr p M2 Z)))``)
	       THEN1(RW_TAC std_ss [subsets_def]
	             THEN RW_TAC std_ss [distr]
	             THEN RW_TAC std_ss [distr, measurable_sets_def, subsets_def, pair_measure, measure_of]
                          THEN1(FULL_SIMP_TAC std_ss[m_space_def]
			  THEN FULL_SIMP_TAC std_ss [pair_measure]
			  THEN FULL_SIMP_TAC std_ss [measurable_sets_def, subsets_def, pair_measure, measure_of])
	             THEN FULL_SIMP_TAC std_ss[m_space_def]
			  THEN FULL_SIMP_TAC std_ss [pair_measure]
			  THEN FULL_SIMP_TAC std_ss [measurable_sets_def, subsets_def, pair_measure, measure_of])
        THEN DISCH_TAC
	THEN FULL_SIMP_TAC std_ss[]
	THEN FULL_SIMP_TAC std_ss [REAL_LE_01]
	THEN KNOW_TAC(``indicator_fn (A:('b#'c)#'d->bool) = (\x. Normal 1 * indicator_fn A x)``)
             THEN1(`indicator_fn (A:('b#'c)#'d->bool) = (\x.  indicator_fn (A:('b#'c)#'d->bool) x)` by  METIS_TAC [ETA_THM]
	            THEN ONCE_ASM_REWRITE_TAC[]
		    THEN `(\x. indicator_fn (A:('b#'c)#'d->bool) x) = (\x. 1:extreal * indicator_fn (A:('b#'c)#'d->bool) x)` by METIS_TAC [mul_lone]
		   THEN METIS_TAC [extreal_of_num_def])
	THEN DISCH_TAC
	THEN ONCE_ASM_REWRITE_TAC[]
	THEN FULL_SIMP_TAC std_ss[])
THEN RW_TAC std_ss [LAMBDA_PROD, PAIR]);



val mem_big_line_pair = store_thm ("mem_big_line_pair",
 ``!x. ?n. x IN line (FST n) CROSS line (SND n)``,
 GEN_TAC THEN SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN MP_TAC (ISPEC ``FST (x:real#real)`` SIMP_REAL_ARCH) THEN
 RW_TAC std_ss [] THEN 
 MP_TAC (ISPEC ``SND (x:real#real)`` SIMP_REAL_ARCH) THEN
  RW_TAC std_ss[] THEN
  SIMP_TAC std_ss [line, GSPECIFICATION] THEN
 ASM_CASES_TAC ``0 <= FST (x:real#real)``  THEN ASM_CASES_TAC ``0<= SND (x:real#real)`` 
 THEN1(EXISTS_TAC ``(n:num,n':num)`` THEN RW_TAC std_ss[] THEN ASM_REAL_ARITH_TAC)
 THEN1( MP_TAC (ISPEC ``-SND (x:real#real)`` SIMP_REAL_ARCH) THEN STRIP_TAC THEN
 EXISTS_TAC ``(n:num,n'':num)`` THEN RW_TAC std_ss[] THEN ASM_REAL_ARITH_TAC)
 THEN1(MP_TAC (ISPEC ``-FST (x:real#real)`` SIMP_REAL_ARCH) THEN STRIP_TAC THEN
 EXISTS_TAC ``(n'':num,n':num)`` THEN RW_TAC std_ss[] THEN ASM_REAL_ARITH_TAC)
 THEN MP_TAC (ISPEC ``-FST (x:real#real)`` SIMP_REAL_ARCH) THEN STRIP_TAC THEN
 MP_TAC (ISPEC ``-SND (x:real#real)`` SIMP_REAL_ARCH) THEN STRIP_TAC THEN 
 EXISTS_TAC ``(n'':num,n''':num)`` THEN RW_TAC std_ss[] THEN ASM_REAL_ARITH_TAC);


val sigma_finite_measure_pair_lborel = store_thm ("sigma_finite_measure_pair_lborel",
 `` sigma_finite_measure (pair_measure lborel lborel)``,

 RW_TAC std_ss [sigma_finite_measure]
 THEN  Q.EXISTS_TAC `{(\ y. line (FST y) CROSS line (SND y)) y | y IN UNIV } ` THEN
  REPEAT CONJ_TAC
  THEN1
  (KNOW_TAC(`` {(\ y. line (FST y) CROSS line (SND y)) y | y IN univ(:num # num)} = IMAGE (\ y. line (FST y) CROSS line (SND y)) univ(:num#num)``)
  THEN1(SIMP_TAC std_ss [GSYM IMAGE_DEF] THEN
  MP_TAC(Q.ISPECL[`(\y . line (FST y) CROSS line (SND y))`, `univ(:num#num)` ] IMAGE_DEF) THEN
  RW_TAC std_ss[LAMBDA_PROD, PAIR])
  THEN DISCH_TAC
 THEN ASM_REWRITE_TAC[] THEN 
 MATCH_MP_TAC pred_setTheory.image_countable THEN
 `univ(:num#num) = univ(:num) CROSS univ(:num)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
 THEN ASM_REWRITE_TAC[] THEN
SIMP_TAC std_ss [COUNTABLE_NUM, CROSS_COUNTABLE, GSPECIFICATION, EXTENSION])
THEN1(

   SIMP_TAC std_ss [SUBSET_DEF, lborel, measurable_sets_def, pair_measure, measure_of] THEN
   REPEAT STRIP_TAC THEN RW_TAC std_ss [borel_line]
   THEN1(FULL_SIMP_TAC std_ss [m_space_def] THEN
         KNOW_TAC(``?y. x = line (FST y)  CROSS line (SND y) ``)
         THEN1(FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
	       THEN Q.EXISTS_TAC`y`
               THEN RW_TAC std_ss[])
	  THEN RW_TAC std_ss[]
	  THEN KNOW_TAC (``line (FST y) CROSS line (SND y) IN {a CROSS b | a IN subsets borel /\ b IN subsets borel} ``)
         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
               THEN Q.EXISTS_TAC `(line (FST y), line (SND y))`
	        THEN RW_TAC std_ss[]
	        THEN METIS_TAC [borel_line])
   THEN DISCH_TAC
   THEN METIS_TAC [sigma_sets_basic])
   THEN RW_TAC std_ss[m_space_def]
   THEN FULL_SIMP_TAC std_ss [m_space_def]
   THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
   THEN FULL_SIMP_TAC std_ss[]
   THEN `POW univ(:real#real) = univ(:real#real->bool)` by RW_TAC std_ss [UNIV_FUN_TO_BOOL]
   THEN FULL_SIMP_TAC std_ss[]
   THEN METIS_TAC [IN_UNIV])
   THEN1(
  `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
  THEN 
   SIMP_TAC std_ss [EXTENSION, lborel, m_space_def, IN_UNIV, pair_measure, measure_of] THEN
   SIMP_TAC std_ss [IN_BIGUNION, GSPECIFICATION] THEN
  `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
   THEN ASM_REWRITE_TAC[]
   THEN SIMP_TAC std_ss [IN_UNIV] THEN STRIP_TAC THEN
   ASSUME_TAC mem_big_line_pair THEN POP_ASSUM (MP_TAC o Q.SPEC `x`) THEN
   SET_TAC []) 
   THEN
 RW_TAC std_ss[]

THEN  KNOW_TAC(``pos_fn_integral lborel
  (\x. pos_fn_integral lborel (\y. indicator_fn a (x,y))) = pos_fn_integral (pair_measure lborel lborel)(\ (x,y). indicator_fn a (x,y))``)
THEN1( 
MP_TAC(Q.ISPECL[`(\ (x,y). indicator_fn a (x:real,y:real))`, `lborel`, `lborel`]  pos_fn_integral_fst )
THEN RW_TAC std_ss [measure_space_lborel, sigma_finite_measure_lborel]
THEN KNOW_TAC(``(!x. 0 <= (\ (x,y). indicator_fn a (x:real,y:real)) x)``)
     THEN1(RW_TAC std_ss[] THEN METIS_TAC [indicator_fn_pos_le, PAIR, LAMBDA_PROD])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC (``(\ (x,y). indicator_fn a (x,y)) IN
      measurable
        (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel)) Borel``)
THEN1 (
MP_TAC(Q.ISPECL[` (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel))` ,`1:real` ,`a:real reln` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)

THEN RW_TAC std_ss [sigma_algebra_pair_lborel, REAL_LE_01] 

THEN KNOW_TAC(``(a:real reln) IN
      subsets
        (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel))``)
THEN1( FULL_SIMP_TAC std_ss [pair_measure, m_space_def, measurable_sets_def, measure_of, lborel, subsets_def]
      THEN  KNOW_TAC(``?y. a = line (FST y)  CROSS line (SND y) ``)
         THEN1(FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
	       THEN Q.EXISTS_TAC`y`
               THEN RW_TAC std_ss[])
THEN STRIP_TAC

       THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
       THEN FULL_SIMP_TAC std_ss[]
       THEN `POW univ(:real#real) = univ(:real#real->bool)` by RW_TAC std_ss [UNIV_FUN_TO_BOOL]
       THEN FULL_SIMP_TAC std_ss[]
       THEN FULL_SIMP_TAC std_ss [SUBSET_UNIV]
       THEN RW_TAC std_ss[]
       THEN KNOW_TAC(``line (FST y) CROSS line (SND (y:num#num)) IN {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	        THEN Q.EXISTS_TAC `(line (FST (y:num#num)), line (SND y))`
       THEN  RW_TAC std_ss [borel_line])
       THEN DISCH_TAC
       THEN METIS_TAC [sigma_sets_basic])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(\x. Normal 1 * indicator_fn a x) = (\ (x,y). indicator_fn (a:real reln) (x,y))``)
     THEN1(METIS_TAC [extreal_of_num_def, mul_lone, PAIR, LAMBDA_PROD])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[])

THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``?y. (a:real reln) = line (FST y ) CROSS line (SND y)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
          THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
	  THEN Q.EXISTS_TAC`y`
	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN STRIP_TAC
THEN KNOW_TAC(``(\ (x,y). indicator_fn (a:real reln) (x,y) ) = indicator_fn a``)	  
    THEN1(METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(a:real reln) IN measurable_sets (pair_measure lborel lborel)``)
    THEN1(FULL_SIMP_TAC std_ss [pair_measure, m_space_def, measurable_sets_def, measure_of, lborel, subsets_def]
THEN RW_TAC std_ss[]
THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
   THEN FULL_SIMP_TAC std_ss[]
   THEN `POW univ(:real#real) = univ(:real#real->bool)` by RW_TAC std_ss [UNIV_FUN_TO_BOOL]
   THEN FULL_SIMP_TAC std_ss[SUBSET_UNIV]
   THEN  KNOW_TAC(``line (FST y) CROSS line (SND (y:num#num)) IN {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	        THEN Q.EXISTS_TAC `(line (FST (y:num#num)), line (SND y))`
       THEN  RW_TAC std_ss [borel_line])
       THEN DISCH_TAC
       THEN METIS_TAC [sigma_sets_basic])
THEN DISCH_TAC
THEN RW_TAC std_ss [pos_fn_integral_indicator, measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]
THEN KNOW_TAC(``line (FST( y:num#num)) IN measurable_sets lborel /\ line (SND y) IN measurable_sets lborel``)
     THEN1(METIS_TAC [measurable_sets_def, lborel, subsets_def, borel_line])
THEN DISCH_TAC
THEN RW_TAC std_ss [measure_pair_measure_Times, sigma_finite_measure_lborel, measure_space_lborel]
THEN KNOW_TAC (``measure lborel (line (FST (y:num#num))) <> PosInf``)
THEN1 (
 RW_TAC std_ss [GSPECIFICATION, measure_def, lborel, measure_lebesgue] THEN
  SIMP_TAC std_ss [lt_infty, extreal_lt_def, le_sup] THEN
  Q.EXISTS_TAC `Normal (integral UNIV (indicator (line (FST (y:num#num)))))` THEN
  SIMP_TAC std_ss [extreal_le_def] THEN GEN_TAC THEN
  GEN_REWR_TAC LAND_CONV [GSYM SPECIFICATION] THEN
  RW_TAC std_ss [GSPECIFICATION] THEN SIMP_TAC std_ss [extreal_le_def] THEN
  GEN_REWR_TAC LAND_CONV [GSYM integral_indicator_UNIV] THEN
  (MP_TAC o Q.SPECL [`FST (y:num#num)`, `n`]) LESS_EQ_CASES THEN STRIP_TAC THEN
  FIRST_ASSUM (MP_TAC o MATCH_MP line_subset) THEN DISCH_TAC THENL
  [`line (FST (y:num#num)) INTER line n = line (FST y)` by ASM_SET_TAC [] THEN
   METIS_TAC [REAL_LE_REFL],
   ALL_TAC] THEN
`line (FST (y:num#num)) INTER line n = line n` by ASM_SET_TAC [] THEN
  ASM_REWRITE_TAC [] THEN MATCH_MP_TAC INTEGRAL_COMPONENT_LE THEN
  REPEAT STRIP_TAC THENL
  [ONCE_REWRITE_TAC [SET_RULE ``line n = line n INTER line n``] THEN
   REWRITE_TAC [integrable_indicator_UNIV] THEN
   SIMP_TAC std_ss [integrable_on] THEN
   Q.EXISTS_TAC `content (line n INTER line n)` THEN
   METIS_TAC [has_integral_interval_cube, line, interval],
 ONCE_REWRITE_TAC [SET_RULE ``line n = line n INTER line n``] THEN
   REWRITE_TAC [integrable_indicator_UNIV] THEN
   SIMP_TAC std_ss [integrable_on] THEN
   Q.EXISTS_TAC `content (line (FST (y:num#num)) INTER line (FST y))` THEN
   METIS_TAC [has_integral_interval_cube, line, interval],
   ALL_TAC] THEN
  SIMP_TAC std_ss [indicator] THEN REPEAT COND_CASES_TAC THEN
  SIMP_TAC real_ss [] THEN ASM_SET_TAC [])
THEN DISCH_TAC
THEN KNOW_TAC (``measure lborel (line (SND (y:num#num))) <> PosInf``)
THEN1 (
 RW_TAC std_ss [GSPECIFICATION, measure_def, lborel, measure_lebesgue] THEN
  SIMP_TAC std_ss [lt_infty, extreal_lt_def, le_sup] THEN
  Q.EXISTS_TAC `Normal (integral UNIV (indicator (line (SND (y:num#num)))))` THEN
  SIMP_TAC std_ss [extreal_le_def] THEN GEN_TAC THEN
  GEN_REWR_TAC LAND_CONV [GSYM SPECIFICATION] THEN
  RW_TAC std_ss [GSPECIFICATION] THEN SIMP_TAC std_ss [extreal_le_def] THEN
  GEN_REWR_TAC LAND_CONV [GSYM integral_indicator_UNIV] THEN
  (MP_TAC o Q.SPECL [`SND (y:num#num)`, `n`]) LESS_EQ_CASES THEN STRIP_TAC THEN
  FIRST_ASSUM (MP_TAC o MATCH_MP line_subset) THEN DISCH_TAC THENL
  [`line (SND (y:num#num)) INTER line n = line (SND y)` by ASM_SET_TAC [] THEN
   METIS_TAC [REAL_LE_REFL],
   ALL_TAC] THEN
`line (SND (y:num#num)) INTER line n = line n` by ASM_SET_TAC [] THEN
  ASM_REWRITE_TAC [] THEN MATCH_MP_TAC INTEGRAL_COMPONENT_LE THEN
  REPEAT STRIP_TAC THENL
  [ONCE_REWRITE_TAC [SET_RULE ``line n = line n INTER line n``] THEN
   REWRITE_TAC [integrable_indicator_UNIV] THEN
   SIMP_TAC std_ss [integrable_on] THEN
   Q.EXISTS_TAC `content (line n INTER line n)` THEN
   METIS_TAC [has_integral_interval_cube, line, interval],
ONCE_REWRITE_TAC [SET_RULE ``line n = line n INTER line n``] THEN
   REWRITE_TAC [integrable_indicator_UNIV] THEN
   SIMP_TAC std_ss [integrable_on] THEN
   Q.EXISTS_TAC `content (line (SND (y:num#num)) INTER line (SND y))` THEN
   METIS_TAC [has_integral_interval_cube, line, interval],
   ALL_TAC] THEN
  SIMP_TAC std_ss [indicator] THEN REPEAT COND_CASES_TAC THEN
  SIMP_TAC real_ss [] THEN ASM_SET_TAC [])
THEN DISCH_TAC
THEN KNOW_TAC(``measure lborel (line (FST (y:num#num))) <> NegInf /\
                 measure lborel (line (SND y)) <> NegInf``)
     THEN1(`sigma_finite_measure lborel` by RW_TAC std_ss [sigma_finite_measure_lborel]
            THEN `positive lborel` by METIS_TAC [measure_space_lborel, measure_space_def]
	    THEN FULL_SIMP_TAC std_ss [positive_def]
	    THEN `0<= measure lborel  (line (FST (y:num#num)))` by RW_TAC std_ss[]
	    THEN `0<= measure lborel  (line (SND (y:num#num)))` by RW_TAC std_ss[]
	    THEN METIS_TAC [extreal_of_num_def, extreal_le_def, extreal_lt_def, lt_infty, le_infty, lt_le, le_lt])
THEN METIS_TAC [mul_extreal_not_infty]);



(*val sigma_finite_measure_pair_measure = store_thm(
  "sigma_finite_measure_pair_measure",
  `` !M1 M2. sigma_finite_measure M1 /\ sigma_finite_measure M2 ==>
             sigma_finite_measure (pair_measure M1 M2) ``,

RW_TAC std_ss [sigma_finite_measure]
Q.EXISTS_TAC `{a CROSS b | a IN A/\ b IN A'}  `
RW_TAC std_ss[]
SRW_TAC [] [GSPECIFICATION, EXTENSION, countable_def]

FULL_SIMP_TAC std_ss [countable_def]
)*)
(*
borel_measurable_pos_integral_fst;
split the outer integral by proving that the inner integral is measurable, then

*)



val sigma_algebra_pair_pair_lborel = store_thm("sigma_algebra_pair_pair_lborel",
  ``  sigma_algebra
        (m_space (pair_measure (pair_measure lborel lborel) lborel),
         measurable_sets
           (pair_measure (pair_measure lborel lborel) lborel))``,

` measure_space (pair_measure (pair_measure lborel lborel )lborel )` by METIS_TAC [measure_space_pair_measure',measure_space_lborel, sigma_finite_measure_lborel, sigma_finite_measure_pair_lborel]
THEN METIS_TAC [measure_space_def]);


val joint_density_3rv_0 = store_thm("joint_density_3rv_0",
  ``  !p X Y Z  A f_xy.
     prob_space p /\  random_variable Z p borel /\ (!x. 0<= f_xy x) /\ distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy /\
     
     A IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel) ==>
    (pos_fn_integral (distr p (pair_measure lborel lborel ) (\x. (X x,Y x)))
        (\ (x,y).
           pos_fn_integral (distr p lborel Z)
             (\z. indicator_fn A ((x,y),z))) =
	        pos_fn_integral (pair_measure lborel lborel) (\ (x,y).  f_xy (x,y) *
		    pos_fn_integral (distr p lborel Z) (\z.  indicator_fn A ((x,y),z) ) )) ``,

RW_TAC std_ss[distributed_def]
THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
			       THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                     THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
                               THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]

THEN  MP_TAC(Q.ISPECL[` f_xy:(real#real)->extreal `, `(\ (x:real,y:real).
     pos_fn_integral (distr p lborel Z)
       (\z. indicator_fn A ((x,y),z)))` , `(pair_measure lborel lborel)` ]  pos_fn_integral_density_1)
THEN RW_TAC std_ss[measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]  THEN UNDISCH_TAC (``f_xy IN
      measurable
        (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel)) Borel``)
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(\(x:real,y:real).
         pos_fn_integral (distr p lborel Z)
           (\z. indicator_fn A ((x,y),z:real))) IN
      measurable
        (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel)) Borel``)

     THEN1(MP_TAC(Q.ISPECL[` indicator_fn (A:((real#real)#real->bool)) `, `pair_measure lborel lborel`, `distr p lborel (Z:'a->real)` ] borel_measurable_pos_integral_fst)
             THEN RW_TAC std_ss [measure_space_pair_measure', sigma_finite_measure_lborel]
	     THEN `measure_space (pair_measure lborel lborel)` by METIS_TAC [measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]
	     THEN FULL_SIMP_TAC std_ss[]
	     THEN `measure_space (distr p lborel (Z:'a->real))` by 
	               (FULL_SIMP_TAC std_ss [random_variable_def]
                          
		         THEN KNOW_TAC(``(Z:'a->real) IN measurable (m_space p, measurable_sets p)
			                (m_space lborel, measurable_sets lborel)``)
			      THEN1( UNDISCH_TAC(``(Z:'a->real) IN measurable (p_space p,events p) (m_space lborel, measurable_sets lborel)``)
			      THEN DISCH_TAC
			      THEN METIS_TAC [measure_space_distr, measure_space_lborel, prob_space_def, random_variable_def, p_space_def, events_def, measurable_sets_def, subsets_def, borel, lborel, m_space_def, space_def, space_borel])
			  THEN  METIS_TAC [measure_space_distr, measure_space_lborel, prob_space_def, random_variable_def, p_space_def, events_def, measurable_sets_def, subsets_def, borel, lborel, m_space_def, space_def, space_borel])
	      THEN FULL_SIMP_TAC std_ss[]
              	      
	      THEN KNOW_TAC(`` sigma_finite_measure (distr p lborel (Z:'a->real)) ``)
		   THEN1(METIS_TAC[sigma_finite_measure_distr, random_variable_def])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!(x:(real#real)#real). 0 <= indicator_fn A x)``)  
     THEN1(METIS_TAC [indicator_fn_pos_le])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``indicator_fn A IN
       measurable
         (m_space
            (pair_measure (pair_measure lborel lborel)
               (distr p lborel (Z:'a->real))),
          measurable_sets
            (pair_measure (pair_measure lborel lborel)
               (distr p lborel Z))) Borel``)
     THEN1(MP_TAC(Q.ISPECL[`(m_space (pair_measure (pair_measure lborel lborel) (distr p lborel (Z:'a->real))),
    measurable_sets
     (pair_measure (pair_measure lborel lborel) (distr p lborel Z))) ` ,`1:real` ,`A:(real#real)#real->bool` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)

          THEN RW_TAC std_ss [ REAL_LE_01]
	  THEN KNOW_TAC(``m_space
            (pair_measure (pair_measure lborel lborel)
               (distr p lborel Z)) = m_space
            (pair_measure (pair_measure lborel lborel)
               lborel)``)
	       THEN1(RW_TAC std_ss [distr, m_space_def, pair_measure, measure_of])
	  THEN DISCH_TAC
	  THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(`` measurable_sets
            (pair_measure (pair_measure lborel lborel)
               (distr p lborel Z)) =  measurable_sets
            (pair_measure (pair_measure lborel lborel)
               lborel)``)
	       THEN1(REWRITE_TAC[m_space_def, distr, measurable_sets_def, pair_measure, measure_of, subsets_def, lborel])
	       
	  THEN DISCH_TAC
	  THEN FULL_SIMP_TAC std_ss[]
	  THEN `sigma_algebra
         (m_space (pair_measure (pair_measure lborel lborel) lborel),
          measurable_sets
            (pair_measure (pair_measure lborel lborel) lborel))` by RW_TAC std_ss[sigma_algebra_pair_pair_lborel ]
	  THEN FULL_SIMP_TAC std_ss[]
	  
          THEN ` A IN
         subsets
        (m_space (pair_measure (pair_measure lborel lborel) lborel),
         measurable_sets
           (pair_measure (pair_measure lborel lborel) lborel))` by METIS_TAC[pair_measure, lborel, measure_of, subsets_def, m_space_def, measurable_sets_def]
	   THEN FULL_SIMP_TAC std_ss[]
          THEN KNOW_TAC(``!(x: (real#real)#real). x = ((FST (FST x),SND (FST x)), SND x) ``)
               THEN1(METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])
          THEN DISCH_TAC     
          THEN KNOW_TAC(``(\ (x:(real#real)#real). Normal 1 * indicator_fn A x) = (\x. indicator_fn A x)``)
               THEN1(METIS_TAC [extreal_of_num_def, mul_lone, PAIR, LAMBDA_PROD])
          THEN DISCH_TAC
          THEN MP_TAC(Q.ISPECL [`indicator_fn (A:(real#real)#real->bool) `]LAMBDA_PROD)                       THEN DISCH_TAC
		     
          THEN FULL_SIMP_TAC std_ss[]
          THEN METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!(x:real#real). x IN m_space (pair_measure lborel lborel)``)
     THEN1(REWRITE_TAC [m_space_def, pair_measure, measure_of, lborel]
           THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CROSS_DEF]
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN METIS_TAC [IN_UNIV])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!(x:real#real) ( y:real). max 0 (indicator_fn A (x,y)) = indicator_fn A (x,y) ``)
     THEN1(RW_TAC std_ss [extreal_max_def])
THEN DISCH_TAC     
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!(x:real#real) (y:real).
   pos_fn_integral (distr p lborel Z) (\y:real. indicator_fn A (x,y)) = 
   pos_fn_integral (distr p lborel Z) (\y. indicator_fn A ((FST x, SND x),y)))``)
    THEN1(METIS_TAC [PAIR, ETA_THM, LAMBDA_PROD])
THEN DISCH_TAC
THEN UNDISCH_TAC (`` (\x.
          pos_fn_integral (distr p lborel (Z:'a->real))
            (\y. indicator_fn A (x,y))) IN
       measurable
         (m_space (pair_measure lborel lborel),
          measurable_sets (pair_measure lborel lborel)) Borel
 ``)
THEN ONCE_ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[PAIR, LAMBDA_PROD, ETA_THM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!(x:real#real).
         0 <=
         (\(x,y).
            pos_fn_integral (distr p lborel (Z:'a->real))
              (\z. indicator_fn A ((x,y),z))) x``)

     THEN1(RW_TAC std_ss[]
          THEN KNOW_TAC(``(\ (x:real,y:real).
   pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn A ((x,y),z))) x = ( pos_fn_integral (distr p lborel Z) (\z. indicator_fn A (x,z)))``)
   
              THEN1(`x:real#real = (FST x, SND x)` by RW_TAC std_ss [PAIR]
	           THEN ONCE_ASM_REWRITE_TAC[]
                   THEN RW_TAC std_ss[PAIR, ETA_THM, LAMBDA_PROD])
         THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`, ` \z. indicator_fn A ((x:real#real),z:real)  `]  pos_fn_integral_pos  )
         THEN RW_TAC std_ss[indicator_fn_pos_le]
         THEN `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC [measure_space_distr,random_variable_def, measure_space_lborel, prob_space_def, p_space_def, events_def, measurable_sets_def, subsets_def, space_def]
         THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss [PAIR, ETA_THM, LAMBDA_PROD]);



(**** event Aa.(A1 before A2) . .(A2 before Aa)***)

val IN_ELIM_PAIR_THM_1 = store_thm("IN_ELIM_PAIR_THM_1",
  ``!P a b c . ((a,b),c) IN {((x,y),z) | P x y z} <=> P a b c``,
  SRW_TAC [][]);


val IN_ELIM_PAIR_THM_2 = store_thm("IN_ELIM_PAIR_THM_2",
  ``!P a b c . (c, (a,b)) IN {(z, (x,y)) | P z x y } <=> P c a b ``,
  SRW_TAC [][]);

val pair_exists = store_thm("pair_exists",
  `` ! x y. ?p. p = (x,y)``,
RW_TAC std_ss [PAIR]);





val event1_shared_spare = store_thm("event1_shared_spare",
  ``  !x y z t:real.
       indicator_fn
         {((x',y'),z') |
          y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
          y' <= t} ((x,y),z) =
       indicator_fn {z' | y < z' /\ z' < x} z *
       indicator_fn {x' |  y < x' /\ x' <= t} x *
       indicator_fn {y' | 0 <= y' /\ y' <= t } y``,
       
REPEAT GEN_TAC
THEN RW_TAC std_ss [indicator_fn_def]
  THEN1(METIS_TAC [mul_lone])

THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)



THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)


THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN METIS_TAC [mul_rzero, mul_comm]);


val ETA_ETA_THM = store_thm("ETA_ETA_THM",
  `` ! P. (\x. (\y. P y)x) = (\y. P y)``, 
RW_TAC std_ss [ETA_THM]);

val ETA_ETA_THM_PAIR = store_thm("ETA_ETA_THM_PAIR",
  `` ! P. (\x. (\ (y1,y2). P (y1,y2))x) = (\ (y1,y2). P (y1,y2))``, 
RW_TAC std_ss [ETA_ETA_THM, PAIR, LAMBDA_PROD]);


val pos_fn_integral_fun_mul1 = store_thm("pos_fn_integral_fun_mul1",
  `` ! M (Y:'b->real) f . measure_space M /\ (!x. x IN m_space M ==> 0 <= f x)/\ (!y. 0<= (Y:'b->real)  y)==>
(!y. pos_fn_integral M (\x. Normal (Y y) * f x ) = Normal (Y y) * pos_fn_integral M (\x. f x))``, 

RW_TAC std_ss[]
THEN KNOW_TAC (``(!y. pos_fn_integral (M:
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)) (\x. Normal ((Y:'b->real) y) * f x ) = Normal (Y y) * pos_fn_integral M (\x. f x))``)
            THEN1  (RW_TAC std_ss[]
	           THEN Cases_on `Y y = (0:real)`
                        THEN1(RW_TAC std_ss [GSYM extreal_of_num_def,mul_lzero,pos_fn_integral_zero])
			  THEN `(0:real) < (Y:'b->real) y` by FULL_SIMP_TAC std_ss [REAL_LT_LE]
                          THEN RW_TAC std_ss [pos_fn_integral_def,sup_eq]
			       THEN1( Q_TAC SUFF_TAC `y' / Normal (Y y )<= sup {r | ?g. r IN psfis M g /\ !x. g x <= f x}`
			       	     THEN1(  METIS_TAC [le_ldiv,mul_comm])
				     THEN  RW_TAC std_ss [le_sup]
				     THEN POP_ASSUM MATCH_MP_TAC
				     THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
				     THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
 				     THEN RW_TAC std_ss [GSPECIFICATION]
				     THEN Q.EXISTS_TAC `(\x. g x / Normal( Y y))`
				     THEN  REVERSE (RW_TAC std_ss [])
				     	  THEN1( METIS_TAC [mul_comm,le_ldiv])
					  THEN RW_TAC std_ss [extreal_div_def]
					  THEN ONCE_REWRITE_TAC [mul_comm]
					  THEN RW_TAC std_ss [extreal_inv_def]
					  THEN KNOW_TAC (``!y. 0<=inv ((Y:'b->real) y)``)
					       THEN1(METIS_TAC [REAL_LE_INV])
					  THEN DISCH_TAC
					  THEN MP_TAC(Q.ISPECL[`(M :
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal))`,`(g :'a -> extreal)`,`(y' :extreal)`,`D:'a->real`] psfis_fun_mul )
	       	  	   	   	   THEN KNOW_TAC (``inv ((Y:'b->real)y) = (inv o Y) y``)
					   		 THEN1( METIS_TAC[o_THM])
					   THEN DISCH_TAC
					   THEN ASM_REWRITE_TAC[]
					   THEN KNOW_TAC (``0<=((inv o (Y :'b -> real)) (y :'b))``)
					   	THEN1(METIS_TAC [REAL_LE_INV, o_THM])
						THEN DISCH_TAC
						THEN METIS_TAC [o_DEF, psfis_fun_mul])
			THEN SUFF_TAC(``sup {r | ?g. r IN psfis M g /\ !x. g x <= f x} <= y' / (Normal (Y y))``)
			      THEN1(METIS_TAC [le_rdiv,extreal_not_infty,mul_comm])

			THEN RW_TAC std_ss [sup_le]
   			THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Suff`(y'' * (Normal (Y y))) <= y'`
			      	   THEN1(METIS_TAC [le_rdiv,extreal_not_infty])
	   		THEN Q.PAT_X_ASSUM `!z. {r | ?g. r IN psfis M g /\ !x. g x <= (Normal (Y y)) * f x} z ==> z <= y'` MATCH_MP_TAC
			THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Q.EXISTS_TAC `(\x. (Normal (Y y ))* g x)`
			THEN RW_TAC std_ss []
			     THEN1( METIS_TAC [psfis_fun_mul,mul_comm,extreal_not_infty])
			THEN  METIS_TAC [le_lmul_imp,extreal_of_num_def,extreal_lt_eq,lt_le])
THEN METIS_TAC []);





val pos_fn_integral_fun_mul2 = store_thm("pos_fn_integral_fun_mul2",
  `` ! M (Y:'b->real) f . measure_space M /\ (!x y. x IN m_space M ==> 0 <= f(x,y))/\ (!y. 0<= (Y:'b->real)  y)==>
(!y. pos_fn_integral M (\x. Normal (Y y) * f (x,y) ) = Normal (Y y) * pos_fn_integral M (\x. f (x,y)))``, 

RW_TAC std_ss[]
THEN KNOW_TAC (``(!y. pos_fn_integral (M:
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)) (\x. Normal ((Y:'b->real) y) * f (x,y) ) = Normal (Y y) * pos_fn_integral M (\x. f (x,y)))``)
            THEN1  (  RW_TAC std_ss[]
	           THEN Cases_on `Y y = (0:real)`
                        THEN1(RW_TAC std_ss [GSYM extreal_of_num_def,mul_lzero,pos_fn_integral_zero])
			  THEN `(0:real) < (Y:'b->real) y` by FULL_SIMP_TAC std_ss [REAL_LT_LE]
                          THEN RW_TAC std_ss [pos_fn_integral_def,sup_eq]
			       THEN1( Q_TAC SUFF_TAC `y' / Normal (Y y )<= sup {r | ?g. r IN psfis M g /\ !x  . g x <= f (x,y)}`
			       	     THEN1(  METIS_TAC [le_ldiv,mul_comm])
				     THEN  RW_TAC std_ss [le_sup]
				     THEN POP_ASSUM MATCH_MP_TAC
				     THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
				     THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
 				     THEN RW_TAC std_ss [GSPECIFICATION]
				     THEN Q.EXISTS_TAC `(\x. g x / Normal( Y y))`
				     THEN  REVERSE (RW_TAC std_ss [])
				     	  THEN1( METIS_TAC [mul_comm,le_ldiv])
					  THEN RW_TAC std_ss [extreal_div_def]
					  THEN ONCE_REWRITE_TAC [mul_comm]
					  THEN RW_TAC std_ss [extreal_inv_def]
					  THEN KNOW_TAC (``!y. 0<=inv ((Y:'b->real) y)``)
					       THEN1(METIS_TAC [REAL_LE_INV])
					  THEN DISCH_TAC
					  THEN MP_TAC(Q.ISPECL[`(M :
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal))`,`(g :'a -> extreal)`,`(y' :extreal)`,`D:'a->real`] psfis_fun_mul )
	       	  	   	   	   THEN KNOW_TAC (``inv ((Y:'b->real)y) = (inv o Y) y``)
					   		 THEN1( METIS_TAC[o_THM])
					   THEN DISCH_TAC
					   THEN ASM_REWRITE_TAC[]
					   THEN KNOW_TAC (``0<=((inv o (Y :'b -> real)) (y :'b))``)
					   	THEN1(METIS_TAC [REAL_LE_INV, o_THM])
						THEN DISCH_TAC
						THEN METIS_TAC [o_DEF, psfis_fun_mul])
			THEN  SUFF_TAC(``sup {r | ?g. r IN psfis M g /\ !x. g x <= f (x,y)} <= y' / (Normal (Y y))``)
			      THEN1(METIS_TAC [le_rdiv,extreal_not_infty,mul_comm])

			THEN  RW_TAC std_ss [sup_le]
   			THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Suff`(y'' * (Normal (Y y))) <= y'`
			      	   THEN1(METIS_TAC [le_rdiv,extreal_not_infty])
	   		THEN Q.PAT_X_ASSUM `!z. {r | ?g. r IN psfis M g /\ !x. g x <= (Normal (Y y)) * f (x,y)} z ==> z <= y'` MATCH_MP_TAC
			THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Q.EXISTS_TAC `(\x. (Normal (Y y ))* g x)`
			THEN RW_TAC std_ss []
			     THEN1( METIS_TAC [psfis_fun_mul,mul_comm,extreal_not_infty])
			THEN  METIS_TAC [le_lmul_imp,extreal_of_num_def,extreal_lt_eq,lt_le])
THEN METIS_TAC []);






val pos_fn_integral_fun_mul3 = store_thm("pos_fn_integral_fun_mul3",
  `` ! M (X:'b->real) f . measure_space M /\ (!x y. y IN m_space M ==> 0 <= f(x,y))/\ (!x. 0<= (X:'b->real)  x)==>
(!x. pos_fn_integral M (\y. Normal (X x) * f (x,y) ) = Normal (X x) * pos_fn_integral M (\y. f (x,y)))``, 

RW_TAC std_ss[]
THEN KNOW_TAC (``(!x. pos_fn_integral (M:
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)) (\y. Normal ((X:'b->real) x) * f (x,y) ) = Normal (X x) * pos_fn_integral M (\y. f (x,y)))``)
            THEN1  (  RW_TAC std_ss[]
	           THEN Cases_on `X x = (0:real)`
                        THEN1(RW_TAC std_ss [GSYM extreal_of_num_def,mul_lzero,pos_fn_integral_zero])
			  THEN `(0:real) < (X:'b->real) x` by FULL_SIMP_TAC std_ss [REAL_LT_LE]
                          THEN RW_TAC std_ss [pos_fn_integral_def,sup_eq]
			       THEN1( Q_TAC SUFF_TAC `y / Normal (X x )<= sup {r | ?g. r IN psfis M g /\ !y  . g y <= f (x,y)}`
			       	     THEN1(  METIS_TAC [le_ldiv,mul_comm])
				     THEN  RW_TAC std_ss [le_sup]
				     THEN POP_ASSUM MATCH_MP_TAC
				     THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
				     THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
 				     THEN RW_TAC std_ss [GSPECIFICATION]
				     THEN Q.EXISTS_TAC `(\y. g y / Normal( X x))`
				     THEN  REVERSE (RW_TAC std_ss [])
				     	  THEN1( METIS_TAC [mul_comm,le_ldiv])
					  THEN RW_TAC std_ss [extreal_div_def]
					  THEN ONCE_REWRITE_TAC [mul_comm]
					  THEN RW_TAC std_ss [extreal_inv_def]
					  THEN KNOW_TAC (``!x. 0<=inv ((X:'b->real) x)``)
					       THEN1(METIS_TAC [REAL_LE_INV])
					  THEN DISCH_TAC
					  THEN MP_TAC(Q.ISPECL[`(M :
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal))`,`(g :'a -> extreal)`,`(y :extreal)`, `D:'a->real`] psfis_fun_mul )
	       	  	   	   	   THEN KNOW_TAC (``inv ((X:'b->real)x) = (inv o X) x``)
					   		 THEN1( METIS_TAC[o_THM])
					   THEN DISCH_TAC
					   THEN ASM_REWRITE_TAC[]
					   THEN KNOW_TAC (``0<=((inv o (X :'b -> real)) (x :'b))``)
					   	THEN1(METIS_TAC [REAL_LE_INV, o_THM])
						THEN DISCH_TAC
						THEN METIS_TAC [o_DEF, psfis_fun_mul])
			THEN  SUFF_TAC(``sup {r | ?g. r IN psfis M g /\ !y. g y <= f (x,y)} <= y / (Normal (X x))``)
			      THEN1(METIS_TAC [le_rdiv,extreal_not_infty,mul_comm])

			THEN  RW_TAC std_ss [sup_le]
   			THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Suff`(y' * (Normal (X x))) <= y`
			      	   THEN1(METIS_TAC [le_rdiv,extreal_not_infty])
	   		THEN Q.PAT_X_ASSUM `!z. {r | ?g. r IN psfis M g /\ !y. g y <= (Normal (X x)) * f (x,y)} z ==> z <= y` MATCH_MP_TAC
			THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Q.EXISTS_TAC `(\y. (Normal (X x ))* g y)`
			THEN RW_TAC std_ss []
			     THEN1( METIS_TAC [psfis_fun_mul,mul_comm,extreal_not_infty])
			THEN  METIS_TAC [le_lmul_imp,extreal_of_num_def,extreal_lt_eq,lt_le])
THEN METIS_TAC []);



val normal_real_indicator_fn = store_thm("normal_real_indicator_fn",
  `` ! A y. indicator_fn A y = Normal (real (indicator_fn A y))``,
     RW_TAC std_ss[]
     THEN `indicator_fn A y <> PosInf /\
		            indicator_fn A y <> NegInf` by 
               (RW_TAC std_ss [indicator_fn_def]
			        THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def])
     THEN RW_TAC std_ss[normal_real]);





val BIGUNION_event1_shared_spare_subevent0 = store_thm(
  "BIGUNION_event1_shared_spare_subevent0",
  `` !t q1 q2.BIGUNION {({x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
                         {y'| y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0 }) CROSS
		      {z' | real q1 < z' /\ z' < real q2}| q0 IN Q_set}=  {(x',y') |
           y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\
           real q2 < x'} CROSS {z' | real q1 < z' /\ z' < real q2}  ``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(STRIP_TAC
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
              
              x IN s <=>
             ((real q0 < FST (FST x) /\ FST (FST x) <= t /\
               real q2 < FST (FST x)) /\ SND (FST x) < real q1 /\
              0 <= SND (FST x) /\ SND (FST x) <= t /\ SND (FST x) < real q0) /\
             real q1 < SND x /\ SND x < real q2``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)
          THEN `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EXISTS_TAC(``FST (x:(real#real)#real) ``)
           THEN RW_TAC std_ss [LAMBDA_PAIR]
	   THEN METIS_TAC [REAL_LT_TRANS])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]

THEN  KNOW_TAC(``?q0. q0 IN Q_set/\ Normal (SND (x':real#real) ) < q0 /\  q0 < Normal (FST x')``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r0. q0 = Normal r0``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (SND (FST (x:(real#real)#real))) < Normal( FST (FST x))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC

THEN EXISTS_TAC (``{x| real (Normal r0) < FST (FST x) /\ FST (FST x) <= t /\
                  real q2 < FST (FST x) /\ SND (FST x) < real q1 /\
                 0 <= SND (FST x) /\ SND (FST x) <= t /\
                 SND (FST x) < real (Normal r0) /\ real q1 < SND x /\ SND x < real q2}  ``)
     

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r0`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);


val event1_shared_spare_subevent0_measurable_set = store_thm(
  "event1_shared_spare_subevent0_measurable_set",
  ``!t q0 q1 q2. ({x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
            {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
            {z' | real q1 < z' /\ z' < real q2} IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,

RW_TAC std_ss [measurable_sets_def, pair_measure, measure_of, lborel, m_space_def]
    THEN1 (KNOW_TAC(``  {x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
       	  		{y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0} CROSS
   			{z' | real q1 < z' /\ z' < real q2} IN
                       {(a:real reln) CROSS (b:real->bool) |
                         a IN
                         sigma_sets (univ(:real) CROSS univ(:real))
                         {a CROSS b | a IN subsets borel /\ b IN subsets borel} /\
                         b IN subsets borel}``)
           THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	         THEN EXISTS_TAC (``({x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
        {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0}, {z' | real q1 < z' /\ z' < real q2}) ``)
                THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                     THEN1(KNOW_TAC(``{x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
		                      {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0} IN
				       {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
                          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			        THEN EXISTS_TAC (``({x' | real q0 < x' /\ x' <= t /\ real q2 < x'},
        {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0}) ``)
			        THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                                     THEN1( KNOW_TAC(``{x' | real q0 < x' /\ x' <= t /\ real q2 < x'} =
					    {x' | real q0 < x'} INTER
					    {x' | x' <= t} INTER
					    {x' | real q2 < x'} ``)
					    THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
					          THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC])
				    THEN DISCH_TAC
				    THEN ASM_REWRITE_TAC[]
                                    THEN `{x' | real q0 < x'} IN subsets borel /\
					    {x' | x' <= t} IN subsets borel /\
					    {x' | real q2 < x'} IN subsets borel` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_LEFT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT] 
 			            THEN  METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
			    THEN KNOW_TAC(``   {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0} =
			    	 	       {y' | y' < real q1} INTER
					       {y'| 0 <= y'} INTER
					       {y' | y' <= t} INTER
					       {y' | y' < real q0}``)
			        THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
				      THEN METIS_TAC[CONJ_ASSOC, CONJ_COMM])
 			   THEN DISCH_TAC				      
			   THEN ASM_REWRITE_TAC[]
			   THEN `{y' | y' < real q1} IN subsets borel /\
			         {y' | 0 <= y'} IN subsets borel /\
				 {y' | y' <= t} IN subsets borel /\
 				 {y' | y' < real q0} IN subsets borel` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_LEFT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, CLOSED_INTERVAL_RIGHT]
    		          THEN 	METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
	           THEN DISCH_TAC 
		   THEN METIS_TAC [sigma_sets_basic])
              THEN `{z' | real q1 < z' /\ z' < real q2} = {z' | real q1 < z'} INTER {z'| z' < real q2} ` 		   by SRW_TAC [] [GSPECIFICATION, EXTENSION]
              THEN ASM_REWRITE_TAC[]
	      THEN ` {z' | real q1 < z'} IN subsets borel /\ {z' | z' < real q2} IN subsets borel`
	         by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
             THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
        THEN DISCH_TAC
        THEN METIS_TAC [sigma_sets_basic])
   THEN1( KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
         THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
	         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(``{(a:real->bool) CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET
          POW univ(:real#real) ``)
             THEN1( METIS_TAC [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
	 THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[])
THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]	
THEN FULL_SIMP_TAC std_ss[GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV]);







val BIGUNION_event1_shared_spare_subevent0_measurable_set = store_thm(
  "BIGUNION_event1_shared_spare_subevent0_measurable_set",
  ``!t q1 q2.   {(x',y') |
           y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\
           real q2 < x'} CROSS {z' | real q1 < z' /\ z' < real q2} IN
	  measurable_sets (pair_measure (pair_measure lborel lborel) lborel)  ``,

RW_TAC std_ss [GSYM BIGUNION_event1_shared_spare_subevent0]

THEN KNOW_TAC (``!q0. q0 IN Q_set ==> (({x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
     {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
     {z' | real q1 < z' /\ z' < real q2})   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[event1_shared_spare_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
   BIGUNION
     {({x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
     {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
     {z' | real q1 < z' /\ z' < real q2} |
     q0 IN Q_set} = BIGUNION
  (IMAGE (\q0. ({x' | real q0 < x' /\ x' <= t /\ real q2 < x'} CROSS
     {y' | y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < real q0}) CROSS
     {z' | real q1 < z' /\ z' < real q2} )
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );



val BIGUNION_event1_shared_spare_subevent = store_thm(
  "BIGUNION_event1_shared_spare_subevent",
  `` !t q2. BIGUNION
           {{(x',y') |
           y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\
           real q2 < x'} CROSS {z' | real q1 < z' /\ z' < real q2} |
           q1 IN Q_set}  =
	   {((x',y'),z') |
       y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN RW_TAC std_ss [LAMBDA_PAIR]
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
               x IN s <=>
            (?x'.
                  (FST x,T) =
                  (\(x',y').
                       ((x',y'),
                        y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\
                        x' <= t /\ real q2 < x')) x') /\ real q1 < SND x /\
             SND x < real q2``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)
          THEN `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN KNOW_TAC(``(\ (x':real,y').
             ((x',y'),  y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\
               real q2 < x')) x''
              = (x'', SND x'' < real q1 /\ 0 <= SND x'' /\ SND x'' <=t /\ SND x''< FST x'' /\ FST x''<=t /\ real q2 < FST x'') ``)
                THEN1 ( KNOW_TAC (`` ((\ ((x':real),y').
        ((x',y'),
         y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\
         real q2 < x')) x'') =  (\ (x':real,y').
        ((x',y'),
          y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\
         real q2 < x')) (FST x'' , SND x'')``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC
	 THEN Q.EXISTS_TAC(`(FST (FST (x:(real#real)#real)), SND (FST x), SND x) `)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]
THEN  KNOW_TAC(``?q1. q1 IN Q_set/\ Normal (FST (SND(x':real#real#real) )) < q1 /\  q1 < Normal (SND (SND x'))``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r1. q1 = Normal r1``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (FST (SND(x':real#real#real))) < Normal(SND  (SND x'))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{x| (SND (FST (x:(real#real)#real)) < real (Normal r1) /\ 0 <= SND (FST x) /\
                 SND (FST x) <= t /\ SND (FST x) < FST (FST x) /\
                 FST (FST x) <= t /\ real q2 < FST (FST x)) /\
                 real (Normal r1) < SND x /\ SND x < real q2}``)
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r1`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);

val BIGUNION_event1_shared_spare_subevent_measurable_set = store_thm(
  "BIGUNION_event1_shared_spare_subevent_measurable_set",
  `` !t q2.
	  {((x',y'),z') |
          y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\
          0 <= y' /\ y' <= t} IN
       measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,


RW_TAC std_ss [GSYM BIGUNION_event1_shared_spare_subevent]
THEN KNOW_TAC (``!q1. q1 IN Q_set ==> ({(x',y') |
     y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\ real q2 < x'} CROSS
     {z' | real q1 < z' /\ z' < real q2})   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[BIGUNION_event1_shared_spare_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
     {{(x',y') |
     y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\ real q2 < x'} CROSS
     {z' | real q1 < z' /\ z' < real q2} |
     q1 IN Q_set} = BIGUNION
  (IMAGE (\q1. {(x',y') |
     y' < real q1 /\ 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t /\ real q2 < x'} CROSS
     {z' | real q1 < z' /\ z' < real q2})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );



val BIGUNION_event1_shared_spare_subevent2 = store_thm(
  "BIGUNION_event1_shared_spare_subevent2",
  `` !t. BIGUNION {{((x',y'),z') |
          y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\
          0 <= y' /\ y' <= t} | q2 IN Q_set} =
	         {((x',y'),z') |
            y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}  ``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN RW_TAC std_ss [LAMBDA_PAIR]
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
               x IN s <=>
           ?x'.
                 (x,T) =
                 (\(x',y',z').
                      (((x',y'),z'),
                       y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\
                       x' <= t /\ 0 <= y' /\ y' <= t)) x'``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)


          THEN  `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN KNOW_TAC(``(\ (x':real,y',z').
              (((x',y'),z'),
                 y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\
               x' <= t /\ 0 <= y' /\ y' <= t)) x''
	       
               = ( ((FST(x'':real#real#real), FST (SND x'')),SND (SND x'')),
	       FST (SND x'') < SND (SND x'') /\ SND (SND x'') < real q2 /\ real q2 < FST  x'' /\ FST (SND x'') < FST  x'' /\ FST  x'' <= t /\ 0<= FST (SND x'') /\ FST (SND x'') <=t ) ``)

            THEN1 ( KNOW_TAC (``  (\ (x',y',z').
        (((x',y'),z'),
          y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\
         0 <= y' /\ y' <= t)) (x'':real#real#real) =  (\ (x',y',z').
        (((x',y'),z'),
         y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\
         0 <= y' /\ y' <= t)) (FST x'', FST (SND x'') , SND (SND x''))``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC

	 THEN Q.EXISTS_TAC(`(FST (FST (x:(real#real)#real)), SND (FST x), SND x) `)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]
THEN  KNOW_TAC(``?q2. q2 IN Q_set/\ q2 < Normal (FST  (x':real#real#real)) /\ Normal (SND (SND x')) < q2 ``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r2. q2 = Normal r2``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal(SND  (SND x')) <Normal  (FST  (x':real#real#real)) ``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{x:(real#real)#real|  SND (FST x) < SND  x /\
                       SND  x < real (Normal r2) /\ real (Normal r2) < FST (FST x) /\
                       SND (FST x) < FST(FST x) /\ FST (FST x) <= t /\
                       0 <= SND (FST x) /\ SND (FST x) <= t}``)
		       
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r2`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
          THEN Q.EXISTS_TAC(`(FST(FST (x'':(real#real)#real)) , SND (FST x''), SND x'') `)  
      
          THEN  FULL_SIMP_TAC std_ss[PAIR])
THEN RW_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[PAIR]);





val BIGUNION_event1_shared_spare_subevent2_measurable_set = store_thm(
  "BIGUNION_event1_shared_spare_subevent2_measurable_set",
  `` !t.
	  {((x',y'),z') |
             y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} IN
       measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,


RW_TAC std_ss [GSYM BIGUNION_event1_shared_spare_subevent2]
THEN KNOW_TAC (``!q2. q2 IN Q_set ==> ({((x',y'),z') |
     y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\
     0 <= y' /\ y' <= t})   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[BIGUNION_event1_shared_spare_subevent_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
     {{((x',y'),z') |
    y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\
     0 <= y' /\ y' <= t} |
     q2 IN Q_set} = BIGUNION
  (IMAGE (\q2. {((x',y'),z') |
     y' < z' /\ z' < real q2 /\ real q2 < x' /\ y' < x' /\ x' <= t /\
     0 <= y' /\ y' <= t})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );


val event1_shared_spare_Axy_measurable_set = store_thm(
  "event1_shared_spare_Axy_measurable_set",
  `` !t. {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
         measurable_sets (pair_measure lborel lborel)``,

RW_TAC std_ss[]
THEN KNOW_TAC(``  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  =
                  BIGUNION {{x'|  real q  < x':real /\ x'<= t }  CROSS {y'| 0<= y' /\ y' <= t /\ y':real <  real q }| q IN Q_set} ``)

     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EQ_TAC
                THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]	    
           	       THEN KNOW_TAC (``SND x < FST x /\ SND x <=t:real /\ 0<= SND x /\ FST x <= t   ``)
   		            THEN1(FULL_SIMP_TAC std_ss[ LAMBDA_PAIR])
           	       THEN DISCH_TAC
           	       THEN KNOW_TAC (``Normal (SND x) < Normal (FST x)``)
	                    THEN1(METIS_TAC [extreal_lt_eq])
                       THEN DISCH_TAC
	               THEN KNOW_TAC(``?q. q IN Q_set/\ Normal (SND x) < q /\  q < Normal (FST x)``)
	                    THEN1(METIS_TAC [Q_DENSE_IN_R])
                       THEN RW_TAC std_ss[]
	               THEN KNOW_TAC(``FST (x:real#real) = real (Normal(FST x))``)
                            THEN1(RW_TAC std_ss [GSYM real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``SND (x:real#real) = real(Normal (SND x))``)
	                    THEN1(METIS_TAC[real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``?r. q = Normal r``) 
                            THEN1(METIS_TAC[Q_not_infty])
  	               THEN RW_TAC std_ss[]
		       THEN Q.EXISTS_TAC (`{(x:real#real)| real (Normal r)  < FST x /\ FST x <= t /\
		                                           0 <= SND x /\ SND x <= t /\
							    SND x < real (Normal r)}`)
		       THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                       THEN Q.EXISTS_TAC(`Normal r`)     
                       THEN SRW_TAC[] [GSPECIFICATION, EXTENSION]
                       THEN EQ_TAC
                       THEN RW_TAC std_ss[])
	               THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                       THEN RW_TAC std_ss [LAMBDA_PAIR]
                       THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR] )
THEN DISCH_TAC	   
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!q. q IN Q_set ==> {x' | real q < x' /\ x' <= t} CROSS
      {y' | 0 <= y' /\ y' <= t /\ y' < real q} IN measurable_sets (pair_measure lborel lborel) ``)
     THEN1 (RW_TAC std_ss [pair_measure, measure_of, lborel,m_space_def, measurable_sets_def ]
           THEN1 (KNOW_TAC(``{x' | real q < x' /\ x' <= t} CROSS
     	   	 	      {y' | 0 <= y' /\ y' <= t /\ y' < real q} IN {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)

                  THEN1 (KNOW_TAC(`` {x' | real q < x' /\ x' <= t} IN subsets borel``) 
                         THEN1(KNOW_TAC(`` {x' | real q < x' /\ x' <= t} =
		                        {x' | real q < x'} INTER
				       {x' | x' <= t }``) 
                               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			             THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
			 THEN DISCH_TAC
			 THEN ONCE_ASM_REWRITE_TAC[]
                         THEN `{x' | real q< x'} IN subsets borel /\
				       {x' | x' <= t } IN subsets borel `
			    by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT] 	        
                        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
	   THEN DISCH_TAC
           THEN KNOW_TAC (`` {y' | 0 <= y' /\ y' <= t /\ y' < real q}  IN subsets borel ``)
                THEN1(`{y' | 0 <= y' /\ y' <= t /\ y' < real q}  =
	                  {y' | 0 <= y'} INTER
	                  {y' |  y' <= t} INTER
			  {y' | y' < real q}` by (SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC])
	                THEN `{y' | 0 <= y'} IN subsets borel /\
		              {y' |  y' <= t} IN subsets borel /\
			      {y' | y' < real q} IN subsets borel`   by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
		        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
          THEN DISCH_TAC
          THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	  THEN Q.EXISTS_TAC (`({x' | real q < x' /\ x' <= t},{y' | 0 <= y' /\ y' <= t /\ y' < real q})   `)
	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
		 
         THEN DISCH_TAC
         THEN METIS_TAC [sigma_sets_basic])
         THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
     THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
     THEN KNOW_TAC(`` {a CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET POW univ(:real#real)``)
          THEN1( RW_TAC std_ss [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
     THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN KNOW_TAC(``BIGUNION
     {{x' | real q < x' /\ x' <= t} CROSS
     {y' | 0 <= y' /\ y' <= t /\ y' < real q} |
     q IN Q_set} = BIGUNION
  (IMAGE (\q. {x' | real q < x' /\ x' <= t} CROSS
     {y' | 0 <= y' /\ y' <= t /\ y' < real q})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN MP_TAC(Q.ISPECL[`pair_measure lborel lborel ` ,` (\q.
             {x' | real q < x' /\ x' <= t} CROSS
     {y' | 0 <= y' /\ y' <= t /\ y' < real q})` ] MEASURE_SPACE_BIGUNION_Q)
THEN RW_TAC std_ss[]);




val event1_shared_spare_indicator_of_indicator_Axy = store_thm(
  "event1_shared_spare_indicator_of_indicator_Axy",
  `` ! x y z (t:real).
        indicator_fn  {((x',y'),z') |
         y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} ((x,y),z) =
	indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)*
	indicator_fn {z' | y < z' /\ z' < x} z  ``,

RW_TAC std_ss [indicator_fn_def]
      THEN1(RW_TAC std_ss[mul_lone])
      THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

      THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
        y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
           y'' < z'' /\ z'' < x'' /\ y'' < x'' /\ x'' <= t /\ 0 <= y'' /\ y'' <= t) x' y' z'}` by RW_TAC std_ss[]
      THEN `(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}<=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
 THEN RW_TAC std_ss [mul_rzero, mul_lzero]);
   



val joint_density_3rv_1 = store_thm("joint_density_3rv_1",

``!p X Y Z  f_xy t.
     prob_space p /\ random_variable Z p borel /\
     (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
     distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy  ==>
     (pos_fn_integral
        (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
        (\ (x,y).
           pos_fn_integral (distr p lborel Z)
             (\z. indicator_fn {((x',y'),z') |
            y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} ((x,y),z))) =
      pos_fn_integral lborel
        (\y.
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {(x',y')|  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                pos_fn_integral (distr p lborel Z)
                  (\z. indicator_fn {z' |y < z' /\ z' <x} z))))``,

RW_TAC std_ss[distributed_def]
THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
			       THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                     THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
                               THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]

THEN MP_TAC(Q.ISPECL[` f_xy:(real#real)->extreal `, `(\ (x:real,y:real).
     pos_fn_integral (distr p lborel Z)
       (\z. indicator_fn {((x',y'),z') |
                    y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t:real} ((x,y),z)))` , `(pair_measure lborel lborel)` ]  pos_fn_integral_density_1)
THEN RW_TAC std_ss[measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]  THEN UNDISCH_TAC (``f_xy IN
      measurable
        (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel)) Borel``)
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``{((x',y'),z') |
                   y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} IN measurable_sets (pair_measure (pair_measure lborel lborel)lborel) ``)
    THEN1(RW_TAC std_ss [BIGUNION_event1_shared_spare_subevent2_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC (`` {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
          measurable_sets (pair_measure lborel lborel)``)
	  THEN1(RW_TAC std_ss[event1_shared_spare_Axy_measurable_set])
THEN DISCH_TAC
THEN `!x y z. indicator_fn
            {((x',y'),z') |
           y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t}
            ((x,y),z) =
          indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) * indicator_fn {z' | y < z' /\ z' < x} z` by RW_TAC std_ss[event1_shared_spare_indicator_of_indicator_Axy]
THEN KNOW_TAC(`` !x y. {z' | y < z' /\ z' < x:real} IN measurable_sets lborel``)
     THEN1( `measurable_sets lborel = subsets borel` by METIS_TAC [borel , lborel, subsets_def, measurable_sets_def]
          THEN ONCE_ASM_REWRITE_TAC[]
	  THEN `!x y . {z' | y < z' /\ z' < x} = {z' | y < z'} INTER {z'| z' < x:real}` by SRW_TAC  [] [GSPECIFICATION, EXTENSION]
	  THEN `!x y. {z' | y < z'} IN subsets borel  /\ {z' | z' < x} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
          THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC	  

THEN KNOW_TAC(``(\(x:real,y:real).  pos_fn_integral (distr p lborel Z) (\z. indicator_fn {((x',y'),z') |
                   y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} ((x,y),z:real))) IN
                                   measurable (m_space (pair_measure lborel lborel),
                                   measurable_sets (pair_measure lborel lborel)) Borel``)

     THEN1( MP_TAC(Q.ISPECL[` indicator_fn ({((x',y'),z') |
                     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t:real}) `, `pair_measure lborel lborel`, `distr p lborel (Z:'a->real)` ] borel_measurable_pos_integral_fst)
             THEN RW_TAC std_ss [measure_space_pair_measure', sigma_finite_measure_lborel]
	     THEN `measure_space (pair_measure lborel lborel)` by METIS_TAC [measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]
	     THEN FULL_SIMP_TAC std_ss[]
	     THEN `measure_space (distr p lborel (Z:'a->real))` by 
	               (FULL_SIMP_TAC std_ss [random_variable_def]
                          
		         THEN KNOW_TAC(``(Z:'a->real) IN measurable (m_space p, measurable_sets p)
			                (m_space lborel, measurable_sets lborel)``)
			      THEN1( UNDISCH_TAC(``(Z:'a->real) IN measurable (p_space p,events p) (m_space lborel, measurable_sets lborel)``)
		         	      THEN DISCH_TAC
			              THEN METIS_TAC [measure_space_distr, measure_space_lborel, prob_space_def, random_variable_def, p_space_def, events_def, measurable_sets_def, subsets_def, borel, lborel, m_space_def, space_def, space_borel])
			THEN  METIS_TAC [measure_space_distr, measure_space_lborel, prob_space_def, random_variable_def, p_space_def, events_def, measurable_sets_def, subsets_def, borel, lborel, m_space_def, space_def, space_borel])
	      THEN FULL_SIMP_TAC std_ss[]
              	      
	      THEN KNOW_TAC(`` sigma_finite_measure (distr p lborel (Z:'a->real)) ``)
		   THEN1(METIS_TAC[sigma_finite_measure_distr, random_variable_def])
              THEN DISCH_TAC
              THEN FULL_SIMP_TAC std_ss[]
              THEN KNOW_TAC(``(!(x:(real#real)#real). 0 <= indicator_fn {((x',y'),z') |
                     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} x)``)  
                   THEN1(METIS_TAC [indicator_fn_pos_le])
              THEN DISCH_TAC
	      THEN FULL_SIMP_TAC std_ss[]
              THEN KNOW_TAC(``indicator_fn {((x',y'),z') |
                     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} IN
                            measurable (m_space (pair_measure (pair_measure lborel lborel)(distr p lborel (Z:'a->real))),
                            measurable_sets (pair_measure (pair_measure lborel lborel)(distr p lborel Z))) Borel``)
                  THEN1(MP_TAC(Q.ISPECL[`(m_space (pair_measure (pair_measure lborel lborel) (distr p lborel (Z:'a->real))),
    measurable_sets(pair_measure (pair_measure lborel lborel) (distr p lborel Z))) ` ,`1:real` ,`{((x',y'),z') |
                     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t:real}` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)
 
                        THEN RW_TAC std_ss [ REAL_LE_01]
	                THEN KNOW_TAC(``m_space (pair_measure (pair_measure lborel lborel)(distr p lborel Z)) =
			                  m_space (pair_measure (pair_measure lborel lborel) lborel)``)
	                       THEN1(RW_TAC std_ss [distr, m_space_def, pair_measure, measure_of])
	                THEN DISCH_TAC
	                THEN FULL_SIMP_TAC std_ss[]
	                THEN KNOW_TAC(``measurable_sets (pair_measure (pair_measure lborel lborel) (distr p lborel Z)) =
			                 measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
                             THEN1(REWRITE_TAC[m_space_def, distr, measurable_sets_def, pair_measure, measure_of, subsets_def, lborel])
	               THEN DISCH_TAC
	               THEN FULL_SIMP_TAC std_ss[]
	               THEN `sigma_algebra (m_space (pair_measure (pair_measure lborel lborel) lborel),
                             measurable_sets (pair_measure (pair_measure lborel lborel) lborel))`
			      by RW_TAC std_ss[sigma_algebra_pair_pair_lborel]
	               THEN FULL_SIMP_TAC std_ss[]
	  
                       THEN ` {((x',y'),z') |
                    y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t:real} IN subsets (m_space (pair_measure (pair_measure lborel lborel) lborel),
                            measurable_sets (pair_measure (pair_measure lborel lborel) lborel))`
			    by METIS_TAC[pair_measure, lborel, measure_of, subsets_def, m_space_def, measurable_sets_def]
	               THEN FULL_SIMP_TAC std_ss[]
                       THEN KNOW_TAC(``!(x: (real#real)#real). x = ((FST (FST x),SND (FST x)), SND x) ``)
                           THEN1(METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])
                       THEN DISCH_TAC     
                       THEN KNOW_TAC(``(\ (x:(real#real)#real). Normal 1 * indicator_fn {((x',y'),z') |
                       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} x) = (\x. indicator_fn {((x',y'),z') |
                       y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} x)``)
                            THEN1(METIS_TAC [extreal_of_num_def, mul_lone, PAIR, LAMBDA_PROD])
                       THEN DISCH_TAC
                       THEN MP_TAC(Q.ISPECL [`indicator_fn ({((x',y'),z') |
                    y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t}:(real#real)#real->bool) `]LAMBDA_PROD)
		       THEN FULL_SIMP_TAC std_ss[]
                       THEN METIS_TAC [PAIR, LAMBDA_PROD, ETA_THM])
             THEN DISCH_TAC
             THEN FULL_SIMP_TAC std_ss[]
             THEN KNOW_TAC(``!(x:real#real). x IN m_space (pair_measure lborel lborel)``)
                  THEN1(REWRITE_TAC [m_space_def, pair_measure, measure_of, lborel]
                        THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CROSS_DEF]
                        THEN ONCE_ASM_REWRITE_TAC[]
	                THEN METIS_TAC [IN_UNIV])
             THEN DISCH_TAC
             THEN FULL_SIMP_TAC std_ss[]
             THEN KNOW_TAC(``!(x:real#real) ( y:real). max 0 (indicator_fn {((x',y'),z') |
                     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} (x,y)) = indicator_fn {((x',y'),z') |
                     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} (x,y) ``)
                  THEN1(RW_TAC std_ss [extreal_max_def])
             THEN DISCH_TAC     
             THEN FULL_SIMP_TAC std_ss[]
             THEN KNOW_TAC(``(!(x:real#real) (y:real). pos_fn_integral (distr p lborel Z) (\y:real. indicator_fn {((x',y'),z') |
                    y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} (x,y)) = 
                               pos_fn_integral (distr p lborel Z) (\y. indicator_fn {((x',y'),z') |
                     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} ((FST x, SND x),y)))``)
                  THEN1(METIS_TAC [PAIR, ETA_THM, LAMBDA_PROD])
             THEN DISCH_TAC
             THEN UNDISCH_TAC (`` (\x. pos_fn_integral (distr p lborel (Z:'a->real))(\y. indicator_fn {((x',y'),z') |
                    y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} (x,y))) IN
                                   measurable (m_space (pair_measure lborel lborel),
                                   measurable_sets (pair_measure lborel lborel)) Borel``)
             THEN ONCE_ASM_REWRITE_TAC[]
             THEN FULL_SIMP_TAC std_ss[PAIR, LAMBDA_PROD, ETA_THM])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!(x:real#real). 0 <= (\(x,y). pos_fn_integral (distr p lborel (Z:'a->real))
                                             (\z. indicator_fn {((x',y'),z') |
                    y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                   y' <= t} ((x,y),z))) x``)

     THEN1(RW_TAC std_ss[]

           THEN KNOW_TAC(``(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                                         (\z. indicator_fn  {z' | y < z' /\ z' < x} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) x =
                               ( pos_fn_integral (distr p lborel Z) (\z. indicator_fn  {z' | SND x < z' /\ z' < FST x} z * indicator_fn   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x))``)
 
                THEN1(`x:real#real = (FST x, SND x)` by RW_TAC std_ss [PAIR]
                       THEN ONCE_ASM_REWRITE_TAC[]
                       THEN RW_TAC std_ss[PAIR, ETA_THM, LAMBDA_PROD])
           THEN DISCH_TAC
           THEN FULL_SIMP_TAC std_ss[]
           THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`, ` (\z:real. indicator_fn {z' | SND x < z' /\ z' < FST x} z * indicator_fn  {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real))  `]  pos_fn_integral_pos)
           THEN RW_TAC std_ss[indicator_fn_pos_le]
           THEN `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC [measure_space_distr,random_variable_def, measure_space_lborel, prob_space_def, p_space_def, events_def, measurable_sets_def, subsets_def, space_def]
           THEN FULL_SIMP_TAC std_ss[]
           THEN KNOW_TAC(`` pos_fn_integral (distr p lborel Z) (\z:real. indicator_fn {z' | SND x < z' /\ z' < FST x} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) =
	                     pos_fn_integral (distr p lborel Z) (\z:real. indicator_fn {z' | SND x < z' /\ z' < FST x} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST (x:real#real),SND x) )``)
               THEN1(METIS_TAC[PAIR])
	   THEN DISCH_TAC
           THEN

               KNOW_TAC(``pos_fn_integral (distr p lborel Z)
	                      (\z:real. indicator_fn {z' | SND x < z' /\ z' < FST x} z* indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) =
                                    pos_fn_integral (distr p lborel Z)
                     (\z. indicator_fn {z' | SND x < z' /\ z' < FST x} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x ,SND x) )``)
                THEN1(METIS_TAC [PAIR, LAMBDA_PROD] )
     
           THEN DISCH_TAC
	   THEN KNOW_TAC(``x:real#real = (FST x, SND x)``)
                THEN1(METIS_TAC [PAIR])
	   THEN DISCH_TAC
           THEN ONCE_ASM_REWRITE_TAC[]
	   THEN KNOW_TAC(``(!x':real. 0 <= indicator_fn {z' | SND x < z' /\ z' < FST x} x' * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) ``)
	        THEN1 (METIS_TAC [le_mul, indicator_fn_pos_le])
           THEN DISCH_TAC
	   THEN FULL_SIMP_TAC std_ss[PAIR, mul_comm] )

THEN DISCH_TAC
THEN KNOW_TAC(``(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
       (\z.indicator_fn {z' |  y < z' /\ z' < x} z * indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) ))  =
                  (\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
       (\z. indicator_fn {((x',y'),z') |
               y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}((x,y),z))) ``)
     THEN1(RW_TAC std_ss[PAIR, LAMBDA_PROD, ETA_THM, pair_exists, mul_comm])
THEN DISCH_TAC
THEN KNOW_TAC (``(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
            (\z.indicator_fn {z' | y < z' /\ z' < x} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) IN
                   measurable (m_space (pair_measure lborel lborel),
                   measurable_sets (pair_measure lborel lborel)) Borel``)

      THEN1(METIS_TAC[PAIR, LAMBDA_PROD, ETA_THM, pair_exists])
THEN DISCH_TAC
THEN KNOW_TAC(``(!x. 0 <=  (\ (x:real,y:real).pos_fn_integral (distr p lborel (Z:'a->real))
               (\z. indicator_fn {z' | y < z' /\ z' < x} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) x) ``)
     THEN1(METIS_TAC[])
THEN DISCH_TAC     
THEN WEAKEN_TAC (equal(Term`(\ (x:real,y:real).  pos_fn_integral (distr p lborel (Z:'a->real))
            (\z. indicator_fn  {z' | y < z' /\ z' < x} z * indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) =
       (\(x,y). pos_fn_integral (distr p lborel Z) (\z. indicator_fn {((x',y'),z') |
                y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                       y' <= t} ((x,y),z)))`))

THEN FULL_SIMP_TAC std_ss[]       
THEN KNOW_TAC(``pos_fn_integral (density (pair_measure lborel lborel) f_xy)
         (\x.(\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z. indicator_fn {z' | y < z' /\ z' < x}  z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) x) =
		    pos_fn_integral (density (pair_measure lborel lborel) f_xy)
                     (\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z.indicator_fn {z' | y < z' /\ z' < x} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) )) ``)
     THEN1(METIS_TAC[ETA_ETA_THM_PAIR])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``pos_fn_integral (pair_measure lborel lborel)
         (\x. f_xy x * (\ (x:real,y:real). pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z. indicator_fn {z' | y < z' /\ z' < x} z * indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x,y))) x) =
		    pos_fn_integral (pair_measure lborel lborel)
           (\ (x:real,y:real). f_xy (x,y) * pos_fn_integral (distr p lborel Z)
                 (\z. indicator_fn {z' | y < z' /\ z' < x} z * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) ``)
      THEN1(RW_TAC std_ss[ETA_ETA_THM_PAIR, PAIR, ETA_THM, LAMBDA_PROD, mul_comm])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]



THEN KNOW_TAC(``(pos_fn_integral (density (pair_measure lborel lborel) f_xy)
        (\ (x,y). pos_fn_integral (distr p lborel Z) (\z. indicator_fn {((x',y'),z') |
               y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                       y' <= t} ((x,y),z))) =
	        pos_fn_integral (pair_measure lborel lborel) (\ (x,y).  f_xy (x,y) *
		    pos_fn_integral (distr p lborel Z) (\z.  indicator_fn {((x',y'),z') |
                y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                       y' <= t} ((x,y),z) ) ))``)

      THEN1(FULL_SIMP_TAC std_ss [PAIR, ETA_THM, LAMBDA_PROD, mul_comm])

THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``  pos_fn_integral (density (pair_measure lborel lborel) f_xy)
     (\ (x,y).
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn
                   {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                 indicator_fn {z' | y < z' /\ z' < x} z))  =  pos_fn_integral (pair_measure lborel lborel)
           (\ (x,y).
                f_xy (x,y) *
                pos_fn_integral (distr p lborel Z)
                  (\z.
                        indicator_fn {z' | y < z' /\ z' < x} z *
                            indicator_fn
                              {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)))  ``)

      THEN1(FULL_SIMP_TAC std_ss[mul_comm])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` (\ (x:real,y:real).
          f_xy (x,y) *
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | y < z' /\ z' < x} z *
                 indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y))) =
		   (\ (x:real,y:real).
          f_xy (x,y) * indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | y < z' /\ z' < x} z  ))``)
      THEN1(KNOW_TAC(``   !(x:real) (y:real).
          
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | y < z' /\ z' < x} z *
                 indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)) =
		  
        indicator_fn
                   {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
          pos_fn_integral (distr p lborel Z)
            (\z.
                 indicator_fn {z' | y < z' /\ z' < x} z  ) ``)
            THEN1( RW_TAC std_ss[]
                  THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`, `(\ (x:real,y:real). real (indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y)) )` ,`indicator_fn {z' | y:real < z' /\ z' < x} `] pos_fn_integral_fun_mul1)
                 THEN RW_TAC std_ss []
                 THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                 THEN `!x. x IN m_space (distr p lborel (Z:'a->real))` by METIS_TAC [distr,lborel, m_space_def, IN_UNIV]
                 THEN `(!x'. 0 <= indicator_fn {z' | y:real < z' /\ z' < x:real} x')` by RW_TAC std_ss [indicator_fn_pos_le]
		 THEN FULL_SIMP_TAC std_ss [PAIR, LAMBDA_PROD]
                 THEN FULL_SIMP_TAC std_ss[GSYM PAIR, GSYM LAMBDA_PROD]
                 THEN  KNOW_TAC(``(!p:real#real. 0 <= real (indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  p))``)  
                       THEN1(RW_TAC std_ss [real_def, indicator_fn_def]
           	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(SELECT_ELIM_TAC
                             THEN RW_TAC std_ss[]
                                   THEN1( Q.EXISTS_TAC `1`
		                        THEN RW_TAC std_ss[normal_real, extreal_of_num_def] )
		                        THEN  `x':real = 1` by METIS_TAC[extreal_11, extreal_of_num_def]
                         	         THEN METIS_TAC [REAL_LE_01])
                            THEN SELECT_ELIM_TAC		 
	                    THEN RW_TAC std_ss[]		    
                                THEN1(Q.EXISTS_TAC`0`
                                     THEN RW_TAC std_ss [normal_real, extreal_of_num_def])
	                    THEN  `x':real = 0` by METIS_TAC [extreal_11, extreal_of_num_def]
                            THEN METIS_TAC [REAL_LE_REFL])
		 THEN DISCH_TAC	    
                 THEN FULL_SIMP_TAC std_ss[]
		 THEN KNOW_TAC(``!(x':real#real). Normal (real (indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x':real#real))) = indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x'``)  
          THEN1(RW_TAC std_ss[]
          THEN `indicator_fn  {(x',y') |
                             0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x':real#real) <> PosInf` by 
                     (RW_TAC std_ss [indicator_fn_def]
	             THEN1(METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
	             THEN METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
	  THEN `indicator_fn  {(x',y') |
                             0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x':real#real) <> NegInf` by 
                     (RW_TAC std_ss [indicator_fn_def]
	             THEN1(METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
	             THEN METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
 	  THEN METIS_TAC [normal_real])
    THEN DISCH_TAC
    THEN FULL_SIMP_TAC std_ss[]
    THEN FULL_SIMP_TAC std_ss[mul_comm])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss [mul_comm, mul_assoc])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN  MP_TAC(Q.ISPECL[`(\ (x:real,y:real). f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) * pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {z' | y < z' /\ z' < x}  z  ))`, ` lborel` ,`lborel`] pos_fn_integral_snd  )
THEN RW_TAC std_ss [measure_space_lborel, sigma_finite_measure_lborel]
THEN KNOW_TAC(``(!(x:real#real).
          0 <= (\ (x:real,y:real). f_xy (x,y) * indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) * 
             pos_fn_integral (distr p lborel (Z:'a->real))
               (\z. indicator_fn  {z' | y < z' /\ z' < x} z )) x)``)
     THEN1(RW_TAC std_ss[]
           THEN `(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
           THEN ONCE_ASM_REWRITE_TAC[]
           THEN KNOW_TAC(``(\ (x:real,y:real). f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
	                pos_fn_integral (distr p lborel Z)
			(\z:real. indicator_fn  {z' | y < z' /\ z' < x} z ))(FST x,SND x) =
            ( f_xy (x:real#real) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x, SND x) *
	    pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn  {z' | SND x< z' /\ z' < FST x} z))``)
                THEN1(RW_TAC std_ss [PAIR, ETA_THM, LAMBDA_PROD])
           THEN DISCH_TAC
           THEN ONCE_ASM_REWRITE_TAC[]
	   THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                
	   THEN  MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`,`(\z:real. indicator_fn  {z' | SND (x:real#real) < z' /\ z' < FST x} z )  `]    pos_fn_integral_pos  )
           
	   THEN  RW_TAC std_ss [measure_space_distr, measure_space_lborel, prob_space_def, le_mul, indicator_fn_pos_le] )
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN WEAKEN_TAC (equal(Term`!x:real#real.
         0 <=
         (\(x,y).
            f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
            pos_fn_integral (distr p lborel Z) (\z. indicator_fn {z' | y < z' /\ z' < x} z))
           x`))

THEN KNOW_TAC(``(\ (x:real,y:real).
          f_xy (x,y) * indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *  pos_fn_integral (distr p lborel (Z:'a->real))
            (\z:real. indicator_fn  {z' | y < z' /\ z' < x} z) ) IN
            measurable (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel``)

    THEN1( KNOW_TAC (``(\ (x,y).
        f_xy (x,y) *
        indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
          (x,y) *
        pos_fn_integral (distr p lborel Z)
          (\z. indicator_fn {z' | y < z' /\ z' < x} z)) IN
   measurable
     (m_space (pair_measure lborel lborel),
      measurable_sets (pair_measure lborel lborel)) Borel
    =  (\ (x,y).
        f_xy (x,y) *
         pos_fn_integral (distr p lborel Z)
          (\z.  indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
          (x,y)* indicator_fn {z' | y < z' /\ z' < x} z)) IN
   measurable
     (m_space (pair_measure lborel lborel),
      measurable_sets (pair_measure lborel lborel)) Borel``)
          THEN1(RW_TAC std_ss [mul_comm, mul_assoc])
	  THEN DISCH_TAC
	  THEN ASM_REWRITE_TAC[]
	  THEN MP_TAC(Q.ISPECL[`(m_space (pair_measure lborel lborel),
           measurable_sets (pair_measure lborel lborel))`
	   , `(f_xy:real#real->extreal) `,
	   ` (\ (x:real,y:real).  pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x,y)*  indicator_fn {z' | y < z' /\ z' < x} z))`, `(\ (x:real,y:real). f_xy (x:real,y:real)  * pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x,y) * indicator_fn {z' | y < z' /\ z' < x}  z))` ]  IN_MEASURABLE_BOREL_MUL)
	   
           THEN RW_TAC std_ss[measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel, sigma_algebra_pair_lborel]
            THEN KNOW_TAC(``!(x:real#real).
                x IN  space  (m_space (pair_measure lborel lborel),
                measurable_sets (pair_measure lborel lborel))``)
                 THEN1(REWRITE_TAC [space_def, pair_measure, measure_of, m_space_def, measurable_sets_def, lborel]
		       THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CROSS_DEF]
		       THEN ONCE_ASM_REWRITE_TAC[]
		       THEN METIS_TAC [IN_UNIV])
            THEN DISCH_TAC		       
            THEN FULL_SIMP_TAC std_ss[]
	    THEN KNOW_TAC(``!(x:real#real). f_xy x <> NegInf``)
                 THEN1(RW_TAC std_ss[]
		      THEN METIS_TAC [extreal_of_num_def,extreal_lt_def, extreal_le_def, le_lt, lt_le,lt_infty,le_infty])
            THEN DISCH_TAC
	    THEN FULL_SIMP_TAC std_ss[]
            THEN KNOW_TAC(``!x.
              (\ (x,y).
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                            0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | y < z' /\ z' < x} z)) x <>
              NegInf``)
	      THEN1( RW_TAC std_ss[PAIR, LAMBDA_PROD]
	             THEN KNOW_TAC(`` (\ (x,y).
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
               indicator_fn {z' | y < z' /\ z' < x} z)) x = 
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x, SND x) *
               indicator_fn {z' | SND x < z' /\ z' < FST x} z) ``)
	       THEN1(`(x:real#real) = (FST x , SND x)` by RW_TAC std_ss[PAIR, LAMBDA_PROD]
	             THEN ONCE_ASM_REWRITE_TAC[]
		     THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	       THEN DISCH_TAC
	       THEN ONCE_ASM_REWRITE_TAC[]
	       THEN RW_TAC std_ss [PAIR]
	       THEN `
	       pos_fn_integral (distr p lborel Z)
     (\z.
          indicator_fn {(x',y') |0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x *
          indicator_fn {z' | SND x < z' /\ z' < FST x} z) =
	      pos_fn_integral (distr p lborel Z)
     (\z. indicator_fn
               {((x',y'),z') |
              y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\
                         0 <= y' /\ y' <= t}
               ((FST x, SND x),z))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	      THEN ONCE_ASM_REWRITE_TAC[]
	      THEN `!(x:real#real) z. 0<= indicator_fn
            {((x',y'),z') | y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\
                         0 <= y' /\ y' <= t}
            ((FST x,SND x),z)` by  METIS_TAC[indicator_fn_pos_le]
              THEN `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
               THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`,`(\z:real. indicator_fn
            {((x',y'),z') |
           y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\
                         0 <= y' /\ y' <= t}
            ((FST x,SND x),z))  `]  pos_fn_integral_pos)
           THEN DISCH_TAC
	   THEN FULL_SIMP_TAC std_ss[indicator_fn_pos_le]
           THEN  METIS_TAC [extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def, mul_extreal_not_infty])
     THEN DISCH_TAC
     THEN FULL_SIMP_TAC std_ss[]
     THEN KNOW_TAC (`` !x.
              (\ (x,y).
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                             0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | y < z' /\ z' < x} z)) x <> PosInf``)

          THEN1(RW_TAC std_ss[]
	        THEN  KNOW_TAC(`` (\ (x,y).
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
               indicator_fn {z' | y < z' /\ z' < x} z)) x = 
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (FST x, SND x) *
               indicator_fn {z' | SND x < z' /\ z' < FST x} z) ``)
	       THEN1(`(x:real#real) = (FST x , SND x)` by RW_TAC std_ss[PAIR, LAMBDA_PROD]
	             THEN ONCE_ASM_REWRITE_TAC[]
		     THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	       THEN DISCH_TAC
	       THEN ONCE_ASM_REWRITE_TAC[]
	       THEN RW_TAC std_ss [PAIR]
               THEN KNOW_TAC(``pos_fn_integral (distr p lborel Z)
     (\z.
          indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x *
          indicator_fn {z' | SND x < z' /\ z' < FST x} z) =
	  indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x*
	  pos_fn_integral (distr p lborel Z)
         (\z. indicator_fn {z' | SND x < z' /\ z' < FST x} z) ``)
                 THEN1(RW_TAC std_ss[]
                  THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`, `(\ (x':real,y:real). real (indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real)) )` ,`indicator_fn {z' | SND ( x:real#real) < z' /\ z' < FST x} `] pos_fn_integral_fun_mul1)
                 THEN RW_TAC std_ss []
                 THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                 THEN `!x. x IN m_space (distr p lborel (Z:'a->real))` by METIS_TAC [distr,lborel, m_space_def, IN_UNIV]
		 THEN FULL_SIMP_TAC std_ss[]
                 THEN `(!x'. 0 <= indicator_fn {z' | SND (x:real#real) < z' /\ z' < FST x} x')` by RW_TAC std_ss [indicator_fn_pos_le]
		 THEN FULL_SIMP_TAC std_ss [PAIR, LAMBDA_PROD]
                 THEN FULL_SIMP_TAC std_ss[GSYM PAIR, GSYM LAMBDA_PROD]
                 THEN  KNOW_TAC(``(0 <= real (indicator_fn  {(x',y') |
                               0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}  (x:real#real)))``)  
                       THEN1(RW_TAC std_ss [real_def, indicator_fn_def]
           	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(RW_TAC std_ss [REAL_LE_REFL])
	   	       THEN1(SELECT_ELIM_TAC
                             THEN RW_TAC std_ss[]
                                   THEN1( Q.EXISTS_TAC `1`
		                        THEN RW_TAC std_ss[normal_real, extreal_of_num_def] )
		                        THEN  `x':real = 1` by METIS_TAC[extreal_11, extreal_of_num_def]
                         	         THEN METIS_TAC [REAL_LE_01])
                            THEN SELECT_ELIM_TAC		 
	                    THEN RW_TAC std_ss[]		    
                                THEN1(Q.EXISTS_TAC`0`
                                     THEN RW_TAC std_ss [normal_real, extreal_of_num_def])
	                    THEN  `x':real = 0` by METIS_TAC [extreal_11, extreal_of_num_def]
                            THEN METIS_TAC [REAL_LE_REFL])
		 THEN DISCH_TAC	    
                 THEN FULL_SIMP_TAC std_ss[]
		 THEN KNOW_TAC(``!(x:real#real). Normal (real (indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real))) = indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} x``)  
          THEN1(RW_TAC std_ss[]
          THEN `indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x':real#real) <> PosInf` by 
                     (RW_TAC std_ss [indicator_fn_def]
	             THEN1(METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
	             THEN METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
	  THEN `indicator_fn  {(x',y') |
                              0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x':real#real) <> NegInf` by 
                     (RW_TAC std_ss [indicator_fn_def]
	             THEN1(METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
	             THEN METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty])
 	  THEN METIS_TAC [normal_real])
    THEN DISCH_TAC
    THEN FULL_SIMP_TAC std_ss[])
    THEN DISCH_TAC
    THEN ONCE_ASM_REWRITE_TAC[]
    THEN `!x. indicator_fn  {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real) <> PosInf /\
		            indicator_fn  {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x:real#real) <> NegInf` by 
			        (RW_TAC std_ss [indicator_fn_def]
			        THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def])

          THEN KNOW_TAC(``  0<=  pos_fn_integral (distr p lborel (Z:'a->real))
          (\z. indicator_fn {z' | SND (x:real#real) < z' /\ z' < FST x} z) ``)
              THEN1(RW_TAC std_ss[]
         

           THEN MP_TAC(Q.ISPECL[`distr p lborel (Z:'a->real)`,`(\z:real. indicator_fn {z' | SND (x:real#real) < z' /\ z' < FST x}  z )  `]  pos_fn_integral_pos)
           THEN DISCH_TAC
	   THEN  `measure_space (distr p lborel (Z:'a->real))` by METIS_TAC[measure_space_distr, prob_space_def, measure_space_lborel, random_variable_def, p_space_def, events_def] 
                
           THEN FULL_SIMP_TAC std_ss [indicator_fn_pos_le])
	   THEN DISCH_TAC
	   THEN  ` pos_fn_integral (distr p lborel (Z:'a->real)) (\z:real. indicator_fn {z' | SND (x:real#real) < z' /\ z' < FST x } z) <> NegInf`
                      by  METIS_TAC [extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def, mul_extreal_not_infty]
	   THEN KNOW_TAC(`` pos_fn_integral (distr p lborel (Z:'a->real)) (\z:real. indicator_fn {z' | SND (x:real#real)< z' /\ z' < FST x} z) <> PosInf``)THEN1 
		                  (RW_TAC std_ss[]
				   THEN KNOW_TAC(``
		                  pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {z' | SND (x:real#real) < z' /\ z' < FST x} z) = measure (distr p lborel Z) {z' | SND x < z' /\ z' < FST x}``)
                                THEN1(RW_TAC std_ss [measure_def, distr, GSYM distribution_def]
               	                      THEN RW_TAC std_ss [GSYM prob_def, GSYM p_space_def]
		      		      THEN `prob p (PREIMAGE (Z:'a->real) {z' | SND (x:real#real) < z' /\ z' < FST x} INTER p_space p) =
			           distribution p Z {z' | SND x < z' /\ z' < FST x}`by METIS_TAC[GSYM distribution_def]
                      		   THEN ONCE_ASM_REWRITE_TAC[]
                      		   THEN `distribution p (Z:'a->real) {z' | SND x < z' /\ z' < FST (x:real#real)} =
			           pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn {z' | SND (x:real#real) < z' /\ z' < FST x} z)` by METIS_TAC [distribution_lebesgue_thm2_pos_fn, measure_space_lborel]
		                   THEN ONCE_ASM_REWRITE_TAC[]
                      		   THEN RW_TAC std_ss [GSYM distribution_def]  
                      		   THEN `(p_space lborel,measurable_sets lborel,distribution p (Z:'a->real)) = distr p lborel Z` by 
                                         (RW_TAC std_ss[distr, p_space_def, distribution_def, prob_def])
		                   THEN ONCE_ASM_REWRITE_TAC[]
		      		   THEN RW_TAC std_ss[]  )
	                  THEN DISCH_TAC  
        		  THEN KNOW_TAC(``measure (distr p lborel (Z:'a->real)) {z' | SND (x:real#real) < z' /\ z' < FST x} <> PosInf``)
	     		       THEN1(`sigma_finite_measure (distr p lborel (Z:'a->real))` by RW_TAC std_ss [sigma_finite_measure_distr]
	                             THEN KNOW_TAC(``{z' | SND (x:real#real) < z' /\ z' < FST x} IN measurable_sets (distr p lborel (Z:'a->real))``)
	   			          THEN1(RW_TAC std_ss[distr, measurable_sets_def])
		                    THEN DISCH_TAC
		    		    THEN WEAKEN_TAC (equal(Term` pos_fn_integral (distr p lborel Z) (\z. indicator_fn {z' | SND (x:real#real) < z' /\ z' < FST x} z) =  measure (distr p lborel (Z:'a->real)) {z' | SND (x:real#real) < z' /\ z' < FST x} `))
                                   THEN RW_TAC std_ss [distr, measure_def]
		                   THEN RW_TAC std_ss [GSYM prob_def, GSYM p_space_def]
		                   THEN KNOW_TAC(``PREIMAGE (Z:'a->real) {z' | SND (x:real#real) < z' /\ z' < FST x} INTER p_space p IN events p``)
                                        THEN1(FULL_SIMP_TAC std_ss [random_variable_def]
                             	              THEN METIS_TAC [IN_MEASURABLE, events_def, measurable_sets_def, random_variable_def, p_space_def, m_space_def, space_def, subsets_def, prob_space_def, measure_space_def])
                                  THEN DISCH_TAC
		                  THEN METIS_TAC [PROB_FINITE])
                         THEN DISCH_TAC
                         THEN FULL_SIMP_TAC std_ss[])
	  THEN DISCH_TAC		 
	  THEN FULL_SIMP_TAC std_ss[]
	  THEN METIS_TAC [mul_extreal_not_infty])
          THEN DISCH_TAC
          THEN FULL_SIMP_TAC std_ss[]
          THEN KNOW_TAC(``    (\(x,y).
              pos_fn_integral (distr p lborel Z)
                (\z.
                     indicator_fn
                       {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
                       (x,y) * indicator_fn {z' | y < z' /\ z' < x} z)) IN
         measurable
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel``)

               THEN1( KNOW_TAC (`` ! x y.
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
                 {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
               indicator_fn {z' | y < z' /\ z' < x} z) =  
        pos_fn_integral (distr p lborel Z)
          (\z.
               indicator_fn
             {((x',y'),z') |
                         y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} ((x,y),z))``)
                     THEN1( RW_TAC std_ss[])
		THEN DISCH_TAC
		THEN ONCE_ASM_REWRITE_TAC[]
 		THEN MP_TAC(Q.ISPECL[`(\ ((x,y),z). indicator_fn
                 {((x',y'),z') |
                 y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t:real} ) ((x,y),z) `,`pair_measure lborel lborel`, `lborel` ] borel_measurable_pos_integral_fst)
		 THEN RW_TAC std_ss[sigma_finite_measure_lborel, measure_space_lborel, measure_space_pair_measure' ])
		 THEN DISCH_TAC
                 THEN FULL_SIMP_TAC std_ss[]
		 THEN KNOW_TAC(``!x.
              (\ (x,y).
                   f_xy (x,y) *
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                            0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | y < z' /\ z' < x} z)) x =
              f_xy x *
              (\ (x,y).
                   pos_fn_integral (distr p lborel Z)
                     (\z.
                          indicator_fn
                            {(x',y') |
                       0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) *
                          indicator_fn {z' | y < z' /\ z' < x} z)) x ``)
                       THEN1(RW_TAC std_ss[]
		            THEN `(x:real#real) = (FST x , SND x)` by RW_TAC std_ss[PAIR]
                            THEN ONCE_ASM_REWRITE_TAC[]
			    THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
		THEN DISCH_TAC	    
                THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]);




val event1_shared_spare_indicator_of_indicator_Axy_Ax_Ay = store_thm(
  "event1_shared_spare_indicator_of_indicator_Axy_Ax_Ay",
  `` ! x y  (t:real).  indicator_fn {(x',y')|  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} (x,y) = 
		        indicator_fn {x' | y < x' /\ x' <= t } x * indicator_fn {y' | 0 <= y' /\ y' <= t} y  ``,

RW_TAC std_ss [indicator_fn_def]
      THEN1(RW_TAC std_ss[mul_lone])
      THEN1(`(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.  0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.  0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.  0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`(x,y) IN {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.  0 <= y'' /\ y'' <= t /\ y'' < x'' /\ x'' <= t) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
 THEN RW_TAC std_ss [mul_rzero, mul_lzero]);
   

val joint_density_3rv_2 = store_thm("joint_density_3rv_2",

``!p X Y Z f_xy t.
     prob_space p /\ random_variable Z p borel /\
     (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
     distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy 
     ==>
     (pos_fn_integral
        (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
        (\ (x,y).
           pos_fn_integral (distr p lborel Z)
             (\z. indicator_fn   {((x',y'),z') |
            y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}((x,y),z))) =
      pos_fn_integral lborel
        (\y. indicator_fn {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t} x *
                pos_fn_integral (distr p lborel Z)
                  (\z. indicator_fn {z' | y < z' /\ z' < x} z))))``,

REPEAT GEN_TAC

THEN  KNOW_TAC(``{((x',y'),z') |
                   y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                    y' <= t} IN measurable_sets (pair_measure (pair_measure lborel lborel)lborel) ``)
    THEN1(RW_TAC std_ss [BIGUNION_event1_shared_spare_subevent2_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC (`` {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
          measurable_sets (pair_measure lborel lborel)``)
	  THEN1(RW_TAC std_ss[event1_shared_spare_Axy_measurable_set])
THEN DISCH_TAC
THEN `!x y z. indicator_fn
            {((x',y'),z') |
            y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                    y' <= t}
            ((x,y),z) =
          indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) * indicator_fn {z' | y < z' /\ z' < x} z` by RW_TAC std_ss[event1_shared_spare_indicator_of_indicator_Axy]
THEN KNOW_TAC(`` !x y. {z' | y < z' /\ z' < x:real} IN measurable_sets lborel``)
     THEN1( `measurable_sets lborel = subsets borel` by METIS_TAC [borel , lborel, subsets_def, measurable_sets_def]
          THEN ONCE_ASM_REWRITE_TAC[]
	  THEN `!x y . {z' | y < z' /\ z' < x} = {z' | y < z'} INTER {z'| z' < x:real}` by SRW_TAC  [] [GSPECIFICATION, EXTENSION]
	  THEN `!x y. {z' | y < z'} IN subsets borel  /\ {z' | z' < x} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
          THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC	  
THEN `!x y.   indicator_fn {(x',y') |   0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) =
          indicator_fn {x' | x' | y < x' /\ x' <= t} x *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y ` by RW_TAC std_ss[event1_shared_spare_indicator_of_indicator_Axy_Ax_Ay]
THEN KNOW_TAC(``!y.  {x' | y < x' /\ x' <= t} IN measurable_sets lborel ``)
     THEN1(RW_TAC std_ss[]
           THEN `measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN `{x' | y < x' /\ x' <= t} =
	         {x' | y <  x'} INTER
		 {x' | x' <= t } ` by (SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
          THEN ONCE_ASM_REWRITE_TAC[]
          THEN `{x' | y < x'} IN subsets borel /\
		 {x' | x' <= t } IN subsets borel  ` by METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC
THEN KNOW_TAC (``{y' | 0 <= y' /\ y' <= t} IN measurable_sets lborel``)
     THEN1(`measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
             THEN `{y' | 0 <= y' /\ y' <= t} = {y' | 0<= y'} INTER {y' | y' <= t}` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
	     THEN `{y' | 0<= y'} IN subsets borel /\ {y' | y' <= t} IN subsets borel` by
	     METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])

THEN DISCH_TAC	   

THEN MP_TAC(Q.ISPECL[`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,`X:'a->real`, `Y:'a->real`, `Z:'a->real`, `f_xy:(real#real)->extreal`, `t:real` ] joint_density_3rv_1)
THEN RW_TAC std_ss[] THEN FULL_SIMP_TAC std_ss[]

THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
			       THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                     THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
                               THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]


THEN  MP_TAC(Q.ISPECL[`lborel`,`(\y. real(indicator_fn {y' | 0 <= y' /\ y' <= t:real} y))`, `(\ (x,y).
          f_xy (x:real,y:real) * indicator_fn {x' | y < x' /\ x' <= t:real} x  *
          pos_fn_integral (distr p lborel (Z:'a->real)) (\z:real. indicator_fn {z' | y  < z' /\ z' < x:real} z))` ] pos_fn_integral_fun_mul2)

THEN RW_TAC std_ss[ measure_space_lborel]
THEN `!(x:real) (y:real).
          x IN m_space lborel` by METIS_TAC [lborel, m_space_def, IN_UNIV]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``(!(x:real) (y:real).
          0 <=
          f_xy (x,y) *
              indicator_fn {x' |y < x' /\ x' <= t} x *
              pos_fn_integral (distr p lborel Z)
                (\z. indicator_fn {z' | y < z' /\ z' < x} z)) ``)
     THEN1(RW_TAC std_ss[]
           THEN `0<= indicator_fn {x' |  y < x' /\ x' <= t:real} x `by RW_TAC std_ss [indicator_fn_pos_le]
           THEN `measure_space (distr p lborel (Z:'a->real))` by  METIS_TAC [measure_space_distr, prob_space_def, p_space_def, events_def, random_variable_def, measure_space_lborel]
	   
          THEN `0<= pos_fn_integral (distr p lborel (Z:'a->real)) (\z. indicator_fn  {z' | y < z' /\ z' < x:real}z)`  by  METIS_TAC [pos_fn_integral_pos, measure_space_lborel, indicator_fn_pos_le]

        THEN RW_TAC std_ss[le_mul])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (!y:real. 0 <= real (indicator_fn {y' | 0 <= y' /\ y' <= t:real} y))``)
     THEN1( `!y. indicator_fn {y' | real 0 <= y' /\ y' <= t:real} (y:real) <> PosInf /\
		            indicator_fn {y' | real 0 <= y' /\ y' <= t:real} y <> NegInf` by 
			        (RW_TAC std_ss [indicator_fn_def]
			        THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def])

           THEN `0 = real 0` by 
                       (RW_TAC std_ss[real_def]
		       THEN SELECT_ELIM_TAC
                       THEN RW_TAC std_ss[]  		      
                            THEN1(Q.EXISTS_TAC `0`
			          THEN METIS_TAC [extreal_of_num_def])
		       THEN METIS_TAC [extreal_of_num_def, extreal_11])
	   THEN ASM_REWRITE_TAC[]
	   THEN `0<> PosInf /\ 0 <> NegInf` by METIS_TAC [extreal_of_num_def, lt_infty]
	   THEN RW_TAC std_ss[extreal_real_le]
	   THEN RW_TAC std_ss[indicator_fn_pos_le])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN `!y:real. indicator_fn {y' |0 <= y' /\ y' <= t:real} y <> PosInf /\ indicator_fn {y' |0 <= y' /\ y' <= t:real} y<> NegInf` by METIS_TAC [indicator_fn_not_eq_infty]
THEN `!y:real. Normal (real (indicator_fn {y' | 0 <= y' /\ y' <= t} y)) = indicator_fn {y' | 0 <= y' /\ y' <= t} y` by RW_TAC std_ss [normal_real_indicator_fn]
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!y:real.
         
           (\x:real.
              indicator_fn {y' | 0 <= y' /\ y' <= t:real} y *
              (f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t} x *
               pos_fn_integral (distr p lborel (Z:'a->real))
                 (\z:real. indicator_fn {z' | y < z' /\ z' < x} z))) =
		 
           (\x:real.
               
              f_xy (x,y) * (indicator_fn {x' | y < x' /\ x' <= t} x * indicator_fn {y' |0 <= y' /\ y' <= t:real} y)*
               pos_fn_integral (distr p lborel Z)
                 (\z:real. indicator_fn {z' | y < z' /\ z' < x} z))``)
      THEN1(RW_TAC std_ss[]
            THEN  ABS_TAC
	    THEN METIS_TAC[mul_comm, mul_assoc])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]);




val joint_density_3rv_3 = store_thm("joint_density_3rv_3",

``!p X Y Z  f_xy f_z t.
     prob_space p /\ random_variable Z p borel /\
     (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
     (!z. 0<= f_z z) /\  
     distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy /\
     distributed p lborel Z  f_z ==>
     (pos_fn_integral
        (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
        (\ (x,y).
           pos_fn_integral (distr p lborel Z)
             (\z. indicator_fn {((x',y'),z') |
                   y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}
((x,y),z))) =
      pos_fn_integral lborel
        (\y. indicator_fn  {y' | 0 <= y' /\ y' <= t} y*
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {x' |  y < x' /\ x' <= t} x *
                pos_fn_integral lborel
                  (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z))))``,

REPEAT GEN_TAC
THEN KNOW_TAC(``{((x',y'),z') |
                  y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} IN measurable_sets (pair_measure (pair_measure lborel lborel)lborel) ``)
    THEN1(RW_TAC std_ss [BIGUNION_event1_shared_spare_subevent2_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC (`` {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t} IN
          measurable_sets (pair_measure lborel lborel)``)
	  THEN1(RW_TAC std_ss[event1_shared_spare_Axy_measurable_set])
THEN DISCH_TAC
THEN `!x y z. indicator_fn
            {((x',y'),z') |
           y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\
                    y' <= t}
            ((x,y),z) =
          indicator_fn {(x',y') |  0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) * indicator_fn {z' | y < z' /\ z' < x} z` by RW_TAC std_ss[event1_shared_spare_indicator_of_indicator_Axy]
THEN KNOW_TAC(`` !x y. {z' | y < z' /\ z' < x:real} IN measurable_sets lborel``)
     THEN1( `measurable_sets lborel = subsets borel` by METIS_TAC [borel , lborel, subsets_def, measurable_sets_def]
          THEN ONCE_ASM_REWRITE_TAC[]
	  THEN `!x y . {z' | y < z' /\ z' < x} = {z' | y < z'} INTER {z'| z' < x:real}` by SRW_TAC  [] [GSPECIFICATION, EXTENSION]
	  THEN `!x y. {z' | y < z'} IN subsets borel  /\ {z' | z' < x} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
          THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC

THEN `!x y.   indicator_fn {(x',y') | 0 <= y' /\ y' <= t /\ y' < x' /\ x' <= t}
            (x,y) =
          indicator_fn {x' | y < x' /\ x' <= t} x *
          indicator_fn {y' | 0 <= y' /\ y' <= t} y ` by RW_TAC std_ss[event1_shared_spare_indicator_of_indicator_Axy_Ax_Ay]
THEN KNOW_TAC(``!y.  {x' |  y < x' /\ x' <= t} IN measurable_sets lborel ``)
     THEN1(RW_TAC std_ss[]
           THEN `measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
	   THEN `{x' | y < x' /\ x' <= t} =
	         {x' | y < x'} INTER
		 {x' | x' <= t }` by (SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
          THEN ONCE_ASM_REWRITE_TAC[]
          THEN `{x' | y < x'} IN subsets borel /\
		 {x' | x' <= t } IN subsets borel  ` by METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
THEN DISCH_TAC
THEN KNOW_TAC (``{y' |0 <= y' /\ y' <= t} IN measurable_sets lborel``)
     THEN1(`measurable_sets lborel = subsets borel` by METIS_TAC [measurable_sets_def, subsets_def, lborel, borel]
	   THEN ONCE_ASM_REWRITE_TAC[]
 	   THEN ` {y' | 0 <= y' /\ y' <= t} = {y' | 0<= y'} INTER {y' | y' <= t}` by SRW_TAC [][GSPECIFICATION, EXTENSION]
	   THEN `{y' | 0<= y'} IN subsets borel /\ {y' | y' <= t} IN subsets borel`  by METIS_TAC [borel_open, borel_closed, OPEN_INTERVAL_LEFT, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_RIGHT]
	 THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])

THEN DISCH_TAC	   


THEN DISCH_TAC
THEN `pos_fn_integral (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
   (\( x:real,y:real).
      pos_fn_integral (distr p lborel (Z:'a->real))
        (\z:real. indicator_fn  {((x',y'),z') |
        y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}  ((x,y),z))) =
	pos_fn_integral lborel
        (\y:real.
           indicator_fn  {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x:real.
                f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t} x *
                pos_fn_integral (distr p lborel (Z:'a->real))
                  (\z:real. indicator_fn {z' | y < z' /\ z' < x} z)))`by METIS_TAC[joint_density_3rv_2]
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` !x y. pos_fn_integral (distr p lborel (Z:'a->real)) (\z:real. indicator_fn {z' | y < z' /\ z' < x} z) =
                  pos_fn_integral lborel (\z:real. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)``)
    THEN1(FULL_SIMP_TAC std_ss[distributed_def]
          THEN RW_TAC std_ss []
          THEN MP_TAC(Q.ISPECL[`f_z:real->extreal`,`indicator_fn {z' | y < z' /\ z' < x:real}`, `lborel` ] pos_fn_integral_density_1)
          THEN RW_TAC std_ss [measure_space_lborel]

          THEN  KNOW_TAC(``indicator_fn {z' | y < z' /\ z' < x} IN
         measurable (m_space lborel,measurable_sets lborel) Borel ``)
	        THEN1(MP_TAC(Q.ISPECL[`(m_space lborel,measurable_sets lborel)`, `{z' | y < z' /\ z' < x:real}`,`(\z:real. indicator_fn {z' | y < z' /\ z' < x:real} z)` ] IN_MEASURABLE_BOREL_INDICATOR)
                      THEN RW_TAC std_ss []
    	              THEN ` {z' | y < z' /\ z' < x} IN subsets (m_space lborel,measurable_sets lborel) ` by RW_TAC std_ss [subsets_def]
	              THEN FULL_SIMP_TAC std_ss[] 
                      THEN KNOW_TAC(``(m_space lborel, measurable_sets lborel)= borel``)
		    	        THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                      THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			        THEN ONCE_ASM_REWRITE_TAC[]
                                THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
	              THEN DISCH_TAC
	              THEN FULL_SIMP_TAC std_ss[]
          	      THEN FULL_SIMP_TAC std_ss[sigma_algebra_borel]
          	      THEN `(\z:real. indicator_fn  {z' | y < z' /\ z' < x} z) = indicator_fn {z' | y < z' /\ z' < x}` by METIS_TAC[ETA_THM]
	  	      THEN FULL_SIMP_TAC std_ss[])
	THEN DISCH_TAC
	THEN FULL_SIMP_TAC std_ss[indicator_fn_pos_le])
THEN DISCH_TAC	
THEN RW_TAC std_ss[]);




val cond_density_3rv = store_thm("cond_density_3rv",
  `` !p X Y Z f_xy  f_y f_z f_cond t.
     prob_space p /\ den_gt0_ninfinity f_xy f_y f_cond  /\
     (!y. cond_density lborel lborel p X Y  y f_xy  f_y f_cond ) /\
     (!z. 0 <= f_z z) ==>
     (pos_fn_integral lborel
        (\y.
           indicator_fn {y' |0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_xy (x,y) * indicator_fn {x' | y < x' /\ x' <= t}  x *
                pos_fn_integral lborel
                  (\z. f_z z * indicator_fn  {z' | y < z' /\ z' < x} z))) =
      pos_fn_integral lborel
        (\y.
           f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
           pos_fn_integral lborel
             (\x.
                f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
                pos_fn_integral lborel
                  (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z))))``,

RW_TAC std_ss[den_gt0_ninfinity_def]
THEN `!y:real. f_y y <> 0 /\  f_y y <> NegInf /\ f_y y <> PosInf ` by METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty, lt_le, le_lt]
THEN `!x y. f_xy (x:real,y:real) = f_cond y x * f_y y` by METIS_TAC [cond_density_joint_marginal]
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` !y.
        pos_fn_integral lborel
          (\x.
               f_y y *
               (\x.
                    indicator_fn {x' |y < x' /\ x' <= t} x *
                    pos_fn_integral lborel
                      (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)) x *
               f_cond y x) =
        f_y y *
        pos_fn_integral lborel
          (\x.
               (\x.
                    indicator_fn {x' |y < x' /\ x' <= t} x *
                    pos_fn_integral lborel
                      (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)) x *
               f_cond y x)``)

     THEN1(RW_TAC std_ss[]

          THEN MP_TAC(Q.ISPECL[`lborel`, `f_cond:real->real->extreal`, `f_y : real->extreal`, ` (\x:real.
            indicator_fn {x' |y < x' /\ x' <= t} x *
          pos_fn_integral lborel (\z:real. f_z z * indicator_fn {z' | y < z' /\ z' < x} z))` ] lemma5_fun_mul_fun)
	  THEN RW_TAC std_ss [measure_space_lborel]
	  THEN `!x:real. x IN m_space lborel ` by RW_TAC std_ss [lborel, m_space_def, IN_UNIV]
	  THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(``(!x:real.
               0 <=
               indicator_fn  {x' | y < x' /\ x' <= t} x *
               pos_fn_integral lborel (\z:real. f_z z * indicator_fn {z' | y < z' /\ z' < x} z))``) THEN1 
               (RW_TAC std_ss[]
	       THEN `0<= indicator_fn   {x' |y < x' /\ x' <= t}(x:real)` by RW_TAC std_ss [indicator_fn_pos_le]
	       THEN `0<= pos_fn_integral lborel (\z:real. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)` by 
	               (`(!z:real. 0<= indicator_fn {z' | y < z' /\ z' < x} z)` by RW_TAC std_ss [indicator_fn_pos_le]
		       THEN `!z:real. 0<= f_z z * indicator_fn {z' | y < z' /\ z' < x} z` by RW_TAC std_ss[le_mul]
                       THEN MP_TAC(Q.ISPECL[`lborel`, `(\z:real. f_z z * indicator_fn  {z' | y < z' /\ z' < x} z)` ] pos_fn_integral_pos)
		       THEN RW_TAC std_ss[measure_space_lborel])
             THEN RW_TAC std_ss[le_mul])
	 THEN DISCH_TAC    
	 THEN FULL_SIMP_TAC std_ss[]		       
	 THEN `!y:real. 0 < f_y y`  by RW_TAC std_ss[]
	 THEN `!y:real. 0<= f_y y /\ 0<> f_y y` by METIS_TAC [lt_le]
	 THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC	 
THEN ASM_REWRITE_TAC[]

THEN `!(y:real) (x:real).  f_y y *
             (indicator_fn {x' | y < x' /\ x' <= t} x *
              pos_fn_integral lborel (\z:real. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)) *
             f_cond y x =  f_cond y x * f_y y *
             indicator_fn {x' |y < x' /\ x' <= t} x *
              pos_fn_integral lborel (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)` by METIS_TAC[mul_assoc, mul_comm]
THEN FULL_SIMP_TAC std_ss[]
THEN `!y:real. indicator_fn {y' | 0<= y' /\ y' <= t}  y *
     (f_y y *
      pos_fn_integral lborel
        (\x:real.
           indicator_fn {x' | y < x' /\ x' <= t} x *
           pos_fn_integral lborel (\z:real. f_z z * indicator_fn {z' | y < z' /\ z' < x} z) *
           f_cond y x)) = indicator_fn {y' | 0<= y' /\  y' <= t}  y *
     f_y y *
      pos_fn_integral lborel
        (\x.
           indicator_fn {x' | y < x' /\ x' <= t} x *
           pos_fn_integral lborel (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z) *
           f_cond y x)  ` by METIS_TAC [mul_assoc, mul_comm]
THEN ASM_REWRITE_TAC[]
THEN `!(y:real) (x:real). indicator_fn {x' | y < x' /\ x' <= t} x *
          pos_fn_integral lborel (\z:real. f_z z * indicator_fn  {z' | y < z' /\ z' < x} z) *
          f_cond y x =
	  f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
          pos_fn_integral lborel (\z. f_z z * indicator_fn  {z' | y < z' /\ z' < x} z)` by METIS_TAC [mul_assoc, mul_comm]
THEN ASM_REWRITE_TAC[]
THEN `!y:real.  indicator_fn  {y' | 0<= y' /\ y' <= t} y * f_y y *
     pos_fn_integral lborel
       (\x:real.
          f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
          pos_fn_integral lborel (\z:real. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)) =
	  f_y y * indicator_fn  {y' | 0<= y' /\ y' <= t} y * 
     pos_fn_integral lborel
       (\x.
          f_cond y x * indicator_fn {x' | y < x' /\ x' <= t} x *
          pos_fn_integral lborel (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z))` by METIS_TAC [mul_comm]
THEN ASM_REWRITE_TAC[]);

(************ prob of event Aa.(A1 before A2)  .(A2 before Aa)***************)




val DFT_event1_shared_spare = store_thm("DFT_event1_shared_spare",
  ``!p X Y Z t.  rv_gt0_ninfinity [X; Y; Z] ==>
  ((DFT_event p  (D_AND (D_AND (X)(D_BEFORE Y Z))(D_BEFORE Z X)) t) =
     PREIMAGE (\x. ((X x, Y x), Z x)) {((x,y),z) |
            y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t} INTER p_space p )``,

RW_TAC std_ss [AND_INTER, rv_gt0_ninfinity_def]
THEN RW_TAC std_ss [DFT_event_def, D_BEFORE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
	         THEN Q.EXISTS_TAC (`(X x,Y x,Z x) `)
                 THEN FULL_SIMP_TAC std_ss [LAMBDA_PROD, PAIR]
	         THEN METIS_TAC [lt_trans])
           THEN METIS_TAC [extreal_not_infty, le_infty, lt_infty])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
     THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	    
     THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
        THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[]
	    THEN METIS_TAC [lt_le, le_lt, lt_trans, le_trans])
	    
         THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
        THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
       THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
       THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
       THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
       THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
       THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
       THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	   
THEN KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
               y <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),  FST (SND x') < SND (SND x') /\ SND (SND x') < FST x' /\ FST (SND x') < FST x' /\ FST x'  <= Normal t /\ 0 <= FST (SND x') /\ FST (SND x') <= Normal t)``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
THEN DISCH_TAC	
THEN FULL_SIMP_TAC std_ss[]);



val PREIMAGE_EXTREAL_REAL_3RV_event1_shared_spare = store_thm(
  "PREIMAGE_EXTREAL_REAL_3RV_event1_shared_spare",
  `` !t X Y Z.  rv_gt0_ninfinity [X;Y;Z] ==>
  (PREIMAGE (\s. ((X s,Y s),Z s))
     {((x,y),z) |
     y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t} =
   PREIMAGE (\s. ((real (X s),real (Y s)),real (Z s)))
     {((x',y'),z') |
     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}) ``,

RW_TAC std_ss [rv_gt0_ninfinity_def, PREIMAGE_def]
THEN KNOW_TAC(``!s. ((X s,Y s),Z s) IN
   {((x,y),z) |
  y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\ y <= Normal t} =
    Y s < Z s /\ Z s < X s /\ Y s < X s /\ X s <= Normal t /\ 0 <= Y s /\ Y s <= Normal t``)

     THEN1(RW_TAC std_ss [IN_ELIM_PAIR_THM_1])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!s. ((real (X s),real (Y s)),real (Z s)) IN
   {((x',y'),z') |
  y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t}=
   real (Y s) < real (Z s) /\  real (Z s) < real (X s) /\ real (Y s) < real (X s) /\
   real (X s) <= t /\ 0 <= real (Y s) /\ real (Y s) <=t``)
  THEN1(RW_TAC std_ss [IN_ELIM_PAIR_THM_1])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN `X x <> PosInf` by RW_TAC std_ss[] 
THEN `Y x <> PosInf` by RW_TAC std_ss[]
THEN `Z x <> PosInf` by RW_TAC std_ss[]
THEN `X x <> NegInf /\ Y x <> NegInf /\ Z x <> NegInf` by METIS_TAC [le_lt, lt_le, extreal_le_def, extreal_le_def, lt_infty, le_infty, le_trans, lt_trans, extreal_of_num_def]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1(METIS_TAC [extreal_real_lt])
	   THEN1(METIS_TAC [extreal_real_lt])
	   THEN1(METIS_TAC [extreal_real_lt])
	   THEN1(RW_TAC std_ss [real_def]
	         THEN SELECT_ELIM_TAC
                 THEN RW_TAC std_ss[]
		      THEN1( Q.EXISTS_TAC `real (X x)`
		            THEN METIS_TAC [normal_real])
                 THEN METIS_TAC [extreal_le_eq])
	   THEN1(` 0<= Y x` by RW_TAC std_ss[]
                 THEN `0 = real 0`  by RW_TAC std_ss [real_of_0]
		 THEN ONCE_ASM_REWRITE_TAC []
		 THEN `0 <> PosInf /\ 0<> NegInf` by METIS_TAC [extreal_lt_def, extreal_le_def, lt_infty, le_infty, lt_le, le_lt, extreal_of_num_def]
		 THEN METIS_TAC [extreal_real_le])
           THEN RW_TAC std_ss [real_def]
	   THEN SELECT_ELIM_TAC
	   THEN RW_TAC std_ss[]
	        THEN1(Q.EXISTS_TAC `real (Y x)`
		            THEN METIS_TAC [normal_real])
                 THEN METIS_TAC [extreal_le_eq])
THEN RW_TAC std_ss[]		 
     THEN1(METIS_TAC [extreal_real_lt])
     THEN1(METIS_TAC [extreal_real_lt])
     THEN1(METIS_TAC [extreal_real_lt])
     THEN1(UNDISCH_TAC(``real (X x) <= t ``)
           THEN REWRITE_TAC[real_def]
	   THEN SELECT_ELIM_TAC
	   THEN RW_TAC std_ss[]
	        THEN1(Q.EXISTS_TAC (`real (X x)`)
		     THEN METIS_TAC [normal_real])
	   THEN RW_TAC std_ss [extreal_le_eq])
THEN UNDISCH_TAC(`` real (Y x) <= t``)	   
THEN REWRITE_TAC [real_def]
THEN SELECT_ELIM_TAC
THEN RW_TAC std_ss[]
     THEN1(Q.EXISTS_TAC (`real (Y x)`)
           THEN METIS_TAC [normal_real])
THEN RW_TAC std_ss [extreal_le_eq]);



val prob_event1_shared_spare = store_thm("prob_event1_shared_spare",

  ``!p t X Y Z f_xy f_cond  f_y f_z .
     prob_space p /\ den_gt0_ninfinity f_xy f_y f_cond /\ rv_gt0_ninfinity [X; Y; Z] /\
       (!y. cond_density lborel lborel p (\x. real (X x))  (\x. real (Y  x))  y f_xy f_y f_cond ) /\ indep_varp p (pair_measure lborel lborel)
           (\x. (real (X x),real (Y x))) lborel (\x. real (Z x)) /\
      (!z. 0 <= f_z z) /\  (!x. f_xy x <> PosInf) /\ distributed p lborel (\x. real (Z x)) f_z ==>
  
   ( prob p (DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t) =
       pos_fn_integral lborel
             (\y.
                  f_y y * indicator_fn {y' |  0 <= y' /\ y' <= t} y *
                  pos_fn_integral lborel
                    (\x.
                         f_cond y x  * indicator_fn {x' | y < x' /\ x' <= t} x *
                         pos_fn_integral lborel
                           (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z))))``,


RW_TAC std_ss[]
THEN KNOW_TAC(``random_variable (\x. real (Z x)) p borel ``)
     THEN1(RW_TAC std_ss[random_variable_def, p_space_def, events_def]
           THEN `borel = (m_space lborel, measurable_sets lborel)` by
	        (RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                 THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
	         THEN ONCE_ASM_REWRITE_TAC[]
                 THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
	    THEN FULL_SIMP_TAC std_ss[distributed_def])
THEN DISCH_TAC
THEN   MP_TAC(Q.ISPECL [`p: ('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`, `(\x. real ((X:'a->extreal) x))` , `(\x. real ((Y:'a->extreal) x))`, `(\x. real ((Z:'a->extreal) x))`, `(pair_measure lborel lborel)` , `lborel`, ` {((x',y'),z') | y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t:real}`] prob_event_3rv_2indep)

THEN RW_TAC std_ss[]
THEN ` measure_space (pair_measure lborel lborel)` by METIS_TAC [measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]
THEN `measure_space lborel` by RW_TAC std_ss [measure_space_lborel]
THEN `sigma_finite_measure (pair_measure lborel lborel)` by RW_TAC std_ss[sigma_finite_measure_pair_lborel]
THEN `sigma_finite_measure lborel` by RW_TAC std_ss [sigma_finite_measure_lborel]
THEN `{((x',y'),z') |
         y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} IN
         measurable_sets (pair_measure (pair_measure lborel lborel) lborel)` by RW_TAC std_ss[BIGUNION_event1_shared_spare_subevent2_measurable_set]
THEN FULL_SIMP_TAC std_ss[]
THEN RW_TAC std_ss[DFT_event1_shared_spare]
THEN RW_TAC std_ss[PREIMAGE_EXTREAL_REAL_3RV_event1_shared_spare, rv_gt0_ninfinity_def] 
THEN KNOW_TAC(``( pos_fn_integral
     (distr p (pair_measure lborel lborel) (\x. (real (X x),real (Y x))))
     (\ (x,y).
          pos_fn_integral (distr p lborel (\x. real (Z x)))
            (\z.
                 indicator_fn
                   {((x',y'),z') |
                   y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} ((x,y),z))) =
     pos_fn_integral lborel
       (\y.
            indicator_fn {y' | 0<= y' /\ y' <= t} y *
            pos_fn_integral lborel
              (\x.
                   f_xy (x,y) *
                   indicator_fn {x' |  y < x' /\ x' <= t} x *
                   pos_fn_integral lborel
                     (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z)))) ``)
      THEN1( MP_TAC(Q.ISPECL [`p: ('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,
                       `(\x. real ((X:'a->extreal) x))` , `(\x. real ((Y:'a->extreal) x))`, `(\x. real ((Z:'a->extreal) x))`, `f_xy : (real#real) ->extreal`, `f_z:real->extreal ` ,`t:real` ] joint_density_3rv_3)
           THEN RW_TAC std_ss[]
           THEN FULL_SIMP_TAC std_ss[den_gt0_ninfinity_def, cond_density_def]
	   THEN KNOW_TAC(``!(x:real#real). 0<= f_xy x``) 
                THEN1(RW_TAC std_ss []
		      THEN `(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
		      THEN ONCE_ASM_REWRITE_TAC[]
		      THEN FULL_SIMP_TAC std_ss[])
	  THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[ETA_THM, PAIR, LAMBDA_PROD])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]	  

THEN DEP_REWRITE_TAC[cond_density_3rv]
THEN Q.EXISTS_TAC(`p`)
THEN EXISTS_TAC(``(\x. real ((X:'a->extreal) x))``)
THEN EXISTS_TAC(``(\x. real ((Y:'a->extreal) x))``)
THEN RW_TAC std_ss[]);




(******************************************************************************)
(*                           shared spare event2                              *)
(*                           A2. (A1 before Aa) ( Aa before A2)               *)
(*                           A1 and Aa are independent of A2                  *)
(******************************************************************************)

val DFT_event2_shared_spare = store_thm("DFT_event2_shared_spare",
  ``!p X Y Z t.  rv_gt0_ninfinity [X; Y; Z] ==>
  ((DFT_event p  (D_AND (D_AND (Z)(D_BEFORE Y X))(D_BEFORE X Z)) t) =
     PREIMAGE (\x. ((X x, Y x), Z x)) {((x,y),z) |
           y < x /\ x < z /\ 0<= y /\ y < z /\ 0 <= z /\ z <= Normal t} INTER p_space p )``,



RW_TAC std_ss [AND_INTER, rv_gt0_ninfinity_def]
THEN RW_TAC std_ss [DFT_event_def, D_BEFORE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1 (SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
	         THEN Q.EXISTS_TAC (`(X x,Y x,Z x) `)
                 THEN FULL_SIMP_TAC std_ss [LAMBDA_PROD, PAIR]
	         THEN METIS_TAC [lt_trans])
           THEN METIS_TAC [extreal_not_infty, le_infty, lt_infty])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION, PREIMAGE_def]
     THEN1 (KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	    THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[]
	    THEN METIS_TAC [lt_le, le_lt, lt_trans, le_trans])
	    THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[]
	    THEN METIS_TAC [lt_le, le_lt, lt_trans, le_trans])
        THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
          THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
	THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
       THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
      THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
      THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
      THEN1(KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
	    THEN DISCH_TAC	
            THEN FULL_SIMP_TAC std_ss[])
THEN KNOW_TAC (`` (\ (x:extreal,y,z).
              (((x,y),z),
                y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t)) (x':extreal#extreal#extreal) = ( ( (FST (x':extreal#extreal#extreal), FST (SND x')), SND (SND x')),
	       FST (SND  x') <  FST  x' /\ FST x' <  SND (SND x') /\
	       0 <= FST (SND x') /\ FST(SND  x') < SND (SND x') /\
	       0 <= SND (SND x') /\ SND (SND x') <= Normal t)	       ``)
                THEN1( `x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss [PAIR, LAMBDA_PROD]
	              THEN ONCE_ASM_REWRITE_TAC[]
                      THEN RW_TAC std_ss [PAIR, LAMBDA_PROD])
THEN DISCH_TAC	
THEN FULL_SIMP_TAC std_ss[]
THEN METIS_TAC [lt_le, le_lt, lt_trans, le_trans]);
	    
 

val event2_shared_spare = store_thm("event2_shared_spare",
  ``  !x y z t:real.
       indicator_fn
         {((x',y'),z') |
            y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z) =
       indicator_fn {z' | 0 <= z' /\ z' <= t} z *
       indicator_fn {y' | 0 <= y' /\ y' < z} y *
       indicator_fn {x' | y < x' /\ x' < z} x``,
       
REPEAT GEN_TAC
THEN RW_TAC std_ss [indicator_fn_def]
  THEN1(METIS_TAC [mul_lone])

THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
      {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN METIS_TAC [mul_rzero, mul_comm]);


val BIGUNION_event2_shared_spare_subevent0 = store_thm(
  "BIGUNION_event2_shared_spare_subevent0",
  `` !t q1 q2.BIGUNION {({x' | real q0 < x' /\ x' < real q1} CROSS
                         {y'| y' < real q0 /\ 0 <= y' /\ y' < real q2 }) CROSS
			{z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} | q0 IN Q_set}=
              {(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} CROSS
	      {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <=t}  ``,


SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(STRIP_TAC
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
              
           x IN s <=>
              ((real q0 < FST (FST x) /\ FST (FST x) < real q1) /\
              SND (FST x) < real q0 /\ 0 <= SND (FST x) /\
              SND (FST x) < real q2) /\ real q1 < SND x /\ real q2 < SND x /\
             0 <= SND x /\ SND x <= t``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)
          THEN `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EXISTS_TAC(``FST (x:(real#real)#real) ``)
           THEN RW_TAC std_ss [LAMBDA_PAIR]
	   THEN METIS_TAC [REAL_LT_TRANS])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]

THEN  KNOW_TAC(``?q0. q0 IN Q_set/\ Normal (SND (x':real#real) ) < q0 /\  q0 < Normal (FST x')``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r0. q0 = Normal r0``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (SND (FST (x:(real#real)#real))) < Normal( FST (FST x))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC

THEN EXISTS_TAC (``{(x:(real#real)#real)| real (Normal r0) < FST (FST x) /\ FST (FST x) < real q1 /\
                       SND (FST x) < real (Normal r0) /\ 0 <= SND (FST x) /\
                       SND (FST x) < real q2 /\ real q1 < SND x /\
                       real q2 < SND x /\ 0 <= SND x /\ SND x <= t }  ``)
     
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r0`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);


val event2_shared_spare_subevent0_measurable_set = store_thm(
  "event2_shared_spare_subevent0_measurable_set",
  ``!t q0 q1 q2. ({x' | real q0 < x' /\ x' < real q1} CROSS
            {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2} ) CROSS
            {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,

RW_TAC std_ss [measurable_sets_def, pair_measure, measure_of, lborel, m_space_def]
    THEN1 (KNOW_TAC(`` {x' | real q0 < x' /\ x' < real q1} CROSS
       	  	       {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2} CROSS
		       {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} IN
                       {(a:real reln) CROSS (b:real->bool) |
                         a IN
                         sigma_sets (univ(:real) CROSS univ(:real))
                         {a CROSS b | a IN subsets borel /\ b IN subsets borel} /\
                         b IN subsets borel}``)
           THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	         THEN EXISTS_TAC (``({x' | real q0 < x' /\ x' < real q1} CROSS
       	  	       		     {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2},
	                             {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t}) ``)
                THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                     THEN1(KNOW_TAC(``{x' | real q0 < x' /\ x' < real q1} CROSS
       	  	       		      {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2} IN
				       {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
                          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			        THEN EXISTS_TAC (``({x' | real q0 < x' /\ x' < real q1},
       	  	       		     		    {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2}) ``)
			        THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                                     THEN1( KNOW_TAC(``{x' | real q0 < x' /\ x' < real q1} =
					    {x' | real q0 < x'} INTER
					    {x' | x' < real q1 }``)
					    THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
					          THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC])
				    THEN DISCH_TAC
				    THEN ASM_REWRITE_TAC[]
                                    THEN `{x' | real q0 < x'} IN subsets borel` by METIS_TAC [borel_open,OPEN_INTERVAL_RIGHT]
				    THEN `{x' | x' < real q1} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT]
			
 			           THEN  METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
			    THEN KNOW_TAC(``  {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2} =
			    	 	       {y' | y' < real q0} INTER
					       {y' | 0 <= y'} INTER
					       {y' | y' < real q2}``)
			        THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
				      THEN METIS_TAC[CONJ_ASSOC, CONJ_COMM])
 			   THEN DISCH_TAC				      
			   THEN ASM_REWRITE_TAC[]
			   THEN `{y' | y' < real q0} IN subsets borel /\
			         {y' | 0 <= y'} IN subsets borel /\
			         {y' | y' < real q2} IN subsets borel ` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_LEFT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, CLOSED_INTERVAL_RIGHT]
    		          THEN 	METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
	           THEN DISCH_TAC 
		   THEN METIS_TAC [sigma_sets_basic])
             THEN  `{z' | real q1 < z' /\ real q2 < z' /\ 0<= z' /\ z' <= t} =
	            {z' | real q1 < z'} INTER {z'| real q2 < z'} INTER
		    {z' | 0<= z'} INTER {z' | z' <= t}` by (SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC ])
              THEN ASM_REWRITE_TAC[]
	      THEN ` {z' | real q1 < z'} IN subsets borel /\ {z' | real q2 < z'} IN subsets borel /\                      {z' | 0 <= z'} IN subsets borel /\  {z' | z' <= t} IN subsets borel`
	         by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT]
             THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
        THEN DISCH_TAC
        THEN METIS_TAC [sigma_sets_basic])
   THEN1( KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
         THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
	         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(``{(a:real->bool) CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET
          POW univ(:real#real) ``)
             THEN1( METIS_TAC [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
	 THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[])
THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]	
THEN FULL_SIMP_TAC std_ss[GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV]);



val BIGUNION_event2_shared_spare_subevent0_measurable_set = store_thm(
  "BIGUNION_event2_shared_spare_subevent0_measurable_set",
  ``!t q1 q2.   {(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} CROSS
          {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} IN
	  measurable_sets (pair_measure (pair_measure lborel lborel) lborel)  ``,

RW_TAC std_ss [GSYM BIGUNION_event2_shared_spare_subevent0]

THEN KNOW_TAC (``!q0. q0 IN Q_set ==> (({x' | real q0 < x' /\ x' < real q1} CROSS
     {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2}) CROSS
     {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t})   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[event2_shared_spare_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
   BIGUNION
    {{x' | real q0 < x' /\ x' < real q1} CROSS
     {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2} CROSS
     {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} |
      q0 IN Q_set} = BIGUNION
  (IMAGE (\q0. ({x' | real q0 < x' /\ x' < real q1} CROSS
     {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2}) CROSS
     {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );


val BIGUNION_event2_shared_spare_subevent = store_thm(
  "BIGUNION_event2_shared_spare_subevent",
  `` !t q2. BIGUNION
           {{(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} CROSS
            {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} |
           q1 IN Q_set}  =
	   {((x',y'),z') |
	    y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t}``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN RW_TAC std_ss [LAMBDA_PAIR]
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
              x IN s <=>
            (?x'.
                  (FST x,T) =
                  (\(x',y').
                       ((x',y'),
                        y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1))
                    x') /\ real q1 < SND x /\ real q2 < SND x /\
             0 <= SND x /\ SND x <= t``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)
          THEN `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN KNOW_TAC(``(\ (x':real,y').
             ((x',y'), y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1)) x''
              = (x'', SND x'' < FST x'' /\ 0 <= SND x'' /\ SND x'' < real q2 /\ FST x''< real q1) ``)
                THEN1 ( KNOW_TAC (`` ((\ ((x':real),y').
        ((x',y'), y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1)) x'') =  (\ (x':real,y').
        ((x',y'), y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1)) (FST x'' , SND x'')``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC
	 THEN Q.EXISTS_TAC(`(FST (FST (x:(real#real)#real)), SND (FST x), SND x) `)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]
THEN  KNOW_TAC(``?q1. q1 IN Q_set/\ Normal (FST  (x':real#real#real))  < q1 /\  q1 < Normal (SND (SND x'))``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r1. q1 = Normal r1``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (FST   (x':real#real#real)) < Normal(SND  (SND x'))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{(x:(real#real)#real)| (SND (FST x) < FST (FST x) /\ 0 <= SND (FST x) /\
                  			  SND (FST x) < real q2 /\ FST (FST x) < real (Normal r1)) /\
                			  real (Normal r1) < SND x /\ real q2 < SND x /\ 0 <= SND x /\
                			  SND x <= t}``)
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r1`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);


val BIGUNION_event2_shared_spare_subevent_measurable_set = store_thm(
  "BIGUNION_event2_shared_spare_subevent_measurable_set",
  `` !t q2.
	  {((x',y'),z') |
          y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
          0 <= z' /\ z' <= t} IN
       measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,


RW_TAC std_ss [GSYM BIGUNION_event2_shared_spare_subevent]
THEN KNOW_TAC (``!q1. q1 IN Q_set ==> ({(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} CROSS
     {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t})   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[BIGUNION_event2_shared_spare_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
   {{(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} CROSS
     {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} |
     q1 IN Q_set} = BIGUNION
  (IMAGE (\q1. {(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} CROSS
     {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );



val BIGUNION_event2_shared_spare_subevent2 = store_thm(
  "BIGUNION_event2_shared_spare_subevent2",
  `` !t. BIGUNION {{((x',y'),z') |
          y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
          0 <= z' /\ z' <= t} | q2 IN Q_set} =
	 {((x',y'),z') |
          y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}  ``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN RW_TAC std_ss [LAMBDA_PAIR]
           THEN UNDISCH_TAC (`` !(x:(real#real)#real).
                x IN s <=>
             ?x'.
                  (x,T) =
                 (\(x',y',z').
                      (((x',y'),z'),
                       y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
                       real q2 < z' /\ 0 <= z' /\ z' <= t)) x'``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:(real#real)#real `)
          THEN  `x:(real#real)#real = ((FST(FST x), SND (FST x)), SND x)` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `FST (x:(real#real)#real) = (FST (FST x), SND (FST x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN KNOW_TAC(``(\ (x':real,y',z').
              (((x',y'),z'),
               y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
               real q2 < z' /\ 0 <= z' /\ z' <= t)) x''
               = ( ((FST(x'':real#real#real), FST (SND x'')),SND (SND x'')) ,
	       FST (SND x'') < FST  x'' /\  0 <= FST (SND x'') /\ FST (SND x'') < real q2 /\
	       FST x'' < SND (SND x'') /\ real q2 < SND (SND x'') /\
	       0 <= SND (SND x'') /\ SND (SND x'') <= t) ``)
             THEN1 ( KNOW_TAC (``  (\ (x',y',z').
         (((x',y'),z'),
              y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
               real q2 < z' /\ 0 <= z' /\ z' <= t))(x'':real#real#real) =
	       (\ (x',y',z').
        (((x',y'),z'),
               y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
               real q2 < z' /\ 0 <= z' /\ z' <= t)) (FST x'', FST (SND x'') , SND (SND x''))``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC

	 THEN Q.EXISTS_TAC(`(FST (FST (x:(real#real)#real)), SND (FST x), SND x) `)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]
THEN KNOW_TAC(``?q2. q2 IN Q_set/\ q2 < Normal (SND  (SND  (x':real#real#real))) /\ Normal (FST (SND x')) < q2 ``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r2. q2 = Normal r2``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal(FST (SND x')) <Normal  (SND (SND (x':real#real#real))) ``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{x| SND (FST (x:(real#real)#real)) < FST (FST x)  /\ 0 <= SND (FST x) /\
                       SND (FST x) < real (Normal r2) /\ FST (FST x) < SND x /\
		       real (Normal r2) < SND x /\ 0 <= SND x /\ SND x <=t }``)

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r2`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
          THEN Q.EXISTS_TAC(`(FST(FST (x'':(real#real)#real)) , SND (FST x''), SND x'') `)  
      
          THEN  FULL_SIMP_TAC std_ss[PAIR])
THEN RW_TAC std_ss[]
THEN FULL_SIMP_TAC std_ss[PAIR]);



val BIGUNION_event2_shared_spare_subevent2_measurable_set = store_thm(
  "BIGUNION_event2_shared_spare_subevent2_measurable_set",
  `` !t.
	  {((x',y'),z') |
            y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} IN
       measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``,
 RW_TAC std_ss [GSYM BIGUNION_event2_shared_spare_subevent2]
THEN KNOW_TAC (``!q2. q2 IN Q_set ==> ({((x',y'),z') |
     y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
     0 <= z' /\ z' <= t} )   IN measurable_sets (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[BIGUNION_event2_shared_spare_subevent_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
     {{((x',y'),z') |
    y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
     0 <= z' /\ z' <= t} |
     q2 IN Q_set} = BIGUNION
  (IMAGE (\q2. {((x',y'),z') |
      y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
     0 <= z' /\ z' <= t})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure (pair_measure lborel lborel) lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );


val event2_shared_spare_Axy_measurable_set = store_thm(
  "event2_shared_spare_Axy_measurable_set",
  `` !z. {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} IN
         measurable_sets (pair_measure lborel lborel)``,

RW_TAC std_ss[]
THEN KNOW_TAC(``  {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}  =
                  BIGUNION {{x'|  x' < z /\  real q < x':real } CROSS
  	                  {y'|  0 <= y' /\ y' < z /\ y' < real q } | q IN Q_set} ``)

     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EQ_TAC
                THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]	    
           	       THEN KNOW_TAC (`` 0 <=  SND (x:real#real) /\ SND x < z /\ SND x < FST x /\ FST x < z   ``)
   		            THEN1(FULL_SIMP_TAC std_ss[ LAMBDA_PAIR])
           	       THEN DISCH_TAC
           	       THEN KNOW_TAC (``Normal (SND (x:real#real)) < Normal (FST x)``)
	                    THEN1(METIS_TAC [extreal_lt_eq])
                       THEN DISCH_TAC
	               THEN KNOW_TAC(``?q. q IN Q_set/\ Normal (SND x) < q /\  q < Normal (FST x)``)
	                    THEN1(METIS_TAC [Q_DENSE_IN_R])
                       THEN RW_TAC std_ss[]
	               THEN KNOW_TAC(``FST (x:real#real) = real (Normal(FST x))``)
                            THEN1(RW_TAC std_ss [GSYM real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``SND (x:real#real) = real(Normal (SND x))``)
	                    THEN1(METIS_TAC[real_normal])
                       THEN DISCH_TAC 
	               THEN KNOW_TAC(``?r. q = Normal r``) 
                            THEN1(METIS_TAC[Q_not_infty])
  	               THEN RW_TAC std_ss[]
		       THEN Q.EXISTS_TAC (`{y| 0 <= SND y /\ SND y < z /\ SND y < real (Normal r) /\ real (Normal r) < FST y /\ FST y < z}`)
		       THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
		             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                             THEN1(METIS_TAC[extreal_real_lt , extreal_not_infty])
                       THEN Q.EXISTS_TAC(`Normal r`)     
                       THEN SRW_TAC[] [GSPECIFICATION, EXTENSION]
                       THEN EQ_TAC
                       THEN RW_TAC std_ss[])
	               THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                       THEN RW_TAC std_ss [LAMBDA_PAIR]
                       THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR] )
THEN DISCH_TAC	   
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!q. q IN Q_set ==>{x' | x' < z /\ real q < x'} CROSS
     {y' | 0 <= y' /\ y' < z /\ y' < real q} IN measurable_sets (pair_measure lborel lborel) ``)
     THEN1 (RW_TAC std_ss [pair_measure, measure_of, lborel,m_space_def, measurable_sets_def ]
           THEN1 (KNOW_TAC(``{x' | x' < z /\ real q < x'} CROSS
     {y' | 0 <= y' /\ y' < z /\ y' < real q} IN {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)

                  THEN1 (KNOW_TAC(`` {y' | 0 <= y' /\ y' < z /\ y' < real q} IN subsets borel``) 
                         THEN1(KNOW_TAC(`` {y' | 0 <= y' /\ y' < z /\ y' < real q} =
		                        {y' | 0 <= y'} INTER
				       {y' | y' < z } INTER
				       {y' | y' < real q}``) 
                               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			             THEN METIS_TAC [CONJ_ASSOC, CONJ_COMM])
			 THEN DISCH_TAC
			 THEN ONCE_ASM_REWRITE_TAC[]
                         THEN `{y' | 0 <= y'} IN subsets borel /\
				       {y' | y' < z } IN subsets borel /\
				       {y' | y' < real q} IN subsets borel`
			    by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT] 	        
                        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
	   THEN DISCH_TAC
           THEN KNOW_TAC (`` {x' | x' < z /\ real q < x' }  IN subsets borel ``)
                THEN1(`{x' | x' < z /\ real q < x'}  =
	                  {x' | real q < x'} INTER
	                  {x' |  x' < z} ` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CONJ_COMM]
	                THEN `{x' | real q < x'} IN subsets borel /\
		              {x' |  x' < z} IN subsets borel`   by METIS_TAC [OPEN_INTERVAL_LEFT, borel_open,OPEN_INTERVAL_RIGHT, borel_open,borel_closed, CLOSED_INTERVAL_RIGHT, CLOSED_INTERVAL_LEFT]
		        THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER])
          THEN DISCH_TAC
          THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	  THEN Q.EXISTS_TAC (`({x' | x' < z /\ real q < x'},
        {y' | 0 <= y' /\ y' < z /\ y' < real q})   `)
	  THEN SRW_TAC [] [GSPECIFICATION, EXTENSION])
		 
         THEN DISCH_TAC
         THEN METIS_TAC [sigma_sets_basic])
         THEN KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
     THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
     THEN KNOW_TAC(`` {a CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET POW univ(:real#real)``)
          THEN1( RW_TAC std_ss [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
     THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN KNOW_TAC(``BIGUNION
    {{x' | x' < z /\ real q < x'} CROSS
     {y' | 0 <= y' /\ y' < z /\ y' < real q} | q IN Q_set} =
     BIGUNION
  (IMAGE (\q. {x' | x' < z /\ real q < x'} CROSS
     {y' | 0 <= y' /\ y' < z /\ y' < real q} )
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel lborel)``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel])
THEN DISCH_TAC
THEN MP_TAC(Q.ISPECL[`pair_measure lborel lborel ` ,` (\q.
              {x' | x' < z /\ real q < x'} CROSS
     {y' | 0 <= y' /\ y' < z /\ y' < real q})` ] MEASURE_SPACE_BIGUNION_Q)
THEN RW_TAC std_ss[]);



val event2_shared_spare_indicator_of_indicator_Axy = store_thm(
  "event2_shared_spare_indicator_of_indicator_Axy",
  `` ! x y z (t:real). indicator_fn {((x',y'),z') |
            y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}((x,y),z) =
	    indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y) *
		              indicator_fn {z' | 0 <= z' /\ z' <=t } z  ``,

RW_TAC std_ss [indicator_fn_def]
      THEN1(RW_TAC std_ss[mul_lone])
      THEN1(`((x:real,y),z) IN
     {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
     <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`((x:real,y),z) IN
     {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
     <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t) x' y' z'}` by RW_TAC std_ss[]

     THEN `(x:real,y) IN
     {(x',y') |
         0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
     <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' < z /\ y'' < x'' /\ x'' < z) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`((x:real,y),z) IN
     {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
     <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t) x' y' z'}` by RW_TAC std_ss[]

     THEN `(x:real,y) IN
     {(x',y') |
         0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
     <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' < z /\ y'' < x'' /\ x'' < z) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
THEN1(`((x:real,y),z) IN
     {((x',y'),z') |
         y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
     <=>
      ((x,y),z) IN
      {((x',y'),z') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t) x' y' z'}` by RW_TAC std_ss[]

     THEN `(x:real,y) IN
     {(x',y') |
         0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
     <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''.
          0 <= y'' /\ y'' < z /\ y'' < x'' /\ x'' < z) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_1, IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

 THEN RW_TAC std_ss [mul_rzero, mul_lzero]);



val BIGUNION_event2_shared_spare_form2_subevent0 = store_thm(
  "BIGUNION_event2_shared_spare_form2_subevent0",
  `` !t q1 q2.BIGUNION {{z' | real q1 < z' /\ real q2 < z' /\ 0<= z' /\ z' <= t} CROSS  ( {x' | real q0 < x' /\ x' < real q1} CROSS {y'| y' < real q0 /\ 0 <= y' /\ y' < real q2 } ) | q0 IN Q_set}=
              {z' | real q1 < z' /\ real q2 < z' /\ 0<= z' /\ z' <=t} CROSS {(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1}  ``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1(STRIP_TAC
           THEN UNDISCH_TAC (`` !(x:real#real#real).
              
           x IN s <=>
                (real q1 < FST x /\ real q2 < FST x /\ 0 <= FST x /\ FST x <= t) /\
             (real q0 < FST (SND x) /\ FST (SND x) < real q1) /\
             SND (SND x) < real q0 /\ 0 <= SND (SND x) /\
             SND (SND x) < real q2``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:real#real#real `)
          THEN `x:real#real#real = (FST x, FST (SND x), SND (SND x))` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `SND (x:real#real#real) = (FST (SND x), SND (SND x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN EXISTS_TAC(``SND (x:real#real#real) ``)
           THEN RW_TAC std_ss [LAMBDA_PAIR]
	   THEN METIS_TAC [REAL_LT_TRANS])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]

THEN  KNOW_TAC(``?q0. q0 IN Q_set/\ Normal (SND (x':real#real) ) < q0 /\  q0 < Normal (FST x')``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r0. q0 = Normal r0``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (SND (SND (x:real#real#real))) < Normal( FST (SND x))``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC

THEN EXISTS_TAC (``{x| real q1 < FST x /\ real q2 < FST x /\ 0 <= FST x /\
                 FST x <= t /\
                real (Normal r0) < FST (SND x) /\ FST (SND x) < real q1 /\
                SND (SND x) < real (Normal r0) /\ 0 <= SND (SND x) /\
                SND (SND x) < real q2}  ``)
     
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r0`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);


val event2_shared_spare_form2_subevent0_measurable_set = store_thm(
  "event2_shared_spare_form2_subevent0_measurable_set",
  ``!t q0 q1 q2. {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS
            ({x' | real q0 < x' /\ x' < real q1} CROSS
             {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2})
             IN measurable_sets (pair_measure lborel (pair_measure lborel lborel))``,

RW_TAC std_ss [measurable_sets_def, pair_measure, measure_of, lborel, m_space_def]
    THEN1 (KNOW_TAC(`` {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS
     	              ({x' | real q0 < x' /\ x' < real q1} CROSS
          	       {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2}) IN
                         {a CROSS b |
			  a IN subsets borel /\
     			  b IN
     			  sigma_sets (univ(:real) CROSS univ(:real))
       			  {a CROSS b | a IN subsets borel /\ b IN subsets borel}}``)
           THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	         THEN EXISTS_TAC (``({z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t},
		 {x' | real q0 < x' /\ x' < real q1} CROSS
                 {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2} ) ``)
                THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                     THEN1( `{z' | real q1 < z' /\ real q2 < z' /\ 0<= z' /\ z' <= t} =
	                     {z' | real q1 < z'} INTER {z'| real q2 < z'} INTER
		             {z' | 0<= z'} INTER {z' | z' <= t}` by (SRW_TAC [] [GSPECIFICATION, EXTENSION] THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC ])
              		     THEN ASM_REWRITE_TAC[]
	      		     THEN ` {z' | real q1 < z'} IN subsets borel /\ {z' | real q2 < z'} IN subsets borel /\                     {z' | 0 <= z'} IN subsets borel /\  {z' | z' <= t} IN subsets borel`
	         by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT, borel_closed, CLOSED_INTERVAL_LEFT, CLOSED_INTERVAL_RIGHT]
                            THEN METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
	THEN KNOW_TAC(``{x' | real q0 < x' /\ x' < real q1} CROSS
   	     		{y' | y' < real q0 /\ 0 <= y' /\ y' < real q2} IN
				       {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
                          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
			        THEN EXISTS_TAC (``({x' | real q0 < x' /\ x' < real q1} ,
   				     		{y' | y' < real q0 /\ 0 <= y' /\ y' < real q2}) ``)
			        THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
                                     THEN1( KNOW_TAC(``{x' | real q0 < x' /\ x' < real q1 } =
					    {x' | real q0 < x'} INTER
					    {x' | x' < real q1 }``)
					    THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
					          THEN METIS_TAC [CONJ_COMM, CONJ_ASSOC])
				    THEN DISCH_TAC
				    THEN ASM_REWRITE_TAC[]
                                    THEN `{x' | real q0 < x'} IN subsets borel` by METIS_TAC [borel_open,OPEN_INTERVAL_RIGHT]
				    
				    THEN `{x' | x' < real q1} IN subsets borel` by METIS_TAC [borel_open, OPEN_INTERVAL_LEFT]
			
 			           THEN  METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
			    THEN KNOW_TAC(``  {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2 } =
			    	 	       {y' | y' < real q0 } INTER
					       {y' | 0 <= y'} INTER
					       {y' | y' < real q2 }``)
			        THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
				      THEN METIS_TAC[CONJ_ASSOC, CONJ_COMM])
 			   THEN DISCH_TAC				      
			   THEN ASM_REWRITE_TAC[]
			   THEN `{y' | y' < real q0 } IN subsets borel /\
			         {y' | 0<= y'} IN subsets borel /\
			         {y' | y' < real q2} IN subsets borel ` by METIS_TAC [borel_open, borel_closed,  CLOSED_INTERVAL_RIGHT, OPEN_INTERVAL_LEFT, OPEN_INTERVAL_RIGHT]
    		          THEN 	METIS_TAC[sigma_algebra_borel, algebra_def, sigma_algebra_def, ALGEBRA_ALT_INTER, INTER_ASSOC])
	           THEN DISCH_TAC 
		   THEN METIS_TAC [sigma_sets_basic])		    
        THEN DISCH_TAC
        THEN METIS_TAC [sigma_sets_basic])
   THEN1( KNOW_TAC(``univ(:real) CROSS univ(:real) = univ(:real#real)``)
               THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
         THEN KNOW_TAC (``univ(:real # real) CROSS univ(:real) = univ(:(real#real)#real) ``)
	         THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
         THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN KNOW_TAC(``{(a:real->bool) CROSS b | a IN subsets borel /\ b IN subsets borel} SUBSET
          POW univ(:real#real) ``)
             THEN1( METIS_TAC [GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV])
	 THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[])
	 
THEN KNOW_TAC(``univ(:real) CROSS (univ(:real) CROSS univ(:real)) = univ(:real#real#real)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[]	
THEN FULL_SIMP_TAC std_ss[GSYM UNIV_FUN_TO_BOOL, SUBSET_UNIV]);



val BIGUNION_event2_shared_spare_form2_subevent0_measurable_set = store_thm(
  "BIGUNION_event2_shared_spare_form2_subevent0_measurable_set",
  ``!t q1 q2.  {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS
               {(x',y') |y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} IN
	  measurable_sets (pair_measure lborel (pair_measure lborel lborel) )  ``,

RW_TAC std_ss [GSYM BIGUNION_event2_shared_spare_form2_subevent0]

THEN KNOW_TAC (``!q0. q0 IN Q_set ==> ({z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS({x' | real q0 < x' /\ x' < real q1} CROSS
      {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2}))   IN measurable_sets (pair_measure lborel (pair_measure lborel lborel) )``)
     THEN1(METIS_TAC[event2_shared_spare_form2_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
   BIGUNION
    { {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS({x' | real q0 < x' /\ x' < real q1} CROSS
      {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2})
     |
      q0 IN Q_set} = BIGUNION
  (IMAGE (\q0.   {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t}CROSS
                 ({x' | real q0 < x' /\ x' < real q1} CROSS
      {y' | y' < real q0 /\ 0 <= y' /\ y' < real q2}))
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel (pair_measure lborel lborel))``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel,sigma_finite_measure_pair_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );



val BIGUNION_event2_shared_spare_form2_subevent = store_thm(
  "BIGUNION_event2_shared_spare_form2_subevent",
  `` !t q2. BIGUNION
           {{z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS({(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1}) |
           q1 IN Q_set}  =
	   {(z', (x',y')) |
	    y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t}``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN ONCE_REWRITE_TAC [LAMBDA_PAIR]
           THEN UNDISCH_TAC (`` !(x:real#real#real).
              x IN s <=>
            (real q1 < FST x /\ real q2 < FST x /\ 0 <= FST x /\ FST x <= t) /\
             ?x'.
                 (SND x,T) =
                 (\(x',y').
                      ((x',y'),
                       y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1))
                   x'``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:real#real#real `)
          THEN `x:real#real#real = (FST x, FST (SND x), SND (SND x))` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN `SND (x:real#real#real) = (FST (SND x), SND (SND x))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
	   THEN `x':real#real = (FST x', SND x')` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN KNOW_TAC(``(\ (x':real,y').
             ((x',y'),y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1)) x''
              = (x'', SND x'' < FST x'' /\ 0 <= SND x'' /\ SND x'' < real q2 /\ FST x''< real q1) ``)
                THEN1 ( KNOW_TAC (`` ((\ ((x':real),y').
        ((x',y'),y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1)) x'') =  (\ (x':real,y').
        ((x',y'),y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1)) (FST x'' , SND x'')``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC
	 THEN KNOW_TAC(`` (\ (x',y').
              ((x',y'),y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1))
           x'' = (x'', SND x'' < FST x'' /\ 0<= SND x'' /\ SND x'' < real q2 /\ FST x'' < real q1)``) 
             THEN1(`(x'':real#real) = (FST x'', SND x'')` by RW_TAC std_ss[PAIR]
	           THEN ONCE_ASM_REWRITE_TAC[]
		   THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
	THEN DISCH_TAC
	THEN FULL_SIMP_TAC std_ss[]
	 THEN Q.EXISTS_TAC(`(FST x, x'') `)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
	 THEN `x'':real#real = (FST x'', SND x'')` by RW_TAC std_ss[PAIR]
	 THEN ONCE_ASM_REWRITE_TAC[]
	 THEN  RW_TAC std_ss[LAMBDA_PROD]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]

THEN  KNOW_TAC(``?q1. q1 IN Q_set/\ q1 < Normal (FST  (x':real#real#real))   /\   Normal (FST (SND x')) < q1``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r1. q1 = Normal r1``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal  (FST (SND  (x:real#real#real))) < Normal (FST x)``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{x|  real (Normal r1) < FST (x:real#real#real) /\ real q2 < FST x /\ 0 <= FST x /\
                 FST x <= t /\ SND (SND x) < FST (SND x) /\
                0 <= SND (SND x) /\ SND (SND x) < real q2 /\
                FST (SND x) < real (Normal r1)}``)
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r1`    
THEN RW_TAC std_ss[]
THEN EQ_TAC
THEN RW_TAC std_ss[]);




val BIGUNION_event2_shared_spare_form2_subevent_measurable_set = store_thm(
  "BIGUNION_event2_shared_spare_form2_subevent_measurable_set",
  `` !t q2.
	  {(z',(x',y')) |
          y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
          0 <= z' /\ z' <= t} IN
       measurable_sets (pair_measure lborel (pair_measure lborel lborel) )``,


RW_TAC std_ss [GSYM BIGUNION_event2_shared_spare_form2_subevent]
THEN KNOW_TAC (``!q1. q1 IN Q_set ==> {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS ({(x',y') | y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1} )   IN measurable_sets (pair_measure lborel (pair_measure lborel lborel))``)
     THEN1(METIS_TAC[BIGUNION_event2_shared_spare_form2_subevent0_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
   {   {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS ({(x',y') |y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1}) 
    |
     q1 IN Q_set} = BIGUNION
  (IMAGE (\q1.   {z' | real q1 < z' /\ real q2 < z' /\ 0 <= z' /\ z' <= t} CROSS ({(x',y') |y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < real q1}))
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel (pair_measure lborel lborel) )``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel, sigma_finite_measure_pair_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );



val BIGUNION_event2_shared_spare_form2_subevent2 = store_thm(
  "BIGUNION_event2_shared_spare_form2_subevent2",
  `` !t. BIGUNION {{(z',(x',y')) |
          y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
          0 <= z' /\ z' <= t} | q2 IN Q_set} =
	 {(z',(x',y')) |
          y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}  ``,

SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN EQ_TAC
     THEN1( SRW_TAC [] [GSPECIFICATION, EXTENSION]
           THEN ONCE_REWRITE_TAC[LAMBDA_PAIR]
           THEN UNDISCH_TAC (`` !(x:real#real#real).
                x IN s <=>
             ?x'.
                  (x,T) =
                 (\(z',x',y').
                      ((z',x',y'),
                       y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
                       real q2 < z' /\ 0 <= z' /\ z' <= t)) x'``)
          THEN  DISCH_TAC
          THEN POP_ASSUM (MP_TAC o Q.SPEC `x:real#real#real `)
          THEN  `x:real#real#real = ((FST x), FST (SND x), SND (SND x))` by  METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD]
           THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
	    THEN `x':real#real#real = (FST x', FST (SND  x'), SND (SND x'))` by METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD, ETA_AX]
            
	   THEN KNOW_TAC(``(\ (z',x':real,y').
                ( (z',x',y'),
               y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
               real q2 < z' /\ 0 <= z' /\ z' <= t)) x'=
	       ((FST (x':real#real#real) , FST (SND x'), SND (SND x')), SND (SND x') < FST (SND x') /\ 0 <= SND (SND x') /\ SND (SND x') < real q2 /\ FST (SND x') < FST x' /\ real q2 < FST x' /\ 0 <= FST x' /\ FST x' <= t) ``)
             THEN1 ( KNOW_TAC (``  (\ (z',x',y').
        ((z',x',y'),
           y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
               real q2 < z' /\ 0 <= z' /\ z' <= t)) (x':real#real#real) =
	       (\ (z',x',y').
        ((z',x',y'),
          y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\
               real q2 < z' /\ 0 <= z' /\ z' <= t)) (FST (x':real#real#real), FST (SND x'), SND (SND x'))``)
                      THEN1(METIS_TAC [PAIR])
                THEN DISCH_TAC
                THEN  ONCE_ASM_REWRITE_TAC[]
     		THEN RW_TAC std_ss [PAIR, LAMBDA_PROD, LAMBDA_PAIR])
         THEN DISCH_TAC
	 THEN Q.EXISTS_TAC(`(FST( x':real#real#real), FST (SND x'), SND (SND x'))`)
	 THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION, PAIR, LAMBDA_PROD, ETA_AX]
  	 THEN METIS_TAC[REAL_LT_LE, REAL_LE_LT, REAL_LT_TRANS, REAL_LE_TRANS, LAMBDA_PAIR, PAIR, LAMBDA_PROD])
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN FULL_SIMP_TAC std_ss [LAMBDA_PAIR]
THEN KNOW_TAC(``?q2. q2 IN Q_set/\ q2 < Normal  (FST  (x':real#real#real)) /\ Normal (SND(SND  x')) < q2 ``)
     THEN1(METIS_TAC [Q_DENSE_IN_R, extreal_lt_eq])
THEN STRIP_TAC
THEN KNOW_TAC(``?r2. q2 = Normal r2``) 
     THEN1(METIS_TAC[Q_not_infty])
THEN RW_TAC std_ss[]
THEN RW_TAC std_ss[]
THEN KNOW_TAC (``Normal(FST(SND  x)) < Normal   (FST (x:real#real#real)) ``)
     THEN1(METIS_TAC [extreal_lt_eq])
THEN DISCH_TAC
THEN EXISTS_TAC (``{x|  SND (SND x) < FST (SND x) /\ 0 <= SND (SND x) /\
                SND (SND x) < real (Normal r2) /\ FST (SND x) < FST x /\
                real (Normal r2) < FST x /\ 0 <= FST x /\ FST x <= t }``)

THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]

     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
     THEN1( RW_TAC std_ss [real_normal]
           THEN METIS_TAC [extreal_lt_eq])
THEN Q.EXISTS_TAC `Normal r2`    
THEN RW_TAC std_ss[]);



val BIGUNION_event2_shared_spare_form2_subevent2_measurable_set = store_thm(
  "BIGUNION_event2_shared_spare_form2_subevent2_measurable_set",
  `` !t.
	  {(z',(x',y')) |
           y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} IN
       measurable_sets (pair_measure lborel (pair_measure lborel lborel) )``,

RW_TAC std_ss [GSYM BIGUNION_event2_shared_spare_form2_subevent2]
THEN KNOW_TAC (``!q2. q2 IN Q_set ==> ({(z',(x',y')) |
     y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
     0 <= z' /\ z' <= t} )   IN measurable_sets (pair_measure lborel (pair_measure lborel lborel))``)
     THEN1(METIS_TAC[BIGUNION_event2_shared_spare_form2_subevent_measurable_set])
THEN DISCH_TAC
THEN KNOW_TAC(``  
 BIGUNION
     {{(z',(x',y')) |
     y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
     0 <= z' /\ z' <= t} |
     q2 IN Q_set} = BIGUNION
  (IMAGE (\q2. {(z',(x',y')) |
       y' < x' /\ 0 <= y' /\ y' < real q2 /\ x' < z' /\ real q2 < z' /\
     0 <= z' /\ z' <= t})
   Q_set)``)
     THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION, IMAGE_DEF])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``measure_space (pair_measure lborel (pair_measure lborel lborel) )``)
     THEN1(METIS_TAC[ measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel, sigma_finite_measure_pair_lborel])
THEN DISCH_TAC
THEN DEP_REWRITE_TAC[MEASURE_SPACE_BIGUNION_Q]
THEN RW_TAC std_ss[] );



val prob_event_3rv_2indep_snd = store_thm("prob_event_3rv_2indep_snd",
  `` ! p X Y Z M1 M2  A.

prob_space p /\ measure_space M1 /\ measure_space M2 /\
     sigma_finite_measure M1 /\ sigma_finite_measure M2 /\
     indep_varp p M1 (\x. (X x,Y x)) M2 Z /\
     A IN measurable_sets (pair_measure M1 M2) ==>
     (prob p (PREIMAGE (\x. ((X x,Y x),Z x)) A INTER p_space p) =
      pos_fn_integral (distr p M2 Z)
        (\z.
           pos_fn_integral (distr p M1 (\x. (X x,Y x)))
             (\(x,y). indicator_fn A ((x,y),z))))``,

RW_TAC std_ss []
THEN KNOW_TAC(``prob p (PREIMAGE (\x. (((X:'a->'b) x,(Y:'a->'c) x),(Z:'a->'d) x)) A INTER p_space p) =
                 distribution p (\x. (X x, Y x), Z x) A  ``)
     THEN1(METIS_TAC [distribution_def])		  
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``(\x. ((X:'a->'b) x, (Y:'a->'c) x)) IN measurable (m_space p, measurable_sets p) (m_space M1, measurable_sets M1) /\ (Z:'a->'d) IN measurable (m_space p, measurable_sets p) (m_space M2, measurable_sets M2)``)
     THEN1(METIS_TAC [random_variable_def, indep_varp_def, p_space_def, events_def])
THEN DISCH_TAC

THEN KNOW_TAC(``(\x. ((X:'a->'b) x, (Y:'a-> 'c) x) , (Z:'a->'d) x )  IN measurable (m_space p, measurable_sets p) (m_space (pair_measure M1 M2), measurable_sets (pair_measure M1 M2))``)
     THEN1(METIS_TAC[p_space_def, m_space_def, measurable_Pair, prob_space_def])
THEN DISCH_TAC
THEN KNOW_TAC (``measure_space (pair_measure (M1: ('b#'c->bool)#(('b#'c->bool)->bool)#(('b#'c->bool)->extreal)) (M2: ('d->bool)#(('d->bool)->bool)#(('d->bool)->extreal)))``)
     THEN1(METIS_TAC [measure_space_pair_measure'])
THEN DISCH_TAC
THEN KNOW_TAC(``random_variable (\x. ((X:'a->'b) x, (Y:'a->'c) x) , (Z:'a->'d) x) p   (m_space (pair_measure M1 M2) , measurable_sets (pair_measure M1 M2)) ``)
    THEN1(METIS_TAC [random_variable_def, measurable_Pair, p_space_def, prob_space_def, events_def, subsets_def, measurable_sets_def, m_space_def, space_def] ) 
THEN DISCH_TAC
THEN KNOW_TAC(``distribution p (\x. (((X:'a->'b) x,(Y:'a->'c) x),(Z:'a->'d) x)) A =
                 pos_fn_integral (distr p (pair_measure M1 M2) (\x. (((X:'a->'b) x,(Y:'a->'c) x),(Z:'a->'d) x)) ) (indicator_fn A)``)

     THEN1( RW_TAC std_ss [distribution_lebesgue_thm2_pos_fn])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN RW_TAC std_ss [indep_varp_pair_distr]
THEN DEP_REWRITE_TAC [GSYM pos_fn_integral_snd]
THEN RW_TAC std_ss []
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
     THEN1(METIS_TAC [measure_space_distr, prob_space_def])
     THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A'`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'b#'c->bool) IN measurable_sets M1``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (\x. (X: 'a->'b) x, (Y:'a->'c) x) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
    THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A''`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'d->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (Z:'a->'d) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
    THEN1(METIS_TAC [indicator_fn_pos_le])
    THEN1( MP_TAC(Q.ISPECL[`(m_space (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d))),
   measurable_sets
     (pair_measure (distr p M1 (\x. (X x,Y x))) (distr p M2 Z)))` ,`1:real` ,`A:('b#'c)#'d->bool` ] IN_MEASURABLE_BOREL_CMUL_INDICATOR)
          THEN RW_TAC std_ss[]
          THEN KNOW_TAC(``sigma_finite_measure (distr p M2 (Z:'a->'d))``)
	      THEN1(RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def] THEN RW_TAC std_ss [GSYM distr]
	           THEN FULL_SIMP_TAC std_ss [sigma_finite_measure]
                   THEN Q.EXISTS_TAC `A''`
                   THEN RW_TAC std_ss[]
                   THEN RW_TAC std_ss [measure_def, distr]
                   THEN RW_TAC std_ss [GSYM prob_def]
                   THEN KNOW_TAC(``(a:'d->bool) IN measurable_sets M2``)
                       THEN1(FULL_SIMP_TAC std_ss [SUBSET_DEF,GSPECIFICATION, EXTENSION])
                   THEN  DISCH_TAC
                   THEN KNOW_TAC (``PREIMAGE (Z:'a->'d) a INTER m_space p IN events p``)
                        THEN1( METIS_TAC [p_space_def, events_def, random_variable_def, IN_MEASURABLE, subsets_def, space_def, m_space_def])
                   THEN DISCH_TAC
                   THEN METIS_TAC [PROB_FINITE])
	  THEN DISCH_TAC	   
          THEN KNOW_TAC(``measure_space (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d)))``)
               THEN1 (METIS_TAC [measure_space_pair_measure', measure_space_distr, random_variable_def, prob_space_def, p_space_def, m_space_def, events_def, subsets_def, measurable_sets_def] )
	  THEN DISCH_TAC
	  THEN KNOW_TAC(``sigma_algebra (m_space (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d))),
   measurable_sets
     (pair_measure (distr p M1 (\x. (X x,Y x))) (distr p M2 Z)))``)
               THEN1(FULL_SIMP_TAC std_ss [measure_space_def])
          THEN DISCH_TAC
          THEN FULL_SIMP_TAC std_ss[]
	  THEN KNOW_TAC(``A IN
       subsets
         (m_space
            (pair_measure (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x))) (distr p M2 (Z:'a->'d))),
          measurable_sets
            (pair_measure (distr p M1 (\x. (X x,Y x)))
               (distr p M2 Z)))``)
	       THEN1(RW_TAC std_ss [subsets_def]
	             THEN RW_TAC std_ss [distr]
	             THEN RW_TAC std_ss [distr, measurable_sets_def, subsets_def, pair_measure, measure_of]
                          THEN1(FULL_SIMP_TAC std_ss[m_space_def]
			  THEN FULL_SIMP_TAC std_ss [pair_measure]
			  THEN FULL_SIMP_TAC std_ss [measurable_sets_def, subsets_def, pair_measure, measure_of])
	             THEN FULL_SIMP_TAC std_ss[m_space_def]
			  THEN FULL_SIMP_TAC std_ss [pair_measure]
			  THEN FULL_SIMP_TAC std_ss [measurable_sets_def, subsets_def, pair_measure, measure_of])
        THEN DISCH_TAC
	THEN FULL_SIMP_TAC std_ss[]
	THEN FULL_SIMP_TAC std_ss [REAL_LE_01]
	THEN KNOW_TAC(``indicator_fn (A:('b#'c)#'d->bool) = (\x. Normal 1 * indicator_fn A x)``)
             THEN1(`indicator_fn (A:('b#'c)#'d->bool) = (\x.  indicator_fn (A:('b#'c)#'d->bool) x)` by  METIS_TAC [ETA_THM]
	            THEN ONCE_ASM_REWRITE_TAC[]
		    THEN `(\x. indicator_fn (A:('b#'c)#'d->bool) x) = (\x. 1:extreal * indicator_fn (A:('b#'c)#'d->bool) x)` by METIS_TAC [mul_lone]
		   THEN METIS_TAC [extreal_of_num_def])
	THEN DISCH_TAC
	THEN ONCE_ASM_REWRITE_TAC[]
	THEN FULL_SIMP_TAC std_ss[])
THEN RW_TAC std_ss [LAMBDA_PROD, PAIR]);


(************************)






val joint_density_3rv_0_snd = store_thm("joint_density_3rv_0_snd",
  `` ! p X Y Z M1 M2  A A1 f_z.

prob_space p /\ measure_space M1 /\ measure_space M2 /\
     sigma_finite_measure M1 /\ sigma_finite_measure M2 /\
     distributed p M2 Z f_z/\
     measure_space (distr p M1 (\x. (X x,Y x)))/\
     (!z. 0<= f_z z) /\
     (!x. x IN m_space (distr p M1 (\x. (X x,Y x)))) /\
     (!x. x IN m_space M2) /\
     A IN measurable_sets (pair_measure M1 M2) /\
     A1 IN measurable_sets (pair_measure M2 M1) /\
     sigma_finite_measure (distr p M1 (\x. (X x,Y x)))/\
     (!x y z. indicator_fn A((x,y),z) = indicator_fn A1(z,(x,y)))==>
     (pos_fn_integral (distr p M2 Z) 
        (\z.
           pos_fn_integral (distr p M1 (\x. (X x,Y x)))
             (\ (x,y). indicator_fn A ((x,y),z))) =
      pos_fn_integral  M2  
        (\z. f_z z *
           pos_fn_integral (distr p M1 (\x. (X x,Y x)))
             (\ (x,y). indicator_fn A ((x,y),z))))``,

RW_TAC std_ss[distributed_def]
THEN  MP_TAC(Q.ISPECL[` f_z:'d->extreal `, `(\z:'d.
          pos_fn_integral (distr p M1 (\x. (X x,Y x)))(\ (x:'b,y:'c). indicator_fn A ((x,y),z)))` , `M2:('d->bool)#(('d->bool)->bool)#(('d->bool)->extreal) ` ]  pos_fn_integral_density_1)
THEN RW_TAC std_ss[]
THEN KNOW_TAC(``A1 IN measurable_sets ( pair_measure M2  (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c)x))))  ``)

     THEN1( UNDISCH_TAC(`` A1 IN measurable_sets (pair_measure (M2: ('d -> bool) # (('d -> bool) -> bool) # (('d -> bool) -> extreal)) (M1:  ('b # 'c -> bool) #
              (('b # 'c -> bool) -> bool) # (('b # 'c -> bool) -> extreal))  ) ``)
     THEN RW_TAC  std_ss[pair_measure, measure_of, measurable_sets_def, distr, m_space_def])
THEN DISCH_TAC
THEN MP_TAC(Q.ISPECL [` indicator_fn (A1:('d#('b#'c) )->bool)`,
`M2: ('d -> bool) # (('d -> bool) -> bool) # (('d -> bool) -> extreal)`,`(distr
               (p :
                  ('a -> bool) #
                  (('a -> bool) -> bool) # (('a -> bool) -> extreal))
               (M1 :
                  ('b # 'c -> bool) #
                  (('b # 'c -> bool) -> bool) #
                  (('b # 'c -> bool) -> extreal))
               (\ (x :'a). ((X :'a -> 'b) x,(Y :'a -> 'c) x))) `] borel_measurable_pos_integral_fst )
THEN RW_TAC std_ss[indicator_fn_pos_le]
THEN KNOW_TAC(`` indicator_fn (A1:('d#('b#'c) )->bool) IN
         measurable
           (m_space (pair_measure M2  (distr p M1 (\x. (X x,Y x))) ),
            measurable_sets (pair_measure M2 (distr p M1 (\x. (X x,Y x))) ))
           Borel ``)

     THEN1( MP_TAC (Q.ISPECL [`(m_space
        (pair_measure (M2 :
              ('d -> bool) #
              (('d -> bool) -> bool) # (('d -> bool) -> extreal))
           (distr
              (p :
                 ('a -> bool) #
                 (('a -> bool) -> bool) # (('a -> bool) -> extreal))
              (M1 :
                 ('b # 'c -> bool) #
                 (('b # 'c -> bool) -> bool) # (('b # 'c -> bool) -> extreal))
              (\ (x :'a). ((X :'a -> 'b) x,(Y :'a -> 'c) x)))
           ),
      measurable_sets (pair_measure M2 (distr p M1 (\ (x :'a). (X x,Y x))) ))
   `, `(A1 :'d#('b # 'c) -> bool)`, `indicator_fn (A1 :'d#('b # 'c) -> bool)`] IN_MEASURABLE_BOREL_INDICATOR)

      THEN RW_TAC std_ss[]
      THEN KNOW_TAC(``sigma_algebra
           (m_space
              (pair_measure
                        (M2 :
                    ('d -> bool) #
                    (('d -> bool) -> bool) # (('d -> bool) -> extreal))(distr
                    (p :
                       ('a -> bool) #
                       (('a -> bool) -> bool) # (('a -> bool) -> extreal))
                    (M1 :
                       ('b # 'c -> bool) #
                       (('b # 'c -> bool) -> bool) #
                       (('b # 'c -> bool) -> extreal))
                    (\ (x :'a). ((X :'a -> 'b) x,(Y :'a -> 'c) x)))
          ),
            measurable_sets
              (pair_measure M2 (distr p M1 (\ (x :'a). (X x,Y x))))) ``)
            THEN1( `measure_space (pair_measure        (M2 :
                    ('d -> bool) #
                    (('d -> bool) -> bool) # (('d -> bool) -> extreal))(distr
                    (p :
                       ('a -> bool) #
                       (('a -> bool) -> bool) # (('a -> bool) -> extreal))
                    (M1 :
                       ('b # 'c -> bool) #
                       (('b # 'c -> bool) -> bool) #
                       (('b # 'c -> bool) -> extreal))
                    (\ (x :'a). ((X :'a -> 'b) x,(Y :'a -> 'c) x)))
                 )` by RW_TAC std_ss[measure_space_pair_measure']
                  THEN FULL_SIMP_TAC std_ss[measure_space_def])
       THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]		  
       THEN KNOW_TAC(``  (A1 :'d # 'b # 'c -> bool) IN
         subsets
           (m_space
              (pair_measure
                 (M2 :
                    ('d -> bool) #
                    (('d -> bool) -> bool) # (('d -> bool) -> extreal))
                 (distr
                    (p :
                       ('a -> bool) #
                       (('a -> bool) -> bool) # (('a -> bool) -> extreal))
                    (M1 :
                       ('b # 'c -> bool) #
                       (('b # 'c -> bool) -> bool) #
                       (('b # 'c -> bool) -> extreal))
                    (\(x :'a). ((X :'a -> 'b) x,(Y :'a -> 'c) x)))),
            measurable_sets
              (pair_measure M2 (distr p M1 (\(x :'a). (X x,Y x)))))``)

           THEN1( METIS_TAC[subsets_def, measurable_sets_def])
      THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!(y:'b#'c) (x:'d). max 0 (indicator_fn A1 (x,y)) = indicator_fn A1 (x,y) ``)
     THEN1(RW_TAC std_ss[]
          THEN METIS_TAC [extreal_max_def, indicator_fn_pos_le])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]	  
THEN FULL_SIMP_TAC std_ss[GSYM PAIR, LAMBDA_PROD]
THEN KNOW_TAC(`` !x:'d.
              0 <=
              pos_fn_integral (distr p M1 (\x. ((X:'a->'b) x,(Y:'a->'c) x)))
                (\(x',y). indicator_fn A1 (x,x',y))``)
    THEN1(RW_TAC std_ss[]
         THEN `!(x:'d) (x':'b) (y:'c). 0<= indicator_fn A1 (x,x',y)` by RW_TAC std_ss[indicator_fn_pos_le]
	 THEN MP_TAC(Q.ISPECL [`(distr (p :
           ('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> extreal))(M1:('b # 'c -> bool) #
                       (('b # 'c -> bool) -> bool) #
                       (('b # 'c -> bool) -> extreal)) (\x. ((X:'a->'b) x,(Y:'a->'c) x)))`,
	 ` (\ (x':'b,y:'c).  indicator_fn (A1:'d#('b#'c)->bool) (x,(x',y)))`]pos_fn_integral_pos)
        THEN RW_TAC std_ss[ indicator_fn_pos_le, PAIR, LAMBDA_PROD, ETA_THM]
	THEN `!(p1:'b, p2:'c). T` by RW_TAC std_ss[GSYM LAMBDA_PROD, GSYM PAIR]
        THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]);	


val event2_shared_spare_form2 = store_thm("event2_shared_spare_form2",
  `` !x y z t:real. indicator_fn
            {(z', (x',y')) |
            y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
            (z, (x,y)) =
          indicator_fn {z' | 0 <= z' /\ z' <= t} z *
          indicator_fn {y' | 0 <= y' /\ y' < z} y *
          indicator_fn {x' | y < x' /\ x' < z} x ``,


REPEAT GEN_TAC
THEN RW_TAC std_ss [indicator_fn_def]
  THEN1(METIS_TAC [mul_lone])

THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      

THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      
THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      
THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      
THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      
THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      
THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
      
THEN1( ` (z,x,y) IN
         {(z',x',y') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
       <=>
      (z,x,y) IN
      {(z',x',y') |
       (\x'' y'' z''.
          y'' < x'' /\ x'' < z'' /\ 0 <= y'' /\ y'' < z'' /\ 0 <= z'' /\ z'' <= t ) x' y' z'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM_2]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
 THEN METIS_TAC [mul_rzero, mul_comm]);



val joint_density_3rv_1_snd = store_thm("joint_density_3rv_1_snd",

``!p X Y Z  f_xy f_z t.
     prob_space p /\ random_variable Z p borel /\
     (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
     distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy /\
     distributed p lborel Z f_z ==>
     (pos_fn_integral lborel 
        (\z. 
           f_z z * pos_fn_integral (distr p (pair_measure lborel lborel) (\x. (X x,Y x)))
             (\ (x,y). indicator_fn {((x',y'),z') |
             y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z))) =
      pos_fn_integral lborel 
        (\z. f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z * 
           pos_fn_integral (pair_measure lborel lborel)
             (\(x,y).
                  f_xy (x,y) *
		     indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))))``,




RW_TAC std_ss[distributed_def]
THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
			       THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                     THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
                               THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(``!z:real.  pos_fn_integral (density (pair_measure lborel lborel) f_xy)
            (\ (x,y).
                 indicator_fn
                   {((x',y'),z') |
                    y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z)) =
		    pos_fn_integral  (pair_measure lborel lborel)
            (\ (x,y). f_xy (x,y) * 
                 indicator_fn
                   {((x',y'),z') |
                    y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z))``)
    THEN1(RW_TAC std_ss[]
          THEN MP_TAC(Q.ISPECL[` f_xy:(real#real)->extreal `, `(\ (x:real,y:real).
          indicator_fn
            {((x',y'),z') |
            y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
            ((x,y),z))` , `(pair_measure lborel lborel)` ]  pos_fn_integral_density_1)
          THEN RW_TAC std_ss[measure_space_pair_measure', measure_space_lborel, sigma_finite_measure_lborel]  THEN KNOW_TAC(``( \ (x:real,y:real).
              indicator_fn
                {((x',y'),z') |
                 y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z)) IN
         measurable
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel ``)
              THEN1(KNOW_TAC(``!(x:real) y z. indicator_fn
          {((x',y'),z') |
           y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}
          ((x,y),z) =  
          indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
            (x,y) * indicator_fn {z' | 0 <= z' /\ z' <= t} z``)
                  THEN1(RW_TAC std_ss[event2_shared_spare_indicator_of_indicator_Axy])
	     THEN DISCH_TAC	  
             THEN ONCE_ASM_REWRITE_TAC[]
	     THEN MP_TAC(Q.ISPECL[`(m_space (pair_measure lborel lborel),
      measurable_sets (pair_measure lborel lborel))`, `real (indicator_fn {z' | 0 <= z' /\ z' <= t:real} z) ` ,` {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z:real}` ]IN_MEASURABLE_BOREL_CMUL_INDICATOR)
            THEN RW_TAC std_ss[sigma_algebra_pair_lborel]
	    THEN KNOW_TAC(``{(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} IN
         subsets
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) ``)
	        THEN1(METIS_TAC [subsets_def, measurable_sets_def, lborel, borel, m_space_def,event2_shared_spare_Axy_measurable_set])
	   THEN DISCH_TAC
	   THEN FULL_SIMP_TAC std_ss[]
	   THEN KNOW_TAC(``   0 <= real (indicator_fn {z' | 0 <= z' /\ z' <= t:real} z) ``)
                THEN1(`!z. indicator_fn {z' | real 0 <= z' /\ z' <= t:real} z <> PosInf /\
		           indicator_fn {z' | real 0 <= z' /\ z' <= t:real} z <> NegInf` by
			   (RW_TAC std_ss [indicator_fn_def]
			        THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def])
		     THEN `0 = real 0` by 
                          (RW_TAC std_ss[real_def]
		          THEN SELECT_ELIM_TAC
                          THEN RW_TAC std_ss[]  		      
                              THEN1(Q.EXISTS_TAC `0`
			            THEN METIS_TAC [extreal_of_num_def])
		          THEN METIS_TAC [extreal_of_num_def, extreal_11])
	             THEN `0<> PosInf /\ 0 <> NegInf` by METIS_TAC [extreal_of_num_def, lt_infty]
	   	     THEN RW_TAC std_ss[extreal_real_le]
	   	     THEN RW_TAC std_ss[indicator_fn_pos_le])
	 THEN DISCH_TAC
	 THEN FULL_SIMP_TAC std_ss[]
	 THEN FULL_SIMP_TAC std_ss[GSYM normal_real_indicator_fn]
	 THEN KNOW_TAC(`` (\x.
              indicator_fn {z' | 0 <= z' /\ z' <= t} z *
              indicator_fn {(x':real,y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                x) =  (\ (x:real,y).
              indicator_fn {z' | 0 <= z' /\ z' <= t} z *
              indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                (x,y) )``)
             THEN1(RW_TAC std_ss[LAMBDA_PROD, PAIR])
	 THEN DISCH_TAC    
         THEN FULL_SIMP_TAC std_ss[mul_comm])
     THEN DISCH_TAC	 
     THEN FULL_SIMP_TAC std_ss[]
     THEN FULL_SIMP_TAC std_ss[LAMBDA_PROD]
     THEN FULL_SIMP_TAC std_ss[indicator_fn_pos_le]
     THEN KNOW_TAC(``(!(p1:real,p2:real). T)``)
          THEN1(RW_TAC std_ss[PAIR, GSYM LAMBDA_PROD])
     THEN DISCH_TAC
     THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC (``!z:real. pos_fn_integral (pair_measure lborel lborel)
            (\ (x,y).
                 f_xy (x,y) *
                 indicator_fn
                   {((x',y'),z') |
                    y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z)) =
		   indicator_fn {z' | 0 <= z' /\ z' <= t} z * 
		   pos_fn_integral (pair_measure lborel lborel)
            (\ (x,y).
                 f_xy (x,y) *
                 indicator_fn
                   {(x',y') |
                    0<= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y)) ``)
    THEN1(RW_TAC std_ss[]
          THEN RW_TAC std_ss[ event2_shared_spare_indicator_of_indicator_Axy]
          THEN  MP_TAC(Q.ISPECL[`pair_measure lborel lborel`,` (\p. real (indicator_fn {z' | 0:real <= z' /\ z' <= t} z))`,` (\ (x:real,y).
          f_xy (x,y) *
          indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z:real}
             (x,y))` ] pos_fn_integral_fun_mul1)
	  THEN RW_TAC std_ss[measure_space_lborel, measure_space_pair_measure', sigma_finite_measure_lborel] 
         THEN KNOW_TAC(``!(x:real#real). x IN m_space (pair_measure lborel lborel)``)
                  THEN1(REWRITE_TAC [m_space_def, pair_measure, measure_of, lborel]
                        THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION, CROSS_DEF]
                        THEN ONCE_ASM_REWRITE_TAC[]
	                THEN METIS_TAC [IN_UNIV])
         THEN DISCH_TAC
         THEN FULL_SIMP_TAC std_ss[]
         THEN KNOW_TAC(`` (!(x:real#real).
              0 <=
              (\ (x,y).
                   f_xy (x,y) *
                   indicator_fn
                     {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
                x) ``)
	      THEN1(RW_TAC std_ss[]
	            THEN KNOW_TAC(`` (\ (x,y).
        f_xy (x,y) *
        indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
     (x:real#real) =
        f_xy x *
        indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x:real#real) ``)
                        THEN1(`(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
                              THEN ONCE_ASM_REWRITE_TAC[] 
           		      THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
		    THEN DISCH_TAC
		    THEN ONCE_ASM_REWRITE_TAC[]
		    THEN RW_TAC std_ss[le_mul, indicator_fn_pos_le])
	THEN DISCH_TAC	    
 	THEN FULL_SIMP_TAC std_ss[]
	THEN `0 <= real (indicator_fn {z' | 0 <= z' /\ z' <= t:real} z)` by
	      (`!z. indicator_fn {z' | real 0 <= z' /\ z' <= t:real} z <> PosInf /\
		           indicator_fn {z' | real 0 <= z' /\ z' <= t:real} z <> NegInf` by
			   (RW_TAC std_ss [indicator_fn_def]
			        THEN METIS_TAC[extreal_lt_def, extreal_le_def, lt_le, le_lt, lt_infty, le_infty,extreal_of_num_def])
		     THEN `0 = real 0` by 
                          (RW_TAC std_ss[real_def]
		          THEN SELECT_ELIM_TAC
                          THEN RW_TAC std_ss[]  		      
                              THEN1(Q.EXISTS_TAC `0`
			            THEN METIS_TAC [extreal_of_num_def])
		          THEN METIS_TAC [extreal_of_num_def, extreal_11])
	             THEN `0<> PosInf /\ 0 <> NegInf` by METIS_TAC [extreal_of_num_def, lt_infty]
	   	     THEN RW_TAC std_ss[extreal_real_le]
	   	     THEN RW_TAC std_ss[indicator_fn_pos_le])

       THEN FULL_SIMP_TAC std_ss[]
       THEN FULL_SIMP_TAC std_ss[GSYM normal_real_indicator_fn]
       THEN KNOW_TAC(``pos_fn_integral (pair_measure lborel lborel)
     (\ (x:real,y:real).
          f_xy (x,y) *
          (indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
             (x,y) * indicator_fn {z' | 0 <= z' /\ z' <= t} z))=
	     pos_fn_integral (pair_measure lborel lborel)
           (\ (x:real#real).
                indicator_fn {z' | 0 <= z' /\ z' <= t} z *
                (\ (x,y).
                     f_xy (x,y) *
                     indicator_fn
                       {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                       (x,y)) x)  ``)
              THEN1(KNOW_TAC(`` !(x:real#real). (\ (x,y).
               f_xy (x,y) *
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y)) x =
		 
               f_xy x *
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} x ``)
                   THEN1(RW_TAC std_ss[]
                         THEN `(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
                         THEN ONCE_ASM_REWRITE_TAC[]
 			 THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
		    THEN DISCH_TAC
		    THEN `pos_fn_integral (pair_measure lborel lborel)
     (\ (x,y).
          f_xy (x,y) *
          (indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
             (x,y) * indicator_fn {z' | 0 <= z' /\ z' <= t} z)) =
	     pos_fn_integral (pair_measure lborel lborel)
     (\ (x,y).
          f_xy (x,y) *
          indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
             (x,y) * indicator_fn {z' | 0 <= z' /\ z' <= t} z)` by RW_TAC std_ss[mul_assoc]
	            THEN WEAKEN_TAC (equal(Term`pos_fn_integral (pair_measure lborel lborel)
           (\x.
                indicator_fn {z' | 0 <= z' /\ z' <= t} z *
                (\ (x,y).
                     f_xy (x,y) *
                     indicator_fn
                       {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                       (x,y)) x) =
         indicator_fn {z' | 0 <= z' /\ z' <= t} z *
         pos_fn_integral (pair_measure lborel lborel)
           (\x.
                (\ (x,y).
                     f_xy (x,y) *
                     indicator_fn
                       {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                       (x,y)) x)`))

 	     	    THEN RW_TAC std_ss[LAMBDA_PROD]
		    THEN RW_TAC std_ss[mul_comm, mul_assoc])
             THEN DISCH_TAC
             THEN RW_TAC std_ss[]
             THEN KNOW_TAC(`` !(x:real#real). (\ (x,y).
               f_xy (x,y) *
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y)) x =
		 
               f_xy x *
               indicator_fn
                 {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} x ``)
                   THEN1(RW_TAC std_ss[]
                         THEN `(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
                         THEN ONCE_ASM_REWRITE_TAC[]
 			 THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
	     THEN DISCH_TAC
  	     THEN ONCE_ASM_REWRITE_TAC[]
	     THEN RW_TAC std_ss[LAMBDA_PROD, PAIR])
THEN DISCH_TAC		    
THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[mul_assoc, mul_comm]);



val joint_density_2rv_2_snd = store_thm("joint_density_2rv_2_snd",
  ``  !p X Y  f_xy  t.
          prob_space p /\ 
          (!x. 0 <= f_xy x /\ f_xy x <> PosInf) /\
	  distributed p (pair_measure lborel lborel) (\x. (X x,Y x)) f_xy  ==>
       ( !z. pos_fn_integral (pair_measure lborel lborel)
           (\ (x,y).
                f_xy (x,y) *
                indicator_fn
                  {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y)) =
	     pos_fn_integral lborel
           (\y.
                pos_fn_integral lborel
                  (\x.
                       f_xy (x,y) *
                       indicator_fn
                         {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                         (x,y))))``   ,

RW_TAC std_ss[distributed_def]

THEN MP_TAC(Q.ISPECL[`(\ (x:real,y:real).
                 f_xy (x,y) *
                 indicator_fn
                   {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))` , `lborel` , `lborel` ] pos_fn_integral_snd)
THEN RW_TAC std_ss [measure_space_lborel, sigma_finite_measure_lborel]
THEN KNOW_TAC(``!(x:real#real).
              0 <=
              (\ (x,y).
                   f_xy (x,y) *
                   indicator_fn
                     {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y)) x ``)
	  THEN1(RW_TAC std_ss[]
	        THEN KNOW_TAC(`` (\ (x,y).
        f_xy (x,y) *
        indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
     (x:real#real) =
        f_xy x *
        indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x:real#real) ``)
                        THEN1(`(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
                              THEN ONCE_ASM_REWRITE_TAC[] 
           		      THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
		    THEN DISCH_TAC
		    THEN ONCE_ASM_REWRITE_TAC[]
		    THEN RW_TAC std_ss[le_mul, indicator_fn_pos_le])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
THEN KNOW_TAC(`` (\ (x,y).
              f_xy (x,y) *
              indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                (x,y)) IN
         measurable
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel``)
	  THEN1(  MP_TAC(Q.ISPECL[`(m_space (pair_measure lborel lborel),
           measurable_sets (pair_measure lborel lborel))`
	   , `(f_xy:real#real->extreal) `,
	   ` (\ (x:real,y:real).  indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))`, `(\ (x:real,y:real). f_xy (x:real,y:real) *
        indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))` ]  IN_MEASURABLE_BOREL_MUL)
                THEN RW_TAC std_ss[sigma_algebra_pair_lborel]
                THEN KNOW_TAC(`` !(x:real#real).
              x IN
              space
                (m_space (pair_measure lborel lborel),
                 measurable_sets (pair_measure lborel lborel))``)
                     THEN1(RW_TAC std_ss[pair_measure, measure_of,  m_space_def, measurable_sets_def]
		          THEN1(REWRITE_TAC [lborel, m_space_def]
			        THEN REWRITE_TAC [space_def]
				THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
 				THEN ONCE_ASM_REWRITE_TAC[]
				THEN RW_TAC std_ss[IN_UNIV])
			 THEN REWRITE_TAC [lborel, space_def]
			 THEN REWRITE_TAC[ m_space_def]
			 THEN  `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
   		         THEN ONCE_ASM_REWRITE_TAC[]
			 THEN RW_TAC std_ss[IN_UNIV])
                 THEN DISCH_TAC
		 THEN FULL_SIMP_TAC std_ss[]
		 THEN KNOW_TAC(``!(x:real#real). f_xy x <> NegInf``)
                 THEN1(RW_TAC std_ss[]
		      THEN METIS_TAC [extreal_of_num_def,extreal_lt_def, extreal_le_def, le_lt, lt_le,lt_infty,le_infty])
            THEN DISCH_TAC
	    THEN FULL_SIMP_TAC std_ss[]
            THEN KNOW_TAC(`` !(x:real#real).
              (\ (x,y).
                   indicator_fn
                     {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
                x <> NegInf /\
              (\ (x,y).
                   indicator_fn
                     {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
                x <> PosInf  ``)
                THEN1(STRIP_TAC
		      THEN KNOW_TAC(`` (\ (x:real,y:real).
        indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
     x =  
        indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}   x ``)
	                  THEN1(`(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
			        THEN ONCE_ASM_REWRITE_TAC[]
				THEN RW_TAC std_ss[LAMBDA_PROD, PAIR])
		    THEN DISCH_TAC
		    THEN FULL_SIMP_TAC std_ss[]
		    THEN RW_TAC std_ss[indicator_fn_not_eq_infty])
           THEN DISCH_TAC		     
	   THEN FULL_SIMP_TAC std_ss[]
	   THEN KNOW_TAC (`` (\(x,y).
              indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                (x,y)) IN
         measurable
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel)) Borel``)
	       THEN1( MP_TAC(Q.ISPECL[`(m_space (pair_measure lborel lborel),
           measurable_sets (pair_measure lborel lborel))`
	   , `{(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z:real} `,
	   ` (\ (x:real,y:real).  indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))`]  IN_MEASURABLE_BOREL_INDICATOR)
	             THEN RW_TAC std_ss[sigma_algebra_pair_lborel]
		     THEN KNOW_TAC(``  {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} IN
         subsets
           (m_space (pair_measure lborel lborel),
            measurable_sets (pair_measure lborel lborel))``)
	                  THEN1(METIS_TAC [event2_shared_spare_Axy_measurable_set, subsets_def, measurable_sets_def])
		    THEN DISCH_TAC
		    THEN FULL_SIMP_TAC std_ss[]
		    THEN KNOW_TAC(`` (!(x':real#real).
              (\ (x,y).
                   indicator_fn
                     {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
                x' =
              indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                x')``)
                         THEN1(RW_TAC std_ss[]
			       THEN `(x':real#real)= (FST x' , SND x')` by RW_TAC std_ss[PAIR]					   THEN ONCE_ASM_REWRITE_TAC[]	
			       THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
		   THEN DISCH_TAC
		   THEN FULL_SIMP_TAC std_ss[])
       THEN DISCH_TAC		   
       THEN FULL_SIMP_TAC std_ss[]
       THEN KNOW_TAC(`` !(x:real#real).
              (\(x,y).
                   f_xy (x,y) *
                   indicator_fn
                     {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
                x =
              f_xy x *
              (\(x,y).
                   indicator_fn
                     {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))
                x``)
           THEN1(RW_TAC std_ss[]
	        THEN `(x:real#real) = (FST x, SND x)` by RW_TAC std_ss[PAIR]
		THEN ONCE_ASM_REWRITE_TAC[]
		THEN RW_TAC std_ss[PAIR, LAMBDA_PROD])
      THEN DISCH_TAC		
      THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC
THEN FULL_SIMP_TAC std_ss[]);



val event2_shared_spare_indicator_of_indicator_Axy_Ax_Ay = store_thm(
  "event2_shared_spare_indicator_of_indicator_Axy_Ax_Ay",
  `` !x y z:real.  indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
            (x,y) =
	             indicator_fn {x' | y < x' /\ x' < z} x *
                     indicator_fn {y' |  0 <= y' /\ y' < z} y  ``,

RW_TAC std_ss [indicator_fn_def]
      THEN1(RW_TAC std_ss[mul_lone])
      THEN1(`(x,y) IN {(x',y') |0 <= y' /\ y' < z /\ y' < x' /\ x' < z} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''. 0 <= y'' /\ y'' < z /\ y'' < x'' /\ x'' < z) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`(x,y) IN {(x',y') |0 <= y' /\ y' < z /\ y' < x' /\ x' < z} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''. 0 <= y'' /\ y'' < z /\ y'' < x'' /\ x'' < z) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)

THEN1(`(x,y) IN {(x',y') |0 <= y' /\ y' < z /\ y' < x' /\ x' < z} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''. 0 <= y'' /\ y'' < z /\ y'' < x'' /\ x'' < z) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
    
THEN1(`(x,y) IN {(x',y') |0 <= y' /\ y' < z /\ y' < x' /\ x' < z} <=>
      (x,y) IN
      {(x',y') |
       (\x'' y''. 0 <= y'' /\ y'' < z /\ y'' < x'' /\ x'' < z) x' y'}` by RW_TAC std_ss[]

      THEN FULL_SIMP_TAC std_ss [IN_ELIM_PAIR_THM]
      THEN FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
      THEN DFT_rewrs_ss)
 THEN RW_TAC std_ss [mul_rzero, mul_lzero]);
   






val joint_density_2rv_2_snd_1 = store_thm("joint_density_2rv_2_snd_1",
  ``  !p X Y  f_xy  t f_y f_cond.
          prob_space p /\ den_gt0_ninfinity f_xy f_y f_cond /\
          (!x.  f_xy x <> PosInf) /\ 
	(! y.   cond_density lborel lborel p X Y  y f_xy  f_y f_cond)  ==>
            (!z.  pos_fn_integral lborel
           (\y.
                pos_fn_integral lborel
                  (\x.
                       f_xy (x,y) *
                       indicator_fn
                         {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                         (x,y))) =
		  pos_fn_integral lborel
           (\y. f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y * 
                pos_fn_integral lborel
                  (\x.
                       f_cond y x  *
                       indicator_fn
                         {x' | y < x' /\ x' < z} x))) ``   ,

RW_TAC std_ss[den_gt0_ninfinity_def]
THEN `!y:real. f_y y <> 0 /\  f_y y <> NegInf /\ f_y y <> PosInf ` by METIS_TAC [extreal_of_num_def, extreal_lt_def, lt_infty, lt_le, le_lt]
THEN `!x y. f_xy (x:real,y:real) = f_cond y x * f_y y` by METIS_TAC [cond_density_joint_marginal]
THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!x (y:real). indicator_fn {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
            (x,y) =
           indicator_fn {x' | y < x' /\ x' < z} x *
	    indicator_fn {y' | 0 <= y' /\ y' < z} y``)
	  
    THEN1( METIS_TAC [event2_shared_spare_indicator_of_indicator_Axy_Ax_Ay])
THEN DISCH_TAC THEN ASM_REWRITE_TAC[]
THEN `!x y.  f_cond y x * f_y y *
                 ( indicator_fn {x' | y < x' /\ x' < z} x *
	    indicator_fn {y' | 0 <= y' /\ y' < z} y) =
            f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                  f_cond y x *  indicator_fn {x' | y < x' /\ x' < z} x` by METIS_TAC [mul_comm, mul_assoc]
THEN ONCE_ASM_REWRITE_TAC[]
THEN MP_TAC (Q.ISPECL [`lborel`, `(\y:real.  real(f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y))` , `(\ (x:real,y:real).   f_cond y x *  indicator_fn {x' | y < x' /\ x' < z} x) `  ] pos_fn_integral_fun_mul2)

THEN RW_TAC std_ss[measure_space_lborel]
THEN `!x y:real. y IN m_space lborel` by METIS_TAC [lborel, m_space_def, IN_UNIV]
THEN FULL_SIMP_TAC std_ss[]
THEN `!(x:real) (y:real) . 0 <= f_cond y x * indicator_fn {x' | y < x' /\ x' < z} x` by METIS_TAC[le_mul, indicator_fn_pos_le]
THEN KNOW_TAC(``(!x:real. 0 <= real (f_y x * indicator_fn {y' | 0 <= y' /\ y' < z} x)) ``)
     THEN1( ` 0 = real 0` by
                (RW_TAC std_ss[real_def]
	          THEN SELECT_ELIM_TAC
                  THEN RW_TAC std_ss[]  		      
                  THEN1(Q.EXISTS_TAC `0`
		  THEN METIS_TAC [extreal_of_num_def])
		  THEN METIS_TAC [extreal_of_num_def, extreal_11])
	      THEN `0<> PosInf /\ 0 <> NegInf` by METIS_TAC [extreal_of_num_def, lt_infty]
	      
	      THEN RW_TAC std_ss[extreal_real_le, mul_extreal_not_infty,indicator_fn_not_eq_infty ]
	      THEN `!y:real. 0<= f_y y` by METIS_TAC [lt_le, le_lt ]
	      THEN RW_TAC std_ss[le_mul, indicator_fn_pos_le])
THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]

THEN `!(x:real). (f_y x * indicator_fn {y' | 0 <= y' /\ y' < z} x) <> PosInf /\ (f_y x * indicator_fn {y' | 0 <= y' /\ y' < z} x) <> NegInf` by RW_TAC std_ss[extreal_real_le, mul_extreal_not_infty,indicator_fn_not_eq_infty ]
THEN FULL_SIMP_TAC std_ss[normal_real]
THEN ONCE_ASM_REWRITE_TAC[]
THEN `!x y:real. f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                    (f_cond y x * indicator_fn {x' | y < x' /\ x' < z} x) =
		    f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                    f_cond y x * indicator_fn {x' | y < x' /\ x' < z} x` by METIS_TAC[mul_comm, mul_assoc]
THEN FULL_SIMP_TAC std_ss[]);		    


val PREIMAGE_EXTREAL_REAL_3RV_event2_shared_spare = store_thm(
  "PREIMAGE_EXTREAL_REAL_3RV_event2_shared_spare",
  `` !t X Y Z.  rv_gt0_ninfinity [X;Y;Z] ==>
  (PREIMAGE (\s. ((X s,Y s),Z s))
     {((x,y),z) |
      y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <=  Normal t} =
   PREIMAGE (\s. ((real (X s),real (Y s)),real (Z s)))
     {((x',y'),z') |
       y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}) ``,

RW_TAC std_ss [rv_gt0_ninfinity_def, PREIMAGE_def]
THEN KNOW_TAC(``!s. ((X s,Y s),Z s) IN
   {((x,y),z) |  y < x /\ x < z /\ 0 <= y /\ y < z /\ 0 <= z /\ z <= Normal t} =
     Y s < X s /\ X s < Z s /\ 0 <= Y s /\ Y s < Z s /\ 0 <= Z s /\ Z s <=  Normal t``)

     THEN1(RW_TAC std_ss [IN_ELIM_PAIR_THM_1])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!s. ((real (X s),real (Y s)),real (Z s)) IN
   {((x',y'),z') |
   y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t}=
   real (Y s) < real (X s) /\  real (X s) < real (Z s) /\ 0 <= real (Y s) /\   real (Y s) <  real (Z s) /\ 0 <= real (Z s) /\ real (Z s) <=t``)
  THEN1(RW_TAC std_ss [IN_ELIM_PAIR_THM_1])
THEN DISCH_TAC
THEN ASM_REWRITE_TAC[]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION]
THEN `X x <> PosInf` by RW_TAC std_ss[] 
THEN `Y x <> PosInf` by RW_TAC std_ss[]
THEN `Z x <> PosInf` by RW_TAC std_ss[]
THEN `X x <> NegInf /\ Y x <> NegInf /\ Z x <> NegInf` by METIS_TAC [le_lt, lt_le, extreal_le_def, extreal_le_def, lt_infty, le_infty, le_trans, lt_trans, extreal_of_num_def]
THEN EQ_TAC
     THEN1(RW_TAC std_ss[]
           THEN1(METIS_TAC [extreal_real_lt])
	   THEN1(METIS_TAC [extreal_real_lt])
           THEN1(RW_TAC std_ss [GSYM real_of_0]
	         THEN `0 <= X x` by RW_TAC std_ss[]
		 THEN  `0 <> PosInf /\ 0<> NegInf` by METIS_TAC [extreal_lt_def, extreal_le_def, lt_infty, le_infty, lt_le, le_lt, extreal_of_num_def]
	         THEN METIS_TAC [extreal_real_le])
	   THEN1( METIS_TAC [extreal_real_lt])
	   THEN1(RW_TAC std_ss [GSYM real_of_0]
	         THEN `0 <= Z x` by RW_TAC std_ss[]
		 THEN  `0 <> PosInf /\ 0<> NegInf` by METIS_TAC [extreal_lt_def, extreal_le_def, lt_infty, le_infty, lt_le, le_lt, extreal_of_num_def]
	         THEN METIS_TAC [extreal_real_le])
	    THEN RW_TAC std_ss [real_def]
	         THEN SELECT_ELIM_TAC
                 THEN RW_TAC std_ss[]
		      THEN1( Q.EXISTS_TAC `real (Z x)`
		            THEN METIS_TAC [normal_real])
                 THEN METIS_TAC [extreal_le_eq])

THEN RW_TAC std_ss[]		 
     THEN1(METIS_TAC [extreal_real_lt])
     THEN1(METIS_TAC [extreal_real_lt])
     THEN1(METIS_TAC [extreal_real_lt])
THEN UNDISCH_TAC(``real (Z x) <= t ``)
THEN REWRITE_TAC[real_def]
THEN SELECT_ELIM_TAC
THEN RW_TAC std_ss[]
    THEN1(Q.EXISTS_TAC (`real (Z x)`)
        THEN METIS_TAC [normal_real])
THEN RW_TAC std_ss [extreal_le_eq]);


val sigma_finite_measure_distr_pair_lborel = store_thm ("sigma_finite_measure_distr_pair_lborel",
 ``!p X Y. prob_space p /\ random_variable (\x. (X x, Y x)) p (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel)) ==> (sigma_finite_measure (distr p (pair_measure lborel lborel) (\x. ( X x, Y x))))``,

 RW_TAC std_ss [sigma_finite_measure] THEN RW_TAC std_ss [distr, measurable_sets_def, m_space_def]
THEN RW_TAC std_ss [GSYM distr] THEN Q.EXISTS_TAC `{(\ y. line (FST y) CROSS line (SND y)) y | y IN UNIV }` THEN
  REPEAT CONJ_TAC
  THEN1(SIMP_TAC std_ss [GSYM IMAGE_DEF] THEN
        MATCH_MP_TAC pred_setTheory.image_countable THEN
   	SIMP_TAC std_ss [COUNTABLE_NUM] THEN 
   	`univ(:num#num) = univ(:num) CROSS univ(:num)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
 	THEN ASM_REWRITE_TAC[] THEN
	SIMP_TAC std_ss [COUNTABLE_NUM, CROSS_COUNTABLE, GSPECIFICATION, EXTENSION])
 THEN1(SIMP_TAC std_ss [SUBSET_DEF, lborel, measurable_sets_def, pair_measure, measure_of] THEN
       REPEAT STRIP_TAC THEN RW_TAC std_ss [borel_line]
       THEN1(FULL_SIMP_TAC std_ss [m_space_def] THEN
             KNOW_TAC(``?y. x = line (FST y)  CROSS line (SND y) ``)
             THEN1(FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
	           THEN Q.EXISTS_TAC`y`
                    THEN RW_TAC std_ss[])
	     THEN RW_TAC std_ss[]
	      THEN KNOW_TAC (``line (FST y) CROSS line (SND y) IN {a CROSS b | a IN subsets borel /\ b IN subsets borel} ``)
                  THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
                      THEN Q.EXISTS_TAC `(line (FST y), line (SND y))`
	              THEN RW_TAC std_ss[]
	              THEN METIS_TAC [borel_line])
              THEN DISCH_TAC
              THEN METIS_TAC [sigma_sets_basic])
       THEN RW_TAC std_ss[m_space_def]
       THEN FULL_SIMP_TAC std_ss [m_space_def]
       THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
       THEN FULL_SIMP_TAC std_ss[]
       THEN `POW univ(:real#real) = univ(:real#real->bool)` by RW_TAC std_ss [UNIV_FUN_TO_BOOL]
       THEN FULL_SIMP_TAC std_ss[]
       THEN METIS_TAC [IN_UNIV])
  THEN1(`univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
        THEN SIMP_TAC std_ss [EXTENSION, lborel, m_space_def, IN_UNIV, pair_measure, measure_of] THEN
	SIMP_TAC std_ss [IN_BIGUNION, GSPECIFICATION] THEN
  	`univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
   	THEN ASM_REWRITE_TAC[]
   	THEN SIMP_TAC std_ss [IN_UNIV] THEN STRIP_TAC THEN
   	ASSUME_TAC mem_big_line_pair THEN POP_ASSUM (MP_TAC o Q.SPEC `x`) THEN
   	SET_TAC [])
THEN RW_TAC std_ss [distr, measure_def] THEN RW_TAC std_ss [GSYM prob_def, GSYM p_space_def]
THEN KNOW_TAC(``(a:real reln) IN
      subsets
        (m_space (pair_measure lborel lborel),
         measurable_sets (pair_measure lborel lborel))``)
    THEN1( FULL_SIMP_TAC std_ss [pair_measure, m_space_def, measurable_sets_def, measure_of, lborel, subsets_def]
       THEN KNOW_TAC(``?y. a = line (FST y)  CROSS line (SND y) ``)
         THEN1(FULL_SIMP_TAC std_ss [GSPECIFICATION, EXTENSION]
	       THEN Q.EXISTS_TAC`y`
               THEN RW_TAC std_ss[])
       THEN STRIP_TAC
       THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
       THEN FULL_SIMP_TAC std_ss[]
       THEN `POW univ(:real#real) = univ(:real#real->bool)` by RW_TAC std_ss [UNIV_FUN_TO_BOOL]
       THEN FULL_SIMP_TAC std_ss[]
       THEN FULL_SIMP_TAC std_ss [SUBSET_UNIV]
       THEN RW_TAC std_ss[]
       THEN KNOW_TAC(``line (FST y) CROSS line (SND (y:num#num)) IN {a CROSS b | a IN subsets borel /\ b IN subsets borel}``)
          THEN1(SRW_TAC [] [GSPECIFICATION, EXTENSION]
	        THEN Q.EXISTS_TAC `(line (FST (y:num#num)), line (SND y))`
       THEN  RW_TAC std_ss [borel_line])
       THEN DISCH_TAC
       THEN METIS_TAC [sigma_sets_basic])
 THEN DISCH_TAC THEN KNOW_TAC(``a IN measurable_sets (pair_measure  lborel lborel)``) THEN1 (METIS_TAC [subsets_def]) THEN DISCH_TAC THEN METIS_TAC [random_variable_def, IN_MEASURABLE, measurable_sets_def, m_space_def, subsets_def, space_def, PROB_FINITE]); 




val prob_event2_shared_spare = store_thm("prob_event2_shared_spare",
  `` !p t X Y Z f_xy f_cond  f_y f_z.
       prob_space p /\ den_gt0_ninfinity f_xy f_y f_cond  /\
       rv_gt0_ninfinity [X; Y; Z] /\ (!z. 0 <= f_z z) /\
       (!x. f_xy x<> PosInf) /\ (!x y.
               cond_density lborel lborel p (\x. real (X x)) (\x. real (Y x))
                  y f_xy  f_y f_cond) /\
		 indep_varp p (pair_measure lborel lborel)
            (\x. (real (X x),real (Y x))) lborel (\x. real (Z x)) /\
	    distributed p lborel (\x. real (Z x)) f_z==>
		(prob p( DFT_event p (D_AND (D_AND Z (D_BEFORE Y X)) (D_BEFORE X Z)) t) =  
            pos_fn_integral lborel
	    (\z. f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z *
              pos_fn_integral lborel
                (\y.
                     f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                     pos_fn_integral lborel
                       (\x.
                            f_cond y x *
                            indicator_fn {x' | y < x' /\ x' < z} x))))``,

RW_TAC std_ss[DFT_event2_shared_spare]			    
THEN RW_TAC std_ss [PREIMAGE_EXTREAL_REAL_3RV_event2_shared_spare]
THEN KNOW_TAC(``  prob p
     (PREIMAGE (\x. ((real (X x),real (Y x)),real (Z x)))
        {((x',y'),z') |
      y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} INTER
      p_space p) =
       pos_fn_integral (distr p lborel (\x. real (Z x) ) )
             (\z.
                  pos_fn_integral (distr p (pair_measure lborel lborel) (\x. (real (X x),real (Y x))))
                    (\ (x,y). indicator_fn {((x',y'),z') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z) )) ``)
     THEN1(MP_TAC (Q.ISPECL [`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,
     `(\x. real ((X:'a->extreal) x))`, `(\x. real ((Y:'a->extreal) x))`,
     `(\x. real ((Z:'a->extreal) x))`, `(pair_measure lborel lborel)`, `lborel`, `{((x',y'),z') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t:real}`] prob_event_3rv_2indep_snd) 
	THEN RW_TAC std_ss[measure_space_lborel, measure_space_pair_measure', sigma_finite_measure_lborel, BIGUNION_event2_shared_spare_subevent2_measurable_set, sigma_finite_measure_pair_lborel])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]	
THEN KNOW_TAC(`` pos_fn_integral (distr p lborel (\x. real (Z x)))
     (\z.
          pos_fn_integral
            (distr p (pair_measure lborel lborel)
               (\x. (real (X x),real (Y x))))
            (\(x,y).
                 indicator_fn
                   {((x',y'),z') |
                   y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z))) =
          pos_fn_integral lborel
     (\z.
         f_z z*  pos_fn_integral
            (distr p (pair_measure lborel lborel)
               (\x. (real (X x),real (Y x))))
            (\(x,y).
                 indicator_fn
                   {((x',y'),z') |
                   y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z)))``)
        THEN1( MP_TAC(Q.ISPECL [`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,
     `(\x. real ((X:'a->extreal) x))`, `(\x. real ((Y:'a->extreal) x))`,
     `(\x. real ((Z:'a->extreal) x))`, `(pair_measure lborel lborel)`, `lborel`,
     `{((x',y'),z') |
        y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t:real}`, ` {(z',x',y') |
            y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t:real}`, `f_z : real->extreal`] joint_density_3rv_0_snd)
               THEN  RW_TAC std_ss[measure_space_lborel, measure_space_pair_measure', sigma_finite_measure_lborel, BIGUNION_event2_shared_spare_subevent2_measurable_set, sigma_finite_measure_pair_lborel]
	       THEN KNOW_TAC(``  measure_space  (distr p (pair_measure lborel lborel)
                                (\x. (real (X x),real (Y x))))``)
		    THEN1( `measure_space p` by FULL_SIMP_TAC std_ss[prob_space_def]
		           THEN `measure_space (pair_measure lborel lborel)` by RW_TAC std_ss[measure_space_lborel, measure_space_pair_measure', sigma_finite_measure_lborel, sigma_finite_measure_pair_lborel]
			   THEN `(\x. (real ((X:'a->extreal) x),
			                         real ((Y:'a->extreal) x)))
			                 IN measurable (m_space p, measurable_sets p)
					               (m_space (pair_measure lborel lborel),
					              measurable_sets (pair_measure lborel lborel)) ` by FULL_SIMP_TAC std_ss [cond_density_def, distributed_def]
  	                  THEN RW_TAC std_ss[measure_space_distr])
               THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	       THEN KNOW_TAC(``!(x:real#real).
              x IN
              m_space
                (distr p (pair_measure lborel lborel)
                   (\x. (real (X x),real (Y x)))) ``)
		    THEN1(RW_TAC std_ss [distr, m_space_def, pair_measure, measure_of, m_space_def, lborel]
		          THEN `univ(:real) CROSS univ(:real) = univ(:real#real)` by SRW_TAC [] [GSPECIFICATION, EXTENSION]
			  THEN METIS_TAC [IN_UNIV])
	       THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	       THEN `!x. x IN m_space lborel` by RW_TAC std_ss [m_space_def, lborel, IN_UNIV]
               THEN FULL_SIMP_TAC std_ss[BIGUNION_event2_shared_spare_form2_subevent2_measurable_set, BIGUNION_event2_shared_spare_subevent2_measurable_set]
	       THEN KNOW_TAC(``random_variable (\x. (real (X x),real (Y x))) p (m_space (pair_measure lborel lborel), measurable_sets (pair_measure lborel lborel))``)
	       THEN1(RW_TAC std_ss[random_variable_def,p_space_def, events_def]
	             THEN FULL_SIMP_TAC std_ss[cond_density_def, distributed_def])
	       THEN DISCH_TAC THEN FULL_SIMP_TAC std_ss[]
	       THEN `sigma_finite_measure
           (distr p (pair_measure lborel lborel)
              (\x. (real (X x),real (Y x))))` by RW_TAC std_ss[sigma_finite_measure_distr_pair_lborel]
	       THEN FULL_SIMP_TAC std_ss[]
	       THEN `(!x y z.
              indicator_fn
                {((x',y'),z') |
                y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z) =
              indicator_fn
                {(z',x',y') |
                y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} (z,x,y))` by RW_TAC std_ss [event2_shared_spare_form2, event2_shared_spare]
		THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(`` pos_fn_integral lborel
             (\z.
                  f_z z *
                  pos_fn_integral
                    (distr p (pair_measure lborel lborel) (\x. (real (X x),real (Y x))))
                    (\ (x,y).
                         indicator_fn
                           {((x',y'),z') |
                           y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} ((x,y),z))) =
           pos_fn_integral lborel
             (\z.
                  f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z *
                  pos_fn_integral (pair_measure lborel lborel)
                    (\ (x,y).
                         f_xy (x,y) *
                         indicator_fn
                           {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                           (x,y))) ``)
      THEN1(MP_TAC(Q.ISPECL [`p:('a->bool)#(('a->bool)->bool)#(('a->bool)->extreal)`,
     `(\x. real ((X:'a->extreal) x))`, `(\x. real ((Y:'a->extreal) x))`,
     `(\x. real ((Z:'a->extreal) x))`,`f_xy:real#real->extreal`, `f_z:real->extreal`, `t:real`] joint_density_3rv_1_snd)
           THEN RW_TAC std_ss []
	   THEN KNOW_TAC(``random_variable (\x. real (Z x)) p borel``)
                THEN1(FULL_SIMP_TAC std_ss[distributed_def]
		      THEN RW_TAC std_ss[random_variable_def, p_space_def, events_def]
		      THEN KNOW_TAC(``borel = (m_space lborel, measurable_sets lborel)``)
			       THEN1(RW_TAC std_ss [lborel, m_space_def, measurable_sets_def]
                                     THEN `borel = (space borel, subsets borel)` by METIS_TAC [space_def, subsets_def, PAIR]
			       THEN ONCE_ASM_REWRITE_TAC[]
                               THEN METIS_TAC [borel, lborel, m_space_def, measurable_sets_def, space_borel, subsets_def, space_def])
                      THEN DISCH_TAC
 		      THEN ONCE_ASM_REWRITE_TAC[]
		      THEN RW_TAC std_ss[])
	   THEN DISCH_TAC
	   THEN KNOW_TAC(``(!(x:real#real). 0 <= f_xy x )``)
	        THEN1( FULL_SIMP_TAC std_ss[den_gt0_ninfinity_def]
		       THEN METIS_TAC[PAIR])
	   THEN DISCH_TAC
	   THEN ` distributed p (pair_measure lborel lborel)
           (\x. (real (X x),real (Y x))) f_xy` by FULL_SIMP_TAC std_ss[cond_density_def]
           THEN FULL_SIMP_TAC std_ss[])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN KNOW_TAC(``!z.
              pos_fn_integral (pair_measure lborel lborel)
                (\(x,y).
                     f_xy (x,y) *
                     indicator_fn
                       {(x',y') | 0 <= y' /\ y' < z /\ y' < x' /\ x' < z}
                       (x,y)) =
              pos_fn_integral lborel
                (\y.
                     pos_fn_integral lborel
                       (\x.
                            f_xy (x,y) *
                            indicator_fn
                              {(x',y') |
                              0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y)))``)
       THEN1(FULL_SIMP_TAC std_ss [cond_density_def, den_gt0_ninfinity_def, PAIR] THEN METIS_TAC[joint_density_2rv_2_snd, PAIR])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]       
THEN KNOW_TAC(``  !z.
              pos_fn_integral lborel
                (\y.
                     pos_fn_integral lborel
                       (\x.
                            f_xy (x,y) *
                            indicator_fn
                              {(x',y') |
                              0 <= y' /\ y' < z /\ y' < x' /\ x' < z} (x,y))) =
              pos_fn_integral lborel
                (\y.
                     f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                     pos_fn_integral lborel
                       (\x.
                            f_cond y x  *
                            indicator_fn {x' | y < x' /\ x' < z} x))``)
     THEN1(METIS_TAC [joint_density_2rv_2_snd_1])
THEN DISCH_TAC THEN ONCE_ASM_REWRITE_TAC[]
THEN RW_TAC std_ss[]);



val event1_event2_shared_spare_disjoint = store_thm(
  "event1_event2_shared_spare_disjoint",
  `` !p t X Y Z. DISJOINT (DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t) (DFT_event p (D_AND (D_AND Z (D_BEFORE Y X)) (D_BEFORE X Z)) t)``,

RW_TAC std_ss [AND_INTER]
THEN RW_TAC std_ss[DFT_event_def, D_BEFORE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION, DISJOINT_DEF]
THEN RW_TAC std_ss[]
THEN METIS_TAC [extreal_lt_def, extreal_le_def, lt_le, le_lt]);


val DFT_event_shared_spare_first_part = store_thm(
  "DFT_event_shared_spare_first_part",
  `` !p X Y Z t. (!s. ALL_DISTINCT [X s; Y s ; Z s]) ==>( DFT_event p (D_AND (D_AND (D_AND (X)(Z))(D_BEFORE Y X))(D_BEFORE Y Z)) t =
                 DFT_event p (D_OR (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X))
		             (D_AND (D_AND Z (D_BEFORE Y X)) (D_BEFORE X Z))) t)``,
RW_TAC std_ss[]
THEN KNOW_TAC(`` (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) =
   		 (D_OR (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X))
        	    (D_AND (D_AND Z (D_BEFORE Y X )) (D_BEFORE X Z)))``)

    THEN1 (RW_TAC std_ss[D_OR_def, D_AND_def, D_BEFORE_def]
          THEN  KNOW_TAC(``!s.(
          max (max (max (X s) (Z s)) (if Y s < X s then Y s else PosInf))
          (if Y s < Z s then Y s else PosInf))  =
   (
        min
          (max (max (X s) (if Y s < Z s then Y s else PosInf))
             (if Z s < X s then Z s else PosInf))
          (max (max (Z s) (if Y s < X s then Y s else PosInf))
             (if X s < Z s then X s else PosInf))) ``)

	     THEN1(RW_TAC std_ss[extreal_max_def, extreal_min_def] THEN FULL_SIMP_TAC std_ss[]
	           THEN  METIS_TAC [min_infty, max_infty, lt_infty, lt_le, lt_total, lte_trans, let_trans, extreal_lt_def, extreal_le_def, extreal_min_def, extreal_max_def, ALL_DISTINCT, MEM])
		   
         THEN DISCH_TAC
	 THEN RW_TAC std_ss[])
THEN RW_TAC std_ss[]);


val prob_DFT_event_shared_spare_2events = store_thm(
  "prob_DFT_event_shared_spare_2events",
  ``!p t X Y Z f_xy f_cond  f_y f_z.
prob_space p /\ den_gt0_ninfinity f_xy f_y f_cond /\
          rv_gt0_ninfinity [X; Y; Z] /\
	  (!s. ALL_DISTINCT [X s; Y s ; Z s]) /\
          (!x y.
               cond_density lborel lborel p (\x. real (X x)) (\x. real (Y x))
                  y f_xy f_y f_cond) /\
          indep_varp p (pair_measure lborel lborel)
            (\x. (real (X x),real (Y x))) lborel (\x. real (Z x)) /\
	    distributed p lborel (\x. real (Z x)) f_z /\
          (!z. 0 <= f_z z) /\ (!x. f_xy x <> PosInf)  ==> 
(prob p
             (DFT_event p (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) t) =

 pos_fn_integral lborel
             (\y.
                  f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
                  pos_fn_integral lborel
                    (\x.
                         f_cond y x *
                         indicator_fn {x' | y < x' /\ x' <= t} x *
                         pos_fn_integral lborel
                           (\z.
                                f_z z *
                                indicator_fn {z' | y < z' /\ z' < x} z))) +

 pos_fn_integral lborel
             (\z.
                  f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z *
                  pos_fn_integral lborel
                    (\y.
                         f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                         pos_fn_integral lborel
                           (\x.
                                f_cond y x *
                                indicator_fn {x' | y < x' /\ x' < z} x))))``,

RW_TAC std_ss[DFT_event_shared_spare_first_part]
THEN RW_TAC std_ss[OR_UNION]
THEN `DISJOINT (DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t)
            (DFT_event p (D_AND (D_AND Z (D_BEFORE Y X)) (D_BEFORE X Z)) t)` by RW_TAC std_ss[event1_event2_shared_spare_disjoint]
THEN `measure_space p` by FULL_SIMP_TAC std_ss[prob_space_def]
THEN `measure_space (pair_measure lborel lborel)` by METIS_TAC [measure_space_lborel, sigma_finite_measure_lborel, measure_space_pair_measure']

THEN `(\x. ((real (X x),real (Y x)))) IN measurable (m_space p , measurable_sets p) (m_space (pair_measure lborel lborel),measurable_sets (pair_measure lborel lborel))` by METIS_TAC[p_space_def, events_def, distributed_def, cond_density_def]

THEN `(\x. real (Z x)) IN measurable (m_space p , measurable_sets p) (m_space lborel, measurable_sets lborel)` by FULL_SIMP_TAC std_ss[distributed_def]

THEN KNOW_TAC(``DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t IN events p ``)
     THEN1(RW_TAC std_ss[DFT_event1_shared_spare]
           THEN RW_TAC std_ss[PREIMAGE_EXTREAL_REAL_3RV_event1_shared_spare]
	   THEN KNOW_TAC(`` {((x',y'),z') |
     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} IN subsets (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel))  ``)
               THEN1(METIS_TAC [subsets_def, measurable_sets_def, BIGUNION_event1_shared_spare_subevent2_measurable_set])
	   THEN DISCH_TAC    
           THEN KNOW_TAC(``(\x. ((real (X x),real (Y x)),real (Z x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel))``)
	   	THEN1(METIS_TAC[measurable_Pair, measure_space_lborel])
	   THEN DISCH_TAC
	   THEN METIS_TAC [space_def, p_space_def, m_space_def, subsets_def, measurable_sets_def, events_def, IN_MEASURABLE])
THEN DISCH_TAC	   
THEN KNOW_TAC(``  DFT_event p (D_AND (D_AND Z (D_BEFORE Y X )) (D_BEFORE X Z)) t IN events p``)
     THEN1(RW_TAC std_ss[DFT_event2_shared_spare]
           THEN RW_TAC std_ss[PREIMAGE_EXTREAL_REAL_3RV_event2_shared_spare]
	   THEN KNOW_TAC(`` {((x',y'),z') |
     y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} IN subsets (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel)) ``)
               THEN1(METIS_TAC [subsets_def, measurable_sets_def, BIGUNION_event2_shared_spare_subevent2_measurable_set])
	   THEN DISCH_TAC    
	    THEN KNOW_TAC(``(\x. ((real (X x),real (Y x)),real (Z x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel))``)
	   	THEN1(METIS_TAC[measurable_Pair, measure_space_lborel])
	   THEN DISCH_TAC
	   THEN METIS_TAC [space_def, p_space_def, m_space_def, subsets_def, measurable_sets_def, events_def, IN_MEASURABLE])
THEN DISCH_TAC	   

THEN `prob p
     (DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t UNION
      DFT_event p (D_AND (D_AND Z (D_BEFORE Y X)) (D_BEFORE X Z)) t) =
      prob p (DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t) +
      prob p (DFT_event p (D_AND (D_AND Z (D_BEFORE Y X)) (D_BEFORE X Z)) t)` by RW_TAC std_ss[PROB_ADDITIVE]
THEN ONCE_ASM_REWRITE_TAC[]
THEN  METIS_TAC[prob_event1_shared_spare, prob_event2_shared_spare]);



val DFT_events_shared_spare_disjoit_fst_snd = store_thm(
  "DFT_events_shared_spare_disjoit_fst_snd",
  `` !X Y Z p t. DISJOINT
               (DFT_event p (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) t)
	       (DFT_event p (D_AND (D_AND (D_AND X Y) (D_BEFORE Z X)) (D_BEFORE Z Y)) t) ``,

RW_TAC std_ss[AND_INTER]
THEN RW_TAC std_ss[DFT_event_def, D_BEFORE_def]
THEN SRW_TAC [] [GSPECIFICATION, EXTENSION, DISJOINT_DEF]
THEN RW_TAC std_ss[] THEN FULL_SIMP_TAC std_ss[]
THEN  METIS_TAC [lt_infty, lt_le, lt_total, lte_trans, let_trans, extreal_lt_def, extreal_le_def]);
		   






(*


val prob_DFT_shared_spare = store_thm("prob_DFT_shared_spare",
  ``  !p t X Y Z f_xy f_condx_y f_condx_z  f_y f_z f_xz.
          prob_space p /\    rv_gt0_ninfinity [X; Y; Z] /\ (!s. ALL_DISTINCT [X s; Y s; Z s] )/\
	  (!x. f_xy x <> PosInf /\ f_xz x <> PosInf) /\
	  (!s. Y s < Z s==> (den_gt0_ninfinity f_xy f_y f_condx_y /\
	        (!x y.
               cond_density lborel lborel p (\x. real (X x)) (\x. real (Y x))
                  y f_xy f_y f_condx_y ) /\
          indep_varp p (pair_measure lborel lborel)
            (\x. (real (X x),real (Y x))) lborel (\x. real (Z x)) /\
          distributed p lborel (\x. real (Z x)) f_z /\ (!z. 0 <= f_z z))) /\
	  (!s. Z s < Y s ==>
	  
	   (den_gt0_ninfinity f_xz f_z f_condx_z /\
	        (!z y.
               cond_density lborel lborel p (\x. real (X x)) (\x. real (Z x))
                  z f_xz f_z  f_condx_z ) /\
          indep_varp p (pair_measure lborel lborel)
            (\x. (real (X x),real (Z x))) lborel (\x. real (Y x)) /\
          distributed p lborel (\x. real (Y x)) f_y /\ (!y. 0 <= f_y y))) ==>

	(prob p (DFT_event p (D_OR ((D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)))
	                          ((D_AND (D_AND (D_AND X Y) (D_BEFORE Z X)) (D_BEFORE Z Y))) ) t) = 
      pos_fn_integral lborel
             (\y.
                  f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
                  pos_fn_integral lborel
                    (\x.
                         f_condx_y y x *
                         indicator_fn {x' | y < x' /\ x' <= t} x *
                         pos_fn_integral lborel
                           (\z.
                                f_z z *
                                indicator_fn {z' | y < z' /\ z' < x} z))) +
           pos_fn_integral lborel
             (\z.
                  f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z *
                  pos_fn_integral lborel
                    (\y.
                         f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                         pos_fn_integral lborel
                           (\x.
                                f_condx_y y x *
                                indicator_fn {x' | y < x' /\ x' < z} x))) +


      pos_fn_integral lborel
             (\z.
                  f_z z * indicator_fn {z' | 0 <=z' /\ z' <= t} z *
                  pos_fn_integral lborel
                    (\x.
                         f_condx_z z x *
                         indicator_fn {x' | z < x' /\ x' <= t} x *
                         pos_fn_integral lborel
                           (\y.
                                f_y y *
                                indicator_fn {y' | z < y' /\ y' < x} y))) +
           pos_fn_integral lborel
             (\y.
                  f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
                  pos_fn_integral lborel
                    (\z.
                         f_z z * indicator_fn {z' | 0 <= z' /\ z' < y} z *
                         pos_fn_integral lborel
                           (\x.
                                f_condx_z z x *
                                indicator_fn {x' | z < x' /\ x' < y} x))))``

RW_TAC std_ss[OR_UNION]
THEN KNOW_TAC(`` prob p (DFT_event p (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) t) + 
      prob p (DFT_event p (D_AND (D_AND (D_AND X Y) (D_BEFORE Z X)) (D_BEFORE Z Y)) t) = pos_fn_integral lborel
     (\y.
          f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\x.
                 f_condx_y y x * indicator_fn {x' | y < x' /\ x' <= t} x *
                 pos_fn_integral lborel
                   (\z. f_z z * indicator_fn {z' | y < z' /\ z' < x} z))) +
   pos_fn_integral lborel
     (\z.
          f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z *
          pos_fn_integral lborel
            (\y.
                 f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                 pos_fn_integral lborel
                   (\x.
                        f_condx_y y x *
                        indicator_fn {x' | y < x' /\ x' < z} x))) +
   pos_fn_integral lborel
     (\z.
          f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z *
          pos_fn_integral lborel
            (\x.
                 f_condx_z z x * indicator_fn {x' | z < x' /\ x' <= t} x *
                 pos_fn_integral lborel
                   (\y. f_y y * indicator_fn {y' | z < y' /\ y' < x} y))) +
   pos_fn_integral lborel
     (\y.
          f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
          pos_fn_integral lborel
            (\z.
                 f_z z * indicator_fn {z' | 0 <= z' /\ z' < y} z *
                 pos_fn_integral lborel
                   (\x.
                        f_condx_z z x *
                        indicator_fn {x' | z < x' /\ x' < y} x)))``)




  DFT_event1_shared_spare : thm
     DFT_event2_shared_spare : thm
    DFT_event_shared_spare_first_part : thm
     DF_events_shared_spare_disjoit_fst_snd : thm
 PREIMAGE_EXTREAL_REAL_3RV_event1_shared_spare : thm
     PREIMAGE_EXTREAL_REAL_3RV_event2_shared_spare : thm
     cond_density_3rv : thm
    event1_event2_shared_spare_disjoint : thm
    event1_shared_spare : thm
      event2_shared_spare : thm
     prob_DFT_event_shared_spare_2events : thm
     prob_event1_shared_spare : thm
     prob_event2_shared_spare : thm
    

PREIMAGE_EXTREAL_REAL_3RV_event1_shared_spare

g` ! x y z s. (PREIMAGE (\s. ((X s,Y s),Z s))
             {((x,y),z) |
             y < z /\ z < x /\ y < x /\ x <= Normal t /\ 0 <= y /\
             y <= Normal t} s) ==> Y s < Z s`




REWRITE_TAC[PREIMAGE_def]



SRW_TAC [] [GSPECIFICATION, EXTENSION]
`x':extreal#extreal#extreal = (FST x', (FST (SND x')), SND (SND x'))` by RW_TAC std_ss[PAIR, LAMBDA_PROD]
ONCE_ASM_REWRITE_TAC[]
THEN FULL_SIMP_TAC std_ss[PAIR, LAMBDA_PROD]
KNOW_TAC(``!s. (DFT_event p
               (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) t) s ==> Y s < Z s``)

DISCH_TAC

KNOW_TAC(``(prob p
             (DFT_event p
                (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) t) =
           pos_fn_integral lborel
             (\y.
                  f_y y * indicator_fn {y' | 0 <= y' /\ y' <= t} y *
                  pos_fn_integral lborel
                    (\x.
                         f_cond y x *
                         indicator_fn {x' | y < x' /\ x' <= t} x *
                         pos_fn_integral lborel
                           (\z.
                                f_z z *
                                indicator_fn {z' | y < z' /\ z' < x} z))) +
           pos_fn_integral lborel
             (\z.
                  f_z z * indicator_fn {z' | 0 <= z' /\ z' <= t} z *
                  pos_fn_integral lborel
                    (\y.
                         f_y y * indicator_fn {y' | 0 <= y' /\ y' < z} y *
                         pos_fn_integral lborel
                           (\x.
                                f_cond y x *
                                indicator_fn {x' | y < x' /\ x' < z} x)))) ``)

THEN FULL_SIMP_TAC std_ss[]
Cases_on `DFT_event p
               (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) t `



RW_TAC std_ss[DFT_event1_shared_spare,DFT_event_shared_spare_first_part, OR_UNION]
RW_TAC std_ss[DFT_event2_shared_spare,DFT_event_shared_spare_first_part, OR_UNION]

THEN Cases_on `Y s < Z s`
THEN FULL_SIMP_TAC std_ss[]

event1_shared_spare;

pos_fn_integral_distr
pos_fn_integral_measure_of
g`!X Y M.  (!s. (X:'a->real) s  < Y s ) ==> (pos_fn_integral (distr p M X)

                                    (\x. pos_fn_integral (distr p M Y)
				       (\y. indicator_fn {(x',y') | y' < x'} (x,y) )) = 0)`  



g`!X Y M.  (!s. (X:'a->real) s  < Y s ) ==> (pos_fn_integral (pair_measure (distr p M X)(distr p M Y))

                                       (\ (x,y). indicator_fn {(x',y') | y' < x'} (x,y) ) = pos_fn_integral (pair_measure (distr p M X)(distr p M Y))

                                       (\ (x,y). indicator_fn {} (x,y) ))`  

sup_eq
RW_TAC std_ss[]
RW_TAC std_ss[pos_fn_integral_def]
RW_TAC std_ss[distr]
SRW_TAC [] [GSPECIFICATION, EXTENSION]
RW_TAC std_ss[psfis_def]
RW_TAC std_ss[pos_simple_fn_integral_def]
RW_TAC std_ss[psfs_def]
RW_TAC std_ss[pos_simple_fn_def]

RW_TAC std_ss[sup_eq]
FULL_SIMP_TAC std_ss[le_sup]


g` (x==> (prob p A = k)) /\ (!s. Y s < Z s ==> x) /\ (!s. A s ==> Y s < Z s) ==> (prob p A = k)``

RW_TAC std_ss[]

sup_le

psfis_def
psfs_def
pos_simple_fn_def
pos_simple_fn_integral_def
pos_fn_integral_def

THEN KNOW_TAC(`` DFT_event p (D_AND (D_AND (D_AND X Z) (D_BEFORE Y X)) (D_BEFORE Y Z)) t IN events p``)
     THEN1(RW_TAC std_ss[DFT_event_shared_spare_first_part]
           THEN RW_TAC std_ss[OR_UNION]
           THEN `measure_space p` by FULL_SIMP_TAC std_ss[prob_space_def]
	   THEN `measure_space (pair_measure lborel lborel)` by METIS_TAC [measure_space_lborel, sigma_finite_measure_lborel, measure_space_pair_measure']

	   THEN `(\x. ((real (X x),real (Y x)))) IN measurable (m_space p , measurable_sets p) (m_space (pair_measure lborel lborel),measurable_sets (pair_measure lborel lborel))` by METIS_TAC[p_space_def, events_def, distributed_def, cond_density_def]

THEN `(\x. real (Z x)) IN measurable (m_space p , measurable_sets p) (m_space lborel, measurable_sets lborel)` by FULL_SIMP_TAC std_ss[distributed_def]

THEN KNOW_TAC(``DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t IN events p ``)
     THEN1(RW_TAC std_ss[DFT_event1_shared_spare]
           THEN RW_TAC std_ss[PREIMAGE_EXTREAL_REAL_3RV_event1_shared_spare]
	   THEN KNOW_TAC(`` {((x',y'),z') |
     y' < z' /\ z' < x' /\ y' < x' /\ x' <= t /\ 0 <= y' /\ y' <= t} IN subsets (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel))  ``)
               THEN1(METIS_TAC [subsets_def, measurable_sets_def, BIGUNION_event1_shared_spare_subevent2_measurable_set])
	   THEN DISCH_TAC    
           THEN KNOW_TAC(``(\x. ((real (X x),real (Y x)),real (Z x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel))``)
	   	THEN1(METIS_TAC[measurable_Pair, measure_space_lborel])
	   THEN DISCH_TAC
	   THEN METIS_TAC [space_def, p_space_def, m_space_def, subsets_def, measurable_sets_def, events_def, IN_MEASURABLE])
THEN DISCH_TAC	   
THEN KNOW_TAC(``  DFT_event p (D_AND (D_AND Z (D_BEFORE Y X )) (D_BEFORE X Z)) t IN events p``)
     THEN1(RW_TAC std_ss[DFT_event2_shared_spare]
           THEN RW_TAC std_ss[PREIMAGE_EXTREAL_REAL_3RV_event2_shared_spare]
	   THEN KNOW_TAC(`` {((x',y'),z') |
     y' < x' /\ x' < z' /\ 0 <= y' /\ y' < z' /\ 0 <= z' /\ z' <= t} IN subsets (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel)) ``)
               THEN1(METIS_TAC [subsets_def, measurable_sets_def, BIGUNION_event2_shared_spare_subevent2_measurable_set])
	   THEN DISCH_TAC    
	    THEN KNOW_TAC(``(\x. ((real (X x),real (Y x)),real (Z x))) IN measurable (m_space p, measurable_sets p) (m_space (pair_measure (pair_measure lborel lborel) lborel) , measurable_sets (pair_measure (pair_measure lborel lborel) lborel))``)
	   	THEN1(METIS_TAC[measurable_Pair, measure_space_lborel])
	   THEN DISCH_TAC
	   THEN METIS_TAC [space_def, p_space_def, m_space_def, subsets_def, measurable_sets_def, events_def, IN_MEASURABLE])
THEN DISCH_TAC	   





THEN `DISJOINT (DFT_event p (D_AND (D_AND X (D_BEFORE Y Z)) (D_BEFORE Z X)) t)
            (DFT_event p (D_AND (D_AND Z (D_BEFORE Y X)) (D_BEFORE X Z)) t)` by RW_TAC std_ss[event1_event2_shared_spare_disjoint]
THEN


g` !p. (if (X < Y) then X else Y) ==> (x = y)``



RW_TAC std_ss[pos_fn_integral_def]

RW_TAC std_ss[sup_eq]
KNOW_TAC(``!x. indicator_fn {}x = 0``)


Q.EXISTS_TAC (` g `)
RW_TAC std_ss[]

val _ = export_theory();



val pos_fn_integral_fun_mul = store_thm("pos_fn_integral_fun_mul",
  `` !M Y f.
          measure_space M /\ (!x. x IN m_space M ==> 0 <= f x) /\
          (!y. 0 <= Y y) ==>
          ((\y. pos_fn_integral M (\x. Normal (Y y) * f x)) =
           (\y. Normal (Y y) * pos_fn_integral M (\x. f x)))``, 

RW_TAC std_ss[]
THEN KNOW_TAC (``(!y. pos_fn_integral (M:
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal)) (\x. Normal ((Y:'b->real) y) * f x ) = Normal (Y y) * pos_fn_integral M (\x. f x))``)
            THEN1  (  RW_TAC std_ss[]
	           THEN Cases_on `Y y = (0:real)`
                        THEN1(RW_TAC std_ss [GSYM extreal_of_num_def,mul_lzero,pos_fn_integral_zero])
			  THEN `(0:real) < (Y:'b->real) y` by FULL_SIMP_TAC std_ss [REAL_LT_LE]
                          THEN RW_TAC std_ss [pos_fn_integral_def,sup_eq]
			       THEN1( Q_TAC SUFF_TAC `y' / Normal (Y y )<= sup {r | ?g. r IN psfis M g /\ !x. g x <= f x}`
			       	     THEN1(  METIS_TAC [le_ldiv,mul_comm])
				     THEN  RW_TAC std_ss [le_sup]
				     THEN POP_ASSUM MATCH_MP_TAC
				     THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
				     THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
 				     THEN RW_TAC std_ss [GSPECIFICATION]
				     THEN Q.EXISTS_TAC `(\x. g x / Normal( Y y))`
				     THEN  REVERSE (RW_TAC std_ss [])
				     	  THEN1( METIS_TAC [mul_comm,le_ldiv])
					  THEN RW_TAC std_ss [extreal_div_def]
					  THEN ONCE_REWRITE_TAC [mul_comm]
					  THEN RW_TAC std_ss [extreal_inv_def]
					  THEN KNOW_TAC (``!y. 0<=inv ((Y:'b->real) y)``)
					       THEN1(METIS_TAC [REAL_LE_INV])
					  THEN DISCH_TAC
					  THEN MP_TAC(Q.ISPECL[`(M :
          ('a -> bool) #
          (('a -> bool) -> bool) # (('a -> bool) -> extreal))`,`(g :'a -> extreal)`,`(y' :extreal)`,`D:'a->real`] psfis_fun_mul )
	       	  	   	   	   THEN KNOW_TAC (``inv ((Y:'b->real)y) = (inv o Y) y``)
					   		 THEN1( METIS_TAC[o_THM])
					   THEN DISCH_TAC
					   THEN ASM_REWRITE_TAC[]
					   THEN KNOW_TAC (``0<=((inv o (Y :'b -> real)) (y :'b))``)
					   	THEN1(METIS_TAC [REAL_LE_INV, o_THM])
						THEN DISCH_TAC
						THEN METIS_TAC [o_DEF, psfis_fun_mul])
			THEN  SUFF_TAC(``sup {r | ?g. r IN psfis M g /\ !x. g x <= f x} <= y' / (Normal (Y y))``)
			      THEN1(METIS_TAC [le_rdiv,extreal_not_infty,mul_comm])

			THEN RW_TAC std_ss [sup_le]
   			THEN POP_ASSUM (MP_TAC o ONCE_REWRITE_RULE [GSYM SPECIFICATION])
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Suff`(y'' * (Normal (Y y))) <= y'`
			      	   THEN1(METIS_TAC [le_rdiv,extreal_not_infty])
	   		THEN Q.PAT_X_ASSUM  `!z. {r | ?g. r IN psfis M g /\ !x. g x <= (Normal (Y y)) * f x} z ==> z <= y'` MATCH_MP_TAC
			THEN ONCE_REWRITE_TAC [GSYM SPECIFICATION]
			THEN RW_TAC std_ss [GSPECIFICATION]
			THEN Q.EXISTS_TAC `(\x. (Normal (Y y ))* g x)`
			THEN RW_TAC std_ss []
			     THEN1( METIS_TAC [psfis_fun_mul,mul_comm,extreal_not_infty])
			THEN  METIS_TAC [le_lmul_imp,extreal_of_num_def,extreal_lt_eq,lt_le])
THEN METIS_TAC []);)) *)

val _ = export_theory();
